// simple.frag

#version 130
#extension GL_EXT_geometry_shader4 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D textureImage;
uniform int lightNumber;
uniform int textureMode;
uniform int fogMode;
uniform int shadowVolumeMode;
uniform float edgeThreshold;

in float t; 
in vec3 position;
in vec3 normal;

void FogEffect(){
    float z = gl_FragCoord.z/ gl_FragCoord.w;
	
    vec4 fogColor    = gl_Fog.color;
    fogColor[3] = gl_FragColor[3];
    float fogDensity = gl_Fog.density;

    float fogFactor = 0.0;
    if (fogMode == 3){
	fogFactor = (gl_Fog.end - z) / (gl_Fog.end - gl_Fog.start);
    }
    else if (fogMode == 2){
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * z * LOG2 );
    }
    else {
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * gl_Fog.density * z * z * LOG2 );
    }
    
    fogFactor = clamp(fogFactor, 0.0, 1.0);
 
    gl_FragColor = mix(fogColor, gl_FragColor, fogFactor);
}

void main (void)
{
    if (gl_FrontFacing == false) discard;
    
    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (alpha > 1.0) alpha = 1.0;
    
    if (t == 0){
	gl_FragColor = vec4(0.0, 0.0, 0.0, alpha);
	if (fogMode != 0) FogEffect();
	return;
    }

    vec4 texColor = vec4(1.0,1.0,1.0,1.0); 
    if (textureMode == 1)
	texColor = texture2D(textureImage, gl_TexCoord[0].st);
    vec3 fnormal = normalize(normal);

    vec3  eye = - normalize(position);
    float eyedot = dot(eye, fnormal);
    if (eyedot * eyedot < edgeThreshold){
	gl_FragColor = vec4(0, 0, 0, alpha);
	if (fogMode != 0) FogEffect();
	return;
    }
    
    // ׂ͂ĔFF
    float diffuseTotal = 0.0;
    float specularTotal = 0.0;

    vec3 viewVec = -normalize(position);
    
    for (int i = 0; i < lightNumber; i++){
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	// fBt[Y 
	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = 1.0;//GetSpotEffect(i, spotAngleCos);
	    
	    // W
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    //˃xNg 
	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    gl_FrontMaterial.shininess) * lightFactor;
	    diffuseTotal += diffuse * gl_LightSource[i].diffuse.r * lightFactor;
	    specularTotal += specular * gl_LightSource[i].diffuse.r;
	}
    }
    
    float s = diffuseTotal * 0.8 + specularTotal;
    //if (s > 1.0) s = 1.0;
    
    vec4 ambient = gl_FrontMaterial.ambient * float(shadowVolumeMode);
    
    if (s < 0.2){
	gl_FragColor = ambient;
    }
    else if (s < 0.75){
	gl_FragColor = ambient +
	    gl_FrontMaterial.diffuse;
    }
    else{
	gl_FragColor = ambient + 
	    gl_FrontMaterial.diffuse + 
	    gl_FrontMaterial.specular;
    }
    
    gl_FragColor *= texColor;
    
    gl_FragColor[3] = alpha;

    if (fogMode != 0) FogEffect();
}
