// -*- c -*-
// (setq c-basic-offset 4)
// ("clsFilename.h")

#include "clsFilename.h"

#include <fstream>
#include <sstream>
#include <iomanip>

#ifdef WIN32
#define WCHARCONV(x) (wxString)wxConvLibc.cMB2WC(x)
#else
#define WCHARCONV(x) (wxString)wxConvUTF8.cMB2WC(x)
#endif

#include "CommonDefinitions.h"

#ifdef WIN32
#define DELIM L"\\"
#define DELIMWX wxT("\\")
#else
#define DELIM L"/"
#define DELIMWX wxT("/")
#endif

// --------------

SplitPath_cls::
SplitPath_cls(wxString pathStr){
    mstrFullPath = pathStr;
    size_t tail = mstrFullPath.Len();

    int pos_delim = mstrFullPath.Find(*DELIMWX, TRUE); //TRUE=fromEnd
    int pos_period = mstrFullPath.Find(wxT('.'), TRUE); //TRUE=fromEnd
    if (pos_period == wxNOT_FOUND ||
	pos_period < pos_delim){
	mstrExt = wxT("");
	mstrFullPathMinusExt = mstrFullPath;
    }
    else{
	mstrExt = mstrFullPath.c_str() + pos_period;
	mstrFullPathMinusExt = mstrFullPath;
	mstrFullPathMinusExt.Truncate(pos_period);
    }
    

    if (pos_delim != wxNOT_FOUND){
	mstrFilename = mstrFullPath.SubString(pos_delim + 1,
					      tail - mstrExt.Len() - 1);
	mstrDirPlusDrive = mstrFullPath.SubString(0, pos_delim - 1);
    }
    else
	mstrFilename = mstrFullPath.SubString(0, tail - mstrExt.Len() - 1);
}

wxString SplitPath_cls::
GetPathStr(int flag){
    switch(flag){
    case _FN_EXT:
	return mstrExt;
    case _FN_FNAME_EXT:
	if (mstrExt != wxT(""))
	    return mstrFilename + mstrExt;
	else
	    return mstrFilename;
    case _FN_FNAME:
	return mstrFilename;
    case _FN_DRIVE_DIR:
	return mstrDirPlusDrive;
    case _FN_DRIVE_DIR_FNAME:
	return mstrFullPathMinusExt;
    case _FN_FULLPATH:
	if (mstrDirPlusDrive.IsEmpty() == TRUE)
	    return mstrFilename + mstrExt;
	return mstrDirPlusDrive + DELIMWX + mstrFilename + mstrExt;
    }
    return mstrFullPath;
}

wxString SplitPath_cls::
GetPathDirLast(){
    size_t tail   = mstrDirPlusDrive.Len();
    int pos_delim = mstrDirPlusDrive.Find(*DELIMWX, TRUE); //TRUE=fromEnd
    if (pos_delim != wxNOT_FOUND)
	return mstrDirPlusDrive.SubString(pos_delim, tail) + DELIMWX;
    return wxT("");
}

wxString SplitPath_cls::
GetImageTypeStr(){
    if (mstrExt.IsSameAs(wxT(".jpeg")) || mstrExt.IsSameAs(wxT(".jpg")) ||
	mstrExt.IsSameAs(wxT(".JPEG")) || mstrExt.IsSameAs(wxT(".JPG")))
	return wxT("jpeg");
    if (mstrExt.IsSameAs(wxT(".png")) || mstrExt.IsSameAs(wxT(".PNG")))
	return wxT("png");
    if (mstrExt.IsSameAs(wxT(".gif")) || mstrExt.IsSameAs(wxT(".GIF")))
	return wxT("gif");
    if (mstrExt.IsSameAs(wxT(".tga")) || mstrExt.IsSameAs(wxT(".TGA")))
	return wxT("tga");
    if (mstrExt.IsSameAs(wxT(".tiff")) || mstrExt.IsSameAs(wxT(".tif")) ||
	mstrExt.IsSameAs(wxT(".TIFF")) || mstrExt.IsSameAs(wxT(".TIF")))
	return wxT("tiff");
    if (mstrExt.IsSameAs(wxT(".hdr")) || mstrExt.IsSameAs(wxT(".HDR")))
	return wxT("hdr");
    if (mstrExt.IsSameAs(wxT(".exr")) || mstrExt.IsSameAs(wxT(".EXR")))
	return wxT("exr");
    return wxT("unknown_format");
}

bool SplitPath_cls::
IsHDF5File(){
    if (mstrExt.IsSameAs(wxT(".h5")) || mstrExt.IsSameAs(wxT(".H5")))
	return TRUE;
    return FALSE;
}

// --------------

Filename_cls::
Filename_cls(){
    mstrPath = wxT("");
}

Filename_cls::
Filename_cls(const wxChar *path){
    mstrPath = path;
}

Filename_cls::
Filename_cls(wxString pathStr){
    mstrPath = pathStr;
}

wxString Filename_cls::
GetPathStr(int flag){
    SplitPath_cls CPath(mstrPath);

    return CPath.GetPathStr(flag);
}

const wxChar* Filename_cls::
c_str(){
    return mstrPath;
}

void Filename_cls::
Set(wxString strPath){
    mstrPath = strPath;
}

void Filename_cls::
DoubleSlash(wxString &str){
    str.Replace(wxT("\\"), wxT("\\\\"), TRUE);
}

//----

#include "../MYAPI/MsgBox.h"

// ----------------------

FileSequence_cls::
FileSequence_cls() : Filename_cls() {
    mDigitLength = -1;
    mSeqLength = -1;
    mInterval = 1;
}

FileSequence_cls::
FileSequence_cls(wxString pathStr) : Filename_cls(pathStr){
    Set(mstrPath, 0, 1);
}

void FileSequence_cls::
Set(const wxString pathStr, int initSeqNum, int seqInterval){
    mInterval = seqInterval;
    Filename_cls::Set(pathStr);
    if (pathStr == wxT("")) return;
    int formatPos = SequenceFormatPos();
    if (formatPos == wxNOT_FOUND) InitFormat(initSeqNum);
    else{
	SplitFormat(formatPos);
	mInit = initSeqNum;
    }
}

int FileSequence_cls::
SequenceFormatPos(){
    for (int index = 1; index < 6; index++){
	wxString buf;
	buf.Printf(wxT("%s0%dd"), wxT("%"), index);
	int pos = mstrPath.Find(buf.c_str());
	if (pos != wxNOT_FOUND) {
	    mDigitLength = index;
	    mInit = 0;
	    return pos;
	}
    }
    return wxNOT_FOUND;
}

void FileSequence_cls::
InitFormat(int initSeqNum){
    // %0[1-6]d甲
    for (int i = 1; i < 7; i++){
	wxString buf;
	buf.Printf(wxT("%s0%dd"), wxT("%"), i);
	if (mstrPath.Find(buf.c_str()) != wxNOT_FOUND) {
	    mDigitLength = i; 
	    mstrFormat = mstrPath;
	    mInit = initSeqNum;
	    return;
	}
    }

    SplitPath_cls CPath(mstrPath);
    wxString strPathMinusExt = CPath.mstrFullPathMinusExt;

    int posTail = (int)strPathMinusExt.Len() - 1;
    if (posTail > 999) return;

    while (strPathMinusExt[posTail] >= '0' && 
	   strPathMinusExt[posTail] <= '9'){
	posTail--;
	if (posTail > 999) break;
    }

    posTail++;

    if (posTail == strPathMinusExt.Len()){
	mInit = 0; mDigitLength = -1;
	mstrFileHead = strPathMinusExt;
	mstrFormat = mstrPath;
	return;
    }
    
    wxString strFH = CPath.mstrFullPathMinusExt.SubString(0, posTail - 1);
    wxString strDigit = CPath.mstrFullPathMinusExt.c_str() + posTail;
    mInit = atoi(strDigit.ToAscii());
    mDigitLength = (int)(CPath.mstrFullPathMinusExt.Len() - posTail);
    wxString strFormatMain;
    strFormatMain.Printf(wxT("%s0%dd"), wxT("%"), mDigitLength);

    mstrFileHead = strFH;
    mstrFormat = strFH + strFormatMain + CPath.mstrExt;
    mstrExt = CPath.mstrExt;
}

void FileSequence_cls::
SplitFormat(int pos){
    SplitPath_cls CPath(mstrPath);
    mstrFileHead = mstrPath.SubString(0, pos);
    mstrFormat = mstrPath;
    mstrExt = CPath.mstrExt;
}

void FileSequence_cls::
CalcMaxOffset(){
    if (mDigitLength == -1) {
	mSeqLength = 1;
	return;
    }

    mSeqLength    = 0; 
    int increment = 1000;
    int offset    = 0;

    wxString strModifiedStr = GetModifiedStr(offset);
    
#ifdef WIN32
    std::wfstream ifstream(strModifiedStr.c_str(), std::ios::in);
#else
    wxString wxFileIn = wxConvFile.cWC2WX(strModifiedStr.c_str());
    std::ifstream ifstream(wxFileIn.ToAscii(), std::ios::in);
#endif
    
    wxString wxMes = wxT("Can not open file");
    
    if (ifstream.fail()) {// ȃt@C
	MsgBoxf(NULL, WCHARCONV("t@Cǂ߂܂"),
		"FileSequence_cls::CalcMaxOffset\n%ls\nFile open fails", 
		GetModifiedStr(offset).c_str() );
	return ; } 
    ifstream.close();

    bool exitFlag = FALSE;

    while (TRUE){
	do {
	    if (increment == 1){ // increment1J艺ȂI
		exitFlag = TRUE; break;
	    }
	    increment /= 10;
	
	    ifstream.clear();
#ifdef WIN32
	    ifstream.open(GetModifiedStr(offset + increment).c_str(),
			  std::ios::in);
#else
	    wxFileIn = wxConvFile.cWC2WX(GetModifiedStr(offset + increment).c_str());
	    ifstream.open(wxFileIn.ToAscii(), std::ios::in);
#endif
	    if (!ifstream.fail()) ifstream.close();
	    /*
	    else
		MsgBoxf(NULL, "FileSequence_cls::CalcMaxOffset",
			"open %ls fails %d", 
			GetModifiedWXStr(offset + increment).c_str(),
			mSeqLength);
	    */
	    
	} while (ifstream.fail()); // t@CincrementJ艺
	
	mSeqLength += increment;
	offset  += increment;
	if (exitFlag == TRUE) break;
	increment *= 10;
    }

    //not yet
}

wxString FileSequence_cls::
GetFormat(){
    return mstrFormat;
}

wxString FileSequence_cls::
GetModifiedStr(int index, int stereo){
    wxString strBuf;
    wxString strFormat = GetFormat();
    wxString strStereo;
    if (stereo != 0){
	if (stereo == 1) strStereo = wxT("L");
	if (stereo == 2) strStereo = wxT("R");
	if (strFormat.Find('%') != -1){
	    strFormat.Replace(wxT("%"), wxT("%s%"));
	    strBuf.Printf(strFormat, strStereo.c_str(), mInit + index * mInterval);
	}
	else{
	    SplitPath_cls CPath(strFormat);
	    strBuf = CPath.GetPathStr(_FN_DRIVE_DIR_FNAME) + strStereo + CPath.GetPathStr(_FN_EXT);
	}
    }
    else
	strBuf.Printf(strFormat, mInit + index * mInterval);
    
    return strBuf;
}

wxString FileSequence_cls::
GetInfoStr(){
    wxString strInfo;
    strInfo << mstrFormat;
    if (mDigitLength != -1)
	strInfo << wxT(" (") << mInit << wxT("-") << mInit + mSeqLength << wxT(")");
    return strInfo;
}

/*
wxString FileSequence_cls::
GetChannelStr(){
    wxString strChannel;
    if (mChannel == 0) strChannel = wxT("Red");
    if (mChannel == 1) strChannel = wxT("Green");
    if (mChannel == 2) strChannel = wxT("Blue");
    if (mChannel == 3) strChannel = wxT("Alpha (not implemented)");
    return strChannel;
}
*/

void FileSequence_cls::
AppendDefaultSequenceTag(){
    Set(mstrFileHead+wxT("%04d")+mstrExt, 0, 1);
}
