
#ifndef _CONF_ITEM_H_
#define _CONF_ITEM_H_

#include <wx/wx.h>

#include <vector>

#define _CONF_ITEM_NONE     0
#define _CONF_ITEM_STR       1
#define _CONF_ITEM_INT       2
#define _CONF_ITEM_INTARRAY  3
#define _CONF_ITEM_UINT      4
#define _CONF_ITEM_UINTARRAY 5
#define _CONF_ITEM_FLOAT     6
#define _CONF_ITEM_FLOATARRAY  7
#define _CONF_ITEM_DOUBLE      8
#define _CONF_ITEM_DOUBLEARRAY 9
#define _CONF_ITEM_BOOL        10
#define _CONF_ITEM_BOOLARRAY   11

#define _CONFARRAY_MAX 16

class ConfItem_cls {
 public:
    ConfItem_cls(int type);

    wxString mstrHeader;
    wxString mstrBody;
    int    mType;

    virtual void SetItem(wxString strHeader, wxString strBody) = 0;
    virtual bool ReadStrLine(wxString strLine) = 0;
    virtual wxString Serialize() = 0;
    virtual wxString GetItemStr() = 0;
    bool ReadStrLineBase(wxString strLine);

    void ReplaceSeparator();
};

class ConfArray_cls {
 public:
    int mDimension;
    int CountDimension(wxString strBody);
};

class ConfItemSTR_cls : public ConfItem_cls {
 public:
    ConfItemSTR_cls();
    ConfItemSTR_cls(wxString strHeader, wxString strBody);

    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(/*std::wstring*/wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemINT_cls : public ConfItem_cls {
 public:
    ConfItemINT_cls();
    ConfItemINT_cls(wxString strHeader, wxString strBody);
    int mBody;

    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemINTARRAY_cls : public ConfItem_cls, public ConfArray_cls {
 public:
    ConfItemINTARRAY_cls();

    // $B%3%T!<%3%s%9%H%i%/%?4XO"$,Lq2p$J$N$G!"M>J,$KG[Ns$r;}$C$F$7$^$&!#(B
    int mpBody[_CONFARRAY_MAX];
    
    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemUINT_cls : public ConfItem_cls {
 public:
    ConfItemUINT_cls();
    ConfItemUINT_cls(wxString strHeader, wxString strBody);

    unsigned int mBody;
    
    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(/*std::wstring*/wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemUINTARRAY_cls : public ConfItem_cls, public ConfArray_cls {
 public:
    // $B%3%T!<%3%s%9%H%i%/%?4XO"$,Lq2p$J$N$G!"M>J,$KG[Ns$r;}$C$F$7$^$&!#(B
    unsigned int mpBody[_CONFARRAY_MAX];
    
    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemFLOAT_cls : public ConfItem_cls {
 public:
    ConfItemFLOAT_cls();
    ConfItemFLOAT_cls(wxString strHeader, wxString strBody);
    
    float mBody;
    
    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(/*std::wstring*/wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemFLOATARRAY_cls : public ConfItem_cls, public ConfArray_cls {
 public:
    ConfItemFLOATARRAY_cls();
    ConfItemFLOATARRAY_cls(wxString strHeader, wxString strBody);

    // $B%3%T!<%3%s%9%H%i%/%?4XO"$,Lq2p$J$N$G!"M>J,$KG[Ns$r;}$C$F$7$^$&!#(B
    float mpBody[_CONFARRAY_MAX];
        
    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};


class ConfItemDOUBLE_cls : public ConfItem_cls {
 public:
    ConfItemDOUBLE_cls();
    //ConfItemDOUBLE_cls(wxString strHeader, wxString strBody);
    double mBody;
    
    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemDOUBLEARRAY_cls : public ConfItem_cls, public ConfArray_cls {
 public:
    ConfItemDOUBLEARRAY_cls();
    ConfItemDOUBLEARRAY_cls(wxString strHeader, wxString strBody);

    // $B%3%T!<%3%s%9%H%i%/%?4XO"$,Lq2p$J$N$G!"M>J,$KG[Ns$r;}$C$F$7$^$&!#(B
    double mpBody[_CONFARRAY_MAX];
        
    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemBOOL_cls : public ConfItem_cls {
 public:
    ConfItemBOOL_cls();
    ConfItemBOOL_cls(wxString strHeader, wxString strBody);

    bool mbBody;

    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemBOOLARRAY_cls : public ConfItem_cls, public ConfArray_cls {
 public:
    ConfItemBOOLARRAY_cls();
    //ConfItemBOOLARRAY_cls(wxString strHeader, wxString strBody);

    bool mpbBody[_CONFARRAY_MAX];

    void SetItem(wxString strHeader, wxString strBody);
    bool ReadStrLine(wxString strLine);
    wxString Serialize();
    wxString GetItemStr();
};

class ConfItemList_cls {
 public:
    ConfItemList_cls();

    wxString mstrTag;

    std::vector < ConfItem_cls* > mVpCList;
    bool mbSetWithInitValue;

    void SetTag(wxString strTag);
    void Clear();
    void Add(ConfItem_cls* pC, 
	     wxString strHeader, wxString strBody = wxT(""));
    wxString GetHeaderStrFromPointer(void *pValue);
    ConfItem_cls* GetPCItem(wxString strHeader);

    void CopyListComponentFromP(ConfItemList_cls* pCFrom);

    void SerializeAllWithTag(wxString &strOut);
    bool ReadStrLineAllWithTag(std::wistream &istream, wxString strLine);

    wxString SerializeAll();
    bool ReadStrLineAll(wxString strLine);
};

#endif
