// -*- c++ -*- 
// (setq c-basic-offset 4)

#include "clsFilterBoolean.h"

#include "../MAINCANVAS/clsFilterPanelPain.h"
#include "../PARTICLEDATA/clsArrayAndIndex.h"
#include "../UNDO/clsUndoManager.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"

#include "../../wxWidgetEnum.h"

#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

ThreadBooleanCheckBase_cls::
ThreadBooleanCheckBase_cls() : wxThread(wxTHREAD_DETACHED) {
}

void ThreadBooleanCheckBase_cls::
SetParameters(bool *pbFinishedFlag,
	      ArrayAndIndex_cls* pIndexAndZ, float (*pX)[3],
	      float *pAlphaFactor,
      	      int   operatorType, float borderFactor,
	      float invMat[16], bool bRot, 
	      int ini, int fin){
    
    mpbFinishedFlagLink = pbFinishedFlag;
    mpIndexAndZ = pIndexAndZ;
    mpX = pX;
    mpAlphaFactor = pAlphaFactor;
    mOperatorType = operatorType;
    mBorderFactor = borderFactor;
    mInvMat = invMat;
    mbRot = bRot;
    mIni = ini; mFin = fin;
    
    if (Create() != wxTHREAD_NO_ERROR) exit (0);
	//MsgBoxExit(NULL, "Error!", "Can't create thread");
}

// --

void ThreadBooleanCheckBox_cls::
CutBoxBorderOffRotateOn(){
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float inner, outer;
    switch(operatorType){
    case _BOOLEAN_INTERSECT:
	inner = 1; outer = 0;
	break;
    case _BOOLEAN_DIFFERENCE:
	inner = 0; outer = 1;
	break;
    case _BOOLEAN_ADDITIVE:
	for (int i = mIni; i < mFin; i++){
	    int index = pIndexAndZ[i].index;
	    float *pos = pX[index];
	    float posX = pMat[0]*pos[X]+pMat[4]*pos[Y]+pMat[8 ]*pos[Z]+pMat[12];
	    float posY = pMat[1]*pos[X]+pMat[5]*pos[Y]+pMat[9 ]*pos[Z]+pMat[13];
	    float posZ = pMat[2]*pos[X]+pMat[6]*pos[Y]+pMat[10]*pos[Z]+pMat[14];
	    if (posX < 0.5 && posX > -0.5 &&
		posY < 0.5 && posY > -0.5 &&
		posZ < 0.5 && posZ > -0.5)
		pAlphaFactor[i] = 1;
	}
	return;
	break;
    }
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float *pos = pX[index];
	float posX = pMat[0]*pos[X]+pMat[4]*pos[Y]+pMat[8 ]*pos[Z]+pMat[12];
	float posY = pMat[1]*pos[X]+pMat[5]*pos[Y]+pMat[9 ]*pos[Z]+pMat[13];
	float posZ = pMat[2]*pos[X]+pMat[6]*pos[Y]+pMat[10]*pos[Z]+pMat[14];
	if (posX < 0.5 && posX > -0.5 &&
	    posY < 0.5 && posY > -0.5 &&
	    posZ < 0.5 && posZ > -0.5)
	    pAlphaFactor[i] *= inner;
	else
	    pAlphaFactor[i] *= outer;
    }
}

void ThreadBooleanCheckBox_cls::
CutBoxBorderOffRotateOff(){
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float inner, outer;
    switch(operatorType){
    case _BOOLEAN_INTERSECT: 
	inner = 1; outer = 0;
	break;
    case _BOOLEAN_DIFFERENCE:
	inner = 0; outer = 1;
	break;
    case _BOOLEAN_ADDITIVE: 
	for (int i = mIni; i < mFin; i++){
	    int index = pIndexAndZ[i].index;
	    float *pos = pX[index];
	    float posX = pMat[0 ]*pos[X]+pMat[12];
	    float posY = pMat[5 ]*pos[Y]+pMat[13];
	    float posZ = pMat[10]*pos[Z]+pMat[14];
	    if (posX < 0.5 && posX > -0.5 &&
		posY < 0.5 && posY > -0.5 &&
		posZ < 0.5 && posZ > -0.5)
		pAlphaFactor[i] = 1;
	}
	return;
	break;
    }
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float *pos = pX[index];
	float posX = pMat[0 ]*pos[X]+pMat[12];
	float posY = pMat[5 ]*pos[Y]+pMat[13];
	float posZ = pMat[10]*pos[Z]+pMat[14];
	if (posX < 0.5 && posX > -0.5 &&
	    posY < 0.5 && posY > -0.5 &&
	    posZ < 0.5 && posZ > -0.5)
	    pAlphaFactor[i] *= inner;
	else
	    pAlphaFactor[i] *= outer;
    }
}

void ThreadBooleanCheckBox_cls::
CutBoxBorderOnRotateOn(){
    float borderFactor = mBorderFactor;
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float border = 1.0 + borderFactor;
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float factor = 1;
	float *pos = pX[index];
	float posX = pMat[0]*pos[X]+pMat[4]*pos[Y]+pMat[8 ]*pos[Z]+pMat[12];
	float posY = pMat[1]*pos[X]+pMat[5]*pos[Y]+pMat[9 ]*pos[Z]+pMat[13];
	float posZ = pMat[2]*pos[X]+pMat[6]*pos[Y]+pMat[10]*pos[Z]+pMat[14];
	
	if      (posX > 0.5 * border) factor *= 0;
	else if (posX > 0.5)  factor *= 1.0 - 2*(posX-0.5)/borderFactor ;
	else if (posX > -0.5) factor *= 1.0;
	else if (posX > -0.5 * border) factor *= 1.0 + 2*(posX+0.5)/borderFactor;
	else factor *= 0;
	
	if      (posY > 0.5 * border) factor *= 0;
	else if (posY > 0.5)  factor *= 1.0 - 2*(posY-0.5)/borderFactor ;
	else if (posY > -0.5) factor *= 1.0;
	else if (posY > -0.5 * border) factor *= 1.0 + 2*(posY+0.5)/borderFactor;
	else factor *= 0;
	
	if      (posZ > 0.5 * border) factor *= 0;
	else if (posZ > 0.5)  factor *= 1.0 - 2*(posZ-0.5)/borderFactor ;
	else if (posZ > -0.5) factor *= 1.0;
	else if (posZ > -0.5 * border) factor *= 1.0 + 2*(posZ+0.5)/borderFactor;
	else factor *= 0;
	
	switch(operatorType){
	case _BOOLEAN_INTERSECT:
	    pAlphaFactor[i] *= factor;
	    break;
	case _BOOLEAN_DIFFERENCE:
	    pAlphaFactor[i] *= (1.0f - factor);
	    break;
	case _BOOLEAN_ADDITIVE:
	    pAlphaFactor[i] += factor;
	    if (pAlphaFactor[i] > 1) pAlphaFactor[i] = 1;
	    break;
	}
    }
}

void ThreadBooleanCheckBox_cls::
CutBoxBorderOnRotateOff(){
    float borderFactor = mBorderFactor;
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float border = 1.0 + borderFactor;
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float factor = 1;
	float *pos = pX[index];
	float posX = pMat[0 ]*pos[X]+pMat[12];
	float posY = pMat[5 ]*pos[Y]+pMat[13];
	float posZ = pMat[10]*pos[Z]+pMat[14];
	
	if      (posX > 0.5 * border) factor *= 0;
	else if (posX > 0.5)  factor *= 1.0 - 2*(posX-0.5)/borderFactor ;
	else if (posX > -0.5) factor *= 1.0;
	else if (posX > -0.5 * border) factor *= 1.0 + 2*(posX+0.5)/borderFactor;
	else factor *= 0;
	
	if      (posY > 0.5 * border) factor *= 0;
	else if (posY > 0.5)  factor *= 1.0 - 2*(posY-0.5)/borderFactor ;
	else if (posY > -0.5) factor *= 1.0;
	else if (posY > -0.5 * border) factor *= 1.0 + 2*(posY+0.5)/borderFactor;
	else factor *= 0;
	
	if      (posZ > 0.5 * border) factor *= 0;
	else if (posZ > 0.5)  factor *= 1.0 - 2*(posZ-0.5)/borderFactor ;
	else if (posZ > -0.5) factor *= 1.0;
	else if (posZ > -0.5 * border) factor *= 1.0 + 2*(posZ+0.5)/borderFactor;
	else factor *= 0;
	
	switch(operatorType){
	case _BOOLEAN_INTERSECT:
	    pAlphaFactor[i] *= factor;
	    break;
	case _BOOLEAN_DIFFERENCE:
	    pAlphaFactor[i] *= (1.0f - factor);
	    break;
	case _BOOLEAN_ADDITIVE:
	    pAlphaFactor[i] += factor;
	    if (pAlphaFactor[i] > 1) pAlphaFactor[i] = 1;
	    break;
	}
    }
}

wxThread::ExitCode ThreadBooleanCheckBox_cls::
Entry(){
    if (mBorderFactor == 0){
	if (mbRot == TRUE)
	    CutBoxBorderOffRotateOn();
	else
	    CutBoxBorderOffRotateOff();
    }
    else{
	if (mbRot == TRUE)
	    CutBoxBorderOnRotateOn();
	else
	    CutBoxBorderOnRotateOff();
    }
    
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ---

void ThreadBooleanCheckPlane_cls::
CutPlaneBorderOffRotateOn(){
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float inner, outer;
    switch(operatorType){
    case _BOOLEAN_INTERSECT:
	inner = 1; outer = 0;
	break;
    case _BOOLEAN_DIFFERENCE:
	inner = 0; outer = 1;
	break;
    case _BOOLEAN_ADDITIVE:
	for (int i = mIni; i < mFin; i++){
	    int index = pIndexAndZ[i].index;
	    float *pos = pX[index];
	    float posX = pMat[0]*pos[X]+pMat[4]*pos[Y]+pMat[8 ]*pos[Z]+pMat[12];
	    float posY = pMat[1]*pos[X]+pMat[5]*pos[Y]+pMat[9 ]*pos[Z]+pMat[13];
	    float posZ = pMat[2]*pos[X]+pMat[6]*pos[Y]+pMat[10]*pos[Z]+pMat[14];
	    if (posZ < 0) pAlphaFactor[i] =1;
	}
	return;
	break;
    }
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float *pos = pX[index];
	float posX = pMat[0]*pos[X]+pMat[4]*pos[Y]+pMat[8 ]*pos[Z]+pMat[12];
	float posY = pMat[1]*pos[X]+pMat[5]*pos[Y]+pMat[9 ]*pos[Z]+pMat[13];
	float posZ = pMat[2]*pos[X]+pMat[6]*pos[Y]+pMat[10]*pos[Z]+pMat[14];
	if (posZ < 0) 
	    pAlphaFactor[i] *= inner;
	else
	    pAlphaFactor[i] *= outer;
    }
}

void ThreadBooleanCheckPlane_cls::
CutPlaneBorderOffRotateOff(){
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float inner, outer;
    switch(operatorType){
    case _BOOLEAN_INTERSECT:
	inner = 1; outer = 0;
	break;
    case _BOOLEAN_DIFFERENCE:
	inner = 0; outer = 1;
	break;
    case _BOOLEAN_ADDITIVE:
	for (int i = mIni; i < mFin; i++){
	    int index = pIndexAndZ[i].index;
	    float *pos = pX[index];
	    float posX = pMat[0 ]*pos[X]+pMat[12];
	    float posY = pMat[5 ]*pos[Y]+pMat[13];
	    float posZ = pMat[10]*pos[Z]+pMat[14];
	    if (posZ < 0) pAlphaFactor[i] = 1;
	}
	return;
	break;
    }
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float *pos = pX[index];
	float posX = pMat[0 ]*pos[X]+pMat[12];
	float posY = pMat[5 ]*pos[Y]+pMat[13];
	float posZ = pMat[10]*pos[Z]+pMat[14];
	if (posZ < 0) 
	    pAlphaFactor[i] *= inner;
	else
	    pAlphaFactor[i] *= outer;
    }
}

void ThreadBooleanCheckPlane_cls::
CutPlaneBorderOnRotateOn(){
    float borderFactor = mBorderFactor;
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float border = 1.0 + borderFactor;
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float factor = 1;
	float *pos = pX[index];
	float posX = pMat[0]*pos[X]+pMat[4]*pos[Y]+pMat[8 ]*pos[Z]+pMat[12];
	float posY = pMat[1]*pos[X]+pMat[5]*pos[Y]+pMat[9 ]*pos[Z]+pMat[13];
	float posZ = pMat[2]*pos[X]+pMat[6]*pos[Y]+pMat[10]*pos[Z]+pMat[14];
	
	if      (posZ > borderFactor) factor *= 0;
	else if (posZ > 0)  factor *= 1.0 - posZ / borderFactor ;
	else factor *= 1;
	
	switch(operatorType){
	case _BOOLEAN_INTERSECT:
	    pAlphaFactor[i] *= factor;
	    break;
	case _BOOLEAN_DIFFERENCE:
	    pAlphaFactor[i] *= (1.0f - factor);
	    break;
	case _BOOLEAN_ADDITIVE:
	    pAlphaFactor[i] += factor;
	    if (pAlphaFactor[i] > 1) pAlphaFactor[i] = 1;
	    break;
	}
    }
}

void ThreadBooleanCheckPlane_cls::
CutPlaneBorderOnRotateOff(){
    float borderFactor = mBorderFactor;
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float border = 1.0 + borderFactor;
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float factor = 1;
	float *pos = pX[index];
	float posX = pMat[0 ]*pos[X]+pMat[12];
	float posY = pMat[5 ]*pos[Y]+pMat[13];
	float posZ = pMat[10]*pos[Z]+pMat[14];

	if      (posZ > borderFactor) factor *= 0;
	else if (posZ > 0)  factor *= 1.0 - posZ / borderFactor ;
	else factor *= 1;
	
	switch(operatorType){
	case _BOOLEAN_INTERSECT:
	    pAlphaFactor[i] *= factor;
	    break;
	case _BOOLEAN_DIFFERENCE:
	    pAlphaFactor[i] *= (1.0f - factor);
	    break;
	case _BOOLEAN_ADDITIVE:
	    pAlphaFactor[i] += factor;
	    if (pAlphaFactor[i] > 1) pAlphaFactor[i] = 1;
	    break;
	}
    }
}

wxThread::ExitCode ThreadBooleanCheckPlane_cls::
Entry(){
    if (mBorderFactor == 0){
	if (mbRot == TRUE)
	    CutPlaneBorderOffRotateOn();
	else
	    CutPlaneBorderOffRotateOff();
    }
    else{
	if (mbRot == TRUE)
	    CutPlaneBorderOnRotateOn();
	else
	    CutPlaneBorderOnRotateOff();
    }
   
    *mpbFinishedFlagLink = TRUE;
    return 0;
}


// ---

void ThreadBooleanCheckSphere_cls::
CutSphereBorderOffRotateOn(){
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float inner, outer;
    switch(operatorType){
    case _BOOLEAN_INTERSECT:
	inner = 1; outer = 0;
	break;
    case _BOOLEAN_DIFFERENCE:
	inner = 0; outer = 1;
	break;
    case _BOOLEAN_ADDITIVE:
	for (int i = mIni; i < mFin; i++){
	    int index = pIndexAndZ[i].index;
	    float *pos = pX[index];
	    float posX = pMat[0]*pos[X]+pMat[4]*pos[Y]+pMat[8 ]*pos[Z]+pMat[12];
	    float posY = pMat[1]*pos[X]+pMat[5]*pos[Y]+pMat[9 ]*pos[Z]+pMat[13];
	    float posZ = pMat[2]*pos[X]+pMat[6]*pos[Y]+pMat[10]*pos[Z]+pMat[14];
	    float difference2 = posX*posX + posY*posY + posZ*posZ;
	    if (difference2 < 0.25) pAlphaFactor[i] =1;
	}
	return;
	break;
    }
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float *pos = pX[index];
	float posX = pMat[0]*pos[X]+pMat[4]*pos[Y]+pMat[8 ]*pos[Z]+pMat[12];
	float posY = pMat[1]*pos[X]+pMat[5]*pos[Y]+pMat[9 ]*pos[Z]+pMat[13];
	float posZ = pMat[2]*pos[X]+pMat[6]*pos[Y]+pMat[10]*pos[Z]+pMat[14];
	float difference2 = posX*posX + posY*posY + posZ*posZ;
	if (difference2 < 0.25) 
	    pAlphaFactor[i] *= inner;
	else
	    pAlphaFactor[i] *= outer;
    }
}

void ThreadBooleanCheckSphere_cls::
CutSphereBorderOffRotateOff(){
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float inner, outer;
    switch(operatorType){
    case _BOOLEAN_INTERSECT:
	inner = 1; outer = 0;
	break;
    case _BOOLEAN_DIFFERENCE:
	inner = 0; outer = 1;
	break;
    case _BOOLEAN_ADDITIVE:
	for (int i = mIni; i < mFin; i++){
	    int index = pIndexAndZ[i].index;
	    float *pos = pX[index];
	    float posX = pMat[0 ]*pos[X]+pMat[12];
	    float posY = pMat[5 ]*pos[Y]+pMat[13];
	    float posZ = pMat[10]*pos[Z]+pMat[14];
	    float difference2 = posX*posX + posY*posY + posZ*posZ;
	    if (difference2 < 0.25) pAlphaFactor[i] =1;
	}
	return; 
	break;
    }
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float *pos = pX[index];
	float posX = pMat[0 ]*pos[X]+pMat[12];
	float posY = pMat[5 ]*pos[Y]+pMat[13];
	float posZ = pMat[10]*pos[Z]+pMat[14];
	float difference2 = posX*posX + posY*posY + posZ*posZ;
	if (difference2 < 0.25) 
	    pAlphaFactor[i] *= inner;
	else
	    pAlphaFactor[i] *= outer;
    }
}

void ThreadBooleanCheckSphere_cls::
CutSphereBorderOnRotateOn(){
    float borderFactor = mBorderFactor;
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float border = 1.0 + borderFactor;
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float factor = 1;
	float *pos = pX[index];
	float posX = pMat[0]*pos[X]+pMat[4]*pos[Y]+pMat[8 ]*pos[Z]+pMat[12];
	float posY = pMat[1]*pos[X]+pMat[5]*pos[Y]+pMat[9 ]*pos[Z]+pMat[13];
	float posZ = pMat[2]*pos[X]+pMat[6]*pos[Y]+pMat[10]*pos[Z]+pMat[14];
	float difference = sqrt(posX*posX + posY*posY + posZ*posZ);
	
	if      (difference > 0.5 * border) factor *= 0;
	else if (difference > 0.5)  factor *= 1.0 - 2*(difference-0.5)/borderFactor ;
	else factor *= 1;
	
	switch(operatorType){
	case _BOOLEAN_INTERSECT:
	    pAlphaFactor[i] *= factor;
	    break;
	case _BOOLEAN_DIFFERENCE:
	    pAlphaFactor[i] *= (1.0f - factor);
	    break;
	case _BOOLEAN_ADDITIVE:
	    pAlphaFactor[i] += factor;
	    if (pAlphaFactor[i] > 1) pAlphaFactor[i] = 1;
	    break;
	}
    }
}

void ThreadBooleanCheckSphere_cls::
CutSphereBorderOnRotateOff(){
    float borderFactor = mBorderFactor;
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;
    float (*pX)[3] = mpX;
    float *pAlphaFactor = mpAlphaFactor;
    float *pMat = mInvMat;
    int   operatorType = mOperatorType;

    float border = 1.0 + borderFactor;
    for (int i = mIni; i < mFin; i++){
	int index = pIndexAndZ[i].index;
	float factor = 1;
	float *pos = pX[index];
	float posX = pMat[0 ]*pos[X]+pMat[12];
	float posY = pMat[5 ]*pos[Y]+pMat[13];
	float posZ = pMat[10]*pos[Z]+pMat[14];
	float difference = sqrt(posX*posX + posY*posY + posZ*posZ);
	
	if      (difference > 0.5 * border) factor *= 0;
	else if (difference > 0.5) factor *= 1.0 - 2*(difference-0.5)/borderFactor ;
	else factor *= 1;
	
	switch(operatorType){
	case _BOOLEAN_INTERSECT:
	    pAlphaFactor[i] *= factor;
	    break;
	case _BOOLEAN_DIFFERENCE:
	    pAlphaFactor[i] *= (1.0f - factor);
	    break;
	case _BOOLEAN_ADDITIVE:
	    pAlphaFactor[i] += factor;
	    if (pAlphaFactor[i] > 1) pAlphaFactor[i] = 1;
	    break;
	}
    }
}

wxThread::ExitCode ThreadBooleanCheckSphere_cls::
Entry(){
    if (mBorderFactor == 0){
	if (mbRot == TRUE) CutSphereBorderOffRotateOn();
	else               CutSphereBorderOffRotateOff();
    }
    else{
	if (mbRot == TRUE) CutSphereBorderOnRotateOn();
	else               CutSphereBorderOnRotateOff();
    }
    
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// -----------------------

FilterBooleanGUI_cls::							
FilterBooleanGUI_cls(FilterBoolean_cls *pCParent, 
		     FilterPanelPain_cls *pCPanelPain) : FilterGUI_cls(pCParent) {
    mpCParent = pCParent;
    SetMNumPanel((wxBoxSizer*)pCPanelPain->GetSizer(), pCPanelPain, 0);
}

void FilterBooleanGUI_cls::
SetMNumPanel(wxBoxSizer *pBoxSizer, wxWindow *pCParentWindow, int index){
    
    MultiInputPanel_cls* pCurr = InitializeMInputCommon
	(pCParentWindow, mpCParent->GetPCData()->mCstrCustomName.mstrBody);
    FilterBooleanData_cls *pCData = mpCParent->GetPCData();
    
    pCurr->mUpdateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;
    
    int boxWidth = 2;
    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;
    pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Operator"));
    wxString listOperator[3] = {wxT("Intersect"), wxT("Difference"), wxT("Add") };
    pCurr->CombinePointerList(&pCData->mCOperatorType.mBody, 3, listOperator);
    wxString listRegion[3] = {wxT("Box"), wxT("Plane"), wxT("Sphere")};
    pCurr->CombinePointerList(&pCData->mCRegionType.mBody, 3, listRegion);
    
    std::vector < int > VIndex = SetMNumBasic(pCurr, pCData);

    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;
    pCurr->CombinePointer(_ITEM_FLOAT, &pCData->mCBorderFactor.mBody, _("Border"));
    
    pCurr->FinishSetting(NULL);

    SetHookBasic(pCurr);

    pCurr->SetColorForXYZ(VIndex[0]+1);
    pCurr->SetColorForXYZ(VIndex[1]+1);
    pCurr->SetColorForXYZ(VIndex[2]+1);
}

// ---------------------

FilterBooleanData_cls::
FilterBooleanData_cls() {
    mCstrCustomName.mstrBody = _("Boolean");
    mCstrCustomName.mstrBody << wxT("%02d");
    mCConfList.Add(&mCOperatorType, wxT("OperatorType="), wxT("0"));
    mCConfList.Add(&mCRegionType, wxT("RegionType="), wxT("0"));
    mCConfList.Add(&mCBorderFactor, wxT("BorderFactor="), wxT("0.01"));
    
    //LOOP_XYZ mDummyRotate[xyz] = 0;
}

// ---------------------

FilterBoolean_cls::
FilterBoolean_cls(FilterPanelPain_cls *pCPanelPain,
		  FilterBooleanData_cls *pCInitData) :
    FilterBase_cls(pCPanelPain->mpCFilterCtrlLink)
{
    wxString strInit; strInit << _FILTER_BOOLEAN;
    mCFilterType.SetItem(wxT("FilterType="), strInit);
    mpCData     = new FilterBooleanData_cls;
    mpCGUIPanel = new FilterBooleanGUI_cls(this, pCPanelPain);
}

FilterBoolean_cls::
~FilterBoolean_cls(){
}

FilterBooleanData_cls* FilterBoolean_cls::
GetPCData(){
    return (FilterBooleanData_cls*)mpCData;
}

void FilterBoolean_cls::
CheckBoolean(ArrayAndIndex_cls* pIndexAndZ, 
	     float (*pX)[3], int downsized, float *pAlphaFactor,
	     double doubleIndex){
    
    FilterBooleanData_cls *pCData = GetPCData();
    ParamGroup_cls *pCTranslate = pCData->mpCTranslate;
    ParamGroup_cls *pCRotate    = pCData->mpCRotation;
    ParamGroup_cls *pCScale     = pCData->mpCScale;

    if (pCData->mCbValid.mbBody == FALSE) return;

    float invMatB[16];
    
    std::vector < double > VScale = 
	pCScale->GetExpressionValue(doubleIndex, _PARAM_USE_COMMON);
    std::vector < double > VRot = 
	pCRotate->GetExpressionValue(doubleIndex, _PARAM_USE_COMMON);

    bool bRot = TRUE;
    if (VRot[0] == 0 && VRot[1] == 0 && VRot[2] == 0) bRot = FALSE;
    
    float scaleX = VScale[0];
    float scaleY = VScale[1];
    float scaleZ = VScale[2];

    pCData->mCCurrentObj.GetGLMatrixInvf(invMatB);
    invMatB[0] /= scaleX; invMatB[4] /= scaleX; invMatB[8 ] /= scaleX;
    invMatB[1] /= scaleY; invMatB[5] /= scaleY; invMatB[9 ] /= scaleY;
    invMatB[2] /= scaleZ; invMatB[6] /= scaleZ; invMatB[10] /= scaleZ;
    invMatB[12] /= scaleX;
    invMatB[13] /= scaleY;
    invMatB[14] /= scaleZ;

    int operatorType   = pCData->mCOperatorType.mBody;
    int regionType     = pCData->mCRegionType.mBody;
    float borderFactor = pCData->mCBorderFactor.mBody;

    int threadNumber= 1;
    if (downsized > 10000) threadNumber = 2;
    if (downsized > 100000) threadNumber = 3;
    if (downsized > 1000000) threadNumber = 4;
    ThreadBooleanCheckBase_cls **ppCThreadBooleanCheck = 
	new ThreadBooleanCheckBase_cls* [threadNumber];
    bool *bAFinished = new bool [threadNumber];
    
    for (int core = 0; core < threadNumber; core++){
	bAFinished[core] = FALSE;

	switch(regionType){
	case _BOOLEAN_BOX:
	    ppCThreadBooleanCheck[core] = new ThreadBooleanCheckBox_cls;
	    break;
	case _BOOLEAN_PLANE:
	    ppCThreadBooleanCheck[core] = new ThreadBooleanCheckPlane_cls;
	    break;
	case _BOOLEAN_SPHERE:
	    ppCThreadBooleanCheck[core] = new ThreadBooleanCheckSphere_cls;
	    break;
	}
	    
	int PInit = downsized / threadNumber * core;
	int PFin  = downsized / threadNumber * (core+1);
	if (core == threadNumber - 1) PFin = downsized;

	ppCThreadBooleanCheck[core]->SetParameters
	    (&bAFinished[core], pIndexAndZ, pX,
	     pAlphaFactor, operatorType, borderFactor,
	     invMatB, bRot, PInit, PFin);
	ppCThreadBooleanCheck[core]->Run();
    }
	
    bool bFinishedAll = FALSE;
    while (bFinishedAll == FALSE){
	wxMilliSleep(3);
	bFinishedAll = TRUE;
	for (int core = 0; core < threadNumber; core++)
	    bFinishedAll &= bAFinished[core];
    }
	
    delete [] bAFinished;
    delete [] ppCThreadBooleanCheck;
}

void FilterBoolean_cls::
GLDraw(double doubleIndex, double scale, bool bHittest){
    FilterBooleanData_cls *pCData = GetPCData();
    if (pCData->mpCSceneObjParam->mbObjectVisible == FALSE) return;

    int wireframe = _WIREFRAME_BOX;
    if (pCData->mCRegionType.mBody == _BOOLEAN_PLANE)
	wireframe = _WIREFRAME_PLANE;
    GetPCData()->GLDrawWireframe(wireframe);
}
