// -*- c++ -*- 
// (setq c-basic-offset 4)

#include "clsFilterCopy.h"
#include "clsFilterCtrl.h"

#include "../MAINCANVAS/clsFilterPanelPain.h"
#include "../PARTICLEDATA/clsPeriodicBoundary.h"
#include "../PARTICLEDATA/clsArrayAndIndex.h"
#include "../POVRAY/clsPovRay.h"
#include "../TIMELINE/clsTimeCtrl.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemFilterParam.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/EX/clsTextEntryDialogEX.h"

#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

extern wxFrame  *GpCMainFrameCLS;

#include <algorithm>

#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

// -----------------------

FilterCopyGUI_cls::							
FilterCopyGUI_cls(FilterCopy_cls *pCParent, 
		  FilterPanelPain_cls *pCPanelPain) : FilterGUI_cls(pCParent) {
    mpCParent = pCParent;
    SetMNumPanel((wxBoxSizer*)pCPanelPain->GetSizer(), pCPanelPain, 0);
}

void FilterCopyGUI_cls::
SetMNumPanel(wxBoxSizer *pBoxSizer, wxWindow *pCParentWindow, int index){
    
    MultiInputPanel_cls* pCurr = InitializeMInputCommon
	(pCParentWindow, mpCParent->GetPCData()->mCstrCustomName.mstrBody);
    FilterCopyData_cls *pCData = mpCParent->GetPCData();

    pCurr->mUpdateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;

    int boxWidth = 2;
    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = 90;
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;

    int indexArray0 = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Array 0"));
    pCurr->CombinePointer(_ITEM_INTEGER, &pCData->mCArrayMin[0].mBody);
    pCurr->CombinePointer(_ITEM_INTEGER, &pCData->mCArrayMax[0].mBody);

    int indexArray1 = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Array 1"));
    pCurr->CombinePointer(_ITEM_INTEGER, &pCData->mCArrayMin[1].mBody);
    pCurr->CombinePointer(_ITEM_INTEGER, &pCData->mCArrayMax[1].mBody);

    int indexArray2 = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Array 2"));
    pCurr->CombinePointer(_ITEM_INTEGER, &pCData->mCArrayMin[2].mBody);
    pCurr->CombinePointer(_ITEM_INTEGER, &pCData->mCArrayMax[2].mBody);
    
    pCurr->CombinePointerSizer(1);
    pCurr->mItemWidth = -1;
    int indexBound = pCurr->CombinePointerButton(wxID_ANY, _("Boundary Size"));
    pCurr->GetLastItem().mAlign = _ITEM_ALIGN_RIGHT;
    
    mIndexExpression = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth);

    pCurr->CombinePointer(_ITEM_BOOL, &pCData->mCbExpression.mbBody, wxT("Use Expression"));
    pCurr->CombinePointerButton(wxID_ANY, _("Editor"));
    pCurr->GetLastItem().mAlign = _ITEM_ALIGN_RIGHT;

    pCurr->FinishSetting(NULL);
    
    SetHookBasic(pCurr);
    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_BUTTON, indexBound);
    pCurr->SetHook(this, &HookItem_cls::HookFunc02, _HOOK_BUTTON, mIndexExpression+2);
}

void FilterCopyGUI_cls::
HookFunc01(int id, void *pArg){
    SendMenuEvent(GpCMainFrameCLS, menuDialogPeriodic);
}

void FilterCopyGUI_cls::
HookFunc02(int id, void *pArg){
    mpCParent->OpenExpressionDialog();
}

// ---------------------

FilterCopyData_cls::
FilterCopyData_cls() : FilterData_cls(FALSE) {
    mCstrCustomName.mstrBody = _("CopyBox");
    mCstrCustomName.mstrBody << wxT("%02d");

    mCConfList.Add(&mCArrayMin[0], wxT("CopyArrayMin0="), wxT("0"));
    mCConfList.Add(&mCArrayMax[0], wxT("CopyArrayMax0="), wxT("1"));
    mCConfList.Add(&mCArrayMin[1], wxT("CopyArrayMin1="), wxT("0"));
    mCConfList.Add(&mCArrayMax[1], wxT("CopyArrayMax1="), wxT("1"));
    mCConfList.Add(&mCArrayMin[2], wxT("CopyArrayMin2="), wxT("0"));
    mCConfList.Add(&mCArrayMax[2], wxT("CopyArrayMax2="), wxT("1"));

    mCConfList.Add(&mCbExpression, wxT("UseExpression="), wxT("0"));
    mCConfList.Add(&mCstrExpression, wxT("Expression="), wxT("return nx,ny,nz,true,true"));
}

// ---------------------

FilterCopy_cls::
FilterCopy_cls(FilterPanelPain_cls *pCPanelPain,
	       FilterCopyData_cls *pCInitData) :
    FilterBase_cls(pCPanelPain->mpCFilterCtrlLink)
{
    wxString strInit; strInit << _FILTER_COPY;
    mCFilterType.SetItem(wxT("FilterType="), strInit);
    mpCData     = new FilterCopyData_cls;
    mpCGUIPanel = new FilterCopyGUI_cls(this, pCPanelPain);

    //mpLua = NULL;
}

FilterCopyData_cls* FilterCopy_cls::
GetPCData(){
    return (FilterCopyData_cls*)mpCData;
}

void FilterCopy_cls::
OpenLua(){
    if (GetPCData()->mCbExpression.mbBody == FALSE) return;

    wxString strFunction;
    strFunction << wxT("function script(index,nx,ny,nz,t,nt)") << ENDL;
    strFunction << GetPCData()->mCstrExpression.mstrBody << ENDL;
    strFunction << wxT("end") << ENDL;
    
    OpenLuaBase(strFunction);
}

bool FilterCopy_cls::
CallLuaValidity(int index, int nx, int ny, int nz, float time, int type){
    if (mpLua == NULL) return TRUE;
    float ntime = Parameters_cls::mspCTimeCtrlLink->GetDoubleIndex();

    lua_getglobal(mpLua,"script"); // ֐ς
    lua_pushnumber(mpLua, index); // ̒lς
    lua_pushnumber(mpLua, nx);   
    lua_pushnumber(mpLua, ny);      
    lua_pushnumber(mpLua, nz);
    lua_pushnumber(mpLua, time);
    lua_pushnumber(mpLua, ntime);

    if (lua_pcall(mpLua, 6, 5, 0) != 0){
	CheckErrorMessage(mpLua, wxT("FilterCopy::"));
	CloseLuaBase();
	return TRUE;
    }

    int stackSize = lua_gettop(mpLua);

    lua_tonumber(mpLua, 1);
    lua_tonumber(mpLua, 2);
    lua_tonumber(mpLua, 3);
    bool bValidParticle = lua_toboolean(mpLua, 4);
    bool bValidObject = lua_toboolean(mpLua, 5);

    if (stackSize < 5) bValidObject = TRUE;
    if (stackSize < 4) bValidParticle = TRUE;
    
    ClearLuaStack();

    if (type == 1) return bValidObject;
    return bValidParticle;
}

void FilterCopy_cls::
CallLuaTranslate(int index, int nx, int ny, int nz, float time,
		 float &fx, float &fy, float &fz){
    if (mpLua == NULL) return;
    float ntime = Parameters_cls::mspCTimeCtrlLink->GetDoubleIndex();
    
    lua_getglobal(mpLua,"script"); // ֐ς
    lua_pushnumber(mpLua, index); // ̒lς
    lua_pushnumber(mpLua, nx); 
    lua_pushnumber(mpLua, ny);      
    lua_pushnumber(mpLua, nz);
    lua_pushnumber(mpLua, time);
    lua_pushnumber(mpLua, ntime);
    
    if (lua_pcall(mpLua, 6, 5, 0) != 0){
	CheckErrorMessage(mpLua, wxT("Copy Filter"));
	CloseLuaBase();
	return;
    }

    int stackSize = lua_gettop(mpLua);

    fx = lua_tonumber(mpLua, 1);
    fy = lua_tonumber(mpLua, 2);
    fz = lua_tonumber(mpLua, 3);
    lua_toboolean(mpLua, 4);
    lua_toboolean(mpLua, 5);
    
    lua_pop(mpLua, 5); // SNA
}

int FilterCopy_cls::
GetNumber(){
    FilterCopyData_cls *pCData = GetPCData();
    if (pCData->mCbValid.mbBody == FALSE) return 1;
    int xnum = pCData->mCArrayMax[0].mBody - pCData->mCArrayMin[0].mBody;
    int ynum = pCData->mCArrayMax[1].mBody - pCData->mCArrayMin[1].mBody;
    int znum = pCData->mCArrayMax[2].mBody - pCData->mCArrayMin[2].mBody;
    return xnum * ynum * znum;
}

int FilterCopy_cls::
GetNumberValid(double time){
    int copyNum = GetNumber();
    int num = 0;
    for (int i = 0; i < copyNum; i++)
	if (GetBValidity(i, time, FALSE, 0) == TRUE) num++;
    return num;
}

int FilterCopy_cls::
GetBitBoundary(int index){
    FilterCopyData_cls *pCData = GetPCData();
    if (pCData->mCbValid.mbBody == FALSE) return 0;
     
    int bit = 7;
    int xnum = pCData->mCArrayMax[X].mBody - pCData->mCArrayMin[X].mBody;
    int ynum = pCData->mCArrayMax[Y].mBody - pCData->mCArrayMin[Y].mBody;
    int znum = pCData->mCArrayMax[Z].mBody - pCData->mCArrayMin[Z].mBody;
    int ix = index % xnum;
    int iy = ((index - ix) / xnum ) % ynum;
    int iz = ((index - ix) - xnum * iy) / (xnum * ynum);

    if (ix == xnum - 1) bit -= 1;
    if (iy == ynum - 1) bit -= 2;
    if (iz == znum - 1) bit -= 4;
    return bit;
}

bool FilterCopy_cls::
GetBValidity(int index, double time, bool bUseSort, int type){
    FilterCopyData_cls *pCData = GetPCData();

    if (pCData->mCbExpression.mbBody == FALSE) return TRUE;

    if (bUseSort == TRUE)
	index = mpCIndexAndZ[index].index;
    
    int xnum = pCData->mCArrayMax[X].mBody - pCData->mCArrayMin[X].mBody;
    int ynum = pCData->mCArrayMax[Y].mBody - pCData->mCArrayMin[Y].mBody;
    int znum = pCData->mCArrayMax[Z].mBody - pCData->mCArrayMin[Z].mBody;
    int ix = index % xnum;
    int iy = ((index - ix) / xnum ) % ynum;
    int iz = ((index - ix) - xnum * iy) / (xnum * ynum);

    ix += pCData->mCArrayMin[X].mBody;
    iy += pCData->mCArrayMin[Y].mBody;
    iz += pCData->mCArrayMin[Z].mBody;

    return CallLuaValidity(index, ix, iy, iz, time, type);
}

std::vector < float > FilterCopy_cls::
GetVTranslate(int index, double time){
    std::vector < float > VRet;
    
    FilterCopyData_cls *pCData = GetPCData();
    if (pCData->mCbValid.mbBody == FALSE) return VRet;
    
    int xnum = pCData->mCArrayMax[X].mBody - pCData->mCArrayMin[X].mBody;
    int ynum = pCData->mCArrayMax[Y].mBody - pCData->mCArrayMin[Y].mBody;
    int znum = pCData->mCArrayMax[Z].mBody - pCData->mCArrayMin[Z].mBody;
    int ix = index % xnum;
    int iy = ((index - ix) / xnum ) % ynum;
    int iz = ((index - ix) - xnum * iy) / (xnum * ynum);

    ix += pCData->mCArrayMin[X].mBody;
    iy += pCData->mCArrayMin[Y].mBody;
    iz += pCData->mCArrayMin[Z].mBody;

    float fx, fy, fz;
    if (pCData->mCbExpression.mbBody == TRUE){
	CallLuaTranslate(index, ix, iy, iz, time, fx, fy, fz);
    }
    else{
	fx = ix;
	fy = iy;
	fz = iz;
    }

    float array[3][3];
    LOOP_XYZ mpCParentCtrl->mpCPeriodicLink->GetBoundaryVector(array[xyz], xyz);
    
    float dx = fx * array[0][X] + fy * array[1][X] + fz * array[2][X];
    float dy = fx * array[0][Y] + fy * array[1][Y] + fz * array[2][Y];
    float dz = fx * array[0][Z] + fy * array[1][Z] + fz * array[2][Z];

    VRet.push_back(dx);
    VRet.push_back(dy);
    VRet.push_back(dz);
    return VRet;
}

void FilterCopy_cls::
ZSortBegin(double cameraPos[3], double cameraDir[3], double time){
    int num = GetNumber();
    if (num < 1) {
	mpCIndexAndZ = NULL;
	return;
    }
    
    mpCIndexAndZ = new ArrayAndIndex_cls[num];

    for (int i = 0; i < num; i++){
	mpCIndexAndZ[i].index = i;
	std::vector < float > VTrans = GetVTranslate(i, time);

	if (VTrans.size() == 0) continue;

	mpCIndexAndZ[i].z = - 
		VTrans[X] * cameraDir[X] - 
		VTrans[Y] * cameraDir[Y] - 
		VTrans[Z] * cameraDir[Z];
	
	/*
	mpCIndexAndZ[i].z = -
	    (VTrans[X]-cameraPos[X]) * (VTrans[X]-cameraPos[X]) -
	    (VTrans[Y]-cameraPos[Y]) * (VTrans[Y]-cameraPos[Y]) -
	    (VTrans[Z]-cameraPos[Z]) * (VTrans[Z]-cameraPos[Z]);
	*/

    }

    std::sort(mpCIndexAndZ, mpCIndexAndZ + num);
}

void FilterCopy_cls::
GLTranslate(int index, double time){
    
    std::vector < float > VTrans = GetVTranslate(mpCIndexAndZ[index].index, time);
    if (VTrans.size() == 0) return;

    glTranslatef(VTrans[X], VTrans[Y], VTrans[Z]);
}

void FilterCopy_cls::
GetCopyTranslate(float *pTranslate, int index, double time){
    std::vector < float > VTrans = GetVTranslate(mpCIndexAndZ[index].index, time);
    if (VTrans.size() == 0) return;
    LOOP_XYZ pTranslate[xyz] = VTrans[xyz];
}

void FilterCopy_cls::
ZSortEnd(){
    if (mpCIndexAndZ != NULL)
	delete [] mpCIndexAndZ;
}

void FilterCopy_cls::
OpenExpressionDialog(){
    wxString strMessage;
    strMessage << wxT("-----") << _("Predefined Variables") << ENDL;
    strMessage << wxT("t = time") << ENDL;
    strMessage << wxT("nt = normalized time") << ENDL;
    strMessage << wxT("index = copy index") << ENDL;
    strMessage << wxT("(nx, ny, nz) = coordinate index") << ENDL;
    strMessage << wxT("-----") << _("Return Value") << ENDL;
    strMessage << wxT("x, y, z, (visibility paticles, visibility objects)") << ENDL;
    strMessage << wxT("x, y, z is normalize by box size") << ENDL;
    strMessage << wxT("visibility (true or false)");
    
    TextEntryDialogEX_cls CDlg(mpCGUIPanel->mpCMNumPanel, strMessage, _("Expression Editor"), GetPCData()->mCstrExpression.mstrBody);

    if (CDlg.ShowModal() == wxID_OK){
	wxString strEdit = CDlg.GetValue().Trim(TRUE);
	FilterCopyData_cls *pCData = GetPCData();
	int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_ATTRIB | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;
	wxString strTag = mpCParentCtrl->GetHeaderStrFromPointer(&pCData->mCstrExpression.mstrBody);
	GCUndoList.PushNewCommand
	    (new CommandFilterValue_cls(mpCParentCtrl, strTag, strEdit, updateBit), 
	     new CommandFilterValue_cls(mpCParentCtrl, strTag, pCData->mCstrExpression.mstrBody, updateBit));
	GCUndoList.ExecCurrent();
    }
}

wxString FilterCopy_cls::
GetStrPovTranslate(int index, double time){
    wxString strTranstale;

    std::vector < float > VTrans = GetVTranslate(index, time);
    if (VTrans.size() == 0) return strTranstale;

    strTranstale << PovRayCtrl_cls::PovOut3f(wxT("translate"), VTrans.data());
    
    return strTranstale;
}
