// (setq c-basic-offset 4)

#include "clsFilterBoolean.h"
#include "clsFilterBG.h"
#include "clsFilterCopy.h"
#include "clsFilterCtrl.h"
#include "clsFilterDataBox.h"
#include "clsFilterFiltering.h"
#include "clsFilterFog.h"
#include "clsFilterGrid.h"
#include "clsFilterObject.h"
#include "clsFilterStick.h"
#include "clsFilterText.h"

#include "../GLATTRIB/clsGLAttribute.h"
#include "../GLCANVAS/clsMoveItem.h"
#include "../PARTICLEDATA/clsArrayAndIndex.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemFilterMake.h"
#include "../MAINCANVAS/clsFilterPanelPain.h"
#include "../MAINCANVAS/clsParamGUICtrl.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"

#include "../../clsMain.h"

#include "../COMMON/CommonDefinitions.h"

FilterCtrl_cls::
FilterCtrl_cls(Main_cls *pCParentMain, int type){
    mType = type;
    mspCParentMainLink = pCParentMain;
}

FilterCtrl_cls::
~FilterCtrl_cls(){
    for (std::list< FilterBase_cls* >::iterator itr = mVpCFilter.begin();
	 itr != mVpCFilter.end(); itr++)
	DeletePolymorph(*itr);
}

void FilterCtrl_cls::
DeletePolymorph(FilterBase_cls *pCFilter){
    switch(pCFilter->mCFilterType.mBody){
    case _FILTER_BG:
	delete (FilterBG_cls*)pCFilter; break;
    case _FILTER_TEXT:
	delete (FilterText_cls*)pCFilter; break;
    case _FILTER_OBJECT:
	delete (FilterObject_cls*)pCFilter; break;
    default:
	delete pCFilter; break;
    }
}

FilterCtrl_cls* FilterCtrl_cls::
GetPCFilterCtrlFromType(int type){
    Main_cls *pC = mspCParentMainLink;
    switch(type){
    case _FILTER_BOOLEAN:  return pC->mpCFilterCtrl[_FTYPE_BOOLEAN];
    case _FILTER_DATABOX:  return pC->mpCFilterCtrl[_FTYPE_WORLD];
    case _FILTER_OBJECT:   return pC->mpCFilterCtrl[_FTYPE_OBJECT];
    case _FILTER_GRID:     return pC->mpCFilterCtrl[_FTYPE_OBJECT];
    case _FILTER_TEXT:     return pC->mpCFilterCtrl[_FTYPE_OBJECT];
    case _FILTER_FOG:      return pC->mpCFilterCtrl[_FTYPE_WORLD];
    case _FILTER_FILTERING:return pC->mpCFilterCtrl[_FTYPE_BOOLEAN];
    case _FILTER_COPY:     return pC->mpCFilterCtrl[_FTYPE_WORLD];
    case _FILTER_STICK:    return pC->mpCFilterCtrl[_FTYPE_OBJECT];
    case _FILTER_BG:       return pC->mpCFilterCtrl[_FTYPE_WORLD];
    }
    return NULL;
}

std::list< FilterBase_cls* >::iterator FilterCtrl_cls::
GetIterN(int index){
    std::list< FilterBase_cls* >::iterator itr = mVpCFilter.begin();
    for (int i = 0; i < index; i++) itr++;
    return itr;
}

FilterBase_cls* FilterCtrl_cls::
GetPCBase(int index){
    return *GetIterN(index);
}

FilterData_cls* FilterCtrl_cls::
GetPCData(int index){
    return GetPCBase(index)->GetPCData();
}

wxString FilterCtrl_cls::
GetHeaderStrFromPointer(void *pValue){
    wxString strHeader;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	wxString strLocalHeader = 
	    GetPCData(i)->GetHeaderStrFromPointer(pValue);
	if ( strLocalHeader != wxT(""))
	    strHeader = GetPCData(i)->mCstrCustomName.mstrBody + wxT("::") << strLocalHeader;
    }
    return strHeader;
}

void FilterCtrl_cls::
SetValue(wxString strTag, double valueDouble, int valueInt, wxString valueStr){
    int pos = strTag.Find(wxT("::"));
    wxString strTagHead = strTag.SubString(0, pos-1);
    wxString strTagBody = strTag.SubString(pos+2, strTag.Len());
    FilterBase_cls *pCFilter = GetPCFilter(strTagHead);
    if (pCFilter == NULL) return;
    pCFilter->GetPCData()->SetValue(strTagBody, valueDouble, valueInt, valueStr);
    pCFilter->mpCGUIPanel->mpCMNumPanel->UpdateValues();
}

void FilterCtrl_cls::
SetParamInit(wxString strTagFilter, wxString strTagParam,
	     int mDimension, double *pValue){
    FilterBase_cls *pCFilter = GetPCFilter(strTagFilter);
    if (pCFilter == NULL) return;
    ParamGroup_cls *pCParamGroup = pCFilter->GetPCData()->
	GetPCParamGroupFromTag(strTagParam);
    if (pCParamGroup == NULL) return;
    if (mDimension != pCParamGroup->mVCParam.size()) return;
    pCParamGroup->SetKeyFrame(pValue, TRUE);
}

void FilterCtrl_cls::
UpdateValuesOnPanel(){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++)
	GetPCBase(i)->mpCGUIPanel->mpCMNumPanel->UpdateValues();
}

void FilterCtrl_cls::
HideAndShowOnPanel(){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	GetPCBase(i)->CallHideAndShow();
    }
}

FilterBase_cls* FilterCtrl_cls::
GetPCFilter(wxString strFilterName){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++)
	if (GetPCData(i)->mCstrCustomName.mstrBody == strFilterName)
	    return GetPCBase(i);
    return NULL;
}

Parameters_cls* FilterCtrl_cls::
GetPCParamFromID(wxString strID){
    Parameters_cls* pCTgt = NULL;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	pCTgt = GetPCData(i)->mpCSceneObjParam->GetPCParamFromID(strID);
	if (pCTgt != NULL) return pCTgt;
    }
    return pCTgt;
}

bool FilterCtrl_cls::
SelectFilter(FilterBase_cls *pCFilterBase){
    bool bChanged = FALSE;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	if (GetPCBase(i) == pCFilterBase)
	    bChanged |= GetPCBase(i)->Select(TRUE);
	else
	    bChanged |= GetPCBase(i)->Select(FALSE);
    }
    return bChanged;
}

void FilterCtrl_cls::
DeleteSelectedFilter(){
    int pos = 0;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	FilterBase_cls *pCBase = GetPCBase(i);
	if (GetPCData(i)->mbSelected == TRUE){
	    GCUndoList.PushNewCommand
		(new CommandFilterMake_cls(this, -1, pos, NULL),
		 new CommandFilterMake_cls(this, pCBase->mCFilterType.mBody, pos, GetPCData(i)));
	    GCUndoList.ExecCurrent();
	    return;
	}
	pos++;
    }
}

void FilterCtrl_cls::
RemoveFilter(int pos){
    mpCPanelPainLink->mpCSizer->Detach((wxPanel*)(*GetIterN(pos))->mpCGUIPanel->mpCMNumPanel);
    ((wxPanel*)GetPCBase(pos)->mpCGUIPanel->mpCMNumPanel)->Destroy();
    mspCParentMainLink->mpCParamGUICtrlLink->RemoveParamGroup(GetPCData(pos)->mpCSceneObjParam);
    mspCParentMainLink->mpCParamGUICtrlLink->mpCSizer->Layout();
    mspCParentMainLink->mpCParamGUICtrlLink->ResizeTL();
    FilterBase_cls *pCFilter = (*GetIterN(pos));
    FilterData_cls *pCFilterData = pCFilter->GetPCData();
    if (MoveItem_cls::mspCSelectedObject == pCFilterData)
	MoveItem_cls::SetSelection(NULL);
    DeletePolymorph(GetPCBase(pos));
    mVpCFilter.erase(GetIterN(pos));
}

void FilterCtrl_cls::
ClearFilters(){
    int num = mVpCFilter.size();
    for (int i = 0; i < num; i++) RemoveFilter(0);
}

void FilterCtrl_cls::
MoveSelectedFilter(int direction){
    int pos = 0;
    int filterNum = (int)mVpCFilter.size();
    for (int i = 0; i < filterNum; i++){
	FilterBase_cls *pCBase = GetPCBase(i);
	if (GetPCData(i)->mbSelected == TRUE &&
	    pos + direction >= 0 &&
	    pos + direction < filterNum) {
	    GCUndoList.PushNewCommand
		(new CommandFilterMove_cls(this, pos, direction),
		 new CommandFilterMove_cls(this, pos+direction, -direction));
	    GCUndoList.ExecCurrent();
	    return;
	}
	pos++;
    }
}

void FilterCtrl_cls::
MoveFilterExe(int pos, int direction){
    std::list < FilterBase_cls* >::iterator itr1 = GetIterN(pos);
    std::list < FilterBase_cls* >::iterator itr2 = GetIterN(pos+direction);
    
    mpCPanelPainLink->mpCSizer->Detach(pos);
    mpCPanelPainLink->mpCSizer->Insert((size_t)(pos+direction), (*itr1)->mpCGUIPanel->mpCMNumPanel, 0, wxEXPAND);

    std::iter_swap(itr1, itr2);
}

wxString FilterCtrl_cls::
GetNewName(wxString strName){
    wxString strNewName;
    for (int i = 0; i < 99; i++){
	strNewName = wxString::Format(strName, i);
	bool bExist = FALSE;
	for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	    FilterBase_cls *pCBase = GetPCBase(i);
	    if (GetPCData(i)->mCstrCustomName.mstrBody == strNewName)
		bExist = TRUE;
	}
	if (bExist == FALSE) break;
    }
    return strNewName;
}

void FilterCtrl_cls::
AddFilter(int type, int pos, FilterData_cls *pCData,
	  ParamGroup_cls *pCParamGroupTranslate,
	  ParamGroup_cls *pCParamGroupRotation,
	  ParamGroup_cls *pCParamGroupScale){
    if (pos == -1) pos = mVpCFilter.size();

    mpCPanelPainLink->Freeze();

    FilterBase_cls *pCFilter;
    switch(type){
    case _FILTER_BOOLEAN:
	pCFilter = new FilterBoolean_cls (mpCPanelPainLink); break;
    case _FILTER_DATABOX:
	pCFilter = new FilterDataBox_cls (mpCPanelPainLink); break;
    case _FILTER_OBJECT:
	pCFilter = new FilterObject_cls  (mpCPanelPainLink); break;
    case _FILTER_GRID:
	pCFilter = new FilterGrid_cls    (mpCPanelPainLink); break;
    case _FILTER_TEXT:
	pCFilter = new FilterText_cls    (mpCPanelPainLink); break;
    case _FILTER_FOG:
	pCFilter = new FilterFog_cls     (mpCPanelPainLink); break;
    case _FILTER_FILTERING:
	pCFilter = new FilterFiltering_cls(mpCPanelPainLink); break;
    case _FILTER_COPY:
	pCFilter = new FilterCopy_cls     (mpCPanelPainLink); break;
    case _FILTER_STICK:
	pCFilter = new FilterStick_cls    (mpCPanelPainLink); break;
    case _FILTER_BG:
	pCFilter = new FilterBG_cls       (mpCPanelPainLink); break;
    }

    if (pCData != NULL)
	pCFilter->GetPCData()->CopyFilterDataFromP(pCData,
						   pCParamGroupTranslate,
						   pCParamGroupRotation,
						   pCParamGroupScale);
    pCData = pCFilter->GetPCData();
    FilterGUI_cls *pCGUIPanel = pCFilter->mpCGUIPanel;

    switch(type){
    case _FILTER_FOG:
	((FilterFogGUI_cls*)pCGUIPanel)->HideAndShow();
	break;
    case _FILTER_STICK:
	((FilterStickGUI_cls*)pCGUIPanel)->HideAndShow();
	break;
    case _FILTER_BG:
	((FilterBGGUI_cls*)pCGUIPanel)->HideAndShow();
	break;
    }
    
    pCGUIPanel->mpCMNumPanel->mbPanelOpen = pCData->mCbGUIPanelOpen.mbBody;
    if (pCData->mCbGUIPanelOpen.mbBody == FALSE)
	pCGUIPanel->mpCMNumPanel->UpdatePanelOpenStatus(3);
    
    if (TRUE || pCData == NULL){
	wxString strNewName = GetNewName(pCData->mCstrCustomName.mstrBody);
	pCData->mCstrCustomName.mstrBody = strNewName;
	pCData->ReplaceParameterName(strNewName);
    }

    pCGUIPanel->UpdateButtonName();
    pCGUIPanel->mpCMNumPanel->UpdateValues();
    
    mVpCFilter.insert(GetIterN(pos), pCFilter);
    mpCPanelPainLink->mpCSizer->Insert((size_t)pos, (wxPanel*)pCFilter->mpCGUIPanel->mpCMNumPanel, 0, wxEXPAND);
    
    // TLɏ悹B
    ParamGroup_cls* pCParamGroup = pCData->mpCSceneObjParam;
    if (pCParamGroup != NULL &&
	pCParamGroup->mbUseTLGUI == TRUE){
	
	mspCParentMainLink->mpCParamGUICtrlLink->AddParamGroup(pCParamGroup);
	pCParamGroup->HideItems();
	
	mspCParentMainLink->mpCParamGUICtrlLink->HideAndShow();
    }
    mpCPanelPainLink->Thaw();
}

bool FilterCtrl_cls::
IsObjectExist(){
    bool bRet = FALSE;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	if (GetPCBase(i)->mCFilterType.mBody != _FILTER_OBJECT &&
	    GetPCBase(i)->mCFilterType.mBody != _FILTER_GRID &&
	    GetPCBase(i)->mCFilterType.mBody != _FILTER_TEXT) continue;
	if (GetPCData(i)->mCbValid.mbBody == TRUE) bRet = TRUE;
    }
    return bRet;
}

bool FilterCtrl_cls::
IsBooleanChangeConstantColor(){
    bool bRet = FALSE;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	if (GetPCBase(i)->mCFilterType.mBody != _FILTER_BOOLEAN) continue;
	FilterBooleanData_cls *pCData = (FilterBooleanData_cls*)GetPCData(i);
	if (pCData->mCbValid.mbBody == TRUE && 
	    pCData->mCBorderFactor.mBody != 0) bRet = TRUE;
    }
    return bRet;
}

int FilterCtrl_cls::
CheckBoolean(ArrayAndIndex_cls* pIndexAndZ, 
	     float (*pX)[3], int downsized, double doubleIndex){
    
    float *pAlphaFactor = new float [downsized];
    for (int i = 0; i < downsized; i++)	pAlphaFactor[i] = 1;

    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	if (GetPCBase(i)->mCFilterType.mBody == _FILTER_BOOLEAN)
	    ((FilterBoolean_cls*)GetPCBase(i))->
		CheckBoolean(pIndexAndZ, pX, downsized, pAlphaFactor, doubleIndex);
	if (GetPCBase(i)->mCFilterType.mBody == _FILTER_FILTERING)
	    ((FilterFiltering_cls*)GetPCBase(i))->
		CheckFiltering(pIndexAndZ, pX, downsized, pAlphaFactor);
    }
    
    int tgt = 0;
    for (int i = 0; i < downsized; i++){
	if (pAlphaFactor[i] > 0){
	    pIndexAndZ[tgt] = pIndexAndZ[i];
	    pIndexAndZ[tgt].alphaFactor = pAlphaFactor[i];
	    tgt++;
	}
    }

    delete [] pAlphaFactor;

    return tgt;
}

SceneObj_cls *FilterCtrl_cls::
GetPCDataBox(){
    if (mVpCFilter.size() == 0) return NULL;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++)
	if (GetPCBase(i)->mCFilterType.mBody == _FILTER_DATABOX){
	    if (GetPCData(i)->mCbValid.mbBody == TRUE)
		return GetPCData(i);
	}
    return NULL;
}

FilterCopy_cls *FilterCtrl_cls::
GetPCCopy(){
    if (mVpCFilter.size() == 0) return NULL;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	FilterBase_cls *pCFilter = (*GetIterN(i));
	if (pCFilter->mCFilterType.mBody == _FILTER_COPY &&
	    pCFilter->GetPCData()->mCbValid.mbBody == TRUE)
	    return (FilterCopy_cls*)GetPCBase(i);
    }
    return NULL;
}

FilterFog_cls *FilterCtrl_cls::
GetPCFog(){
    if (mVpCFilter.size() == 0) return NULL;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	FilterBase_cls *pCFilter = (*GetIterN(i));
	if (pCFilter->mCFilterType.mBody == _FILTER_FOG &&
	    pCFilter->GetPCData()->mCbValid.mbBody == TRUE)
	    return (FilterFog_cls*)GetPCBase(i);
    }
    return NULL;
}

FilterObject_cls *FilterCtrl_cls::
GetPCObject(int index){
    if (index < 0 || index >= mVpCFilter.size()) return NULL;
    FilterBase_cls *pCBase = GetPCBase(index);
    if (pCBase->mCFilterType.mBody != _FILTER_OBJECT) return NULL;
    return (FilterObject_cls*)pCBase;
}

std::vector < FilterStick_cls* > FilterCtrl_cls::
GetVpCSticks(){
    std::vector < FilterStick_cls* > VpCFilters;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++)
	if (GetPCBase(i)->mCFilterType.mBody == _FILTER_STICK)
	    if (GetPCData(i)->mCbValid.mbBody == TRUE)
		VpCFilters.push_back((FilterStick_cls*)GetPCBase(i));
    return VpCFilters;
}

std::vector < FilterBG_cls* > FilterCtrl_cls::
GetVpCBG(){
    std::vector < FilterBG_cls* > VpCFilters;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++)
	if (GetPCBase(i)->mCFilterType.mBody == _FILTER_BG){
	    if (GetPCData(i)->mCbValid.mbBody == TRUE)
		VpCFilters.push_back((FilterBG_cls*)GetPCBase(i));
	}
    return VpCFilters;
}

void FilterCtrl_cls::
UpdateInterpolatedValue(double doubleIndex){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++)
	GetPCData(i)->UpdateCurrentObj(doubleIndex);
}

void FilterCtrl_cls::
OpenLua(){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	FilterBase_cls *pCFilter = GetPCBase(i);
	switch(pCFilter->mCFilterType.mBody){
	case _FILTER_COPY:
	    ((FilterCopy_cls*)pCFilter)->OpenLua();
	    break;
	case _FILTER_OBJECT:
	    ((FilterObject_cls*)pCFilter)->OpenLua();
	    break;
	}
    }
}

void FilterCtrl_cls::
CloseLua(){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	FilterBase_cls *pCFilter = GetPCBase(i);
	switch(pCFilter->mCFilterType.mBody){
	case _FILTER_COPY:
	    ((FilterCopy_cls*)pCFilter)->CloseLuaBase();
	    break;
	case _FILTER_OBJECT:
	    ((FilterObject_cls*)pCFilter)->CloseLuaBase();
	    break;
	}
    }
}

void FilterCtrl_cls::
GLDraw(double doubleIndex, double scale, bool bHittest){
    int filterIndex = 0;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	FilterBase_cls *pCFilter = GetPCBase(i);

	glPushName(filterIndex + 20 * (mType+1)); //

	switch(pCFilter->mCFilterType.mBody){
	case _FILTER_BOOLEAN:
	    ((FilterBoolean_cls*)pCFilter)->GLDraw(doubleIndex, scale, bHittest);
	    break;
	case _FILTER_DATABOX:
	    ((FilterDataBox_cls*)pCFilter)->GLDraw(doubleIndex, scale, bHittest);
	    break;
	case _FILTER_OBJECT:
	    ((FilterObject_cls*)pCFilter)->GLDraw(doubleIndex, scale, bHittest);
	    break;
	}

	glPopName();
	filterIndex++;
    }
}

void FilterCtrl_cls::
GLDrawRealObject(int currentIndex, int lightNumber, bool bCopyInstance,
		 int drawStep, PositionAndDirection_cls CCamPos,
		 int frame){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	switch(GetPCBase(i)->mCFilterType.mBody){
	case _FILTER_OBJECT:
	    ((FilterObject_cls*)GetPCBase(i))->
		GLDrawRealObject(currentIndex, lightNumber,
				 bCopyInstance, drawStep, frame);
	    break;
	}
    }
    
    // ͌ŕ`
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	switch(GetPCBase(i)->mCFilterType.mBody){
	case _FILTER_TEXT:
	    ((FilterText_cls*)GetPCBase(i))->
		GLDrawRealText(bCopyInstance, CCamPos);
	    break;
	}
    }
}

void FilterCtrl_cls::
GLDrawRealGrid(double doubleIndex, PositionAndDirection_cls CCamPos,
	       std::vector < TagObject_cls > *pVCTagObject){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	switch(GetPCBase(i)->mCFilterType.mBody){
	case _FILTER_GRID:
	    ((FilterGrid_cls*)GetPCBase(i))->GLDrawRealGrid
		(doubleIndex, CCamPos, pVCTagObject);
	    break;
	}
    }
}
    
// -----------------------

void FilterCtrl_cls::
GetStrPovInclude(wxString &strPov, GLAttribute_cls **ppCGLAttribute){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	switch(GetPCBase(i)->mCFilterType.mBody){
	case _FILTER_OBJECT:{
	    bool bCalledFromParticles = FALSE;
	    for (int j = 0; j < _MAX_PARTICLE_TYPE; j++){
		bCalledFromParticles |= ppCGLAttribute[j]->IsObjectCalling(i);
	    }
	    ((FilterObject_cls*)GetPCBase(i))->GetStrPovInclude
		(strPov, bCalledFromParticles);
	    break; }
	}
    }
    return; 
}

void FilterCtrl_cls::
GetStrPovBG(wxString &strBG, double doubleIndex, PovRayCtrl_cls *pCPovRayCtrl){
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	FilterBase_cls *pCFilter = GetPCBase(i);
	switch(pCFilter->mCFilterType.mBody){
	    case _FILTER_BG:
		((FilterBG_cls*)pCFilter)->GetStrPovBG
		    (strBG, doubleIndex, pCPovRayCtrl);
	    break;
	}
    }
}

void FilterCtrl_cls::
GetStrPovRealObject(wxString &strObject, 
		    double doubleIndex, bool bCopyInstance, int tabDepth, 
		    PositionAndDirection_cls CCamPos){
    wxString strTab;
    for (int i = 0; i < tabDepth; i++) strTab << wxT("\t");
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	FilterBase_cls *pCFilter = GetPCBase(i);
	switch(pCFilter->mCFilterType.mBody){
	case _FILTER_OBJECT:
	    ((FilterObject_cls*)pCFilter)->
		GetStrPovRealObject(strObject, doubleIndex, mstrInstanceTranslate, bCopyInstance, strTab);
	    break;
	case _FILTER_GRID:
	    ((FilterGrid_cls*)pCFilter)->
		GetStrPovRealGrid(strObject, doubleIndex, bCopyInstance, strTab, CCamPos);
	    break;
	case _FILTER_TEXT:
	    ((FilterText_cls*)pCFilter)->
		GetStrPovRealText(strObject, doubleIndex, bCopyInstance, strTab, CCamPos);
	    break;

	}
    }
    //return strObject;
}

wxString FilterCtrl_cls::
Serialize(){
    wxString strOut;
    strOut << wxT("---FilterCtrl") << mType << wxT("Begin") << ENDL;
    for (unsigned int i = 0; i < mVpCFilter.size(); i++){
	strOut << wxT("-FilterBegin") << ENDL;
	strOut << (*GetIterN(i))->Serialize();
	strOut << wxT("-FilterEnd") << ENDL;
    }
    strOut << wxT("---FilterCtrl") << mType << wxT("End") << ENDL;
    return strOut;
}

void FilterCtrl_cls::
ReadStrLineFilter(std::wistream& istream){
    std::wstring wstrLine;
    std::getline (istream, wstrLine);
    wxString strLine = wstrLine.c_str();
    ConfItemINT_cls mCFilterType(wxT("FilterType="), wxT(""));
    mCFilterType.ReadStrLine(strLine);
    int filterType = mCFilterType.mBody;
    FilterData_cls* pCFilterData;

    switch(filterType){
    case _FILTER_BOOLEAN:
	pCFilterData = new FilterBooleanData_cls; break;
    case _FILTER_DATABOX:
	pCFilterData = new FilterDataBoxData_cls; break;
    case _FILTER_OBJECT:
	pCFilterData = new FilterObjectData_cls; break;
    case _FILTER_GRID:
	pCFilterData = new FilterGridData_cls; break;
    case _FILTER_TEXT:
	pCFilterData = new FilterTextData_cls; break;
    case _FILTER_FOG:
	pCFilterData = new FilterFogData_cls; break;
    case _FILTER_FILTERING:
	pCFilterData = new FilterFilteringData_cls; break;
    case _FILTER_COPY:
	pCFilterData = new FilterCopyData_cls; break;
    case _FILTER_STICK:
	pCFilterData = new FilterStickData_cls; break;
    case _FILTER_BG:
	pCFilterData = new FilterBGData_cls; break;
    }

    pCFilterData->ReadStrLine(istream, wxT(""));
    
    // ǂ̃NXŏĂǂ悤
    GetPCFilterCtrlFromType(filterType)->
	AddFilter(filterType, -1, pCFilterData);
    
    delete pCFilterData;
}

void FilterCtrl_cls::
ReadStrLine(std::wistream& istream, wxString strLineTag){
    std::wstring wstrLine;
    wxString strTagBegin, strTagEnd;
    strTagBegin << wxT("---FilterCtrl") << mType << wxT("Begin");
    strTagEnd   << wxT("---FilterCtrl") << mType << wxT("End");
    if (strLineTag == strTagBegin){
	while (std::getline (istream, wstrLine)){
	    wxString strLine = wstrLine.c_str();
	    if (strLine == wxT("-FilterBegin"))
		ReadStrLineFilter(istream);
	    if (strLine == strTagEnd)
		break;
	}
    }
}

Main_cls *FilterCtrl_cls::mspCParentMainLink = NULL;
