// -*- c++ -*- 
// (setq c-basic-offset 4)

#include "clsFilterDataBox.h"

#include "../GLCANVAS/clsMoveItem.h"
#include "../MAINCANVAS/clsFilterPanelPain.h"
#include "../PARTICLEDATA/clsGLDrawData.h"
#include "../UNDO/clsUndoManager.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"

#include "../../wxWidgetEnum.h"

#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

// -----------------------

FilterDataBoxGUI_cls::							
FilterDataBoxGUI_cls(FilterDataBox_cls *pCParent, 
		     FilterPanelPain_cls *pCPanelPain) : FilterGUI_cls(pCParent) {
    mpCParent = pCParent;
    SetMNumPanel((wxBoxSizer*)pCPanelPain->GetSizer(), pCPanelPain, 0);
}

void FilterDataBoxGUI_cls::
SetMNumPanel(wxBoxSizer *pBoxSizer, wxWindow *pCParentWindow, int index){
    
    MultiInputPanel_cls* pCurr = InitializeMInputCommon
	(pCParentWindow, mpCParent->GetPCData()->mCstrCustomName.mstrBody);
    FilterDataBoxData_cls *pCData = mpCParent->GetPCData();

    pCurr->mUpdateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;
    
    int boxWidth = 3;
    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;

    std::vector < int > VIndex = SetMNumBasic(pCurr, pCData);

    int indexAncor = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("A"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCData->mpCAnchor->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCData->mpCAnchor->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCData->mpCAnchor->mVCParam[Z].mInterpolatedValue);

    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;
    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;
    wxString listType[3] = {_("Particle Only"), _("Particle and Objects"), _("All") };
    pCurr->CombinePointerList(&pCData->mCInfluence.mBody, 3, listType, _("Influence"));

    wxString listRotate[6] = {wxT("Euler XYZ"), wxT("Euler XZY"), wxT("Euler YZX"), wxT("Euler YXZ"), wxT("Euler ZXY"), wxT("Euler ZYX")};
    pCurr->CombinePointerList(&pCData->mCRotateOrder.mBody, 6, listRotate, _("Rotation"));
    
    pCurr->FinishSetting(NULL);

    pCurr->SetColorForXYZ(VIndex[0]+1);
    pCurr->SetColorForXYZ(VIndex[1]+1);
    pCurr->SetColorForXYZ(VIndex[2]+1);
    
    pCurr->SetColorForXYZ(indexAncor+1);

    SetHookBasic(pCurr);
}

// ---------------------

FilterDataBoxData_cls::
FilterDataBoxData_cls() {
    wxString listTag[3] = {wxT("x"), wxT("y"), wxT("z")};
    int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;
    wxString strObjAnchor = wxT("Filter");
    strObjAnchor << wxT(" Anchor");
    mpCAnchor = new ParamGroup_cls(strObjAnchor, 3, listTag, updateBit, FALSE);
    mpCSceneObjParam->mVpCParamGroup.push_back(mpCAnchor);
    
    mCstrCustomName.mstrBody = _("DataBox");
    mCstrCustomName.mstrBody << wxT("%02d");
    
    mCConfList.Add(&mCInfluence, wxT("DataBoxInfluence="), wxT("1"));
    mCConfList.Add(&mCRotateOrder, wxT("RotateOrder="), wxT("0"));
    //mCConfList.Add(&mCRegionType, wxT("RegionType="), wxT("0"));
    //mCConfList.Add(&mCBorderFactor, wxT("BorderFactor="), wxT("0.01"));
}

FilterDataBoxData_cls::
~FilterDataBoxData_cls(){
    delete mpCAnchor;
}

/*
void FilterDataBoxData_cls::
AssignSceneObjParam(){
    mpCAnchor->SetKeyFrameIfChanged();
    SceneObj_cls::AssignSceneObjParam();
}
*/

void FilterDataBoxData_cls::
GetStrPovModelView(wxString &strModelView, double doubleIndex, int objectType){
    
    if (mCInfluence.mBody == _INFLU_PDATA_ONLY && 
	objectType == _OBJECT_NOT_PDATA) return;
    
    std::vector < double > VAnchor = 
	mpCAnchor->GetExpressionValue(doubleIndex, 0);    
    return SceneObj_cls::GetStrPovModelView(strModelView, doubleIndex, VAnchor,
					    mCRotateOrder.mBody);
}

void FilterDataBoxData_cls::
UpdateCurrentObj(double doubleIndex){
    std::vector < double > VPos = 
	mpCTranslate->GetExpressionValue(doubleIndex, 0);
    std::vector < double > VRot = 
	mpCRotation->GetExpressionValue(doubleIndex, 0);
    std::vector < double > VScale = 
	mpCScale->GetExpressionValue(doubleIndex, 0);
    std::vector < double > VAnchor = 
	mpCAnchor->GetExpressionValue(doubleIndex, 0);

    if (mbSelected == TRUE && mbDragging == TRUE){
	LOOP_XYZ VPos[xyz] = MoveItem_cls::msSelectedItemPos[xyz];
	LOOP_XYZ VRot[xyz] += MoveItem_cls::msSelectedItemDRot[xyz];
    }
    
    Position_cls CPos(VPos[X] + VAnchor[X], VPos[Y] + VAnchor[Y], VPos[Z] + VAnchor[Z]);
    mCCurrentObj.Set(CPos);
    double tgt[3] = {1, 0, 0};
    double top[3] = {0, 0, 1};
    mCCurrentObj.Setv(tgt, top);
    mCCurrentObj.NormalizeAndOrthogonalize();
    switch(mCRotateOrder.mBody){
    case _ROTATE_EULER_XYZ:
	mCCurrentObj.RotateYPR(VRot[Z], 0, 0);
	mCCurrentObj.RotateYPR(0, VRot[Y], 0);
	mCCurrentObj.RotateYPR(0, 0, VRot[X]);
	break;
    case _ROTATE_EULER_XZY:
	mCCurrentObj.RotateYPR(0, VRot[Y], 0);
	mCCurrentObj.RotateYPR(VRot[Z], 0, 0);
	mCCurrentObj.RotateYPR(0, 0, VRot[X]);
	break;
    case _ROTATE_EULER_YZX:
	mCCurrentObj.RotateYPR(VRot[Z], 0, 0);
	mCCurrentObj.RotateYPR(0, 0, VRot[X]);
	mCCurrentObj.RotateYPR(0, VRot[Y], 0);
	break;
    case _ROTATE_EULER_YXZ:
	mCCurrentObj.RotateYPR(0, 0, VRot[X]);
	mCCurrentObj.RotateYPR(VRot[Z], 0, 0);
	mCCurrentObj.RotateYPR(0, VRot[Y], 0);
	break;
    case _ROTATE_EULER_ZXY:
	mCCurrentObj.RotateYPR(0, VRot[Y], 0);
	mCCurrentObj.RotateYPR(0, 0, VRot[X]);
	mCCurrentObj.RotateYPR(VRot[Z], 0, 0);
	break;
    case _ROTATE_EULER_ZYX:
	mCCurrentObj.RotateYPR(0, 0, VRot[X]);
	mCCurrentObj.RotateYPR(0, VRot[Y], 0);
	mCCurrentObj.RotateYPR(VRot[Z], 0, 0);
	break;
    }

    mCCurrentObj.UpdateYPR();

    LOOP_XYZ mCCurrentObj.mPos[xyz] -= VAnchor[X] * mCCurrentObj.mTgt[xyz];
    LOOP_XYZ mCCurrentObj.mPos[xyz] += VAnchor[Y] * mCCurrentObj.mRgt[xyz];
    LOOP_XYZ mCCurrentObj.mPos[xyz] -= VAnchor[Z] * mCCurrentObj.mTop[xyz];
    
    LOOP_XYZ mCCurrentObj.mScale[xyz] = VScale[xyz];
}

void FilterDataBoxData_cls::
CopyFilterDataFromP(FilterData_cls *pCFrom,
		    ParamGroup_cls *pCTranslate,
		    ParamGroup_cls *pCRotete   ,
		    ParamGroup_cls *pCScale    ,
		    ParamGroup_cls *pCAdditional){
    if (pCAdditional == NULL) 
	mpCAnchor->CopyFromP
	    (((FilterDataBoxData_cls*)pCFrom)->mpCAnchor);
    else
	mpCAnchor->CopyFromP(pCAdditional);
    FilterData_cls::CopyFilterDataFromP
	(pCFrom, pCTranslate, pCRotete, pCScale);
}

// ---------------------

FilterDataBox_cls::
FilterDataBox_cls(FilterPanelPain_cls *pCPanelPain,
		  FilterDataBoxData_cls *pCInitData) :
    FilterBase_cls(pCPanelPain->mpCFilterCtrlLink)
{
    wxString strInit; strInit << _FILTER_DATABOX;
    mCFilterType.SetItem(wxT("FilterType="), strInit);
    mpCData     = new FilterDataBoxData_cls;
    mpCGUIPanel = new FilterDataBoxGUI_cls(this, pCPanelPain);
}

FilterDataBox_cls::
~FilterDataBox_cls(){
}

FilterDataBoxData_cls* FilterDataBox_cls::
GetPCData(){
    return (FilterDataBoxData_cls*)mpCData;
}

void FilterDataBox_cls::
GLDraw(double doubleIndex, double scale, bool bHittest){
    FilterDataBoxData_cls *pCData = GetPCData();
    if (pCData->mpCSceneObjParam->mbObjectVisible == FALSE) return;
    if (pCData->mCbValid.mbBody == FALSE) return;

    GetPCData()->GLDrawWireframe(0);
}
