// -*- c++ -*- 
// (setq c-basic-offset 4)

#include "clsFilterFiltering.h"

#include "../MAINCANVAS/clsFilterPanelPain.h"
#include "../UNDO/clsUndoManager.h"
#include "../PARTICLEDATA/clsArrayAndIndex.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/EX/clsTextCtrlEX.h"

#include "../../wxWidgetEnum.h"
#include "../COMMON/CommonDefinitions.h"

FilterFilteringGUI_cls::
FilterFilteringGUI_cls(FilterFiltering_cls *pCParent,
		       FilterPanelPain_cls *pCPanelPain) : FilterGUI_cls(pCParent){
    mpCParent = pCParent;
    SetMNumPanel((wxBoxSizer*)pCPanelPain->GetSizer(), pCPanelPain, 0);
}

void FilterFilteringGUI_cls::
SetMNumPanel(wxBoxSizer *pBoxSizer, wxWindow *pCParentWindow, int index){

    MultiInputPanel_cls* pCurr = InitializeMInputCommon
	(pCParentWindow, mpCParent->GetPCData()->mCstrCustomName.mstrBody);
    FilterFilteringData_cls *pCData = mpCParent->GetPCData();

    pCurr->mUpdateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;

    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;

    wxString listType[3] = {_("Show following particles"), _("Hide following particles"), _("Force showing") };
    pCurr->CombinePointerList(&pCData->mCFilteringType.mBody, 3, listType, _("Type"));

    wxString listIndex[2] = {_("Particle index"), _("Particle ID")};
    pCurr->CombinePointerList(&pCData->mCIndexType.mBody, 2, listIndex, _("Classify by"));

    pCurr->mUndoType = _MN_UNDO_MANUAL;
    int indexIndexStr = pCurr->CombinePointer(_ITEM_STRING, &pCData->mCString.mstrBody, _("Particles"));

    pCurr->FinishSetting(NULL);

    ((TextCtrlEX_cls*)(pCurr->mVCItemList[indexIndexStr].GetPCwxWindow()))->EnableHookSpace(FALSE);

    SetHookBasic(pCurr);
}

// ---------------------

FilterFilteringData_cls::
FilterFilteringData_cls() : FilterData_cls(FALSE) {
    mCstrCustomName.mstrBody = _("Filtering");
    mCstrCustomName.mstrBody << wxT("%02d");
    mCConfList.Add(&mCFilteringType, wxT("FilteringType="), wxT("0"));
    mCConfList.Add(&mCIndexType, wxT("IndexType="), wxT("0"));
    mCConfList.Add(&mCString, wxT("FilteringString="), wxT("0"));
}
    
// ---------------------

FilterFiltering_cls::
FilterFiltering_cls(FilterPanelPain_cls *pCPanelPain,
		    FilterFilteringData_cls *pCInitData) :
    FilterBase_cls(pCPanelPain->mpCFilterCtrlLink)
{
    wxString strInit; strInit << _FILTER_FILTERING;
    mCFilterType.SetItem(wxT("FilterType="), strInit);
    mpCData     = new FilterFilteringData_cls;
    mpCGUIPanel = new FilterFilteringGUI_cls(this, pCPanelPain);
    
    mIndexSize = 0;
    mpbAFilterList = NULL;
    mstrAll = wxT("");
}

FilterFiltering_cls::
~FilterFiltering_cls(){
    if (mpbAFilterList != NULL) delete [] mpbAFilterList;
}

FilterFilteringData_cls* FilterFiltering_cls::
GetPCData(){
    return (FilterFilteringData_cls*)mpCData;
}

wxString FilterFiltering_cls::
GetStringBlock(wxString *pstrAll){
    wxString strBlock = *pstrAll;
    int pos = pstrAll->Find(wxT(","));
    if (pos == wxNOT_FOUND){
	//strBlock = *pstrAll;
	(*pstrAll) = wxT("");
	return strBlock;
    }
    strBlock.Truncate(pos);
    (*pstrAll) = pstrAll->Mid(pos+1);
    return strBlock;
}

int FilterFiltering_cls::
GetIni(wxString strBlock){
    int pos = strBlock.Find(wxT("-"));
    if (pos != wxNOT_FOUND)
	strBlock = strBlock.Truncate(pos);

    strBlock.Trim(FALSE);
    strBlock.Trim(TRUE);
    if (strBlock.IsNumber() == FALSE) return 0;
    long value;
    strBlock.ToLong(&value);
    return value;
}

int FilterFiltering_cls::
GetFin(wxString strBlock){
    int pos = strBlock.Find(wxT("-"));
    if (pos == wxNOT_FOUND) return -1;
    strBlock = strBlock.Mid(pos+1);
    
    strBlock.Trim(FALSE);
    strBlock.Trim(TRUE);
    if (strBlock.IsNumber() == FALSE) return -1;
    long value;
    strBlock.ToLong(&value);
    return value;
}

void FilterFiltering_cls::
MakeFilterList(){
    wxString strAll = GetPCData()->mCString.mstrBody;

    if (strAll == mstrAll) return;

    mstrAll = strAll;

    int indexMax = 0;
    while (strAll != wxT("")){
	wxString strBlock = GetStringBlock(&strAll);
	int ini = GetIni(strBlock);
	int fin = GetFin(strBlock);

	if (fin == -1)
	    if (indexMax < ini+1) indexMax = ini+1;
	if (indexMax < fin+1) indexMax = fin+1;
    }

    if (indexMax != mIndexSize){
	if (mpbAFilterList != NULL){
	    delete [] mpbAFilterList;
	    mpbAFilterList = NULL;
	}
	
    }
    
    if (indexMax != 0 && mpbAFilterList == NULL)
	mpbAFilterList = new bool [indexMax];
    
    for (int i = 0; i < indexMax; i++) mpbAFilterList[i] = FALSE;
    mIndexSize = indexMax;
    
    strAll = mstrAll;
    while (strAll != wxT("")){
	wxString strBlock = GetStringBlock(&strAll);
	int ini = GetIni(strBlock);
	int fin = GetFin(strBlock);

	if (fin == -1) 
	    mpbAFilterList[ini] = TRUE;
	else
	    for (int i = ini; i <= fin; i++)
		mpbAFilterList[i] = TRUE;
    }
}

void FilterFiltering_cls::
CheckFiltering(ArrayAndIndex_cls* pIndexAndZ,
	       float (*pX)[3], int downsized, float *pAlphaFactor){
    FilterFilteringData_cls *pCData = GetPCData();

    if (pCData->mCbValid.mbBody == FALSE) return;
    
    MakeFilterList();
    
    switch(pCData->mCFilteringType.mBody){
    case 0: {
	for (int i = 0; i < downsized; i++){
	    int index = pIndexAndZ[i].index;
	    if (index >= mIndexSize)
		pAlphaFactor[i] *= 0;
	    else
		pAlphaFactor[i] *= mpbAFilterList[index];
	}
	break; }
    case 1: {
	for (int i = 0; i < downsized; i++){
	    int index = pIndexAndZ[i].index;
	    if (index >= mIndexSize)
		pAlphaFactor[i] *= 1;
	    else
		pAlphaFactor[i] *= !mpbAFilterList[index];
	}
	break; }
    case 2: {
	for (int i = 0; i < downsized; i++){
	    int index = pIndexAndZ[i].index;
	    if (index >= mIndexSize)
		pAlphaFactor[i] *= 0;
	    else
		if (mpbAFilterList[index] == TRUE)
		    pAlphaFactor[i] = 1;
	}
	break; }
    }
}
