// -*- c++ -*- 
// (setq c-basic-offset 4)

#include "clsFilterGrid.h"

#include "../GLOBJECTS/clsCircle.h"
#include "../GLOBJECTS/clsCube.h"
#include "../GLOBJECTS/clsGrid.h"
#include "../GLOBJECTS/clsSphere.h"
#include "../GLCANVAS/clsMainGLCanvas.h"
extern Sphere_cls *GpCSphere;
#include "../MAINCANVAS/clsFilterPanelPain.h"
#include "../PARTICLEDATA/clsGLDrawData.h"
#include "../POVRAY/clsPovRay.h"
#include "../UNDO/clsUndoManager.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"

#include "../../wxWidgetEnum.h"

#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

// -----------------------

FilterGridGUI_cls::							
FilterGridGUI_cls(FilterGrid_cls *pCParent, 
		     FilterPanelPain_cls *pCPanelPain) : FilterGUI_cls(pCParent) {
    mpCParent = pCParent;
    SetMNumPanel((wxBoxSizer*)pCPanelPain->GetSizer(), pCPanelPain, 0);
}

void FilterGridGUI_cls::
SetMNumPanel(wxBoxSizer *pBoxSizer, wxWindow *pCParentWindow, int index){
    
    MultiInputPanel_cls* pCurr = InitializeMInputCommon
	(pCParentWindow, mpCParent->GetPCData()->mCstrCustomName.mstrBody);
    FilterGridData_cls *pCData = mpCParent->GetPCData();
    
    pCurr->mUpdateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;

    int boxWidth = 2;
    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;
    
    wxString listGrid[6] = {_("Cartesian(x-y)"), _("Cartesian(y-z)"), _("Cartesian(x-z)"), _("Cubic"), _("Circle"), _("Spherical")};
    pCurr->CombinePointerList(&pCData->mCGridType.mBody, 6, listGrid, _("Grid Type"));

    pCurr->CombinePointer(_ITEM_BOOL, &pCData->mCbCoordTag.mbBody, _("Coordinate"));
    
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;
        
    //std::vector < int > VIndex = SetMNumBasic(pCurr, pCData);

    pCurr->mTextWidth = 90;
    pCurr->mItemWidth = -1;
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;

    //pCurr->CombinePointer(_ITEM_FLOAT, &pCData->mCBorderFactor.mBody, _("Border"));
    
    boxWidth = 3;
    pCurr->mItemWidth = 55;
    mIndexFlatParam = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Min/Max/Num"));
    pCurr->CombinePointer(_ITEM_FLOAT, &pCData->mCGridMin.mBody);
    pCurr->CombinePointer(_ITEM_FLOAT, &pCData->mCGridMax.mBody);
    pCurr->CombinePointer(_ITEM_INTEGER, &pCData->mCGridLineNum.mBody);

    boxWidth = 1;
    pCurr->mItemWidth = -1;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Color"));
    ColorCurveBtn_cls *pCBtnColor = new ColorCurveBtn_cls
	(pCurr, pCData->mVpCClrCurve[0], TRUE, FALSE);
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnColor, wxT("Color"));
    
    pCurr->FinishSetting(NULL);
    
    //pCurr->SetColorForXYZ(VIndex[0]+1);
    //pCurr->SetColorForXYZ(VIndex[1]+1);
    //pCurr->SetColorForXYZ(VIndex[2]+1);
    
    SetHookBasic(pCurr);
}

// ---------------------

FilterGridData_cls::
FilterGridData_cls() {
    mCstrCustomName.mstrBody = _("Grid");
    mCstrCustomName.mstrBody << wxT("%02d");
    mCConfList.Add(&mCGridType, wxT("GridType="), wxT("0"));
    mCConfList.Add(&mCGridMin, wxT("GridMin="), wxT("-1"));
    mCConfList.Add(&mCGridMax, wxT("GridMax="), wxT("+1"));
    mCConfList.Add(&mCGridLineNum, wxT("GridLineNum="), wxT("20"));
    mCConfList.Add(&mCbCoordTag, wxT("GridCoord="), wxT("1"));

    ColorCurve_cls *pCClrCurve = new ColorCurve_cls
	(wxT("Color"), wxT("Color"), wxT("0.5 0.5 0.5 1"));
    
    mVpCClrCurve.push_back(pCClrCurve);
    //mVpCClrCurve.push_back(pCClrCurveDiffuse);
    //mVpCClrCurve.push_back(pCClrCurveSpeculer);
}

// ---------------------

FilterGrid_cls::
FilterGrid_cls(FilterPanelPain_cls *pCPanelPain,
		  FilterGridData_cls *pCInitData) :
    FilterBase_cls(pCPanelPain->mpCFilterCtrlLink)
{
    wxString strInit; strInit << _FILTER_GRID;
    mCFilterType.SetItem(wxT("FilterType="), strInit);
    mpCData     = new FilterGridData_cls;
    mpCGUIPanel = new FilterGridGUI_cls(this, pCPanelPain);
}

FilterGrid_cls::
~FilterGrid_cls(){
}

FilterGridData_cls* FilterGrid_cls::
GetPCData(){
    return (FilterGridData_cls*)mpCData;
}

void FilterGrid_cls::
GLDraw(double doubleIndex, double scale, bool bHittest){
    FilterGridData_cls *pCData = GetPCData();
    if (pCData->mpCSceneObjParam->mbObjectVisible == FALSE) return;
    
    glPushMatrix();
    ParamGroup_cls *pCTranslate = pCData->mpCTranslate;
    ParamGroup_cls *pCRotate    = pCData->mpCRotation;
    ParamGroup_cls *pCScale     = pCData->mpCScale;

    if (GetPCData()->mbSelected == FALSE)
	glColor4f(0.3f, 0.8f, 0.3f, 1.0f);
    else
	glColor4f(0.8f, 0.7f, 0.3f, 1.0f);

    std::vector < double > VTrans = 
	pCTranslate->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON);
    glTranslatef(VTrans[0], VTrans[1], VTrans[2]);

    std::vector < double > VRot = 
	pCRotate->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON);
    
    glRotatef(VRot[2], 0, 0, 1);
    glRotatef(VRot[0], 1, 0, 0);
    glRotatef(VRot[1], 0, 1, 0);

    std::vector < double > VScale = 
	pCScale->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON);
    glScalef(VScale[0], VScale[1], VScale[2]);
    
    glBegin(GL_LINES);
    glVertex3d(-0.5, -0.5, -0.5);
    glVertex3d(-0.5, -0.5, +0.5);
    glVertex3d(+0.5, -0.5, -0.5);
    glVertex3d(+0.5, -0.5, +0.5);
    glVertex3d(+0.5, +0.5, -0.5);
    glVertex3d(+0.5, +0.5, +0.5);
    glVertex3d(-0.5, +0.5, -0.5);
    glVertex3d(-0.5, +0.5, +0.5);
    glEnd();

    glBegin(GL_LINE_LOOP);
    glVertex3d(-0.5, -0.5, -0.5);
    glVertex3d(+0.5, -0.5, -0.5);
    glVertex3d(+0.5, +0.5, -0.5);
    glVertex3d(-0.5, +0.5, -0.5);
    glEnd();

    glBegin(GL_LINE_LOOP);
    glVertex3d(-0.5, -0.5, +0.5);
    glVertex3d(+0.5, -0.5, +0.5);
    glVertex3d(+0.5, +0.5, +0.5);
    glVertex3d(-0.5, +0.5, +0.5);
    glEnd();

    glPopMatrix();
}

void FilterGrid_cls::
GLDrawRealGrid(double doubleIndex, PositionAndDirection_cls CCamPos,
	       std::vector < TagObject_cls > *pVCTagObject){
    
    FilterGridData_cls *pCData = GetPCData();
    if (pCData->mCbValid.mbBody == FALSE) return;
    if (pCData->mCbVisible.mbBody == FALSE) return;

    glPushMatrix();

    ParamGroup_cls *pCTranslate = pCData->mpCTranslate;
    ParamGroup_cls *pCRotate    = pCData->mpCRotation;
    ParamGroup_cls *pCScale     = pCData->mpCScale;
    
    std::vector < double > VTrans = 
	pCTranslate->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON);
    glTranslatef(VTrans[0], VTrans[1], VTrans[2]);

    std::vector < double > VRot = 
	pCRotate->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON);
    
    float color[4];
    pCData->mVpCClrCurve[0]->GetColor4fNormal(color,  0);
    glColor4fv(color);

    glRotatef(VRot[2], 0, 0, 1);
    glRotatef(VRot[0], 1, 0, 0);
    glRotatef(VRot[1], 0, 1, 0);

    std::vector < double > VScale = 
	pCScale->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON);

    glScalef(VScale[0], VScale[1], VScale[2]);
    //glScalef(0.5, 0.5, 0.5);

    if (GetPCData()->mCbCoordTag.mbBody == FALSE) pVCTagObject = NULL;
    
    glEnable(GL_NORMALIZE);
    switch(pCData->mCGridType.mBody){
    case _GRID_XY:
	Grid_cls::DrawBasicGridZ(pCData->mCGridMin.mBody, 
				 pCData->mCGridMax.mBody, 
				 pCData->mCGridLineNum.mBody, 0, pVCTagObject);
	break;
    case _GRID_YZ:
	Grid_cls::DrawBasicGridX(pCData->mCGridMin.mBody, 
				 pCData->mCGridMax.mBody, 
				 pCData->mCGridLineNum.mBody, 0, pVCTagObject);
	break;
    case _GRID_ZX:
	Grid_cls::DrawBasicGridY(pCData->mCGridMin.mBody, 
				 pCData->mCGridMax.mBody, 
				 pCData->mCGridLineNum.mBody, 0, pVCTagObject);
	break;
    case _GRID_CUBIC:
	Grid_cls::DrawCubicGrid(pCData->mCGridMin.mBody, 
				pCData->mCGridMax.mBody, 
				pCData->mCGridLineNum.mBody,
				CCamPos, pVCTagObject);
	break;
    case 4:
	Circle_cls::DrawCircles(pCData->mCGridMin.mBody, 
				pCData->mCGridMax.mBody, 
				pCData->mCGridLineNum.mBody,
				CCamPos, pVCTagObject);
	break;
    case 5:
	GpCSphere->Draw(3);
	break;
    }

    glPopMatrix();
}

void FilterGrid_cls::
GetStrPovRealGrid(wxString &strObject, double doubleIndex, bool bCopyInstance, wxString strTab, PositionAndDirection_cls CCamPos){
    if (bCopyInstance == TRUE) return;
    
    FilterGridData_cls *pCData = GetPCData();
    float min = pCData->mCGridMin.mBody, max = pCData->mCGridMax.mBody;
    int lineNum = pCData->mCGridLineNum.mBody;

    float lineWidth = MainGLCanvas_cls::msCLineWidth.mBody * 0.002;

    strObject << strTab << wxT("union {") << ENDL;
    mCPovGrid.mstrTab = strTab + wxT("\t");
    switch(pCData->mCGridType.mBody){
    case _GRID_XY:
	mCPovGrid.GetStrPovBasicGridZ
	    (strObject, min, max, lineNum, 0, lineWidth);
	break;
    case _GRID_YZ:
	mCPovGrid.GetStrPovBasicGridX
	    (strObject, min, max, lineNum, 0, lineWidth);
	break;
    case _GRID_ZX:
	mCPovGrid.GetStrPovBasicGridY
	    (strObject, min, max, lineNum, 0, lineWidth);
	break;
    case _GRID_CUBIC:
	mCPovGrid.GetStrPovBasicGridCubic
	    (strObject, min, max, lineNum, lineWidth, CCamPos);
	break;
    case _GRID_CIRCULAR:
	mCPovGrid.GetStrPovGridCircular
	    (strObject, min, max, lineNum, lineWidth, CCamPos);
	break;
    }

    float ambient[4];
    pCData->mVpCClrCurve[0]->GetColor4fNormal(ambient, 0);
    mCPovGrid.GetStrPovTexture(strObject, ambient);
    strObject << strTab << ENDLCLOSE;
}

