// -*- c++ -*- 
// (setq c-basic-offset 4)

#include "clsFilterObject.h"

#include "clsFilterCtrl.h"

#include "../GLATTRIB/clsGLAttribute.h"
#include "../GLATTRIB/clsShaderCtrl.h"
#include "../GLOBJECTS/clsCube.h"
extern Cube_cls *GpCCube;
#include "../GLOBJECTS/clsSphere.h"
extern Sphere_cls *GpCSphere;
#include "../MAINCANVAS/clsFilterPanelPain.h"
#include "../PARTICLEDATA/clsGLDrawData.h"
#include "../POVRAY/clsPovRay.h"
#include "../POVRAY/clsPovMaterialColor.h"
#include "../POVRAY/OPTIONAL/clsPovFormat.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemFilterParam.h"
#include "../UNDO/clsUndoItemConstParam.h"
#include "../WIDGETCTRL/clsBitmapPanel.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/EX/clsTextEntryDialogEX.h"

#include "../WIDGETCTRL/EX/clsFileDialogEX.h"
extern wxFileDialogEX *GpCFileDialog;

#include <wx/filename.h>

#include "../../clsMain.h"
#include "../../wxWidgetEnum.h"

#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

// -----------------------

FilterObjectGUI_cls::							
FilterObjectGUI_cls(FilterObject_cls *pCParent, 
		     FilterPanelPain_cls *pCPanelPain) : FilterGUI_cls(pCParent) {
    mpCParent = pCParent;
    SetMNumPanel((wxBoxSizer*)pCPanelPain->GetSizer(), pCPanelPain, 0);
    HideAndShow();
}

void FilterObjectGUI_cls::
SetMNumPanel(wxBoxSizer *pBoxSizer, wxWindow *pCParentWindow, int index){
    
    MultiInputPanel_cls* pCurr = InitializeMInputCommon
	(pCParentWindow, mpCParent->GetPCData()->mCstrCustomName.mstrBody);
    FilterObjectData_cls *pCData = mpCParent->GetPCData();
    
    pCurr->mUpdateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_TEXTURE ;
    
    pCurr->mItemWidth = -1;
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;
    
    pCurr->mTextWidth = 120;
    pCurr->CombinePointerSizer(3);
    pCurr->CombinePointer(_ITEM_BOOL, &pCData->mCbDrawLater.mbBody, wxT("Later"));
    pCurr->CombinePointer(_ITEM_BOOL, &pCData->mCbCullFace.mbBody, _("Culling"));
    pCurr->CombinePointer(_ITEM_BOOL, &pCData->mCbWorld.mbBody, _("World"));
    
    
    pCurr->mTextWidth = 70;
    mIndexType = pCurr->CombinePointerSizer(2, _("Form"));
    
    wxString listObject[4] = {wxT("Box"), wxT("Plane"), wxT("Sphere"), wxT("Import")};
    pCurr->CombinePointerList(&pCData->mCObjectType.mBody, 4, listObject);
    pCurr->mItemWidth = 100;
    mIndexArrayVector = pCurr->CombinePointerButton(wxID_ANY, _("Affine map"));
    pCurr->mItemWidth = -1;
    
    mIndexPath = pCurr->CombinePointerSizer(2);
    
    pCurr->mItemWidth = pCurr->mMinimumWidth - 30 + 10;
    pCurr->mUndoType = _MN_UNDO_MANUAL;
    pCurr->CombinePointer(_ITEM_STRING, &pCData->mCstrImportPath.mstrBody);
    pCurr->mItemWidth = 30;
    pCurr->CombinePointerButton(wxID_ANY, wxT(".."));

    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;
			  
    pCurr->mItemWidth = 86;
    mIndexMultiple = pCurr->CombinePointerSizer(3);
    
    pCurr->CombinePointer(_ITEM_BOOL, &pCData->mCbMultipleMesh.mbBody, _("Multiple"));
    wxString listGrid[3] = {wxT("1x1"), wxT("2x2"), wxT("4x4")};
    
    pCurr->CombinePointerList(&pCData->mCMeshGrid.mBody, 3, listGrid);
    mIndexExpression = pCurr->CombinePointerButton(wxID_ANY, _("Editor"));

    mpCFolderXYZ = new BitmapPanel_cls(pCurr, wxID_ANY, wxDP, wxSize(12, 12));
    mpCFolderXYZ->mbButton = TRUE;
    mpCFolderXYZ->AddBitmapStr(wxT("wxART_OPEN_FOLDER"));
    mpCFolderXYZ->AddBitmapStr(wxT("wxART_CLOSE_FOLDER"));
    int indexFolderXYZ = pCurr->CombinePointer(_ITEM_WXPANEL, mpCFolderXYZ);
    pCurr->ConnectButton(pCurr, mpCFolderXYZ->GetId());
    
    pCurr->mItemWidth = -1;
    std::vector < int > VIndex = SetMNumBasic(pCurr, pCData);
    mIndexTranslate = VIndex[0];
    mIndexRotate    = VIndex[1];
    mIndexScale     = VIndex[2];

    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;
    pCurr->mTextWidth = 70;
    pCurr->mItemWidth = -1;
    mIndexMaterial = pCurr->CombinePointerSizer(4);

    pCurr->mComboWidth = 120;
    wxString strListMaterial[2] = {_("Material Color"), _("Original Color")};
    mIndexMaterialMode = pCurr->CombinePointerList(&pCData->mCMaterialType.mBody, 2, strListMaterial);
    
    ColorCurveBtn_cls *pCBtnAmbient = new ColorCurveBtn_cls
	(pCurr, pCData->mVpCClrCurve[0], TRUE, FALSE);
    ColorCurveBtn_cls *pCBtnDiffuse = new ColorCurveBtn_cls
	(pCurr, pCData->mVpCClrCurve[1], TRUE, FALSE);
    ColorCurveBtn_cls *pCBtnSpecular = new ColorCurveBtn_cls
	(pCurr, pCData->mVpCClrCurve[2], TRUE, FALSE);
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnAmbient, wxT("Ambient"));
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnDiffuse, wxT("Diffuse"));
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnSpecular, wxT("Specular"));
    
    pCurr->FinishSetting(NULL);
    
    pCurr->SetColorForXYZ(VIndex[0]+1);
    pCurr->SetColorForXYZ(VIndex[1]+1);
    pCurr->SetColorForXYZ(VIndex[2]+1);

    SetHookBasic(pCurr);

    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX, mIndexType+1);
    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX, mIndexMaterialMode);
    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_BUTTON, indexFolderXYZ);
    
    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_BUTTON, mIndexPath+2);
    pCurr->SetHook(this, &HookItem_cls::HookFunc02, _HOOK_BUTTON, mIndexArrayVector);
    pCurr->SetHook(this, &HookItem_cls::HookFunc03, _HOOK_BUTTON, mIndexExpression);
}

void FilterObjectGUI_cls::
HookFuncHS(int id, void *pArg){
    HideAndShow();
}

void FilterObjectGUI_cls::
HookFunc01(int id, void *pArg){
    PathSelection();
}

void FilterObjectGUI_cls::
HookFunc02(int id, void *pArg){
    ArrayVector();
}

void FilterObjectGUI_cls::
HookFunc03(int id, void *pArg){
    mpCParent->OpenExpressionDialog();
}

void FilterObjectGUI_cls::
HideAndShow(){
    HideAndShowBegin();

    bool bFolderShow = (mpCFolderXYZ->mState == 0);
    if (bFolderShow == TRUE){
	mpCMNumPanel->ShowItem(mIndexTranslate, TRUE);
	mpCMNumPanel->ShowItem(mIndexRotate, TRUE);
	mpCMNumPanel->ShowItem(mIndexScale, TRUE);
	mpCMNumPanel->ShowItem(mIndexMaterial, TRUE);
    }
    else{
	mpCMNumPanel->ShowItem(mIndexTranslate, FALSE);
	mpCMNumPanel->ShowItem(mIndexRotate, FALSE);
	mpCMNumPanel->ShowItem(mIndexScale, FALSE);
	mpCMNumPanel->ShowItem(mIndexMaterial, FALSE);
    }

    FilterObjectData_cls *pCData = mpCParent->GetPCData();
    if (pCData->mCObjectType.mBody == _FILTER_OBJECT_IMPORT &&
	mpCMNumPanel->mbPanelOpen == TRUE){
	mpCMNumPanel->ShowItem(mIndexPath, TRUE);
	mpCMNumPanel->ShowItem(mIndexMultiple, TRUE);
	mpCMNumPanel->ShowItem(mIndexMaterialMode, TRUE & bFolderShow);
    }
    else{
	mpCMNumPanel->ShowItem(mIndexPath, FALSE);
	mpCMNumPanel->ShowItem(mIndexMultiple, FALSE);
	mpCMNumPanel->ShowItem(mIndexMaterialMode, FALSE);
    }

    HideAndShowEnd();
}

void FilterObjectGUI_cls::
PathSelection(){
    GpCFileDialog->SetMessage(wxT("Select Mesh File"));
    GpCFileDialog->SetWildcard(wxT("3DMolel|*.pov;*.obj|PovMesh (*.pov)|*.pov|Wavefront Object (*.obj)|*.obj"));
    wxFileName CName = FilterCtrl_cls::mspCParentMainLink->mpCFrame->mCLastOpenedFileOBJ.mstrBody;
    GpCFileDialog->SetInitPath(CName.GetPath(), CName.GetFullName());
    
    if (GpCFileDialog->ShowModal() == wxID_CANCEL) return;
    
    FilterObjectData_cls *pCData = mpCParent->GetPCData();
    int updateBit = _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_GL;

    GCUndoList.PushNewCommand
	(new CommandSetValue_cls(&pCData->mCstrImportPath.mstrBody, GpCFileDialog->GetPath(), updateBit), 
	 new CommandSetValue_cls(&pCData->mCstrImportPath.mstrBody, pCData->mCstrImportPath.mstrBody, updateBit));
    GCUndoList.ExecCurrent();

    FilterCtrl_cls::mspCParentMainLink->mpCFrame->mCLastOpenedFileOBJ.mstrBody = GpCFileDialog->GetPath();
	
    //mpCMNumPanel->UpdateValues();
}

void FilterObjectGUI_cls::
ArrayVector(){
    MultiInputDialog_cls *pCDLG = new
	MultiInputDialog_cls(mpCMNumPanel, 
			     wxID_ANY, wxT("Array Vector"), wxDP);
    MultiInputPanel_cls *pCurr = pCDLG->mpCPanel;
    FilterObjectData_cls *pCData = mpCParent->GetPCData();
    
    float array0[3], array1[3], array2[3];
    LOOP_XYZ{
	array0[xyz] = pCData->mCArray0Vector.mpBody[xyz];
	array1[xyz] = pCData->mCArray1Vector.mpBody[xyz];
	array2[xyz] = pCData->mCArray2Vector.mpBody[xyz];
    }

    int indexXaxis = pCurr->CombinePointerSizer(3, wxT("X-axis"));
    pCurr->CombinePointer(_ITEM_FLOAT, &array0[X]);
    pCurr->CombinePointer(_ITEM_FLOAT, &array0[Y]);
    pCurr->CombinePointer(_ITEM_FLOAT, &array0[Z]);
    int indexYaxis = pCurr->CombinePointerSizer(3, wxT("Y-axis"));
    pCurr->CombinePointer(_ITEM_FLOAT, &array1[X]);
    pCurr->CombinePointer(_ITEM_FLOAT, &array1[Y]);
    pCurr->CombinePointer(_ITEM_FLOAT, &array1[Z]);

    int indexZaxis = pCurr->CombinePointerSizer(3, wxT("Z-axis"));
    pCurr->CombinePointer(_ITEM_FLOAT, &array2[X]);
    pCurr->CombinePointer(_ITEM_FLOAT, &array2[Y]);
    pCurr->CombinePointer(_ITEM_FLOAT, &array2[Z]);

    pCDLG->MakeGeometry();

    pCurr->SetColorForXYZ(indexXaxis+1);
    pCurr->SetColorForXYZ(indexYaxis+1);
    pCurr->SetColorForXYZ(indexZaxis+1);

    if (pCDLG->ShowModal() == wxID_CANCEL) return;

    bool bDifferent = FALSE;
    LOOP_XYZ{
	bDifferent |= (array0[xyz] != pCData->mCArray0Vector.mpBody[xyz]);
	bDifferent |= (array1[xyz] != pCData->mCArray1Vector.mpBody[xyz]);
	bDifferent |= (array2[xyz] != pCData->mCArray2Vector.mpBody[xyz]);
    }
    if (bDifferent == FALSE) return;

    GCUndoList.OpenBlock();
    
    float *pOrgValueFloat;
    int updateBit = _COMMAND_UPDATE_DISPLAY_EXPOSURE | _COMMAND_UPDATE_DISPLAY_GL;
    LOOP_XYZ{
	pOrgValueFloat = &pCData->mCArray0Vector.mpBody[xyz];
	GCUndoList.PushNewCommand
	    (new CommandSetValue_cls(pOrgValueFloat, array0[xyz], updateBit),
	     new CommandSetValue_cls(pOrgValueFloat, *pOrgValueFloat, updateBit));
	GCUndoList.ExecCurrent();
    }

    LOOP_XYZ{
	pOrgValueFloat = &pCData->mCArray1Vector.mpBody[xyz];
	GCUndoList.PushNewCommand
	    (new CommandSetValue_cls(pOrgValueFloat, array1[xyz], updateBit),
	     new CommandSetValue_cls(pOrgValueFloat, *pOrgValueFloat, updateBit));
	GCUndoList.ExecCurrent();
    }

    LOOP_XYZ{
	pOrgValueFloat = &pCData->mCArray2Vector.mpBody[xyz];
	GCUndoList.PushNewCommand
	    (new CommandSetValue_cls(pOrgValueFloat, array2[xyz], updateBit),
	     new CommandSetValue_cls(pOrgValueFloat, *pOrgValueFloat, updateBit));
	GCUndoList.ExecCurrent();
    }

    GCUndoList.CloseBlock();
}

// ---------------------

FilterObjectData_cls::
FilterObjectData_cls() {
    mCstrCustomName.mstrBody = _("Object");
    mCstrCustomName.mstrBody << wxT("%02d");
    
    mCConfList.Add(&mCObjectType, wxT("ObjectType="), wxT("0"));
    mCConfList.Add(&mCMaterialType, wxT("MaterialType="), wxT("0"));
    mCConfList.Add(&mCstrImportPath, wxT("ImportPath="), wxT("NONE"));
    mCConfList.Add(&mCbDrawLater, wxT("DrawLater="), wxT("0"));
    mCConfList.Add(&mCbCullFace, wxT("CullFace="), wxT("1"));
    mCConfList.Add(&mCbWorld, wxT("WorldCoord="), wxT("1"));
    mCConfList.Add(&mCbMultipleMesh, wxT("MultipleMesh="), wxT("0"));
    mCConfList.Add(&mCMeshGrid, wxT("MeshGrid="), wxT("0"));
    mCConfList.Add(&mCstrExpression, wxT("Expression"), wxT("return 0"));

    mCConfList.Add(&mCArray0Vector, wxT("ArrayVector0="), wxT("1 0 0"));
    mCConfList.Add(&mCArray1Vector, wxT("ArrayVector1="), wxT("0 1 0"));
    mCConfList.Add(&mCArray2Vector, wxT("ArrayVector2="), wxT("0 0 1"));
    
    mCTexture.AppendDefaultItems(mCConfList, _TEX_FOR_MATERIAL);

    ColorCurve_cls *pCClrCurveAmbient = new ColorCurve_cls
	(wxT("Ambient"), wxT("Ambient"), wxT("0.2 0.16 0.04 1"));
    ColorCurve_cls *pCClrCurveDiffuse = new ColorCurve_cls
	(wxT("Diffuse"), wxT("Diffuse"));
    ColorCurve_cls *pCClrCurveSpeculer = new ColorCurve_cls
	(wxT("Specular"), wxT("Specular"));
    
    mVpCClrCurve.push_back(pCClrCurveAmbient);
    mVpCClrCurve.push_back(pCClrCurveDiffuse);
    mVpCClrCurve.push_back(pCClrCurveSpeculer);
}

void FilterObjectData_cls::
GetStrPovInclude(wxString& strPov, PovObjectMesh_cls &CMesh){
    float clr[3][4];
    GetColors(clr, &mCTexture);
    ChangeCustomName(0);
    wxString strFinishOption = mCTexture.GetStrPovFinishOptions();
    wxString strNormal = mCTexture.GetStrPovNormal();
    CMesh.GetStrPovInclude
	(strPov, mCstrCustomName.mstrBody, 
	 mCMaterialType.mBody == _MATERIAL_USE_ORIGINAL, clr, 
	 strFinishOption, strNormal);
    ChangeCustomName(1);
}

wxString FilterObjectData_cls::
GetStrPovMatrix(){
    wxString strMatrix;
    float *p0 = mCArray0Vector.mpBody;
    float *p1 = mCArray1Vector.mpBody;
    float *p2 = mCArray2Vector.mpBody;
    strMatrix << wxT("\tmatrix<") 
	      << p0[X] << wxT(",") << p0[Y] << wxT(",") << p0[Z] << wxT(",")
	      << p1[X] << wxT(",") << p1[Y] << wxT(",") << p1[Z] << wxT(",")
	      << p2[X] << wxT(",") << p2[Y] << wxT(",") << p2[Z] << wxT(",")
	      << 0 << wxT(",") << 0 << wxT(",") << 0 << wxT(">");
    return strMatrix;
}

void FilterObjectData_cls::
SetAffinMatrix(){
    GLfloat matrix[16];
    matrix[ 0] = mCArray0Vector.mpBody[X];
    matrix[ 1] = mCArray0Vector.mpBody[Y];
    matrix[ 2] = mCArray0Vector.mpBody[Z];
    matrix[ 3] = 0;
    matrix[ 4] = mCArray1Vector.mpBody[X];
    matrix[ 5] = mCArray1Vector.mpBody[Y];
    matrix[ 6] = mCArray1Vector.mpBody[Z];
    matrix[ 7] = 0;
    matrix[ 8] = mCArray2Vector.mpBody[X];
    matrix[ 9] = mCArray2Vector.mpBody[Y];
    matrix[10] = mCArray2Vector.mpBody[Z];
    matrix[11] = 0;
    matrix[12] = 0;
    matrix[13] = 0;
    matrix[14] = 0;
    matrix[15] = 1;
    glMultMatrixf(matrix);
}

void FilterObject_cls::
ReadMesh(int meshIndex, int currentIndex){
    wxString strFile = wxString::Format(GetPCData()->mCstrImportPath.mstrBody, currentIndex);
    mCMeshObject[meshIndex].ReadMeshFromFile(strFile);
}

void FilterObject_cls::
DrawMesh(int meshIndex){
    mCMeshObject[meshIndex].GLDraw(GetPCData()->mCMaterialType.mBody == 1);
}

// ---------------------

FilterObject_cls::
FilterObject_cls(FilterPanelPain_cls *pCPanelPain,
		  FilterObjectData_cls *pCInitData) :
    FilterBase_cls(pCPanelPain->mpCFilterCtrlLink)
{
    wxString strInit; strInit << _FILTER_OBJECT;
    mCFilterType.SetItem(wxT("FilterType="), strInit);
    mpCData     = new FilterObjectData_cls;
    mpCGUIPanel = new FilterObjectGUI_cls(this, pCPanelPain);

    for (int i = 0; i < _MAX_MESH; i++)
	mDisplayList[i] = -1;

    mCompiledListType = -1;
    mCompiledMaterialType = -1;
    for (int i = 0; i < 16; i++) mCompiledMatrix[i] = 0;
    LOOP_XYZ mCompiledScale[xyz] = -1;
    
    mbMatrixIdentity = FALSE;

    mCGLTexture.mTexWrap = GL_REPEAT;
}

FilterObject_cls::
~FilterObject_cls(){
    ClearDisplayList();
}

void FilterObject_cls::
ClearDisplayList(){
    for (int  i = 0; i < _MAX_MESH; i++){
	if (mDisplayList[i] != -1){
	    glDeleteLists(mDisplayList[i], 1);
	    mDisplayList[i] = -1;
	}
    }
}

FilterObjectData_cls* FilterObject_cls::
GetPCData(){
    return (FilterObjectData_cls*)mpCData;
}

void FilterObject_cls::
OpenLua(){
    wxString strFunction;
    strFunction << wxT("function script(i)") << ENDL;
    strFunction << GetPCData()->mCstrExpression.mstrBody << ENDL;
    strFunction << wxT("end") << ENDL;

    OpenLuaBase(strFunction);
}

int FilterObject_cls::
CallLuaSubIndex(int index){
    if (mpLua == NULL) return 0;

    lua_getglobal(mpLua, "script"); // ֐ς
    lua_pushnumber(mpLua, index); // ̒lς

    if (lua_pcall(mpLua, 1, 1, 0) != 0){
	CheckErrorMessage(mpLua, wxT("FilterObject::"));
	CloseLuaBase();
	return 0;
    }

    int stackSize = lua_gettop(mpLua);
    int subIndex = lua_tonumber(mpLua, 1);
    ClearLuaStack();

    return subIndex;
}

int FilterObject_cls::
GetTexGridSize(){
    FilterObjectData_cls *pCData = GetPCData();
    if (pCData->mCbMultipleMesh.mbBody == FALSE) return 1;
    return pCData->mCMeshGrid.mBody + 1;
}

void FilterObject_cls::
GLDraw(double doubleIndex, double scale, bool bHittest){
    FilterObjectData_cls *pCData = GetPCData();
    if (pCData->mpCSceneObjParam->mbObjectVisible == FALSE) return;

    int wireframe = _WIREFRAME_BOX;
    if (pCData->mCObjectType.mBody == _FILTER_OBJECT_PLANE)
	wireframe = _WIREFRAME_PLANE;
    GetPCData()->GLDrawWireframe(wireframe, 2);
}

void FilterObject_cls::
GLDrawPlane(){
    glBegin(GL_TRIANGLES);
    glNormal3d(0, 0, 1); glTexCoord2d(0, 0); glVertex3d(-1, -1, 0); 
    glNormal3d(0, 0, 1); glTexCoord2d(1, 0); glVertex3d(+1, -1, 0); 
    glNormal3d(0, 0, 1); glTexCoord2d(1, 1); glVertex3d(+1, +1, 0); 

    glNormal3d(0, 0, 1); glTexCoord2d(0, 0); glVertex3d(-1, -1, 0); 
    glNormal3d(0, 0, 1); glTexCoord2d(1, 1); glVertex3d(+1, +1, 0); 
    glNormal3d(0, 0, 1); glTexCoord2d(0, 1); glVertex3d(-1, +1, 0); 
    glEnd();
}

void FilterObject_cls::
GLDrawRealObject(int currentIndex, int lightNumber, 
		 bool bCopyInstance, int drawStep, int frame){
    
    FilterObjectData_cls *pCData = GetPCData();
    if (pCData->mCbValid.mbBody == FALSE) return;

    if (pCData->mCbWorld.mbBody == TRUE && bCopyInstance == TRUE) return;
    if (pCData->mCbWorld.mbBody == FALSE && bCopyInstance == FALSE) return;

    if (drawStep == 0 && pCData->mCbDrawLater.mbBody == TRUE) return;
    if (drawStep == 1 && pCData->mCbDrawLater.mbBody == FALSE) return;

    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glPushMatrix();

    GLDrawSetMaterial();
    CheckForClearDisplayList();
    GLDrawSetTexture(frame);

    GLDrawSetShaderBegin(lightNumber);
    GLDrawBody(currentIndex);
    GLDrawSetShaderEnd();
    
    glPopMatrix();
    glPopAttrib();
}

void FilterObject_cls::
GLDrawSetMaterial(){
    FilterObjectData_cls *pCData = GetPCData();

    float clrMaterial[3][4];
    pCData->mVpCClrCurve[0]->GetColor4fNormal(clrMaterial[0], 0);
    pCData->mVpCClrCurve[1]->GetColor4fNormal(clrMaterial[1], 0);
    pCData->mVpCClrCurve[2]->GetColor4fNormal(clrMaterial[2], 0);

    pCData->mCTexture.GetAlphaInfo(clrMaterial);
    
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, clrMaterial[0]);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, clrMaterial[1]);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, clrMaterial[2]);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 20);
    
    if (pCData->mCbCullFace.mbBody == TRUE) glEnable(GL_CULL_FACE);
    else                                    glDisable(GL_CULL_FACE);
}

void FilterObject_cls::
GLDrawSetTexture(int frame){
    FilterObjectData_cls *pCData = GetPCData();

    bool bUseTex = TRUE;
    if (pCData->mCTexture.mCstrTexFormat[0].mstrBody.IsSameAs(wxT("NONE"))) 
	bUseTex = FALSE;
    //if (bUseTex == TRUE) glEnable(GL_TEXTURE_2D);

    int type = pCData->mCObjectType.mBody;
    switch(type){
    case _FILTER_OBJECT_IMPORT:
	glEnable(GL_TEXTURE_2D);
	break;
    default:
	if (bUseTex == TRUE){
	    wxString strFile = pCData->mCTexture.GetStrTexFile(frame);
	    if (strFile.IsEmpty() == FALSE)
		mCGLTexture.LoadImage(strFile, TRUE);
	    glEnable(GL_TEXTURE_2D);
	    mCGLTexture.BindTexture();
	    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	}
	else
	    glDisable(GL_TEXTURE_2D);
	break;
    }
}

void FilterObject_cls::
CheckMatrixIdentity(){
    mbMatrixIdentity = FALSE;
    double m[16];
    GetPCData()->mCCurrentObj.GetGLMatrix(m);
    if (m[0] == 1 && m[4] == 0 && m[8 ] == 0 && m[12] == 0 &&
	m[1] == 0 && m[5] == 1 && m[9 ] == 0 && m[13] == 0 &&
	m[2] == 0 && m[6] == 0 && m[10] == 1 && m[14] == 0 &&
	m[3] == 0 && m[7] == 0 && m[11] == 0 && m[15] == 1){
	if (GetPCData()->mCCurrentObj.mScale[X] == 1&&
	    GetPCData()->mCCurrentObj.mScale[Y] == 1&&
	    GetPCData()->mCCurrentObj.mScale[Z] == 1)
	    mbMatrixIdentity = TRUE;
    }
}

void FilterObject_cls::
MemorizeCompiledState(){
    FilterObjectData_cls *pCData = GetPCData();

    mCompiledListType = pCData->mCObjectType.mBody;
    mCompiledMaterialType = pCData->mCMaterialType.mBody;
    mCompiledMappingType = pCData->mCTexture.mCMappingType.mBody;
    mstrCompiledListFile = pCData->mCstrImportPath.mstrBody;

    double m[16];
    pCData->mCCurrentObj.GetGLMatrix(m);
    for (int i = 0; i < 16; i++) mCompiledMatrix[i] = m[i];
    LOOP_XYZ mCompiledScale[xyz] = pCData->mCCurrentObj.mScale[xyz];
}

bool FilterObject_cls::
CheckForClearDisplayList(){
    FilterObjectData_cls *pCData = GetPCData();
	
    int type = pCData->mCObjectType.mBody;
    int materialType = pCData->mCMaterialType.mBody;
    int mappingType = pCData->mCTexture.mCMappingType.mBody;
    if (mCompiledListType != type ||
	mCompiledMaterialType != materialType ||
	mCompiledMappingType != mappingType ||
	mstrCompiledListFile.IsSameAs(pCData->mCstrImportPath.mstrBody) == FALSE){
	ClearDisplayList();
	return TRUE;
    }

    double m[16];
    pCData->mCCurrentObj.GetGLMatrix(m);
    bool bDifferent = FALSE;
    for (int i = 0; i < 16; i++)
	if (mCompiledMatrix[i] != m[i]) bDifferent = TRUE;
    LOOP_XYZ if (mCompiledScale[xyz] != pCData->mCCurrentObj.mScale[xyz])
	bDifferent = TRUE;
    if (bDifferent == TRUE){
	ClearDisplayList();
	return TRUE;
    }
    return FALSE;
}

void FilterObject_cls::
UpdateDisplayList(int currentIndex){
    bool bClearList = FALSE;
    
    CheckMatrixIdentity();
    if (CheckForClearDisplayList() == TRUE) bClearList = TRUE;
    GLDrawSetTexture(0); // Aj[VeNX`͑zOƂ

    if (bClearList == TRUE){
	for (int i = 0; i < _MAX_MESH; i++)
	    GLMakeDisplayList(i, currentIndex, GL_COMPILE);
    }
}

void FilterObject_cls::
GLDrawSetShaderBegin(int lightNumber){
    GLAttribute_cls::mspCShaderCtrl->
	BeginShader(&GetPCData()->mCTexture, lightNumber);
}

void FilterObject_cls::
GLDrawSetShaderEnd(){
    GLAttribute_cls::mspCShaderCtrl->
	EndShader(GetPCData()->mCTexture.mCShaderType.mBody);
}

void FilterObject_cls::
GLDrawBody(int currentIndex, int compile){

    glEnable(GL_BLEND);
    glEnable(GL_NORMALIZE);

    FilterObjectData_cls *pCData = GetPCData();
    bool bMultiMesh = pCData->mCbMultipleMesh.mbBody;

    int meshIndex = 0;
    if (bMultiMesh == TRUE){
	meshIndex = CallLuaSubIndex(0);
	if (meshIndex < 0) meshIndex = 0;
	if (meshIndex >= _MAX_MESH) meshIndex = _MAX_MESH - 1;
	currentIndex = meshIndex;
    }

    if (mDisplayList[meshIndex] != -1) {
	if (compile == GL_COMPILE) return;
	glCallList(mDisplayList[meshIndex]); 
	return;
    }

    if (bMultiMesh == TRUE){
	for (int i = 0; i < _MAX_MESH; i++){
	    if (i != CallLuaSubIndex(0))	
		GLMakeDisplayList(i, i, GL_COMPILE);
	    else
		GLMakeDisplayList(i, i, compile);
	}
    }
    else
	GLMakeDisplayList(meshIndex, currentIndex, compile);

    glDisable(GL_TEXTURE_2D);
}

void FilterObject_cls::
GLMakeDisplayList(int meshIndex, int currentIndex, int compile){
    FilterObjectData_cls *pCData = GetPCData();
    bool bMultiMesh = pCData->mCbMultipleMesh.mbBody;
    
    int type = pCData->mCObjectType.mBody;
    
    if (type == _FILTER_OBJECT_IMPORT){
	ReadMesh(meshIndex, meshIndex); // ɓǂł
	mCMeshObject[meshIndex].mCalledMaterialType = -1;
	mCMeshObject[meshIndex].mbUseMaterialColor=(pCData->mCMaterialType.mBody == 1);
	mCMeshObject[meshIndex].CallMaterial(meshIndex); // ɐݒ肵Ă(eNX`쐬܂ŃXgɓȂ悤)
    }
    
    bool bUseTex = TRUE;
    if (pCData->mCTexture.mCstrTexFormat[0].mstrBody.IsSameAs(wxT("NONE"))) 
	bUseTex = FALSE;
    int  mapType = pCData->mCTexture.mCMappingType.mBody;
    
    mDisplayList[meshIndex] = glGenLists(1);
    glNewList(mDisplayList[meshIndex], compile);
    
    MemorizeCompiledState();

    if (mbMatrixIdentity == FALSE) GetPCData()->GLSetMatrix();
    
    switch(pCData->mCObjectType.mBody){
    case _FILTER_OBJECT_BOX:
	//glScalef(0.5, 0.5, 0.5);
	pCData->SetAffinMatrix();
	GpCCube->Draw(mapType);
	break;
    case _FILTER_OBJECT_PLANE:
	//glScalef(0.5, 0.5, 0.5);
	pCData->SetAffinMatrix();
	GLDrawPlane();
	break;
    case _FILTER_OBJECT_SPHERE:
	//glScalef(0.5, 0.5, 0.5);
	pCData->SetAffinMatrix();
	GpCSphere->Draw(4, mapType);
	break;
    case _FILTER_OBJECT_IMPORT:
	pCData->SetAffinMatrix();
	DrawMesh(meshIndex);
	break;
    }
    
    if (mbMatrixIdentity == FALSE) GetPCData()->GLSetMatrixInverse();    
    glEndList();
}

void FilterObject_cls::
OnSelected(){
    FilterObjectData_cls *pCData = GetPCData();
    mpCParentCtrl->mspCParentMainLink->ConnectTextureToDialog(&pCData->mCTexture, pCData->mCstrCustomName.mstrBody);
}

void FilterObject_cls::
OpenExpressionDialog(){
    wxString strMessage;
    strMessage << wxT("-----") << _("Predefined Variables") << ENDL;
    strMessage << wxT("i = particle index") << ENDL;
    strMessage << wxT("-----") << _("Return Value") << ENDL;
    strMessage << wxT("meshIndex") << ENDL;
    strMessage << wxT("index for Multiple Objects");

    TextEntryDialogEX_cls CDlg(mpCGUIPanel->mpCMNumPanel, strMessage, _("Expression Editor"), GetPCData()->mCstrExpression.mstrBody);

    if (CDlg.ShowModal() == wxID_OK){
	wxString strEdit = CDlg.GetValue().Trim(TRUE);
	FilterObjectData_cls *pCData = GetPCData();
	int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_ATTRIB | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;
	wxString strTag = mpCParentCtrl->GetHeaderStrFromPointer(&pCData->mCstrExpression.mstrBody);
	GCUndoList.PushNewCommand
	    (new CommandFilterValue_cls(mpCParentCtrl, strTag, strEdit, updateBit), 
	     new CommandFilterValue_cls(mpCParentCtrl, strTag, pCData->mCstrExpression.mstrBody, updateBit));
	GCUndoList.ExecCurrent();
    }
}

void FilterObject_cls::
GetStrPovInclude(wxString &strPov, bool bForceValid){
    FilterObjectData_cls *pCData = GetPCData();
    if (pCData->mCbValid.mbBody == FALSE && bForceValid == FALSE) return;
    if (pCData->mCObjectType.mBody != _FILTER_OBJECT_IMPORT) return;
    
    pCData->GetStrPovInclude(strPov, mCMeshObject[0]);
}

void FilterObject_cls::
GetStrPovRealObjectTexDeclare(wxString &strPov, wxString strTex,
			      float clr[3][4], int mapType,
			      wxString strFinishOption, wxString strNormal){

    FilterObjectData_cls *pCData = GetPCData();
    
    wxString strPigment;
    strPigment << strTex << wxT("_PIGMENT");

    strPov << wxT("#declare ") << strPigment << wxT("=pigment{");
    GetStrPovPigmentImageMap(strPov, pCData->mCTexture.mCstrTexFormat[0].mstrBody, FALSE, mapType, 1, 1);
    strPov << ENDLCLOSE;

    if (PovRayCtrl_cls::mspCThis->IsFastColor() == TRUE){
	strPov << wxT("#declare ") << strTex << wxT("_TEXTURE=") << ENDL;
	GetStrPovTextureColor
	    (strPov, wxES, TRUE, 
	     clr, 0.156*0.156, strFinishOption, strNormal, strPigment, FALSE, FALSE);
	return;
    }

    strPov << wxT("#declare ") << strTex << wxT("_DIFFUSE=") << ENDL;
    PovSerialize_cls::GetStrPovTextureDiffuse(strPov, strPigment, clr, strFinishOption, strNormal);
    strPov << wxT(";") << ENDL;
    strPov << wxT("#declare ") << strTex << wxT("_AMBIENT_SPECULAR=") << ENDL;
    GetStrPovTextureAmbientSpecular(strPov, strPigment, clr, strFinishOption, strNormal);
    strPov << wxT(";") << ENDL;
    strPov << wxT("#declare ") << strTex << wxT("_TEXTURE=") << ENDL;
	strPov << wxT("texture {") << ENDL
	       << GetTab(1) << wxT("average") << ENDL
	       << GetTab(1) << wxT("texture_map {") << ENDL
	       << GetTab(2) << wxT("[ ") << strTex << wxT("_DIFFUSE ]") << ENDL
	       << GetTab(2) << wxT("[ ") << strTex << wxT("_AMBIENT_SPECULAR ]") << ENDL;
	strPov << GetTab(1) << ENDLCLOSE;
	strPov << wxT("};") << ENDL;

}

void FilterObject_cls::
GetStrPovRealObject(wxString& strPov,
		    double doubleIndex, wxString strTranslate, 
		    bool bCopyInstance, wxString strTab,
		    bool bForceValid){

    FilterObjectData_cls *pCData = GetPCData();
    if (pCData->mCbValid.mbBody == FALSE && bForceValid == FALSE) return;

    if (pCData->mCbWorld.mbBody == TRUE && bCopyInstance == TRUE) return;
    if (pCData->mCbWorld.mbBody == FALSE && bCopyInstance == FALSE) return;

    wxString strFinishOption = pCData->mCTexture.GetStrPovFinishOptions();
    wxString strNormal = pCData->mCTexture.GetStrPovNormal();

    float clr[3][4];
    pCData->GetColors(clr, &pCData->mCTexture);

    pCData->ChangeCustomName(0);
    wxString strTex;
    strTex << pCData->mCstrCustomName.mstrBody;
    pCData->ChangeCustomName(1);

    bool bUseTexture = FALSE;
    if (pCData->mCTexture.mCstrTexFormat[0].mstrBody.IsSameAs(wxT("NONE")) == FALSE) bUseTexture = TRUE;

    wxString strTexTransform;

    if (bUseTexture == TRUE &&
	pCData->mCObjectType.mBody != _FILTER_OBJECT_IMPORT){
	int mapType = 0;
	
	switch(pCData->mCTexture.mCMappingType.mBody){
	case _MAPPING_LOCAL_X:
	    strTexTransform << wxT("translate<-0.5, -0.5, -0.5> rotate <90, 0, 0> rotate <0, 0, 90>");
	    break;
	    case _MAPPING_LOCAL_Y:
		strTexTransform << wxT("translate<-0.5, -0.5, -0.5> rotate <90, 0, 0>");
		break;
	case _MAPPING_LOCAL_Z:
	    strTexTransform << wxT("translate<-0.5, -0.5, -0.5>");
	    break;
	case _MAPPING_UV:
	    if (pCData->mCObjectType.mBody == _FILTER_OBJECT_SPHERE){
		mapType = 1;
		strTexTransform << wxT("rotate <90, 0, 0>");
	    }
	    else
		strTexTransform << wxT("translate<-0.5, -0.5, -0.5>");
	    break;
	}
	GetStrPovRealObjectTexDeclare(strPov, strTex, clr, mapType,
				      strFinishOption, strNormal);
    }
    
    switch(pCData->mCObjectType.mBody){
    case _FILTER_OBJECT_BOX:
	strPov << strTab << wxT("box { <-1, -1, -1>, <1, 1, 1>") << ENDL;
	break;
    case _FILTER_OBJECT_PLANE:
	strPov << strTab << wxT("polygon { 4, <-1, -1, 0>, <1, -1, 0>, <1, 1, 0>, <-1, 1, 0>") << ENDL;
	break;
    case _FILTER_OBJECT_SPHERE:
	strPov << strTab << wxT("sphere { <0, 0, 0>, 1") << ENDL;
	break;
    case _FILTER_OBJECT_IMPORT:
	strPov << wxT("object {") << ENDL
	       << wxT("\t") << mCMeshObject[0].mstrObjectLabel << ENDL;
    }

    if (pCData->mCObjectType.mBody != _FILTER_OBJECT_IMPORT &&
	pCData->mCMaterialType.mBody == _MATERIAL_USE_MATERIAL_COLOR){

	if (bUseTexture == FALSE)
	    PovSerialize_cls::GetStrPovTextureColor
		(strPov, strTab + GetTab(1), 
		 PovRayCtrl_cls::mspCThis->mpCFormat->mCbFastColoring.mbBody,
		 clr, 0.156*0.156, strFinishOption, strNormal);
	else{
	    strPov << strTab + GetTab(1) << wxT("texture{ ") << strTex << wxT("_TEXTURE ") << strTexTransform << ENDLCLOSE;
	}
    }
    
    wxString strInteriorOptions = 
	pCData->mCTexture.GetStrPovInteriorOptions();
    if (strInteriorOptions.IsEmpty() == FALSE)
	strPov << strTab << wxT("\tinterior {") << strInteriorOptions << wxT("}") << ENDL;
    
    double alpha = pCData->mCTexture.GetAlpha();
    double reflectMin  = pCData->mCTexture.mCReflectMin.mBody; // ˗
    double reflectMax  = pCData->mCTexture.mCReflectMax.mBody;
    bool   bTarget = pCData->mCTexture.mCbPhotonTarget.mbBody;
    bool   bCollect = pCData->mCTexture.mCbPhotonCollect.mbBody;
    PovRayCtrl_cls::GetStrPovPhotonSetting(strPov, alpha, reflectMin, reflectMax, bTarget, bCollect);

    strPov << strTab << pCData->GetStrPovMatrix() << ENDL;    
    strPov << strTab;
    pCData->GetStrPovModelView(strPov, doubleIndex);
    if (strTranslate.IsEmpty() != TRUE)
	strPov << strTab << wxT("\t") << strTranslate << ENDL;
    
    strPov << strTab << wxT("}") << ENDL;
}

