
#ifndef _FILTER_OBJECT_H_
#define _FILTER_OBJECT_H_

#include "../FILTER/clsFilterBase.h"
#include "../LUA/clsLuaBase.h"
#include "../POVRAY/MESH/clsPovObjectMesh.h"
#include "../SCENEOBJ/clsSceneObj.h"
#include "../SCENEOBJ/clsSceneObjTexture.h"

#include "../COMMON/CommonDefinitionsForH.h"

class ArrayAndIndex_cls;
class BitmapPanel_cls;
class FilterObject_cls;
class FilterPanelPain_cls;

#define _FILTER_OBJECT_BOX    0
#define _FILTER_OBJECT_PLANE  1
#define _FILTER_OBJECT_SPHERE 2
#define _FILTER_OBJECT_IMPORT 3

#define _MATERIAL_USE_MATERIAL_COLOR 0
#define _MATERIAL_USE_ORIGINAL 1

class FilterObjectGUI_cls : public FilterGUI_cls {
 public:
    FilterObjectGUI_cls(FilterObject_cls *pCParent, 
			FilterPanelPain_cls *pCPanelPain);
    FilterObject_cls *mpCParent;
    int mIndexType;
    int mIndexPath, mIndexMultiple, mIndexExpression;
    int mIndexMaterialMode, mIndexMaterial;
    int mIndexArrayVector;
    int mIndexTranslate, mIndexRotate, mIndexScale;

    BitmapPanel_cls *mpCFolderXYZ;

    void SetMNumPanel(wxBoxSizer *pBoxSizer, wxWindow *pCParentWindow,
		      int index);
    void HookFuncHS(int id, void *pArg);
    void HookFunc01(int id, void *pArg); // Button
    void HookFunc02(int id, void *pArg); // ArrayVector
    void HookFunc03(int id, void *pArg); // Expression
    void HideAndShow();
    void PathSelection();
    void ArrayVector();
};

class FilterObjectData_cls : public FilterData_cls{
 public:
    FilterObjectData_cls();

    SceneObjTexture_cls mCTexture;

    ConfItemUINT_cls mCObjectType;
    ConfItemUINT_cls mCMaterialType;
    ConfItemSTR_cls  mCstrImportPath;
    ConfItemBOOL_cls mCbDrawLater;
    ConfItemBOOL_cls mCbCullFace;
    ConfItemBOOL_cls mCbWorld;
    ConfItemBOOL_cls mCbMultipleMesh;
    ConfItemUINT_cls mCMeshGrid;
    ConfItemSTR_cls mCstrExpression;
    
    ConfItemFLOATARRAY_cls mCArray0Vector;
    ConfItemFLOATARRAY_cls mCArray1Vector;
    ConfItemFLOATARRAY_cls mCArray2Vector;

    void GetStrPovInclude(wxString &strPov, PovObjectMesh_cls& CMesh);
    wxString GetStrPovMatrix();

    void SetAffinMatrix();
};

class FilterObject_cls : public FilterBase_cls, public LuaBase_cls {
 public:
    FilterObject_cls (FilterPanelPain_cls *pCPanelPain,
		       FilterObjectData_cls *pCInitData = NULL);
    ~FilterObject_cls();

    FilterObjectData_cls *GetPCData();
    GLTexture_cls       mCGLTexture;
    PovObjectMesh_cls   mCMeshObject[_MAX_MESH];

    void OpenLua();
    int  CallLuaSubIndex(int index);
    int  GetTexGridSize();

    void ClearDisplayList();
    int mDisplayList[_MAX_MESH];
    int mCompiledListType, mCompiledMaterialType, mCompiledMappingType;
    wxString mstrCompiledListFile;
    double mCompiledMatrix[16];
    double mCompiledScale[3];

    bool mbMatrixIdentity;

    void GLDraw(double doubleIndex, double scale, bool bHittest);
    void GLDrawPlane();
    void GLDrawRealObject(int currentIndex, int lightNumber,
			  bool bCopyInstance, int drawStep, int frame);
    void GLDrawSetMaterial();
    void GLDrawSetTexture(int frame);
    void CheckMatrixIdentity();
    void MemorizeCompiledState();
    bool CheckForClearDisplayList();
    void UpdateDisplayList(int currentIndex);
    void GLDrawSetShaderBegin(int lightNumber);
    void GLDrawSetShaderEnd();
    void GLDrawBody(int currentIndex, int compile = GL_COMPILE_AND_EXECUTE);
    void GLMakeDisplayList(int meshIndex, int currentIndex, int compile);

    void ReadMesh(int meshIndex, int currentIndex);
    void DrawMesh(int meshIndex);
    
    void OnSelected();
    void OpenExpressionDialog();

    void GetStrPovInclude(wxString &strPov, bool bForceValid = FALSE);
    void GetStrPovRealObjectTexDeclare
	(wxString &strPov, wxString strTex, 
	 float clr[3][4], int mapType,
	 wxString strFinishOption, wxString strNormal);
    void GetStrPovRealObject(wxString& strPov,
			     double doubleIndex, wxString strTranslate,
			     bool bCopyInstance, wxString strTab,
			     bool bForceValid = FALSE);
};

#endif
