// -*- c++ -*- 
// (setq c-basic-offset 4)

#include "clsFilterText.h"
#include "clsFilterCtrl.h"

#include "../GLCANVAS/clsMainGLCanvas.h"
#include "../MAINCANVAS/clsFilterPanelPain.h"
#include "../PARTICLEDATA/clsGLDrawData.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;

#include "../UNDO/clsUndoItemConstParam.h"
#include "../UNDO/clsUndoItemFilterParam.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/WX/clsFontProvider.h"
#include "../WIDGETCTRL/EX/clsTextEntryDialogEX.h"

#include "../../wxWidgetEnum.h"

#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

#ifdef WIN32
extern PFNGLBLENDFUNCSEPARATEPROC glBlendFuncSeparate;
#endif

// -----------------------

FilterTextGUI_cls::							
FilterTextGUI_cls(FilterText_cls *pCParent, 
		     FilterPanelPain_cls *pCPanelPain) : FilterGUI_cls(pCParent) {
    mpCParent = pCParent;
    SetMNumPanel((wxBoxSizer*)pCPanelPain->GetSizer(), pCPanelPain, 0);
}

void FilterTextGUI_cls::
SetMNumPanel(wxBoxSizer *pBoxSizer, wxWindow *pCParentWindow, int index){

    MultiInputPanel_cls* pCurr = InitializeMInputCommon
	(pCParentWindow, mpCParent->GetPCData()->mCstrCustomName.mstrBody);
    FilterTextData_cls *pCData = mpCParent->GetPCData();
    
    pCurr->mUpdateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;

    int boxWidth = 2;
    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;
    
    /*
    wxString listType[2] = {_("Basic"), _("Pointer")};
    pCurr->CombinePointerList
	(_ITEM_UINT_LIST, &pCData->mCType.mBody, 2, listType, _("Text Type"));
    */

    boxWidth = 2;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth);
    pCurr->CombinePointer(_ITEM_BOOL, &pCData->mCbFaceCamera.mbBody, _("Face Camera"));
    pCurr->CombinePointer(_ITEM_BOOL, &pCData->mCbShadow.mbBody, _("Cast Shadow"));

#ifdef WIN32
    int buttonWidth = 60;
#else
    int buttonWidth = 80;
#endif

    pCurr->mUndoType = _MN_UNDO_MANUAL;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth);
    pCurr->mItemWidth = pCurr->mMinimumWidth - buttonWidth + 10;
    pCurr->CombinePointer(_ITEM_STRING, &pCData->mCText.mstrBody);
    pCurr->GetLastItem().mStyle = wxTE_MULTILINE | wxTE_PROCESS_TAB;
    pCurr->mItemWidth = buttonWidth;
    mIndexEdit = pCurr->CombinePointerButton(wxID_ANY, _("Editor"));
    
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth);
    pCurr->mItemWidth = pCurr->mMinimumWidth - buttonWidth + 10;
    pCurr->CombinePointer(_ITEM_STRING, &pCData->mCFontDesc.mstrBody);
    pCurr->mItemWidth = buttonWidth;
    mIndexFont = pCurr->CombinePointerButton(wxID_ANY, _("Font"));

    pCurr->mItemWidth = -1;
    
    std::vector < int > VIndex = SetMNumBasic(pCurr, pCData);
    
    pCurr->mTextWidth = 90;
    pCurr->mItemWidth = -1;
    pCurr->mUndoType = _MN_UNDO_AUTO_TAG;

    boxWidth = 1;
    pCurr->mItemWidth = -1;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Color"));
    ColorCurveBtn_cls *pCBtnColor = new ColorCurveBtn_cls
	(pCurr, pCData->mVpCClrCurve[0], TRUE, FALSE);
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnColor, wxT("Color"));
    
    pCurr->FinishSetting(NULL);

    SetHookBasic(pCurr);

    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_BUTTON);
    pCurr->GetPCwxWindow(mIndexEdit)->SetMaxSize(wxSize(40, -1));
    pCurr->GetPCwxWindow(mIndexFont)->SetMaxSize(wxSize(40, -1));

    pCurr->SetColorForXYZ(VIndex[0]+1);
    pCurr->SetColorForXYZ(VIndex[1]+1);
    pCurr->SetColorForXYZ(VIndex[2]+1);
}

void FilterTextGUI_cls::
HookFunc01(int id, void *pArg){
    if (id == mpCMNumPanel->GetItemId(mIndexEdit)) mpCParent->OpenEditor();
    if (id == mpCMNumPanel->GetItemId(mIndexFont)) mpCParent->SelectFont();
}

// ---------------------

FilterTextData_cls::
FilterTextData_cls() {
    mCstrCustomName.mstrBody = _("Text");
    mCstrCustomName.mstrBody << wxT("%02d");

    mCConfList.Add(&mCType, wxT("TextType="), wxT("0"));
    mCConfList.Add(&mCText, wxT("TextText="), wxT("Text"));
#ifdef WIN32
    mCConfList.Add(&mCFontDesc, wxT("TextFontDesc="), wxT("CI 24 windows-932"));
#endif
    mCConfList.Add(&mCbFaceCamera, wxT("TextFaceCamera="), wxT("1"));
    mCConfList.Add(&mCbShadow, wxT("TextCastShadow="), wxT("0"));

    mCTexture.AppendDefaultItems(mCConfList, _TEX_FOR_MATERIAL);

    ColorCurve_cls *pCClrCurve = new ColorCurve_cls
	(wxT("Color"), wxT("Color"), wxT("0.5 0.5 0.5 1"));

    mVpCClrCurve.push_back(pCClrCurve);
    //mVpCClrCurve.push_back(pCClrCurveDiffuse);
    //mVpCClrCurve.push_back(pCClrCurveSpeculer);
    
}

// ---------------------

FilterText_cls::
FilterText_cls(FilterPanelPain_cls *pCPanelPain,
	       FilterTextData_cls *pCInitData) :
    FilterBase_cls(pCPanelPain->mpCFilterCtrlLink)
{
    wxString strInit; strInit << _FILTER_TEXT;
    mCFilterType.SetItem(wxT("FilterType="), strInit);
    mpCData     = new FilterTextData_cls;
    mpCGUIPanel = new FilterTextGUI_cls(this, pCPanelPain);
}

FilterText_cls::
~FilterText_cls(){
}

FilterTextData_cls* FilterText_cls::
GetPCData(){
    return (FilterTextData_cls*)mpCData;
}

void FilterText_cls::
OpenEditor(){
    TextEntryDialogEX_cls CDlg(mpCGUIPanel->mpCMNumPanel, wxEmptyString, _("Text Editor"), GetPCData()->mCText.mstrBody);
    
    if (CDlg.ShowModal() == wxID_OK){
	wxString strEdit = CDlg.GetValue().Trim(TRUE);
	FilterTextData_cls *pCData = GetPCData();
	int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_ATTRIB | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;
	wxString strTag = mpCParentCtrl->GetHeaderStrFromPointer(&pCData->mCText.mstrBody);
	GCUndoList.PushNewCommand
	    (new CommandFilterValue_cls(mpCParentCtrl, strTag, strEdit, updateBit), 
	     new CommandFilterValue_cls(mpCParentCtrl, strTag, pCData->mCText.mstrBody, updateBit));
	GCUndoList.ExecCurrent();
    }
}

#include <wx/fontdlg.h>

void FilterText_cls::
SelectFont(){
    wxFont CFont;
    CFont.SetNativeFontInfoUserDesc(GetPCData()->mCFontDesc.mstrBody);

    CFont = wxGetFontFromUser(mpCGUIPanel->mpCMNumPanel, CFont, wxEmptyString);

    if (CFont.IsOk() == TRUE){
	FilterTextData_cls *pCData = GetPCData();
	int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_ATTRIB | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;
	
	wxString strTag = mpCParentCtrl->GetHeaderStrFromPointer(&pCData->mCFontDesc.mstrBody);
	GCUndoList.PushNewCommand
	    (new CommandFilterValue_cls(mpCParentCtrl, strTag, CFont.GetNativeFontInfoUserDesc(), updateBit), 
	     new CommandFilterValue_cls(mpCParentCtrl, strTag, pCData->mCFontDesc.mstrBody, updateBit));
	
	GCUndoList.ExecCurrent();

	wxString strInfo = CFont.GetNativeFontInfoUserDesc();
    }
}

void FilterText_cls::
MakeTextTexture(){
    FilterTextData_cls *pCData = GetPCData();

    long style = wxFONTWEIGHT_NORMAL;
    long weight = wxFONTWEIGHT_NORMAL;
    int fontSize = 36;
    wxFont CFont;
    CFont.SetNativeFontInfoUserDesc(pCData->mCFontDesc.mstrBody);

    wxCoord singleW, singleH, singleD;
    wxCoord w,h;
    wxScreenDC DC;
    DC.SetFont(CFont);
    DC.GetTextExtent(pCData->mCText.mstrBody, &singleW, &singleH, &singleD);
    DC.GetMultiLineTextExtent(pCData->mCText.mstrBody, &w, &h, &singleH);
    
    wxRect CRectCanvas(0, 0, w, h);
    wxRect CRectBounding;
    wxMemoryDC *pMemDC = mCGLTexture.BeginDC(w, h);
    pMemDC->SetBackground(wxColour(0, 0, 0));
    pMemDC->Clear();
    pMemDC->SetTextForeground(*wxWHITE);
    pMemDC->SetFont(CFont);
    pMemDC->DrawLabel(pCData->mCText.mstrBody, wxNullBitmap, CRectCanvas, wxALIGN_LEFT | wxALIGN_TOP, -1, &CRectBounding);
    mCGLTexture.EndDC(TRUE);

    mstrTextMade = pCData->mCText.mstrBody;
    mstrFontMade = pCData->mCFontDesc.mstrBody;
    mTexWidth = w; 
    mTexHeight = h;
    mTexDescend = (float)singleD/singleH;
}

void FilterText_cls::
GLDraw(double doubleIndex, double scale, bool bHittest){
    FilterTextData_cls *pCData = GetPCData();
    if (pCData->mpCSceneObjParam->mbObjectVisible == FALSE) return;

    /*
    
    glPushMatrix();
    ParamGroup_cls *pCTranslate = pCData->mpCTranslate;
    ParamGroup_cls *pCRotate    = pCData->mpCRotation;
    ParamGroup_cls *pCScale     = pCData->mpCScale;

    if (GetPCData()->mbSelected == FALSE)
	glColor4f(0.3f, 0.8f, 0.3f, 1.0f);
    else
	glColor4f(0.8f, 0.7f, 0.3f, 1.0f);

    std::vector < double > VTrans = 
	pCTranslate->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON);
    glTranslatef(VTrans[0], VTrans[1], VTrans[2]);

    std::vector < double > VRot = 
	pCRotate->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON);
    
    glRotatef(VRot[2], 0, 0, 1);
    glRotatef(VRot[0], 1, 0, 0);
    glRotatef(VRot[1], 0, 1, 0);

    std::vector < double > VScale = 
	pCScale->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON);
    glScalef(VScale[0], VScale[1], VScale[2]);
    
    glBegin(GL_LINES);
    glVertex3d(-0.5, -0.5, -0.5);
    glVertex3d(-0.5, -0.5, +0.5);
    glVertex3d(+0.5, -0.5, -0.5);
    glVertex3d(+0.5, -0.5, +0.5);
    glVertex3d(+0.5, +0.5, -0.5);
    glVertex3d(+0.5, +0.5, +0.5);
    glVertex3d(-0.5, +0.5, -0.5);
    glVertex3d(-0.5, +0.5, +0.5);
    glEnd();

    glBegin(GL_LINE_LOOP);
    glVertex3d(-0.5, -0.5, -0.5);
    glVertex3d(+0.5, -0.5, -0.5);
    glVertex3d(+0.5, +0.5, -0.5);
    glVertex3d(-0.5, +0.5, -0.5);
    glEnd();

    glBegin(GL_LINE_LOOP);
    glVertex3d(-0.5, -0.5, +0.5);
    glVertex3d(+0.5, -0.5, +0.5);
    glVertex3d(+0.5, +0.5, +0.5);
    glVertex3d(-0.5, +0.5, +0.5);
    glEnd();

    glPopMatrix();
    */
}

void FilterText_cls::
GLDrawRealText(bool bCopyInstance, 
	       PositionAndDirection_cls CCamPos){
    
    if (bCopyInstance == TRUE) return;

    FilterTextData_cls *pCData = GetPCData();
    if (pCData->mCbValid.mbBody == FALSE) return;
    if (pCData->mCbVisible.mbBody == FALSE) return;

    if (pCData->mCText.mstrBody.IsEmpty() == TRUE) return;
    if (pCData->mCText.mstrBody.IsSameAs(mstrTextMade) == FALSE ||
	pCData->mCFontDesc.mstrBody.IsSameAs(mstrFontMade) == FALSE) 
	MakeTextTexture();
    
    if (pCData->mCbFaceCamera.mbBody == TRUE){
	LOOP_XYZ pCData->mCCurrentObj.mTgt[xyz] = CCamPos.mRgt[xyz];
	LOOP_XYZ pCData->mCCurrentObj.mTop[xyz] = -CCamPos.mTgt[xyz];
	LOOP_XYZ pCData->mCCurrentObj.mRgt[xyz] = CCamPos.mTop[xyz];
    }

    glPushAttrib(GL_ALL_ATTRIB_BITS);
    //glDisable(GL_CULL_FACE);
    glDepthMask(GL_FALSE);

    glEnable(GL_LIGHTING);
    glPushMatrix();

    GetPCData()->GLSetMatrix();

    glEnable(GL_NORMALIZE);

    glColor4f(1, 1, 1, 1);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,
			GL_ONE_MINUS_DST_ALPHA, GL_ONE);

    float color[4], zero[4] = {0, 0, 0, 1};
    pCData->mVpCClrCurve[0]->GetColor4fNormal(color,  0);
    color[3] = 1;

    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, color);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, zero);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, zero);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 20);
    
    //glColor4fv(color);
    //glDisable(GL_LIGHTING);
    
    //mpCMaterial->SetMaterial(0, TRUE);
    mCGLTexture.BindTexture();

    int lineNum = pCData->mCText.mstrBody.Freq(wxT('\n')) + 1;
    float scale = 0.5 * lineNum;

    glScalef((float)mTexWidth/mTexHeight * scale, scale, scale);

    glBegin(GL_QUADS);
    glTexCoord2f(0, 0);glVertex3f(-1, +1, 0);
    glTexCoord2f(0, 1);glVertex3f(-1, -1, 0);
    glTexCoord2f(1, 1);glVertex3f(+1, -1, 0);
    glTexCoord2f(1, 0);glVertex3f(+1, +1, 0);
    glEnd();
        
    glPopMatrix();
    glPopAttrib();
}

void FilterText_cls::
GetStrPovRealText(wxString &strPov, double doubleIndex, bool bCopyInstance, wxString strTab, PositionAndDirection_cls CCamPos){
    if (bCopyInstance == TRUE) return;
    
    FilterTextData_cls *pCData = GetPCData();

    std::vector < double > VScale = 
	pCData->mpCScale->GetInterpolatedValue(doubleIndex, 0);

    if (VScale[0] == 0 || VScale[1] == 0 || VScale[2] == 0) return;
    
    int type = pCData->mCType.mBody;
    wxString strText = pCData->mCText.mstrBody;
    int lineNum = strText.Freq(wxT('\n'));

    wxFont CFont;
    CFont.SetNativeFontInfoUserDesc(pCData->mCFontDesc.mstrBody);
    wxString strFontName = CFont.GetFaceName();
    bool bBold = FALSE, bItalic = FALSE;
    if (CFont.GetStyle() == wxFONTSTYLE_ITALIC) bItalic = TRUE;
    if (CFont.GetWeight() == wxFONTWEIGHT_BOLD) bBold = TRUE;
    wxString strFontFile = 
	FontProvider_cls::GetFontFileFromList(strFontName, bBold, bItalic);
    
    strPov << wxT("union {") << ENDL;
    
    float scale = 1.0f-mTexDescend;
    int lineCount = 0;
    wxString strLine;
    while (TRUE){
	int pos  = strText.Find(wxT('\n'));
	if (pos != wxNOT_FOUND)
	    strLine = strText.Mid(0, pos).Trim(TRUE);
	else
	    strLine = strText.Trim(TRUE);
	strPov << wxT("\ttext { ttf \"") << strFontFile << wxT("\" \"") << strLine << wxT("\" 0.01, 0 scale ") << PovOutf(scale, 4) << wxT(" translate <0,") << PovOutf(-lineCount+mTexDescend/1, 4) << wxT(",0> }") << ENDL;
	strText.Remove(0, pos+1);
	lineCount++;
	if (pos == wxNOT_FOUND) break;
    }
    
    strPov << GetTab(1) << wxT("texture{") << ENDL;

    /*
    wxString strMapping;
    if (type == _OBJECT_PLANE)
	strMapping << wxT("gradient x");
    else
	strMapping << wxT("gradient z");
    */
    
    float color[4];
    pCData->mVpCClrCurve[0]->GetColor4fNormal(color,  0);
    color[3] = 1;

       
    strPov << GetTab(2) << wxT("pigment{ ") << PovOut4fRGBTfromRGBA(color, 3, TRUE) << ENDLCLOSE;
    
    //mpCMaterial->GetStrPovColorMap(strPov, GetTab(3), strMapping);
    strPov << GetTab(2) << wxT("finish { emission 1 }") << ENDL;

    //strPov << GetTab(2) << mpCMaterial->GetStrPovFinish() << ENDL;
    strPov << GetTab(1) << ENDLCLOSE;
    
    float translate[3] = {0, 0, 0};
    translate[Y] = 0.5 * lineCount - 1;
    translate[X] = -0.5 * (float)mTexWidth/mTexHeight * lineCount;
    strPov << GetTab(1) << PovOut3f(wxT("translate "), translate, 4) << ENDL;

    strPov << GetTab(1) << PovOut3d(wxT("scale "), VScale.data(),4) << ENDL;

    pCData->GetStrPovModelViewMatrix(strPov);

    if (pCData->mCbShadow.mbBody == FALSE)
	strPov << GetTab(1) << wxT("no_shadow") << ENDL;

    strPov << ENDLCLOSE;
}

