// "clsColorCurveGUI.h"

#include "clsColorCurve.h"
#include "clsColorCurveGUI.h"
#include "clsColorCurveTemplate.h"

#include "../WIDGETCTRL/EX/clsButtonEX.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/WX/clsFontProvider.h"
#include "../WIDGETCTRL/WX/myArtProv.h"
#include "../UNDO/clsUndoManager.h"

#include "../../clsFrameMain.h"
#include "../../wxWidgetEnumMenu.h"
#include "../../wxWidgetEnum.h"

#include "../COMMON/CommonDefinitions.h"

extern wxFrame  *GpCMainFrameCLS;

// ------

BEGIN_EVENT_TABLE(ColorCurveTics_cls, wxPanel)
EVT_PAINT(ColorCurveTics_cls::OnPaint)
END_EVENT_TABLE()

ColorCurveTics_cls::
ColorCurveTics_cls(ColorCurveGUI_cls *pCParent, wxWindow *pC, wxWindowID id) :
wxPanel(pC, id, wxDP, wxSize(-1, 16)){
    mpCParentGUILink = pCParent;
    SetBackgroundStyle(wxBG_STYLE_CUSTOM);
}

void ColorCurveTics_cls::
OnPaint(wxPaintEvent& event){
    ColorCurve_cls *pCCurveLink = mpCParentGUILink->mpCCtrlLink->mpCCurveLinked;
    if (pCCurveLink != NULL) {

	wxAutoBufferedPaintDC dc(this);
	SetParameter(GetClientSize(), 
		     pCCurveLink->mCScalingMin.mBody,
		     pCCurveLink->mCScalingMax.mBody, 1);
	DrawTics(dc);
    }
    event.Skip();
}

// ----

BEGIN_EVENT_TABLE(ColorCurveGUI_cls, wxDialog)
EVT_PAINT(ColorCurveGUI_cls::OnPaint)
EVT_MENU(menuClrRefreshButton, ColorCurveGUI_cls::OnMenuRefreshButton)
EVT_BUTTON(btnClrAddKey, ColorCurveGUI_cls::OnButtonAddKey)
EVT_BUTTON(btnClrRemKey, ColorCurveGUI_cls::OnButtonRemKey)
EVT_BUTTON(btnClrColor,  ColorCurveGUI_cls::OnButtonColor)
EVT_BUTTON(btnClrRed, ColorCurveGUI_cls::OnButtonRGBA)
EVT_BUTTON(btnClrGreen, ColorCurveGUI_cls::OnButtonRGBA)
EVT_BUTTON(btnClrBlue, ColorCurveGUI_cls::OnButtonRGBA)
EVT_BUTTON(btnClrAlpha, ColorCurveGUI_cls::OnButtonRGBA)
EVT_BUTTON(btnClrLft,   ColorCurveGUI_cls::OnButtonLft)
EVT_BUTTON(btnClrRgt,   ColorCurveGUI_cls::OnButtonRgt)
EVT_KEY_DOWN(ColorCurveGUI_cls::OnKeyDown)
END_EVENT_TABLE()

ColorCurveGUI_cls::
ColorCurveGUI_cls(ColorCurveCtrl_cls* pCCtrlLink,
		  wxWindow *pC) :
    wxDialogEXKey(pC, wxID_ANY, _("Color Curve Editor"), wxDPS, 
		  ARTWXFRAMEICON("wxART_COLOR"), 
		  wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER){
    mpCCtrlLink = pCCtrlLink;
    SetFont(FontProvider_cls::GetFont(0));

    mDummy = 1;
    mDummy = 0;
    mDummyValueX = 0;
    mDummyValueType = 0;
    for (int i = 0; i < 4; i++) mDummyPColor[i] = 0;

    wxBoxSizer *pBoxSizerSrc = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *pBoxSizerTop = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *pBoxSizerRGBA = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *pBoxSizerLft = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *pBoxSizerBTNsL = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *pBoxSizerRgt = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *pBoxSizerBTNsR = new wxBoxSizer(wxHORIZONTAL);

    wxSize sizeBtn(40, -1);
    mpCBtnRed   = new wxButtonEX (this, btnClrRed, wxT("R"), wxDP, sizeBtn);
    mpCBtnGreen = new wxButtonEX (this, btnClrGreen, wxT("G"), wxDP, sizeBtn);
    mpCBtnBlue  = new wxButtonEX (this, btnClrBlue, wxT("B"), wxDP, sizeBtn);
    mpCBtnAlpha = new wxButtonEX (this, btnClrAlpha, wxT("A"), wxDP, sizeBtn);

    pBoxSizerRGBA->AddSpacer(34);
    pBoxSizerRGBA->Add(mpCBtnRed, 0, wxEXPAND);
    pBoxSizerRGBA->Add(mpCBtnGreen, 0, wxEXPAND);
    pBoxSizerRGBA->Add(mpCBtnBlue, 0, wxEXPAND);
    pBoxSizerRGBA->Add(mpCBtnAlpha, 0, wxEXPAND);

    mpCBtnClrCurve = new ColorCurveBtn_cls (this, NULL, FALSE);
    mpCTics      =   new ColorCurveTics_cls(this, this, wxID_ANY); 
    mpCMainGraph =   new GraphPanel_cls (this, wxID_ANY, wxDP, wxSize(480, 180));
    mpCBtnClrCurve->SetXViewRange(-0.05, 1.05);
    mpCTics->SetXViewRange(-0.05f, 1.05f);
    mpCMainGraph->SetXViewRange(-0.05, 1.05);
    mpCMainGraph->SetYViewRange(-0.10, 1.10);
    
    pBoxSizerLft->Add(mpCBtnClrCurve, 0, wxEXPAND);
    pBoxSizerLft->Add(mpCTics, 0, wxEXPAND);
    pBoxSizerLft->Add(mpCMainGraph, 1, wxEXPAND);
    
    mpCBtnAddKey = new wxButtonEX (this, btnClrAddKey, _("Insert Key"));
    mpCBtnRemKey = new wxButtonEX (this, btnClrRemKey, _("Delete Key"));
    mpCBtnColor = new wxButtonEX (this, btnClrColor, wxT(""), wxDP, wxSize(80, -1));
    mpCBtnLft = new wxButtonEX (this, btnClrLft, wxT("<"), wxDP, wxSize(40, -1));
    mpCBtnRgt = new wxButtonEX (this, btnClrRgt, wxT(">"), wxDP, wxSize(40, -1));

    pBoxSizerBTNsL->AddStretchSpacer(1);
    pBoxSizerBTNsL->Add(mpCBtnLft, 0, wxEXPAND);
    pBoxSizerBTNsL->AddSpacer(12);
    pBoxSizerBTNsL->Add(mpCBtnAddKey, 0, wxEXPAND);
    pBoxSizerBTNsL->Add(mpCBtnRemKey, 0, wxEXPAND);
    pBoxSizerBTNsL->Add(mpCBtnColor, 0, wxEXPAND);
    pBoxSizerBTNsL->AddSpacer(12);
    pBoxSizerBTNsL->Add(mpCBtnRgt, 0, wxEXPAND);
    pBoxSizerBTNsL->AddStretchSpacer(1);
    pBoxSizerLft->Add(pBoxSizerBTNsL, 0, wxEXPAND);

    SetMNumScale(pBoxSizerRgt);
    SetMNumColor(pBoxSizerRgt);
    
    pBoxSizerRgt->Add(mpCMInputPanelScale, 0, wxEXPAND);
    pBoxSizerRgt->Add(mpCMInputPanelColor, 0, wxEXPAND);

    mpCBtnFlipColor = new wxButtonEX(this, wxID_ANY, _("Flip Color"));
    mpCBtnFlipRGB   = new wxButtonEX(this, wxID_ANY, _("Swap RGB"));
    pBoxSizerBTNsR->Add(mpCBtnFlipColor, 1, wxEXPAND);
    pBoxSizerBTNsR->Add(mpCBtnFlipRGB, 1, wxEXPAND);
    pBoxSizerRgt->Add(pBoxSizerBTNsR, 0, wxEXPAND);
    ConnectButton(this, mpCBtnFlipColor, wxCEH(ColorCurveGUI_cls::OnButtonFlip));
    ConnectButton(this, mpCBtnFlipRGB, wxCEH(ColorCurveGUI_cls::OnButtonSwapRGB));
    pBoxSizerTop->Add(pBoxSizerRGBA, 0, wxEXPAND);
    pBoxSizerTop->Add(pBoxSizerLft, 1, wxEXPAND);
    pBoxSizerTop->Add(pBoxSizerRgt, 0, wxEXPAND);

    mpCPanelTemplate = new ColorCurveTemplate_cls(this, this, wxID_ANY);

    pBoxSizerSrc->Add(pBoxSizerTop, 1, wxEXPAND);
    pBoxSizerSrc->Add(mpCPanelTemplate, 0, wxEXPAND);
    
    SetSizer(pBoxSizerSrc);
 
    ResizeDialog();
}

void ColorCurveGUI_cls::
RecconectColor(ColorCurve_cls *pCColorLinked){
    MultiInputPanel_cls* pCurr = mpCMInputPanelScale;

    if (pCColorLinked == NULL){
	pCurr->mVCItemList[mIndexScaleMin].CombinePointer(_ITEM_FLOAT, &mDummy);
	pCurr->mVCItemList[mIndexScaleMax].CombinePointer(_ITEM_FLOAT, &mDummy);
	pCurr->mVCItemList[mIndexWrapType].CombinePointer(_ITEM_UINT_LIST, &mDummyValueType);
	pCurr->mVCItemList[mIndexValueType].CombinePointer(_ITEM_UINT_LIST, &mDummyValueType);
    }
    else {
	pCurr->mVCItemList[mIndexScaleMin].CombinePointer
	    (_ITEM_FLOAT, &pCColorLinked->mCScalingMin.mBody);
	pCurr->mVCItemList[mIndexScaleMax].CombinePointer
	    (_ITEM_FLOAT, &pCColorLinked->mCScalingMax.mBody);
	pCurr->mVCItemList[mIndexWrapType].CombinePointer
	    (_ITEM_UINT_LIST, &pCColorLinked->mCWrapType.mBody);
	pCurr->mVCItemList[mIndexValueType].CombinePointer
	    (_ITEM_UINT_LIST, &pCColorLinked->mCDependType.mBody);
    }
    UpdateMNumValues();
    pCurr->UpdateValues();
}

void ColorCurveGUI_cls::
SetMNumScale(wxBoxSizer *pBoxSizer){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputPanelScale, this);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_SNAPSHOT_DATA | _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_COLOR;
    
    int boxWidth = 2;
    pCurr->mTextWidth = 60;
    pCurr->mItemWidth = -1;
    
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Min/Max"));
    mIndexScaleMin = pCurr->CombinePointer(_ITEM_FLOAT, &mDummy);
    mIndexScaleMax = pCurr->CombinePointer(_ITEM_FLOAT, &mDummy);

    pCurr->mTextWidth = 90;

    wxString listWrap[2] = {_("Clamp to Edge"), _("Repeat")};
    mIndexWrapType = pCurr->CombinePointerList(&mDummyValueType, 2, listWrap, _("Wrapping"));

    wxString listValue[11] = {wxT("VALUE1"), _("Radius"), _("Position"), _("Position(X)"), _("Position(Y)"), _("Position(Z)"), _("Velocity"), _("Velocity(X)"), _("Velocity(Y)"), _("Velocity(Z)"), wxT("ID") };
    mIndexValueType = pCurr->CombinePointerList(&mDummyValueType, 11, listValue, _("Depend on"));
    
    pCurr->mMinimumWidth = -1;
    pCurr->FinishSetting(NULL);

    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_CHANGE);
}

void ColorCurveGUI_cls::
SetMNumColor(wxBoxSizer *pBoxSizer){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputPanelColor, this);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_GL;
    
    int boxWidth = 2;
    pCurr->mTextWidth = 60;
    pCurr->mItemWidth = -1;
    
    wxString strTag[4] = {wxT("R"), wxT("G"), wxT("B"), wxT("A")};
    for (int i = 0; i < 4; i++){
	mIndexColor[i] = 
	    pCurr->SetInputAndSlider(_ITEM_FLOAT, &mDummyPColor[i], strTag[i]);
	pCurr->SetRange(_ITEM_LASTITEM, 0, 255, 255);
    }
    
    pCurr->mMinimumWidth = -1;
    pCurr->FinishSetting(NULL);

    for (int i = 0; i < 4; i++){
	pCurr->SetHook(this, &HookItem_cls::HookFunc02, _HOOK_CHANGE, mIndexColor[i]+1);
	pCurr->SetHook(this, &HookItem_cls::HookFunc02, _HOOK_CHANGE, mIndexColor[i]+2);
    }
}

int ColorCurveGUI_cls::
GetContolMode(){
    if (mpCCtrlLink->mpCCurveLinked == NULL) return -1;
    return mpCCtrlLink->mpCCurveLinked->mpCClrCurve->mControlMode;
}

void ColorCurveGUI_cls::
SetColorComponent(int clrIndexId){
    MultiInputPanel_cls *pCurr = mpCMInputPanelColor;

    // XC_[𓮂2񗈂邱Ƃɒ
    int clr = 0;
        for (int i = 0; i < 4; i++)
	if (clrIndexId == pCurr->mVCItemList[mIndexColor[i]+1].mId ||
	    clrIndexId == pCurr->mVCItemList[mIndexColor[i]+2].mId)
	    clr = i;
    
    mDummyValueX = mpCMainGraph->GetDoubleX(mpCMainGraph->mGuideX);
    if (mpCCtrlLink->mpCCurveLinked != NULL)
	mpCCtrlLink->mpCCurveLinked->mpCClrCurve->
	    InsertKey(clr, mDummyValueX, mDummyPColor[clr]);
}

void ColorCurveGUI_cls::
UpdateMNumValues(){
    if (mpCCtrlLink == NULL || mpCCtrlLink->mpCCurveLinked == NULL) return;
    
    //mpCCtrlLink->mpCCurveLinked->PreCalcFactor();
    mDummyValueX = mpCMainGraph->GetDoubleX(mpCMainGraph->mGuideX);
    mpCCtrlLink->mpCCurveLinked->GetColor4fNormal(mDummyPColor, mDummyValueX);
    mpCMInputPanelScale->UpdateValues();
    mpCMInputPanelColor->UpdateValues();
}

void ColorCurveGUI_cls::
UpdateButtons(){
    float clr[3];
    mpCMainGraph->GetCurrentColor(clr);
    
    wxColour CClr = wxColour(clr[0] * 255, clr[1] * 255, clr[2] * 255);
    mpCBtnColor->SetBackgroundColour(CClr);
}

void ColorCurveGUI_cls::
UpdateButtonsRGBA(){
    switch(GetContolMode()){
    case -1:
	mpCBtnRed->SetBackgroundColour(wxNullColour);
	mpCBtnGreen->SetBackgroundColour(wxNullColour);
	mpCBtnBlue->SetBackgroundColour(wxNullColour);
	mpCBtnBlue->SetForegroundColour(wxNullColour);
	mpCBtnAlpha->SetBackgroundColour(wxNullColour);
	break;
    case 0:
	mpCBtnRed->SetBackgroundColour(wxColour(255, 0, 0));
	mpCBtnGreen->SetBackgroundColour(wxNullColour);
	mpCBtnBlue->SetBackgroundColour(wxNullColour);
	mpCBtnBlue->SetForegroundColour(wxNullColour);
	mpCBtnAlpha->SetBackgroundColour(wxNullColour);
	break;
    case 1:
	mpCBtnRed->SetBackgroundColour(wxNullColour);
	mpCBtnGreen->SetBackgroundColour(wxColour(0, 255, 0));
	mpCBtnBlue->SetBackgroundColour(wxNullColour);
	mpCBtnBlue->SetForegroundColour(wxNullColour);
	mpCBtnAlpha->SetBackgroundColour(wxNullColour);
	break;
    case 2:
	mpCBtnRed->SetBackgroundColour(wxNullColour);
	mpCBtnGreen->SetBackgroundColour(wxNullColour);
	mpCBtnBlue->SetBackgroundColour(wxColour(0, 0, 255));
	mpCBtnBlue->SetForegroundColour(wxColour(255, 255, 255));
	mpCBtnAlpha->SetBackgroundColour(wxNullColour);
	break;
    case 3:
	mpCBtnRed->SetBackgroundColour(wxNullColour);
	mpCBtnGreen->SetBackgroundColour(wxNullColour);
	mpCBtnBlue->SetBackgroundColour(wxNullColour);
	mpCBtnBlue->SetForegroundColour(wxNullColour);
	mpCBtnAlpha->SetBackgroundColour(wxColour(255, 255, 255)); 
	break;
    }    
}

void ColorCurveGUI_cls::
OnPaint(wxPaintEvent& event){
    mpCBtnClrCurve->Refresh(FALSE);
    mpCTics->Refresh(FALSE);
    event.Skip();
}

void ColorCurveGUI_cls::
OnMenuRefreshButton(wxCommandEvent& event){
    UpdateButtons();
    UpdateMNumValues();
}

void ColorCurveGUI_cls::
OnButtonAddKey(wxCommandEvent& event){
    mpCMainGraph->InsertKey();
}

void ColorCurveGUI_cls::
OnButtonRemKey(wxCommandEvent& event){
    mpCMainGraph->DeleteKey();
}

void ColorCurveGUI_cls::
OnButtonColor(wxCommandEvent& event){
    mpCMainGraph->SetKeyColor();
}

void ColorCurveGUI_cls::
OnButtonRGBA(wxCommandEvent& event){
    int mode = event.GetId() - btnClrRed;
    if (GetContolMode() == mode) mode = -1;
    mpCCtrlLink->SetControlMode(mode);
    mpCMainGraph->Refresh();
    UpdateButtonsRGBA();
}

void ColorCurveGUI_cls::
OnButtonLft(wxCommandEvent& event){
    double guide = mpCMainGraph->GetDoubleX(mpCMainGraph->mGuideX);
    guide = mpCCtrlLink->mpCCurveLinked->
	mpCClrCurve->GetPrevKeyPosDoubleIndex(guide, 1.0f/255);
    mpCMainGraph->SetGuide(guide);
    UpdateMNumValues();
    UpdateButtons();
}

void ColorCurveGUI_cls::
OnButtonRgt(wxCommandEvent& event){
    double guide = mpCMainGraph->GetDoubleX(mpCMainGraph->mGuideX);
    guide = mpCCtrlLink->mpCCurveLinked->
	mpCClrCurve->GetNextKeyPosDoubleIndex(guide, 1.0f/255);
    mpCMainGraph->SetGuide(guide);
    UpdateMNumValues();
    UpdateButtons();
}

void ColorCurveGUI_cls::
OnKeyDown(wxKeyEvent& event){
    if (event.GetModifiers() == wxMOD_NONE){
	if (event.GetKeyCode() == 'r')
	    mpCCtrlLink->SetControlMode(0);
	else if (event.GetKeyCode() == 'g')
	    mpCCtrlLink->SetControlMode(1);
	else if (event.GetKeyCode() == 'b')
	    mpCCtrlLink->SetControlMode(2);
	else if (event.GetKeyCode() == 'a')
	    mpCCtrlLink->SetControlMode(3);
	else mpCCtrlLink->SetControlMode(-1);
    }
    if (event.GetModifiers() == wxMOD_CONTROL)
	switch(event.GetKeyCode()){
	case 'Z':
	case 'Y':
	    SendKeyEventToTop(GpCMainFrameCLS, event);
	}
    event.Skip();
}

void ColorCurveGUI_cls::
OnButtonFlip(wxCommandEvent& event){
    mpCCtrlLink->mpCCurveLinked->Flip();
}

void ColorCurveGUI_cls::
OnButtonSwapRGB(wxCommandEvent& event){
    mpCCtrlLink->mpCCurveLinked->SwapRGB();
}

void ColorCurveGUI_cls::
HookFunc01(int id, void *pArg){
    Refresh(FALSE);
}

void ColorCurveGUI_cls::
HookFunc02(int id, void *pArg){
    SetColorComponent(id);
}

// -----

ColorCurveCtrl_cls::
ColorCurveCtrl_cls(/*FrameMain_cls* pCParentFrame*/){
    mpCCurveLinked = NULL;
    mpCGUI = NULL;
    //mpCParentFrame = pCParentFrame;
    //InitGUI(pCParentFrame);
}

void ColorCurveCtrl_cls::
DetachLink(ColorCurve_cls *pCClrCurve){
    if (mpCCurveLinked == pCClrCurve) LinkColorCurve(NULL);
}

void ColorCurveCtrl_cls::
LinkColorCurve(ColorCurve_cls *pCClrCurve){
    mpCCurveLinked = pCClrCurve;
    mpCGUI->RecconectColor(pCClrCurve);
    mpCGUI->mpCBtnClrCurve->LinkColorCurve(pCClrCurve);
    if (pCClrCurve == NULL){
	mpCGUI->mpCMainGraph->SetParamGroup(NULL);
	wxString strLabel;
	strLabel << _("Color Curve Editor") << wxT(" : NULL");
	mpCGUI->SetLabel(strLabel);
	return;
    }

    mpCGUI->mpCMainGraph->SetParamGroup(pCClrCurve->mpCClrCurve);
    mpCGUI->mpCMainGraph->Refresh(FALSE);
    mpCGUI->mpCTics->Refresh(FALSE);
    mpCGUI->mpCBtnClrCurve->Refresh(FALSE);
    
    wxString strLabel;
    strLabel << _("Color Curve Editor") << wxT(" : ") << pCClrCurve->mstrLabel;
    mpCGUI->SetLabel(strLabel);
}

void ColorCurveCtrl_cls::
SetControlMode(int mode){
    if (mpCCurveLinked == NULL) return;
    mpCCurveLinked->mpCClrCurve->mControlMode = mode;
}

void ColorCurveCtrl_cls::
InitGUI(FrameMain_cls *pCParentFrameLink){
    mpCParentFrame = pCParentFrameLink;
    mpCGUI = new ColorCurveGUI_cls(this, pCParentFrameLink);
    SetPDialog(mpCGUI);
}

void ColorCurveCtrl_cls::
UpdateItems(bool bForce){
    if (mpCGUI == NULL) return;

    mpCGUI->UpdateButtons();
    mpCGUI->Refresh(FALSE);
    mpCGUI->UpdateMNumValues();
    mpCGUI->mpCMInputPanelColor->UpdateValues();
}

void ColorCurveCtrl_cls::
CopyColorFromP(ColorCurve_cls *pCClrCurve, bool bCurveOnly){
    if (mpCCurveLinked != NULL)
	mpCCurveLinked->CopyColorFromP(pCClrCurve, bCurveOnly);
}
