
#include "clsGLAttribute.h"
#include "clsGLAttributeGUI.h"
#include "clsShaderCtrl.h"

#include "../FILTER/clsFilterCtrl.h"
#include "../FILTER/clsFilterObject.h"
#include "../GLATTRIB/clsColorCurve.h"
#include "../GLOBJECTS/clsSphere.h"
extern Sphere_cls *GpCSphere;
#include "../MYGL/clsFBO.h"
#include "../PARTICLEDATA/clsParticleDataSequence.h"
#include "../SCENEOBJ/clsLightSourceGUI.h"

#include "../MYAPI/MsgBox.h"

#include "../../GL/glsl.h"

#include "../COMMON/CommonDefinitions.h"

#include "../COMMON/CommonDefinitionsGL.h"

#ifdef WIN32
extern PFNGLBLENDFUNCSEPARATEPROC glBlendFuncSeparate;
#endif

#include "../COMMON/CommonDefinitions.h"

GLAttribute_cls::
GLAttribute_cls(PDataSequence_cls *pCParentSequence, int selfIndex) : 
    mpCGUI(NULL)
{
    mpCParentSequence = pCParentSequence;
    mpBufferRGBA = NULL;
    for (int i = 0; i < 2; i++){
	mBufferWidth[i] = 0; mBufferHeight[i] = 0;
	mTexture[i] = 0;
    }
    mSelfIndex = selfIndex;

    wxString strAttribTag;
    strAttribTag << wxT("SEQ") << mpCParentSequence->mSelfIndex
		 << wxT("TYPE") << mSelfIndex;
    wxString strLabel;
    strLabel << wxT("Attrib ") << mSelfIndex;
    
    mpCClrCurveCol = new ColorCurve_cls(strAttribTag + wxT("Col"),
					strLabel + wxT(" : Color"));
    mpCClrCurveMat[_GL_AMBIENT]  = new ColorCurve_cls
	(strAttribTag + wxT("Ambient"), strLabel + wxT(" : Ambient"), 
	 wxT("0.2 0.16 0.04 1"));
    mpCClrCurveMat[_GL_DIFFUSE]  = new ColorCurve_cls
	(strAttribTag + wxT("Diffuse"), strLabel + wxT(" : Diffuse"));
    mpCClrCurveMat[_GL_SPECULAR] = new ColorCurve_cls
	(strAttribTag + wxT("Specular"), strLabel + wxT(" : Specular"));

    mCConfList.SetTag(strAttribTag + wxT("GLAttrib"));
    mCConfList.Add(&mCDrawType, strAttribTag + wxT("DrawType="), wxT("0"));
    mCConfList.Add(&mCDrawQuality, strAttribTag + wxT("DrawQuality="), wxT("0"));
    mCConfList.Add(&mCBlendType, strAttribTag + wxT("BlendType="), wxT("0"));
    mCConfList.Add(&mCRadiusType, strAttribTag + wxT("RadiusType="), wxT("0"));
    mCConfList.Add(&mCRadiusFactor, strAttribTag + wxT("RadiusFactor="), wxT("1"));
    mCConfList.Add(&mCRadiusFactorForConst, strAttribTag + wxT("RadiusFactorForConst="), wxT("0.001"));
    mCConfList.Add(&mCPointSize, strAttribTag + wxT("PointSize="), wxT("10"));
    mCConfList.Add(&mCObjectIndex, strAttribTag + wxT("ObjectIndex="), wxT("0"));
    mCConfList.Add(&mCAlphaType, strAttribTag + wxT("AlphaType="), wxT("0"));
    mCConfList.Add(&mCAlphaFactor, strAttribTag + wxT("AlphaFactro="), wxT("1"));
    mCConfList.Add(&mCAlphaDenendBase, strAttribTag + wxT("AlphaDependBase="), wxT("1"));
    mCConfList.Add(&mCAlphaMax, strAttribTag + wxT("AlphaMax="), wxT("1"));
    mCConfList.Add(&mCGasScattering, strAttribTag+wxT("GasScattering="), wxT("1"));
    mCTexture.AppendDefaultItems(mCConfList, _TEX_FOR_PARTILCE, strAttribTag);

    mCTexture.mCstrTexFormat[0].mstrBody = wxT("Texture/ParticleA.png");

    mpCFBOBillboard = NULL;
    mpCFBOBillboardBuffer = NULL;
}

GLAttribute_cls::
~GLAttribute_cls(){
    if (mpCGUI != NULL) delete mpCGUI;
    if (mspCShaderCtrl != NULL){
	delete mspCShaderCtrl; mspCShaderCtrl = NULL;
    }
    if (mpCFBOBillboard != NULL){
	delete mpCFBOBillboard; mpCFBOBillboard = NULL;
    }
    if (mpCFBOBillboardBuffer != NULL){
	delete mpCFBOBillboardBuffer; mpCFBOBillboardBuffer = NULL;
    }
    delete mpCClrCurveCol;
    delete mpCClrCurveMat[_GL_AMBIENT];
    delete mpCClrCurveMat[_GL_DIFFUSE];
    delete mpCClrCurveMat[_GL_SPECULAR];
}

void GLAttribute_cls::
InitializeShader(wxWindow *pCParent, bool bShaderAvailable){
    if (mspCShaderCtrl == NULL){
	mspCShaderCtrl = new ShaderCtrl_cls();
	mspCShaderCtrl->InitializeShader(pCParent, bShaderAvailable);
    }
}

void GLAttribute_cls::
EnableShader(bool bUseShader){
    mspCShaderCtrl->EnableShader(bUseShader);
}

void GLAttribute_cls::
BeginGasShader(int lightNumber){
    switch(mCTexture.mCGasShader.mBody){	
    case 0:
	mspCShaderCtrl->BeginShader(NULL, 0);
	break;
    case 1:
	mspCShaderCtrl->BeginGasShader(&mCTexture, lightNumber, mCGasScattering.mBody);
	break;
    case 2:
	mspCShaderCtrl->BeginAlphaClipShader(&mCTexture, lightNumber);
	break;
    }
}

void GLAttribute_cls::
BeginShader(int lightNumber, bool bShadowVolume){
    mspCShaderCtrl->BeginShader(&mCTexture, lightNumber, bShadowVolume);
}

void GLAttribute_cls::
EndShader(){
    mspCShaderCtrl->EndShader(mCTexture.mCShaderType.mBody);
}

void GLAttribute_cls::
SetBlendFunc(){
    switch (mCBlendType.mBody){
    case _GL_BLEND:
	if (msBlendType == 0)
	    glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,
				GL_ONE_MINUS_DST_ALPHA, GL_ONE);
	else
	    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	break;
    case _GL_BLEND_ADD:
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glDisable(GL_DEPTH_TEST);
	break;
    }
}

void GLAttribute_cls::
InstantiateGUI(wxWindow *pCParent){
    mpCGUI = new GLAttributeGUI_cls(this, pCParent);
}

void GLAttribute_cls::
RefreshTex(int tex){
    wxString strTex = mCTexture.mCstrTexFormat[tex].mstrBody;
    if (mstrLoadedTex[tex].IsSameAs(strTex) == TRUE) 
	return;

    if (strTex.IsSameAs(wxT("NONE")) || strTex.IsEmpty() == TRUE){
	mCTexImg[tex] = wxImage(1, 1);
	mCTexImg[tex].SetRGB(0, 0, 255, 255, 255);
	mbTextureValid[tex] = FALSE;
    }
    else{
	mCTexImg[tex].LoadFile(strTex);
	mbTextureValid[tex] = TRUE;
    }

    if (mCTexImg[tex].IsOk() == TRUE)
	mstrLoadedTex[tex] = strTex;
    else 
	return;
    
    mCTexImg[tex] = mCTexImg[tex].Mirror(FALSE);

    if (mTexture[tex] != 0){
	glDeleteTextures(1, &mTexture[tex]);
    }
    
    glGenTextures(1, &mTexture[tex]);
    
    glBindTexture(GL_TEXTURE_2D, mTexture[tex]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    
    GenerateBufferRGBA(tex);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 
		 mBufferWidth[tex], mBufferHeight[tex],
		 0, GL_RGBA, GL_UNSIGNED_BYTE, mpBufferRGBA);
    DeleteBufferRGBA();
}

void GLAttribute_cls::
GenerateBufferRGBA(int tex){
    DeleteBufferRGBA(); 
    mBufferWidth[tex]  = mCTexImg[tex].GetWidth();
    mBufferHeight[tex] = mCTexImg[tex].GetHeight();
    unsigned char *pData  = mCTexImg[tex].GetData();
    mpBufferRGBA = new unsigned char [mBufferWidth[tex] * mBufferHeight[tex] * 4];
    if (mCTexImg[tex].HasAlpha() == TRUE){
	unsigned char *pAlpha = mCTexImg[tex].GetAlpha();
	for (int y = 0; y < mBufferHeight[tex]; y++){
	    int indexy = mBufferWidth[tex] * y;
	    for (int x = 0; x <  mBufferWidth[tex]; x++){
		int index = indexy + x;
		mpBufferRGBA[index * 4 + 0] = pData[index * 3 + 0];
		mpBufferRGBA[index * 4 + 1] = pData[index * 3 + 1];
		mpBufferRGBA[index * 4 + 2] = pData[index * 3 + 2];
		mpBufferRGBA[index * 4 + 3] = pAlpha[index];
	    }
	}
    }
    else{
	for (int y = 0; y < mBufferHeight[tex]; y++){
	    int indexy = mBufferWidth[tex] * y;
	    for (int x = 0; x <  mBufferWidth[tex]; x++){
		int index = indexy + x;
		mpBufferRGBA[index * 4 + 0] = pData[index * 3 + 0];
		mpBufferRGBA[index * 4 + 1] = pData[index * 3 + 1];
		mpBufferRGBA[index * 4 + 2] = pData[index * 3 + 2];
		mpBufferRGBA[index * 4 + 3] = 255;
	    }
	}
    }
}

void GLAttribute_cls::
DeleteBufferRGBA(){
    if (mpBufferRGBA != NULL) delete[] mpBufferRGBA;
    mpBufferRGBA = NULL;
    for (int i = 0; i < 2; i++){
	mBufferWidth[i] = 0;
	mBufferHeight[i] = 0;
    }
}

void GLAttribute_cls::
BindTexture(){
    switch(mCDrawType.mBody){
    case _GL_DRAW_BILLBOARD:
	glBindTexture(GL_TEXTURE_2D, mTexture[0]);
	break;
    case _GL_DRAW_SPHERE:
	glBindTexture(GL_TEXTURE_2D, mTexture[1]);
	break;
    case _GL_DRAW_PSEUDO_SPHERE:
    case _GL_DRAW_PSEUDO_OBJECT:
	mpCFBOBillboard->BindTexture();
	break;
    }
}

void GLAttribute_cls::
SetGLSettingForAtras(int bufferSize, double doubleIndex, 
		     FilterDataBoxData_cls *pCScene,
		     PositionAndDirection_cls CCamPos,
		     LightSRCCtrl_cls *pCLightSRCCtrlLink){
    EndShader();

    glEnable(GL_DEPTH_TEST);
    
    glEnable(GL_LIGHTING);
    pCLightSRCCtrlLink->SetNonShadowLight(doubleIndex, pCScene);

    GLPerspective_cls CGLPerspective(bufferSize, bufferSize, 2);
    CGLPerspective.mDepthMIN = 1.0;
    CGLPerspective.mDepthMAX = 100.0;
    PositionAndDirection_cls CNaviCameraPos;
    CNaviCameraPos = CCamPos;
    LOOP_XYZ CNaviCameraPos.mPos[xyz] = - 1/tan(1 * 3.14159/180) * CCamPos.mTgt[xyz];
    
    CGLPerspective.SetViewportOnGL();
    CGLPerspective.SetPerspectiveOnGL(&CNaviCameraPos);

    CGLPerspective.SetModelOrProjectionViewOnGL
	(&CNaviCameraPos, GL_MODELVIEW);
}

FilterObject_cls* GLAttribute_cls::
GetPCObject(){
    return mspCFilterCtrlObjectLink->GetPCObject(mCObjectIndex.mBody);
}

bool GLAttribute_cls::
IsAtrasNeeded(){
    int type = mCDrawType.mBody;
    if (type == _GL_DRAW_PSEUDO_OBJECT){
	FilterObject_cls *pCObj = GetPCObject();
	if (pCObj != NULL && 
	    pCObj->GetPCData()->mCbMultipleMesh.mbBody == TRUE &&
	    pCObj->GetTexGridSize() != 1) return TRUE;
    }
    return FALSE;
}

void GLAttribute_cls::
UpdateBillboardTexture
(PositionAndDirection_cls CCamPos, LightSRCCtrl_cls *pCLightSRCCtrlLink,
 double doubleIndex, FilterDataBoxData_cls *pCScene){
    FilterObject_cls *pCObj = GetPCObject();
    
    int type = mCDrawType.mBody;
    if (type != _GL_DRAW_PSEUDO_SPHERE && type != _GL_DRAW_PSEUDO_OBJECT) 
	return;
    if (type == _GL_DRAW_PSEUDO_OBJECT && pCObj == NULL) 
	return;

    int bufferSize = 0;
    switch(mCTexture.mCTextureSize.mBody){
    case 0: bufferSize = 128; break;
    case 1: bufferSize = 256; break;
    case 2: bufferSize = 512; break;
    case 3: bufferSize = 1024; break;
    }

    if (mpCFBOBillboard == NULL){
	mpCFBOBillboard = new FBO_cls (bufferSize, bufferSize);
	mpCFBOBillboard->SetDepth(8);
    }
    else
	mpCFBOBillboard->SetGeometryBuffer(bufferSize, bufferSize);

    int firstBufferSize = bufferSize * 2;
    int atrasGrid = 1;
    if (type == _GL_DRAW_PSEUDO_OBJECT &&
	pCObj->GetPCData()->mCbMultipleMesh.mbBody == TRUE){
	firstBufferSize = bufferSize;
    }

    if (mpCFBOBillboardBuffer == NULL){
	mpCFBOBillboardBuffer = new FBO_cls (firstBufferSize, firstBufferSize);
	mpCFBOBillboardBuffer->SetDepth(8);
    }
    else
	mpCFBOBillboardBuffer->SetGeometryBuffer(firstBufferSize, firstBufferSize);

    mpCFBOBillboard->MakeFBO(_FBO_USE_RGBA);
    mpCFBOBillboardBuffer->MakeFBO(_FBO_USE_RGBA);

    int lightNum = pCLightSRCCtrlLink->mCLightNum.mBody;
    if (type == _GL_DRAW_PSEUDO_SPHERE){

	mpCFBOBillboardBuffer->Begin(TRUE);
	SetGLSettingForAtras(firstBufferSize, doubleIndex,
			     pCScene, CCamPos, pCLightSRCCtrlLink);
	
	float pClrMat[3][4];
	for (int mat = 0; mat < 3; mat++){
	    mpCClrCurveMat[mat]->GetColor4fScaled(pClrMat[mat], 0); //!!
	}

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  pClrMat[0]);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  pClrMat[1]);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, pClrMat[2]);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 20);
    
	if (mbTextureValid[1] == TRUE){
	    glEnable(GL_TEXTURE_2D);
	    glBindTexture(GL_TEXTURE_2D, mTexture[1]);
	}
	else{
	    glDisable(GL_TEXTURE_2D);
	    glBindTexture(GL_TEXTURE_2D, 0);
	}

	BeginShader(lightNum);
	GpCSphere->Draw(4, 0);
	EndShader();

	mpCFBOBillboardBuffer->End();

	mpCFBOBillboard->Begin(TRUE);

	mpCFBOBillboardBuffer->DrawOnScreen2DFit
	    (bufferSize, bufferSize, _TEX_FIT_VERTICAL);
	
	mpCFBOBillboard->End();
	return;
    }

    if (type == _GL_DRAW_PSEUDO_OBJECT){
	MakeTextureAtras(pCObj, bufferSize, firstBufferSize,
			 doubleIndex, pCScene, CCamPos, pCLightSRCCtrlLink);
	return;
    }
}



void GLAttribute_cls::
MakeTextureAtras(FilterObject_cls *pCObj, 
		 int bufferSize, int firstBufferSize,
		 double doubleIndex, FilterDataBoxData_cls *pCScene,
		 PositionAndDirection_cls CCamPos, 
		 LightSRCCtrl_cls *pCLightSRCCtrlLink){
    // łɃobt@ł̃_Oݒ

    int lightNum = pCLightSRCCtrlLink->mCLightNum.mBody;
    int atrasGrid = pCObj->GetTexGridSize();
	    
    pCObj->CheckMatrixIdentity();
    pCObj->CheckForClearDisplayList();

    for (int i = 0; i < atrasGrid * atrasGrid; i++){
	
	mpCFBOBillboardBuffer->Begin(TRUE);
	SetGLSettingForAtras(firstBufferSize, doubleIndex,
			     pCScene, CCamPos, pCLightSRCCtrlLink);
	//pCObj->GLDrawSetMaterial();
	//pCObj->GLDrawSetTexture(0);
	
	if (pCObj->mDisplayList[i] == -1){
	    pCObj->GLMakeDisplayList(i, i, GL_COMPILE);
	}
	
	pCObj->GLDrawSetMaterial();
	pCObj->GLDrawSetTexture(0);
	pCObj->GLDrawSetShaderBegin(lightNum);
	glCallList(pCObj->mDisplayList[i]);
	EndShader();

	mpCFBOBillboardBuffer->End();
	
	if (i == 0)
	    mpCFBOBillboard->Begin(TRUE);
	else
	    mpCFBOBillboard->Begin(FALSE);

	mpCFBOBillboardBuffer->DrawOnScreen2DAtras
	    (bufferSize, bufferSize, i, atrasGrid, _TEX_FIT_VERTICAL);
 	
	mpCFBOBillboard->End();
    }
}

float GLAttribute_cls::
GetRadiusFactor(){
    if (mCRadiusType.mBody == _GL_RADIUS_CONSTANT)
	return mCRadiusFactorForConst.mBody;
    return mCRadiusFactor.mBody;
}

// ----

void GLAttribute_cls::
Memorize(){
    mstrStatusSerialized = Serialize();
    mstrStatusSerialized << IsAtrasNeeded();
}

bool GLAttribute_cls::
DoesUseDepthMask(){
    if (mCTexture.mCGasShader.mBody == 2 ||
	mCTexture.mCGasShader.mBody == 1) return TRUE; // !! FIX_ME !!
    return FALSE;
}

bool GLAttribute_cls::
IsObjectCalling(int index){
    if (mCDrawType.mBody == _GL_DRAW_OBJECT ||
	mCDrawType.mBody == _GL_DRAW_PSEUDO_OBJECT){
	if (mCObjectIndex.mBody == index) return TRUE;
    }
    return FALSE;
}

bool GLAttribute_cls::
IsDifferenceExist(){
    if (mstrStatusSerialized.IsSameAs(Serialize()))
	return FALSE;
    return TRUE;
}

bool GLAttribute_cls::
IsColorCurve(int index){
    return mpCClrCurveMat[index]->mCbColorCurve.mbBody;
}

bool GLAttribute_cls::
IsBlack(int index){
    if (mpCClrCurveMat[index]->mCbColorCurve.mbBody == TRUE) return FALSE;
    if (mpCClrCurveMat[index]->mCConstantColor.mpBody[0] == 0 &&
	mpCClrCurveMat[index]->mCConstantColor.mpBody[1] == 0 &&
	mpCClrCurveMat[index]->mCConstantColor.mpBody[2] == 0) return TRUE;
    return FALSE;
}

// ----

wxString GLAttribute_cls::
Serialize(){
    wxString strOut;
    for (int i = 0; i < 3; i++)
	strOut << mpCClrCurveMat[i]->Serialize();
    strOut << mpCClrCurveCol->Serialize();
    mCConfList.SerializeAllWithTag(strOut);
    return strOut;
}

void GLAttribute_cls::
ReadStrLine(std::wistream &istream, wxString strLine){
    for (int i = 0; i < 3; i++)
	mpCClrCurveMat[i]->ReadStrLine(istream, strLine);
    mpCClrCurveCol->ReadStrLine(istream, strLine);
    
    if (mCConfList.ReadStrLineAllWithTag(istream, strLine) == TRUE) return;;
    mCConfList.ReadStrLineAll(strLine); // !!_݊̂߂΂炭c_!!
}

FilterCtrl_cls* GLAttribute_cls::mspCFilterCtrlObjectLink = NULL;
ShaderCtrl_cls* GLAttribute_cls::mspCShaderCtrl = NULL;
DrawOption_cls* GLAttribute_cls::mspCDrawOptionLink = NULL;
int GLAttribute_cls::msBlendType = 0;
