// -*- c++ -*-
// "clsGLAttributeGUI.h"

#include "clsColorCurve.h"
#include "clsGLAttributeGUI.h"
#include "clsGLAttribute.h"

#include "../MAINCANVAS/clsTopHalfPanel.h"
#include "../PARTICLEDATA/clsSnapData.h"
extern bool GbSnapDataRefresh;
#include "../UNDO/clsUndoManager.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"

#include "../../wxWidgetEnumMenu.h"

#include "../COMMON/CommonDefinitions.h"

GLAttributeGUI_cls::
GLAttributeGUI_cls(GLAttribute_cls *pCAttribLink, wxWindow *pCParent){
    mpCAttribLink = pCAttribLink;
    mpCParent = pCParent;
    mpCMInputPanel = NULL;
    mTypePrev = -1;
    mAlphaPrev = -1;
    mRadiusTypePrev = -1;
    mGasShaderPrev = -1;
}

void GLAttributeGUI_cls::
SetPanel(){
    mpCMInputPanel = new MultiInputPanel_cls(mpCParent);
    
    MultiInputPanel_cls* pC = mpCMInputPanel;

#ifdef __WXMAC__
    pC->mTextWidth = 90;
#else
    pC->mTextWidth = 80;
#endif
    pC->mItemWidth = -1;
    pC->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_ATTRIB | _COMMAND_UPDATE_DISPLAY_TEXTURE |
	_COMMAND_UPDATE_DISPLAY_GL ;
    
    int boxWidth = 2;

    pC->CombinePointerSizer(3, _("Type/Quality"));
    wxString listType[8] = {_("Point"), _("Billboard"), _("Sphere"),
			    _("Both"), _("Object"), _("Pseudo Sphere"), 
			    _("Pseudo Object"), _("Hide") };
    pC->CombinePointerList
	(&mpCAttribLink->mCDrawType.mBody, 8, listType);
    
    wxString listQuality[3] = {_("Low"), _("Middle"), _("High")};
    mIndexQuality = pC->CombinePointerList
	(&mpCAttribLink->mCDrawQuality.mBody, 3, listQuality);

    mIndexObjIndex = pC->CombinePointer
	(_ITEM_INTEGER_SPIN, &mpCAttribLink->mCObjectIndex.mBody);

    boxWidth = 3;
    pC->CombinePointerSizer(3, _("Modes"));
    wxString listBlend[2] = {_("AlphaBlend"), _("Additive") };
    pC->CombinePointerList
	(&mpCAttribLink->mCBlendType.mBody, 
	 2, listBlend);
    wxString strListShader[4] = {_("original"), _("Gouroud"), _("Phong"), _("Toon")};
    mIndexShaderType = pC->CombinePointerList
	(&mpCAttribLink->mCTexture.mCShaderType.mBody,
	 4, strListShader);

    wxString strListGas[3] = {_("original"), _("Scatter"), _("Alpha Clip")};
    mIndexGasShader = pC->CombinePointerList
	(&mpCAttribLink->mCTexture.mCGasShader.mBody,
	 3, strListGas);

    wxString listRadisu[2] = {_("Use Data"), _("Constant (unity)")};
    mIndexRadiusType = pC->CombinePointerList
	(&mpCAttribLink->mCRadiusType.mBody,
	 2, listRadisu, _("Radius"));

    mIndexRadius = pC->CombinePointer
	(_ITEM_FLOAT, &mpCAttribLink->mCRadiusFactor.mBody, _("Radius Factor"));

    mIndexRadiusForConst = pC->CombinePointer
	(_ITEM_FLOAT, &mpCAttribLink->mCRadiusFactorForConst.mBody, _("Radius Factor"));
    
    pC->mItemWidth = 90;    
    boxWidth = 2;
    mIndexPointSize = pC->SetInputAndSlider
	(_ITEM_INTEGER_SPIN, &mpCAttribLink->mCPointSize.mBody, _("Point Size"));
    pC->SetRange(_ITEM_LASTITEM2, 1, 50);
    pC->SetRange(_ITEM_LASTITEM, 1, 50);
    
    pC->mItemWidth = -1;
    wxString listAlpha[2] = {_("Constant (unity)"), _("Depend on R (power -2)")};

    pC->CombinePointerList(&mpCAttribLink->mCAlphaType.mBody,
			   2, listAlpha, _("Alpha"));

    pC->mItemWidth = 90;
    mIndexAlphaFactor = pC->SetInputAndSlider
	(_ITEM_FLOAT, &mpCAttribLink->mCAlphaFactor.mBody, _("Alpha Factor"));
    pC->SetRange(_ITEM_LASTITEM, 0, 100, 100);

    mIndexAlphaBase = pC->CombinePointerSizer(2, _("Base/Max"));
    pC->CombinePointer
	(_ITEM_FLOAT, &mpCAttribLink->mCAlphaDenendBase.mBody);
    pC->CombinePointer
	(_ITEM_FLOAT, &mpCAttribLink->mCAlphaMax.mBody);
    
    ColorCurveBtn_cls *pCBtnCol = new ColorCurveBtn_cls
	(pC, mpCAttribLink->mpCClrCurveCol);

    ColorCurveBtn_cls *pCBtnAmient = new ColorCurveBtn_cls
	(pC, mpCAttribLink->mpCClrCurveMat[_GL_AMBIENT]);
    ColorCurveBtn_cls *pCBtnDiffuse = new ColorCurveBtn_cls
	(pC, mpCAttribLink->mpCClrCurveMat[_GL_DIFFUSE]);
    ColorCurveBtn_cls *pCBtnSpecular = new ColorCurveBtn_cls
	(pC, mpCAttribLink->mpCClrCurveMat[_GL_SPECULAR]);

    boxWidth = 2;
    pC->mItemWidth = -1;
    mIndexClr = pC->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Color"));
    pC->CombinePointer(_ITEM_WXPANEL, pCBtnCol,  _("Color"));
    mIndexScatter = pC->CombinePointer(_ITEM_FLOAT, &mpCAttribLink->mCGasScattering.mBody);

    boxWidth = 3;
    mIndexMat = pC->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Material"));
    pC->CombinePointer(_ITEM_WXPANEL, pCBtnAmient,  wxT("Ambient"));
    pC->CombinePointer(_ITEM_WXPANEL, pCBtnDiffuse, wxT("Diffuse"));
    pC->CombinePointer(_ITEM_WXPANEL, pCBtnSpecular, wxT("Specular"));

    wxPanel *pCDummy = new wxPanel(pC, wxID_ANY, wxDefaultPosition, wxSize(-1, 4));
    pC->CombinePointer(_ITEM_WXPANEL, pCDummy);
    
    pC->FinishSetting(NULL);
    
    pC->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX);
    pC->SetHook(this, &HookItem_cls::HookFuncMouse, _HOOK_MOUSE);

    HideAndShow();
}

void GLAttributeGUI_cls::
HideAndShow(){
    if (mTypePrev == mpCAttribLink->mCDrawType.mBody &&
	mAlphaPrev == mpCAttribLink->mCAlphaType.mBody &&
	mRadiusTypePrev == mpCAttribLink->mCRadiusType.mBody &&
	mGasShaderPrev == mpCAttribLink->mCTexture.mCGasShader.mBody) return;

    mpCMInputPanel->Freeze();

    // wxNoteBook̃TCYLayoutƏlɂǂiƎvĵ
    // \͈ɂB
    int drawType = mpCAttribLink->mCDrawType.mBody;

    if (drawType == _GL_DRAW_POINT){
	mpCMInputPanel->HideItem(mIndexRadius);
	mpCMInputPanel->HideItem(mIndexRadiusForConst);
	mpCMInputPanel->ShowItem(mIndexPointSize);
	mpCMInputPanel->EnableItem(mIndexRadiusType, FALSE);
    }
    else{
	mpCMInputPanel->HideItem(mIndexPointSize);
	if (mpCAttribLink->mCRadiusType.mBody == _GL_RADIUS_CONSTANT){
	    mpCMInputPanel->ShowItem(mIndexRadiusForConst);
	    mpCMInputPanel->HideItem(mIndexRadius);
	}
	else{
	    mpCMInputPanel->ShowItem(mIndexRadius);
	    mpCMInputPanel->HideItem(mIndexRadiusForConst);
	}
	mpCMInputPanel->EnableItem(mIndexRadiusType, TRUE);
    }
    
    if (drawType == _GL_DRAW_POINT || drawType == _GL_DRAW_BILLBOARD ||
	drawType == _GL_DRAW_PSEUDO_SPHERE || 
	drawType == _GL_DRAW_PSEUDO_OBJECT){
	mpCMInputPanel->ShowItem(mIndexClr);
	mpCMInputPanel->HideItem(mIndexMat);
    }
    else{
	mpCMInputPanel->HideItem(mIndexClr);
	mpCMInputPanel->ShowItem(mIndexMat);
    }

    if (drawType == _GL_DRAW_BILLBOARD || drawType == _GL_DRAW_PSEUDO_SPHERE ||
	drawType == _GL_DRAW_PSEUDO_OBJECT){
	mpCMInputPanel->ShowItem(mIndexGasShader);
	mpCMInputPanel->HideItem(mIndexShaderType);
	if (mpCAttribLink->mCTexture.mCGasShader.mBody == 1)
	    mpCMInputPanel->ShowItem(mIndexScatter);
	else
	    mpCMInputPanel->HideItem(mIndexScatter);
    }
    else{
	mpCMInputPanel->HideItem(mIndexGasShader);
	mpCMInputPanel->ShowItem(mIndexShaderType);
	mpCMInputPanel->HideItem(mIndexScatter);
    }

    if (drawType == _GL_DRAW_OBJECT){
	mpCMInputPanel->HideItem(mIndexQuality);
	mpCMInputPanel->ShowItem(mIndexObjIndex);
    }
    else{
	mpCMInputPanel->ShowItem(mIndexQuality);
	mpCMInputPanel->HideItem(mIndexObjIndex);
    }

    if (mpCAttribLink->mCAlphaType.mBody == _GL_ALPHA_INVERSE_SQUARE){
	mpCMInputPanel->HideItem(mIndexAlphaFactor);
	mpCMInputPanel->ShowItem(mIndexAlphaBase);
    }
    else{
	mpCMInputPanel->ShowItem(mIndexAlphaFactor);
	mpCMInputPanel->HideItem(mIndexAlphaBase);
    }

    mpCMInputPanel->Layout();

    mpCMInputPanel->Thaw();

    mTypePrev = mpCAttribLink->mCDrawType.mBody;
    mAlphaPrev = mpCAttribLink->mCAlphaType.mBody;
    mRadiusTypePrev = mpCAttribLink->mCRadiusType.mBody;
    mGasShaderPrev = mpCAttribLink->mCTexture.mCGasShader.mBody;
}

void GLAttributeGUI_cls::
HookFuncHS(int id, void *pArg){
    HideAndShow();
}

void GLAttributeGUI_cls::
HookFuncMouse(int id, void *pArg){
    wxMouseEvent event = *(wxMouseEvent*)pArg;

    if (event.LeftDown() == TRUE){
	SendMenuEventToTop(mpCMInputPanel, menuEditSelectGLAttrib);
    }
}

/*
void GLAttributeGUI_cls::
HookFunc01(int id, void *pArg){
    mpCMInputPanel->UpdateValues();
    GbSnapDataRefresh = TRUE;
    SendMenuEventToTop(mpCMInputPanel, menuEditRefreshGL);
}
*/

