
#include "clsShaderBase.h"

#include "../../GL/glsl.h"

#include "../MYAPI/MsgBox.h"

#ifdef WIN32
PFNGLPROGRAMPARAMETERIEXTPROC glProgramParameteri;
#endif

void ShaderBase_cls::
EnableShader(bool bUseShader){
    mbUseShader = bUseShader;
}

void ShaderBase_cls::
ShowLog(wxWindow *pCParent, int shader, wxString strShader){
    int logSize;
    int length;
    if (shader != -1){
	glGetShaderiv(shader, GL_INFO_LOG_LENGTH , &logSize);
	char *pLog = new char [logSize];
	glGetShaderInfoLog(shader, logSize, &length, pLog);
	wxString strLog(pLog, wxConvFile);
	wxString strMes = wxT("Compile error in ");
	strMes << strShader << wxT("\n");
	strMes << strLog;
	MsgBox(pCParent, strMes, wxT("!!"));
	delete [] pLog;
    }
    else{
	wxString strMes = wxT("Linking error in ");
	strMes << strShader << wxT("\n");
	MsgBox(pCParent, strMes, wxT("!!"));
    }
}

void ShaderBase_cls::
MakeShader(wxWindow *pCParent, GLuint &program,
	   wxString strVert, wxString strGeom, wxString strFrag){
    
    // VF[_vÕRpC^Nʂ𓾂ϐ
    GLint compiled, linked;
    
    bool bUseDummy = FALSE;

    GLuint vertShader = glCreateShader(GL_VERTEX_SHADER);
    GLuint geomShader = 0;
    GLuint fragShader = glCreateShader(GL_FRAGMENT_SHADER);
    GLuint fragShaderDummy = glCreateShader(GL_FRAGMENT_SHADER);

    // VF[_̃\[XvO̓ǂݍ 
    if (readShaderSource(vertShader, strVert.char_str()) == -1) 
	mbShaderAvailable = FALSE; 
    if (strGeom.IsEmpty() == FALSE){
	geomShader = glCreateShader(GL_GEOMETRY_SHADER);
	if (readShaderSource(geomShader, strGeom.char_str()) == -1)
	    mbShaderAvailable = FALSE;
    }
    if (readShaderSource(fragShader, strFrag.char_str()) == -1)
	mbShaderAvailable = FALSE;
    if (readShaderSource(fragShaderDummy, "GL/Dummy.frag") == -1)
	mbShaderAvailable = FALSE;

    if (mbShaderAvailable == FALSE) return;
    
    // o[ebNXVF[_̃\[XvÕRpC 
    glCompileShader(vertShader);
    glGetShaderiv(vertShader, GL_COMPILE_STATUS, &compiled);
    if (compiled == GL_FALSE) {
	ShowLog(pCParent, vertShader, strVert);	
	mbShaderAvailable = FALSE; return;
    }

    // WIgVF[_̃\[XvÕRpC 
    if (geomShader != 0){
	glCompileShader(geomShader);
	glGetShaderiv(geomShader, GL_COMPILE_STATUS, &compiled);
	if (compiled == GL_FALSE) {
	    ShowLog(pCParent, geomShader, strGeom);	
	    mbShaderAvailable = FALSE; return;
	}
    }

    // tOgVF[_̃\[XvÕRpC 
    glCompileShader(fragShader);
    glGetShaderiv(fragShader, GL_COMPILE_STATUS, &compiled);
    if (compiled == GL_FALSE) {
	ShowLog(pCParent, fragShader, strFrag);

	glCompileShader(fragShaderDummy);
	glGetShaderiv(fragShaderDummy, GL_COMPILE_STATUS, &compiled);
	if (compiled == GL_FALSE) {
	    ShowLog(pCParent, fragShader, wxT("dummy-fragment"));
	    mbShaderAvailable = FALSE; return;
	}
	bUseDummy = TRUE;
    }
    
    // vOIuWFNg̍쐬 
    program = glCreateProgram( );
    
    // VF[_IuWFNg̃VF[_vOւ̓o^ 
    glAttachShader(program, vertShader);
    if (geomShader != 0){
	glAttachShader(program, geomShader);

	int max;
	glGetIntegerv(GL_MAX_GEOMETRY_OUTPUT_VERTICES_EXT, &max);

	if (glProgramParameteri != 0){
	    glProgramParameteri(program, GL_GEOMETRY_INPUT_TYPE_EXT, GL_TRIANGLES);
	    glProgramParameteri(program, GL_GEOMETRY_OUTPUT_TYPE_EXT, GL_TRIANGLES);
	    glProgramParameteri(program, GL_GEOMETRY_VERTICES_OUT_EXT, 6);
	}
    }

    if (bUseDummy == TRUE)
	glAttachShader(program, fragShaderDummy);
    else
	glAttachShader(program, fragShader);

    // VF[_vÕN 
    glLinkProgram(program);
    glGetProgramiv(program, GL_LINK_STATUS, &linked);
    if (linked == GL_FALSE) {
	ShowLog(pCParent, -1, strFrag);
	mbShaderAvailable = FALSE;
	return;
    }

    // VF[_IuWFNg̍폜 
    glDeleteShader(vertShader);
    glDeleteShader(fragShader);
    glDeleteShader(fragShaderDummy);
}
