// -*- c -*- 
// "clsShaderCtrl.h"

#include "clsShaderCtrl.h"

#include "../SCENEOBJ/clsSceneObjTexture.h"

#include "../../GL/glsl.h"
#include "../MYAPI/MsgBox.h"

#include "../COMMON/CommonDefinitions.h"

#ifdef WIN32
extern PFNGLPROGRAMPARAMETERIEXTPROC glProgramParameteri;
#endif

ShaderCtrl_cls::
ShaderCtrl_cls(){
    mCurrentShader = 0;
}

void ShaderCtrl_cls::
InitializeShader(wxWindow *pCParent, bool bShaderAvailable){
    mbShaderAvailable = bShaderAvailable;
    if (mbShaderAvailable == FALSE) return;
    
#ifdef WIN32
    glProgramParameteri = (PFNGLPROGRAMPARAMETERIEXTPROC)wglGetProcAddress("glProgramParameteri");
#endif
    
    MakeShader(pCParent, mGL2ProgramGouraud, 
	       wxT("GL/Gouraud.vert"), wxES, wxT("GL/Gouraud.frag"));
    MakeShader(pCParent, mGL2ProgramPhong,
	       wxT("GL/Phong.vert"), wxES, wxT("GL/Phong.frag"));
    MakeShader(pCParent, mGL2ProgramCartoon,
	       wxT("GL/Cartoon.vert"), wxES, wxT("GL/Cartoon.frag"));
    MakeShader(pCParent, mGL2ProgramOutline,
	       wxT("GL/Outline.vert"), wxT("GL/Outline.geom"),
	       wxT("GL/Outline.frag"));
    MakeShader(pCParent, mGL2ProgramGas,
	       wxT("GL/Gas.vert"), wxES, wxT("GL/Gas.frag"));
    MakeShader(pCParent, mGL2ProgramAlphaClip,
	       wxT("GL/AlphaClip.vert"), wxES, wxT("GL/AlphaClip.frag"));
    
    MakeShader(pCParent, mGL2ProgramShadow,
	       wxT("GL/Shadow.vert"), wxES, wxT("GL/Shadow.frag"));
    
    
    SetShaderParamIDs();

    UseProgram(0);
}

int ShaderCtrl_cls::
GetFogMode(){
    GLboolean bFog;
    int fogMode; 
    glGetBooleanv(GL_FOG, &bFog);
    glGetIntegerv(GL_FOG_MODE, &fogMode);
    if (bFog == GL_FALSE) fogMode = 0;
    else 
	switch(fogMode){
	case GL_EXP   : fogMode = 1; break;
	case GL_EXP2  : fogMode = 2; break;
	case GL_LINEAR: fogMode = 3; break;
	}
    return fogMode;
}

int ShaderCtrl_cls::
GetTextureMode(){
    GLboolean bTexture2D;
    int textureMode = 0;
    glGetBooleanv(GL_TEXTURE_2D, &bTexture2D);
    if (bTexture2D == GL_TRUE) textureMode = 1;
    return textureMode;
}

void ShaderCtrl_cls::
GetLightingInfo(int lightRecieve[7], bool *pbLightRecieve){
    if (msbSingleLightingMode == TRUE){
	lightRecieve[0] = pbLightRecieve[msSingleLightingIndex];
	return;
    }
    for (int i = 0; i < 7; i++)
	lightRecieve[i] = (int)pbLightRecieve[i];
}

void ShaderCtrl_cls::
UseProgram(int program){
    if (mCurrentShader != program){
	glUseProgram(program);
	mCurrentShader = program;
    }
}

void ShaderCtrl_cls::
BeginGasShader(SceneObjTexture_cls *pCTexture, int lightNum, float scatter){
    if (mbShaderAvailable == FALSE) return;
    if (mbUseShader == FALSE){
	UseProgram(0);
	return;
    }

    bool bFogTarget = pCTexture->mCbFogRecieve.mbBody;
    int fogMode = (bFogTarget == TRUE) ? GetFogMode() : 0;
    float clipThreshold = pCTexture->mCAlphaClipping.mBody;
    float ambientFactor = pCTexture->mCScatterAmbientFacter.mBody;
    
    UseProgram(mGL2ProgramGas);
    SetShaderParamGas(lightNum, fogMode, clipThreshold, scatter, ambientFactor);
}

void ShaderCtrl_cls::
BeginAlphaClipShader(SceneObjTexture_cls *pCTexture, int lightNum){
    if (mbShaderAvailable == FALSE) return;
    if (mbUseShader == FALSE){
	UseProgram(0);
	return;
    }

    bool bFogTarget = pCTexture->mCbFogRecieve.mbBody;
    int fogMode = (bFogTarget == TRUE) ? GetFogMode() : 0;
    float clipThreshold = pCTexture->mCAlphaClipping.mBody;

    UseProgram(mGL2ProgramAlphaClip);
    SetShaderParamAlphaClip(lightNum, fogMode, clipThreshold);
}

void ShaderCtrl_cls::
BeginShader(SceneObjTexture_cls *pCTexture, int lightNumber, bool bShadowVolume){
    if (mbShaderAvailable == FALSE) return;
    if (pCTexture == NULL || mbUseShader == FALSE){
	UseProgram(0);
	return;
    }

    int shaderType = pCTexture->mCShaderType.mBody;
    bool bFogTarget = pCTexture->mCbFogRecieve.mbBody;
    int fogMode = (bFogTarget == TRUE) ? GetFogMode() : 0;
    bool *pLightRecieve = pCTexture->mCbLightRecieve.mpbBody;
    int textureState = (GetTextureMode() == 1) ? _TEXTURE_ENABLED : _TEXTURE_DISABLED;
    if (bShadowVolume == TRUE) shaderType = 4;
    switch(shaderType){
    case 0:
	UseProgram(0);
	break;
    case 1:
	UseProgram(mGL2ProgramGouraud);
	SetTextureState(textureState);
	SetShaderParamGouraud(lightNumber, fogMode, pLightRecieve);
	break;
    case 2:{
	UseProgram(mGL2ProgramPhong);
	SetTextureState(textureState);
	SetShaderParamPhong(lightNumber, fogMode, pLightRecieve);
	break; }
    case 3: {
	if (pCTexture->mCOutlineBoldness.mBody == 0 ||
	    glProgramParameteri == NULL){
	    UseProgram(mGL2ProgramCartoon);
	    SetTextureState(textureState);
	    SetShaderParamPhong(lightNumber, fogMode, pLightRecieve);
	    SetShaderParamCartoon(lightNumber, fogMode);
	    SetShaderParamCartoonBoldness(pCTexture->mCCartoonBoldness.mBody);
	    float ambient[4]; 
	    glGetLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	    if (lightNumber == 1 && ambient[0] == 0)
		SetShaderParamCartoonShadowMode(1);
	    else
		SetShaderParamCartoonShadowMode(0);
	    
	}
	else{
	    glEnable(GL_CULL_FACE);
	    UseProgram(mGL2ProgramOutline);
	    SetTextureState(textureState);
	    SetShaderParamPhong(lightNumber, fogMode, pLightRecieve);
	    SetShaderParamOutline(lightNumber, fogMode);
	    SetShaderParamOutlineBoldness(pCTexture->mCCartoonBoldness.mBody,
					  pCTexture->mCOutlineBoldness.mBody);
	    float ambient[4]; 
	    glGetLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	    if (lightNumber == 1 && ambient[0] == 0)
		SetShaderParamOutlineShadowMode(1);
	    else
		SetShaderParamOutlineShadowMode(0);
	}
	break; }
    case 4:
	UseProgram(mGL2ProgramShadow);
	break;
    }
}

void ShaderCtrl_cls::
EndShader(int shaderType){
    if (mbShaderAvailable == FALSE) return;
    UseProgram(0);
}

void ShaderCtrl_cls::
SetShaderParamIDs(){
    UseProgram(mGL2ProgramGouraud);
    mIDLNumGouraud = 
	glGetUniformLocation(mGL2ProgramGouraud, "lightNumber");
    mIDTextureGouraud = 
	glGetUniformLocation(mGL2ProgramGouraud, "textureImage");
    mIDTexModeGouraud = 
	glGetUniformLocation(mGL2ProgramGouraud, "textureMode");
    mIDFogModeGouraud = 
	glGetUniformLocation(mGL2ProgramGouraud, "fogMode");
    mIDLightRecieveGouraud = 
	glGetUniformLocation(mGL2ProgramPhong, "lightRecieve");

    UseProgram(mGL2ProgramPhong);
    mIDLNumPhong   = 
	glGetUniformLocation(mGL2ProgramPhong, "lightNumber");
    mIDTexturePhong = 
	glGetUniformLocation(mGL2ProgramPhong, "textureImage");
    mIDTexModePhong = 
	glGetUniformLocation(mGL2ProgramPhong, "textureMode");
    mIDFogModePhong = 
	glGetUniformLocation(mGL2ProgramPhong, "fogMode");
    mIDLightRecievePhong = 
	glGetUniformLocation(mGL2ProgramPhong, "lightRecieve");
    
    UseProgram(mGL2ProgramCartoon);
    mIDLNumCartoon = 
	glGetUniformLocation(mGL2ProgramCartoon, "lightNumber");
    mIDTextureCartoon = 
	glGetUniformLocation(mGL2ProgramCartoon, "textureImage");
    mIDTexModeCartoon = 
	glGetUniformLocation(mGL2ProgramCartoon, "textureMode");
    mIDFogModeCartoon = 
	glGetUniformLocation(mGL2ProgramCartoon, "fogMode");
    mIDShadowModeCartoon = 
	glGetUniformLocation(mGL2ProgramCartoon, "shadowVolumeMode");
    mIDEdgeThresholdCartoon = 
	glGetUniformLocation(mGL2ProgramCartoon, "edgeThreshold");

    UseProgram(mGL2ProgramOutline);
    mIDLNumOutline = 
	glGetUniformLocation(mGL2ProgramOutline, "lightNumber");
    mIDTextureOutline = 
	glGetUniformLocation(mGL2ProgramOutline, "textureImage");
    mIDTexModeOutline = 
	glGetUniformLocation(mGL2ProgramOutline, "textureMode");
    mIDFogModeOutline = 
	glGetUniformLocation(mGL2ProgramOutline, "fogMode");
    mIDShadowModeOutline = 
	glGetUniformLocation(mGL2ProgramOutline, "shadowVolumeMode");
    mIDEdgeThresholdOutline = 
	glGetUniformLocation(mGL2ProgramOutline, "edgeThreshold");
    mIDBoldnessOutline = 
	glGetUniformLocation(mGL2ProgramOutline, "boldness");

    UseProgram(mGL2ProgramGas);
    mIDTextureGas = 
	glGetUniformLocation(mGL2ProgramGas, "textureImage");
    mIDLNumGas = 
	glGetUniformLocation(mGL2ProgramGas, "lightNumber");
    mIDFogModeGas = 
	glGetUniformLocation(mGL2ProgramGas, "fogMode");
    mIDScatterGas = 
	glGetUniformLocation(mGL2ProgramGas, "scatter");
    mIDClipThresholdGas = 
	glGetUniformLocation(mGL2ProgramGas, "clipThreshold");
    mIDAmbientFactorScatter = 
	glGetUniformLocation(mGL2ProgramGas, "ambientFactor");

    UseProgram(mGL2ProgramAlphaClip);
    mIDTextureAlphaClip = 
	glGetUniformLocation(mGL2ProgramAlphaClip, "textureImage");
    mIDLNumAlphaClip = 
	glGetUniformLocation(mGL2ProgramAlphaClip, "lightNumber");
    mIDFogModeAlphaClip = 
	glGetUniformLocation(mGL2ProgramAlphaClip, "fogMode");
    mIDClipThresholdClip = 
	glGetUniformLocation(mGL2ProgramAlphaClip, "clipThreshold");
}

void ShaderCtrl_cls::
SetTextureState(int texState){
    GLint program;
    glGetIntegerv(GL_CURRENT_PROGRAM, (GLint*) &program); 
    if (program == mGL2ProgramPhong)
	glUniform1i(mIDTexModePhong, texState);
    if (program == mGL2ProgramGouraud)
	glUniform1i(mIDTexModeGouraud, texState);
    if (program == mGL2ProgramCartoon)
	glUniform1i(mIDTexModeCartoon, texState);
    if (program == mGL2ProgramOutline)
	glUniform1i(mIDTexModeOutline, texState);
}

void ShaderCtrl_cls::
SetShaderParamGouraud(int LNum, int fogMode, bool *pbLightRecieve){
    glUniform1i(mIDTextureGouraud, 0);
    glUniform1i(mIDLNumGouraud, LNum);
    glUniform1i(mIDFogModeGouraud, fogMode);
    int lightRecieve[7];
    GetLightingInfo(lightRecieve, pbLightRecieve);
    /*
    for (int i = 0; i < 7; i++)
	lightRecieve[i] = (int)pbLightRecieve[i];
    */
    glUniform1iv(mIDLightRecieveGouraud, 7, lightRecieve);
}

void ShaderCtrl_cls::
SetShaderParamPhong(int LNum, int fogMode, bool *pbLightRecieve){
    glUniform1i(mIDTexturePhong, 0);
    glUniform1i(mIDLNumPhong, LNum);
    glUniform1i(mIDFogModePhong, fogMode);
    int lightRecieve[7];
    GetLightingInfo(lightRecieve, pbLightRecieve);
    /*
    for (int i = 0; i < 7; i++)
	lightRecieve[i] = (int)pbLightRecieve[i];
    */
    glUniform1iv(mIDLightRecievePhong, 7, lightRecieve);
}

void ShaderCtrl_cls::
SetShaderParamCartoon(int LNum, int fogMode){
    glUniform1i(mIDTextureCartoon, 0);
    glUniform1i(mIDLNumCartoon, LNum);
    glUniform1i(mIDFogModeCartoon, fogMode);
}

void ShaderCtrl_cls::
SetShaderParamCartoonBoldness(float edgeThreshold){
    glUniform1f(mIDEdgeThresholdCartoon, edgeThreshold);
}

void ShaderCtrl_cls::
SetShaderParamCartoonShadowMode(bool bShadowMode){
    if (bShadowMode == 0)
	glUniform1i(mIDShadowModeCartoon, 1);
    else
	glUniform1i(mIDShadowModeCartoon, 0);
}

void ShaderCtrl_cls::
SetShaderParamOutline(int LNum, int fogMode){
    glUniform1i(mIDTextureOutline, 0);
    glUniform1i(mIDLNumOutline, LNum);
    glUniform1i(mIDFogModeOutline, fogMode);
}

void ShaderCtrl_cls::
SetShaderParamOutlineBoldness(float edgeThreshold, float boldness){
    glUniform1f(mIDEdgeThresholdOutline, edgeThreshold);
    glUniform1f(mIDBoldnessOutline, boldness);
}

void ShaderCtrl_cls::
SetShaderParamOutlineShadowMode(bool bShadowMode){
    if (bShadowMode == 0)
	glUniform1i(mIDShadowModeOutline, 1);
    else
	glUniform1i(mIDShadowModeOutline, 0);
}

void ShaderCtrl_cls::
SetShaderParamGas(int LNum, int fogMode, float clipThreshold, float scatter, float ambientFactor){
    glUniform1i(mIDTextureGas, 0);
    glUniform1i(mIDLNumGas, LNum);
    glUniform1i(mIDFogModeGas, fogMode);
    glUniform1f(mIDClipThresholdGas, clipThreshold);
    glUniform1f(mIDScatterGas, scatter);
    glUniform1f(mIDAmbientFactorScatter, ambientFactor);
}

void ShaderCtrl_cls::
SetShaderParamAlphaClip(int LNum, int fogMode, float clipThreshold){
    glUniform1i(mIDTextureAlphaClip, 0);
    glUniform1i(mIDLNumAlphaClip, LNum);
    glUniform1f(mIDClipThresholdClip, clipThreshold);
    glUniform1i(mIDFogModeAlphaClip, fogMode);
}

bool ShaderCtrl_cls::msbSingleLightingMode = FALSE;
int  ShaderCtrl_cls::msSingleLightingIndex = 0;
