// "clsGLButton.h"
// (setq c-basic-offset 4)

#include "clsGLButton.h"

#include "../../COMMON/CommonDefinitions.h"

GLButton_cls::
GLButton_cls(int offsetType, int offsetX, int offsetY){
    mOffsetType = offsetType;
    mOffsetX = offsetX;
    mOffsetY = offsetY;
}

void GLButton_cls::
MakeTextureFromImage(wxString strPath){
    glGenTextures(1, &mTexID);

    wxImage CImage(strPath);
    CImage = CImage.Mirror(FALSE);
    mWidth = CImage.GetWidth();
    mHeight = CImage.GetHeight();

    unsigned char *pRGB = CImage.GetData();
    unsigned char *pAlpha = CImage.GetAlpha();

    int size = mWidth * mHeight;
    unsigned char *pTgt = new unsigned char [size * 4];

    for (int i = 0; i < size; i++){
	pTgt[i * 4] = pRGB[i*3];
	pTgt[i * 4 + 1] = pRGB[i*3 + 1];
	pTgt[i * 4 + 2] = pRGB[i*3 + 2];
	pTgt[i * 4 + 3] = pAlpha[i];
    }

    glBindTexture(GL_TEXTURE_2D, mTexID);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, mWidth, mHeight, 0, 
		 GL_RGBA, GL_UNSIGNED_BYTE, pTgt);

    delete pTgt;
}

void GLButton_cls::
DrawButton(int canvasWidth, bool bState){

    int x0, y0;
    switch(mOffsetType){
    case _BTM_LEFT:
	x0 = mOffsetX;
	y0 = mOffsetY;
	break;
    case _BTM_RIGHT:
	x0 = canvasWidth - mOffsetX;
	y0 = mOffsetY;
	break;
    }
    int x1 = x0 + mWidth;
    int y1 = y0 + mHeight;

    glPushAttrib(GL_ALL_ATTRIB_BITS);
    
    if (mbMouseOn == TRUE){
	glDisable(GL_TEXTURE_2D);
	glColor4f(0.2f, 0.2f, 0.3f, 1.0f);
	glBegin(GL_QUADS);
	glVertex2d(x0, y0); glVertex2d(x1, y0);
	glVertex2d(x1, y1); glVertex2d(x0, y1);
	glEnd();
	glColor4f(0.3f, 0.3f, 0.5f, 1.0f);
	glBegin(GL_LINE_LOOP);
	glVertex2d(x0, y0); glVertex2d(x1, y0);
	glVertex2d(x1, y1); glVertex2d(x0, y1);
	glEnd();
    }

    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); //At@

    glBindTexture( GL_TEXTURE_2D, mTexID);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glBegin(GL_QUADS);
    if (bState == FALSE)
	glColor4f(0.3f, 0.3f, 0.28f, 1.0f);
    else
	glColor4f(0.9f, 0.9f, 1.0f, 1.0f);
    
    glTexCoord2f(0, 0); glVertex2d(x0, y0);
    glTexCoord2f(1, 0); glVertex2d(x1, y0);
    glTexCoord2f(1, 1); glVertex2d(x1, y1);
    glTexCoord2f(0, 1); glVertex2d(x0, y1);

    glEnd();

    glPopAttrib();
}

bool GLButton_cls::
IsMouseOnButton(int canvasWidth, int canvasHeight,
		int mouseX, int mouseY){

    int x0, y0;
    switch(mOffsetType){
    case _BTM_LEFT:
	x0 = mOffsetX;
	y0 = mOffsetY;
	break;
    case _BTM_RIGHT:
	x0 = canvasWidth - mOffsetX;
	y0 = mOffsetY;
	break;
    }
    int x1 = x0 + mWidth;
    int y1 = y0 + mHeight;

    // yWnt
    y0 = canvasHeight - y0;
    y1 = canvasHeight - y1;

    if (mouseX > x0 && mouseX < x1 &&
	mouseY > y1 && mouseY < y0) return TRUE;

    return FALSE;
}

bool GLButton_cls::
UpdateStatus(int canvasWidth, int canvasHeight,
	     int mouseX, int mouseY){
    bool bMouseOn = IsMouseOnButton(canvasWidth, canvasHeight, mouseX, mouseY);
    if (mbMouseOn != bMouseOn){
	mbMouseOn = bMouseOn;
	return TRUE;
    }
    return FALSE;
}

