// -*- c++ -*- 
// "clsDistortion.h"
// (setq c-basic-offset 4)

#include "clsDistortion.h"

#include "../MYGL/clsGLPerspective.h"

#include "../COMMON/CommonDefinitions.h"

Distortion_cls::
Distortion_cls(){
    mRotDeg = 0;
    mTiltDeg = 0;
    mAngleDeg = 180;
    mbMeshOK = FALSE;
}

void Distortion_cls::
ReadWavefrontObject(wxString strFilePath){
    std::fstream ifstream(wxFNCONV(strFilePath), std::ios::in);

    if (ifstream.fail() == TRUE) return;

    // USɂ߂Ă݂B
    int len;
    ifstream.seekg (0, std::ios::end); 
    len = ifstream.tellg ();
    ifstream.seekg (0, std::ios::beg);
    std::string strBuf; strBuf.reserve(len); strBuf.resize(len);
    
    ifstream.read (&(strBuf[0]), len);

    std::ios::sync_with_stdio(false);

    std::istringstream sstream(strBuf);
    std::string strLine;
    std::getline(sstream, strLine);
    std::getline(sstream, strLine);
    std::getline(sstream, strLine);
    while (std::getline(sstream, strLine)){
	char charTag[8];
	float x, y, z;
	sscanf(strLine.data(), "%s %f %f %f",
	       charTag, &x, &y, &z);
	wxString strTag = wxString::FromAscii(charTag);
	bool bBreak = FALSE;
	if (strTag == wxT("v")){
	    mVVertexX.push_back(x);
	    mVVertexY.push_back(y);
	    mVVertexZ.push_back(z);
	}
	else if (strTag == wxT("vt")){
	    if (x < 0) x = 0;
	    if (x > 1) x = 1;
	    if (y < 0) y = 0;
	    if (y > 1) y = 1;
	    mVTexcordX.push_back(x);
	    mVTexcordY.push_back(y);
	}
	else if (strTag == wxT("vn")){
	    mVNormalX.push_back(x);
	    mVNormalY.push_back(y);
	}
	else if (strTag == wxT("s"))
	    break;
    }
    std::getline(sstream, strLine);
    while (std::getline(sstream, strLine)){
	char  charTag[8];
	int verex1, coord1, normal1;
	int verex2, coord2, normal2;
	int verex3, coord3, normal3;
	int verex4, coord4, normal4;
	sscanf(strLine.data(), "%s %d/%d/%d %d/%d/%d %d/%d/%d %d/%d/%d",
	       charTag, 
	       &verex1, &coord1, &normal1,
	       &verex2, &coord2, &normal2,
	       &verex3, &coord3, &normal3,
	       &verex4, &coord4, &normal4);

	mVIndexVert.push_back(verex1);
	mVIndexVert.push_back(verex2);
	mVIndexVert.push_back(verex3);
	mVIndexVert.push_back(verex4);
	mVIndexCoord.push_back(coord1);
	mVIndexCoord.push_back(coord2);
	mVIndexCoord.push_back(coord3);
	mVIndexCoord.push_back(coord4);
	mVIndexNormal.push_back(normal1);
	mVIndexNormal.push_back(normal2);
	mVIndexNormal.push_back(normal3);
	mVIndexNormal.push_back(normal4);
    }

    mVTexTransformedX = mVTexcordX;
    mVTexTransformedY = mVTexcordY;
    
    CheckGeometry();
    mbMeshOK = TRUE;
}

void Distortion_cls::
CheckGeometry(){
    int size = mVVertexX.size();
    float minX = 0, maxX = 0, minZ = 0, maxZ = 0;
    for (int i = 0; i < size; i++){
	if (minX > mVVertexX[i]) minX = mVVertexX[i];
	if (maxX < mVVertexX[i]) maxX = mVVertexX[i];
	if (minZ > mVVertexZ[i]) minZ = mVVertexZ[i];
	if (maxZ < mVVertexZ[i]) maxZ = mVVertexZ[i];
    }
    mObjSizeX = maxX - minX;
    mObjSizeZ = maxZ - minZ;

    int sizeTex = mVTexcordX.size();
    float minTexX = 1, maxTexX = 0, minTexY = 1, maxTexY = 0; 
    for (int i = 0; i < sizeTex; i++){
	if (minTexX > mVTexcordX[i]) minTexX = mVTexcordX[i];
	if (maxTexX < mVTexcordX[i]) maxTexX = mVTexcordX[i];
	if (minTexY > mVTexcordY[i]) minTexY = mVTexcordY[i];
	if (maxTexY < mVTexcordY[i]) maxTexY = mVTexcordY[i];
    }
    mTexMinX = minTexX;
    mTexMaxX = maxTexX;
    mTexMinY = minTexY;
    mTexMaxY = maxTexY;
}



bool Distortion_cls::
IsDefault(){
    if (mRotDeg == 0 && mTiltDeg == 0 && mAngleDeg == 180)
	return TRUE;
    return FALSE;
}

void Distortion_cls::
UpdateTransformedTex(double rotDeg, double tiltDeg, double angleDeg){
    mRotDeg = rotDeg;
    mTiltDeg = tiltDeg;
    mAngleDeg = angleDeg;

    double rotRadian = rotDeg / 180 * 3.14159;
    double tiltRadian = tiltDeg / 180 * 3.14159;
    //mAngle = angleDeg;

    std::vector < float > VTexTiltX = mVTexcordX;
    std::vector < float > VTexTiltY = mVTexcordY;

    if (tiltDeg != 0 || angleDeg != 180){
	for (unsigned int i = 0; i < VTexTiltX.size(); i++){
	    float s = (mVTexcordX[i] - 0.5) * angleDeg / 180;
	    float t = (mVTexcordY[i] - 0.5) * angleDeg / 180;
	    float phi = atan2(t, s);
	    float theta = sqrt(s*s + t*t) / 0.5 * 3.14159 / 2;
	    float x = sin(theta) * cos(phi);
	    float y = sin(theta) * sin(phi);
	    float z = cos(theta);

	    float newX =  x;
	    float newY =  y * cos(tiltRadian) + z * sin(tiltRadian);
	    float newZ = -y * sin(tiltRadian) + z * cos(tiltRadian);

	    float alpha = atan2(newY, newX);
	    float Theta = acos(newZ);
	    
	    float S = Theta / 3.14159 * cos(alpha);
	    float T = Theta / 3.14159 * sin(alpha);

	    VTexTiltX[i] = S + 0.5;
	    VTexTiltY[i] = T + 0.5;
	}
    }
    
    for (unsigned int i = 0; i < mVTexcordX.size(); i++){
	float SX = 
	    (VTexTiltX[i] - 0.5) * cos(rotRadian) - 
	    (VTexTiltY[i] - 0.5) * sin(rotRadian) + 0.5;
	float SY = 
	    (VTexTiltX[i] - 0.5) * sin(rotRadian) +
	    (VTexTiltY[i] - 0.5) * cos(rotRadian) + 0.5;
	if (SX < 0) SX = 0;
	if (SX > 1) SX = 1;
	if (SY < 0) SY = 0;
	if (SY > 1) SY = 1;
	mVTexTransformedX[i] = SX;
	mVTexTransformedY[i] = SY;
    }
}

void Distortion_cls::
SetGLSetting(int width, int height){
    GLPerspective_cls CGLPerspective(width, height);

    PositionAndDirection_cls CPos(0, -10, 0);
    CPos.mTgt[X] = 0; CPos.mTgt[Y] = 1; CPos.mTgt[Z] = 0;
    CPos.mRgt[X] = 1; CPos.mRgt[Y] = 0; CPos.mRgt[Z] = 0;
    CPos.mTop[X] = 0; CPos.mTop[Y] = 0; CPos.mTop[Z] = 1;
    
    CGLPerspective.mbPerspective = FALSE;
    CGLPerspective.SetViewportOnGL();
    CGLPerspective.SetPerspectiveOnGL(&CPos);
    CGLPerspective.SetModelOrProjectionViewOnGL(&CPos, GL_MODELVIEW);
}

void Distortion_cls::
GLDrawDistortedTextureOrg(double aspectWindow, double aspectSrc,
			  int flipFactor){
    double scaleZ = 2 * 5.0 / mObjSizeZ;
    double scaleX = 2 * 5.0 * aspectWindow / mObjSizeX;

    glPushMatrix();

    glScalef(scaleX, scaleX, scaleZ);
    
    unsigned int indexSize = mVIndexVert.size();
    if (flipFactor == -1){
	glBegin(GL_QUADS);
	//glBegin(GL_LINES);
	for (unsigned int i = 0; i < indexSize; i++){
	    int indexCoord = mVIndexCoord[i] - 1;
	    glTexCoord2f(0.5 + (mVTexcordX[indexCoord] - 0.5) / aspectSrc,
			 1.0f - mVTexcordY[indexCoord]);

	    int indexVert = mVIndexVert[i] - 1;
	    glVertex3f(mVVertexX[indexVert], mVVertexY[indexVert],
		       -mVVertexZ[indexVert]);
	}
	glEnd();
    }
    else{
	glBegin(GL_QUADS);
	for (unsigned int i = 0; i < indexSize; i++){
	    int indexCoord = mVIndexCoord[i] - 1;
	    glTexCoord2f(0.5 + (mVTexcordX[indexCoord] - 0.5) / aspectSrc,
			 mVTexcordY[indexCoord]);
	    
	    int indexVert = mVIndexVert[i] - 1;
	    glVertex3f(mVVertexX[indexVert], mVVertexY[indexVert],
		       -mVVertexZ[indexVert]);
	}
	glEnd();
    }

    glPopMatrix();
}

void Distortion_cls::
GLDrawDistortedTexture(double aspectWindow, double aspectSrc, 
		       bool bUseTransform, int flipFactor){
    
    if (mbMeshOK == FALSE) return;
    
    if (bUseTransform == FALSE){
	GLDrawDistortedTextureOrg(aspectWindow, aspectSrc, flipFactor);
	return;
    }

    double scaleZ = 2 * 5.0 / mObjSizeZ;
    double scaleX = 2 * 5.0 * aspectWindow / mObjSizeX;

    glPushMatrix(); 

    glScalef(scaleX, scaleX, scaleZ);

    unsigned int indexSize = mVIndexVert.size();
    if (flipFactor == -1){
	glBegin(GL_QUADS);
	for (unsigned int i = 0; i < indexSize; i++){
	    int indexCoord = mVIndexCoord[i] - 1;
	    glTexCoord2f(0.5 + (mVTexTransformedX[indexCoord]- 0.5) / aspectSrc,
			 1.0f - mVTexTransformedY[indexCoord]);

	    int indexVert = mVIndexVert[i] - 1;
	    glVertex3f(mVVertexX[indexVert], mVVertexY[indexVert],
		       -mVVertexZ[indexVert]);
	}
	glEnd();
    }
    else{
	glBegin(GL_QUADS);
	for (unsigned int i = 0; i < indexSize; i++){
	    int indexCoord = mVIndexCoord[i] - 1;
	    glTexCoord2f(0.5 + (mVTexTransformedX[indexCoord]-0.5) / aspectSrc,
			 mVTexTransformedY[indexCoord]);
	    
	    int indexVert = mVIndexVert[i] - 1;
	    glVertex3f(mVVertexX[indexVert], mVVertexY[indexVert],
		       -mVVertexZ[indexVert]);
	}
	glEnd();
    }

    glPopMatrix();
    
}
