// -*- c -*-
// "clsDolly.h"

#include "clsDolly.h"
#include "clsMouseCtrl.h"
#include "clsMainGLCanvas.h"
#include "../MYGL/clsFBO.h"
#include "../PANELS/clsPreference.h"

#include "../COMMON/CommonDefinitions.h"

Dolly_cls::
Dolly_cls(Preference_cls *pCPreferenceLink){
    mpCPreferenceLink = pCPreferenceLink;
}

void Dolly_cls::
SetParameter(double sensitivity, MouseNormalized_cls *pCMP, 
	     wxSize sz, int displayMode){
    mSensitivity = sensitivity;
    mpCMP = pCMP;
    mMouseX = mpCMP->mCurrX;
    mMouseY = mpCMP->mCurrY;
    mAspectRatio = (double)sz.x / sz.y;
    mDisplayMode = displayMode;
}

bool Dolly_cls::
Gyration(PositionAndDirection_cls *pCCamPos){
    PositionAndDirection_cls CamPosBK = *pCCamPos;

    if (mpCPreferenceLink->mCbGyrationInvert.mbBody == TRUE) 
	mpCMP->Invert();

    pCCamPos->SetSensitivity(mSensitivity);
    if (mDisplayMode == _DIRECTION_FREE ||
	mDisplayMode >= _DOLLY_LINKED)
	pCCamPos->TrackBall(mpCMP->mPrevX, mpCMP->mPrevY, 
			    mpCMP->mCurrX, mpCMP->mCurrY);
    else{
	mSensitivity = mSensitivity * pCCamPos->GetProjectedDistanceToAnc();
	pCCamPos->SetSensitivity(mSensitivity);
	pCCamPos->SetAspectRatio(mAspectRatio);
	pCCamPos->Translate(mpCMP->GetDX(), mpCMP->GetDY());
    }

    return TRUE;
}

bool Dolly_cls::
Zoom(PositionAndDirection_cls *pCCamPos, float *pOrthoScale){
    if (mpCPreferenceLink->mCbZoomInvert.mbBody == TRUE) 
	mpCMP->Invert();
    
    if (mDisplayMode >= _DIRECTION_X_PLUS &&
	mDisplayMode <= _DIRECTION_Z_MINUS){
	*pOrthoScale *= (1.0 + mpCMP->GetDY() - mpCMP->GetDX() * 0.2);
    }
    else {
	pCCamPos->SetSensitivity(1);
	pCCamPos->Zoom(mpCMP->GetDX(), mpCMP->GetDY());
    }

    return TRUE;
}

bool Dolly_cls::
MoveCenter(PositionAndDirection_cls *pCCamPos,
	   Position_cls *pCAncPos){
    PositionAndDirection_cls CAncTmp(*pCAncPos, *pCCamPos);
    
    mSensitivity = 0.5 * mSensitivity * pCCamPos->GetProjectedDistanceToAnc();
    
    pCCamPos->SetSensitivity(mSensitivity);
    pCCamPos->SetAspectRatio(mAspectRatio);
    
    if (/*mpCLinkedPath == NULL*/TRUE)
	pCCamPos->Translate(mpCMP->GetDX(), mpCMP->GetDY());
    else mSensitivity *= -1;
    
    CAncTmp.SetAncPos(pCCamPos->mPos);
    CAncTmp.SetSensitivity(mSensitivity);
    CAncTmp.SetAspectRatio(mAspectRatio);
    CAncTmp.Translate(mpCMP->GetDX(), mpCMP->GetDY());
    pCAncPos->Set(CAncTmp);
    
    //*trigAnchor = TRUE;
    return TRUE;
}

bool Dolly_cls::
ZoomWheel(PositionAndDirection_cls *pCCamPos,
	  double wheelRotation, float *pOrthoScale){
    if (mpCPreferenceLink->mCbZoomInvert.mbBody == TRUE) 
	wheelRotation *= -1;

    if (mDisplayMode >= _DIRECTION_X_PLUS &&
	mDisplayMode <= _DIRECTION_Z_MINUS){
	*pOrthoScale *= (1.0 + wheelRotation * 0.1);
    }
    else{
	pCCamPos->SetSensitivity(1);
	
	pCCamPos->Zoom(0, wheelRotation * 0.1);
    }
    return TRUE;
}

