// -*- c -*-
// "clsMainGLCanvas.cpp"

#ifndef _MAIN_GL_CANVAS_H_
#define _MAIN_GL_CANVAS_H_

#define GL_GLEXT_PROTOTYPES 
#define _MANIP_TRANSLATE 0
#define _MANIP_SCALE 1
#define _MANIP_ROTATE 2

#include <wx/wx.h>
#include <wx/glcanvas.h>
#include <wx/colordlg.h>
#include <wx/stopwatch.h>

class DF3SeqCtrl_cls;
class DF3Sequence_cls;
class TreeBoxRule_cls;

class AnchorPath_cls;
class CameraPath_cls; 
class Canvas_cls; 
class DataExporter_cls;
class Distortion_cls;
class DrawingBlock_cls;
class DrawTimeIndex_cls; 
class DrawClrLegend_cls;
class DrawOption_cls;
class FilterCtrl_cls;
class FilterDataBoxData_cls;
class GLGuides_cls;
class GLTexture_cls;
class ImgSequence_cls;
class MainGLCanvas_cls; 
class MouseCtrl_cls;
class MoveItem_cls;
class MultiNumericalInputs_cls; 
class Navigators_cls;
class Overlay_cls;
class PDataSequence_cls;
class PeriodicBoundary_cls;
class PovRayCtrl_cls;
class Preference_cls; 
class SceneObj_cls;
class TimeCtrl_cls;

#include "../CONFITEM/clsConfItem.h"
#include "../GLCANVAS/clsStructDrawInfo.h"
#include "../LUA/clsLuaBase.h"
#include "../MYGL/clsCameraStructure.h"
#include "../MYGL/clsGLCameraProperty.h"
#include "../MYGL/clsPositionAndDirection.h"
#include "../MYGL/clsGLPerspective.h"
#include "../WIDGETCTRL/clsWidgetCommon.h"

#include "clsHitTest.h"
#include "NAVIGATORS/clsTagObject.h"

#include <vector>

class FrameMain_cls; 

class ColorCurve_cls;
class Dolly_cls;
class FBO_cls;
class FishEye_cls;
class GLStrings2D_cls;
class LightSRC_cls; 
class LightSRCCtrl_cls;
class Parameters_cls; 
class StereoGram_cls;
class GLCanvasCtrl_cls;
class MouseNormalized_cls;

#define _GL_CANVAS_DISPLAY_FREE    _DIRECTION_FREE
#define _GL_CANVAS_DISPLAY_X_PLUS  _DIRECTION_X_PLUS
#define _GL_CANVAS_DISPLAY_X_MINUS _DIRECTION_X_MINUS
#define _GL_CANVAS_DISPLAY_Y_PLUS  _DIRECTION_Y_PLUS
#define _GL_CANVAS_DISPLAY_Y_MINUS _DIRECTION_Y_MINUS
#define _GL_CANVAS_DISPLAY_Z_PLUS  _DIRECTION_Z_PLUS
#define _GL_CANVAS_DISPLAY_Z_MINUS _DIRECTION_Z_MINUS
#define _GL_CANVAS_LINKED_CAM0 7

#define _GL_CANVAS_GRID_BASIC 0
#define _GL_CANVAS_GRID_BASIC_CIRCULAR 1
#define _GL_CANVAS_GRID_POLAR 2

#define _GL_CANVAS_NOASSIN  0
#define _GL_CANVAS_GYRATION 1

#define _GYRATION_FREE 0
#define _GYRATION_FIXPOS 1

#define _PIVOT_FOLLOW_PATH 1

class MainGLCanvas_cls : 
public wxGLCanvas, public WidgetCommon_cls//, public LuaBase_cls
{
 public:
    MainGLCanvas_cls
	(GLCanvasCtrl_cls *pCParentCtrl, FrameMain_cls *pCParentFrame,
	 int* attribList = 0, long style = 0, 
	 const wxString& name = wxT("MainGLCanvas"),
	 const wxGLContext *pContextToShare = NULL,
	 unsigned int selfIndex = 0,
	 int displayMode = _GL_CANVAS_DISPLAY_FREE);

    MainGLCanvas_cls
	(GLCanvasCtrl_cls *pCParentCtrl, FrameMain_cls *pCParentFrame,
	 int* attribList = 0, long style = 0, 
	 const wxString& name = wxT("MainGLCanvas"),
	 unsigned int selfIndex = 0,
	 int displayMode = _GL_CANVAS_DISPLAY_FREE);

    void ConstructorMain
	(GLCanvasCtrl_cls *pCParentCtrl, FrameMain_cls *pCParentFrame,
	 unsigned int selfIndex, int displayMode, 
	 const wxGLContext *pContextToShare = NULL);
	
    ~MainGLCanvas_cls();
    void DeleteStaticTexture(GLTexture_cls **ppCTexture);
    
    GLCanvasCtrl_cls  *mpCGLCtrl;
    MouseCtrl_cls     *mpCMouseCtrl;
    MouseCtrl_cls     *mpCMouseInfoInnertia;
    Dolly_cls         *mpCDolly;
    MoveItem_cls      *mpCMoveItem;
    Navigators_cls    *mpCNavigators;
    GLStrings2D_cls   *mpCGLString2DDirect, *mpCGLString2DBuffer;
    static DrawTimeIndex_cls *mspCDrawTimeIndex;
    static DrawClrLegend_cls *mspCDrawClrLegend;
    static FishEye_cls       *mspCFishEyeCtrl;
    static Distortion_cls    *mspCDistortion;
    static GLTexture_cls     *mspCWhite;
    static GLTexture_cls     *mspCMask;
    static GLTexture_cls     *mspCFisheyeGrid;
    static GLTexture_cls     *mspCCheckerboard;
    
    FBO_cls           *mpCFBOForFishEyeBuffer;
    FBO_cls           *mpCFBOForShadow;
    FBO_cls           *mpCFBOForOverlay;

    std::vector < MainGLCanvas_cls* > mVPOtherCanvas;

    static bool      msbGLUTInitialized;
    wxString         mstrWarning;
    unsigned int     mSelfIndex;
    int              mPanelWidth, mPanelHeight;
    wxSize           mPanelSize;
    wxStopWatch      mCStopWatchSnap;
    double mDoubleIndexPrev;

    double mMatrixModelView[16];
    double mMatrixProjection[16];
    int    mMatrixViewport[4];

    TagObjectCtrl_cls *mpCTagObjectCtrl;
    
    double mDoubleIndex;
    int    mCurrentIndex;
    double mDoubleInterpStep;
    double mDT;
    int    mCounter;
    bool   mbInnertia;
    
    bool   mbHitTestItemSelect;
    bool   mbHitTestGuideSelect;

    bool   mbShowCoordinateTagXYZ[3];
    int    mLightNumber;
    static int msGLLightIndex[8];
    static HitTest_cls msCHittest;
    
 private:
    bool SetGLContext();
    void InitGL();
    void InitGLUT();
    void InitializeShader();
    void MakeFisheyeGridTexture();
    bool InitializeGLIfNot();
    //void UpdateWindowSizeInChildClass();
    
    // ----

 public:
    void SetCameraToDefaultPos();
    void UpdateDrawInfo(int width, int height);

    PovRayCtrl_cls           *GetPCPovRayCtrlLinked();
    PositionAndDirection_cls *GetPCCameraPosition();
    Position_cls             *GetPCAnchorPosition();
    PDataSequence_cls        *GetPCActiveFileSequence();
    TimeCtrl_cls* GetPCTimeCtrl();
    void UpdateCameraPos();
    CameraPath_cls* GetPLinkedCameraPath();
    AnchorPath_cls* GetPLinkedAnchorPath();
    LightSRCCtrl_cls* GetPLinkLightSourceCtrl();
    void UpdateAnc();
    Position_cls GetAncPos();

    CameraStructure_cls GetCCameraStructure(int stereo);
    
    void GetTimeInfo(double &time, double &dT);

    bool IsDrawingConditionSatisfied();
    bool IsStereo();
    bool IsStereoDualOutput();
    bool IsLineByLineFishEye();
    bool IsOrthographic();
    bool IsAlongAxis();
    bool IsFishEye();
    bool IsCylinder();
    void WaitPrepaing();
    int  GetFBOMax();
    int  GetMultiPanelMode();
    //bool IsFBOUse();
    void ToggleOrthographic();
    void SetAngleIfLinked();
    void ModifyByDomeTilt(PositionAndDirection_cls *pCCam);
    double* GetAnchorPosForRender();
    void UpdateViewInformation();
    //void CalcTagCoordinate();
    //void DrawTags();

    void SetPivotToItem(int type);
    void SetKeyPos();
    void SetKeyPosRot();
    void SetKeyAnc();
    void SetViewport();
    void SetPerspective(PositionAndDirection_cls *pCCam);
    void SetModelView(PositionAndDirection_cls *pCCam,
		      bool bUpdateMembers = TRUE);
    void SetLineWidth(FBO_cls *pCFBO = NULL);

    void ClearBuffer(double doubleIndex);
    void ClearCanvas();
    
    void SetGLLightSource(double doubleIndex);
    void DrawDataCopyLoop
	(FilterDataBoxData_cls *pCScene, int currentIndex, int fboIndex, int lightNumber,
	 float pLightPosForSV[4], float pLightPhiTheta[2], bool bShadowVolume);
    void DrawRealObjectCopyLoop
	(FilterDataBoxData_cls *pCScene, int currentIndex, 
	 double doubleIndex, int lightNumber, int drawStep);
    void DrawLightSource(FilterDataBoxData_cls *pCScene, int drawStep);
    void DrawObjectAndData
	(FilterDataBoxData_cls *pCScene,
	 int currentIndex, double doubleIndex, int lightNumber,
	 int fboIndex, float pLightPosForSV[4], float pLightPhiTheta[2]);
    void DrawDataLitDiffuse(double doubleIndex, double doubleRemapped,
			    int fboIndex, int lightIndex);
    void DrawDataMain(double doubleIndex, double doubleRemapped,
		      int fboIndex, int shadowNumber);
    void DrawGrid(double scale, double scale10);
    void DrawCameraPath(double doubleIndex, double scale, bool bHittest = FALSE);
    void DrawLightSource(double doubleIndex, double scale, bool bHittest = FALSE);
    void DrawFilters(double doubleIndex, double scale, bool bHittest = FALSE);
    void DrawAnchor(double scale);
    bool DrawWarning(wxPaintDC& dc);
    void CheckWarnigForDirectDraw();
    void DrawMessage();
    void DrawModeAxis(double scale);
    void DrawSelectedItem(double scale);
    
    void RefreshEX();
    bool IsMouseOnClient();
    
    void ShowPreviousFramePBO();
    void DrawOtherObjects(double doubleIndex, double scale, double scale10);
    void RefreshOtherOrthoCanvas();
    bool CheckGLRefresh(int frame, int interpStep);
    void DrawOnGL(int stereo, double doubleIndex, double doubleRemapped);
    int  GetFBOBufferSize(int type);
    void DrawOnFBO(int stereo, double doubleIndex, double doubleRemapped,
		   int bufferSize);
    void GatherFBOMultiPanel(int width, int height, int stereo);
    void GatherFBOImages(int width, int height, int stereo);
    void BakeResultToFBOForImage(int stereo);
    void FBODraw(double doubleIndex, double doubleRemapped, int stereo);
    void DirectDraw(double doubleIndex, double doubleRemapped);
    void PrepareBaseObjectSettings();
    void GLDrawPrepareParticleSnap();
    bool GLDrawMain();
    void DrawOverLay();
    void ProcessImageSave();
    void DrawNavigatorAndButtons();
    void PrepareDrawOrPovOut();
    static void PrepareDrawOrPovOutStatic();
    static void EndDrawOrPovOut();
    void PaintDirectly(wxPaintDC &dc);
    void OnPaint(wxPaintEvent& event);
    void FinishPaint(wxPaintEvent &event);
    void ProcReadImage(int fbo, FBO_cls *pCFBOOverlay, int stereo);
    void ProcReadImageSequence(int fbo, FBO_cls *pCFBOOverlay, int stereo);
    void StartSaveImgSeqProcess(ImgSequence_cls* pCImgSeq);
    void StopSaveImgSeqProcess();
    void StartExportDataProcess(DataExporter_cls* pCDataExporter);
    void StopExportDataProcess();
    void StartExportPovStringProcess();
    void StopExportPovStringProcess();

    bool (*mApFuncMove[3])(double);

    void OpenExpressionDialog();
    void ShowMenu(int x, int y);
    
    // $B%^%&%9A`:n4X78(B
    bool SetThisToBeActiveGL();
    bool SetOtherCanvasToBeActiveGL();
    bool ProcViewOrItemDragging(bool bOnInnertia = FALSE);
    void ProcItemDraggingEnd();
    bool ProcCaptureMouse();
    void StartHitTest();
    
    void OnMouse(wxMouseEvent& event);
    void OnMouseMotion(wxMouseEvent& event);
    void OnKeyDown(wxKeyEvent& event);

    static void InitializeStaticMembers(Overlay_cls *pCOverlayLink);
    static void DestructStaticMembers();
    static bool IsGLBlendEquationEXOK();

 public:
    static FilterCtrl_cls* mspCFilterCtrlBooleanLink;
    static FilterCtrl_cls* mspCFilterCtrlObjectLink;
    static FilterCtrl_cls* mspCFilterCtrlWorldLink;
    static DF3SeqCtrl_cls* mspCDF3SeqCtrl;
    
    static PeriodicBoundary_cls* mspCPeriodicLink;
    static StereoGram_cls* mspCStereogramLink;
    
    void TogglePerspective();
    void SetDisplayMode(int displayMode, bool bUpdateCombobox = FALSE);
    void LinkageConnect(/*int pathIndex*/);
    void OnPanelConfigure(wxCommandEvent &event);
    void OnMenuPers(wxCommandEvent& event);
    void OnMenuGridAnti(wxCommandEvent& event);
    void OnMenuOtherCam(wxCommandEvent& event);
    void OnMenuMoveToCamPos(wxCommandEvent& event);
    void OnMenuSetKeyPos   (wxCommandEvent& event);
    void OnMenuSetKeyPosRot(wxCommandEvent& event);
    void OnMenuSetKeyAnc   (wxCommandEvent& event);
    void OnMenuToggleLink  (wxCommandEvent& event);
    void OnMenuToggleLink2 (wxCommandEvent& event);
    void OnMenuTogglePerspective(wxCommandEvent& event);
    void OnMenuCenterItem(wxCommandEvent& event);
    void OnMenuCenterOrigin(wxCommandEvent& event);

    void CopySettingToAll();
    
    void SetGLBGColor(int index);
   
    static void SelectItem(Parameters_cls *pCParam);
    //static void HookGridComboBox(void* pArg, void *pEvent);

    void   ModifyOrthoCamera();
    double CalcAutoScale();

    FrameMain_cls* mpCParentFrameLink;
    //DrawOption_cls* mpCDrawOptionLink;
    
    wxMenuItem *mpMenuItemShowPanel;

    bool mbInitialized;
    unsigned int mDrawingFrame;
    unsigned int mDrawingStep;
    double       mDrawingBlurFactor;
    unsigned int mPrevFrame;
    unsigned int mPrevStep;
    double       mPrevBlurFactor;
    
 public:
    std::vector < int > mVHitObj; //
    
    int  mIndexForDebug;

    static ColorCurve_cls *mspCClrCurveBG, *mspCClrCurveOut;
    
 public:
    GLCameraProperty_cls  mCGLCameraProperty;

    ConfItemList_cls      mCConfList;

    ConfItemBOOL_cls      mCbShowMessage, mCbShowItems;
    ConfItemBOOL_cls      mCbShowGrid, mCbShowData, mCbShowNavigator;
    ConfItemBOOL_cls      mCbPivotMode;
    
    //ConfItemSTR_cls   mCstrGLDepthExpression;
    //ConfItemFLOAT_cls mCGLDepthMin, mCGLDepthMax;
    //ConfItemFLOAT_cls mCGLPersAngle;
    //ConfItemFLOAT_cls mCGLOrthoScale;
    
    ConfItemINT_cls       mCDisplayMode, mCGLGeometry;
    ConfItemINT_cls       mCPathIndex;
    ConfItemUINT_cls      mCGridType;
    ConfItemBOOL_cls      mCbShowOtherCam;
    ConfItemBOOL_cls      mCbXOR;
    ConfItemBOOL_cls      mCbGridAnti;
    ConfItemBOOL_cls      mCbShowGridTag;
    ConfItemBOOL_cls      mCbDistortion;
    ConfItemBOOL_cls      mCbFisheyeGrid;
    
    bool mbSaveImageQueue;
    bool mbSaveImgSeqQueue;
    bool mbExportDataQueue;
    bool mbExportPovStringQueue;
    bool mbRefreshQueue;

 public:
    wxGLContext*              mpCGLContext;

    //RedrawPBO_cls             mCRedrawPBO[4];
    StructDrawInfo            mCDrawInfo, mCDrawInfoPrev;
    
    //GLPerspective_cls         mCGLPerspBake;
    GLPerspective_cls         mCGLPerspDraw;
    static GLGuides_cls *mspCGLGuide, *mspCGLGuideYPR, *mspCGLGuideScale;
    PositionAndDirection_cls  mCCameraPos;

    PositionAndDirection_cls  mCCameraPosFree, mCCameraPosCamera;
    PositionAndDirection_cls  mCCameraPosOrtho;
    
    PositionAndDirection_cls  mCDataParentCoordinate;
    
    static Position_cls       msCAnchorPos, msCAnchorPosOrtho; 
    static PositionAndDirection_cls msCAnchorPosWithDummyDirection;

    double                    mdPosTmp[3];
    
    static int                msAFBOSize[7];

    static ConfItemList_cls  msCConfListStatic;
    static ConfItemUINT_cls  msCFBOSizeIndex;
    static ConfItemUINT_cls  msCFBOSizeFishEye;
    static ConfItemFLOAT_cls msCLineWidth;
    static ConfItemUINT_cls  msCRenderType;
    
    // -- $B%j%s%/(B --
    CameraPath_cls*           mpCCameraPathLinked;
    AnchorPath_cls*           mpCAnchorPathLinked;
    static FrameMain_cls*     mspCParentFrameLink;
    static ImgSequence_cls*   mspCImgSeqLinked;
    static DrawOption_cls*    mspCDrawOptionLink;
    DataExporter_cls*         mpCDataExporterLinked;
    // ----

    void ReadStrLine(std::wistream &istream, wxString strLine);
    static void ReadStrLineStaticMember(std::wistream &istream, wxString strLine);
    wxString Serialize();
    static wxString SerializeStaticMember();

    DECLARE_EVENT_TABLE()
};

#endif //_MAIN_GL_CANVAS_H_
