// -*- c -*-
// "clsMoveItem.h"

#include "clsMoveItem.h"
#include "clsMouseCtrl.h"
#include "clsMainGLCanvas.h"
#include "../MYGL/clsFBO.h"
#include "../MYGL/clsGLGuides.h"
#include "../PANELS/clsPreference.h"
#include "../SCENEOBJ/clsSceneObj.h"

#include "../COMMON/CommonDefinitions.h"

MoveItem_cls::
MoveItem_cls(Preference_cls *pCPreferenceLink,
    	     GLGuides_cls     *pCGLGuideTranslateLink,
	     GLGuides_cls     *pCGLGuideScaleLink,
	     GLGuides_cls     *pCGLGuideYPRLink){
    mpCPreferenceLink = pCPreferenceLink;
    mpCGLGuideTranslateLink = pCGLGuideTranslateLink;
    mpCGLGuideScaleLink = pCGLGuideScaleLink;
    mpCGLGuideYPRLink = pCGLGuideYPRLink;
    mbMovingItemNow = FALSE;
}

void MoveItem_cls::
SetSelection(SceneObj_cls *pCObj){
    mspCSelectedObject = pCObj;

    if (pCObj == NULL) return;
    
    LOOP_XYZ msSelectedItemPos[xyz] = 
	pCObj->mpCTranslate->mVCParam[xyz].GetInterpolatedValue();
    LOOP_XYZ msSelectedItemPosOrg[xyz] = msSelectedItemPos[xyz];
    LOOP_XYZ msSelectedItemYPROrg[xyz] = 
	pCObj->mpCRotation->mVCParam[xyz].GetInterpolatedValue();
    LOOP_XYZ msSelectedItemScaleOrg[xyz] = 
	pCObj->mpCScale->mVCParam[xyz].GetInterpolatedValue();
    LOOP_XYZ msSelectedItemScale[xyz] = 1;
    LOOP_XYZ msSelectedItemDRot[xyz] = 0;

    gluProject(msSelectedItemPosOrg[X], 
	       msSelectedItemPosOrg[Y], 
	       msSelectedItemPosOrg[Z],
	       mMatrixModelView, mMatrixProjection, mMatrixViewport,
	       &mPosItemOrgOnW[0], &mPosItemOrgOnW[1], &mPosItemOrgOnW[2]);
}

void MoveItem_cls::
GetGLMatrixInfo(){
    glGetDoublev(GL_MODELVIEW_MATRIX,  mMatrixModelView);
    glGetDoublev(GL_PROJECTION_MATRIX, mMatrixProjection);
    glGetIntegerv(GL_VIEWPORT, mMatrixViewport);
}

void MoveItem_cls::
SetParameter(double sensitivity, MouseNormalized_cls *pCMP, 
	     wxSize sz, int displayMode){
    mSensitivity = sensitivity;
    mpCMP = pCMP;
    mMouseX = mpCMP->mCurrX;
    mMouseY = mpCMP->mCurrY;
    mAspectRatio = (double)sz.x / sz.y;
    mDisplayMode = displayMode;
}

void MoveItem_cls::
MoveItemTranslate(PositionAndDirection_cls *pCCamPos,
		  double aspectRatio, double dragX, double dragY,
		  int dragMode){
    Position_cls CPosTmp(msSelectedItemPos[0],
			 msSelectedItemPos[1],
			 msSelectedItemPos[2], 0, 0, 0);
    PositionAndDirection_cls CPos(CPosTmp, *pCCamPos);
    PositionAndDirection_cls CParentCoord;
    
    LOOP_XYZ CPos.mATranslateScale[xyz] = 1;
    
    //double sensitivity = -0.5 * pCCamPos->GetProjectedDistanceToAnc();
    double sensitivity = -0.3*pCCamPos->GetProjectedDistance(msSelectedItemPos);
    CPos.SetAncPos(pCCamPos->mPos);
    CPos.SetSensitivity(sensitivity);
    CPos.SetAspectRatio(aspectRatio);
    CPos.Translate(dragX, dragY, 
		   _POS_AND_DIR_MOVE_FREE, &CParentCoord);
    LOOP_XYZ msSelectedItemPos[xyz] = CPos.mPos[xyz];
    
    if (dragMode == X){
	msSelectedItemPos[Y] = msSelectedItemPosOrg[Y];
	msSelectedItemPos[Z] = msSelectedItemPosOrg[Z];
    }
    if (dragMode == Y){
	msSelectedItemPos[Z] = msSelectedItemPosOrg[Z];
	msSelectedItemPos[X] = msSelectedItemPosOrg[X];
    }
    if (dragMode == Z){
	msSelectedItemPos[X] = msSelectedItemPosOrg[X];
	msSelectedItemPos[Y] = msSelectedItemPosOrg[Y];
    }
}

void MoveItem_cls::
MoveItemRotate(double dragRot, int dragMode){
    if (dragMode >= 0)
	msSelectedItemDRot[dragMode] += dragRot * 75;
}

void MoveItem_cls::
MoveItemScale(double dragScale, int dragMode){
    if (dragMode == -1)
	LOOP_XYZ msSelectedItemScale[xyz] = dragScale;
    else
	msSelectedItemScale[dragMode] = dragScale;
}

void MoveItem_cls::
DrawMouseGuide(int moveMode, int dragMode, int panelWidth, int panelHeight,
	       int bufferWidth, int bufferHeight){
    
    GLfloat matrixB[16];
    glGetFloatv(GL_TRANSPOSE_MODELVIEW_MATRIX, matrixB);
    double scale = -matrixB[11];

    PositionAndDirection_cls CPosOrigin;
    PositionAndDirection_cls CPos(msSelectedItemPos[X],
				  msSelectedItemPos[Y],
				  msSelectedItemPos[Z]);

    if (moveMode == _DOLLY_TRANSLATE) {
	mpCGLGuideTranslateLink->DrawGuide(CPos, scale, CPosOrigin, dragMode);
	return;
    }

    double YPR[3] = {
	msSelectedItemYPROrg[X] + msSelectedItemDRot[X],
	msSelectedItemYPROrg[Y] + msSelectedItemDRot[Y],
	msSelectedItemYPROrg[Z] + msSelectedItemDRot[Z]};
    
    if (moveMode == _DOLLY_ROTATE){
	if (dragMode == -1) dragMode = _ROTATE_MOVE_Y;
	mpCGLGuideYPRLink->DrawGuide(CPos, scale, CPosOrigin, dragMode, YPR);
    }
    
    if (moveMode == _DOLLY_SCALE)
	mpCGLGuideScaleLink->DrawGuide(CPos, scale, CPosOrigin, dragMode, YPR);
    
    if (mbMovingItemNow == FALSE) return;


    switch(dragMode){
    case -1:
	glColor4d(0.8, 0.8, 0.8, 1); break;
    case _ROTATE_MOVE_Y:
	glColor4d(0.2, 0.2, 1, 1); break;
    case _ROTATE_MOVE_P:
	glColor4d(0.2, 1, 0.2, 1); break;
    case _ROTATE_MOVE_R:
	glColor4d(1, 0.3, 0.3, 1); break;
    }
    
    glEnable(GL_LINE_STIPPLE);
    glLineStipple(1,0xf0f0);
    FBO_cls::Push2DMatrix(bufferWidth, bufferHeight);
    
    int orgX, orgY;
    orgX = mPosItemOrgOnW[X];
    orgY = mPosItemOrgOnW[Y];
    int curX = (mMouseX/2+0.5) * bufferWidth;
    int curY = (mMouseY/2/mAspectRatio + 0.5) * bufferHeight;
    glBegin(GL_LINES); 
    glVertex2d(orgX, orgY);
    glVertex2d(curX, curY);
    glEnd();
    FBO_cls::Pop2DMatrix();
    glDisable(GL_LINE_STIPPLE);
}

void MoveItem_cls::
InitializeMoveItem(){
    if (mspCSelectedObject == NULL) return;
    
    SetSelection(mspCSelectedObject);
}

bool MoveItem_cls::
MoveItem(PositionAndDirection_cls *pCCamPos, wxSize sz,
	 int moveMode, int dragMode, bool bInv){
    
    if (mspCSelectedObject == NULL) return FALSE;
    //int dragMode = mspCSelectedObject->mDragMode;
    
    if (mbMovingItemNow == FALSE){
	InitializeMoveItem();
	mbMovingItemNow = TRUE;
    }

    //GetGLMatrixInfo();

    /*
    gluProject(msSelectedItemPosOrg[X], 
	       msSelectedItemPosOrg[Y], 
	       msSelectedItemPosOrg[Z],
	       mMatrixModelView, mMatrixProjection, mMatrixViewport,
	       &mPosItemOrgOnW[0], &mPosItemOrgOnW[1], &mPosItemOrgOnW[2]);
    */

    gluProject(msSelectedItemPos[X], 
	       msSelectedItemPos[Y], 
	       msSelectedItemPos[Z],
	       mMatrixModelView, mMatrixProjection, mMatrixViewport,
	       &mPosItemOnW[0], &mPosItemOnW[1], &mPosItemOnW[2]);
    
    /*
    double normalPosX = (2*mPosItemOnW[X] - sz.x)/sz.x;
    double normalPosY = (2*mPosItemOnW[Y] - sz.y)/sz.y;
    */

    double normalPosX = 2*mPosItemOnW[X] / mMatrixViewport[2] - 1;
    double normalPosY = (2*mPosItemOnW[Y] / mMatrixViewport[3] - 1) * mAspectRatio;

    double dragX = mpCMP->mCurrX - mpCMP->mPrevX;
    double dragY = mpCMP->mCurrY - mpCMP->mPrevY;
    double relX  = mpCMP->mCurrX - normalPosX;
    double relY  = mpCMP->mCurrY - normalPosY;
    double relDownX = mpCMP->mDownX - normalPosX;
    double relDownY = mpCMP->mDownY - normalPosY;
    
    switch (moveMode){
    case _DOLLY_TRANSLATE: 
	MoveItemTranslate(pCCamPos, mAspectRatio, dragX, dragY, dragMode);
	break; 
    case _DOLLY_ROTATE: {
	double dragRot = (dragX * relY - dragY * relX) / (relX * relX + relY * relY);
	if (bInv == TRUE) dragRot *= -1;
	if (dragMode == -1) dragMode = _ROTATE_MOVE_Y;
	MoveItemRotate(dragRot, dragMode);
	break; }
    case _DOLLY_SCALE: {
	double dragScale = sqrt(relX * relX + relY * relY) / sqrt(relDownX * relDownX + relDownY * relDownY);
	MoveItemScale(dragScale, dragMode); 
	break; }
    }

    return TRUE;
}

void MoveItem_cls::
FinishDragging(){
    LOOP_XYZ mspCSelectedObject->mpCTranslate->mVCParam[xyz].mInterpolatedValue 
	= msSelectedItemPos[xyz];
    LOOP_XYZ mspCSelectedObject->mpCRotation->mVCParam[xyz].mInterpolatedValue 
	+= msSelectedItemDRot[xyz];
    LOOP_XYZ mspCSelectedObject->mpCScale->mVCParam[xyz].mInterpolatedValue
	*= msSelectedItemScale[xyz];
    LOOP_XYZ msSelectedItemDRot[xyz] = 0;
    mbMovingItemNow = FALSE;
}

SceneObj_cls *MoveItem_cls::mspCSelectedObject = NULL;
double MoveItem_cls::msSelectedItemPos[3] = {0, 0 ,0};
double MoveItem_cls::msSelectedItemPosOrg[3] = {0, 0 ,0};
double MoveItem_cls::msSelectedItemScale[3] = {1, 1 ,1};
double MoveItem_cls::msSelectedItemDRot[3] = {0, 0, 0};
double MoveItem_cls::msSelectedItemYPROrg[3] = {0, 0, 0};
double MoveItem_cls::msSelectedItemScaleOrg[3] = {0, 0, 0};

double MoveItem_cls::mMatrixModelView[16];
double MoveItem_cls::mMatrixProjection[16];
int    MoveItem_cls::mMatrixViewport[4];

double MoveItem_cls::mPosItemOnW[3];
double MoveItem_cls::mPosItemOrgOnW[3];
