// -*- c++ -*- 
// (setq c-basic-offset 4)
// "clsStereoGram.h"

#include "clsStereoGram.h"

#include "../GLCANVAS/clsMainGLCanvas.h"
#include "../GLOBJECTS/clsCamera.h"
#include "../MYAPI/MsgBox.h"
#include "../MYGL/clsFBO.h"
#include "../PARAMETER/clsParamGroup.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/EX/clsButtonEX.h"
#include "../WIDGETCTRL/WX/myArtProv.h"

#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"
#include "../../clsMain.h"
#include "../TIMELINE/clsTimeCtrl.h"

#include "../COMMON/CommonDefinitions.h"

#ifndef M_PI
#define M_PI 3.14159
#endif

// ÓIɃCxgт
BEGIN_EVENT_TABLE(StereoGramGUI_cls, wxDialog)
     EVT_BUTTON(btnStereoDefault, StereoGramGUI_cls::OnButtonDefault)
     EVT_BUTTON(btnStereoClose,   StereoGramGUI_cls::OnButtonClose)
END_EVENT_TABLE()
     
StereoGramGUI_cls::
    StereoGramGUI_cls(StereoGram_cls* pCParent, wxWindow *pC) :
    wxDialogEXKey(pC, wxID_ANY, _("Stereogram Configure"),
		  wxDPS, ARTWXFRAMEICON("wxART_STEREOGRAM"))
{
    mpCParent = pCParent;
    SetCloseButtonID(btnStereoClose);
    //SetScreenRectPos(&pCParent->mCScreenRect);
    SetPCPosInfoAtClose(&pCParent->mCDialogPos);

    // TCU[
    wxBoxSizer       *pBoxSizerSRC   = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer       *pBoxSizerTOP   = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer       *pBoxSizerParamL = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer       *pBoxSizerBTM    = new wxBoxSizer(wxHORIZONTAL);

    SetMNumMainParam(pBoxSizerParamL);

    wxButtonEX *pCButtonDEF   = new wxButtonEX (this, btnStereoDefault, 
						_("Set to Defalt"));
    wxButtonEX *pCButtonClose = new wxButtonEX (this, btnStereoClose,
						wxT("Close"));
    
    pBoxSizerBTM->Add( pCButtonDEF,    0, wxEXPAND);
    pBoxSizerBTM->Add( -1, -1, 1, wxEXPAND);
    pBoxSizerBTM->Add( pCButtonClose,  0, wxEXPAND);

    pBoxSizerTOP->Add (pBoxSizerParamL, 0, wxEXPAND);
    pBoxSizerSRC->Add (pBoxSizerTOP, 0, wxEXPAND);
    pBoxSizerSRC->Add (pBoxSizerBTM, 0, wxEXPAND);

    SetSizerAndFit(pBoxSizerSRC);

    pCButtonClose->SetFocus();
}

StereoGramGUI_cls::
~StereoGramGUI_cls(){
}

void StereoGramGUI_cls::
SetMNumMainParam(wxBoxSizer *pBoxSizer){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputPanel, this);
    pCurr->mUpdateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_STEREO;
    
    pCurr->mTextWidth = 140;
    pCurr->mItemWidth = -1; // 120
    wxString strListType[3] = {
	_("0: Monogram"), wxT("1: Off-Axis"),wxT("2: Toe-In")};
    pCurr->CombinePointerList(&mpCParent->mCType.mBody,
			      3, strListType, _("Camera Setting"));

    wxString strListEye[8] = {
	_("No Parallax"), _("LEFT"), _("RIGHT"), _("LINE BY LINE"), _("RED-CYAN"), _("YELLOW-BLUE"), _("SIDE BY SIDE"), _("QUAD BUFFERING") };
    mIndexEye = pCurr->CombinePointerList(&mpCParent->mCEye.mBody,
					  8, strListEye, _("Stereogram Type"));

    wxString strListDual[2] = {
	_("Combined"), _("Separately")};
    mIndexDualOutput = pCurr->CombinePointerList
	(&mpCParent->mCDualOutput.mBody,
	 2, strListDual, _("File Output"));
    
    mIndexToeIn = pCurr->CombinePointer
	(_ITEM_DOUBLE, &mpCParent->mCToeInAngle.mBody, _("Toe-In Angle"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 45);

    int boxWidth = 2;
    mIndexDistance = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("parallax/focus"));
    
    /*mIndexDistance = */pCurr->CombinePointer
	(_ITEM_DOUBLE, &mpCParent->mpCStereoParam->mVCParam[0].mInterpolatedValue/*, _("Parallax Distance")*/);
    mIndexDistanceFocus = pCurr->CombinePointer
	(_ITEM_DOUBLE, &mpCParent->mDistanceToFocus, wxEmptyString/*_("Distance to Focus")*/, FALSE);
    
    mIndexFlip = pCurr->CombinePointer
	(_ITEM_BOOL, &mpCParent->mCbFlip.mbBody, _("Flip Stereo"));
    
#ifdef WIN32
    pCurr->mMinimumWidth = 240;
#else
    pCurr->mMinimumWidth = 300;
#endif
    pCurr->FinishSetting(pBoxSizer);

    pCurr->SetHook(this, &HookItem_cls::HookFuncAssign, _HOOK_ASSIGN);
    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_COMBOBOX);
}

void StereoGramGUI_cls::
HideAndShowItems(){
    
    MultiInputPanel_cls *pCurr = mpCMInputPanel;

    bool bEye = FALSE, bToeIn = FALSE, bDistance = FALSE;
    bool bDual = FALSE, bMultiple = FALSE;
    bool bFlip = FALSE, bAlways = FALSE;
    
    if (mpCParent->mCType.mBody != _STEREO_NONE){
	bEye = TRUE;
	if (mpCParent->mCEye.mBody != _STEREO_NONE){
	    bDistance = TRUE;
	    bAlways = TRUE;
	    if (mpCParent->mCType.mBody == _STEREO_TOEIN) bToeIn = TRUE;

	    //if (mpCParent->mCEye.mBody == _STEREO_MULTIPLE) bMultiple = TRUE;
	    
	    if (mpCParent->mCEye.mBody >= _STEREO_SCANLINE) bDual = TRUE;
	    if (mpCParent->GetStereoNum() == 2) bFlip = TRUE;
	    
	}
    }

    pCurr->ShowItem(mIndexEye, bEye);
    pCurr->ShowItem(mIndexToeIn, bToeIn);
    pCurr->ShowItem(mIndexDistance, bDistance);
    pCurr->ShowItem(mIndexDualOutput, bDual);    
    pCurr->ShowItem(mIndexFlip, bFlip);
    //pCurr->ShowItem(mIndexMultipleNumber, bMultiple);
    //pCurr->ShowItem(mIndexMultipleCurrent, bMultiple);
    //pCurr->ShowItem(mIndexAlways, bAlways);

    ResizeDialog();
}

void StereoGramGUI_cls::
OnButtonDefault(wxCommandEvent &event){
    if (MsgBoxYesno(this, _("Query"), _("Set to default ?")) == FALSE) return;
    mpCParent->SetToDefault();
    mpCParent->UpdateItems(_MN_NOFORCE);
    HideAndShowItems();
}

void StereoGramGUI_cls::
OnButtonClose(wxCommandEvent &event){
    mpCParent->UpdateItems(_MN_NOFORCE);
    CloseEXKey();
}

void StereoGramGUI_cls::
OnComboBox(){
    
    if (mpCParent->mCEye.mBody == _STEREO_SCANLINE &&
	mpCParent->mCDualOutput.mBody == 0){
	MsgBox(mpCParent->mpCParentFrame, _("Saving LINE BY LINE images are not supported."), _("Warning"));
	GCUndoList.UndoCurrent();
    }

    mpCParent->UpdateItems();
    mpCParent->UpdateTLShown();

    HideAndShowItems();
    
    mpCParent->SendMenuEvent(mpCParent->mpCParentFrame, menuEditRefreshGL);
}

void StereoGramGUI_cls::
HookFunc01(int id, void *pArg){
    OnComboBox();
}

void StereoGramGUI_cls::
HookFuncAssign(int id, void *pArg){
    mpCParent->mpCStereoParam->SetKeyFrameIfChanged();
    mpCParent->UpdateItems();
}

// ---------------------

StereoGram_cls::
StereoGram_cls() 
{
    mpCGUI  = NULL;
    mbQuadStereo = FALSE;
    SetToDefault();

    mFlipList[0][0] = 0;
    mFlipList[1][0] = 1;
    mFlipList[0][1] = 1;
    mFlipList[1][1] = 0;

    wxString listTag[1] = {wxT("Distance of Eye")};
    int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_STEREO ;
    mpCStereoParam = new ParamGroup_cls(wxT("Stereogram"), 1, listTag, updateBit, TRUE);
    double pStereoInit[1] = {0.1};
    mpCStereoParam->SetKeyFrame(pStereoInit, TRUE);

    for (int i = 0; i < 2; i++) mpCFBO[i] = NULL;

    //InitGUI(pC); // GUI ̃CX^X
}

StereoGram_cls::
~StereoGram_cls(){
    delete mpCStereoParam;
    for (int i = 0; i < 2; i++)
	if (mpCFBO[i] != NULL) delete mpCFBO[i];
}

void StereoGram_cls::
UpdateItems(bool bForce){
    mDistanceToFocus = GetDistanceToFocus
	(mpCParentFrame->mpCParent->mpCTime->GetDoubleIndex());
    if (mpCGUI != NULL)
	mpCGUI->mpCMInputPanel->UpdateValues(_MN_ALLITEM);
}

void StereoGram_cls::
SetToDefault(){
    mCConfList.Clear();
    mCConfList.Add(&mCType, L"StereogramType=", L"0");
    mCConfList.Add(&mCEye,  L"StereogramEye=", L"0");
    mCConfList.Add(&mCToeInAngle, L"StereoGramToeInAngle=", L"0.5");
    //mCConfList.Add(&mCPitchAngle, L"StereoGramPitchAngle=", L"0");
    //mCConfList.Add(&mCDome, L"DomeDype=", L"0");
    mCConfList.Add(&mCDualOutput, L"SterogramDualOutput=", L"1");
    mCConfList.Add(&mCbFlip, L"StereoGramFlip=", L"0");
}

void StereoGram_cls::
InitGUI(FrameMain_cls *pCFrameMainLink){
    mpCParentFrame = pCFrameMainLink;
    mpCGUI = new StereoGramGUI_cls(this, pCFrameMainLink);
    SetPDialog(mpCGUI);
}

void StereoGram_cls::
UpdateTLShown(){
    mpCStereoParam->mbShownOnTL = (mCType.mBody != 0);
    mpCStereoParam->HideAndShowOnTL();
}

void StereoGram_cls::
SetInfo(int panelWidth, int panelHeight, bool bCameraStatus){
    mPanelWidth = panelWidth;
    mPanelHeight = panelHeight;
    mbCameraStatus = bCameraStatus;
}

int StereoGram_cls::
GetStereoNum(){
    //if (mbCameraStatus == FALSE && mCbApplyAlways.mbBody == FALSE) return 1;
    if (mCType.mBody == _STEREO_NONE) return 1;
    switch(mCEye.mBody){
    case _STEREO_NONE: return 1;
    case _STEREO_LFT: case _STEREO_RGT: return 1;
    case _STEREO_SCANLINE: case _STEREO_SIDEBYSIDE: return 2;
    case _STEREO_RED_CYAN: case _STEREO_YELLOW_BLUE: return 2;
    case _STEREO_QUADBUFFER: return 2;
	//case _STEREO_MULTIPLE: return 1;
    }
    return 1;
}

float StereoGram_cls::
GetStereoFocus(){
    return mDistanceToFocus;
}

void StereoGram_cls::
ClearFBO(){
    for (int i = 0; i < 2; i++)
	if (mpCFBO[i] != NULL) mpCFBO[i]->ClearFBO();
}

void StereoGram_cls::
PrepareStereogram(int stereo){
    if (mpCFBO[stereo] == NULL)
	mpCFBO[stereo] = new FBO_cls(mBufferSize, mBufferSize);
    else
	mpCFBO[stereo]->SetGeometryBuffer(mBufferSize, mBufferSize);
}

void StereoGram_cls::
DrawOnFBOBegin(int stereo){
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    mpCFBO[stereo]->MakeFBO(_FBO_USE_RGBA);
    mpCFBO[stereo]->Begin(FALSE);
}

void StereoGram_cls::
DrawOnFBOEnd(int stereo){
    mpCFBO[stereo]->End();
    glPopAttrib();
}

void StereoGram_cls::
DrawFunctionOnScreen(MainGLCanvas_cls *pCGLCanvas, void (MainGLCanvas_cls::*func)()){
    if (mbQuadStereo == FALSE){
	(pCGLCanvas->*func)();
	return;
    }
    glDrawBuffer(GL_BACK_LEFT);
    (pCGLCanvas->*func)();
    glDrawBuffer(GL_BACK_RIGHT);
    (pCGLCanvas->*func)();
}

void StereoGram_cls::
DrawOnScreen2D_SideBySide_FitCorner(){
    int clipWidth = mPanelWidth / 2, clipHeight = mPanelHeight;
    mpCFBO[0]->DrawOnScreen2D(mPanelWidth, mPanelHeight,
			      clipWidth, clipHeight,
			      0, 0);
    mpCFBO[1]->DrawOnScreen2D(mPanelWidth, mPanelHeight,
			      clipWidth, clipHeight, 
			      clipWidth, 0);
}

void StereoGram_cls::
DrawOnScreenSimple(int fit){
    mpCFBO[0]->DrawOnScreen2DFit(mPanelWidth, mPanelHeight, fit, TRUE);
}

void StereoGram_cls::
DrawOnScreenSimpleQuad(int fit){
    glDrawBuffer(GL_BACK_LEFT);
    DrawOnScreenSimple(fit);
    glDrawBuffer(GL_BACK_RIGHT);
    DrawOnScreenSimple(fit);
}

void StereoGram_cls::
DrawOnScreenScanLine(int fit){
    mpCFBO[0]->DrawOnScreen2DFit(mPanelWidth, mPanelHeight, fit, TRUE);
    StencilMaskBegin(mPanelWidth, mPanelHeight);
    mpCFBO[1]->DrawOnScreen2DFit(mPanelWidth, mPanelHeight, fit, TRUE);
    StencilMaskEnd();
}

void StereoGram_cls::
DrawOnScreenScanLineQuad(int fit, MainGLCanvas_cls *pCGLCanvas){
    pCGLCanvas->mstrWarning << wxT("Scanline in Quad buffering mode is not supported.") << ENDL;
    DrawOnScreenQuadQuad(fit);
}

void StereoGram_cls::
DrawOnScreenQuadNotQuad(int fit, MainGLCanvas_cls *pCGLCanvas){
    DrawOnScreenSimple(fit);
    pCGLCanvas->mstrWarning << wxT("Quad buffering is not supported.") << ENDL;
}

void StereoGram_cls::
DrawOnScreenQuadQuad(int fit){
    glDrawBuffer(GL_BACK_LEFT);
    mpCFBO[mFlipList[0][mCbFlip.mbBody]]->
	DrawOnScreen2DFit(mPanelWidth, mPanelHeight, fit, TRUE);
    glDrawBuffer(GL_BACK_RIGHT);
    mpCFBO[mFlipList[1][mCbFlip.mbBody]]->
	DrawOnScreen2DFit(mPanelWidth, mPanelHeight, fit, TRUE);
}

void StereoGram_cls::
DrawOnScreenSideBySide(int fit){
    if (fit == _TEX_FIT_CORNER)
	DrawOnScreen2D_SideBySide_FitCorner();
    else{
	int offsetX = 0, offsetY = 0;
	int clipWidth = mPanelWidth / 2, clipHeight = mPanelHeight;
	if (mbFitFishEye == TRUE){
	    int texWidth = clipHeight / 2;
	    if (texWidth < clipWidth)
		offsetX = (clipWidth - texWidth) / 2;
	    if (texWidth > clipWidth){
		texWidth = clipWidth;
		clipHeight = texWidth * 2;
		offsetY = (mPanelHeight - clipHeight) / 2;
	    }
	    mpCFBO[mFlipList[0][mCbFlip.mbBody]]->
		DrawOnScreen2D(mPanelWidth, mPanelHeight,
			       texWidth, clipHeight, 
			       offsetX, offsetY, TRUE);
	    mpCFBO[mFlipList[1][mCbFlip.mbBody]]->
		DrawOnScreen2D(mPanelWidth, mPanelHeight,
			       texWidth, clipHeight, 
			       clipWidth+offsetX, offsetY, TRUE);
	}
	else{
	    offsetY = -(mPanelWidth - mPanelHeight) / 2;
	    clipHeight = mPanelWidth;
	    mpCFBO[mFlipList[0][mCbFlip.mbBody]]->
		DrawOnScreen2D(mPanelWidth, mPanelHeight,
			       clipWidth, clipHeight,
			       0, offsetY, TRUE);
	    mpCFBO[mFlipList[0][mCbFlip.mbBody]]->
		DrawOnScreen2D(mPanelWidth, mPanelHeight,
			       clipWidth, clipHeight,
			       clipWidth, offsetY, TRUE);
	}
    }
}

void StereoGram_cls::
DrawOnScreenSideBySideQuad(int fit){
    glDrawBuffer(GL_BACK_LEFT);
    DrawOnScreenSideBySide(fit);
    glDrawBuffer(GL_BACK_RIGHT);
    DrawOnScreenSideBySide(fit);
}

void StereoGram_cls::
DrawOnScreenAnaglyph(int fit){
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glEnable(GL_BLEND);
    glBlendFunc(GL_ONE, GL_ONE);
    glDisable(GL_DEPTH_TEST);
    for (int i = 0; i < 2; i++){
	ColorMaskBegin(i);
	mpCFBO[i]->DrawOnScreen2DFit(mPanelWidth, mPanelHeight, fit, TRUE);
	ColorMaskEnd();
    }
    glPopAttrib();
}

void StereoGram_cls::
DrawOnScreenAnaglyphQuad(int fit){
    glDrawBuffer(GL_BACK_LEFT);
    DrawOnScreenAnaglyph(fit);
    glDrawBuffer(GL_BACK_RIGHT);
    DrawOnScreenAnaglyph(fit);
}

void StereoGram_cls::
DrawOnScreen2D(bool bUseDistortion, MainGLCanvas_cls *pCGLCanvas){

    GLboolean bQuadStereo;
    glGetBooleanv(GL_STEREO, &bQuadStereo);
    mbQuadStereo = bQuadStereo;

    int fit = (mbFitFishEye == TRUE) ? _TEX_FIT_VERTICAL : _TEX_FIT_HORIZONTAL;
    if (bUseDistortion == TRUE) fit = _TEX_FIT_CORNER;
    if (GetStereoNum() == 1){
	if (bQuadStereo == TRUE) DrawOnScreenSimpleQuad(fit);
	else                     DrawOnScreenSimple(fit);  
    }
    
    if (mCEye.mBody == _STEREO_SCANLINE){
	if (bQuadStereo == TRUE) DrawOnScreenScanLineQuad(fit, pCGLCanvas);
	else                     DrawOnScreenScanLine(fit);  
    }
    
    if (mCEye.mBody == _STEREO_QUADBUFFER){
	if (bQuadStereo == TRUE) DrawOnScreenQuadQuad(fit);
	else                     DrawOnScreenQuadNotQuad(fit, pCGLCanvas);
    }
    
    if (mCEye.mBody == _STEREO_SIDEBYSIDE){
	if (bQuadStereo == TRUE) DrawOnScreenSideBySideQuad(fit);
	else                     DrawOnScreenSideBySide(fit);
    }
    
    if (mCEye.mBody == _STEREO_RED_CYAN ||
	mCEye.mBody == _STEREO_YELLOW_BLUE){
	if (bQuadStereo == TRUE) DrawOnScreenAnaglyphQuad(fit);
	else                     DrawOnScreenAnaglyph(fit);
    }

}

void StereoGram_cls::
CombineStereoBuffer(){
    if (mpCFBO[0] == NULL || mpCFBO[1] == NULL) return;
    if (mCEye.mBody != _STEREO_SIDEBYSIDE){
	mpCFBO[0]->Begin(FALSE);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glDisable(GL_DEPTH_TEST);
	mpCFBO[1]->DrawOnScreen2DFit(mBufferSize, mBufferSize, _TEX_FIT_VERTICAL, TRUE);
	mpCFBO[0]->End();
	return;
    }
    
    // IMPLEMENT_ME!!
    // TChoCTCh̍
    if (mCEye.mBody == _STEREO_SIDEBYSIDE){
	FBO_cls *pCBuffer = new FBO_cls(mBufferSize, mBufferSize);
	pCBuffer->MakeFBO(_FBO_USE_RGBA);
	
	pCBuffer->Begin();
	mpCFBO[0]->DrawOnScreen2DFit(mBufferSize, mBufferSize, _TEX_FIT_VERTICAL, TRUE);
	pCBuffer->End();

	mpCFBO[0]->Begin();
	pCBuffer->DrawOnScreen2D(mBufferSize, mBufferSize,
				  mBufferSize/2, mBufferSize,
				  0, 0);
	mpCFBO[1]->DrawOnScreen2D(mBufferSize, mBufferSize,
				  mBufferSize/2, mBufferSize,
				  mBufferSize/2, 0);
	mpCFBO[1]->End();

	delete pCBuffer;
    }
}

void StereoGram_cls::
ModifyByStereogram(PositionAndDirection_cls *pCCam, int stereoEye){
    if (mCType.mBody == _STEREO_NONE) return;
    LOOP_XYZ pCCam->mPos[xyz] +=
	0.5 * GetOffsetRight(stereoEye) * pCCam->mRgt[xyz];
    pCCam->RotateYPR(GetStereogramYawAngle(stereoEye), 0, 0);
}

void StereoGram_cls::
DrawFocus(PositionAndDirection_cls CCamPos,
	  double doubleIndex, double scale){
    if (mCType.mBody == _STEREO_NONE) return;
    if (mCType.mBody == _STEREO_OFFAXIS) return;
    if (mCEye.mBody  == _STEREO_NONE) return;
    //if (mCDistanceOfEyes.mCbVisible.mbBody == FALSE) return;
    double posFocus[3];
    LOOP_XYZ posFocus[xyz] = CCamPos.mPos[xyz] + 
	CCamPos.mTgt[xyz] * GetDistanceToFocus(doubleIndex);
    glPushMatrix();
    glTranslated(posFocus[0], posFocus[1], posFocus[2]);
    glScaled(scale * 0.02, scale * 0.02, scale * 0.02);
    glCallList(Camera_cls::mDListCrosshair);
    /*
    GLOBJ::Object::Draw(_CAMERA_CROSSHAIR);
    */
    glPopMatrix();
}

int StereoGram_cls::
GetStereoEye(int stereoIndex){
    if (mCType.mBody == _STEREO_NONE) return 0;
    if (mCEye.mBody == _STEREO_NONE) return 0;
    if (mCEye.mBody == _STEREO_LFT) return _STEREO_LFT;
    if (mCEye.mBody == _STEREO_RGT) return _STEREO_RGT;
    if (stereoIndex == 0) return _STEREO_LFT;
    return _STEREO_RGT;
}

bool StereoGram_cls::
IsSideBySideForOverlayRequired(){
    if (mCType.mBody == _STEREO_NONE) return FALSE;
    if (mCEye.mBody == _STEREO_SIDEBYSIDE &&
	mCDualOutput.mBody == FALSE) return TRUE;
    return FALSE;
}

double StereoGram_cls::
GetDistanceToFocus(double doubleIndex){
    return mpCStereoParam->GetInterpolatedValue(doubleIndex, _PARAM_USE_COMMON)[0] * 0.5 / tan (mCToeInAngle.mBody * M_PI / 180);
}

double StereoGram_cls::
GetStereogramYawAngle(int stereoEye){
    if (stereoEye == -1) stereoEye = mCType.mBody;
    double angle = mCToeInAngle.mBody;
    if (mCType.mBody == _STEREO_OFFAXIS) angle = 0;
    if (stereoEye == _STEREO_NONE) angle = 0;
    if (stereoEye == _STEREO_LFT) angle *= -1;
    return angle;
}

double StereoGram_cls::
GetOffsetRight(int eyeMode){
    if (eyeMode == -1) eyeMode = mCEye.mBody;
    if (eyeMode == _STEREO_NONE) return 0;
    double offset = mpCStereoParam->mVCParam[0].mInterpolatedValue;
    if (eyeMode == _STEREO_LFT) offset *= -1;
    return offset;
}

void StereoGram_cls::
ColorMaskBegin(int stereo/*, bool bFlip, bool bTrueScanline*/){
    //if (bFlip == TRUE) stereoIndex = 1 - stereoIndex;
    
    if (mCbFlip.mbBody == TRUE) stereo = 1 - stereo;

    glColor4f(1, 1, 1, 1);
    if (mCType.mBody != _STEREO_NONE){
	if (mCEye.mBody == _STEREO_RED_CYAN)
	    switch(stereo){
	    case 0: glColorMask(1, 0, 0, 1); return;
	    case 1: glColorMask(0, 1, 1, 1); return;
	    }
	if (mCEye.mBody == _STEREO_YELLOW_BLUE)
	    switch(stereo){
	    case 0: glColorMask(1, 1, 0, 1); return;
	    case 1:glColorMask(0, 0, 1, 1); return;
	    }
	/*
	if (mCEye.mBody == _STEREO_SCANLINE && bTrueScanline == FALSE){
	    glColor4f(1, 1, 1, 0.707);
	    return;
	}
	*/
    }
    glColorMask(1, 1, 1, 1);
}

void StereoGram_cls::
StencilMaskBegin(int width, int height/*, int stereoIndex, bool bFlip*/){
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    if (mCEye.mBody != _STEREO_SCANLINE) return;
    glEnable(GL_STENCIL_TEST);
    //bool bOdd = TRUE;//(stereoIndex == 1);
    //if (bFlip == TRUE) bOdd = !bOdd;
    MakeStripePatternOnStencil(width, height, mCbFlip.mbBody);
    glStencilFunc( GL_EQUAL, 1, ~0); //1̃sNZɕ`
}

void StereoGram_cls::
MakeStripePatternOnStencil(int width, int height, bool bOdd){
    FBO_cls::Push2DMatrix(width, height);

    // XeṼNA
    glClear( GL_STENCIL_BUFFER_BIT);

    //ꂩ`悷̂̃XeVlɂׂĂP^O    
    glStencilFunc( GL_ALWAYS, 1, ~0); 
    glStencilOp(GL_REPLACE,GL_REPLACE,GL_REPLACE);
    
    //XeVobt@ȊÕobt@ɂ͏܂Ȃ
    glColorMask(0,0,0,0);
    glDepthMask(0);
    
    glLineWidth(1);
    for (int y = (int)bOdd; y < height; y+=2){
	glBegin(GL_LINES);
	glVertex2f(     0, y);
	glVertex2f( width, y);
	glEnd();
    }

    //̉
    glColorMask(1,1,1,1);
    glDepthMask(1);
    glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);

    FBO_cls::Pop2DMatrix();
}

void StereoGram_cls::
ColorMaskEnd(){
    glColorMask(1, 1, 1, 1);
}

void StereoGram_cls::
StencilMaskEnd(){
    glDisable(GL_STENCIL_TEST);
    glPopAttrib();
}

bool StereoGram_cls::
IsGUIShownOnScreen(){
    if (mpCGUI == NULL) return FALSE;
    return mpCGUI->IsShownOnScreen();
}

void StereoGram_cls::
ReadStrLine(std::wistream &istream, wxString strLine){
    if (strLine == wxT("--StereoGram")){
	std::wstring wstrLine;
	wxString strLine;
	while ( std::getline (istream, wstrLine)){
	    strLine = wstrLine.c_str();
	    if (strLine.Find(wxT("ParamGroup=")) != wxNOT_FOUND) break;
	    mCConfList.ReadStrLineAll(strLine);
	}

	/*mCDistanceOfEyes.*/mpCStereoParam->ReadStrLine(istream, strLine);
    }
    //mCConfList.ReadStrLineAll(strLine);
}

wxString StereoGram_cls::
Serialize(){
    wxString sOut;
    sOut << wxT("--StereoGram") << ENDL;
    sOut << mCConfList.SerializeAll();
    sOut << /*mCDistanceOfEyes.*/mpCStereoParam->Serialize();
    return sOut;
}
