// -*- c -*-
// "clsGrid.h"
// (setq c-basic-offset 4)

#include "clsSphere.h"

#include "../COMMON/MyVector.h"
#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

#include <math.h>

#define GLOBJ_SPHERE1 0.447213595f
#define GLOBJ_SPHERE2 0.894427190f
#define GLOBJ_SPHERE3 0.276393202f
#define GLOBJ_SPHERE4 0.723606797f
#define GLOBJ_SPHERE5 0.525731112f
#define GLOBJ_SPHERE6 0.850650808f

#ifndef M_PI
#define M_PI 3.14159
#endif 

Sphere_cls::
Sphere_cls(){
    mbInitialized = FALSE;
    for (int i = 0; i < 4; i++){
	for (int j = 0; j < 4; j++)
	    mDList[i][j] = -1;
	mDListNoTex[i] = -1;
    }

    GLdouble l_icosahedronv[12][3] = {
	{             0,             0,             1},
	{             0, GLOBJ_SPHERE2, GLOBJ_SPHERE1},
	{-GLOBJ_SPHERE6, GLOBJ_SPHERE3, GLOBJ_SPHERE1},
	{-GLOBJ_SPHERE5,-GLOBJ_SPHERE4, GLOBJ_SPHERE1},
	{ GLOBJ_SPHERE5,-GLOBJ_SPHERE4, GLOBJ_SPHERE1},
	{ GLOBJ_SPHERE6, GLOBJ_SPHERE3, GLOBJ_SPHERE1}, 
	{-GLOBJ_SPHERE5, GLOBJ_SPHERE4,-GLOBJ_SPHERE1},
	{-GLOBJ_SPHERE6, GLOBJ_SPHERE3,-GLOBJ_SPHERE1},
	{             0,-GLOBJ_SPHERE2,-GLOBJ_SPHERE1},
	{ GLOBJ_SPHERE6,-GLOBJ_SPHERE3,-GLOBJ_SPHERE1},
	{ GLOBJ_SPHERE5, GLOBJ_SPHERE4,-GLOBJ_SPHERE1},
	{             0,             0,            -1} };

    GLint l_icosahedronface[20][3] = {
	{1,2,0},{2,3,0},{3,4,0},{4,5,0},{5,1,0},
	{1,6,2},{2,6,7},{2,7,3},{3,7,8},{4,3,8},
	{4,8,9},{4,9,5},{5,9,10},{5,10,1},{1,10,6},
	{6,10,11},{7,6,11},{8,7,11},{9,8,11},{10,9,11} };

    for (int i = 0; i < 12; i++)
	LOOP_XYZ
	    icosahedronv[i][xyz] = l_icosahedronv[i][xyz];

    for (int i = 0; i < 20; i++)
	for (int j = 0; j < 3; j++)
	    icosahedronface[i][j] = l_icosahedronface[i][j];

    dummytex2D[0] = 0;
    dummytex2D[1] = 0;
}

/*!
  @brief XYZW theta-phi Wɕϊ
*/
void Sphere_cls::
ChangeCoordinate3Dto2D(double vec3D[3], double vec2D[2]){
    int phi = 0, theta = 1;
    vec2D[phi] = atan2(vec3D[Y], -vec3D[X]);
    vec2D[phi] = (vec2D[phi] + M_PI)/(2.0 * M_PI);
    vec2D[theta] = atan2(vec3D[Z], 
			 sqrt(vec3D[X]*vec3D[X]+vec3D[Y]*vec3D[Y]));
    vec2D[theta] = (vec2D[theta]+M_PI/2.0)/M_PI;
}
    
/*!
  @brief x[0,1] y[0,1] ͈̔͂͂ݏoW̏C
*/
void Sphere_cls::
AdjustVec2DOverBoundary(double vector[2], double vectormem[2]){
    for (int xy = 0; xy <= 1; xy++){
	if (vector[xy]-vectormem[xy] > 0.5)
	    vector[xy] = vector[xy] - 1.0f; 
	if (vector[xy]-vectormem[xy] < -0.5)
	    vector[xy] = vector[xy] + 1.0f; 
    }
}

/*!
  @brief @AeNX`W݂ŁARp`|S\B
*/
void Sphere_cls::
DrawTriangle(double v1[3], double v2[3], double v3[3], bool bUseTex, int mapType){
    
    double normal1[3],normal2[3],normal3[3];
    double texvector1[2],texvector2[2],texvector3[2];
    
    LOOP_XYZ normal1[xyz] = v1[xyz];
    LOOP_XYZ normal2[xyz] = v2[xyz];
    LOOP_XYZ normal3[xyz] = v3[xyz]; 

    //glBegin(GL_TRIANGLES);
    
    // @xNgAeNX`WA_W ^ ̂P
    MyVector::Normalize(normal1, 1.0);
    glNormal3dv(normal1);
    if (bUseTex == TRUE){
	switch (mapType){
	case 0:
	    texvector1[0] = v1[Y]*0.5+0.5; texvector1[1] = v1[Z]*0.5+0.5; 
	    break;
	case 1:
	    texvector1[0] = v1[X]*0.5+0.5; texvector1[1] = v1[Z]*0.5+0.5; 
	    break;
	case 2:
	    texvector1[0] = v1[X]*0.5+0.5; texvector1[1] = v1[Y]*0.5+0.5; 
	    break;
	case 3:
	    ChangeCoordinate3Dto2D(normal1, texvector1); break;
	}
	glTexCoord2dv(texvector1);
    }
    glVertex3dv(v1);
  
    // @xNgAeNX`WA_W ^ ̂Q
    MyVector::Normalize(normal2,(double)1.0);
    glNormal3dv(normal2);
    if (bUseTex == TRUE){
	switch (mapType){
	case 0:
	    texvector2[0] = v2[Y]*0.5+0.5; texvector2[1] = v2[Z]*0.5+0.5;
	    break;
	case 1:
	    texvector2[0] = v2[X]*0.5+0.5; texvector2[1] = v2[Z]*0.5+0.5;
	    break;
	case 2:
	    texvector2[0] = v2[X]*0.5+0.5; texvector2[1] = v2[Y]*0.5+0.5; 
	    break;
	case 3:
	    ChangeCoordinate3Dto2D(normal2, texvector2); 
	    AdjustVec2DOverBoundary(texvector2, texvector1); 
	    break;
	}
	glTexCoord2dv(texvector2);
    }
    glVertex3dv(v2);

    // @xNgAeNX`WA_W ^ ̂R
    // eNX`Ŵ͂ݏoĂB
    MyVector::Normalize(normal3,(double)1.0);
    glNormal3dv(normal3);
    if (bUseTex == TRUE){
	switch (mapType){
	case 0:
	    texvector3[0] = v3[Y]*0.5+0.5; texvector3[1] = v3[Z]*0.5+0.5;
	    break;
	case 1:
	    texvector3[0] = v3[X]*0.5+0.5; texvector3[1] = v3[Z]*0.5+0.5;
	    break;
	case 2:
	    texvector3[0] = v3[X]*0.5+0.5; texvector3[1] = v3[Y]*0.5+0.5;
	    break;
	case 3:
	    ChangeCoordinate3Dto2D(normal3, texvector3); 
	    if (texvector3[Y] > 0.99 || texvector3[Y] < 0.01) 
		texvector3[X] = (texvector1[X] + texvector2[X])/2.0f; 
	    else 
		AdjustVec2DOverBoundary(texvector3, texvector2); 
	    break;
	}
	glTexCoord2dv(texvector3);
    }
    glVertex3dv(v3);
    
    //glEnd();
}

/*!
  @brief ċAIɂRp`|S𕪊āA炩ȋ`悷
*/
void Sphere_cls::
DivideTriangle
(double v1[3], double v2[3], double v3[3], bool bUseTex, int depth, int mapType){
    
    double v12[3], v23[3], v31[3];
    
    if (depth <= 0){
	DrawTriangle(v1,v2,v3, bUseTex, mapType);
	return; }

    LOOP_XYZ v12[xyz] = (v1[xyz] + v2[xyz]);
    LOOP_XYZ v23[xyz] = (v2[xyz] + v3[xyz]);
    LOOP_XYZ v31[xyz] = (v3[xyz] + v1[xyz]); 
    
    MyVector::Normalize(v12,(double)1.0);
    MyVector::Normalize(v23,(double)1.0);
    MyVector::Normalize(v31,(double)1.0);

    DivideTriangle(v1,v12,v31, bUseTex, depth-1, mapType);
    DivideTriangle(v2,v23,v12, bUseTex, depth-1, mapType);
    DivideTriangle(v12,v23,v31, bUseTex, depth-1, mapType);
    DivideTriangle(v31,v23,v3, bUseTex, depth-1, mapType);
}


int Sphere_cls::
DListDraw(int GLMode, bool bUseTex, int depth, int mapType){
    int    listName = 0;
	
    if (GLMode == GL_COMPILE || GLMode == GL_COMPILE_AND_EXECUTE){
	listName = glGenLists(1); // VKXg̔ԍ
	glNewList (listName, GLMode); 
    }

    glEnable(GL_NORMALIZE);
    
    double tmpvector0[3], tmpvector1[3], tmpvector2[3];
    for (int i=0; i<20; i++){
	LOOP_XYZ{
	    tmpvector0[xyz] = icosahedronv[icosahedronface[i][0]][xyz];
	    tmpvector1[xyz] = icosahedronv[icosahedronface[i][1]][xyz];
	    tmpvector2[xyz] = icosahedronv[icosahedronface[i][2]][xyz];
	}
	glBegin(GL_TRIANGLES);
	DivideTriangle(tmpvector0,tmpvector1,tmpvector2, 
		       bUseTex, depth, mapType);
	glEnd();
    }

    if (GLMode == GL_COMPILE || GLMode == GL_COMPILE_AND_EXECUTE)
	glEndList(); 
    
    return listName;
}

void Sphere_cls::
SequentialDraw(int type){
    glBegin(GL_TRIANGLES);
    for (int face = 0; face < 6; face++){
	glTexCoord2dv(dummytex2D);
	glNormal3dv(icosahedronv[icosahedronface[face][0]]);
	glVertex3dv(icosahedronv[icosahedronface[face][0]]);

	glTexCoord2dv(dummytex2D);
	glNormal3dv(icosahedronv[icosahedronface[face][1]]);
	glVertex3dv(icosahedronv[icosahedronface[face][1]]);

	glTexCoord2dv(dummytex2D);
	glNormal3dv(icosahedronv[icosahedronface[face][2]]);
	glVertex3dv(icosahedronv[icosahedronface[face][2]]);
    }
    glEnd();
}

void Sphere_cls::
InitializeIfNot(){
    if (mbInitialized == TRUE) return;
    for (int i = 0; i < 5; i++)
	for (int j = 0; j < 4; j++)
	    mDList[i][j] = DListDraw(GL_COMPILE, TRUE, i, j);
    for (int i = 0; i < 5; i++)
	mDListNoTex[i] = DListDraw(GL_COMPILE, FALSE, i, 0);
    mbInitialized = TRUE;
}

void Sphere_cls::
Draw(int quality, int mapType){
    glCallList(mDList[quality][mapType]);
}

void Sphere_cls::
DrawNoTex(int quality){
    glCallList(mDListNoTex[quality]);
}

int Sphere_cls::
GetDList(int quality, int mapType){
    return mDList[quality][mapType];
}

int Sphere_cls::
GetDListNoTex(int quality){
    return mDListNoTex[quality];
}

Sphere_cls *GpCSphere = NULL;
