// "clsMainGLCanvas.h"
// (setq c-basic-offset 4)

#include "clsImageIO.h"
#include "clsStereoGram.h"
#include "../COMMON/clsFilename.h"
#include "../MYAPI/MsgBox.h"
#include "../MYGL/clsFBO.h"
#include "../WIDGETCTRL/EX/clsFileDialogEX.h"
#include <wx/wx.h>

#include "../COMMON/CommonDefinitionsGL.h"
#include "../COMMON/CommonDefinitions.h"

extern wxFileDialogEX*   GpCFileDialog;

#ifdef WIN32
PFNGLGENBUFFERSARBPROC glGenBuffersARB;
PFNGLDELETEBUFFERSARBPROC glDeleteBuffersARB;
PFNGLBINDBUFFERARBPROC glBindBufferARB;
PFNGLBUFFERDATAARBPROC glBufferDataARB;
PFNGLMAPBUFFERARBPROC glMapBufferARB;
PFNGLUNMAPBUFFERARBPROC glUnmapBufferARB;
PFNGLGETBUFFERPARAMETERIVARBPROC glGetBufferParameterivARB;
#endif

ThreadSave_cls::
ThreadSave_cls(wxThreadKind kind) :
    wxThread(kind){
}

void ThreadSave_cls::
SetParameters(bool *pbFinishedFlag, 
	      ImageIO_cls *pCParent, int width, int height,
	      wxString strFileOut, int imgType,
	      bool bUseOverlay, bool bSideBySide, 
	      unsigned char *pBuffer, 
	      bool bFreeDataAtEnd){
    mpbFinishedFlagLink = pbFinishedFlag;
    mpCParent = pCParent;
    mWidth = width; mHeight = height;
    mstrFileOut = strFileOut;

    mbAllocLocalSRC = FALSE;
    mpBufferSRC = NULL;
    
    mImgType = imgType;

    mpBuffer = NULL;
    mbAllocLocal = FALSE;
    if (mpBuffer == NULL){
	mpBuffer = (unsigned char*)malloc(mWidth * mHeight * 3);
	if (mpBuffer == NULL){
	    printf("mallc failed retry!!\n");
	    mpBuffer = (unsigned char*)malloc(mWidth * mHeight * 3);
	    if (mpBuffer == NULL)
		printf("malloc retry failed!!\n");
	}
	if (mpBuffer != NULL)
	    mbAllocLocal = TRUE;
    }
    else{
	mpBuffer = pBuffer;
	mbAllocLocal = bFreeDataAtEnd;
    }

    mpOverlay = NULL;
    mbUseOverlay = bUseOverlay;
    mbSideBySide = bSideBySide;
    if (mbUseOverlay == TRUE){
	mpOverlay = (unsigned char*)malloc(mWidth * mHeight * 4);
	if (mpOverlay == NULL)
	    mbUseOverlay = FALSE;
    }

    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadSave_cls::
DeclareResizing(int bufferWidth, int bufferHeight){
    mBufferWidth  = bufferWidth;
    mBufferHeight = bufferHeight;

    int channel = 3;
    if (mImgType == _TYPE_RGBA) channel = 4;
    
    mpBufferSRC = (unsigned char*)malloc(mBufferWidth * mBufferHeight * channel);
    if (mpBufferSRC != NULL) 
	mbAllocLocalSRC = TRUE;
}

void ThreadSave_cls::
GetAverageRGB(int widthBuffer, int heightBuffer,
	      double bufferX, double bufferY,
	      double bufferX2, double bufferY2,
	      unsigned char* pBufferSRC,
	      unsigned char* pBufferTGT,
	      int srcChannel){
    double RSum = 0, GSum = 0, BSum = 0;
    double areaSum = 0;
    if (bufferY2 >= heightBuffer) bufferY2 = heightBuffer - 0.001;
    if (bufferX2 >= widthBuffer) bufferX2 = widthBuffer - 0.001;
    for (int bufferiY = (int)bufferY; bufferiY <= bufferY2; bufferiY++){
	double factorY = 1;
	if (bufferY - bufferiY > 0)
	    factorY -= bufferY - bufferiY;
	if (bufferY2 - bufferiY < 1)
	    factorY -= (bufferiY + 1 - bufferY2);
	int indexY = bufferiY * widthBuffer;
	for (int bufferiX = (int)bufferX; bufferiX <= bufferX2; bufferiX++){
	    double factorX = 1;
	    if (bufferX - bufferiX > 0)
		factorX -= bufferX - bufferiX;
	    if (bufferX2 - bufferiX < 1)
		factorX -= (bufferiX + 1 - bufferX2);
	    int index = (indexY + bufferiX) * srcChannel;
	    double area = factorX * factorY;
	    RSum += pBufferSRC[index + 0] * area;
	    GSum += pBufferSRC[index + 1] * area;
	    BSum += pBufferSRC[index + 2] * area;
	    areaSum += area;
	}
    }
    
    pBufferTGT[0] = (unsigned char)(RSum / areaSum);
    pBufferTGT[1] = (unsigned char)(GSum / areaSum);
    pBufferTGT[2] = (unsigned char)(BSum / areaSum);
}

void ThreadSave_cls::
GetAverageAlpha(int widthBuffer, int heightBuffer,
		double bufferX, double bufferY,
		double bufferX2, double bufferY2,
		unsigned char* pBufferSRC,
		unsigned char* pBufferTGT){
    double ASum = 0;
    double areaSum = 0;
    if (bufferY2 >= heightBuffer) bufferY2 = heightBuffer - 0.001;
    if (bufferX2 >= widthBuffer) bufferX2 = widthBuffer - 0.001;
    for (int bufferiY = (int)bufferY; bufferiY <= bufferY2; bufferiY++){
	double factorY = 1;
	if (bufferY - bufferiY > 0)
	    factorY -= bufferY - bufferiY;
	if (bufferY2 - bufferiY < 1)
	    factorY -= (bufferiY + 1 - bufferY2);
	int indexY = bufferiY * widthBuffer;
	for (int bufferiX = (int)bufferX; bufferiX <= bufferX2; bufferiX++){
	    double factorX = 1;
	    if (bufferX - bufferiX > 0)
		factorX -= bufferX - bufferiX;
	    if (bufferX2 - bufferiX < 1)
		factorX -= (bufferiX + 1 - bufferX2);
	    int index = (indexY + bufferiX) * 4;
	    double area = factorX * factorY;
	    ASum += pBufferSRC[index + 3] * area;
	    areaSum += area;
	}
    }
    
    
    pBufferTGT[0] = (unsigned char)(ASum / areaSum);
}

void ThreadSave_cls::
ResizeImage(unsigned char *pBuffer){

    int channel = 3;
    if (mImgType == _TYPE_RGBA) channel = 4;

    if (mbAllocLocalSRC == TRUE){
	if (mWidth < mHeight){
	    double dOffsetX = (1.0 - (double)mWidth/mHeight) / 2;
	    for (int y = 0; y < mHeight; y++){
		double bufferY = (double)y / mHeight * mBufferHeight;
		double dHeight = 1.0 / mHeight * mBufferHeight;
		int indexy = y * mWidth;
		for (int x = 0; x < mWidth; x++){
		    double bufferX = ((double)x/mHeight + dOffsetX) * mBufferWidth;
		    double dWidth = 1.0 / mHeight * mBufferWidth;
		    GetAverageRGB(mBufferWidth, mBufferHeight,
				  bufferX, bufferY, 
				  bufferX + dWidth, 
				  bufferY + dHeight,
				  mpBufferSRC, pBuffer + (indexy + x) * 3,
				  channel);
		}
	    }
	}
	else{
	    double dOffsetY = (1.0 - (double)mHeight/mWidth) / 2;
	    for (int y = 0; y < mHeight; y++){
		double bufferY = ((double)y / mWidth + dOffsetY) * mBufferHeight;
		double dHeight = 1.0 / mWidth * mBufferHeight;
		int indexy = y * mWidth;
		for (int x = 0; x < mWidth; x++){
		    double bufferX = (double)x /mWidth * mBufferWidth;
		    double dWidth = 1.0 / mWidth * mBufferWidth;
		    GetAverageRGB(mBufferWidth, mBufferHeight,
				  bufferX, bufferY, 
				  bufferX + dWidth, 
				  bufferY + dHeight,
				  mpBufferSRC, pBuffer + (indexy + x) * 3,
				  channel);
		}
	    }
	}
    }
}

void ThreadSave_cls::
ResizeAlpha(unsigned char *pAlphaBuffer){
    if (mWidth < mHeight){
	double dOffsetX = (1.0 - (double)mWidth/mHeight) / 2;
	for (int y = 0; y < mHeight; y++){
	    double bufferY = (double)y / mHeight * mBufferHeight;
	    double dHeight = 1.0 / mHeight * mBufferHeight;
	    int indexy = y * mWidth;
	    for (int x = 0; x < mWidth; x++){
		double bufferX = ((double)x/mHeight + dOffsetX) * mBufferWidth;
		double dWidth = 1.0 / mHeight * mBufferWidth;
		GetAverageAlpha(mBufferWidth, mBufferHeight,
				bufferX, bufferY, 
				bufferX + dWidth, 
				bufferY + dHeight,
				mpBufferSRC, pAlphaBuffer + (indexy + x));
	    }
	}
    }
    else{
	double dOffsetY = (1.0 - (double)mHeight/mWidth) / 2;
	for (int y = 0; y < mHeight; y++){
	    double bufferY = ((double)y / mWidth + dOffsetY) * mBufferHeight;
	    double dHeight = 1.0 / mWidth * mBufferHeight;
	    int indexy = y * mWidth;
	    for (int x = 0; x < mWidth; x++){
		double bufferX = (double)x /mWidth * mBufferWidth;
		double dWidth = 1.0 / mWidth * mBufferWidth;
		GetAverageAlpha(mBufferWidth, mBufferHeight,
				bufferX, bufferY, 
				bufferX + dWidth, 
				bufferY + dHeight,
				mpBufferSRC, pAlphaBuffer + (indexy + x));
	    }
	}
    }
}

void ThreadSave_cls::
AdjustColor(unsigned char *pBuffer, unsigned char *pAlphaBuffer){
    for (int y = 0; y < mHeight; y++){
	int indexy = y * mWidth;
	for (int x = 0; x < mWidth; x++){
	    int index = indexy + x;
	    float alpha = pAlphaBuffer[index];
	    if (alpha == 0) alpha = 1;
	    alpha /= 255;
	    LOOP_RGB {
		float orgClr = pBuffer[index * 3 + rgb];
		float clr = orgClr / alpha;
		if (clr > 255) clr = 255;
		pBuffer[index * 3 + rgb] = clr;
	    }
	}
    }
}

void ThreadSave_cls::
OverlaySideBySide(unsigned char *pBuffer){
    unsigned char *pOverlay = mpOverlay;
    for (int y = 0; y < mHeight; y++){
	int indexy = y * mWidth;
	for (int x = 0; x < mWidth; x+=2){
	    int index = indexy + x;
	    int indexTgt1 = indexy + x/2;
	    int indexTgt2 = indexy + x/2 + mWidth/2;
	    float alpha = (float)pOverlay[index * 4 + 3]/255;
	    for (int rgb = 0; rgb < 3; rgb++)
		pBuffer[indexTgt1 * 3 + rgb] = 
		    pOverlay[index * 4 + rgb] * alpha +
		    pBuffer[indexTgt1 * 3 + rgb] * (1.0f-alpha);
	    for (int rgb = 0; rgb < 3; rgb++)
		pBuffer[indexTgt2 * 3 + rgb] = 
		    pOverlay[index * 4 + rgb] * alpha +
		    pBuffer[indexTgt2 * 3 + rgb] * (1.0f-alpha);
	}
    }
}

void ThreadSave_cls::
OverlaySingle(unsigned char *pBuffer){
    unsigned char *pOverlay = mpOverlay;
    for (int y = 0; y < mHeight; y++){
	int indexy = y * mWidth;
	for (int x = 0; x < mWidth; x++){
	    int index = indexy + x;
	    float alpha = (float)pOverlay[index * 4 + 3]/255;
	    for (int rgb = 0; rgb < 3; rgb++)
		pBuffer[index * 3 + rgb] = 
		    pOverlay[index * 4 + rgb] * alpha +
		    pBuffer[index * 3 + rgb] * (1.0f-alpha);
	}
    }
}

void ThreadSave_cls::
OverlayAlphaSingle(unsigned char *pAlphaBuffer){
    unsigned char *pOverlay = mpOverlay;
    for (int y = 0; y < mHeight; y++){
	int indexy = y * mWidth;
	for (int x = 0; x < mWidth; x++){
	    int index = indexy + x;
	    for (int rgb = 0; rgb < 3; rgb++)
		if (pOverlay[index * 4] > pAlphaBuffer[index])
		    pAlphaBuffer[index] = pOverlay[index * 4];

	}
    }
}

wxThread::ExitCode 
ThreadSave_cls::
Entry(){
    unsigned char *pBuffer = mpBuffer;
    
    if (pBuffer == NULL){
	printf("Image Save Skipped!!\n");
	if (mbUseOverlay == TRUE) free(mpOverlay);
	if (mbAllocLocalSRC == TRUE) free(mpBufferSRC);
	*mpbFinishedFlagLink = TRUE;
	return 0;
    }

    ResizeImage(pBuffer);
    
    unsigned char *pAlphaBuffer = NULL;
    if (mImgType == _TYPE_RGBA){
	pAlphaBuffer = (unsigned char*)malloc(mWidth * mHeight);
	ResizeAlpha(pAlphaBuffer);
	AdjustColor(pBuffer, pAlphaBuffer);
    }

    if (mbUseOverlay == TRUE){
	switch(mbSideBySide){
	case TRUE:
	    OverlaySideBySide(pBuffer);
	    break;
	case FALSE:
	    OverlaySingle(pBuffer);
	    if (mImgType == _TYPE_RGBA) OverlayAlphaSingle(pAlphaBuffer);
	    break;
	}
    }

    if (mpBuffer != NULL){
	wxImage CImage(mWidth, mHeight, mpBuffer, TRUE); 

	if (pAlphaBuffer != NULL){
	    CImage.SetAlpha(pAlphaBuffer);
	    pAlphaBuffer = NULL;
	}

	if (CImage.IsOk() == TRUE){
	    SplitPath_cls CPath(mstrFileOut);
	    if (CPath.GetPathStr(_FN_EXT) == wxT("")) mstrFileOut += wxT(".png");
	    
	    //mpCParent->mstrLastSavedFile = mstrFileOut; //Xbh瑀댯

	    wxImage CMirror(CImage.Mirror(FALSE)); // ]ftH
	    CMirror.SetOption(wxT("quality"), mpCParent->mCJpegQuality.mBody);
	    if (CMirror.IsOk() == TRUE){
		printf("Saving\n");
		if (CMirror.SaveFile(mstrFileOut) == FALSE){
		    printf("ImgOutputFailed!!\n");
		}
		printf("Saving End\n");
	    }
	}
    }
    
    // NULLȂɂȂ̂ňSi{ȂȂ̂ňُ킾j
    if (mbAllocLocal == TRUE && mpBuffer != NULL) free(mpBuffer);
    if (mbUseOverlay == TRUE) free(mpOverlay);
    if (mbAllocLocalSRC == TRUE) free(mpBufferSRC);
    if (mImgType == _TYPE_RGBA) free(pAlphaBuffer);
    
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// -----

ImageIO_cls::
ImageIO_cls(){
    for (int i = 0; i < _IMAGE_OUT_MAX_THREAD; i++){
	mbSaveThreadActive[i] = FALSE;
	mbSaveThreadFinished[i] = FALSE;
    }
    mActiveThread = 0;

    mCConfList.Add(&mCJpegQuality, L"ImageJpegQuality=", L"95");
    mCConfList.Add(&mCMultiThread, L"ImageMultiThrad=", L"1");
}

ImageIO_cls::
~ImageIO_cls(){
}

bool ImageIO_cls::
SaveImageFromData(int width, int height, wxString strFileOut, 
		  unsigned char* pBuffer){
    if (mbSaveThreadActive[mActiveThread] == TRUE){
	while (mbSaveThreadFinished[mActiveThread] == FALSE)
	    wxMilliSleep(3);
	mbSaveThreadActive[mActiveThread] = FALSE;
	mbSaveThreadFinished[mActiveThread] = FALSE;
    }

    bool retVal = FALSE;

    mpCThreadSave[mActiveThread] = new ThreadSave_cls(wxTHREAD_DETACHED);
    mpCThreadSave[mActiveThread]->
	SetParameters(&mbSaveThreadFinished[mActiveThread],
		      this, width, height, strFileOut, _TYPE_RGB,
		      FALSE, FALSE,
		      pBuffer, TRUE);
 
    mbSaveThreadActive[mActiveThread] = TRUE;    
    mpCThreadSave[mActiveThread]->Run();
    mActiveThread++;
    if (mActiveThread >= _IMAGE_OUT_MAX_THREAD) mActiveThread = 0;
    
    return TRUE;
}

bool ImageIO_cls::
SaveImageFromGL(int width, int height){
    return SaveImageFromGL(width, height, mstrFile);
}

bool ImageIO_cls::
SaveImageFromGL(int width, int height, wxString strFileOut,
		int offsetX, int offsetY){

    if (mbSaveThreadActive[mActiveThread] == TRUE){
	while (mbSaveThreadFinished[mActiveThread] == FALSE)
	    wxMilliSleep(3);
	mbSaveThreadActive[mActiveThread] = FALSE;
	mbSaveThreadFinished[mActiveThread] = FALSE;
    }

    bool retVal = FALSE;
    int trimWidth = width - (width % 8);
    
    mpCThreadSave[mActiveThread] = new ThreadSave_cls(wxTHREAD_DETACHED);
    mpCThreadSave[mActiveThread]->
	SetParameters(&mbSaveThreadFinished[mActiveThread],
		      this, trimWidth, height, strFileOut, _TYPE_RGB, FALSE);

    glReadPixels(offsetX, offsetY, trimWidth, height, GL_RGB, GL_UNSIGNED_BYTE, mpCThreadSave[mActiveThread]->mpBuffer);

    mbSaveThreadActive[mActiveThread] = TRUE;    
    mpCThreadSave[mActiveThread]->Run();
    mActiveThread++;
    if (mActiveThread >= _IMAGE_OUT_MAX_THREAD) mActiveThread = 0;
    
    return TRUE;
}

bool ImageIO_cls::
SaveImageFromFBO(FBO_cls *pCFBOMain, FBO_cls *pCFBOOverlay,
		 int width, int height,	wxString strFileOut, 
		 int imgType, bool bSideBySide, 
		 int offsetX, int offsetY){
    // gXbht
    if (mbSaveThreadActive[mActiveThread] == TRUE){
	while (mbSaveThreadFinished[mActiveThread] == FALSE)
	    wxMilliSleep(3);
	mbSaveThreadActive[mActiveThread] = FALSE;
	mbSaveThreadFinished[mActiveThread] = FALSE;
    }

    int widthBuffer = pCFBOMain->mWidthBuffer;
    int heightBuffer = pCFBOMain->mHeightBuffer;
    int texID = pCFBOMain->mTextureID;
    int texIDOverlay = pCFBOOverlay->mTextureID;

    bool retVal = FALSE;
    //    int trimWidth = width - (width % 8);

    // width, height͏o̓TCY
    
    mpCThreadSave[mActiveThread] = new ThreadSave_cls(wxTHREAD_DETACHED);
    
    mpCThreadSave[mActiveThread]->
	SetParameters(&mbSaveThreadFinished[mActiveThread],
		      this, width, height, strFileOut, imgType, 
		      TRUE, bSideBySide);
    
    unsigned char *pBuffer = mpCThreadSave[mActiveThread]->mpBuffer;

    glEnable(GL_TEXTURE_2D);
    
    glBindTexture(GL_TEXTURE_2D, texIDOverlay);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
    unsigned char *pBufferOver = mpCThreadSave[mActiveThread]->mpOverlay;
    if (pBufferOver != NULL){
	int bufSize = pCFBOOverlay->mWidthBuffer * pCFBOOverlay->mHeightBuffer * 4;
	GLuint pboID;
	glGenBuffersARB(1, &pboID);
	glBindBufferARB(GL_PIXEL_PACK_BUFFER_ARB, pboID);
	glBufferDataARB(GL_PIXEL_PACK_BUFFER_ARB, bufSize, 0, GL_STREAM_READ_ARB);
	glGetTexImage( GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
	unsigned char *color = (unsigned char*)glMapBufferARB(GL_PIXEL_PACK_BUFFER_ARB, GL_READ_ONLY_ARB);
	
	memcpy(pBufferOver, color, bufSize);
	glUnmapBufferARB(GL_PIXEL_PACK_BUFFER_ARB);//}bsO
	
	glBindBufferARB(GL_PIXEL_PACK_BUFFER_ARB, 0);
	glDeleteBuffersARB(1, &pboID);
	
	//glGetTexImage( GL_TEXTURE_2D, 0, GL_RGBA, GL_UNSIGNED_BYTE, pBufferOver);
    }

    bool bDirectRGB = FALSE;
    if (imgType == _TYPE_RGB && widthBuffer == width && heightBuffer == height)
	bDirectRGB = TRUE;
    
    glBindTexture(GL_TEXTURE_2D, texID);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glPixelStorei(GL_PACK_ALIGNMENT, 1);
    if (bDirectRGB == TRUE){
	if (pBuffer != NULL){
	    glGetTexImage( GL_TEXTURE_2D, 0, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);
	}
	if (pBuffer == NULL)
	    printf("*pBuffer == NULL\n");
    }
    else{
	mpCThreadSave[mActiveThread]->DeclareResizing(widthBuffer, heightBuffer);
	
	int channelType = GL_RGB;
	if (imgType == _TYPE_RGBA) channelType = GL_RGBA;
	
	unsigned char *pBufferSRC = mpCThreadSave[mActiveThread]->mpBufferSRC;
	if (pBufferSRC != NULL){

	    int bufSize = widthBuffer * heightBuffer;
	    if (channelType == GL_RGB) bufSize *= 3;
	    if (channelType == GL_RGBA) bufSize *= 4;
	    GLuint pboID;
	    glGenBuffersARB(1, &pboID);
	    glBindBufferARB(GL_PIXEL_PACK_BUFFER_ARB, pboID);
	    glBufferDataARB(GL_PIXEL_PACK_BUFFER_ARB, bufSize, 0, GL_STREAM_READ_ARB);
	    glGetTexImage( GL_TEXTURE_2D, 0, channelType, GL_UNSIGNED_BYTE, 0);
	    unsigned char *color = (unsigned char*)glMapBufferARB(GL_PIXEL_PACK_BUFFER_ARB, GL_READ_ONLY_ARB);

	    memcpy(pBufferSRC, color, bufSize);
	    glUnmapBufferARB(GL_PIXEL_PACK_BUFFER_ARB);//}bsO

	    glBindBufferARB(GL_PIXEL_PACK_BUFFER_ARB, 0);
	    glDeleteBuffersARB(1, &pboID);


	    //glGetTexImage( GL_TEXTURE_2D, 0, channelType, GL_UNSIGNED_BYTE, pBufferSRC);
	}
	if (pBufferSRC == NULL)
	    printf("*pBufferSRC == NULL\n");
    }

    mbSaveThreadActive[mActiveThread]   = TRUE;
    mbSaveThreadFinished[mActiveThread] = FALSE;
    mpCThreadSave[mActiveThread]->Run();

    // ǂŕۑ𑖂炷ƕsɂȂ₷̂őҋ@
    if (mCMultiThread.mBody == 0){
	if (mbSaveThreadActive[mActiveThread] == TRUE){
	    while (mbSaveThreadFinished[mActiveThread] == FALSE)
		wxMilliSleep(3);
	}
    }
    
    mActiveThread++;
    if (mActiveThread >= _IMAGE_OUT_MAX_THREAD) mActiveThread = 0;
    
    glDisable(GL_TEXTURE_2D);
    return TRUE;
}

bool ImageIO_cls::
SelectFile(wxString strDefaultDir, wxWindow *pC){

    GpCFileDialog->SetMessage(_("Select a image file to save"));
    GpCFileDialog->SetWildcard(wxT("image(*.png;*.jpg;*.tiff)|*.png;*.jpg;*.tiff|png (*.png)|*.png|jpg (*.jpg)|*.jpg|tiff (*.tiff)|*.tiff"));
    GpCFileDialog->SetStyle(wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
    
    if (strDefaultDir == wxT("None"))
	GpCFileDialog->SetInitPath(wxT(""), wxT("snapshot.png"));
    else if (strDefaultDir != wxT("")){
	GpCFileDialog->SetInitPathFull(strDefaultDir);
    }
    
    bool bRetval = (GpCFileDialog->ShowModal() == wxID_OK);
    if (bRetval == TRUE) mstrFile = GpCFileDialog->GetPath();
    
    return bRetval;
}

void ImageIO_cls::
SetFilenameStereo(int eyeType){
    SplitPath_cls CSplit(mstrFile);

    wxString strLast = CSplit.mstrFilename.Right(1);
    if (strLast.IsSameAs(wxT("L")) || strLast.IsSameAs(wxT("R")))
	CSplit.mstrFilename.RemoveLast();
    
    switch(eyeType){
    case _STEREO_LFT:
	CSplit.mstrFilename.Append(wxT("L"));
	break;
    case _STEREO_RGT:
	CSplit.mstrFilename.Append(wxT("R"));
	break;
    }
			 
    mstrFile = CSplit.GetPathStr();
}

void ImageIO_cls::
ReadStrLine(wxString strLine){
    mCConfList.ReadStrLineAll(strLine);
}

wxString ImageIO_cls::
Serialize(){
    return mCConfList.SerializeAll();
}

ImageIO_cls GCImgIO;

//int ImageIO_cls::msJPEGQuality = 95;
