// -*- c -*- 
// (setq c-basic-offset 4)
// "clsImgSequence.h"

#include "clsImgSequence.h"
#include "clsImageIO.h"
#include "../MYGL/clsPositionAndDirection.h"
#include "../MYGL/clsFBO.h"
#include "../MYGL/clsGLInfo.h"
#include "../TIMELINE/clsTimeCtrl.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemConstParam.h"
#include "../WIDGETCTRL/EX/clsFileDialogEX.h"
extern wxFileDialogEX*   GpCFileDialog;
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"

#include "../../clsFrameMain.h"
#include "../../clsMain.h"

#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

#include "../COMMON/CommonDefinitions.h"

#include <wx/file.h>

#ifdef WIN32
#define DELIM L"\\"
#define DELIMWX wxT("\\")
#else
#define DELIM L"/"
#define DELIMWX wxT("/")
#endif

BEGIN_EVENT_TABLE(ImgSequenceGUI_cls, wxDialog)
    EVT_BUTTON(btnImgSeqRender,    ImgSequenceGUI_cls::OnButtonRender)
    EVT_BUTTON(btnImgSeqSnap,      ImgSequenceGUI_cls::OnButtonSnap)
    EVT_BUTTON(btnImgSeqInitSeq,   ImgSequenceGUI_cls::OnButtonInitSeq)
    EVT_BUTTON(btnImgSeqReference, ImgSequenceGUI_cls::OnButtonReference)
    EVT_BUTTON(wxID_CANCEL, ImgSequenceGUI_cls::OnButtonClose)
END_EVENT_TABLE()

ImgSequenceGUI_cls::
ImgSequenceGUI_cls(ImgSequence_cls *pCParent, wxWindow *pC,
    		   wxPoint pos, wxSize size) :
    wxDialogEXKey(pC, wxID_ANY, _("Save Image Sequence"),
		  pos, size){
    mpCParent = pCParent;
    //mspCThis = this;
    //SetScreenRectPos(&pCParent->mCScreenRect);
    SetPCPosInfoAtClose(&pCParent->mCDialogPos);

    wxIcon icon(wxT("ICONS/ImgSequence.png"), wxBITMAP_TYPE_PNG);
    SetIcon(icon);

    // TCU[
    wxBoxSizer       *pBoxSizerSrc     = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer       *pBoxSizerParam   = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer       *pBoxSizerExpect  = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer       *pBoxSizerButtons = new wxBoxSizer(wxHORIZONTAL);
 
    SetMNumImgSeqParam(pBoxSizerParam);

    mpCTextExpect = new wxStaticText(this, wxID_ANY, wxT("Expected Time"));

    pBoxSizerExpect->Add(mpCTextExpect, 1, wxEXPAND);
    
#ifdef WIN32
    int buttonWidth=100;
#else
    int buttonWidth=140;
#endif
    mpCButtonRender =
	new wxButtonEX(this, btnImgSeqRender, wxT("Render Anim"),
		       wxDP, wxSize(buttonWidth, -1));
    mpCButtonSnap = 
	new wxButtonEX(this, btnImgSeqSnap, wxT("Render Snap"),
		       wxDP, wxSize(buttonWidth, -1));

    mpCGauge = 
	new wxGauge(this, wxID_ANY, 100, wxDPS, wxGA_HORIZONTAL);
    
    wxButtonEX *pCButtonCancel =
	new wxButtonEX(this, wxID_CANCEL, wxT("Close"),
		       wxDP, wxSize(buttonWidth, -1));

    pBoxSizerButtons->Add(mpCButtonRender, 0, wxEXPAND);
    pBoxSizerButtons->Add(mpCButtonSnap  , 0, wxEXPAND);
    pBoxSizerButtons->Add( 4, -1, 0, wxEXPAND);
    pBoxSizerButtons->Add(mpCGauge, 1, wxEXPAND);
    pBoxSizerButtons->Add( 4, -1, 0, wxEXPAND);
    pBoxSizerButtons->Add(pCButtonCancel, 0, wxEXPAND);
    
    pBoxSizerSrc->Add(pBoxSizerParam, 0, wxEXPAND);
    pBoxSizerSrc->Add(pBoxSizerExpect, 0, wxEXPAND);
    pBoxSizerSrc->Add(pBoxSizerButtons, 0, wxEXPAND);

    SetSizerAndFit(pBoxSizerSrc);

    mpCButtonRender->SetFocus();
}

void ImgSequenceGUI_cls::
SetMNumImgSeqParam(wxBoxSizer *pBoxSizer){
    mpCMNumImgSeqParam = new MultiInputPanel_cls(this);

    MultiInputPanel_cls *pCurr = mpCMNumImgSeqParam;
    pCurr->mUpdateBit = _COMMAND_UPDATE_DISPLAY_IMGSEQ;
    
    int boxWidth;
    pCurr->mTextWidth = 160/*-1*/;
    pCurr->mItemWidth = 90;

    boxWidth = 3;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth,
			  _("Preview (W, H, V)"));
    pCurr->CombinePointer(_ITEM_INTEGER, &mpCParent->mClipWidthMax, wxT(""), FALSE);
    pCurr->CombinePointer(_ITEM_INTEGER, &mpCParent->mClipHeightMax, wxT(""), FALSE);
    pCurr->CombinePointer(_ITEM_FLOAT, &mpCParent->mAngleOrg, wxT(""), FALSE);

    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth,
			  _("Output Size"));

    wxString listTemplate[_IMG_SIZE_MAX];
    for (int i = 0; i < _IMG_SIZE_MAX; i++) listTemplate[i] = GetStrGeom(i);
    pCurr->CombinePointer(_ITEM_INTEGER_SPIN, &mpCParent->mClipWidth);
    pCurr->SetRange(_ITEM_LASTITEM, 0, 32768);
    pCurr->CombinePointer(_ITEM_INTEGER_SPIN, &mpCParent->mClipHeight);
    pCurr->SetRange(_ITEM_LASTITEM, 0, 32768);
    mIndexGeomTemplate = pCurr->CombinePointerList
	(&mGeometryTemplate, _IMG_SIZE_MAX, listTemplate);
    
    boxWidth = 3;

    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth,
			  _("Initial sequence"));
    pCurr->CombinePointer(_ITEM_INTEGER, &mpCParent->mCInitNum.mBody);
    pCurr->CombinePointerButton(btnImgSeqInitSeq, _("Calc. Auto"));
    pCurr->CombinePointerButton(btnImgSeqReference, _("Reference"));
    
    pCurr->mItemWidth = -1;
    pCurr->CombinePointer(_ITEM_STRING, &mpCParent->mCFileSequence.mstrFormat);

    pCurr->mItemWidth = 90;
    mIndexBlur = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth,
			  _("Motion Blur (Length/Quality)"));
    pCurr->CombinePointer(_ITEM_BOOL, &mpCParent->GetPCTimeCtrl()->mCbUseMotionBlur.mbBody, _("Use"));
    pCurr->CombinePointer(_ITEM_FLOAT, &mpCParent->GetPCTimeCtrl()->mCBlurCtrl.mBlurLength);
    pCurr->CombinePointer(_ITEM_INTEGER, &mpCParent->GetPCTimeCtrl()->mCBlurCtrl.mBlurQuality);
    pCurr->GetLastItem().SetRange(2, 16, 1);

    mIndexTile = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth,
				       _("Tiling (Res./Index)"));
    pCurr->CombinePointer(_ITEM_BOOL,    &mpCParent->mbTiling, _("Use"));
    pCurr->CombinePointer(_ITEM_INTEGER, &mpCParent->mTileResolution);
    pCurr->CombinePointer(_ITEM_INTEGER, &mpCParent->mTileIndex);

    /*
    pCurr->mItemWidth = -1;
    wxString listChannel[2] = { wxT("RGB"), wxT("RGBA") };
    pCurr->CombinePointerList
	(&mpCParent->mCImageType.mBody, 2, listChannel, _("Igmage channel"));
    */
    
    pCurr->mMinimumWidth = 400;

    pCurr->FinishSetting(pBoxSizer);

    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_CHECKBOX);
    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_COMBOBOX);

    HideAndShow();
}

#include "../MYAPI/MsgBox.h"

void ImgSequenceGUI_cls::
OnButtonInitSeq(wxCommandEvent &event){
    mpCParent->SetAutoInit();
    mpCMNumImgSeqParam->UpdateValues();
}

void ImgSequenceGUI_cls::
OnButtonReference(wxCommandEvent &event){
    GpCFileDialog->SetMessage(_("Select a image file to save"));
    GpCFileDialog->SetWildcard(wxT("image(*.png;*.jpg;*.tif)|*.png;*.jpg;*.tif|png (*.png)|*.png|jpg (*.jpg)|*.jpg|tiff (*.tif)|*.tif"));
    GpCFileDialog->SetStyle(wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
    GpCFileDialog->SetInitPathFull(mpCParent->mCFileSequence.GetPathStr(_FN_FULLPATH));

    if (GpCFileDialog->ShowModal() == wxID_OK){
	GCUndoList.PushNewCommand
	    (new CommandSetValue_cls(&mpCParent->mCFileSequence.mstrFormat, GpCFileDialog->GetPath(), _COMMAND_UPDATE_DISPLAY_IMGSEQ),
	     new CommandSetValue_cls(&mpCParent->mCFileSequence.mstrFormat, mpCParent->mCFileSequence.mstrPath, _COMMAND_UPDATE_DISPLAY_IMGSEQ));
	GCUndoList.ExecCurrent();
	mpCParent->mCFileSequence.Set(mpCParent->mCFileSequence.mstrFormat, 0, 1);
	mpCMNumImgSeqParam->UpdateValues();
    }
}

void ImgSequenceGUI_cls::
OnButtonRender(wxCommandEvent &event){
    if (mpCParent->mbInRendering == TRUE){
	AbortRender(); return; }
    
    mpCParent->mCStopWatch.Start(0);

    mpCMNumImgSeqParam->AssignBase();
    
    mpCParent->mCFileSequence.Set(mpCParent->mCFileSequence.GetFormat(), 0, 1);
    mpCMNumImgSeqParam->UpdateValues();

    mpCParent->mpCFrameMainLink->mCLastOpenedFileOUT.mstrBody = 
	mpCParent->mCFileSequence.GetFormat();

    if (mpCParent->mCFileSequence.mDigitLength == -1){
	if (MsgBoxYesno(this, _("Sequence Format error"), 
			_("Not Sequence. Append 4-digit tag ?")) == FALSE)
	    return;
	mpCParent->mCFileSequence.AppendDefaultSequenceTag();
	mpCMNumImgSeqParam->UpdateValues();
    }

    if (mpCParent->mCInitNum.mBody !=
	- mpCParent->GetPCTimeCtrl()->GetOffsetForImgSeq(0, 0)){
	wxString strMessage;
	strMessage << _("Initial Number is set to ") << mpCParent->mCInitNum.mBody << ENDL;
	strMessage << _("Working area starts at ") << mpCParent->GetPCTimeCtrl()->mCWAMin.mBody << ENDL;
	strMessage << _("Recommended Initial Number is ") << - mpCParent->GetPCTimeCtrl()->GetOffsetForImgSeq(0, 0) << ENDL;
	strMessage << _("Change Initial Number ?");
	if (MsgBoxYesno(this, _("Confirmation"), strMessage) == TRUE)
	    mpCParent->SetAutoInit();
    }

    // !!_FIX_ME_!! XeÎƂ̃`FbN܂
    if (wxFile::Exists(mpCParent->mCFileSequence.GetModifiedStr(mpCParent->mCInitNum.mBody, 0)) == TRUE){
	wxString strMessage;
	strMessage << mpCParent->mCFileSequence.GetModifiedStr(mpCParent->mCInitNum.mBody, 0) << ENDL;
	strMessage << _("This file already exists. Overwrite it ?");
	if (MsgBoxYesno(this, _("Confirmation"), strMessage) == FALSE)
	    return;
    }
	
    SendMenuEvent(GetParent(), menuItemSaveImgSeqBegin);
    mpCButtonRender->SetLabel(wxT("Abort Anim"));
    mpCButtonRender->SetBackgroundColour(wxColour(wxT("RED")));
    mpCButtonRender->SetForegroundColour(wxColour(wxT("WHITE")));
    mpCParent->mbInRendering = TRUE;
}

void ImgSequenceGUI_cls::
OnButtonSnap(wxCommandEvent &event){
    if (mpCParent->mbInRendering == TRUE) return;
    mpCMNumImgSeqParam->AssignBase();

    mpCParent->mCFileSequence.Set(mpCParent->mCFileSequence.GetFormat(), 0, 1);
    mpCMNumImgSeqParam->UpdateValues();

    mpCParent->mpCFrameMainLink->mCLastOpenedFileOUT.mstrBody = 
	mpCParent->mCFileSequence.GetFormat();
    
    SendMenuEvent(GetParent(), menuItemSaveImgSeqBegin);
    mpCParent->mbInRendering = TRUE;
    mpCParent->mbSnapMode = TRUE;
    mpCParent->GetPCTimeCtrl()->SetResetPosition();
}

void ImgSequenceGUI_cls::
OnButtonClose(wxCommandEvent &event){
    mpCMNumImgSeqParam->UpdateValues();
    CloseEXKey();
}

void ImgSequenceGUI_cls::
AbortRender(){
    SendMenuEvent(GetParent(), menuPlayStop);
    OutputFinish();
}

void ImgSequenceGUI_cls::
OutputFinish(){
    mpCButtonRender->SetLabel(wxT("Render Anim"));
    mpCButtonRender->SetBackgroundColour(wxNullColour);
    mpCButtonRender->SetForegroundColour(wxNullColour);
    mpCParent->mbInRendering = FALSE;
    mpCParent->UpdateGauge();
}

void ImgSequenceGUI_cls::
Geometry(){
    MultiInputPanel_cls* pCurr = mpCMNumImgSeqParam;
    int geometry = pCurr->mVCItemList[mIndexGeomTemplate].GetValueInt();
    
    SetGeometry(geometry, &mpCParent->mClipWidth, &mpCParent->mClipHeight);
    
    if (geometry == 0){
	pCurr->mVCItemList[mIndexGeomTemplate-1].GetPCwxWindow()->Enable();
	pCurr->mVCItemList[mIndexGeomTemplate-2].GetPCwxWindow()->Enable();
    }
    else{
	pCurr->mVCItemList[mIndexGeomTemplate-1].GetPCwxWindow()->Disable();
	pCurr->mVCItemList[mIndexGeomTemplate-2].GetPCwxWindow()->Disable();
    }

    pCurr->UpdateValues();
}

void ImgSequenceGUI_cls::
HideAndShow(){
    MultiInputPanel_cls* pCurr = mpCMNumImgSeqParam;
    bool bBlur = mpCParent->GetPCTimeCtrl()->mCbUseMotionBlur.mbBody;

    pCurr->GetPCwxWindow(mIndexBlur+2)->Enable(bBlur);
    pCurr->GetPCwxWindow(mIndexBlur+3)->Enable(bBlur);

    pCurr->GetPCwxWindow(mIndexTile+2)->Enable(mpCParent->mbTiling);
    pCurr->GetPCwxWindow(mIndexTile+3)->Enable(mpCParent->mbTiling);
}

void ImgSequenceGUI_cls::
HookFuncHS(int id, void *pArg){
    HideAndShow();
}

void ImgSequenceGUI_cls::
HookFunc01(int id, void *pArg){
    Geometry();
}

// ----

ImgSequence_cls::
ImgSequence_cls(Main_cls *pCMainLink) :
    mCFileSequence(wxGetCwd()+DELIMWX+wxT("Preview%04d.png"))
{
    mClipWidthMax = 640;
    mClipHeightMax = 480;
    mAngleOrg = 35;
    mClipWidth = 640;
    mClipHeight = 480;
    mOffsetX = 0;
    mOffsetY = 0;
    mbInRendering = FALSE;
    mbSnapMode    = FALSE;
    mbTiling      = FALSE;
    mTileResolution = 2;
    mTileIndex      = 0;
    mpCGUI = NULL;
    mpCMainLink = pCMainLink;
    mpCFrameMainLink = pCMainLink->mpCFrame;
    SetToDefault();
    //InitGUI(pC); // GUĨCX^X
}

void ImgSequence_cls::
SetGeom(int width, int height, float angle, bool bSetTrim){
    mClipWidthMax  = width;
    mClipHeightMax = height;
    mAngleOrg = angle;
    if (bSetTrim == TRUE){
	mClipWidth = width;
	mClipHeight = height;
    }
    if (mpCGUI != NULL && mpCGUI->IsShownOnScreen())
	UpdateItems();
}

TimeCtrl_cls* ImgSequence_cls::
GetPCTimeCtrl(){
    return mpCMainLink->mpCTime;
}

void ImgSequence_cls::
CalcOffset(int widthOrg, int heightOrg){
    mOffsetX = (widthOrg - mClipWidth) / 2;
    mOffsetY = (heightOrg - mClipHeight) / 2;
}

void ImgSequence_cls::
SetAutoInit(){
    mCInitNum.mBody = - GetPCTimeCtrl()->GetOffsetForImgSeq(0, 0);
}

wxString ImgSequence_cls::
GetWXStrModifiedFilename(){
    return mCFileSequence.GetModifiedStr(mCurrentOffset + mCInitNum.mBody,
					 mCurrentStereo);
}

bool ImgSequence_cls::
IsSizeValid(GLInfo_cls *pCGLInfo){
    if (mClipWidth  > pCGLInfo->mMaxTextureSize) return FALSE;
    if (mClipHeight > pCGLInfo->mMaxTextureSize) return FALSE;
    return TRUE;
}

void ImgSequence_cls::
SetDefaultPath(wxString strPath){
    if (strPath == wxT("") || strPath == wxT("None")) return;
    SplitPath_cls CPath(strPath);
    if (CPath.GetPathStr(_FN_FNAME).Matches(wxT("snapshot"))){
	CPath.mstrFilename.Replace(wxT("snapshot"), wxT("preview%04d"));
	strPath = CPath.GetPathStr(_FN_FULLPATH);
    }
    mCFileSequence.Set(strPath, 0, 1);
}

void ImgSequence_cls::
UpdateGauge(){
    if (mbInRendering == FALSE){
	mpCGUI->mpCGauge->SetValue(0); mpCGUI->mpCGauge->Disable(); return;
    }

    if (mpCGUI->mpCGauge->IsEnabled() == FALSE) mpCGUI->mpCGauge->Enable();
    double percentage = GetPCTimeCtrl()->GetPercentage();
    mpCGUI->mpCGauge->SetValue((int)percentage);

    if (percentage < 0.1 && mCStopWatch.Time() < 3000) return;

    wxString str;

    str << (int)percentage << wxT("% : ");

    int ms = mCStopWatch.Time();
    int sec = ms / 1000;
    int min = sec / 60; sec = sec % 60;
    int hour = min / 60; min = min % 60;
    int day = hour / 24; hour = hour % 24;

    if (day > 0) str << day << wxT("day");
    if (hour > 0) str << hour << wxT("hour");
    if (min > 0) str << min << wxT("min");
    str << sec << wxT("sec : expected finish time = ");

    int expectms = ms / percentage * 100;
    int expectsec = expectms / 1000;
    int expectmin = expectsec / 60; expectsec = expectsec % 60;
    int expecthour = expectmin / 60; expectmin = expectmin % 60;
    int expectday = expecthour / 60; expecthour = expecthour % 24;
    if (expectday > 0) str << expectday << wxT("expectday");
    if (expecthour > 0) str << expecthour << wxT("hour");
    if (expectmin > 0) str << expectmin << wxT("min");
    str << expectsec << wxT("sec");

    mpCGUI->mpCTextExpect->SetLabel(str);
}

bool ImgSequence_cls::
SaveImageFromGL(ImageIO_cls* pCImageIO, int widthOrg, int heightOrg, int offset, int stereo){
    // CHECK_ME!! łQ`݂̖łĂH
    if (mCurrentStereo == stereo && offset < mCurrentOffset)
	return TRUE;

    mCurrentStereo = stereo;
    mCurrentOffset = offset;
    bool retVal = FALSE;
    CalcOffset(widthOrg, heightOrg);
    retVal = pCImageIO->SaveImageFromGL(mClipWidth, mClipHeight,
					GetWXStrModifiedFilename(),
					mOffsetX, mOffsetY);

    UpdateGauge();

    return retVal;
}

bool ImgSequence_cls::
SaveImageFromFBO(ImageIO_cls* pCImageIO, FBO_cls *pCFBO,
		 FBO_cls *pCFBOOverlay, int imgType, 
		 int widthOrg, int heightOrg, 
		 int offset, int stereo, bool bSideBySide){
    // CHECK_ME!! łQ`݂̖łĂH
    if (mCurrentStereo == stereo && offset < mCurrentOffset)
	return TRUE;

    mCurrentStereo = stereo;
    mCurrentOffset = offset;
    bool retVal = FALSE;
    CalcOffset(widthOrg, heightOrg);

    retVal = pCImageIO->SaveImageFromFBO
	(pCFBO, pCFBOOverlay, mClipWidth, mClipHeight, 
	 GetWXStrModifiedFilename(), imgType, bSideBySide);

    UpdateGauge();

    return retVal;
}

void ImgSequence_cls::
SetToDefault(){
    mCInitNum.SetItem(L"ImgSequenceInitNum=", L"0");
    //mCImageType.SetItem(L"ImgSequenceImageType=", L"0");
}

void ImgSequence_cls::
InitGUI(FrameMain_cls *pCParentFrameLink){
    mpCGUI = new ImgSequenceGUI_cls(this, pCParentFrameLink, mCDialogPos, wxDS);
    SetPDialog(mpCGUI);
}

void ImgSequence_cls::
UpdateItems(bool bForce){
    if (mpCGUI != NULL){
	mpCGUI->mpCMNumImgSeqParam->UpdateValues(_MN_ALLITEM);
	mpCGUI->HideAndShow();
    }
}

void ImgSequence_cls::
RenderFrameBoxInRender(int width, int height, GLPerspective_cls CGLPerspective){
    glPushMatrix();
    PositionAndDirection_cls CPos(0, 0, -1);

    CGLPerspective.mbPerspective = FALSE;
    CGLPerspective.SetPerspectiveOnGL(&CPos);
    double rateX = mClipWidth/width;
    double rateY = mClipHeight/height;
    double aspect = CGLPerspective.GetAspectRatio();
    glBegin(GL_LINE_LOOP);
    glVertex2f(-0.5 * aspect * rateX, -0.5 * rateY);
    glVertex2f(+0.5 * aspect * rateX, -0.5 * rateY);
    glVertex2f(+0.5 * aspect * rateX, +0.5 * rateY);
    glVertex2f(-0.5 * aspect * rateX, +0.5 * rateY);
    glEnd();
    glPopMatrix();
}

void ImgSequence_cls::
OutputFinish(){
    if (mpCGUI != NULL)
	mpCGUI->OutputFinish();
}
