// "clsHDF5GUI.h"
// (setq c-basic-offset 4)

#include "clsHDF5Common.h"

#include "clsHDF5GUI.h"

#ifdef _USE_HDF5

#include "clsAttribItem.h"
#include "clsDataSetAndName.h"
#include "clsReadHDF5Data.h"

#include "../COMMON/CommonDefinitions.h"

HDF5GUI_cls::
HDF5GUI_cls(wxWindow *pCParent, int width){

    mpCStaticTextAttr = new wxStaticText(pCParent, wxID_ANY, _("Attribute"));
    mpCStaticTextBody = new wxStaticText(pCParent, wxID_ANY, _("Data Structure"));

    long style = wxLC_REPORT | wxSUNKEN_BORDER;
    mpCListCtrlAttr = new wxListCtrl
	(pCParent, wxID_ANY, wxDP, wxSize(width, 180), style);
    SetColumnAttr(mpCListCtrlAttr);
    
    mpCListCtrlBody = new wxListCtrl
	(pCParent, wxID_ANY, wxDP, wxSize(width, 180), style);
    SetColumnBody(mpCListCtrlBody);
}

void HDF5GUI_cls::
SetColumnAttr(wxListCtrl* pCList){
    pCList->InsertColumn(0, wxEmptyString, wxLIST_FORMAT_CENTER, 30);
    pCList->InsertColumn(1, _("Owner"), wxLIST_FORMAT_LEFT, 60);
    pCList->InsertColumn(2, _("Name"), wxLIST_FORMAT_LEFT, 94);
    pCList->InsertColumn(3, _("Value"), wxLIST_FORMAT_LEFT, 100);
}

void HDF5GUI_cls::
SetColumnBody(wxListCtrl* pCList){
    pCList->InsertColumn(0, wxEmptyString, wxLIST_FORMAT_CENTER, 30);
    pCList->InsertColumn(1, _("Group"), wxLIST_FORMAT_LEFT, 70);
    pCList->InsertColumn(2, _("Data"), wxLIST_FORMAT_LEFT, 60);
    pCList->InsertColumn(3, _("Information"), wxLIST_FORMAT_LEFT, 124);
}

bool HDF5GUI_cls::
CheckDataHDF5(wxString strPath, GrammerHDF5_cls &CGrammerHDF5){
    ReadHDF5Data_cls CRead;
    CRead.ReadHDF5Data(strPath);
    UpdateListAttr(CRead.mVpCAttribItem);
    UpdateListBody(CRead.mVpCDataSetAndName);
    bool bRet = CRead.CheckGrammer(CGrammerHDF5);
    if (bRet == FALSE) CRead.ShowErrorMessage();
    return bRet;
}

void HDF5GUI_cls::
UpdateListAttr(std::vector < AttribItem_cls* > &VpCAttribItem){
    mpCListCtrlAttr->DeleteAllItems();
    int size = (int)VpCAttribItem.size();
    wxString strParentPrev;
    for (int i = 0; i < size; i++){
	wxString strParent = VpCAttribItem[i]->mstrParent;
	if (strParent == strParentPrev) strParent = wxT("");
	AddListItemBody(mpCListCtrlAttr, -1, 
			strParent, 
			VpCAttribItem[i]->mstrName, 
			VpCAttribItem[i]->mstrValue);
	strParentPrev = VpCAttribItem[i]->mstrParent;
    }
}

void HDF5GUI_cls::
UpdateListBody(std::vector < DataSetAndName_cls* > &VpCDataSetAndName){
    mpCListCtrlBody->DeleteAllItems();
    int size = (int)VpCDataSetAndName.size();
    for (int i = 0; i < size; i++){
	wxString strGroup = VpCDataSetAndName[i]->mstrType;
	wxString strName = wxString::FromAscii(VpCDataSetAndName[i]->mstdstrName.c_str());
	if (strGroup.IsSameAs(wxT("DATASET"))) strGroup = wxT("");
	if (strGroup.IsSameAs(wxT("GROUP"))){
	    strGroup = strName; strName = wxT("");
	}
	AddListItemBody(mpCListCtrlBody, -1, 
			strGroup, strName,
			VpCDataSetAndName[i]->mstrDatasetInfo);
    }
}
    
void HDF5GUI_cls::
AddListItemBody(wxListCtrl *pCList, wxInt32 nID, wxString strType, wxString strName, wxString strInfo){
    wxListItem listItem;

    if (nID == -1)
	nID = pCList->GetNextItem(-1, wxLIST_NEXT_ALL, 
				  wxLIST_STATE_SELECTED);
    
    if (nID == -1)// ǉ
	nID = pCList->GetItemCount();
    else
	;//nID++; //Î̎
    
    listItem.SetId(nID);
    listItem.SetMask(wxLIST_MASK_DATA | wxLIST_MASK_STATE);
    listItem.SetData(1);
    
    wxString strNum; strNum << pCList->GetItemCount();
    listItem.SetText(strNum);
    pCList->InsertItem(listItem);

    pCList->SetItem(nID, 1, strType);
    pCList->SetItem(nID, 2, strName);
    pCList->SetItem(nID, 3, strInfo);
}

#else
bool HDF5GUI_cls::
CheckDataHDF5(wxString strPathm, GrammerHDF5_cls &CGrammerHDF5){
    return FALSE;
}
#endif
