
#include "clsLuaBase.h"

#include "../MYGL/clsPositionAndDirection.h"
#include "../SCENEOBJ/clsSceneObj.h"
#include "../PARTICLEDATA/clsSnapData.h"

#include "../PARTICLEDATA/clsGLDrawData.h"

#include "../COMMON/CommonDefinitions.h"

static wxMutex GCMutexLuaBase;

LuaBase_cls::
LuaBase_cls(){
    mpLua = NULL;
}

LuaBase_cls::
~LuaBase_cls(){
    CloseLuaBase();
}

void LuaBase_cls::
OpenLuaBase(wxString strFunction){
    if (mpLua == NULL){
	mpLua = luaL_newstate();
	luaL_openlibs(mpLua);
	lua_newtable(mpLua);
	luaL_setfuncs(mpLua, LuaBase_cls::CLibTable, 0);
	lua_setglobal(mpLua, "C");
    }
    int ret = luaL_dostring(mpLua, strFunction.ToAscii());
}

void LuaBase_cls::
CloseLuaBase(){
    if (mpLua != NULL){
	lua_close(mpLua);
	mpLua = NULL;
    }
}

void LuaBase_cls::
ClearLuaStack(){
    int stackSize = lua_gettop(mpLua);
    lua_pop(mpLua, stackSize); // SNA
}

void LuaBase_cls::
CheckErrorMessage(lua_State *pLua, wxString strTag){
    const char *err = lua_tostring(pLua, -1);

    wxMutexLocker CLock(GCMutexLuaBase);
    msstrErr << strTag << wxT("::") << wxString::FromAscii(err) << ENDL;
}

//---

luaL_Reg LuaBase_cls::CLibTable[] = {
    {"GetObjectInfo", LuaBase_cls::GetObjectInfo },
    { NULL, NULL} };

int LuaBase_cls::
GetPosInfo(lua_State *L, PositionAndDirection_cls *pCPos, wxString strTag){
    if (strTag.IsSameAs(wxT("position"))){
	lua_pushnumber(L, pCPos->mPos[0]);
	lua_pushnumber(L, pCPos->mPos[1]);
	lua_pushnumber(L, pCPos->mPos[2]);
	return 3;
    }
    else if (strTag.IsSameAs(wxT("direction"))){
	lua_pushnumber(L, pCPos->mTgt[0]);
	lua_pushnumber(L, pCPos->mTgt[1]);
	lua_pushnumber(L, pCPos->mTgt[2]);
	return 3;
    }
    else if (strTag.IsSameAs(wxT("top"))){
	lua_pushnumber(L, pCPos->mTop[0]);
	lua_pushnumber(L, pCPos->mTop[1]);
	lua_pushnumber(L, pCPos->mTop[2]);
	return 3;
    }
    else if (strTag.IsSameAs(wxT("right"))){
	lua_pushnumber(L, pCPos->mRgt[0]);
	lua_pushnumber(L, pCPos->mRgt[1]);
	lua_pushnumber(L, pCPos->mRgt[2]);
	return 3;
    }
    return 0;
}

int LuaBase_cls::
GetSnapshotPos(lua_State* L, int index){
    if (mspCSnapShotLink == NULL) return 0;
    if (index >= mspCSnapShotLink->mPNumBlock || index < 0)
	return 0;
    if (mspCSnapShotLink != GLDrawData_cls::mspCSnapShot) return 0;
    lua_pushnumber(L, mspCSnapShotLink->mpX[index][0]);
    lua_pushnumber(L, mspCSnapShotLink->mpX[index][1]);
    lua_pushnumber(L, mspCSnapShotLink->mpX[index][2]);
    return 3;
}

int LuaBase_cls::
GetSnapshotValue1(lua_State* L, int index){
    if (mspCSnapShotLink == NULL) return 0;
    if (index >= mspCSnapShotLink->mPNumBlock || index < 0)
	return 0;
    lua_pushnumber(L, mspCSnapShotLink->mpValue1[index]);
    return 1;
}

int LuaBase_cls::
GetObjectInfo(lua_State* L){
    int num = lua_gettop(L);
    wxString strFrom1; strFrom1 << wxString::FromAscii(lua_tostring(L, 1));
    wxString strFrom2; strFrom2 << wxString::FromAscii(lua_tostring(L, 2));
    wxString strFrom3; strFrom3 << wxString::FromAscii(lua_tostring(L, 3));
    long index; 
    if (strFrom3.IsSameAs(wxT("i"))) index = lua_tonumber(L, 3);
    else                             strFrom3.ToLong(&index);
    
    lua_pop(L, lua_gettop(L));

    /*
    for (int i = 0; i < msVpCSceneObjLink.size(); i++){
	if (strFrom1.IsSameAs(msVstrSceneObjTag[i])){
	    SceneObj_cls *pCSceneObj = msVpCSceneObjLink[i];
	    return GetPosInfo(&pCSceneObj->mCCurrentObj, strFrom2);
	}
    }
    */
    
    for (int i = 0; i < msVpCPositionLink.size(); i++){
	if (strFrom1.IsSameAs(msVstrPositionTag[i])){
	    return GetPosInfo(L, msVpCPositionLink[i], strFrom2);
	}
    }

    if (strFrom1.IsSameAs(wxT("particle"))){
	
	
	    
	if (strFrom2.IsSameAs(wxT("position")))
	    return GetSnapshotPos(L, index);
	if (strFrom3.IsSameAs(wxT("value1")))
	    return GetSnapshotValue1(L, index);
    }

    return 0;
}

/*
void LuaBase_cls::
PushSceneObjLink(SceneObj_cls *pCSceneObj, wxString strTag){
    msVpCSceneObjLink.push_back(pCSceneObj);
    msVstrSceneObjTag.push_back(strTag);
}
*/
void LuaBase_cls::
SetPositionLink(PositionAndDirection_cls *pCPos, wxString strTag){
    for (int i = 0; i < msVpCPositionLink.size(); i++)
	if (strTag.IsSameAs(msVstrPositionTag[i])){
	    msVpCPositionLink[i] = pCPos;
	    return;
	}
	
    msVpCPositionLink.push_back(pCPos);
    msVstrPositionTag.push_back(strTag);
}

void LuaBase_cls::
SetSnapshotLink(SnapData_cls *pCSnapShotLink){
    mspCSnapShotLink = pCSnapShotLink;
}

//std::vector < SceneObj_cls* > LuaBase_cls::msVpCSceneObjLink;
std::vector < PositionAndDirection_cls* > LuaBase_cls::msVpCPositionLink;
//std::vector < wxString > LuaBase_cls::msVstrSceneObjTag;
std::vector < wxString > LuaBase_cls::msVstrPositionTag;
SnapData_cls* LuaBase_cls::mspCSnapShotLink;


wxString LuaBase_cls::msstrErr;
