
#include "clsDrawOption.h"
#include "clsDrawOptionGUI.h"

#include "../LUA/clsLuaBase.h"

#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemConstParam.h"

#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/EX/clsTextEntryDialogEX.h"

#include "../COMMON/CommonDefinitions.h"

ThreadLuaCalassify_cls::
ThreadLuaCalassify_cls() : wxThread(wxTHREAD_DETACHED) {
}

bool ThreadLuaCalassify_cls::
SetParameters(bool *pbFinishFlag, lua_State **ppLua,
	      double doubleIndex, int frame,
	      int ini, int fin,
	      int *pID, unsigned char *pTypeOrg, unsigned char *pType){

    mpbFinishFlag = pbFinishFlag;
    mppLua = ppLua;
    mDoubleIndex = doubleIndex;
    mFrame = frame;
    mIni = ini;
    mFin = fin;
    mpID = pID;
    mpTypeOrg = pTypeOrg;
    mpType = pType;

    if ( Create() != wxTHREAD_NO_ERROR) return FALSE;
    return TRUE;
}

void ThreadLuaCalassify_cls::
CloseLua(){
    lua_close(*mppLua);
    *mppLua = NULL;
}

void ThreadLuaCalassify_cls::
CallLua(){
    lua_State *pLua = *mppLua;
    int ini = mIni;
    int fin = mFin;
    int *pID = mpID;
    unsigned char *pTypeOrg = mpTypeOrg;
    unsigned char *pType = mpType;
    
    if (pLua == NULL){
	for (int i = ini; i < fin; i++) pType[i] = 0;
	return;
    }

    for (int i = ini; i < fin; i++){
	lua_getglobal(pLua, "script"); // ֐ς
	lua_pushnumber(pLua, i);
	lua_pushnumber(pLua, pID[i]);
	lua_pushnumber(pLua, pTypeOrg[i]);
	lua_pushnumber(pLua, mDoubleIndex);
	lua_pushnumber(pLua, mFrame);
	
	if (lua_pcall(pLua, 5, 1, 0) != 0){
	    CheckErrorMessage(pLua, wxT("Type Classifiaction::"));
	    CloseLua();
	    for (int j = i; j < fin; j++) pType[j] = 0;
	    return;
	}

	unsigned char type = lua_tonumber(pLua, -1);
	if (type >= _MAX_PARTICLE_TYPE) type = 0;
	pType[i] = type; 
	lua_pop(pLua, 1); // SNA
    }
}

wxThread::ExitCode ThreadLuaCalassify_cls::
Entry(){
    CallLua();
    *mpbFinishFlag = TRUE;
    return 0;
}

// --------------------

DrawOption_cls::
DrawOption_cls() : mpCGUI(NULL){
    mCConfList.Add(&mCDownSize, wxT("DownSizing="), wxT("1"));
    mCConfList.Add(&mCSortOrder, wxT("SortOrder="), wxT("0"));
    mCConfList.Add(&mCClassify, wxT("Classify="), wxT("1"));
    mCConfList.Add(&mCShadow, wxT("Shadow="), wxT("0"));
    mCConfList.Add(&mCstrExpression, wxT("ClassifyExpression="), wxT("return 0"));
    for (int i = 0; i < _MAX_THREAD_LUA; i++) mApLua[i] = NULL;
}

DrawOption_cls::
~DrawOption_cls(){
    if (mpCGUI != NULL) delete mpCGUI;
}

void DrawOption_cls::
InstantiateGUI(wxWindow *pCParent){
    mpCGUI = new DrawOptionGUI_cls(this, pCParent);
}

void DrawOption_cls::
OpenExpressionDialog(){
    wxString strMessage;
    strMessage << wxT("-----") << _("Predefined Variables") << ENDL;
    strMessage << wxT("i = index") << ENDL;
    strMessage << wxT("id = ID") << ENDL;
    strMessage << wxT("type = original type") << ENDL;
    strMessage << wxT("nt = normalized time") << ENDL;
    strMessage << wxT("frame = frame") << ENDL;
    strMessage << wxT("-----") << _("Return Value") << ENDL;
    strMessage << wxT("type") << ENDL;
    
    TextEntryDialogEX_cls CDlg(mpCGUI->mpCMInputPanel, strMessage, _("Expression Editor"), mCstrExpression.mstrBody);
    
    if (CDlg.ShowModal() == wxID_OK){
	wxString strEdit = CDlg.GetValue().Trim(TRUE);
	int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_SNAPSHOT_DATA;
	GCUndoList.PushNewCommand
	    (new CommandSetValue_cls(&mCstrExpression.mstrBody, strEdit, updateBit), 
	     new CommandSetValue_cls(&mCstrExpression.mstrBody, mCstrExpression.mstrBody, updateBit));
	GCUndoList.ExecCurrent();
    }
}

void DrawOption_cls::
OpenLua(){
    if (mCClassify.mBody != _CLASSIFY_EXPRESSION) return;

    wxString strFunction;
    strFunction << wxT("function script(i, id, type, nt, frame)") << ENDL;
    strFunction << mCstrExpression.mstrBody << ENDL;
    strFunction << wxT("end") << ENDL;
    
    for (int i = 0; i < _MAX_THREAD_LUA; i++){
	mApLua[i] = luaL_newstate();
	luaL_openlibs( mApLua[i] );
	lua_newtable(mApLua[i]);
	luaL_setfuncs(mApLua[i], LuaBase_cls::CLibTable, 0);
	lua_setglobal(mApLua[i], "C");
	int ret = luaL_dostring(mApLua[i], strFunction.ToAscii());
    }
}

void DrawOption_cls::
CloseLua(){
    for (int i = 0; i < _MAX_THREAD_LUA; i++){
	if (mApLua[i] != NULL){
	    lua_close(mApLua[i]);
	    mApLua[i] = NULL;
	}
    }
}

void DrawOption_cls::
GetTypeClassified(int num, int *pID, unsigned char *pTypeOrg, unsigned char *pType, double doubleIndex, int frame){

    int threadNum = 4;
    bool *pbFinishFlag = new bool [threadNum];

    ThreadLuaCalassify_cls **ppCCallLua = 
	new ThreadLuaCalassify_cls* [threadNum];
    for (int i = 0; i < threadNum; i++){
	int ini = i*num/threadNum;
	int fin = (i+1)*num/threadNum;
	pbFinishFlag[i] = FALSE;

	ppCCallLua[i] = new ThreadLuaCalassify_cls;
	ppCCallLua[i]->SetParameters(&pbFinishFlag[i], &mApLua[i],
				     doubleIndex, frame,
				     ini, fin,
				     pID, pTypeOrg, pType);
	ppCCallLua[i]->Run();
    }

    for (int i = 0; i < threadNum; i++)
	while (pbFinishFlag[i] == FALSE) wxMilliSleep(3);
    
    delete [] ppCCallLua;
    delete [] pbFinishFlag;

    return;
}

void DrawOption_cls::
ReadStrLine(wxString strLine){
    mCConfList.ReadStrLineAll(strLine);
}

wxString DrawOption_cls::
Serialize(){
    return mCConfList.SerializeAll();
}
