// -*- c -*-
// "clsMeshBase.h"

#include "clsMeshBase.h"

#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

MeshBase_cls::
MeshBase_cls(CallBackMaterial_cls *pCCallBackParent){
    mpCCallBackParent = pCCallBackParent;
}

void MeshBase_cls::
ClearAll(){
    std::vector < float >().swap(mVVertex);
    std::vector < float >().swap(mVNormal);
    std::vector < float >().swap(mVUV2);
    std::vector < unsigned int >().swap(mVFaceIndexVertex);
    std::vector < unsigned int >().swap(mVFaceIndexNormal);
    std::vector < unsigned int >().swap(mVFaceIndexUV2);
    std::vector < int >().swap(mVMaterialAndPolygon);
}

void MeshBase_cls::
MinimizeMemoryUse(){
    std::vector < float >(mVVertex).swap(mVVertex);
    std::vector < float >(mVNormal).swap(mVNormal);
    std::vector < float >(mVUV2).swap(mVUV2);
    std::vector < unsigned int >(mVFaceIndexVertex).swap(mVFaceIndexVertex);
    std::vector < unsigned int >(mVFaceIndexNormal).swap(mVFaceIndexNormal);
    std::vector < unsigned int >(mVFaceIndexUV2).swap(mVFaceIndexUV2);
    std::vector < int >(mVMaterialAndPolygon).swap(mVMaterialAndPolygon);
}

void MeshBase_cls::
ReserveVertexNumber(int num){
    mVVertex.reserve(num*3);
    mVNormal.reserve(num*3);
    mVUV2.reserve(num*2);
}

void MeshBase_cls::
ReserveFaceNumber(int num){
    mVFaceIndexVertex.reserve(num*4);
    mVFaceIndexNormal.reserve(num*4);
    mVFaceIndexUV2.reserve(num*4);
}

void MeshBase_cls::
SetVertexNumber(int num){
    mVVertex.resize(num*3);
    mVNormal.resize(num*3);
    mVUV2.resize(num*2);
}

void MeshBase_cls::
SetFaceNumber(int num){
    mVFaceIndexVertex.resize(num*4);
    mVFaceIndexNormal.resize(num*4);
    mVFaceIndexUV2.resize(num*4);
}

void MeshBase_cls::
GLDrawUseNormalUseTexture(){
    std::vector < unsigned int > &VFaceIndexNormal = mVFaceIndexNormal;
    std::vector < unsigned int > &VFaceIndexUV2 = mVFaceIndexUV2;

    if (VFaceIndexNormal.size() == 0) VFaceIndexNormal = mVFaceIndexVertex;
    if (VFaceIndexUV2.size() == 0) VFaceIndexUV2 = mVFaceIndexVertex;

    int faceIndexOffset = 0;
    int size = (int)mVMaterialAndPolygon.size();
    for (int i = 0; i < size; i++){
	int ngon = mVMaterialAndPolygon[i];
	if (ngon == _MESH_MATERIAL) {
	    i++; int materialType = mVMaterialAndPolygon[i];
	    mpCCallBackParent->CallMaterial(materialType);
	    continue;
	}
	if (ngon == _MESH_POLYGONS_ARRAY) {
	    i++; int polyType = mVMaterialAndPolygon[i];
	    i++; int faceNum = mVMaterialAndPolygon[i];
	    GLDrawArrayMesh(polyType, faceNum, faceIndexOffset, TRUE, TRUE);
	    continue;
	}

	switch(ngon){
	case 3: glBegin(GL_TRIANGLES); break;
	case 4: glBegin(GL_QUADS); break;
	default: glBegin(GL_POLYGON); break;
	}
	for (int i = 0; i < ngon; i++){
	    int indexVertex = mVFaceIndexVertex[faceIndexOffset + i];
	    int indexNormal = VFaceIndexNormal[faceIndexOffset + i];
	    int indexUV = VFaceIndexUV2[faceIndexOffset + i];
	    glTexCoord2fv(&mVUV2[indexUV*2]);
	    glNormal3fv(&mVNormal[indexNormal*3]);
	    glVertex3fv(&mVVertex[indexVertex*3]);
	}
	glEnd();

	faceIndexOffset += ngon;
    }
}

void MeshBase_cls::
GLDrawUseNormal(){
    std::vector < unsigned int > &VFaceIndexNormal = mVFaceIndexNormal;

    if (VFaceIndexNormal.size() == 0) VFaceIndexNormal = mVFaceIndexVertex;

    int faceIndexOffset = 0;
    int size = (int)mVMaterialAndPolygon.size();
    for (int i = 0; i < size; i++){
	int ngon = mVMaterialAndPolygon[i];
	if (ngon == _MESH_MATERIAL) {
	    i++; int materialType = mVMaterialAndPolygon[i];
	    mpCCallBackParent->CallMaterial(materialType);
	    continue;
	}
	if (ngon == _MESH_POLYGONS_ARRAY) {
	    i++; int polyType = mVMaterialAndPolygon[i];
	    i++; int faceNum = mVMaterialAndPolygon[i];
	    GLDrawArrayMesh(polyType, faceNum, faceIndexOffset, TRUE, FALSE);
	    continue;
	}

	switch(ngon){
	case 3: glBegin(GL_TRIANGLES); break;
	case 4: glBegin(GL_QUADS); break;
	default: glBegin(GL_POLYGON); break;
	}
	for (int i = 0; i < ngon; i++){
	    int indexVertex = mVFaceIndexVertex[faceIndexOffset + i];
	    int indexNormal = VFaceIndexNormal[faceIndexOffset + i];
	    glNormal3fv(&mVNormal[indexNormal*3]);
	    glVertex3fv(&mVVertex[indexVertex*3]);
	}
	glEnd();

	faceIndexOffset += ngon;
    }
}

void MeshBase_cls::
GLDraw(bool bUseNornal, bool bUseUV){

    if (bUseNornal == TRUE && bUseUV == TRUE){
	GLDrawUseNormalUseTexture();
	return;
    }

    if (bUseNornal == TRUE && bUseUV == FALSE){
	GLDrawUseNormal();
	return;
    }

    int faceIndexOffset = 0;
    int size = (int)mVMaterialAndPolygon.size();
    for (int i = 0; i < size; i++){
	int ngon = mVMaterialAndPolygon[i];
	if (ngon == _MESH_MATERIAL) {
	    i++; int materialType = mVMaterialAndPolygon[i];
	    mpCCallBackParent->CallMaterial(materialType);
	    continue;
	}
	if (ngon == _MESH_POLYGONS_ARRAY) {
	    i++; int polyType = mVMaterialAndPolygon[i];
	    i++; int faceNum = mVMaterialAndPolygon[i];
	    GLDrawArrayMesh(polyType, faceNum, faceIndexOffset,
			    bUseNornal, bUseUV);
	    continue;
	}

	switch(ngon){
	case 3: glBegin(GL_TRIANGLES); break;
	case 4: glBegin(GL_QUADS); break;
	default: glBegin(GL_POLYGON); break;
	}
	for (int i = 0; i < ngon; i++){
	    int index = mVFaceIndexVertex[faceIndexOffset + i];
	    glNormal3fv(&mVNormal[index*3]);
	}
	glEnd();

	faceIndexOffset += ngon;
    }
}

void MeshBase_cls::
GLDrawArrayMesh(int polyType, int faceNum, int &faceIndexOffset,
		bool bUseNornal, bool bUseUV){
    int poly = 3;
    if (polyType == GL_QUADS) poly = 4;

    glEnableClientState(GL_VERTEX_ARRAY);
    if (bUseNornal == TRUE) glEnableClientState(GL_NORMAL_ARRAY);
    if (bUseUV == TRUE)     glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    
    glVertexPointer(3, GL_FLOAT, 0, mVVertex.data());
    if (bUseNornal == TRUE) glNormalPointer(GL_FLOAT, 0, mVNormal.data());
    if (bUseUV == TRUE)     glTexCoordPointer(2, GL_FLOAT, 0, mVUV2.data());
    
    glDrawElements(polyType, faceNum*poly, GL_UNSIGNED_INT,
		   mVFaceIndexVertex.data() + faceIndexOffset);
    
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);

    faceIndexOffset += faceNum * poly;
}
