// (setq c-basic-offset 4)

#include "clsInitializePararell.h"

#include "clsMacAddress.h"
#include "clsIpAddress.h"

#include "../COMMON/clsFilename.h"
#include "../FILTER/clsFilterCtrl.h"
#include "../GLATTRIB/clsColorCurve.h"
#include "../GLATTRIB/clsColorCurveGUI.h"
#include "../GLCANVAS/clsStereoGram.h"
#include "../GRAPHIC/clsImgSequence.h"
#include "../MYGL/clsGLInfo.h"
#include "../PANELS/clsDataExporter.h"
#include "../PANELS/clsDataLoader.h"
#include "../PANELS/clsScene.h"
#include "../PANELS/clsShortcut.h"
#include "../PANELS/clsPreference.h"
#include "../PARTICLEDATA/clsPeriodicBoundary.h"
#include "../PARTICLEDATA/clsParticleDataSeqCtrl.h"
#include "../POVRAY/clsPovRay.h"
#include "../SCENEOBJ/clsCameraPathGUI.h"
#include "../SCENEOBJ/clsCameraPath.h"
#include "../SCENEOBJ/clsTexPropertyGUI.h"
#include "../SCENEOBJ/clsLightSourceGUI.h"
#include "../TIMELINE/clsTimeCtrl.h"
#include "../MYAPI/MsgBox.h"

#include "../WIDGETCTRL/EX/clsColourDialogEX.h"
#include "../WIDGETCTRL/EX/clsFileDialogEX.h"
extern wxColourDialogEX *GpCClrDialog;
extern wxFileDialogEX   *GpCFileDialog;

#include "../../clsMain.h"
#include "../../clsFrameMain.h"

#include <wx/stdpaths.h>

#ifdef WIN32
#include <wx/msw/registry.h>
#endif

#ifdef WIN32
#include <process.h>
#else
#include <sys/types.h>
#include <unistd.h>
#endif

#include "../../wxWidgetString.h"

#ifdef WIN32
#define _WXDELIM_ wxT("\\")
#else
#define _WXDELIM_ wxT("/")
#endif

InitializeParallel_cls::
InitializeParallel_cls(wxThreadKind kind) :
    wxThread(kind) {
    return;
}

void InitializeParallel_cls::
SetParameters(bool *pbFinishedFlagLink, Main_cls *pCMainLink){
    mpbFinishedFlagLink = pbFinishedFlagLink;
    mpCMainLink = pCMainLink;

    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void InitializeParallel_cls::
InitializeParallel(){
    SplitPath_cls CPath(wxStandardPathsBase::Get().GetExecutablePath());
    mpCMainLink->mstrExecuteDir = CPath.GetPathStr(_FN_DRIVE_DIR);
    
#ifdef WIN32
    mpCMainLink->mPID = _getpid();
    mpCMainLink->mpCMacAddress   = new MacAddress_cls;
    mpCMainLink->mpCIpAddress    = new IpAddress_cls;
#else
    mpCMainLink->mPID = getpid();
#endif

    mpCMainLink->mpCCameraPath   = new CameraPath_cls;
    mpCMainLink->mpCAnchorPath   = new AnchorPath_cls;
    mpCMainLink->mpCDataLoader   = new DataLoader_cls(mpCMainLink);
    mpCMainLink->mpCDataExporter = new DataExporter_cls(mpCMainLink, mpCMainLink->mpCDataLoader);

    mpCMainLink->mpCPeriodic     = new PeriodicBoundary_cls();
    for (int i = 0; i < _MAX_FILTER_CTRL; i++)
	mpCMainLink->mpCFilterCtrl[i]->mpCPeriodicLink = mpCMainLink->mpCPeriodic;

    mpCMainLink->mpCCameraPathCtrl = new CameraPathCtrl_cls
	(mpCMainLink->mpCCameraPath, mpCMainLink->mpCAnchorPath);

    mpCMainLink->mpCLightSRCCtrl = new LightSRCCtrl_cls;
    mpCMainLink->mpCColorCurveCtrl = new ColorCurveCtrl_cls;
    mpCMainLink->mpCTexPropertyCtrl = new TexPropertyCtrl_cls;
    mpCMainLink->mpCImgSeq     = new ImgSequence_cls(mpCMainLink);
    mpCMainLink->mpCStereogram = new StereoGram_cls;
    mpCMainLink->mpCPovRayCtrl = new PovRayCtrl_cls;
    mpCMainLink->mpCPDataSeqCtrl = new PDataSeqCtrl_cls;
    mpCMainLink->mpCTime = new TimeCtrl_cls(mpCMainLink);
    
    FrameMain_cls *pCFrameMain = mpCMainLink->mpCFrame;

    pCFrameMain->mpCGLInfo   = new GLInfo_cls;
    pCFrameMain->mpCShortcut = new Shortcut_cls(pCFrameMain);
    mpCMainLink->mpCScene    = new Scene_cls(pCFrameMain, mpCMainLink->mpCTime);

    GpCFileDialog = new wxFileDialogEX(pCFrameMain);
    GpCClrDialog  = new wxColourDialogEX(pCFrameMain, wxColour(wxT("BLACK")), TRUE);
    
    mpCMainLink->mHungAppTimeoutMS = -1;
#ifdef WIN32
    wxRegKey key(wxRegKey::HKCU, wxT("Control Panel\\DeskTop"));
    if (key.Exists() == TRUE){
	wxString keyvaluestr;
	if (key.HasValue(wxT("HungAppTimeout")) == TRUE)
	    if (key.QueryValue(wxT("HungAppTimeout"), keyvaluestr) == TRUE)
		keyvaluestr.ToLong(&mpCMainLink->mHungAppTimeoutMS);
    }
#endif
}

wxThread::ExitCode InitializeParallel_cls::
Entry(){
    InitializeParallel();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ----

InitializeParallelB_cls::
InitializeParallelB_cls(wxThreadKind kind) :
    wxThread(kind) {
    return;
}

void InitializeParallelB_cls::
SetParameters(bool *pbFinishedFlagLink, Main_cls *pCMainLink){
    mpbFinishedFlagLink = pbFinishedFlagLink;
    mpCMainLink = pCMainLink;

    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void InitializeParallelB_cls::
InitializeParallel(){
    mpCMainLink->ReadProjectFile(mpCMainLink->mstrExecuteDir + _WXDELIM_ + _CONFIGURE_FILE);
}

wxThread::ExitCode InitializeParallelB_cls::
Entry(){
    InitializeParallel();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ----
