
#include "clsMacAddress.h"

#ifdef WIN32

#include <winsock2.h>
#include <ws2tcpip.h>
#include <iphlpapi.h> /* for IP Helper API */
#include <limits.h>
#pragma comment(lib, "ws2_32.lib")
#pragma comment(lib, "iphlpapi.lib")

#endif

#include <wx/wx.h>

#include "../COMMON/CommonDefinitions.h"

MacAddress_cls::
MacAddress_cls(){
#ifdef WIN32
    InitializeWin32();
#else
    InitializeLinux();
#endif
}

void MacAddress_cls::
InitializeLinux(){
    wxArrayString strArrayOutput, strArrayError;
    //wxExecuteEnv CEnv;
    //wxGetEnvMap  ( &CEnv.env ) ;
    //CEnv.env["LANG"] = "C";
    ::wxExecute(wxT("LANG=C ifconfig"), strArrayOutput, strArrayError, 0/*, &CEnv*/);
    while (strArrayOutput.IsEmpty() == FALSE){
	wxString strLine = strArrayOutput[0];
	size_t pos = strLine.Find(wxT("HWaddr"));
	if (pos != wxNOT_FOUND){
	    wxString strDesc = strLine.Mid(0, pos);
	    wxString strAddress = strLine.Mid(pos + 7, 18);
	    strAddress.Replace(wxT(":"), wxT("-"), TRUE);
	 
	    if (strAddress.IsSameAs(wxT("00-00-00-00-00-00")) == FALSE){
		mVstrAddress.push_back(strAddress);
		mVstrDesc.push_back(strDesc);
	    }
	}
	strArrayOutput.RemoveAt(0);
    }
}

void MacAddress_cls::
InitializeWin32(){
#ifdef WIN32 
    DWORD	 ret;
    ULONG	 i;
    ULONG        dwSize;
    BYTE*	 pBuff;
    PMIB_IFTABLE pIfTable;
    
    ::GetIfTable(NULL, &dwSize, TRUE); //Kvobt@TCY擾
    pBuff = new BYTE[dwSize];
    pIfTable = (PMIB_IFTABLE) pBuff;
    ret = ::GetIfTable(pIfTable, &dwSize, TRUE);

    if(ret == NO_ERROR) {
	for(i = 0; i < pIfTable->dwNumEntries; i++) {
	    DWORD dwRetVal = 0;
	    MIB_IFROW MibIfRow;
	    MibIfRow.dwIndex = i;

	    int type = -1;
	    int status = -1;
	    wxString strDesc;
	    //wxString strName;
	    if ((dwRetVal = GetIfEntry(&MibIfRow)) == NO_ERROR) {
		//strDesc << (char*)MibIfRow.bDescr;
		strDesc << wxString::FromAscii((char*)MibIfRow.bDescr);
		type = MibIfRow.dwType;
		//strName << MibIfRow.wszName;
		status = MibIfRow.dwOperStatus;
		wxString strStatus = GetStrStatus(status);
		strDesc << wxT(":") << strStatus;
	    }
	    
	    if (type == IF_TYPE_ETHERNET_CSMACD){
		wxString strAddress = 
		    wxString::Format(wxT("%02X-%02X-%02X-%02X-%02X-%02X"),
				     pIfTable->table[i].bPhysAddr[0],
				     pIfTable->table[i].bPhysAddr[1],
				     pIfTable->table[i].bPhysAddr[2],
				     pIfTable->table[i].bPhysAddr[3],
				     pIfTable->table[i].bPhysAddr[4],
				     pIfTable->table[i].bPhysAddr[5]);
		
		if (strAddress.IsSameAs(wxT("00-00-00-00-00-00")) == FALSE){
		    mVstrAddress.push_back(strAddress);
		    mVstrDesc.push_back(strDesc);
		}
	    }
	}
    }
    delete	pBuff;
#endif
}

void MacAddress_cls::
GetStrInfo(wxString &strInfo){
    for (unsigned int i = 0; i < mVstrAddress.size(); i++)
	strInfo << mVstrAddress[i] << wxT(" : ") << mVstrDesc[i] << ENDL;
}

void MacAddress_cls::
GetStrAddress(wxString &strAddress){
    for (unsigned int i = 0; i < mVstrAddress.size(); i++)
	strAddress << mVstrAddress[i] << ENDL;
}

wxString MacAddress_cls::
GetStrStatus(unsigned long status){
#ifdef WIN32
    switch(status){
    case IF_OPER_STATUS_NON_OPERATIONAL:
	return wxT("Non Operational");
    case IF_OPER_STATUS_UNREACHABLE:
	return wxT("Unreachable");
    case IF_OPER_STATUS_DISCONNECTED:
	return wxT("Disconnected");
    case IF_OPER_STATUS_CONNECTING:
	return wxT("Connecting");
    case IF_OPER_STATUS_CONNECTED:
	return wxT("Connected");
    case IF_OPER_STATUS_OPERATIONAL:
	return wxT("Operational");
    }
#endif
    return wxT("Undefined");
}

bool MacAddress_cls::
IsAddressExist(wxString strAddress){
    bool bMatch = FALSE;
    for (unsigned int i = 0; i < mVstrAddress.size(); i++){
	if (mVstrAddress[i].IsSameAs(strAddress))
	    bMatch = TRUE;
    }
    return bMatch;
}
