// "clsFishEye.h"
// (setq c-basic-offset 4)

#include "clsFishEye.h"
#include "../GLCANVAS/clsStereoGram.h"

#include "../COMMON/CommonDefinitions.h"
#include "clsFBO.h"

FishEye_cls::
FishEye_cls(){
    mBufferSize = 2048;
    for (int i = 0; i < 4; i++){
	mpCFBOParts[i] = NULL;
	mDList[i] = 0;
    }
}

FishEye_cls::
~FishEye_cls(){
    for (int i = 0; i < 4; i++){
	if (mpCFBOParts[i] != NULL) delete mpCFBOParts[i];
	mpCFBOParts[i] = NULL;
    }
}

void FishEye_cls::
ModifyByDomeTilt(PositionAndDirection_cls *pCCam){
}

void FishEye_cls::
ModifyForFishEye(PositionAndDirection_cls *pCCam, int fisheyeIndex){
    switch(fisheyeIndex){
    case 0: pCCam->RotateYPR(45, 0 , 0);  break;
    case 1: pCCam->RotateYPR(-45, 0, 0);  break;
    case 2: pCCam->RotateYPR(45, -90, 0); break;
    case 3: pCCam->RotateYPR(-45, 90, 0); break;
    }
}

void FishEye_cls::
ModifyForCylinder(PositionAndDirection_cls *pCCam, int fisheyeIndex){
    switch(fisheyeIndex){
    case 0: pCCam->RotateYPR(+45, 0 , 0);  break;
    case 1: pCCam->RotateYPR(-45, 0, 0);  break;
    case 2: pCCam->RotateYPR(+135, 0, 0); break;
    case 3: pCCam->RotateYPR(-135, 0, 0); break;
    }
}

void FishEye_cls::
SetUptoViewport(GLPerspective_cls *pCGLPerspective,
		PositionAndDirection_cls *pCCam, int fisheyeIndex,
		int multiPanelMode){
    switch(multiPanelMode){
    case _MULTIPANEL_FISHEYE:
	ModifyByDomeTilt(pCCam);
	ModifyForFishEye(pCCam, fisheyeIndex);
	pCGLPerspective->mbDivideRender = FALSE;
	pCGLPerspective->mDeg = 90;
	pCGLPerspective->SetClipSize(mBufferSize, mBufferSize);
	break;
    case _MULTIPANEL_CYLINDER:
	ModifyForCylinder(pCCam, fisheyeIndex);
	pCGLPerspective->mbDivideRender = FALSE;
	pCGLPerspective->mDeg = 90;
	pCGLPerspective->SetClipSize(mBufferSize, mBufferSize);
	break;
    default:
	pCGLPerspective->SetClipSize(mPanelWidth, mPanelHeight);
	break;
    }

    pCGLPerspective->SetViewportOnGL();
}

void FishEye_cls::
MakeStencilBegin(){
    //ꂩ`悷̂̃XeVlɂׂĂP^O    
    glStencilFunc( GL_ALWAYS, 1, ~0); 
    glStencilOp(GL_REPLACE,GL_REPLACE,GL_REPLACE);
    //XeVȊOɂ͏܂Ȃ
    glColorMask(0,0,0,0);
    glDepthMask(0);
}

void FishEye_cls::
MakeStencilEnd(){
    //̉
    glColorMask(1,1,1,1);
    glDepthMask(1);
    glStencilOp(GL_KEEP,GL_KEEP,GL_KEEP);
}

void FishEye_cls::
MakeCirclePatternOnStencil(){
    MakeStencilBegin();
    glDisable(GL_DEPTH_TEST);
    glBegin(GL_TRIANGLE_FAN);
    glVertex3d(0, 0, 0);
    for (int i = 0; i <= 72; i++){
	double rad = (double)i / 72 * 2 * 3.1416;
	glVertex3d(0.5 * cos(rad), 0.5 * sin(rad), 0);
    }
    glEnd();
    MakeStencilEnd();
}

void FishEye_cls::
MakePanoramaMaskPatternOnStencil(double deg){
    MakeStencilBegin();
    double factor = 180 / deg;
    
    glBegin(GL_QUADS);
    glVertex3d(+1, -0.196*factor, 0);
    glVertex3d(+1,  0.196*factor, 0);
    glVertex3d(-1,  0.196*factor, 0);
    glVertex3d(-1, -0.196*factor, 0);
    glEnd();
    MakeStencilEnd();
}

void FishEye_cls::
DrawDomeTexSegment(double coordu, double coordv0, double coordv1,
		   double x0, double x1, double y0, double y1, 
		   double z0, double z1, double deg){
    double r0 = sqrt(x0*x0 + y0*y0 + z0*z0);
    double r1 = sqrt(x1*x1 + y1*y1 + z1*z1);
    double theta0 = asin(z0/r0);
    double theta1 = asin(z1/r1);
    double phi0 = atan2(y0, x0);
    double phi1 = atan2(y1, x1);

    glTexCoord2f(coordu, coordv1);
    glVertex3d((1.5708 - theta1) / 3.1415 * 180 / deg * cos(phi1), 
	       (1.5708 - theta1) / 3.1415 * 180 / deg * sin(phi1), 0);
    glTexCoord2f(coordu, coordv0);
    glVertex3d((1.5708 - theta0) / 3.1415 * 180 / deg * cos(phi0), 
	       (1.5708 - theta0) / 3.1415 * 180 / deg * sin(phi0), 0);
}

void FishEye_cls::
GatherGLSetting(bool bAdd, int clipWidth, int clipHeight){
    if (bAdd == FALSE)
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT|
		GL_ACCUM_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);
    else
	glClear(GL_DEPTH_BUFFER_BIT|
		GL_ACCUM_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);
    
    glLoadIdentity();

    GLPerspective_cls CGLPerspective;
    CGLPerspective.SetWindowSize(clipWidth, clipHeight);
    CGLPerspective.SetClipSize(clipWidth, clipHeight);
    PositionAndDirection_cls CPos(0, 0, -1);

    CGLPerspective.mbPerspective = FALSE;
    CGLPerspective.SetViewportOnGL();
    CGLPerspective.SetPerspectiveOnGL(&CPos);
    CGLPerspective.SetModelOrProjectionViewOnGL(&CPos, GL_MODELVIEW);
   
    glEnable(GL_TEXTURE_2D);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S , GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T , GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void FishEye_cls::
MakeLists(double deg){
    if (mDList[0] != 0)
	for (int i = 0; i < 4; i++) 
	    glDeleteLists(mDList[i], 1);
    
    for (int i = 0; i < 4; i++)
	mDList[i] = glGenLists(1);

    double sqrt2 = sqrt(2.0);
    //int v_max = _FISHEYE_MAX, u_max = _FISHEYE_MAX;

    glNewList(mDList[0], GL_COMPILE);
    for (int v = 0; v < _FISHEYE_MAX; v++){
	double coordv0 = (double)v / _FISHEYE_MAX;
	double coordv1 = (double)(v+1) / _FISHEYE_MAX;
	glBegin(GL_QUAD_STRIP);
	for (int u = 0; u <= _FISHEYE_MAX; u++){
	    double coordu = (double)u / _FISHEYE_MAX;
	    double x  = (coordu - 1) * sqrt2; 
	    double y0 = coordv0 * 2 - 1;
	    double y1 = coordv1 * 2 - 1;
	    double z =  coordu * sqrt2;
	    x += 0.001;
	    DrawDomeTexSegment(coordu, coordv0, coordv1,
			       x, x, y0, y1, z, z, deg);
	}
	glEnd();
    }
    glEndList();

    glNewList(mDList[1], GL_COMPILE);
    for (int v = 0; v < _FISHEYE_MAX; v++){
	double coordv0 = (double)v / _FISHEYE_MAX;
	double coordv1 = (double)(v+1) / _FISHEYE_MAX;
	glBegin(GL_QUAD_STRIP);
	for (int u = 0; u <= _FISHEYE_MAX; u++){
	    double coordu = (double)u / _FISHEYE_MAX;
	    double x  = coordu * sqrt2; 
	    double y0 = coordv0 * 2 - 1;
	    double y1 = coordv1 * 2 - 1;
	    double z = (1.0 - coordu) * sqrt2;
	    DrawDomeTexSegment(coordu, coordv0, coordv1,
			       x, x, y0, y1, z, z, deg);
	}
	glEnd();
    }
    glEndList();
    
    glNewList(mDList[2], GL_COMPILE);
    for (int v = 0; v < _FISHEYE_MAX; v++){
	double coordv0 = (double)v / _FISHEYE_MAX;
	double coordv1 = (double)(v+1) / _FISHEYE_MAX;
	glBegin(GL_QUAD_STRIP);
	for (int u = 0; u <= _FISHEYE_MAX; u++){
	    double coordu = (double)u / _FISHEYE_MAX;
	    double x0 = (coordu + coordv0 - 1) * sqrt2; 
	    double x1 = (coordu + coordv1 - 1) * sqrt2; 
	    double y = 1;
	    double z0 = (coordu - coordv0) * sqrt2;
	    double z1 = (coordu - coordv1) * sqrt2;
	    DrawDomeTexSegment(coordu, coordv0, coordv1,
			       x0, x1, y, y, z0, z1, deg);
	}
	glEnd();
    }
    glEndList();

    glNewList(mDList[3], GL_COMPILE);
    for (int v = 0; v < _FISHEYE_MAX; v++){
	double coordv0 = (double)v / _FISHEYE_MAX;
	double coordv1 = (double)(v+1) / _FISHEYE_MAX;
	glBegin(GL_QUAD_STRIP);
	for (int u = 0; u <= _FISHEYE_MAX; u++){
	    double coordu = (double)u / _FISHEYE_MAX;
	    double x0 = (coordu + coordv0 - 1) * sqrt2; 
	    double x1 = (coordu + coordv1 - 1) * sqrt2; 
	    double y = -1;
	    double z0 = (-coordu + coordv0) * sqrt2;
	    double z1 = (-coordu + coordv1) * sqrt2;
	    DrawDomeTexSegment(coordu, coordv0, coordv1,
			       x0, x1, y, y, z0, z1, deg);
	}
	glEnd();
    }
    glEndList();
}

void FishEye_cls::
GatherFishEye(double deg, bool bAdd, int clipWidth, int clipHeight, 
	      StereoGram_cls *pCSGLink, int stereo, bool bUseDepthTexture){
    
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glPushMatrix();
    
    if (deg != mDeg){
	MakeLists(deg);
	mDeg = deg;
    }

    GatherGLSetting(bAdd, clipWidth, clipHeight);

    glDisable(GL_DEPTH_TEST);    
    glEnable(GL_STENCIL_TEST);
    MakeCirclePatternOnStencil();
    glStencilFunc( GL_EQUAL, 1, ~0); //1̃sNZɕ`
    glDisable(GL_POLYGON_SMOOTH);
    
    if (bAdd == FALSE){
	glDisable(GL_BLEND);
	glColor4f(0.5, 0, 0, 1.0);
	glBegin(GL_QUADS);
	glVertex3d(-10, -10, 0); glVertex3d(+10, -10, 0);
	glVertex3d(+10, +10, 0); glVertex3d(-10, +10, 0);
	glEnd(); // ͈͓œhԂ
	glEnable(GL_BLEND);
    }
    
    if (bAdd == TRUE){
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    }
    
    pCSGLink->ColorMaskBegin(stereo);
    for (int i = 0; i < 4; i++){
	if (bUseDepthTexture == TRUE && mpCFBOParts[i]->mpCDepthTexture != NULL)
	    mpCFBOParts[i]->mpCDepthTexture->BindTexture();
	else
	    mpCFBOParts[i]->BindTexture();
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
    
	glCallList(mDList[i]);
    }
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    
    pCSGLink->ColorMaskEnd();
    glDisable(GL_STENCIL_TEST);

    glPopMatrix();
    glPopAttrib();
}

void FishEye_cls::
GatherCylinder(double deg, bool bAdd, int clipWidth, int clipHeight,
	       StereoGram_cls *pCSGLink, int stereo, bool bUseDepthTexture){
    deg *= 2;
    
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glPushMatrix();
    GatherGLSetting(bAdd, clipWidth, clipHeight);
    
    // pm}̕`Jn
    glEnable(GL_STENCIL_TEST);
    MakePanoramaMaskPatternOnStencil(deg);
    glStencilFunc( GL_EQUAL, 1, ~0); //1̃sNZɕ`

    glDisable(GL_DEPTH_TEST);
    if (bAdd == FALSE){
	glDisable(GL_BLEND);
	glColor4f(0, 0, 0, 0);
	glBegin(GL_QUADS);
	glVertex3d(-10, -10, 0); glVertex3d(+10, -10, 0);
	glVertex3d(+10, +10, 0); glVertex3d(-10, +10, 0);
	glEnd(); // ͈͓œhԂ
	glEnable(GL_BLEND);
    }
    if (bAdd == TRUE){
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    }

    pCSGLink->ColorMaskBegin(stereo);
    
    double offset[4] = {-45, 45, -135, 135};
    
    int u_max = 40, v_max = 40;
    for (int i = 0; i < 4; i++){
	if (bUseDepthTexture == TRUE && mpCFBOParts[i]->mpCDepthTexture != NULL)
	    mpCFBOParts[i]->mpCDepthTexture->BindTexture();
	else
	    mpCFBOParts[i]->BindTexture();
	for (int u = 0; u < u_max; u++){
	    double coordu0 = (double)u / u_max;
	    double theta0    = atan2(coordu0-0.5, 0.5);
	    double theta0deg = atan2(coordu0-0.5, 0.5) * 180.0/3.14159;
	    if (u == 0) theta0deg = -45.001;
	    double theta0degOffset = (theta0deg + offset[i])/deg;
	    double cosTheta0 = cos(theta0);
	    double coordu1 = (double)(u+1) / u_max;
	    double theta1    = atan2(coordu1-0.5, 0.5);
	    double theta1deg = atan2(coordu1-0.5, 0.5) * 180.0/3.14159;
	    if (u == u_max - 1) theta1deg = +45.001;
	    double theta1degOffset = (theta1deg + offset[i])/deg;
	    double cosTheta1 = cos(theta1);
	    glBegin(GL_QUAD_STRIP);
	    for (int v = 0; v <= v_max; v++){
		double coordv0 = (double)v / v_max;
		double phi0deg = atan2(coordv0-0.5, 
				       0.5/cosTheta0)*180.0/3.14159;
		double phi1deg = atan2(coordv0-0.5, 
				       0.5/cosTheta1)*180.0/3.14159;
		glTexCoord2f(coordu0, coordv0);
		glVertex3d(theta0degOffset, phi0deg/deg, 0);
		glTexCoord2f(coordu1, coordv0);
		glVertex3d(theta1degOffset, phi1deg/deg, 0);
	    }
	    glEnd();
	}
    }
    
    pCSGLink->ColorMaskEnd();
    glDisable(GL_STENCIL_TEST);

    glPopMatrix();
    glPopAttrib();
}

void FishEye_cls::
MakeFishEyePrepare(int width, int height, int partsBufSize, int depth,
		   int fboMax){
    mPartsBufSize = partsBufSize;
    mBufferSize = partsBufSize;
    mPanelWidth = width;
    mPanelHeight = height;
    
    for (int i = 0; i < fboMax; i++){ 
	if (mpCFBOParts[i] == NULL){
	    mpCFBOParts[i] = new FBO_cls(mPartsBufSize, mPartsBufSize);
	    mpCFBOParts[i]->SetDepth(depth);
	}
	else{
	    mpCFBOParts[i]->SetGeometryBuffer(mPartsBufSize, mPartsBufSize);
	    mpCFBOParts[i]->SetDepth(depth);
	}
    }
}

void FishEye_cls::
MakeFishEyeProcessFH(int fboIdx, PositionAndDirection_cls CPos,
		     GLPerspective_cls CGLPerspective, int multiPanelMode,
		     bool bUseDepthTexture){
    mpCFBOParts[fboIdx]->MakeFBO(_FBO_USE_RGBA, bUseDepthTexture);
    mpCFBOParts[fboIdx]->Begin(TRUE);
    //glClearColor(0.5,0,0,1);
    //glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
    CGLPerspective.SetWindowSize(mPartsBufSize, mPartsBufSize);	
    SetUptoViewport(&CGLPerspective, &CPos, fboIdx, multiPanelMode);
    CGLPerspective.SetPerspectiveOnGL(&CPos);
    CGLPerspective.SetModelOrProjectionViewOnGL(&CPos, GL_MODELVIEW);
}

void FishEye_cls::
MakeFishEyeProcessLH(int fboIdx){
    mpCFBOParts[fboIdx]->End();
}
