// -*- c++ -*- 
// "clsFrameMain.h"
// (setq c-basic-offset 4)

#include "clsGLTexture.h"

#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

GLTexture_cls::
GLTexture_cls(){
    mTextureID = 0;
    mMinFilter = GL_LINEAR;
    mMagFilter = GL_LINEAR;
    mTexWrap = GL_CLAMP_TO_EDGE;
}

wxMemoryDC* GLTexture_cls::
BeginDC(int width, int height){
    mWidth  = width;
    mHeight = height;
    mpCBitmap = new wxBitmap(width, height, -1);
    mpCMemoryDC = new wxMemoryDC(*mpCBitmap);

    return mpCMemoryDC;
}

void GLTexture_cls::
EndDC(bool bUseAlpha){
    if (mTextureID == 0)
	glGenTextures(1, &mTextureID);
    
    wxImage CImg = mpCBitmap->ConvertToImage();
    
    unsigned char *pDataMain  = CImg.GetData();

    unsigned char *pBufferLA = new unsigned char [mWidth * mHeight *  2];

    if (bUseAlpha == TRUE)
	for (int y = 0; y < mHeight; y++){
	    int indexy = mWidth * y;
	    for (int x = 0; x < mWidth; x++){
		int index = indexy + x;
		pBufferLA[index * 2 + 1] = pDataMain[index * 3 + 0];
		pBufferLA[index * 2 + 0] = 255;
	    }
	}
    else
	for (int y = 0; y < mHeight; y++){
	    int indexy = mWidth * y;
	    for (int x = 0; x < mWidth; x++){
		int index = indexy + x;
		pBufferLA[index * 2 + 1] = 255;
		pBufferLA[index * 2 + 0] = pDataMain[index * 3 + 0];
	    }
	}

    glBindTexture(GL_TEXTURE_2D, mTextureID);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, mTexWrap);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, mTexWrap);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE_ALPHA, mWidth, mHeight,
		 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, pBufferLA);
    delete[] pBufferLA;
    
    delete mpCMemoryDC;
    delete mpCBitmap;
}

void GLTexture_cls::
SetEmptyImage(){
    if (mTextureID == 0)
	glGenTextures(1, &mTextureID);

    glBindTexture(GL_TEXTURE_2D, mTextureID);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, mTexWrap);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, mTexWrap);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

    unsigned char buf[4] = {0, 0, 0, 0};
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
		 1, 1, 0,
		 GL_RGBA, GL_UNSIGNED_BYTE, buf);
}

bool GLTexture_cls::
ClearImage(){
    if (mTextureID != 0){
	glDeleteTextures(1, &mTextureID);
	mTextureID = 0;
	return TRUE;
    }
    return FALSE;
}

bool GLTexture_cls::
LoadImage(wxString strFile, bool bFlip){
    if (strFile.IsSameAs(wxT("NONE"))) return FALSE;
    if (mstrImageLoaded.IsSameAs(strFile)) return FALSE;
    
    wxImage CImg(strFile);
    if (CImg.IsOk() == FALSE) return FALSE; // !!_IMPLEMENT_ME_!!
    
    if (bFlip == TRUE) CImg = CImg.Mirror(FALSE);
    CopyFromImage(CImg);
    mstrImageLoaded = strFile;
    return TRUE;
}

void GLTexture_cls::
glTexImage2D_RGBSafe(wxImage &CImg){
    int width  = CImg.GetWidth();
    int height = CImg.GetHeight();

    unsigned char *buf = new unsigned char [width * height * 3];
    for (int y = 0; y < height; y++){
	int indexY = y * width;
	for (int x = 0; x < width; x++){
	    int index = indexY + x;
	    buf[index*3 + 0] = CImg.GetRed(x, y);
	    buf[index*3 + 1] = CImg.GetGreen(x, y);
	    buf[index*3 + 2] = CImg.GetBlue(x, y);
	}
    }
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB,
		 width, height, 0,
		 GL_RGB, GL_UNSIGNED_BYTE, buf);
    delete[] buf;
}

void GLTexture_cls::
CopyFromImage(wxImage &CImg){
    if (mTextureID == 0)
	glGenTextures(1, &mTextureID);
    glBindTexture(GL_TEXTURE_2D, mTextureID);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, mTexWrap);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, mTexWrap);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mMagFilter);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, mMinFilter);

    mWidth = CImg.GetWidth();
    mHeight= CImg.GetHeight();

    if (CImg.HasAlpha() == FALSE){
	if (mWidth % 4 == 0)
	    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB,
			 mWidth, mHeight, 0,
			 GL_RGB, GL_UNSIGNED_BYTE, CImg.GetData());
	else
	    glTexImage2D_RGBSafe(CImg);
	glBindTexture(GL_TEXTURE_2D, 0);
	return; 
    }
    
    int width = mWidth;
    int height = mHeight;
    unsigned char *buf = new unsigned char [width * height * 4];
    for (int y = 0; y < height; y++){
	int indexY = y * width;
	for (int x = 0; x < width; x++){
	    int index = indexY + x;
	    buf[index*4 + 0] = CImg.GetRed(x,y);
	    buf[index*4 + 1] = CImg.GetGreen(x,y);
	    buf[index*4 + 2] = CImg.GetBlue(x,y);
	    buf[index*4 + 3] = CImg.GetAlpha(x,y);
	}
    }
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
		 width, height, 0,
		 GL_RGBA, GL_UNSIGNED_BYTE, buf);
    delete[] buf;

    glBindTexture(GL_TEXTURE_2D, 0);
}

void GLTexture_cls::
CopyFromDataRGBA(unsigned char *pData, int width, int height){
    if (mTextureID == 0)
	glGenTextures(1, &mTextureID);
    glBindTexture(GL_TEXTURE_2D, mTextureID);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, mTexWrap);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, mTexWrap);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mMagFilter);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, mMinFilter);

    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
		 width, height, 0,
		 GL_RGBA, GL_UNSIGNED_BYTE, pData);

    glBindTexture(GL_TEXTURE_2D, 0);
}

