// "clsTextureBase.h"
// (setq c-basic-offset 4)

#include "clsTextureBase.h"

#include "../COMMON/CommonDefinitionsGL.h"
#include "../COMMON/CommonDefinitions.h"

TextureBase_cls::
TextureBase_cls(){
    mTextureID = 0;
}

TextureBase_cls::
~TextureBase_cls(){
    if (mTextureID != 0)
	glDeleteTextures(1, &mTextureID);
}

void TextureBase_cls::
GenerateAndBindTexture(){
    glGenTextures(1, &mTextureID);
    glBindTexture(GL_TEXTURE_2D, mTextureID);
}

void TextureBase_cls::
BindTexture(){
    glBindTexture(GL_TEXTURE_2D, mTextureID);
}

void TextureBase_cls::
Push2DMatrix(int width, int height){
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glViewport( 0, 0, width, height);
    glLoadIdentity();
    gluOrtho2D(0, width, 0, height);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();
}

void TextureBase_cls::
Pop2DMatrix(){
    glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW); 
}

void TextureBase_cls::
GetFittingGeom(int clientWidth, int clientHeight, int fitType,
	       int &offsetX, int &offsetY, int &clipWidth, int &clipHeight){
    offsetX = 0; offsetY = 0;
    clipWidth = clientWidth; clipHeight = clientHeight;
    switch(fitType){
    case _TEX_FIT_HORIZONTAL:
	offsetY = -(clientWidth - clientHeight) / 2;
	clipHeight = clientWidth;
	break;
    case _TEX_FIT_VERTICAL:
	offsetX = (clientWidth - clientHeight) / 2;
	clipWidth = clientHeight;
	break;
    case _TEX_FIT_CORNER:
	clipWidth = clientWidth;
	clipHeight = clientHeight;
	break;
    }
}

void TextureBase_cls::
DrawOnScreen2DTileFit(int clientWidth, int clientHeight, int fitType,
		      int tileWidth, int tileHeight){
    int offsetX, offsetY;
    int clipWidth, clipHeight;
    GetFittingGeom(clientWidth, clientHeight, fitType,
		   offsetX, offsetY, clipWidth, clipHeight);
    
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glDisable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
    BindTexture();
    Push2DMatrix(clientWidth, clientHeight);

    float S = (float)clipWidth/tileWidth;
    float T = (float)clipHeight/tileHeight;

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S , GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T , GL_REPEAT);

    glDisable(GL_CULL_FACE);
    glBegin(GL_QUADS);
    glTexCoord2f(0, 0); glVertex2d( offsetX, offsetY);
    glTexCoord2f(S, 0); glVertex2d( offsetX + clipWidth, offsetY);
    glTexCoord2f(S, T); glVertex2d( offsetX + clipWidth, offsetY + clipHeight);
    glTexCoord2f(0, T); glVertex2d( offsetX, offsetY + clipHeight);
    glEnd();

    Pop2DMatrix();
    glPopAttrib();
}

void TextureBase_cls::
DrawOnScreen2DFit(int clientWidth, int clientHeight, 
		  int fitType, bool bCustom){
    int offsetX, offsetY;
    int clipWidth, clipHeight;
    GetFittingGeom(clientWidth, clientHeight, fitType,
		   offsetX, offsetY, clipWidth, clipHeight);

    DrawOnScreen2D(clientWidth, clientHeight,
		   clipWidth, clipHeight, offsetX, offsetY, bCustom);
}

void TextureBase_cls::
DrawOnScreen2DAtras(int clientWidth, int clientHeight,
		    int atrasIndex, int atrasGrid, int fitType){
    int offsetX, offsetY;
    int clipWidth, clipHeight;
    GetFittingGeom(clientWidth, clientHeight, fitType,
		   offsetX, offsetY, clipWidth, clipHeight);

    int clipWidthPart = clipWidth / atrasGrid;
    int clipHeightPart = clipHeight / atrasGrid;
    int indexOffsetX = atrasIndex % atrasGrid;
    int indexOffsetY = atrasIndex / atrasGrid;

    DrawOnScreen2D(clipWidth, clientHeight,
		   clipWidthPart, clipHeightPart,
		   offsetX + clipWidthPart * indexOffsetX,
		   offsetY + clipHeightPart * indexOffsetY, TRUE);
}

void TextureBase_cls::
DrawOnScreen2D(int clientWidth, int clientHeight,
	       int clipWidth, int clipHeight,
	       int offsetX, int offsetY, bool bCustom){

    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glDisable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);
    if (bCustom == FALSE){
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    }
    
    BindTexture();
    Push2DMatrix(clientWidth, clientHeight);

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S , GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T , GL_CLAMP_TO_EDGE);

    // ʏ̕`
    
    glDisable(GL_CULL_FACE);
    glBegin(GL_QUADS);
    glTexCoord2f(0, 0); glVertex2d( offsetX, offsetY);
    glTexCoord2f(1, 0); glVertex2d( offsetX + clipWidth, offsetY);
    glTexCoord2f(1, 1); glVertex2d( offsetX + clipWidth, offsetY + clipHeight);
    glTexCoord2f(0, 1); glVertex2d( offsetX, offsetY + clipHeight);
    glEnd();

    Pop2DMatrix();
    glPopAttrib();
}
