// -*- c++ -*- 
// "clsDataLoader.h"
// (setq c-basic-offset 4)

#include "clsDataLoader.h"
#include "clsDataLoaderGUI.h"

#include "../PARTICLEDATA/clsPeriodicBoundary.h"
#include "../WIDGETCTRL/EX/clsListCtrlEX.h"

#include "../../clsMain.h"

#include "../COMMON/CommonDefinitions.h"

#include <fstream>
#include <wx/file.h>

void DataLoaderInfo_cls::
SetInfo(DataLoader_cls *pCParent){
    mDataType     = pCParent->mCDataType    .mBody;
    mInterpType   = pCParent->mCInterpType  .mBody;
    mVariablePNum = pCParent->mCVariablePNum.mBody;
    mPNumLimit    = pCParent->mCPNumLimit   .mBody;
    mSearchLimit  = pCParent->mCSearchLimit .mBody;
}

// -----

DataLoader_cls::
DataLoader_cls(Main_cls *pCMain) :
    GrammerBase_cls(wxT("DataLoad"))
{
    mpCGUI = NULL;
    mpCMainLink = pCMain;
    
    mCConfList.Add(&mCLastOpenedFile, wxT("LastOpenedFile="), wxT("NONE"));
    mCConfList.Add(&mCInterpType, wxT("InterpolateType="), wxT("1"));
    mCConfList.Add(&mCVariablePNum, wxT("VariableNumber="), wxT("0"));
    mCConfList.Add(&mCReadSequence, wxT("ReadAsSequence="), wxT("1"));
    mCConfList.Add(&mCSearchLimit, wxT("SearchLimit="), wxT("0"));
    mCConfList.Add(&mCPNumLimit, wxT("PNumLimit="), wxT("0"));
    mCConfList.Add(&mCAdjustTranslate, wxT("LoaderTranslate="), wxT("0 0 0"));
    mCConfList.Add(&mCAdjustScale, wxT("LoaderScale="), wxT("1"));
    mCConfList.Add(&mCAdjustValue1, wxT("LoaderScaleValue1="), wxT("1"));
    mCConfList.Add(&mCLogarithmValue1, wxT("LoaderLogarithm="), wxT("0"));
    mCConfList.Add(&mCSequenceInit, wxT("LoaderSequenceInit="), wxT("0"));
    mCConfList.Add(&mCSequenceInterval, wxT("LoaderSequenceInterval="), wxT("1"));
}

DataLoader_cls::
~DataLoader_cls(){
}

void DataLoader_cls::
SetToDefault(){
    mCReadSequence.mBody = 1;
    mCSearchLimit.mBody = 0;
    mCPNumLimit.mBody = 0;
    LOOP_XYZ
	mCAdjustTranslate.mpBody[xyz] = 0;
    mCAdjustScale.mBody  = 1;
    mCAdjustValue1.mBody = 1;
    mCLogarithmValue1.mBody = 0;
    mCSequenceInit.mBody = 0;
    mCSequenceInterval.mBody = 1;
}

void DataLoader_cls::
InitGUI(wxWindow *pCParent){
    mpCGUI = new DataLoaderGUI_cls(pCParent, this);
}

void DataLoader_cls::
OpenWindow(wxWindow *pCParent){
    if (mpCGUI == NULL) InitGUI(pCParent);
    if (mpCGUI->IsShownOnScreen() == FALSE){
	mpCGUI->Show(TRUE);
	mpCGUI->SetFocus();
    }
    else
	mpCGUI->SetFocus();

    mpCGUI->Raise();
}

void DataLoader_cls::
UpdateListFromGrammer(){
    if (mpCGUI != NULL)
	mpCGUI->UpdateListFromGrammer();
}

PeriodicBoundary_cls *DataLoader_cls::
GetPCPeriodic(){
    return mpCMainLink->mpCPeriodic;
}

int  DataLoader_cls::
GetLineNumber(FILE *fp){
    fseek(fp, 0, SEEK_SET );
    long ini = ftell(fp);
    fseek(fp, 0, SEEK_END );
    long fin = ftell(fp);
    long length = fin - ini;
    
    fseek(fp, 0, SEEK_SET );
    int count = 0;
    char buf [1024*16];
    for (long i = 0; i < length; i += 1024*16){
	int size = 1024*16;
	if (length - i < size) size = length - i;
	fread(buf, sizeof(unsigned char), size, fp);
	for (int j = 0; j < size; j++)
	    if (buf[j] == '\n') count++;
    }
    return count;
}

int  DataLoader_cls::
GetHDF5Order(){
    int itemNum = mCGrammerHDF5.mCGrammerBody.mDimension;
    for (int i = 0; i < itemNum; i++)
	if (mCGrammerHDF5.mCGrammerBody.mpBody[i] == _GRAMMER_AXAYAZ ||
	    mCGrammerHDF5.mCGrammerBody.mpBody[i] == _GRAMMER_JXJYJZ)
	    return 3;
    return 1;
}

void DataLoader_cls::
SetLoadInfo(){
    mCLoadInfo.SetInfo(this);
}

bool DataLoader_cls::
OpenDataFileHDF5(wxString strFilePath){
    mCLastOpenedFile.mstrBody = strFilePath;
    if (mpCGUI->CheckDataHDF5(strFilePath) == FALSE)
	return FALSE;
    SetLoadInfo();
    mpCMainLink->OpenDataFileCommand(strFilePath);
    return TRUE;
}

void DataLoader_cls::
OpenDataFile(wxString strFilePath){
    SetLoadInfo();
    mpCMainLink->OpenDataFileCommand(strFilePath);
    mCLastOpenedFile.mstrBody = strFilePath;
}

bool DataLoader_cls::
SaveGrammer(wxString strFileOut){
    wxFile CFile(strFileOut, wxFile::write);
    if (CFile.IsOpened() == FALSE) return FALSE;

    UpdateGrammerFromList(mpCGUI);
    wxString strOut; 
    strOut << Serialize();
    CFile.Write(strOut, wxConvFile);
    return TRUE;
}

bool DataLoader_cls::
LoadGrammer(wxString strFile){
    std::wifstream ifstream(strFile.char_str(), std::ios::in);
    if (ifstream.fail()) return FALSE;
    std::wstring wstrLine;
    while ( std::getline (ifstream, wstrLine)){
	wxString strLine = wstrLine.c_str();
	ReadStrLine(strLine);
    }
    
    return TRUE;
}

// ---------

void DataLoader_cls::
ReadStrLine(wxString strLine){
    mCConfList.ReadStrLineAll(strLine);
}

wxString DataLoader_cls::
Serialize(){
    return mCConfList.SerializeAll();
}
