// (setq c-basic-offset 4)
// "clsDataLoaderGUI.h"
// "wxWidgetEnum.h"

#include "clsDataLoaderGUI.h"
#include "clsDataLoader.h"

#include "../COMMON/clsFilename.h"
#include "../MYAPI/MsgBox.h"
#include "../PARTICLEDATA/clsPeriodicBoundary.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/EX/clsFileDialogEX.h"
extern wxFileDialogEX *GpCFileDialog;
#include "../WIDGETCTRL/EX/clsButtonEX.h"
#include "../WIDGETCTRL/EX/clsListCtrlEX.h"

#include "../COMMON/CommonDefinitions.h"

#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

extern wxFrame  *GpCMainFrameCLS;

#include "../../clsMain.h"
#include "../../clsFrameMain.h"

#include "../COMMON/CommonDefinitions.h"

#include <wx/wx.h>

BEGIN_EVENT_TABLE(DataLoaderGUI_cls, wxDialog)
EVT_KEY_DOWN(DataLoaderGUI_cls::OnKeyDown)
EVT_BUTTON(btnDataLoaderOpenFile, DataLoaderGUI_cls::OnBtnOpenFile)
EVT_BUTTON(btnDataLoaderCheckHDF5, DataLoaderGUI_cls::OnBtnCheckHDF5)
EVT_BUTTON(btnDataLoaderDefault,   DataLoaderGUI_cls::OnBtnDefault)
EVT_BUTTON(btnDataLoaderPeriodic,  DataLoaderGUI_cls::OnBtnPeriodic)
EVT_BUTTON(btnDataLoaderSave,      DataLoaderGUI_cls::OnBtnSave)
EVT_BUTTON(btnDataLoaderLoad,      DataLoaderGUI_cls::OnBtnLoad)
EVT_MENU(menuGrammerInsertDummy1,  GrammerBaseGUI_cls::OnInsertDummy1)
EVT_MENU(menuGrammerInsertDummy3,  GrammerBaseGUI_cls::OnInsertDummy3)
EVT_MENU(menuGrammerInsertDummy10, GrammerBaseGUI_cls::OnInsertDummy10)
EVT_MENU(menuGrammerInsertXYZ,     DataLoaderGUI_cls::OnInsertXYZ)
EVT_MENU(menuGrammerInsertVxVyVz,  DataLoaderGUI_cls::OnInsertVXVYVZ)
EVT_MENU(menuGrammerInsertRadius,  DataLoaderGUI_cls::OnInsertRadius)
EVT_MENU(menuGrammerInsertType,    DataLoaderGUI_cls::OnInsertType)
EVT_MENU(menuGrammerInsertId,      DataLoaderGUI_cls::OnInsertId)
EVT_MENU(menuGrammerInsertValue1,  DataLoaderGUI_cls::OnInsertValue1)
EVT_MENU(menuGrammerInsertTime,    DataLoaderGUI_cls::OnInsertTime)
EVT_MENU(menuGrammerInsertNumber,  DataLoaderGUI_cls::OnInsertNumber)
EVT_MENU(menuGrammerDelete,        DataLoaderGUI_cls::OnDelete)
EVT_MOUSE_EVENTS(GrammerBaseGUI_cls::OnMouse)
EVT_LIST_KEY_DOWN(wxID_ANY, GrammerBaseGUI_cls::OnListKeyDown)
END_EVENT_TABLE()

DataLoaderGUI_cls::
DataLoaderGUI_cls(wxWindow *pCParent, DataLoader_cls *pCCtrl) :
GrammerBaseGUI_cls(pCCtrl, pCParent, _("Data Loader"), wxT("wxART_PARTICLE"))
{
    mpCParent = pCParent;
    mpCCtrl   = pCCtrl;

    int listCtrlWidth = 300;

    mpCHDF5GUI = NULL;
#ifdef _USE_HDF5
    mpCHDF5GUI = new HDF5GUI_cls(this, listCtrlWidth);
#endif
    
    wxBoxSizer  *pBoxSizerSRC = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer  *pBoxSizerTOP = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer  *pBoxSizerBTM = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer  *pBoxSizerBL  = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer  *pBoxSizerBLT = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer  *pBoxSizerBR  = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer  *pBoxSizerBRT = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer  *pBoxSizerBB  = new wxBoxSizer(wxHORIZONTAL);
    
    wxBoxSizer  *pBoxSizerTL = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer  *pBoxSizerTR = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer  *pBoxSizerTR2 = new wxBoxSizer(wxVERTICAL);

    SetMInput(pBoxSizerTL);

    mpCBook1 = new wxNotebook (this, ID_BOOK_LOADER);
    SetMInputModifier(mpCBook1);
    SetMInputLimitter(mpCBook1);
    SetMInputSequence(mpCBook1);
    
    mpCBook1->AddPage(mpCMInputManipulate, _("Modifier"));
    mpCBook1->AddPage(mpCMInputLimitter, _("Limitter"));
    mpCBook1->AddPage(mpCMInputSequence, _("Sequence"));
    pBoxSizerTL->Add(mpCBook1, 0, wxEXPAND);
    
    SetMInputPeriodic(pBoxSizerTR2);

    wxButton *pCButtonOpen = new wxButton (this, btnDataLoaderOpenFile, _("Open File Sequence"), wxDP, wxDS);

    mpCButtonCheckHDF5 = new wxButton (this, btnDataLoaderCheckHDF5, _("Check HDF5 File"), wxDP, wxDS);

    pBoxSizerTR->Add(pCButtonOpen, 2, wxEXPAND);
    pBoxSizerTR->Add(mpCButtonCheckHDF5, 1, wxEXPAND);

    pBoxSizerTR->Add(pBoxSizerTR2, 0, wxEXPAND);
    pBoxSizerTOP->Add(pBoxSizerTL, 1, wxEXPAND);
    pBoxSizerTOP->Add(pBoxSizerTR, 1, wxEXPAND);

    mpCStaticTextHead = new wxStaticText(this, wxID_ANY, _("Header Format"));
    mpCStaticTextBody = new wxStaticText(this, wxID_ANY, _("Body Format"));
    
    pBoxSizerBLT->Add(4, -1);
    pBoxSizerBLT->Add(mpCStaticTextHead, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL);
    pBoxSizerBRT->Add(4, -1);
    pBoxSizerBRT->Add(mpCStaticTextBody, 0, wxEXPAND|wxALIGN_CENTER_HORIZONTAL);

    pBoxSizerBL->Add(pBoxSizerBLT, 0, wxEXPAND);
    pBoxSizerBR->Add(pBoxSizerBRT, 0, wxEXPAND);

    mpCListCtrlHead = new wxListCtrlEX
	(this, wxID_ANY, wxDP, wxSize(listCtrlWidth, 180),
	 wxLC_REPORT | wxSUNKEN_BORDER);
    SetColumn(mpCListCtrlHead);
    UpdateList(mpCListCtrlHead, mpCCtrl->mCGrammerHead);
    
    mpCListCtrlBody = new wxListCtrlEX
	(this, wxID_ANY, wxDP, wxSize(listCtrlWidth, 180),
	 wxLC_REPORT | wxSUNKEN_BORDER);
    SetColumn(mpCListCtrlBody);
    UpdateList(mpCListCtrlBody, mpCCtrl->mCGrammerBody);
    
    if (mpCHDF5GUI != NULL){    
	pBoxSizerBL->Add(mpCHDF5GUI->mpCStaticTextAttr, 0, wxEXPAND);
	pBoxSizerBL->Add(mpCHDF5GUI->mpCListCtrlAttr, 1, wxEXPAND);
	pBoxSizerBR->Add(mpCHDF5GUI->mpCStaticTextBody, 0, wxEXPAND);
	pBoxSizerBR->Add(mpCHDF5GUI->mpCListCtrlBody, 1, wxEXPAND);
    }
    
    pBoxSizerBL->Add(mpCListCtrlHead, 1, wxEXPAND);
    pBoxSizerBR->Add(mpCListCtrlBody, 1, wxEXPAND);

    //pBoxSizerBLB->Add(mpCBtnUpHead, 0, wxEXPAND);
    //pBoxSizerBLB->Add(mpCBtnDownHead, 0, wxEXPAND);

    mpCBtnDefault = new wxButtonEX(this, btnDataLoaderDefault, wxT("Default"));    
    //pBoxSizerBRB->Add(mpCBtnUpBody, 0, wxEXPAND);
    //pBoxSizerBRB->Add(mpCBtnDownBody, 0, wxEXPAND);
    //pBoxSizerBRB->Add(-1, 0, wxEXPAND);
    
    mpCBtnLoad = new wxButtonEX(this, btnDataLoaderLoad, _("Grammer Load"));
    mpCBtnSave = new wxButtonEX(this, btnDataLoaderSave, _("Grammer Save"));
    pBoxSizerBB->Add(mpCBtnDefault, 0, wxEXPAND);
    pBoxSizerBB->Add(-1, 0, wxEXPAND);
    pBoxSizerBB->Add(mpCBtnLoad, 0, wxEXPAND);
    pBoxSizerBB->Add(mpCBtnSave, 0, wxEXPAND);

    //pBoxSizerBL->Add(pBoxSizerBLB, 0, wxEXPAND);
    //pBoxSizerBR->Add(pBoxSizerBRB, 0, wxEXPAND);

    pBoxSizerBTM->Add(pBoxSizerBL, 1, wxEXPAND);
    pBoxSizerBTM->Add(pBoxSizerBR, 1, wxEXPAND);
    pBoxSizerSRC->Add(pBoxSizerTOP, 0, wxEXPAND);
    pBoxSizerSRC->Add(pBoxSizerBTM, 0, wxEXPAND);
    pBoxSizerSRC->Add(pBoxSizerBB, 0, wxEXPAND);
    
    SetSizerAndFit(pBoxSizerSRC);
    HideAndShow();
}

DataLoaderGUI_cls::
~DataLoaderGUI_cls(){
    if (mpCHDF5GUI != NULL) delete mpCHDF5GUI;
    mpCCtrl->mpCGUI = NULL;
}

void DataLoaderGUI_cls::
SetMInput(wxBoxSizer *pBoxSizer){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInput, this);

    pCurr->mTextWidth = 90;
    pCurr->mItemWidth = -1;
    
    wxString strListType[4] = {
	wxT("Simple Binary"), wxT("Simple Ascii"), wxT("HDF5 (SIMPLE)"), wxT("HDF5 (TOKI)") };

    int typeNum = 2;
    if (mpCHDF5GUI == NULL)
	if (mpCCtrl->mCDataType.mBody >= _DATA_HDF5_SIMPLE)
	    mpCCtrl->mCDataType.mBody = _DATA_SIMPLEBINARY;
    if (mpCHDF5GUI != NULL) typeNum = 4;
    
    pCurr->CombinePointerList(&mpCCtrl->mCDataType.mBody,
			      typeNum, strListType, _("Data Type"));
    wxString strInterpType[3] = {
	_("Linear"), _("Hermite"), _("No Interpolate") };
    pCurr->CombinePointerList(&mpCCtrl->mCInterpType.mBody,
			      3, strInterpType, _("Interpolation"));
    
    wxString strVariablePNum[3] = {
	_("Constant"), _("Variable"), _("Variable (Sorted by ID)") };
    pCurr->CombinePointerList(&mpCCtrl->mCVariablePNum.mBody,
			      3, strVariablePNum, _("Particle Number"));
    
    pCurr->mMinimumWidth = 300;
    pCurr->FinishSetting(pBoxSizer);
    //pCurr->SetColorForXYZ(indexTranslate+1);

    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_COMBOBOX);
}

void DataLoaderGUI_cls::
SetMInputModifier(wxWindow *pCParent){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputManipulate, pCParent);
    
    pCurr->mTextWidth = 90;
    pCurr->mItemWidth = 65;

    int boxWidth = 3;
    int indexTranslate = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Translate"));
    pCurr->CombinePointer
	(_ITEM_FLOAT, &mpCCtrl->mCAdjustTranslate.mpBody[0]);
    pCurr->CombinePointer
	(_ITEM_FLOAT, &mpCCtrl->mCAdjustTranslate.mpBody[1]);
    pCurr->CombinePointer
	(_ITEM_FLOAT, &mpCCtrl->mCAdjustTranslate.mpBody[2]);
    pCurr->mItemWidth = -1;
    pCurr->CombinePointer
	(_ITEM_FLOAT, &mpCCtrl->mCAdjustScale.mBody,_("Scaling"));

    pCurr->mItemWidth = -1;
    boxWidth = 2;
    pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Modify Value1"));
    pCurr->CombinePointer
	(_ITEM_FLOAT, &mpCCtrl->mCAdjustValue1.mBody);
     pCurr->mItemWidth = -1;
    wxString strValue1[2] = {_("Raw Data"), _("Take log10") };
    pCurr->CombinePointerList(&mpCCtrl->mCLogarithmValue1.mBody,
			      2, strValue1);

    pCurr->mMinimumWidth = 292;
    pCurr->FinishSetting();
    pCurr->SetColorForXYZ(indexTranslate+1);
}

void DataLoaderGUI_cls::
SetMInputLimitter(wxWindow *pCParent){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputLimitter, pCParent);
    pCurr->mTextWidth = 110;
    pCurr->mItemWidth = -1;
    
    wxString strSequence[2] = {_("Read as snapshot"), _("Read as sequence") };
    pCurr->CombinePointerList
	(&mpCCtrl->mCReadSequence.mBody, 2, strSequence, _("Sequential data"));
    
    pCurr->mItemWidth = 90;
    int boxWidth = 2;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Particle Number"));
    pCurr->CombinePointer(_ITEM_INTEGER, &mpCCtrl->mCPNumLimit.mBody);
    wxString strNoLimit = wxT(" 0= "); strNoLimit << _("No Limit");
    pCurr->CombinePointer(_ITEM_STATIC_TEXT, &strNoLimit);

    pCurr->mItemWidth = -1;
    wxString strSearchLimit[3] = {_("No Limit"), _("Near 100"), _("Near 10000") };
    pCurr->CombinePointerList
	(&mpCCtrl->mCSearchLimit.mBody, 3, strSearchLimit, _("ID Search"));

    pCurr->mMinimumWidth = 292;
    pCurr->FinishSetting();
}

void DataLoaderGUI_cls::
SetMInputSequence(wxWindow *pCParent){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputSequence, pCParent);
    pCurr->mTextWidth = 110;
    pCurr->mItemWidth = -1;

    pCurr->CombinePointer(_ITEM_INTEGER, &mpCCtrl->mCSequenceInit.mBody, _("Initial number"));
    pCurr->CombinePointer(_ITEM_INTEGER, &mpCCtrl->mCSequenceInterval.mBody, _("Sequence interval"));

    pCurr->mMinimumWidth = 292;
    pCurr->FinishSetting();
}

void DataLoaderGUI_cls::
SetMInputPeriodic(wxBoxSizer *pBoxSizer){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputPeriodic, this);

    pCurr->mTextWidth = 100;
    pCurr->mItemWidth = -1;

    wxString listPeriodic[3] = {_("No Periodic Condition"),
				_("Periodic in x-y plane"),
				_("Periodic in xyz space")};
    pCurr->CombinePointerList(&mpCCtrl->GetPCPeriodic()->mCType.mBody, 3, 
			      listPeriodic, _("Periodic Boundary"));

    wxString listEffect[2] = {_("Guess velocity"),
			      _("Force velocity to be 0")};
    pCurr->CombinePointerList(&mpCCtrl->GetPCPeriodic()->mCEffect.mBody, 2,
			      listEffect, _("Periodic Effect"));

    int indexBtn = pCurr->CombinePointerButton(btnDataLoaderPeriodic, _("Boundary Size"));
    pCurr->GetLastItem().mAlign = _ITEM_ALIGN_RIGHT;
    
    pCurr->mMinimumWidth = 300;

    pCurr->FinishSetting(pBoxSizer);
}

bool DataLoaderGUI_cls::
CheckDataHDF5(wxString strFilePath){
    if (mpCHDF5GUI != NULL)
	return mpCHDF5GUI->CheckDataHDF5(strFilePath, mpCCtrl->mCGrammerHDF5);
    return FALSE;
}

void DataLoaderGUI_cls::
OnBtnCheckHDF5(wxCommandEvent& event){
    SetFileDialogInfo();
    if (GpCFileDialog->ShowModal() == wxID_OK){
	if (mpCHDF5GUI->CheckDataHDF5(GpCFileDialog->GetPath(),
				      mpCCtrl->mCGrammerHDF5) == FALSE)
	    ;
    }
}

void DataLoaderGUI_cls::
SetFileDialogInfo(){
    int type = mpCCtrl->mCDataType.mBody;
    wxString strWildCard = wxT("*");
    if (type < _DATA_HDF5_SIMPLE){
	if (mpCCtrlBase->IsItemExistBody(_GRAMMER_XYZ) == FALSE){
	    MsgBox(this, _("No XYZ data."), _("Grammer Error"));
	    return;
	}
    }
    else
	strWildCard = wxT("HDF5|*.h5;*.H5");

    mpCCtrlBase->SetHeaderGrammer(GetVGrammer(mpCListCtrlHead));
    mpCCtrlBase->SetBodyGrammer(  GetVGrammer(mpCListCtrlBody));

    wxString strLastOpened = mpCCtrl->mCLastOpenedFile.mstrBody;
    if (strLastOpened == wxT("NONE"))
	GpCFileDialog->SetInitPath(wxT("./"), wxEmptyString);
    else
	GpCFileDialog->SetInitPathFull(strLastOpened);

    GpCFileDialog->SetMessage(wxT("Choose first file"));
    GpCFileDialog->SetWildcard(strWildCard);
    //GpCFileDialog->SetStyle(wxFD_FILE_MUST_EXIST);
}

void DataLoaderGUI_cls::
OnBtnOpenFile(wxCommandEvent &event){
    int type = mpCCtrl->mCDataType.mBody;

    SetFileDialogInfo();
    if (GpCFileDialog->ShowModal() == wxID_OK){

	if (type >= _DATA_HDF5_SIMPLE){
	    if (mpCCtrl->OpenDataFileHDF5(GpCFileDialog->GetPath()) == TRUE){
		Hide();
		return;
	    }
	}
	
	mpCCtrl->UpdateGrammerFromList(this);
	mpCCtrl->OpenDataFile(GpCFileDialog->GetPath());
    }
    else
	return;

    Hide();
}

void DataLoaderGUI_cls::
OnBtnDefault(wxCommandEvent& event){
    SetToDefault(this, mpCCtrl->mCGrammerHead, mpCCtrl->mCGrammerBody);
    mpCCtrl->SetToDefault();
    mpCMInput->UpdateValues();
    mpCMInputManipulate->UpdateValues();
    mpCMInputLimitter->UpdateValues();
    mpCMInputSequence->UpdateValues();
    HideAndShow();
}

void DataLoaderGUI_cls::
OnBtnPeriodic(wxCommandEvent &event){
    SendMenuEvent(GpCMainFrameCLS, menuDialogPeriodic);
}

void DataLoaderGUI_cls::
OnBtnSave(wxCommandEvent& event){
    GpCFileDialog->SetMessage(wxT("Select grammer file name to save"));
    GpCFileDialog->SetWildcard(wxT("Grammer file (*.grm)|*.grm"));
    GpCFileDialog->SetStyle(wxFD_SAVE | wxFD_OVERWRITE_PROMPT);

    wxString strLastOpened = mpCCtrl->mpCMainLink->mpCFrame->mCLastOpenedFileGRM.mstrBody;
    if (strLastOpened.IsEmpty() == TRUE)
	strLastOpened = mpCCtrl->mpCMainLink->mpCFrame->mCLastOpenedFileZJP.mstrBody;
    
    if (strLastOpened.IsEmpty() == FALSE){
	SplitPath_cls CPath(strLastOpened);
	if (CPath.GetPathStr(_FN_EXT).IsSameAs(wxT(".grm"))){
	    GpCFileDialog->SetInitPath
		(CPath.GetPathStr(_FN_DRIVE_DIR_FNAME),
		 CPath.GetPathStr(_FN_FNAME_EXT));
	}
	else
	    GpCFileDialog->SetInitPath
		(CPath.GetPathStr(_FN_DRIVE_DIR), wxT(".grm"));
    }
    
    if (GpCFileDialog->ShowModal() == wxID_OK){
	bool bSucceed = mpCCtrl->SaveGrammer(GpCFileDialog->GetPath());
	if (bSucceed == FALSE)
	    MsgBox(this, wxT("Error"), wxT("File writing failed."));
	else
	    mpCCtrl->mpCMainLink->mpCFrame->mCLastOpenedFileGRM.mstrBody = 
		GpCFileDialog->GetPath();
    }
}

void DataLoaderGUI_cls::
OnBtnLoad(wxCommandEvent& event){
    GpCFileDialog->SetMessage(wxT("Choose grammer file to load"));
    GpCFileDialog->SetWildcard(wxT("Grammer file (*.grm)|*.grm"));
    GpCFileDialog->SetStyle(wxFD_FILE_MUST_EXIST);
    if (GpCFileDialog->ShowModal() == wxID_OK){
	bool bSucceed = mpCCtrl->LoadGrammer(GpCFileDialog->GetPath());
	UpdateListFromGrammer();
	mpCMInput->UpdateValues();
	HideAndShow();
	if (bSucceed == FALSE)
	    MsgBox(this, wxT("Error"), wxT("File open failed."));
	else
	    mpCCtrl->mpCMainLink->mpCFrame->mCLastOpenedFileGRM.mstrBody = 
		GpCFileDialog->GetPath();
    }
}

void DataLoaderGUI_cls::
OnDelete(wxCommandEvent &event){
    DeleteSelectedItem(GetPActiveListCtrl());
    UpdateIndexAndColor(GetPActiveListCtrl());
}

void DataLoaderGUI_cls::
OnKeyDown(wxKeyEvent& event){
    SendKeyEventToTop(GpCMainFrameCLS, event);
}

void DataLoaderGUI_cls::
HideAndShow(){
    int type = mpCCtrl->mCDataType.mBody;

    if (mpCHDF5GUI != NULL){
	switch(type){
	case _DATA_SIMPLEBINARY:
	case _DATA_SIMPLEASCII:
	    mpCStaticTextHead->Show();
	    mpCStaticTextBody->Show();
	    mpCListCtrlHead->Show();
	    mpCListCtrlBody->Show();
	    mpCBtnLoad->Show();
	    mpCBtnSave->Show();
	    mpCButtonCheckHDF5->Hide();
	    mpCHDF5GUI->mpCStaticTextAttr->Hide();
	    mpCHDF5GUI->mpCStaticTextBody->Hide();
	    mpCHDF5GUI->mpCListCtrlAttr->Hide();
	    mpCHDF5GUI->mpCListCtrlBody->Hide();
	    break;
	case _DATA_HDF5_SIMPLE:
	case _DATA_HDF5_TOKI:
	    mpCStaticTextHead->Hide();
	    mpCStaticTextBody->Hide();
	    mpCListCtrlHead->Hide();
	    mpCListCtrlBody->Hide();
	    mpCBtnLoad->Hide();
	    mpCBtnSave->Hide();
	    mpCButtonCheckHDF5->Show();
	    mpCHDF5GUI->mpCStaticTextAttr->Show();
	    mpCHDF5GUI->mpCStaticTextBody->Show();
	    mpCHDF5GUI->mpCListCtrlAttr->Show();
	    mpCHDF5GUI->mpCListCtrlBody->Show();
	    break;
	}
    }
    else{
	mpCButtonCheckHDF5->Hide();
    }
    
    ResizeDialog();
}

void DataLoaderGUI_cls::
HookFunc01(int id, void *pArg){
    OnHookCombobox();
    HideAndShow();
}

