// "clsShortcut.h"

#include "clsShortcut.h"
#include "../MAINCANVAS/clsTopHalfPanel.h"
#include "../GLCANVAS/clsGLCanvasCtrl.h"
#include "../GLCANVAS/clsMainGLCanvas.h"
#include "../GLCANVAS/clsMoveItem.h"
#include "../GLCANVAS/NAVIGATORS/clsNavigators.h"
#include "../SCENEOBJ/clsSceneObj.h"
#include "../TIMELINE/clsTimeCtrl.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/EX/clsListCtrlEX.h"
#include "../WIDGETCTRL/WX/myArtProv.h"
#include "../WIDGETCTRL/WX/clsFontProvider.h"

#include "../../clsMain.h"
#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

#include "../COMMON/CommonDefinitions.h"

ShortcutItem_cls::
ShortcutItem_cls(int type, wxString strBase, wxString strName, 
		 int charactor, bool bShift, bool bCtrl, bool bAlt){
    mType = type;
    mstrBase = strBase;
    mstrName = strName;
    mChar = charactor;
    mbShift = bShift;
    mbCtrl = bCtrl;
    mbAlt = bAlt;
}

int ShortcutItem_cls::
GetType(int code, bool bShift, bool bCtrl, bool bAlt){
    if ((code == mChar) && (bShift == mbShift) && (bCtrl == mbCtrl) && (bAlt == mbAlt))
	return mType;
    
    return -1;
}

// ------

MenuCommandItem_cls::
MenuCommandItem_cls(int type, int menuID){
    mType = type;
    mMenuID = menuID;
}

// ------

Shortcut_cls::
Shortcut_cls(FrameMain_cls *pCParentFrame){
    mpCParentFrame = pCParentFrame;
    
    SetDefault(0);
    
    AddMenuCommand(_SC_OPEN_PROJECT, menuFileOpenProject);
    AddMenuCommand(_SC_SAVE_PROJECT, menuFileSaveProject);
    AddMenuCommand(_SC_SAVE_PROJECT_AS, menuFileSaveProjectAs);
    AddMenuCommand(_SC_OPEN_DATA, menuFileOpenData);
    AddMenuCommand(_SC_SAVE_SNAPSHOT, menuFileSaveImageSnap);
    AddMenuCommand(_SC_SAVE_IMGSEQ, menuFileSaveImageSeq);
    AddMenuCommand(_SC_EXPORT_PDATA, menuFileExportData);
    AddMenuCommand(_SC_EXPORT_PATH, menuFileExportPathPov);
    AddMenuCommand(_SC_CLOSE_DATA, menuFileCloseData);
    AddMenuCommand(_SC_QUIT, Minimal_Quit);
    AddMenuCommand(_SC_UNDO, menuEditUndo);
    AddMenuCommand(_SC_REDO, menuEditRedo);
    AddMenuCommand(_SC_GO, menuEditGo);
    AddMenuCommand(_SC_TIMEREMAP, menuEditTimeRemap);
    AddMenuCommand(_SC_REMOVE_SNAPSHOT, menuEditRemoveCacheSnap);
    AddMenuCommand(_SC_REMOVE_ALLCACHE, menuEditRemoveCacheAll);
    AddMenuCommand(_SC_SCENE, menuEditScene);
    AddMenuCommand(_SC_PREFERENCE, menuEditPreference);
    AddMenuCommand(_SC_POVRAY, menuDialogPovRay);
    AddMenuCommand(_SC_SPLIT1, menuViewMainPanelType0);
    AddMenuCommand(_SC_SPLIT2, menuViewMainPanelType1);
    AddMenuCommand(_SC_SPLIT3, menuViewMainPanelType2);
    AddMenuCommand(_SC_SPLIT4, menuViewMainPanelType3);
    AddMenuCommand(_SC_SPLIT5, menuViewMainPanelType4);
    AddMenuCommand(_SC_EXTRA,  menuViewExtraPanel);

    AddMenuCommand(_SC_CAMERAPATH, menuDialogCameraPath);
    AddMenuCommand(_SC_LIGHTSOURCE, menuDialogLightSRC);
    AddMenuCommand(_SC_COLORCURVE, menuDialogColorCurve);
    AddMenuCommand(_SC_TEXPROPERTY, menuDialogTexProperty);
    AddMenuCommand(_SC_STEREOGRAM, menuDialogStereogram);
    AddMenuCommand(_SC_PERIODIC, menuDialogPeriodic);

    AddMenuCommand(_SC_HELP, menuHelp);
    AddMenuCommand(_SC_ABOUT, Minimal_About);
    AddMenuCommand(_SC_ABOUT_GL, menuAboutGL);
    AddMenuCommand(_SC_ABOUT_NETWORK, menuAboutNetwork);
    
    AddMenuCommand(_SC_FULLSCREEN, menuFullScreen);
    AddMenuCommand(_SC_ESCAPE, menuEscape);

    AddMenuCommand(_SC_VIEW_5, GLMenuTogglePerspective);
    AddMenuCommand(_SC_CENTER_ITEM, GLMenuCenterItem);
    AddMenuCommand(_SC_CENTER_ORIGIN, GLMenuCenterOrigin);
    
    mCPos = wxDefaultPosition;
}

void Shortcut_cls::
SetDefault(int type){
    if (type != 0){
	switch (type){
	case 1:
	    ReadTemplate(wxT("Resource/SCTemplateBlender.txt"));
	    break;
	}
	return;
    }

    mVCItems.clear();

    Add(_SC_OPEN_PROJECT, wxT("Open Project"), _("Open Project"), 
	'O', FALSE, TRUE);
    Add(_SC_SAVE_PROJECT, wxT("Save Project"), _("Save Project"),
	'S', FALSE, TRUE);
    Add(_SC_SAVE_PROJECT_AS, wxT("Save Project As"), _("Save Project As"),
	'S', TRUE, TRUE);
    Add(_SC_OPEN_DATA, wxT("Load Particle Data"), _("Load Particle Data"),
	'O', FALSE, TRUE, TRUE);
    Add(_SC_SAVE_SNAPSHOT, wxT("Save Snapshot"), _("Save Snapshot"), 
	WXK_F7);
    Add(_SC_SAVE_IMGSEQ, wxT("Save Image Sequence"), _("Save Image Sequence"), 
	WXK_F8);
    Add(_SC_EXPORT_PDATA, wxT("Export Particle Data"), _("Export Particle Data"),
	'E', FALSE, TRUE);
    Add(_SC_EXPORT_PATH, wxT("Export Path to Clipboard (.pov)"), _("Export Path to Clipboard (.pov)"),
	'E', FALSE, TRUE, TRUE);
    Add(_SC_CLOSE_DATA, wxT("Close Particle Data"), _("Close Particle Data"),
	'W', FALSE, TRUE);
    Add(_SC_QUIT, wxT("Quit"), _("Quit"),
	'Q');
    Add(_SC_UNDO, wxT("Undo"), _("Undo"), 
	'Z', FALSE, TRUE);
    Add(_SC_REDO, wxT("Redo"), _("Redo"),
	'Y', FALSE, TRUE);
    Add(_SC_GO, wxT("Move Frame"), _("Move Frame"),
	'G', FALSE, TRUE);
    Add(_SC_TIMEREMAP, wxT("Time Remap"), _("Time Remap"));
    Add(_SC_REMOVE_SNAPSHOT, wxT("Snapshot"), _("Snapshot"), 
	WXK_F5);
    Add(_SC_REMOVE_ALLCACHE, wxT("All Cache"), _("All Cache"), 
	WXK_F5, FALSE, TRUE, TRUE);
    Add(_SC_SCENE, wxT("Scene Config"), _("Scene Config"),
	'K', FALSE, TRUE);
    Add(_SC_PREFERENCE, wxT("Preference"), _("Preference"),
	'P', FALSE, TRUE, TRUE);
    Add(_SC_SPLIT1, wxT("1 panel"), _("1 panel"), 
	'1', FALSE, FALSE, TRUE);
    Add(_SC_SPLIT2, wxT("2 panels (vertical)"), _("2 panels (vertical)"), 
	'2', FALSE, FALSE, TRUE);
    Add(_SC_SPLIT3, wxT("2 panels (horizontal)"), _("2 panels (horizontal)"), 
	'3', FALSE, FALSE, TRUE);
    Add(_SC_SPLIT4, wxT("4 panelsA"), _("4 panelsA"), 
	'4', FALSE, FALSE, TRUE);
    Add(_SC_SPLIT5, wxT("4 panelsB"), _("4 panelsB"), 
	'5', FALSE, FALSE, TRUE);
    Add(_SC_EXTRA, wxT("Extra Panel"), _("Extra Panel"), 
	'0', FALSE, FALSE, TRUE);

    Add(_SC_CAMERAPATH, wxT("View and Camera Config"), _("View and Camera Config"),
	'U', FALSE, TRUE);
    Add(_SC_LIGHTSOURCE, wxT("Light Source Editor"), _("Light Source Editor"),
	'L', FALSE, TRUE);
    Add(_SC_COLORCURVE, wxT("Color Curve Editor"), _("Color Curve Editor"),
	'J', FALSE, TRUE);
    Add(_SC_TEXPROPERTY, wxT("Texture Editor"), _("Texture Editor"),
	'T', FALSE, TRUE);
    Add(_SC_STEREOGRAM, wxT("Stereogram Config"), _("Stereogram Config"),
	'M', FALSE, TRUE);
    Add(_SC_PERIODIC, wxT("Periodic Boundary"), _("Periodic Boundary"),
	'B', FALSE, TRUE);

    Add(_SC_POVRAY, wxT("Pov-Ray Rendering Configure"), _("Pov-Ray Rendering Configure"),
	'R', FALSE, TRUE);
    
    Add(_SC_HELP, wxT("Help"), _("Help"),
	WXK_F1);
    Add(_SC_ABOUT, wxT("About"), _("About"), 
	WXK_F1, FALSE, TRUE);
    Add(_SC_ABOUT_GL, wxT("About Open GL"), _("About Open GL"), 
	WXK_F1, FALSE, FALSE, TRUE);
    Add(_SC_ABOUT_NETWORK, wxT("About Networtk"), _("About Networtk"), 
	WXK_F1, FALSE, TRUE, TRUE);

    Add(_SC_CENTER_ORIGIN, wxT("Center Origin"), _("Center Origin"), 
	'.', FALSE, TRUE);
    Add(_SC_CENTER_ITEM, wxT("Center Item"), _("Center Item"),	'.');
    
    Add(_SC_VIEW_0, wxT("Free 1"), wxES, '0');
    Add(_SC_VIEW_0, wxT("Free 2"), wxES, WXK_NUMPAD0);
    Add(_SC_VIEW_5, wxT("Toggle Perspective 1"), _("Toggle Perspective"), 
	'5');
    Add(_SC_VIEW_5, wxT("Toggle Perspective 2"),  _("Toggle Perspective"), 
	WXK_NUMPAD5);
    Add(_SC_VIEW_1, wxT("+X to -X 1"), wxES, '1');
    Add(_SC_VIEW_1, wxT("+X to -X 2"), wxES, WXK_NUMPAD1);
    Add(_SC_VIEW_1_CTRL, wxT("-X to +X 1"), wxES, '1', FALSE, TRUE);
    Add(_SC_VIEW_1_CTRL, wxT("-X to +X 2"), wxES, WXK_NUMPAD1, FALSE, TRUE);
    Add(_SC_VIEW_3, wxT("+Y to -Y 1"), wxES, '3');
    Add(_SC_VIEW_3, wxT("+Y to -Y 2"), wxES, WXK_NUMPAD3);
    Add(_SC_VIEW_3_CTRL, wxT("-Y to +Y 1"), wxES, '3', FALSE, TRUE);
    Add(_SC_VIEW_3_CTRL, wxT("-Y to +Y 2"), wxES, WXK_NUMPAD3, FALSE, TRUE);
    Add(_SC_VIEW_7, wxT("-Z to +Z 1"), wxES, '7');
    Add(_SC_VIEW_7, wxT("-Z to +Z 2"), wxES, WXK_NUMPAD7);
    Add(_SC_VIEW_7_CTRL, wxT("+Z to -Z 1"), wxES, '7', FALSE, TRUE);
    Add(_SC_VIEW_7_CTRL, wxT("+Z to -Z 2"), wxES, WXK_NUMPAD7, FALSE, TRUE);

    Add(_SC_DRAG_X, wxT("Drag X"), wxT("Drag X"), 
	'X');
    Add(_SC_DRAG_Y, wxT("Drag Y"), wxT("Drag Y"), 
	'Y');
    Add(_SC_DRAG_Z, wxT("Drag Z"), wxT("Drag Z"), 
	'Z');
    Add(_SC_ROTATION, wxT("Mode Rotation"), wxT("Mode Rotation"),
	'R');
    Add(_SC_SCALE, wxT("Mode Scale"), wxT("Mode Scale"),
	'S');
    Add(_SC_TRANSLATION, wxT("Mode Translation"), wxT("Mode Translation"),
	'T');
    Add(_SC_NEXT_KEY, wxT("Next Key"), _("Next Key"), 
	'K');
    Add(_SC_PREV_KEY, wxT("Prev Key"), _("Prev Key"),
	'J');
    Add(_SC_SET_WA_MIN, wxT("Set Working Area Min"), _("Set Working Area Min"),
	'B');
    Add(_SC_SET_WA_MAX, wxT("Set Working Area Max"), _("Set Working Area Max"),
	'N');
    Add(_SC_TL_ZOOM_OUT, wxT("Timeline Zoom Out"), _("Timeline Zoom Out"),
	'-');
    Add(_SC_TL_ZOOM_IN, wxT("Timeline Zoom In"), _("Timeline Zoom In"), 
	39/*'^'*/);
    Add(_SC_PLAY, wxT("Play"), _("Play"), 
	WXK_SPACE);
    Add(_SC_LEFT, wxT("Prev Frame"), _("Prev Frame"), 
	WXK_LEFT);
    Add(_SC_RIGHT, wxT("Next Frame"), _("Next Frame"),
	WXK_RIGHT);
    Add(_SC_FULLSCREEN, wxT("Full Screen"), _("Full Screen"),
	WXK_RETURN, FALSE, FALSE, TRUE);
    Add(_SC_ESCAPE, wxT("Full Screen Off"), _("Full Screen Off"),
	WXK_ESCAPE);
}

void Shortcut_cls::
ReadTemplate(wxString strFile){
    std::wifstream ifstream(strFile.char_str(), std::ios::in);

    if (ifstream.fail()){
	wxString strMessage;;
	strMessage << wxT("Read Configure Fails") << ENDL;
	strMessage << strFile;
	wxMessageBox(strMessage, wxT("Warning")); 
	return;
    }

    std::wstring wstrLine;
    while ( std::getline (ifstream, wstrLine)){
	wxString strLine = wstrLine.c_str();
	ReadStrLine(ifstream, strLine);
    }
}

void Shortcut_cls::
Add(int type, wxString strBase, wxString strName, int charactor, 
    bool bShift, bool bCtrl, bool bAlt){
    if (strName.IsEmpty() == TRUE) strName = strBase;
    mVCItems.push_back(ShortcutItem_cls(type, strBase, strName, charactor,
					bShift, bCtrl, bAlt));
}

void Shortcut_cls::
AddMenuCommand(int type, int menuID){
    mVCMenuCommand.push_back(MenuCommandItem_cls(type, menuID));
}

int Shortcut_cls::
GetSpecialKeyFromIndex(int index){
    switch(index){
    case 1: return WXK_F1;
    case 2: return WXK_F2;
    case 3: return WXK_F3;
    case 4: return WXK_F4;
    case 5: return WXK_F5;
    case 6: return WXK_F6;
    case 7: return WXK_F7;
    case 8: return WXK_F8;
    case 9: return WXK_F9;
    case 10: return WXK_F10;
    case 11: return WXK_F11;
    case 12: return WXK_F12;
    case 13: return WXK_LEFT;
    case 14: return WXK_RIGHT;
    case 15: return WXK_UP;
    case 16: return WXK_DOWN;
    case 17: return WXK_SPACE;
    case 18: return WXK_ESCAPE;
    case 19: return WXK_RETURN;
    case 20: return WXK_NUMPAD0;
    case 21: return WXK_NUMPAD1;
    case 22: return WXK_NUMPAD2;
    case 23: return WXK_NUMPAD3;
    case 24: return WXK_NUMPAD4;
    case 25: return WXK_NUMPAD5;
    case 26: return WXK_NUMPAD6;
    case 27: return WXK_NUMPAD7;
    case 28: return WXK_NUMPAD8;
    case 29: return WXK_NUMPAD9;
    }
    return 0;
}

int Shortcut_cls::
GetSpecialKeyIndex(int keyChar){
    switch(keyChar){
    case WXK_F1: return 1;
    case WXK_F2: return 2;
    case WXK_F3: return 3;
    case WXK_F4: return 4;
    case WXK_F5: return 5;
    case WXK_F6: return 6;
    case WXK_F7: return 7;
    case WXK_F8: return 8;
    case WXK_F9: return 9;
    case WXK_F10: return 10;
    case WXK_F11: return 11;
    case WXK_F12: return 12;
    case WXK_LEFT: return 13;
    case WXK_RIGHT: return 14;
case WXK_UP:    return 15;
    case WXK_DOWN:  return 16;
    case WXK_SPACE: return 17;
    case WXK_ESCAPE: return 18;
    case WXK_RETURN: return 19;
    case WXK_NUMPAD0: return 20;
    case WXK_NUMPAD1: return 21;
    case WXK_NUMPAD2: return 22;
    case WXK_NUMPAD3: return 23;
    case WXK_NUMPAD4: return 24;
    case WXK_NUMPAD5: return 25;
    case WXK_NUMPAD6: return 26;
    case WXK_NUMPAD7: return 27;
    case WXK_NUMPAD8: return 28;
    case WXK_NUMPAD9: return 29;
    }
    return 0;
}

wxString Shortcut_cls::
GetStrSpecialKey(int keyChar){
    switch(keyChar){
    case WXK_F1: return wxT("F1");
    case WXK_F2: return wxT("F2");
    case WXK_F3: return wxT("F3");
    case WXK_F4: return wxT("F4");
    case WXK_F5: return wxT("F5");
    case WXK_F6: return wxT("F6");
    case WXK_F7: return wxT("F7");
    case WXK_F8: return wxT("F8");
    case WXK_F9: return wxT("F9");
    case WXK_F10: return wxT("F10");
    case WXK_F11: return wxT("F11");
    case WXK_F12: return wxT("F12");
    case WXK_LEFT: return wxT("LEFT");
    case WXK_RIGHT: return wxT("RIGHT");
    case WXK_UP:    return wxT("UP");
    case WXK_DOWN:  return wxT("DOWN");
    case WXK_SPACE: return wxT("SPACE");
    case WXK_ESCAPE: return wxT("ESC");
    case WXK_RETURN: return wxT("ENTER");
    case WXK_NUMPAD0: return wxT("NUM0");
    case WXK_NUMPAD1: return wxT("NUM1");
    case WXK_NUMPAD2: return wxT("NUM2");
    case WXK_NUMPAD3: return wxT("NUM3");
    case WXK_NUMPAD4: return wxT("NUM4");
    case WXK_NUMPAD5: return wxT("NUM5");
    case WXK_NUMPAD6: return wxT("NUM6");
    case WXK_NUMPAD7: return wxT("NUM7");
    case WXK_NUMPAD8: return wxT("NUM8");
    case WXK_NUMPAD9: return wxT("NUM9");
    }
    return wxEmptyString;
}

wxString Shortcut_cls::
GetStrShortcut(int index){
    wxString strShortcut;
    
    if (mVCItems[index].mbAlt == TRUE) strShortcut << wxT("Alt-");
    if (mVCItems[index].mbShift == TRUE) strShortcut << wxT("Shift-");
    if (mVCItems[index].mbCtrl == TRUE) strShortcut << wxT("Ctrl-");
    
    wxString strSpecialKey = GetStrSpecialKey(mVCItems[index].mChar);
    if (strSpecialKey.IsEmpty() == TRUE)
	strShortcut << wxString::FromAscii((char)mVCItems[index].mChar);
    else
	strShortcut << strSpecialKey;

    return strShortcut;
}

wxString Shortcut_cls::
GetStrFullString(int type, bool bShortCut){
    for (unsigned int i = 0; i < mVCItems.size(); i++){
	if (type == mVCItems[i].mType){
	    wxString strCommand = mVCItems[i].mstrName;
	    if (mVCItems[i].mChar == '\0') return strCommand;
	    if (bShortCut == TRUE){
		strCommand << wxT("\t");
		strCommand << GetStrShortcut(i);
	    }
	    return strCommand;
	}
    }
    return wxEmptyString;
}

std::vector < wxString > Shortcut_cls::
GetVstrItem(int index){
    std::vector < wxString > VstrItem;
    
    if (mVCItems[index].mbShift == TRUE) VstrItem.push_back(wxT("shift"));
    else                                 VstrItem.push_back(wxT(" "));
    if (mVCItems[index].mbCtrl  == TRUE) VstrItem.push_back(wxT("ctrl"));
    else                                 VstrItem.push_back(wxT(" "));
    if (mVCItems[index].mbAlt   == TRUE) VstrItem.push_back(wxT("alt"));
    else                                 VstrItem.push_back(wxT(" "));
    
    wxString strChar;
    wxString strSpecialKey = GetStrSpecialKey(mVCItems[index].mChar);
    if (strSpecialKey.IsEmpty() == TRUE)
	strChar << wxString::FromAscii((char)mVCItems[index].mChar);
    else
	strChar << strSpecialKey;
    VstrItem.push_back(strChar);
    VstrItem.push_back(mVCItems[index].mstrName);
    return VstrItem;
}

int Shortcut_cls::
GetIndexFromBase(wxString strLine){
    int count = (int)mVCItems.size();
    for (int i = 0; i < count; i++)
	if (strLine.IsSameAs(mVCItems[i].mstrBase)) return i;
    return -1;
}

wxString Shortcut_cls::
GetMenuString(int ID){
    for (unsigned int i = 0; i < mVCMenuCommand.size(); i++){
	if (mVCMenuCommand[i].mMenuID == ID)
	    return GetStrFullString(mVCMenuCommand[i].mType);
    }
    return wxT("!!");//wxEmptyString;
}

wxString Shortcut_cls::
GetMenuHelpString(int ID){
    for (unsigned int i = 0; i < mVCMenuCommand.size(); i++){
	if (mVCMenuCommand[i].mMenuID == ID)
	    return GetStrFullString(mVCMenuCommand[i].mType, FALSE);
    }
    return wxT("!!");//wxEmptyString;
}

wxMenuItem* Shortcut_cls::
GetPNewMenuItem(wxMenu *pMenu, int ID, wxString strHelpString, bool bChackItem){
    if (strHelpString == wxEmptyString){
	if (bChackItem == TRUE)
	    return new wxMenuItem(pMenu, ID, GetMenuString(ID), GetMenuHelpString(ID), wxITEM_CHECK );
	else
	    return new wxMenuItem(pMenu, ID, GetMenuString(ID), GetMenuHelpString(ID));
    }
    if (bChackItem == TRUE)
	return new wxMenuItem(pMenu, ID, GetMenuString(ID), strHelpString, wxITEM_CHECK);
    else
	return new wxMenuItem(pMenu, ID, GetMenuString(ID), strHelpString);
}

void Shortcut_cls::
SetMenuItem(wxMenu *pMenu, int ID, wxString strMenuBitmap, wxString strHelpString){
    wxMenuItem *pItem = GetPNewMenuItem(pMenu, ID, strHelpString);
    if (strMenuBitmap.IsEmpty() != TRUE)
	pItem->SetBitmap(ART(strMenuBitmap));
    pMenu->Append(pItem);
}

void Shortcut_cls::
SetMenuCheckItem(wxMenu *pMenu, int ID, wxString strMenuBitmap, wxString strHelpString){
    wxMenuItem *pItem = GetPNewMenuItem(pMenu, ID, strHelpString, TRUE);
    if (strMenuBitmap.IsEmpty() != TRUE)
	pItem->SetBitmap(ART(strMenuBitmap));
    pMenu->Append(pItem);
}

// return TRUE ƑSplXV
bool Shortcut_cls::
ProcFromList(int code, bool bShift, bool bCtrl, bool bAlt){
    int type = -1;
    int size = (int)mVCItems.size();
    for (int i = 0; i < size; i++){
	type = mVCItems[i].GetType(code, bShift, bCtrl, bAlt);
	if (type != -1){
	    i = size; continue;
	}
    }
    if (type == -1) return FALSE;

    FrameMain_cls *pC = mpCParentFrame;
    //int comR = wxEVT_COMMAND_RIBBONTOOL_CLICKED;
    int comM = wxEVT_COMMAND_MENU_SELECTED;
    MainGLCanvas_cls *pCGL = pC->mpCActiveGLCanvasLink;

    for (unsigned int i = 0; i < mVCMenuCommand.size(); i++)
	if (type == mVCMenuCommand[i].mType)
	    SendEvent(pC, comM, mVCMenuCommand[i].mMenuID);

    switch(type){
    case _SC_VIEW_0:
	pCGL->SetDisplayMode(_GL_CANVAS_DISPLAY_FREE);
	pCGL->mpCGLCtrl->UpdateWindow(); break;
    case _SC_VIEW_1:
	pCGL->SetDisplayMode(_GL_CANVAS_DISPLAY_X_PLUS);
	pCGL->mpCGLCtrl->UpdateWindow(); break;
    case _SC_VIEW_3:
	pCGL->SetDisplayMode(_GL_CANVAS_DISPLAY_Y_PLUS);
	pCGL->mpCGLCtrl->UpdateWindow(); break;
    case _SC_VIEW_7:
	pCGL->SetDisplayMode(_GL_CANVAS_DISPLAY_Z_MINUS);
	pCGL->mpCGLCtrl->UpdateWindow(); break;
    case _SC_VIEW_1_CTRL:
	pCGL->SetDisplayMode(_GL_CANVAS_DISPLAY_X_MINUS);
	pCGL->mpCGLCtrl->UpdateWindow(); break;
    case _SC_VIEW_3_CTRL:
	pCGL->SetDisplayMode(_GL_CANVAS_DISPLAY_Y_MINUS);
	pCGL->mpCGLCtrl->UpdateWindow(); break;
    case _SC_VIEW_7_CTRL:
	pCGL->SetDisplayMode(_GL_CANVAS_DISPLAY_Z_PLUS);
	pCGL->mpCGLCtrl->UpdateWindow(); break;
    case _SC_DRAG_X:
	pCGL->mpCNavigators->SetDragMode(_ROTATE_MOVE_R); return TRUE;
    case _SC_DRAG_Y:
	pCGL->mpCNavigators->SetDragMode(_ROTATE_MOVE_P); return TRUE;
    case _SC_DRAG_Z:
	pCGL->mpCNavigators->SetDragMode(_ROTATE_MOVE_Y); return TRUE;
    case _SC_ROTATION:
	pCGL->mpCNavigators->RotationMode(); return TRUE;
    case _SC_SCALE:
	pCGL->mpCNavigators->ScaleMode(); return TRUE;
    case _SC_TRANSLATION:
	pCGL->mpCNavigators->TranslationMode(); return TRUE;
    case _SC_NEXT_KEY:
	pC->GetPCTimeCtrl()->MoveFrameCommandSnip(+1); break;
    case _SC_PREV_KEY:
	pC->GetPCTimeCtrl()->MoveFrameCommandSnip(-1); break;
    case _SC_SET_WA_MIN:
	pC->GetPCTimeCtrl()->SetWAMinCommand(pC->GetPCTimeCtrl()->mFrame, _TIME_CTRL_CURRENT); break;
    case _SC_SET_WA_MAX:
	pC->GetPCTimeCtrl()->SetWAMaxCommand(pC->GetPCTimeCtrl()->mFrame, _TIME_CTRL_CURRENT); break;
    case _SC_TL_ZOOM_IN:
	pC->GetPCTimeCtrl()->ExpandAACommand(TRUE); break;
    case _SC_TL_ZOOM_OUT:
	pC->GetPCTimeCtrl()->ExpandAACommand(FALSE); break;
    case _SC_PLAY:
	pC->mpCParent->mpCTime->StartPlayCommand(+1); break;
    case _SC_RIGHT:
	pC->mpCParent->mpCTime->MoveFrameCommand(+1); break;
    case _SC_LEFT:
	pC->mpCParent->mpCTime->MoveFrameCommand(-1); break;
    }
    return FALSE;
}

void Shortcut_cls::
ProcShortcutKey(int modifier, int code){
    bool bRefreshGLPanel = FALSE;

    bool bShift = ((modifier & wxMOD_SHIFT) != 0) ? TRUE : FALSE;
    bool bCtrl = ((modifier & wxMOD_CONTROL) != 0) ? TRUE : FALSE;
    bool bAlt = ((modifier & wxMOD_ALT) != 0) ? TRUE : FALSE;

    //if (bShift == TRUE && bCtrl == TRUE)return;
    
    bRefreshGLPanel |= ProcFromList(code, bShift, bCtrl, bAlt);
    
    if (bRefreshGLPanel == TRUE) 
	mpCParentFrame->mpCTopHalfPanel->RefreshGLPanels();
}

void Shortcut_cls::
SetList(wxListCtrlEX *pCList){
    pCList->SetFont(FontProvider_cls::GetFont(0));
    int widthShift = FontProvider_cls::GetTextWidth(0, wxT("shift")) + 12;
    int widthCtrl  = FontProvider_cls::GetTextWidth(0, wxT("ctrl")) + 12;
    int widthAlt   = FontProvider_cls::GetTextWidth(0, wxT("alt")) + 12;
    int widthKey   = FontProvider_cls::GetTextWidth(0, wxT("RIGHT")) + 12;
    pCList->InsertColumn(0,wxT(""), wxLIST_FORMAT_CENTER, widthShift);
    pCList->InsertColumn(1,wxT(""), wxLIST_FORMAT_CENTER, widthCtrl);
    pCList->InsertColumn(2,wxT(""), wxLIST_FORMAT_CENTER, widthAlt);
    pCList->InsertColumn(3,wxT(""), wxLIST_FORMAT_LEFT, widthKey);
    pCList->InsertColumn(4,wxT("function"), wxLIST_FORMAT_LEFT, 380 - widthShift - widthCtrl - widthAlt - widthKey);

    int num = (int)mVCItems.size();
    for (int i = 0; i < num; i++)
	pCList->AddListItem(-1, GetVstrItem(i));
}

void Shortcut_cls::
UpdateListItem(wxListCtrlEX *pCList, int index){
    if (index == -1)
	for (unsigned int i = 0; i < mVCItems.size(); i++)
	    pCList->SetListItem(i, GetVstrItem(i));
    else
	pCList->SetListItem(index, GetVstrItem(index));
}

#include "../MYAPI/MsgBox.h"

bool Shortcut_cls::
PopupEditor(int index, wxWindow *pC){
    MultiInputDialog_cls *pCDLG = new
	MultiInputDialog_cls(pC, wxID_ANY, mVCItems[index].mstrName,
			     mCPos, wxDS, FontProvider_cls::GetFont(0),
			     ARTSIZE(wxT("Scene.png"), wxSize(16,16)));
    
    pCDLG->SetPCPosInfoAtClose(&mCPos);

    MultiInputPanel_cls* pCurr = pCDLG->mpCPanel;

    bool bShift = mVCItems[index].mbShift;
    bool bCtrl = mVCItems[index].mbCtrl;
    bool bAlt = mVCItems[index].mbAlt;
    int  keyType = 0;
    wxString strKey;

    if (GetStrSpecialKey(mVCItems[index].mChar).IsEmpty() == FALSE){
	strKey = wxEmptyString;
	keyType = GetSpecialKeyIndex(mVCItems[index].mChar);
    }
    else
	strKey = wxString::FromAscii('\0'+mVCItems[index].mChar);

    wxString strKeyList[_MAX_SHORTCUT] = {wxT("Basic Key")};
    for (int i = 1; i < _MAX_SHORTCUT; i++){
	wxString strKey = GetStrSpecialKey(GetSpecialKeyFromIndex(i));
	strKeyList[i] = GetStrSpecialKey(GetSpecialKeyFromIndex(i));
    }

    pCurr->mUndoType = _MN_UNDO_MANUAL;
    pCurr->mItemWidth = -1;
    int boxWidth = 2;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth);
    pCurr->CombinePointerList(&keyType, _MAX_SHORTCUT, strKeyList);
    pCurr->CombinePointer(_ITEM_STRING, &strKey);
    
    boxWidth = 3;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth);
    pCurr->CombinePointer(_ITEM_BOOL, &bShift, wxT("Shift"));
    pCurr->CombinePointer(_ITEM_BOOL, &bCtrl, wxT("Ctrl"));
    pCurr->CombinePointer(_ITEM_BOOL, &bAlt, wxT("Alt"));

    pCDLG->MakeGeometry();
    pCDLG->UpdateValues();

    if (pCDLG->ShowModal() != wxID_OK) return FALSE;

    int keyChar = 0;
    if (keyType != 0){
	keyChar = GetSpecialKeyFromIndex(keyType);
    }
    else{
	strKey.MakeUpper();
	if (strKey.IsEmpty() == FALSE) keyChar = (int)strKey[0];
	//if (keyChar < '0' || keyChar > 'Z') keyChar = 0;
    }

    if (mVCItems[index].mChar == keyChar &&
	mVCItems[index].mbShift == bShift &&
	mVCItems[index].mbCtrl == bCtrl &&
	mVCItems[index].mbAlt == bAlt) return FALSE;

    if (keyChar == 0){
	bShift = FALSE;
	bCtrl = FALSE;
	bAlt = FALSE;
    }

    int conflict = -1;
    int type = -1;
    int size = (int)mVCItems.size();
    for (int i = 0; i < size; i++){
	type = mVCItems[i].GetType(keyChar, bShift, bCtrl, bAlt);
	if (type != -1){
	    conflict = i;
	    i = size; continue;
	}
    }
    if (type != -1 && (conflict != index && conflict != -1) && keyChar != 0){
	wxString strMessage;
	strMessage << GetStrShortcut(conflict) << ENDL;
	strMessage << _("is already used in") << ENDL;
	strMessage << mVCItems[conflict].mstrName << ENDL;
	strMessage << _("Overwrite it ?") << ENDL;
	if (MsgBoxYesno(pC, _("Conflict"), strMessage) == FALSE)
	    return FALSE;
	mVCItems[conflict].mChar = 0;
	mVCItems[conflict].mbShift = FALSE;
	mVCItems[conflict].mbCtrl = FALSE;
	mVCItems[conflict].mbAlt = FALSE;
	mConflict = conflict;
    }
    else
	mConflict = -1;
    
    mVCItems[index].mChar = keyChar;
    mVCItems[index].mbShift = bShift;
    mVCItems[index].mbCtrl = bCtrl;
    mVCItems[index].mbAlt = bAlt;
 
    return TRUE;
}

void Shortcut_cls::
ReadStrLine(std::wistream &istream, wxString strLineTag){
    if (strLineTag.IsSameAs(wxT("-ShortcutBegin")) == FALSE) return;

    std::wstring wstrLine;
    wxString strLine;
    while ( std::getline (istream, wstrLine)){
	strLine = wstrLine.c_str();
	if (strLine.IsSameAs(wxT("-ShortcutEnd")) == TRUE) return;
	int index = GetIndexFromBase(strLine);

	if (index == -1) continue;

	std::getline (istream, wstrLine);
	strLine = wstrLine.c_str();
	
	bool bShift, bCtrl, bAlt;
	int  charCode;
	std::string strBuffer = (char*)strLine.char_str();
	std::stringstream strTmp(strBuffer);
	strTmp >> bShift >> bCtrl >> bAlt >> charCode;

	mVCItems[index].mbShift = bShift;
	mVCItems[index].mbCtrl = bCtrl;
	mVCItems[index].mbAlt = bAlt;
	mVCItems[index].mChar = charCode;
    }
}

wxString Shortcut_cls::
SerializeForConfigure(){
    wxString strOut;
    
    strOut << wxT("-ShortcutBegin") << ENDL;
    int count = (int)mVCItems.size();
    for (int i = 0; i < count; i++){
	strOut << mVCItems[i].mstrBase << ENDL;
	strOut << mVCItems[i].mbShift << wxT(" ") << mVCItems[i].mbCtrl << wxT(" ") << mVCItems[i].mbAlt << wxT(" ") << mVCItems[i].mChar << ENDL;
    }

    strOut << wxT("-ShortcutEnd") << ENDL;

    return strOut;
}
