// -*- c -*-
// "clsParamGroup.h"

#include "clsParamGroup.h"
#include "clsParamGroupGUI.h"

#include "clsParamCommands.h"
#include "clsParameters.h"
#include "clsParametersPanel.h"
#include "clsParamTL.h"

#include "../MAINCANVAS/clsParamGUICtrl.h"
#include "../SCENEOBJ/clsSceneObj.h"
#include "../WIDGETCTRL/clsBitmapPanel.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/EX/clsTextEntryDialogEX.h"
#include "../WIDGETCTRL/WX/clsFontProvider.h"
#include "../WIDGETCTRL/WX/clsSizeAdjuster.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;

#include "../COMMON/CommonDefinitions.h"

#include "../../wxWidgetEnumMenu.h"

#include <wx/clipbrd.h>

#include <sstream>

ParamGroup_cls::
ParamGroup_cls(wxString strGroutpLabel,
	       int dimension, wxString *pstrSubLabel, 
	       long long updateBit, bool bTimeDependent) {
    mpCGUI = NULL;
    mControlMode = -1;
    mstrGroupLabel = strGroutpLabel;
    mstrGroupLabelMLG = mstrGroupLabel;
    wxString strDimension; 
    strDimension << dimension;
    mCConfList.Add(&mCDimension, wxT("dimension="), strDimension);
    mCConfList.Add(&mCbLinking, wxT("link="), wxT("1"));
    mCConfList.Add(&mCbExpression, wxT("expression="), wxT("0"));

    wxString strExpression = wxT("return ");    
    for (int i = 0; i < dimension; i++){
	mVCParam.push_back(Parameters_cls(strGroutpLabel + wxT(" ") + pstrSubLabel[i], pstrSubLabel[i], updateBit, bTimeDependent));
	mVCParam[i].mpCParentGroupLink = this;
	mVCParam[i].mSelfIndex = i;
	if (i != 0) strExpression << wxT(",");
	strExpression << mVCParam[i].mstrLabel;
	mVSubLabel.push_back(pstrSubLabel[i]);
    }
    mCConfList.Add(&mCstrExpression, wxT("strExpression="), strExpression);
    
    mbGroupingGroup = FALSE;
    //mbLinking = TRUE;
    mbObjectVisible = TRUE;
    mbUseTLGUI      = TRUE;
    mbShownOnTL     = TRUE;
    mbModeShy       = FALSE;
    if (dimension == 0){
	mbGroupingGroup = TRUE;
    }
    mpCSceneObjLink = NULL;
}

ParamGroup_cls::
~ParamGroup_cls(){
}

void ParamGroup_cls::
DestroyGUI(){
    if (mpCGUI != NULL){
	mpCGUI->Destroy();
	mpCGUI = NULL;
    }
    for (unsigned int i = 0; i < mVpCParamGUI.size(); i++)
	mVpCParamGUI[i]->Destroy();
    mVpCParamGUI.clear();
}

void ParamGroup_cls::
CopyParamGroupFromP(ParamGroup_cls *pParamGroup){
    mstrGroupLabel    = pParamGroup->mstrGroupLabel;
    mstrGroupLabelMLG = pParamGroup->mstrGroupLabelMLG;
    mControlMode    = pParamGroup->mControlMode;
    mbGroupingGroup = pParamGroup->mbGroupingGroup;
    mbShownOnTL     = pParamGroup->mbShownOnTL;
    //mbLinking       = pParamGroup->mbLinking;
    
    for (unsigned int i = 0; i < mVCParam.size(); i++)
	mVCParam[i].CopyFromP(&pParamGroup->mVCParam[i]);
}

void ParamGroup_cls::
ReplaceID(wxString strFrom, wxString strNew){
    mstrGroupLabelMLG.Replace(strFrom, strNew);
    for (unsigned int i = 0; i < mVCParam.size(); i++)
	mVCParam[i].mstrID.Replace(strFrom, strNew);
    for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	mVpCParamGroup[i]->ReplaceID(strFrom, strNew);
}

Parameters_cls* ParamGroup_cls::
GetPCParamFromID(wxString strID){
    Parameters_cls *pCParam = NULL;
    for (unsigned int i = 0; i < mVCParam.size(); i++){
	if (mVCParam[i].mstrID.IsSameAs(strID)) 
	    return &mVCParam[i];
    }
    for (unsigned int i = 0; i < mVpCParamGroup.size(); i++){
	pCParam = mVpCParamGroup[i]->GetPCParamFromID(strID);
	if (pCParam != NULL) return pCParam;
    }
    return pCParam;
}

void ParamGroup_cls::
OpenUndoBlock(wxString strTag){
    if (Parameters_cls::msbUndoCommandMake == TRUE)
	GCUndoList.OpenBlock(strTag);
}

void ParamGroup_cls::
CloseUndoBlock(wxString strTag){
    if (Parameters_cls::msbUndoCommandMake == TRUE)
	GCUndoList.CloseBlock(strTag);
}

void ParamGroup_cls::
MakeParamGUI(ParamGUICtrl_cls *pCParent, wxBoxSizer *pCBoxSizer, int depth){
    mpCGUI = new ParamGroupGUI_cls
	(this, pCParent, mstrGroupLabelMLG, depth, wxID_ANY, 
	 wxDefaultPosition, wxSize(-1, 12));
    
    for (int i = 0; i < mCDimension.mBody; i++){
	mVpCParamGUI.push_back
	    (new ParametersPanel_cls
	     (&mVCParam[i], pCParent, TRUE,
	      wxDefaultPosition, wxSize(-1, 16), wxID_ANY));
    }

    pCBoxSizer->Add(mpCGUI, 0, wxEXPAND);
    for (int i = 0; i < mCDimension.mBody; i++){
	pCBoxSizer->Add(mVpCParamGUI[i], 0, wxEXPAND);
    }

    mpCGUI->UpdateButtonStatus();

}

void ParamGroup_cls::
SetModeShy(bool bModeShy){
    mbModeShy = bModeShy;
    mpCGUI->mpCCtrlLink->HideAndShow();
}

void ParamGroup_cls::
SetShownOnTLRecursive(bool bShownOnTL){
    mbShownOnTL = bShownOnTL;
    for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	mVpCParamGroup[i]->SetShownOnTLRecursive(bShownOnTL);
}

void ParamGroup_cls::
HideItems(){
    mpCGUI->ShowButtons(FALSE);
    
    for (int i = 0; i < mCDimension.mBody; i++)
	mVpCParamGUI[i]->Show(FALSE);

    for (unsigned int i = 0; i < mVpCParamGroup.size(); i++){
	mVpCParamGroup[i]->mpCGUI->Show(FALSE);
	mVpCParamGroup[i]->HideItems();
    }
    mpCGUI->mpCBPanelFolder->mState = 1;
    mpCGUI->mpCCtrlLink->ResizeTL();
}

void ParamGroup_cls::
ShowItems(){
    mpCGUI->ShowButtons(TRUE);

    for (int i = 0; i < mCDimension.mBody; i++)
	mVpCParamGUI[i]->Show(TRUE);

    for (unsigned int i = 0; i < mVpCParamGroup.size(); i++){
	mVpCParamGroup[i]->mpCGUI->Show(TRUE);
	if (mVpCParamGroup[i]->mpCGUI->mpCBPanelFolder->mState == 0)
	    mVpCParamGroup[i]->ShowItems();
    }
    mpCGUI->mpCCtrlLink->ResizeTL();
}

void ParamGroup_cls::
HideAndShowOnTL(){
    mpCGUI->mpCCtrlLink->HideAndShow();
}

void ParamGroup_cls::
RefreshPanels(){
    for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	mVpCParamGroup[i]->RefreshPanels();

    if (mbShownOnTL == FALSE) return;

    for (int i = 0; i < mCDimension.mBody; i++){
	if (mpCGUI->mpCBPanelFolder->mState == 0){
	    mVpCParamGUI[i]->UpdateDisplay();
	    mVpCParamGUI[i]->mpCParamTL->Refresh();
	}
    }
}

bool ParamGroup_cls::
SelectObject(SceneObj_cls* pCSceneObj){
    bool bRet = FALSE;
    
    if (mbGroupingGroup == TRUE)
	for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	    bRet |= mVpCParamGroup[i]->SelectObject(pCSceneObj);
    
    if (mpCSceneObjLink == NULL) return bRet;

    if (pCSceneObj == mpCSceneObjLink){
	if (mpCSceneObjLink->mbSelected == FALSE) {
	    mpCSceneObjLink->mbSelected = TRUE;
	    mpCSceneObjLink->mbDragging = FALSE;
	    mpCGUI->SetParamBackgroundColour();
	    bRet = TRUE;
	}
    }
    else{ 
	if (mpCSceneObjLink->mbSelected == TRUE){
	    mpCSceneObjLink->mbSelected = FALSE;
	    mpCGUI->SetParamBackgroundColour();
	    bRet = TRUE;
	}
    }
    
    return bRet;
}

SceneObj_cls* ParamGroup_cls::
GetPCObject(int objectIndex){
    SceneObj_cls *pCSelectedObject = NULL;
    
    if (mbGroupingGroup == TRUE)
	for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	    pCSelectedObject = mVpCParamGroup[i]->GetPCObject(objectIndex);
    
    if (mpCSceneObjLink == NULL) return pCSelectedObject;

    if (objectIndex == mpCSceneObjLink->mObjIndex)
	pCSelectedObject = mpCSceneObjLink;

    return pCSelectedObject;
}

bool ParamGroup_cls::
SelectObject(int objectIndex, int guideIndex){
    bool bRet = FALSE;
    
    if (mbGroupingGroup == TRUE)
	for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	    bRet |= mVpCParamGroup[i]->SelectObject(objectIndex, guideIndex);
    
    if (mpCSceneObjLink == NULL) return bRet;

    if (objectIndex == mpCSceneObjLink->mObjIndex){
	if (mpCSceneObjLink->mbSelected == FALSE) {
	    mpCSceneObjLink->mbSelected = TRUE;
	    mpCSceneObjLink->mbDragging = FALSE;
	    mpCGUI->SetParamBackgroundColour();
	    bRet = TRUE;
	}
    }
    else{ 
	if (mpCSceneObjLink->mbSelected == TRUE){
	    mpCSceneObjLink->mbSelected = FALSE;
	    mpCGUI->SetParamBackgroundColour();
	    bRet = TRUE;
	}
    }
    
    return bRet;
}

void ParamGroup_cls::
CheckSnippingTag(std::vector < bool > *pVbSnipTag){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].CheckSnippingTag(pVbSnipTag);
}

bool ParamGroup_cls::
IsConstant(){
    bool bConstant = TRUE;
    for (int i = 0; i < mCDimension.mBody; i++)
	if (mVCParam[i].mCbTimeDependent.mbBody == TRUE) bConstant = FALSE;
    return bConstant;
}

bool ParamGroup_cls::
IsKeyFrameAll(double doubleIndex){
    bool bIsKeyFrame = TRUE;
    for (int i = 0; i < mCDimension.mBody; i++)
	bIsKeyFrame &= mVCParam[i].IsKeyFrame(doubleIndex);
    return bIsKeyFrame;
}

wxString ParamGroup_cls::
GetStrExpression(){
    wxString strExp;
    for (int i = 0; i < mCDimension.mBody; i++){
	wxString strTag = mVCParam[i].mstrID;
	strTag.Replace(wxT(" "), wxT("_"));
	strExp << mVCParam[i].GetStrExpression(strTag);
    }
    return strExp;
}

void ParamGroup_cls::
Export(){
    wxString strExp = GetStrExpression();
    
    wxTextDataObject *pCData = new wxTextDataObject(strExp);
    wxTheClipboard->Open();
    wxTheClipboard->SetData(pCData);
    wxTheClipboard->Close();
}

void ParamGroup_cls::
CopyToClipboard(){
    wxString strCpy = Serialize();

    wxTextDataObject *pCData = new wxTextDataObject(strCpy);
    wxTheClipboard->Open();
    wxTheClipboard->SetData(pCData);
    wxTheClipboard->Close();
}

void ParamGroup_cls::
PasteFromClipboard(){
    if (wxTheClipboard->IsSupported(wxDataFormat(wxDF_TEXT)) == FALSE) return;
    wxTextDataObject CData;
    wxTheClipboard->Open();
    wxTheClipboard->GetData(CData);
    wxTheClipboard->Close();
    wxString strData = CData.GetText();

    if (strData.Matches(wxT("-ParamGroup=*")) == FALSE) return;

    int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_COLOR | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_ATTRIB | _COMMAND_UPDATE_SNAPSHOT_DATA;
	
    ParamGroup_cls CParam(mstrGroupLabel,
			  mCDimension.mBody,
			  mVSubLabel.data(), updateBit, TRUE);
    
    Parameters_cls::msbUndoCommandMake = FALSE;
    CParam.CopyFromP(this);
    Parameters_cls::msbUndoCommandMake = TRUE;

    std::wstring wstrLine;
    std::wistringstream istream((wchar_t*)strData.c_str());
    std::getline(istream, wstrLine);
    wstrLine = wxT("-ParamGroup=ANY");
    CParam.ReadStrLine(istream, wstrLine.c_str());

    CopyFromP(&CParam);
}

void ParamGroup_cls::
CreateKeyDLG(wxPoint posPanel, int xpos, int ypos, 
	     int frame, double doubleFrame){

    wxString strLabel = mstrGroupLabelMLG;
    MultiInputDialog_cls *pCDLG = new
	MultiInputDialog_cls(mpCGUI, wxID_ANY, strLabel,
			     posPanel + wxPoint(xpos, ypos), wxDS,
			     FontProvider_cls::GetFont(0));
    MultiInputPanel_cls *pCurr = pCDLG->mpCPanel;
    pCDLG->SetIcon(ParamTL_cls::msCIcon);

    double paramCache[4]; // FIX_ME!!
    for (int i = 0; i < mCDimension.mBody; i++)
	paramCache[i] = mVCParam[i].GetInterpolatedValue(doubleFrame);

    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;

    pCurr->CombinePointer(_ITEM_INTEGER, &frame, _("Frame"), FALSE);
    for (int i = 0; i < mCDimension.mBody; i++)
	pCurr->CombinePointer(_ITEM_DOUBLE, &paramCache[i], 
			      mVCParam[i].mstrLabel);

    mTypeCache = mVCParam[0].GetInterpTypeFromDouble(doubleFrame);
    mTypeCacheOrg = mTypeCache;
    int indexType = pCurr->CombinePointerList
	(&mTypeCache, 5, KeyFrame_cls::msAstrInterp, _("Interpolate Type"));
    
    int boxWidth = 2;
    int keyIndex[4];
    mpDValue1Cache = new double [mCDimension.mBody];
    mpDValue2Cache = new double [mCDimension.mBody];

    for (int i = 0; i < mCDimension.mBody; i++){
	keyIndex[i] = 
	    mVCParam[i].GetKeyFrameIndexFromDouble(doubleFrame);
	if (keyIndex[i] == -1)
	    mpDValue2Cache[i] = 0;
	else
	    mpDValue2Cache[i] = mVCParam[i].mVCKeys[keyIndex[i]].mDValue2;
	
	int index = pCurr->SetInputAndSlider(_ITEM_DOUBLE, &mpDValue2Cache[i], wxT("Ease Factor"));
	pCurr->SetRange(_ITEM_LASTITEM, 0, 30, 10);
	if (i == 0) mIndexHermiteParam = index;
    }

    for (int i = 0; i < mCDimension.mBody; i++){
	keyIndex[i] = 
	    mVCParam[i].GetKeyFrameIndexFromDouble(doubleFrame);
	if (keyIndex[i] == -1)
	    mpDValue1Cache[i] = 0;
	else
	    mpDValue1Cache[i] = mVCParam[i].mVCKeys[keyIndex[i]].mDValue1;
	
	wxString strVelocity = wxT("v");
	strVelocity << mVCParam[i].mstrLabel;
	mIndexVelocity[i] = pCurr->CombinePointer(_ITEM_DOUBLE, &mpDValue1Cache[i], strVelocity);
    }
    
    pCurr->MakeGeometry();
    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX, indexType, pCDLG);
    HideAndShowParameters(pCDLG);
    
    pCurr->UpdateValues();
    if (pCDLG->ShowModal() != wxID_OK){ 
	delete [] mpDValue1Cache;
	delete [] mpDValue2Cache;
	return;
    }

    // EaseFactor͑ŜłP݂̂Ƃ
    for (int i = 0; i < mCDimension.mBody; i++)
	mpDValue2Cache[i] = mpDValue2Cache[0];

    bool bChanged = FALSE;
    bChanged |= (mTypeCache != mTypeCacheOrg);
    for (int i = 0; i < mCDimension.mBody; i++){
	mVCParam[i].mInterpolatedValue = paramCache[i];
	bChanged |= (mVCParam[i].IsValueChanged() == TRUE);
	if (keyIndex[i] != -1){
	    bChanged |= (mVCParam[i].mVCKeys[keyIndex[i]].mDValue1 != mpDValue1Cache[i]);
	    bChanged |= (mVCParam[i].mVCKeys[keyIndex[i]].mDValue2 != mpDValue2Cache[i]);
	}
    }
    
    if (bChanged == TRUE){
	OpenUndoBlock(wxT("Create Key Dialog"));
	for (int i = 0; i < mCDimension.mBody; i++){
	    //mVCParam[i].SetKeyType(keyIndex[i], mTypeCache);
	    mVCParam[i].SetChangedValueKey(doubleFrame, mpDValue1Cache[i], mpDValue2Cache[i], _PARAM_USE_COMMON, mTypeCache);
	}
	CloseUndoBlock(wxT("Create Key Dialog"));
    }

    delete [] mpDValue1Cache;
    delete [] mpDValue2Cache;
}

void ParamGroup_cls::
HideAndShowParameters(void *pArg){
    MultiInputDialog_cls* pCDLG = (MultiInputDialog_cls*)pArg;
    
    int dimension = (int)mVCParam.size();
    
    for (int i = 0; i < dimension; i++)
	pCDLG->HideItem(mIndexHermiteParam + i*3); //sizer
    for (int i = 0; i < dimension; i++)
	pCDLG->HideItem(mIndexVelocity[i]); //sizer

    if (mTypeCache == _PARAM_HERMITE_ADJ){
	// EaseFactor͑Ŝ1݂̂Ƃ
	for (int i = 0; i < 1; i++)
	    pCDLG->ShowItem(mIndexHermiteParam + i*3);
    }
    if (mTypeCache == _PARAM_HERMITE){
	for (int i = 0; i < dimension; i++)
	    pCDLG->ShowItem(mIndexVelocity[i]);
    }
    
    pCDLG->Layout();
    pCDLG->ResizeDialog();
    pCDLG->UpdateValues();
    SizeAdjuster_cls::AdjustWindowFitting(pCDLG);
}

void ParamGroup_cls::
HookFuncHS(int id, void *pArg){
    HideAndShowParameters(pArg);
}

void ParamGroup_cls::
OpenLua(){
    if (mpLua != NULL) CloseLuaBase();

    wxString strFunction, strLabels;
    for (int i = 0; i < mCDimension.mBody; i++){
	if (i != 0) strLabels << wxT(",");
	strLabels << mVCParam[i].mstrLabel;
    }

    strFunction << wxT("function script(t,nt,") << strLabels << wxT(")") << ENDL;
    strFunction << mCstrExpression.mstrBody << ENDL;
    strFunction << wxT("end") << ENDL;

    mstrExpressionDefined = mCstrExpression.mstrBody;
    
    OpenLuaBase(strFunction);
}

std::vector < double > ParamGroup_cls::
CallLuaExpression(std::vector < double > VOrg){
    std::vector < double > VValue;
    if (mpLua == NULL) return VOrg;

    lua_getglobal(mpLua, "script");
    lua_pushnumber(mpLua, Parameters_cls::msCurrentTime);
    lua_pushnumber(mpLua, Parameters_cls::msCurrentDoubleIndex);
    
    int dimension = mCDimension.mBody;
    for (int i = 0; i < dimension; i++) lua_pushnumber(mpLua, VOrg[i]);

    if (lua_pcall(mpLua, 2+dimension, dimension, 0) != 0){
	CheckErrorMessage(mpLua, mstrGroupLabel);
	CloseLuaBase();
	return VOrg;
    }

    for (int i = 0; i < dimension; i++)
	VValue.push_back(lua_tonumber(mpLua, -dimension+i));
    lua_pop(mpLua, dimension);
    return VValue;
}

std::vector < double > ParamGroup_cls::
GetExpressionValue(double valueX, double resolution, bool bRepeat){
    if (mCbExpression.mbBody == FALSE)
	return GetInterpolatedValue(valueX, resolution, bRepeat);

    if (mstrExpressionDefined.IsSameAs(mCstrExpression.mstrBody) == FALSE)
	OpenLua();
    
    std::vector < double > VValue = CallLuaExpression
	(GetInterpolatedValue(valueX, resolution, bRepeat));
    
    return VValue;
}

std::vector < double > ParamGroup_cls::
GetInterpolatedValue(double valueX, double resolution, bool bRepeat){
    std::vector < double > VValue;
    if (valueX == -1)
	for (int i = 0; i < mCDimension.mBody; i++)
	    VValue.push_back(mVCParam[i].mInterpolatedValue);
    else
	for (int i = 0; i < mCDimension.mBody; i++)
	    VValue.push_back(mVCParam[i].GetInterpolatedValue(valueX, resolution, bRepeat));
    return VValue;
}

void ParamGroup_cls::
GetInterpolatedValue4Fast(float *pValue, 
			  double valueX, double resolution, bool bRepeat){
    for (int i = 0; i < 4; i++)
	pValue[i] = mVCParam[i].GetInterpolatedValue(valueX, resolution, bRepeat);
}

double ParamGroup_cls::
GetNextKeyPosDoubleIndex(double doubleIndex, double resolution){
    double candid = 1.0;
    LOOP_RGBA 
	if (candid + resolution/2 > mVCParam[rgba].GetNextKeyPosDoubleIndex(doubleIndex, resolution))
	    candid = mVCParam[rgba].GetNextKeyPosDoubleIndex(doubleIndex, resolution);
    return candid;
}

double ParamGroup_cls::
GetPrevKeyPosDoubleIndex(double doubleIndex, double resolution){
    double candid = 0;
    LOOP_RGBA 
	if (candid - resolution/2< mVCParam[rgba].GetPrevKeyPosDoubleIndex(doubleIndex, resolution))
	    candid = mVCParam[rgba].GetPrevKeyPosDoubleIndex(doubleIndex, resolution);
    return candid;
}

void ParamGroup_cls::
SetTimeDependent(bool bTimeDependent){
    OpenUndoBlock(wxT("SetTimeDependent"));
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].SetTimeDependent(bTimeDependent);
    CloseUndoBlock(wxT("SetTimeDependent"));
}

void ParamGroup_cls::
SetKeyFrame(double *pValue, bool bForce, double doubleIndex){
    bool bChanged = bForce;
    for (int i = 0; i < mCDimension.mBody; i++){
	mVCParam[i].mInterpolatedValue = pValue[i];
	bChanged |= mVCParam[i].IsValueChanged(doubleIndex);
    }
    
    if (bChanged == TRUE){
	OpenUndoBlock(wxT("SetKeyFrame"));
	for (int i = 0; i < mCDimension.mBody; i++){
	    if ((mControlMode != -1) && (i != mControlMode)) continue;
	    mVCParam[i].SetChangedValueKey(doubleIndex);
	}
	CloseUndoBlock(wxT("SetKeyFrame"));
    }
}

void ParamGroup_cls::
SetKeyFrameIfChanged(double doubleIndex){
    if (mCbLinking.mbBody == TRUE){
	bool bChanged = FALSE;
	for (int i = 0; i < mCDimension.mBody; i++)
	    if (mVCParam[i].IsValueChanged(doubleIndex) == TRUE) 
		bChanged = TRUE;
	if (bChanged == TRUE){
	    GCUndoList.OpenBlock();
	    for (int i = 0; i < mCDimension.mBody; i++)
		mVCParam[i].SetChangedValueKey(doubleIndex);
	    GCUndoList.CloseBlock();
	}
	return;
    }
    for (int i = 0; i < mCDimension.mBody; i++){
	if (mVCParam[i].IsValueChanged(doubleIndex) == TRUE)
	    mVCParam[i].SetChangedValueKey(doubleIndex);
    }
}

void ParamGroup_cls::
RemapKeys(double rate){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].RemapKeys(rate);

    // ParamGUICtrl͑SĂPramGroupĂ΂̂ŁA
    // qParamGroupremapKv
}

// ----

void ParamGroup_cls::
DeselectAll(){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].DeselectAll();
}

void ParamGroup_cls::
InsertKey(int index, double valueX, double valueY, double resolution){
    if (index == _PARAM_ALL){
	OpenUndoBlock(wxT("InsertKey"));
	for (int i = 0; i < mCDimension.mBody; i++){
	    if ((mControlMode != -1) && (i != mControlMode)) continue;
	    mVCParam[i].InsertKey(valueX, mVCParam[i].GetInterpolatedValue(valueX), 0, 0, resolution);
	}
	CloseUndoBlock(wxT("InsertKey"));
    }
    else if (valueY != _PARAM_MAGICNUM){
	// łɂ̈ʒȕꏊɂׂL[
	// XLbv
	if (mVCParam[index].IsKeyFrame(valueX, 0.03) == TRUE &&
	    mVCParam[index].GetInterpolatedValue(valueX) == valueY)
	    return;
	mVCParam[index].InsertKey(valueX, valueY);
	
    }
    else {
	if (mVCParam[index].IsKeyFrame(valueX, 0.03) == TRUE)
	    return;
	mVCParam[index].InsertKey(valueX, mVCParam[index].GetInterpolatedValue(valueX));
    }
}

void ParamGroup_cls::
DeleteSelectedKey(){
    OpenUndoBlock(wxT("DeleteSelectedKey"));
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].DeleteSelectedKey();
    CloseUndoBlock(wxT("DeleteSelectedKey"));
}

bool ParamGroup_cls::
DeleteKey(double doubleIndex){
    bool bDeleted = FALSE;
    OpenUndoBlock(wxT("DeleteKey"));
    for (int i = 0; i < mCDimension.mBody; i++)
	bDeleted |= mVCParam[i].DeleteKey(doubleIndex);
    CloseUndoBlock(wxT("DeleteKey"));
    return bDeleted;
}

bool ParamGroup_cls::
SelectKeyFrameRegion(double doubleIndexA, double doubleIndexB,
		     double valueA, double valueB,
		     bool bSelect){
    bool bSelected = FALSE;
    for (int i = 0; i < mCDimension.mBody; i++){
	if ((mControlMode != -1) && (i != mControlMode)) continue;
	bSelected |= mVCParam[i].SelectKeyFrameRegion
	    (doubleIndexA, doubleIndexB, valueA, valueB, bSelect);
    }
    return bSelected;
}

void ParamGroup_cls::
SelectKeyFrame(double doubleIndex,  double resolutionByPixel,
	       bool   bMultiSelect, bool bDeselect){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].SelectKeyFrame
	    (doubleIndex, resolutionByPixel, bMultiSelect, bDeselect);
}

bool ParamGroup_cls::
SelectSelectedKeyByXY(double valueX, double resolutionX,
		      double valueY, double resolutionY){
    bool bSelected = FALSE;
    for (int i = 0; i < mCDimension.mBody; i++){
	bSelected |= mVCParam[i].SelectSelectedKeyByXY(valueX, resolutionX,
						      valueY, resolutionY);
    }
    return bSelected;
}

bool ParamGroup_cls::
SelectKeyByXY(double valueX, double resolutionX,
	      double valueY, double resolutionY, bool bSingleSelect){
    bool bSelected = FALSE;
    for (int i = 0; i < mCDimension.mBody; i++){
	if ((mControlMode != -1) && (i != mControlMode)) continue;
	bSelected |= mVCParam[i].SelectKeyByXY(valueX, resolutionX,
					       valueY, resolutionY);
	if (bSelected == TRUE && bSingleSelect == TRUE)
	    return TRUE;
    }
    return bSelected;
}

bool ParamGroup_cls::
MoveSelectedKeyByXY(double dX, double dY, double resolution,
		    double minX, double maxX, double minY, double maxY){
    bool bRetval = FALSE;
    OpenUndoBlock(wxT("MoveSelectedKeyByXY"));
    for (int i = 0; i < mCDimension.mBody; i++)
	bRetval |= mVCParam[i].MoveSelectedKeyByXY(dX, dY, resolution,
						   minX, maxX, minY, maxY);
    CloseUndoBlock(wxT("MoveSelectedKeyByXY"));
    return bRetval;
}

void ParamGroup_cls::
CopyFromP(ParamGroup_cls *pCParamGroup){
    if (mCDimension.mBody != pCParamGroup->mCDimension.mBody) return;

    mCConfList.CopyListComponentFromP(&pCParamGroup->mCConfList);

    OpenUndoBlock(wxT("CopyFromP"));
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].CopyParameterFromP(&pCParamGroup->mVCParam[i]);
    CloseUndoBlock(wxT("CopyFromP"));
}

void ParamGroup_cls::
Flip(){
    OpenUndoBlock(wxT("Flip"));
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].Flip();
    CloseUndoBlock(wxT("Flip"));
}

void ParamGroup_cls::
Swap(int dimension){
    if (mCDimension.mBody == 1) return;
    if (dimension > mCDimension.mBody) return;
    Parameters_cls CTmp(mVCParam[0]);
    OpenUndoBlock(wxT("Swap"));
    for (int i = 0; i < dimension - 1; i++)
	mVCParam[i].CopyParameterFromP(&mVCParam[i+1]);
    mVCParam[dimension-1].CopyParameterFromP(&CTmp);
    CloseUndoBlock(wxT("Swap"));
}

void ParamGroup_cls::
ProcStartDragging(double origin, bool bCtrl, bool bAlt){
    OpenUndoBlock();
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].mpCParamTLLink->ProcStartDragging(origin, bCtrl, bAlt);
    CloseUndoBlock();
}

void ParamGroup_cls::
ProcDragging(wxMouseEvent& event){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].mpCParamTLLink->ProcDragging(event);
}

void ParamGroup_cls::
SetRegionOrigin(double origin){
    for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].mpCParamTLLink->SetRegionOrigin(origin);
}

void ParamGroup_cls::
TurnOffDragging(){
   for (int i = 0; i < mCDimension.mBody; i++)
	mVCParam[i].mpCParamTLLink->TurnOffDragging();
}

// ----

void ParamGroup_cls::
CopyKeyFrameToClipBoard(){
    // IMPLEMENT_ME !!
}

void ParamGroup_cls::
PasteKeyFrameFromClipBoard(){
   // IMPLEMENT_ME !!
}

void ParamGroup_cls::
OpenExpressionDialog(){
    wxString strMessage, strLabels;
    for (int i = 0; i < mCDimension.mBody; i++){
	if (i != 0) strLabels << wxT(",");
	strLabels << mVCParam[i].mstrLabel;
    }

    strMessage << wxT("-----") << _("Predefined Variables") << ENDL;
    strMessage << wxT("t = time") << ENDL;
    strMessage << wxT("nt = normalized time") << ENDL;
    strMessage << strLabels << wxT(" = original values") << ENDL;
    //strMessage << wxT("index = copy index") << ENDL;
    //strMessage << wxT("(nx, ny, nz) = coordinate index") << ENDL;
    strMessage << wxT("-----") << _("Return Value") << ENDL;
    strMessage << strLabels;
    
    TextEntryDialogEX_cls CDlg(mpCGUI, strMessage, _("Expression Editor"), mCstrExpression.mstrBody);
    
    if (CDlg.ShowModal() == wxID_OK){
	wxString strEdit = CDlg.GetValue().Trim(TRUE);
	//FilterCopyData_cls *pCData = GetPCData();
	//int updateBit = _COMMAND_UPDATE_DISPLAY_GL;
	//wxString strTag = mpCParentCtrl->GetHeaderStrFromPointer(&pCData->mCstrExpression.mstrBody);
	GCUndoList.PushNewCommand
	    (new ComParamSetExpression_cls(&mVCParam[0], strEdit), 
	     new ComParamSetExpression_cls(&mVCParam[0], mCstrExpression.mstrBody));
	GCUndoList.ExecCurrent();
    }
}

wxString ParamGroup_cls::
Serialize(){
    if (mbGroupingGroup == TRUE) return SerializeChilds();

    wxString strOut;
    strOut << wxT("-ParamGroup=") << mstrGroupLabel << ENDL;
    strOut << mCConfList.SerializeAll();
    strOut << wxT("Start") << ENDL;
    for (int i = 0; i < mCDimension.mBody; i++)
	strOut << mVCParam[i].Serialize();
    return strOut;
}

wxString ParamGroup_cls::
SerializeChilds(){
    wxString strOut;
    for (unsigned int i = 0; i < mVpCParamGroup.size(); i++)
	strOut << mVpCParamGroup[i]->Serialize();

    return strOut;
}

void ParamGroup_cls::
ReadStrLine(std::wistream &istream, wxString strLineTag){
    wxString strTag;
    strTag << wxT("-ParamGroup=") << mstrGroupLabel;
    if (strLineTag.IsSameAs(strTag) ||
	strLineTag.IsSameAs(wxT("-ParamGroup=ANY"))){
	std::wstring wstrLine;
	wxString strLine;
	while ( std::getline (istream, wstrLine)){
	    strLine = wstrLine.c_str();
	    strLine.Trim();
	    if (strLine.IsSameAs(wxT("Start"))) break;
	    mCConfList.ReadStrLineAll(strLine);
	}
	for (int i = 0; i < mCDimension.mBody; i++){
	    std::getline (istream, wstrLine);
	    strLine = wstrLine.c_str();
	    mVCParam[i].ReadStrLine(istream, strLine);
	}
    }
    if (mpCGUI != NULL)
	mpCGUI->UpdateButtonStatus();
}

// ----------

bool ParamGroupStatus_cls::
IsParamGroupChanged(ParamGroup_cls *pCParamGroup){
    int dim = pCParamGroup->mCDimension.mBody;
    if (mVPrevStamp.size() != dim){
	mVPrevStamp.resize(dim);
	for (int i = 0; i < dim; i++)
	    mVPrevStamp[i] = pCParamGroup->mVCParam[i].mStamp;
	return TRUE;
    }
    bool bRet = FALSE;
    for (int i = 0; i < dim; i++)
	if (mVPrevStamp[i] != pCParamGroup->mVCParam[i].mStamp){
	    bRet = TRUE;
	    mVPrevStamp[i] = pCParamGroup->mVCParam[i].mStamp;
	}
    return bRet;
}
