// -*- c -*-
// "clsParamGroup.cpp"

#ifndef _PARAM_GROUP_H
#define _PARAM_GROUP_H

#include "../CONFITEM/clsConfItem.h"
#include "../LUA/clsLuaBase.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "clsParameters.h"

#include <vector>
#include <fstream>

class ParametersPanel_cls;
class ParamGroup_cls;
class ParamGroupGUI_cls;
class ParamGUICtrl_cls;
class SceneObj_cls;
class TimeCtrl_cls;

#define _PARAM_ALL -1
#define _PARAM_MAGICNUM -9999

class ParamGroup_cls : public HookItem_cls, public LuaBase_cls {
 public:
    ParamGroup_cls(wxString strGroutpLabel, 
		   int dimension, wxString *pstrSubLabel,
		   long long updateBit, bool bTimeDependent = FALSE);
    ~ParamGroup_cls();
    
    void DestroyGUI();
    
    std::vector < Parameters_cls >       mVCParam;
    std::vector < ParametersPanel_cls* > mVpCParamGUI;
    std::vector < ParamGroup_cls* >      mVpCParamGroup;
    std::vector < wxString > mVSubLabel;
    wxString mstrGroupLabel;
    wxString mstrGroupLabelMLG;
    ParamGroupGUI_cls *mpCGUI;
    SceneObj_cls *mpCSceneObjLink;
    int mControlMode;
    bool mbGroupingGroup; // O[v̏W
    bool mbUseTLGUI;
    bool mbShownOnTL;
    //bool mbLinking;
    bool mbObjectVisible;
    bool mbModeShy;
    
    double       *mpDValue1Cache;
    double       *mpDValue2Cache;
    unsigned int mTypeCache, mTypeCacheOrg;
    int mIndexHermiteParam;
    int mIndexVelocity[4];

    void CopyParamGroupFromP(ParamGroup_cls *pParamGroup);
    void ReplaceID(wxString strFrom, wxString strNew);
    Parameters_cls* GetPCParamFromID(wxString strID);

    void OpenUndoBlock(wxString strTag = wxT(""));
    void CloseUndoBlock(wxString strTag = wxT(""));

    void MakeParamGUI(ParamGUICtrl_cls *pCParent, wxBoxSizer *pCBoxSizer,
		      int depth = 0);
    void SetModeShy(bool bModeShy);
    void SetShownOnTLRecursive(bool bShownOnTL);
    void HideItems();
    void ShowItems();
    void HideAndShowOnTL();
    void RefreshPanels();
    bool SelectObject(SceneObj_cls* pCSceneObj);
    SceneObj_cls* GetPCObject(int objectIndex);
    bool SelectObject(int objectIndex, int guideIndex);
    void CheckSnippingTag(std::vector < bool > *pVbSnipTag);
    bool IsConstant();
    bool IsKeyFrameAll(double doubleFrame);
    wxString GetStrExpression();
    void Export();
    void CopyToClipboard();
    void PasteFromClipboard();
    void CreateKeyDLG(wxPoint posPanel, int xpos, int ypos, 
		      int frame, double doubleFrame);

    void HookFuncHS(int id, void *pArg);
    void  HideAndShowParameters(void* pArg);

    std::vector < double > CallLuaExpression(std::vector < double > VOrg);
	
    void OpenLua();
    std::vector < double > GetExpressionValue
	(double valueX, double resolution, bool bRepeat = false);
    std::vector < double > GetInterpolatedValue
	(double valueX, double resolution, bool bRepeat = false);
    void GetInterpolatedValue4Fast(float *pValue, 
				   double valueX, double resolution, bool bRepeat = false);
    double GetNextKeyPosDoubleIndex(double doubleIndex, double resolution);
    double GetPrevKeyPosDoubleIndex(double doubleIndex, double resolution);
    void SetTimeDependent(bool bTimeDependent);
    void SetKeyFrame(double *pValue, bool bForce = FALSE,
		     double doubleIndex = _PARAM_CURRENT);
    void SetKeyFrameIfChanged(double doubleIndex = _PARAM_CURRENT);
    void RemapKeys(double rate);

    // ----

    void DeselectAll();
    void InsertKey(int index, double valueX, double valueY = _PARAM_MAGICNUM, double resolution = _PARAM_CURRENT);
    void DeleteSelectedKey();
    bool DeleteKey(double doubleIndex);
    bool SelectKeyFrameRegion(double doubleIndexA, double doubleIndexB,
			      double valueA, double valueB,
			      bool bSelected = TRUE);
    void SelectKeyFrame(double doubleIndex,  double resolutionByPixel,
			bool   bMultiSelect, bool bDeselect);
    bool SelectSelectedKeyByXY(double valueX, double resolutionX,
			       double valueY, double resolutionY);
    bool SelectKeyByXY(double valueX, double resolutionX,
		       double valueY, double resolutionY, 
		       bool bSingleSelect = TRUE);
    bool MoveSelectedKeyByXY(double dX, double dY, double resolution,
			     double minX = 0, double maxX = 0,
			     double minY = 0, double maxY = 0);
    void CopyFromP(ParamGroup_cls *pCParamGroup);
    void Flip();
    void Swap(int dimension);

    void ProcStartDragging(double origin, bool bCtrl, bool bAlt);
    void ProcDragging(wxMouseEvent& event);
    void SetRegionOrigin(double origin);
    void TurnOffDragging();

    // ----

    void CopyKeyFrameToClipBoard();
    void PasteKeyFrameFromClipBoard();
    
    ConfItemList_cls mCConfList;
    ConfItemINT_cls mCDimension;
    ConfItemBOOL_cls mCbLinking;
    ConfItemBOOL_cls mCbExpression;
    ConfItemSTR_cls mCstrExpression;
    wxString mstrExpressionDefined;
	
    void OpenExpressionDialog();
    
    wxString Serialize();
    wxString SerializeChilds();
    void ReadStrLine(std::wistream &istream, wxString strLineTag);
};


// Xe[^Xς`FbN邽߂̃NX
class ParamGroupStatus_cls{
 public:
    std::vector < unsigned int > mVPrevStamp;

    bool IsParamGroupChanged(ParamGroup_cls *pCParamGroup);
};

#endif
