// -*- c -*- 
// "clsTics.h"

#include "clsTics.h"

#include "../WIDGETCTRL/WX/clsFontProvider.h"

Tics_cls::
Tics_cls(){
    mBGColor  = wxColour(216, 216, 216);
    mFGColor1 = wxColour(32, 32, 32);
    mFGColor2 = wxColour(128, 128, 128);
}

void Tics_cls::
SetParameter(wxSize size, float min, float max, int offset){
    
    mTicWidth  = size.GetWidth();
    mTicHeight = size.GetHeight();

    float width = max - min;
    mRegionMin = min;
    mRegionMax = max;
    mTicOffset = offset;
    
    if (fabs(width) < 2) mbUseFloat = TRUE;
    else                mbUseFloat = FALSE;

    float ratio = fabs(width) / mTicWidth;

    if (mRegionMax > 1000) ratio *= 2;
		       
    mStepLarge = 1;

    mStepFloatOrder = 1;
    float stepOrder = 1;
    while (fabs(width) < stepOrder * 1.01){
	mStepFloatOrder ++;
	stepOrder /= 10;
    }

    mStepFloatScaling = 0.1f;
    float floatScaling = 0.1f;
    while (fabs(width) < mStepFloatScaling * 5){
	mStepFloatScaling /= 10;
    }
    if (width / mStepFloatScaling > 10) mStepFloatScaling *= 2;
    if (width / mStepFloatScaling > 10) mStepFloatScaling *= 5;
    
    if (ratio > 0.02)  mStepLarge = 2;
    if (ratio > 0.05)  mStepLarge = 5;
    if (ratio > 0.10)  mStepLarge = 10;
    if (ratio > 0.20)  mStepLarge = 20;
    if (ratio > 0.50)  mStepLarge = 50;
    if (ratio > 1.00)  mStepLarge = 100;
    if (ratio > 2.00)  mStepLarge = 200;
    if (ratio > 5.00)  mStepLarge = 500;
    if (ratio > 10.00)  mStepLarge = 1000;
    if (ratio > 20.00)  mStepLarge = 2000;
    if (ratio > 100.00)  mStepLarge = 10000;
    if (ratio > 200.00)  mStepLarge = 20000;
    if (ratio > 500.00)  mStepLarge = 50000;
    if (ratio > 1000.00)  mStepLarge = 100000;
    if (ratio > 2000.00)  mStepLarge = 200000;
    if (ratio > 5000.00)  mStepLarge = 500000;
    if (ratio > 10000.00)  mStepLarge = 1000000;
    if (ratio > 20000.00)  mStepLarge = 2000000;
    if (ratio > 50000.00)  mStepLarge = 5000000;
    if (ratio > 100000.00)  mStepLarge = 10000000;
    if (ratio > 200000.00)  mStepLarge = 20000000;
    if (ratio > 500000.00)  mStepLarge = 50000000;
    if (ratio > 1000000.00)  mStepLarge = 100000000;
    
    mStepMiddle = mStepLarge / 2;
    mStepSmall = mStepLarge / 10; 
    if (mStepLarge == 5) mStepMiddle = 5;
    if (mStepSmall == 0) mStepSmall = 1;
    if (mStepMiddle == 0) mStepMiddle = 1;
}

void Tics_cls::
SetColorParameter(wxColour BGColor, wxColour FGColor1, wxColour FGColor2){
    mBGColor = BGColor;
    mFGColor1 = FGColor1;
    mFGColor2 = FGColor2;
}

int Tics_cls::
GetTicDeviceX(float x){
    return (int)(GetMappedX(x) * (mTicWidth - 2. * mTicOffset) + mTicOffset);
}

void Tics_cls::
CalcTicDeviceCoordinate(int *pX0, int *pX1,
			float frame0, float frame1){
    if (mRegionMax == mRegionMin){
	*pX0 = 0; *pX1 = 1;
	return;
    }

    float diff0 = frame0 - mRegionMin;
    float diff1 = frame1 - mRegionMin;
    float width = mRegionMax - mRegionMin;
    float x0 = diff0/width;
    float x1 = diff1/width;
    *pX0 = GetTicDeviceX(x0); *pX1 = GetTicDeviceX(x1);
}

void Tics_cls::
Swap(int &tics0, int &tics1){
    int tmp = tics0;
    tics0 = tics1;
    tics1 = tmp;
}

void Tics_cls::
DrawTics(wxDC &dc){
    dc.SetBackground(wxBrush(mBGColor, wxSOLID));
    dc.Clear();

    if (mTicHeight > 100)
	dc.SetFont(FontProvider_cls::GetFont(7));
    if (mTicHeight > 50)
	dc.SetFont(FontProvider_cls::GetFont(6));
    if (mTicHeight > 30)
	dc.SetFont(FontProvider_cls::GetFont(5));
    else if (mTicHeight > 25)
	dc.SetFont(FontProvider_cls::GetFont(4));
    else if (mTicHeight > 20)
	dc.SetFont(FontProvider_cls::GetFont(1));
    else
	dc.SetFont(FontProvider_cls::GetFont(0));

    wxCoord w, h, singleH;
    dc.GetMultiLineTextExtent(wxT("0.0"), &w, &h, &singleH);
#ifdef WIN32
    int texOffsetY = singleH * 7 / 8;
#else
    int texOffsetY = singleH;
#endif

    dc.SetPen(mFGColor2);
    dc.SetTextForeground(mFGColor2);
    int prevTexEnd = -1;
    if (mbUseFloat == TRUE){
	float scale = mStepFloatScaling;
	int   tics0 = (int)(mRegionMin / scale);
	int   tics1 = (int)(mRegionMax / scale);
	if (tics1 < tics0) Swap (tics0, tics1);
	for (int i = tics0/*-1*/; i <= tics1/*+1*/; i++){
	    float value = i * scale;
	    CalcTicDeviceCoordinate(&mTicsX0, &mTicsX1, 
				    value, value);
	    dc.SetPen(mFGColor1);
		    
	    wxString strMes, strFormat;
	    if (mStepFloatOrder < 5)
		strFormat << wxT("%0.") << mStepFloatOrder << wxT("f");
	    else{
		strFormat << wxT("%.1e");
	    }
	    strMes << wxString::Format(strFormat, value);
	    if (strMes.Last() == '0') strMes.RemoveLast();
	    strMes.Replace(wxT("e+0"), wxT("e+"));
	    strMes.Replace(wxT("e-0"), wxT("e-"));
	    int mesOffset = 2;
	    if (prevTexEnd < mTicsX0 + mesOffset){
		dc.DrawText(strMes, mTicsX0+mesOffset, mTicHeight-texOffsetY);
		dc.DrawRectangle(mTicsX0, mTicHeight-singleH/4, 1, singleH/4);
		dc.GetMultiLineTextExtent(strMes, &w, &h, &singleH);
		prevTexEnd = mTicsX0+mesOffset+w;
	    }
	}
    }
    else{
	int   tics0 = (int)mRegionMin;
	int   tics1 = (int)mRegionMax;
	if (tics1 < tics0) Swap (tics0, tics1);
	for (int i = tics0/*-1*/; i <= tics1/*+1*/; i++){
	    if (i % mStepSmall != 0) continue;
	    CalcTicDeviceCoordinate(&mTicsX0, &mTicsX1, 
				    i, i);
	    if (mTicsX1 < 0 || mTicsX0 > mTicWidth) continue;
	    if (i % mStepMiddle == 0) 
		dc.SetPen(mFGColor1);
	    else
		dc.SetPen(mFGColor2);
	    if (i % mStepLarge == 0){
		wxString strMes;
		if (mStepLarge <= 1000)
		    strMes << i;
		else
		    strMes << wxString::Format(wxT("%.1e"), (float)i);
		strMes.Replace(wxT("e+0"), wxT("e+"));
		strMes.Replace(wxT("e-0"), wxT("e-"));

		int mesOffset = 2;
		if (prevTexEnd < mTicsX0 + mesOffset){
		    dc.DrawText(strMes, mTicsX0+mesOffset,mTicHeight-texOffsetY);
		    dc.DrawRectangle(mTicsX0, mTicHeight-singleH/4, 1,singleH/4);
		    dc.GetMultiLineTextExtent(strMes, &w, &h, &singleH);
		    prevTexEnd = mTicsX0+mesOffset+w;
		}
		continue;
	    }
	    dc.DrawRectangle(mTicsX0, mTicHeight-3, 1, 3);
	    if (mStepLarge > 100)
		i += (mStepLarge / 10 - 1);
	}
    }
}
