
#include "clsThreadInterpolateLinear.h"
#include "clsThreadInterpolate.h"
#include "../clsPeriodicBoundary.h"
#include "../../MYAPI/MsgBox.h"
#include "../../PANELS/clsDataLoader.h"

#include "../../../wxWidgetEnumMenu.h"

#include <omp.h>

#include "../../COMMON/CommonDefinitions.h"

ThreadInterpBlockLinear_cls::
ThreadInterpBlockLinear_cls(wxThreadKind kind) :
    wxThread(kind) {
}

void ThreadInterpBlockLinear_cls::
SetParameters(int PInit, int PFin,
	      int PNumNext, float dtime, int variablePNum,
	      int searchLimit,
	      PeriodicBoundary_cls *pCPeriodicLink,
	      BasicDataPointers_cls *pCDataPointers,
	      BasicDataPointers_cls *pCDataPointersNext){

    mVariablePNum = variablePNum;
    mSearchLimit = searchLimit;
    mPeriodicType = pCPeriodicLink->mCType.mBody;
    mPeriodicEffect = pCPeriodicLink->mCEffect.mBody;

    float boundaryVector[3][3];
    pCPeriodicLink->GetBoundaryVector(boundaryVector[X], X);
    pCPeriodicLink->GetBoundaryVector(boundaryVector[Y], Y);
    pCPeriodicLink->GetBoundaryVector(boundaryVector[Z], Z);

    // R͂e\
    LOOP_XYZ mPeriodicSize[xyz] = boundaryVector[xyz][xyz];

    mPIni = PInit;
    mPFin = PFin;
    pCDataPointers->CopyPointers(&mpX, &mpV, &mpA, &mpJ,
				 &mpRadius, &mpDRadius, 
				 &mpValue1, &mpDValue1,
				 &mpType, &mpID);

    mPNumNext = PNumNext;
    pCDataPointersNext->CopyPointers(&mpXNext, &mpVNext, &mpANext, &mpJNext,
				     &mpRadiusNext, &mpDRadiusNext,
				     &mpValue1Next, &mpDValue1Next,
				     &mpTypeNext, &mpIDNext);
    mDTime = dtime;
    
    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadInterpBlockLinear_cls::
InterpolateDataLinearCore(int i, int iNext, float dtime){

    float (*pX)[3] = mpX;
    float (*pV)[3] = mpV;
    float (*pXNext)[3] = mpXNext;

    pV[i][X] = (pXNext[iNext][X] - pX[i][X]) / dtime;
    pV[i][Y] = (pXNext[iNext][Y] - pX[i][Y]) / dtime;
    pV[i][Z] = (pXNext[iNext][Z] - pX[i][Z]) / dtime;
    mpDRadius[i] = (mpRadiusNext[iNext] - mpRadius[i]) / dtime;
    mpDValue1[i] = (mpValue1Next[iNext] - mpValue1[i]) / dtime;
}

void ThreadInterpBlockLinear_cls::
InterpolateDataLinearCorePeriodicXYEeffect0(int i, int iNext, float dtime){
    float dX = mpXNext[iNext][X] - mpX[i][X];
    if      (dX >  mPeriodicSize[X]/2) mpV[i][X] = dX - mPeriodicSize[X];
    else if (dX < -mPeriodicSize[X]/2) mpV[i][X] = dX + mPeriodicSize[X];
    else                               mpV[i][X] = dX;
    float dY = mpXNext[iNext][Y] - mpX[i][Y];
    if      (dY >  mPeriodicSize[Y]/2) mpV[i][Y] = dY - mPeriodicSize[Y];
    else if (dY < -mPeriodicSize[Y]/2) mpV[i][Y] = dY + mPeriodicSize[Y];
    else                               mpV[i][Y] = dY;
    
    mpV[i][Z] = (mpXNext[iNext][Z] - mpX[i][Z]) / dtime;
    mpV[i][X] /= dtime;
    mpV[i][Y] /= dtime;

    mpDRadius[i] = (mpRadiusNext[iNext] - mpRadius[i]) / dtime;
    mpDValue1[i] = (mpValue1Next[iNext] - mpValue1[i]) / dtime;
}

void ThreadInterpBlockLinear_cls::
InterpolateDataLinearCorePeriodicXYEeffect1(int i, int iNext, float dtime){
    float dX = mpXNext[iNext][X] - mpX[i][X];
    if      (dX >  mPeriodicSize[X]/2) mpV[i][X] = 0;
    else if (dX < -mPeriodicSize[X]/2) mpV[i][X] = 0;
    else                               mpV[i][X] = dX;
    float dY = mpXNext[iNext][Y] - mpX[i][Y];
    if      (dY >  mPeriodicSize[Y]/2) mpV[i][Y] = 0;
    else if (dY < -mPeriodicSize[Y]/2) mpV[i][Y] = 0;
    else                               mpV[i][Y] = dY;
    
    mpV[i][Z] = (mpXNext[iNext][Z] - mpX[i][Z]) / dtime;
    mpV[i][X] /= dtime;
    mpV[i][Y] /= dtime;
    
    mpDRadius[i] = (mpRadiusNext[iNext] - mpRadius[i]) / dtime;
    mpDValue1[i] = (mpValue1Next[iNext] - mpValue1[i]) / dtime; 
}

void ThreadInterpBlockLinear_cls::
InterpolateDataLinearCorePeriodicXYZEeffect0(int i, int iNext, float dtime){
    float dX = mpXNext[iNext][X] - mpX[i][X];
    if      (dX >  mPeriodicSize[X]/2) mpV[i][X] = dX - mPeriodicSize[X];
    else if (dX < -mPeriodicSize[X]/2) mpV[i][X] = dX + mPeriodicSize[X];
    else                               mpV[i][X] = dX;
    float dY = mpXNext[iNext][Y] - mpX[i][Y];
    if      (dY >  mPeriodicSize[Y]/2) mpV[i][Y] = dY - mPeriodicSize[Y];
    else if (dY < -mPeriodicSize[Y]/2) mpV[i][Y] = dY + mPeriodicSize[Y];
    else                               mpV[i][Y] = dY;
    float dZ = mpXNext[iNext][Z] - mpX[i][Z];
    if      (dZ >  mPeriodicSize[Z]/2) mpV[i][Z] = dZ - mPeriodicSize[Z];
    else if (dZ < -mPeriodicSize[Z]/2) mpV[i][Z] = dZ + mPeriodicSize[Z];
    else                               mpV[i][Z] = dZ;
    
    mpV[i][X] /= dtime;
    mpV[i][Y] /= dtime;
    mpV[i][Z] /= dtime;
    mpDRadius[i] = (mpRadiusNext[i] - mpRadius[i]) / dtime;
    mpDValue1[i] = (mpValue1Next[i] - mpValue1[i]) / dtime;
}

void ThreadInterpBlockLinear_cls::
InterpolateDataLinearCorePeriodicXYZEeffect1(int i, int iNext, float dtime){
    float dX = mpXNext[iNext][X] - mpX[i][X];
    if      (dX >  mPeriodicSize[X]/2) mpV[i][X] = 0;
    else if (dX < -mPeriodicSize[X]/2) mpV[i][X] = 0;
    else                               mpV[i][X] = dX;
    float dY = mpXNext[iNext][Y] - mpX[i][Y];
    if      (dY >  mPeriodicSize[Y]/2) mpV[i][Y] = 0;
    else if (dY < -mPeriodicSize[Y]/2) mpV[i][Y] = 0;
    else                               mpV[i][Y] = dY;
    float dZ = mpXNext[iNext][Z] - mpX[i][Z];
    if      (dZ >  mPeriodicSize[Z]/2) mpV[i][Z] = 0;
    else if (dZ < -mPeriodicSize[Z]/2) mpV[i][Z] = 0;
    else                               mpV[i][Z] = dZ;
    
    mpV[i][X] /= dtime;
    mpV[i][Y] /= dtime;
    mpV[i][Z] /= dtime;
    mpDRadius[i] = (mpRadiusNext[i] - mpRadius[i]) / dtime;
    mpDValue1[i] = (mpValue1Next[i] - mpValue1[i]) / dtime; 
}

void ThreadInterpBlockLinear_cls::
InterpolateDataLinear(){
    float dtime = mDTime;

    for (int i = mPIni; i < mPFin; i++){
	mpV[i][X] = (mpXNext[i][X] - mpX[i][X]) / dtime;
	mpV[i][Y] = (mpXNext[i][Y] - mpX[i][Y]) / dtime;
	mpV[i][Z] = (mpXNext[i][Z] - mpX[i][Z]) / dtime;
	mpDRadius[i] = (mpRadiusNext[i] - mpRadius[i]) / dtime;
	mpDValue1[i] = (mpValue1Next[i] - mpValue1[i]) / dtime;
    }
}

void ThreadInterpBlockLinear_cls::
InterpolateDataLinearPeriodic(){
    float dtime = mDTime;

    if (mPeriodicType == 1){
	if (mPeriodicEffect == 0)
	    for (int i = mPIni; i < mPFin; i++)
		InterpolateDataLinearCorePeriodicXYEeffect0(i, i, dtime);
	if (mPeriodicEffect == 1)
	    for (int i = mPIni; i < mPFin; i++)
		InterpolateDataLinearCorePeriodicXYEeffect1(i, i, dtime); }
    if (mPeriodicType == 2){
	if (mPeriodicEffect == 0)
	    for (int i = mPIni; i < mPFin; i++)
		InterpolateDataLinearCorePeriodicXYZEeffect0(i, i, dtime);
	if (mPeriodicEffect == 1)
	    for (int i = mPIni; i < mPFin; i++)
		InterpolateDataLinearCorePeriodicXYZEeffect1(i, i, dtime); }
}

void ThreadInterpBlockLinear_cls::
InterpolateDataLinearSearchingID(){
    float dtime = mDTime;
    
    void (ThreadInterpBlockLinear_cls::*pFunction)(int, int, float) = NULL;
    if (mPeriodicType == 0)
	pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCore;
    if (mPeriodicType == 1){
	if (mPeriodicEffect == 0) pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCorePeriodicXYEeffect0;
	if (mPeriodicEffect == 1) pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCorePeriodicXYEeffect1;
    }
    if (mPeriodicType == 2){
	if (mPeriodicEffect == 0) pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCorePeriodicXYZEeffect0;
	if (mPeriodicEffect == 1) pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCorePeriodicXYZEeffect1;
    }

    for (int i = mPIni; i < mPFin; i++){
	int ID = mpID[i];
	int iNext = -1;
	if (i < mPNumNext && ID == mpIDNext[i]) {
	    (this->*pFunction)(i, i, dtime);
	    continue;
	}
	iNext = SearchIDNearBy(i, ID, mSearchLimit);

	/*
	iNext = SearchIDNear(i, ID, 10);
	if (iNext == -1) iNext = SearchIDNear(i, ID, 100);
	if (mSearchLimit < 2){
	    if (iNext == -1) iNext = SearchIDNear(i, ID, 1000);
	    if (iNext == -1) iNext = SearchIDNear(i, ID, 10000);
	    if (mSearchLimit < 1)
		if (iNext == -1) iNext = SearchIDFull(ID);
	}
	*/
	if (iNext == -1){
	    mpV[i][X] = 0; mpV[i][Y] = 0; mpV[i][Z] = 0;
	    mpDRadius[i] = 0; mpDValue1[i] = 0;
	    continue;
	}
	
	(this->*pFunction)(i, iNext, dtime);
    }
}

void ThreadInterpBlockLinear_cls::
InterpolateDataLinearSearchingIDSorted(){
    float dtime = mDTime;
    
    void (ThreadInterpBlockLinear_cls::*pFunction)(int, int, float) = NULL;
    if (mPeriodicType == 0)
	pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCore;
    if (mPeriodicType == 1){
	if (mPeriodicEffect == 0) pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCorePeriodicXYEeffect0;
	if (mPeriodicEffect == 1) pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCorePeriodicXYEeffect1;
    }
    if (mPeriodicType == 2){
	if (mPeriodicEffect == 0) pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCorePeriodicXYZEeffect0;
	if (mPeriodicEffect == 1) pFunction = &ThreadInterpBlockLinear_cls::InterpolateDataLinearCorePeriodicXYZEeffect1;
    }

    int tgtIndex = 0;
    for (int i = mPIni; i < mPFin; i++){
	int ID = mpID[i];
	int iNext = SearchIDFrom(&tgtIndex, ID);
	if (iNext == -1){
	    mpV[i][X] = 0; mpV[i][Y] = 0; mpV[i][Z] = 0;
	    mpDRadius[i] = 0; mpDValue1[i] = 0;
	    continue;
	}

	(this->*pFunction)(i, iNext, dtime);
    }
}

wxThread::ExitCode ThreadInterpBlockLinear_cls::
Entry(){
    if (mVariablePNum == _INTERPOLATE_PNUM_VARIABLE)
	InterpolateDataLinearSearchingID();
    
    if (mVariablePNum == _INTERPOLATE_PNUM_ID_SORTED)
	InterpolateDataLinearSearchingIDSorted();
    
    if (mVariablePNum == _INTERPOLATE_PNUM_CONSTANT) {
	if (mPeriodicType == 0)
	    InterpolateDataLinear();
	else
	    InterpolateDataLinearPeriodic();
    }
    return 0;
}
