// -*- c -*- 
// "clsGLDrawBillboard.h"
// (setq c-basic-offset 4)

#include "clsGLDrawBillboard.h"
#include "clsArrayAndIndex.h"

#include "../FILTER/clsFilterCtrl.h"
#include "../FILTER/clsFilterObject.h"
#include "../GLATTRIB/clsGLAttribute.h"

#include "clsSnapData.h"

#include "../../wxWidgetEnumMenu.h"

extern int       GDrawingPercentage;
extern int       GDrawingStage;
extern wxFrame  *GpCMainFrameCLS;

#include "../COMMON/CommonDefinitions.h"
#include "../COMMON/CommonDefinitionsGL.h"

GLDrawBillboard_cls::
GLDrawBillboard_cls(){
    mpVertexArrayBufferA = NULL;
    mpVertexArrayBufferB = NULL;
    mpTexCoordArrayBufferA = NULL;
    mpTexCoordArrayBufferB = NULL;
    mpColorBufferA = NULL;
    mpColorBufferB = NULL;
}

GLDrawBillboard_cls::
~GLDrawBillboard_cls(){
    if (mpVertexArrayBufferA != NULL) delete[] mpVertexArrayBufferA;
    if (mpVertexArrayBufferB != NULL) delete[] mpVertexArrayBufferB;
    if (mpTexCoordArrayBufferA != NULL) delete [] mpTexCoordArrayBufferA;
    if (mpTexCoordArrayBufferB != NULL) delete [] mpTexCoordArrayBufferB;
    if (mpColorBufferA != NULL) delete[] mpColorBufferA;
    if (mpColorBufferB != NULL) delete[] mpColorBufferB;
}

void GLDrawBillboard_cls::
GLDrawBillboardBlockMain(int offset, int pnumber, 
			 float *pRotatedArray, float radiusFactor, 
			 bool bConstantRadius, bool bConstantColor,
			 float steppingAlphaFactor,
			 SnapData_cls *pCDataSnap,
			 GLAttribute_cls *pCGLAttribute, float *pValue1){
    if (radiusFactor == 0) return;

    float (*pX)[3]  = pCDataSnap->mpX;
    float  *pRadius = pCDataSnap->mpRadius;

    int   drawType = pCGLAttribute->mCDrawType.mBody;

    int   alphaType       = pCGLAttribute->mCAlphaType.mBody;
    float alphaDependBase = pCGLAttribute->mCAlphaDenendBase.mBody;
    float alphaMax        = pCGLAttribute->mCAlphaMax.mBody;

    if (bConstantRadius == TRUE){
	for (int p = 0; p < pnumber; p++){
	    int i = (mspIndexAndZ + offset + p)->index;
	    int poffset = p*12;
	    for (int j = 0; j < 12; j++)
		msVertexArray[poffset + j] = pRotatedArray[j] * radiusFactor;
	    float *pParticleX = pX[i];
	    float *pArray = msVertexArray+poffset;
	    pArray[ 0] += pParticleX[X];
	    pArray[ 1] += pParticleX[Y];
	    pArray[ 2] += pParticleX[Z];
	    pArray[ 3] += pParticleX[X];
	    pArray[ 4] += pParticleX[Y];
	    pArray[ 5] += pParticleX[Z];
	    pArray[ 6] += pParticleX[X];
	    pArray[ 7] += pParticleX[Y];
	    pArray[ 8] += pParticleX[Z];
	    pArray[ 9] += pParticleX[X];
	    pArray[10] += pParticleX[Y];
	    pArray[11] += pParticleX[Z];
	}
    }
    else
	for (int p = 0; p < pnumber; p++){
	    int i = (mspIndexAndZ + offset + p)->index;
	    int poffset = p*12;
	    float radius = radiusFactor * pRadius[i];
	    for (int j = 0; j < 12; j++)
		msVertexArray[poffset + j] = pRotatedArray[j] * radius;
	    float *pParticleX = pX[i];
	    float *pArray = msVertexArray+poffset;
	    pArray[ 0] += pParticleX[X];
	    pArray[ 1] += pParticleX[Y];
	    pArray[ 2] += pParticleX[Z];
	    pArray[ 3] += pParticleX[X];
	    pArray[ 4] += pParticleX[Y];
	    pArray[ 5] += pParticleX[Z];
	    pArray[ 6] += pParticleX[X];
	    pArray[ 7] += pParticleX[Y];
	    pArray[ 8] += pParticleX[Z];
	    pArray[ 9] += pParticleX[X];
	    pArray[10] += pParticleX[Y];
	    pArray[11] += pParticleX[Z];
	}

    float *pTexCoord = msTexCoordArray;
    if (pCGLAttribute->IsAtrasNeeded() == TRUE){
	int atrasGrid = pCGLAttribute->GetPCObject()->GetTexGridSize();
	float atrasWidth = 1.0 / atrasGrid;
	float atrasHeight = 1.0 / atrasGrid;
	for (int p = 0; p < pnumber; p++){
	    int i = (mspIndexAndZ + offset + p)->index;
	    int subIndex = pCDataSnap->mpAtrasIndex[i];
	    int poffset = p*8;
	    float *pArray = msTexCoordArrayAtras + poffset;
	    int offsetX = subIndex % atrasGrid;
	    int offsetY = subIndex / atrasGrid;
	    pArray[ 0] = offsetX * atrasWidth;
	    pArray[ 1] = offsetY * atrasHeight;
	    pArray[ 2] = (offsetX + 1) * atrasWidth;
	    pArray[ 3] = offsetY * atrasHeight;
	    pArray[ 4] = (offsetX + 1) * atrasWidth;
	    pArray[ 5] = (offsetY + 1) * atrasHeight;
	    pArray[ 6] = offsetX * atrasWidth;
	    pArray[ 7] = (offsetY + 1) * atrasHeight;
	}
	pTexCoord = msTexCoordArrayAtras;
    }

    if (bConstantColor == TRUE && alphaType == _GL_ALPHA_NORMAL){ 
	glTexCoordPointer(2, GL_FLOAT, 0, pTexCoord);
	glVertexPointer(3, GL_FLOAT, 0, msVertexArray);
	glDrawElements(GL_QUADS, 4 * pnumber, GL_UNSIGNED_INT, mspIndices);
    }
    else{
	float pColor[4];
	//ColorCurve_cls *pCClrCurve = pCGLAttribute->mpCClrCurveCol;
	
	float *pVertex = msVertexArray;
	if (alphaType == _GL_ALPHA_INVERSE_SQUARE){
	    float alphaFactor = steppingAlphaFactor;
	    for (int p = 0; p < pnumber; p++){
		int i = (mspIndexAndZ + offset + p)->index;
		float radius = radiusFactor * pRadius[i];
		memcpy(pColor, pCDataSnap->mpColor[i], sizeof(float)*4);
		pColor[3] /= (radius * radius);
		pColor[3] *= alphaFactor;
		if (pColor[3] > alphaMax)
		    pColor[3] = alphaMax;

		if (pColor[3] < 0.0035) continue;

		glColor4fv(pColor);
		glBegin(GL_QUADS);
		glTexCoord2fv(pTexCoord+p*8  );glVertex3fv(pVertex+p*12  );
		glTexCoord2fv(pTexCoord+p*8+2);glVertex3fv(pVertex+p*12+3);
		glTexCoord2fv(pTexCoord+p*8+4);glVertex3fv(pVertex+p*12+6);
		glTexCoord2fv(pTexCoord+p*8+6);glVertex3fv(pVertex+p*12+9);
		glEnd();
		/*
		glTexCoordPointer(2, GL_FLOAT, 0, pTexCoord);
		glVertexPointer(3, GL_FLOAT, 0, msVertexArray);
		glDrawElements(GL_QUADS, 4, GL_UNSIGNED_INT, mspIndices + 4*p);
		*/
	    }
		
	}
	else{
	    float alphaFactor = pCGLAttribute->mCAlphaFactor.mBody * steppingAlphaFactor;
	    for (int p = 0; p < pnumber; p++){
		int i = (mspIndexAndZ + offset + p)->index;
		memcpy(pColor, pCDataSnap->mpColor[i], sizeof(float)*4);
		pColor[3] *= alphaFactor;
		glColor4fv(pColor);
		glBegin(GL_QUADS);
		glTexCoord2fv(pTexCoord+p*8  );glVertex3fv(pVertex+p*12  );
		glTexCoord2fv(pTexCoord+p*8+2);glVertex3fv(pVertex+p*12+3);
		glTexCoord2fv(pTexCoord+p*8+4);glVertex3fv(pVertex+p*12+6);
		glTexCoord2fv(pTexCoord+p*8+6);glVertex3fv(pVertex+p*12+9);
		glEnd();
		/*
		glTexCoordPointer(2, GL_FLOAT, 0, pTexCoord);
		glVertexPointer(3, GL_FLOAT, 0, msVertexArray);
		glDrawElements(GL_QUADS, 4, GL_UNSIGNED_INT, mspIndices + 4*p);
		*/
	    }
	}
    }
    
    //printf("Main End\n");
}

void GLDrawBillboard_cls::
GLDrawBillboardBlock(int downsized, 
		     float *pRotatedArray, float radiusFactor, 
		     bool bConstantRadius, bool bConstantColor,
		     float steppingAlphaFactor,
		     SnapData_cls *pCDataSnap,
		     GLAttribute_cls *pCGLAttribute, float *pValue1){
    
    int dummy = GetProgressInterval(downsized);
    
    for (int ibase = 0; ibase < downsized; ibase+=PACKNUM) {
	int number = PACKNUM;
	if (ibase + number >= downsized) number = downsized - ibase;

	if (ibase % (PACKNUM * 500) == 0 && IsTimeLapsedEnough() == TRUE){
	    GDrawingPercentage = CalcPercentage(ibase);
	    UpdateStatusBar();
	    if (CheckAbortion() == TRUE){
		ibase = downsized; continue;
	    }
	}

	GLDrawBillboardBlockMain(ibase , number, pRotatedArray,
				 radiusFactor, bConstantRadius, bConstantColor,
				 steppingAlphaFactor, pCDataSnap, 
				 pCGLAttribute, pValue1);
    }
}

// ----------------------

ThreadCaleVertex_cls::
ThreadCaleVertex_cls(wxThreadKind kind) : 
    wxThread(kind) {
}

void ThreadCaleVertex_cls::
SetParameters(bool *pbFinishedFlag, int offset, int pnumber,
	      float *pVertexBuffer, float *pTexBufer, float *pColorBuffer,
	      float *pRotatedArray, 
	      GLAttribute_cls   *pCGLAttribute,
	      ArrayAndIndex_cls *pArrayIndexAndZ,
	      SnapData_cls      *pCDataSnap,
	      bool bConstantRadius,
	      float radiusFactor, float steppingAlphaFactor){
    mpbFinishedFlagLink = pbFinishedFlag;
    mOffset        = offset;
    mPNumber       = pnumber;
    mpVertexBuffer = pVertexBuffer;
    mpTexBuffer    = pTexBufer;
    mpColorBuffer  = pColorBuffer;
    mpRotatedArray = pRotatedArray;
    mpCGLAttribute = pCGLAttribute;
    mpArrayIndexAndZ = pArrayIndexAndZ;
    mpCDataSnap    = pCDataSnap;
    mbConstantRadius = bConstantRadius;
    mRadiusFactor = radiusFactor;
    mSteppingAlphaFactor = steppingAlphaFactor;

    if ( Create() != wxTHREAD_NO_ERROR) 
	//MsgBoxExit(NULL, "Error!", "Can't create thread");
	exit(0);
}

void ThreadCaleVertex_cls::
CalcVertex(){
    int    offset        = mOffset;
    int    pnumber       = mPNumber;
    float *pVertexBuffer = mpVertexBuffer;
    float *pTexBufer     = mpTexBuffer;
    float *pColorBuffer  = mpColorBuffer;
    float *pRotatedArray = mpRotatedArray;
    ArrayAndIndex_cls *pArrayAndIndexZ = mpArrayIndexAndZ;
    SnapData_cls      *pCDataSnap = mpCDataSnap;
    float (*pX)[3]       = pCDataSnap->mpX;
    float *pRadius       = pCDataSnap->mpRadius;
    float radiusFactor   = mRadiusFactor;
    float steppingAlphaFactor = mSteppingAlphaFactor;

    int   alphaType       = mpCGLAttribute->mCAlphaType.mBody;
    float alphaDependBase = mpCGLAttribute->mCAlphaDenendBase.mBody;
    float alphaMax        = mpCGLAttribute->mCAlphaMax.mBody;
    
    if (mbConstantRadius == TRUE)
	for (int p = 0; p < pnumber; p++){
	    int i = (pArrayAndIndexZ + offset + p)->index;
	    int poffset = p*12;
	    float *pVertex = pVertexBuffer + poffset;
	    for (int j = 0; j < 12; j++)
		pVertex[j] = pRotatedArray[j] * radiusFactor;
	    float *pParticleX = pX[i];
	    pVertex[ 0] += pParticleX[X];
	    pVertex[ 1] += pParticleX[Y];
	    pVertex[ 2] += pParticleX[Z];
	    pVertex[ 3] += pParticleX[X];
	    pVertex[ 4] += pParticleX[Y];
	    pVertex[ 5] += pParticleX[Z];
	    pVertex[ 6] += pParticleX[X];
	    pVertex[ 7] += pParticleX[Y];
	    pVertex[ 8] += pParticleX[Z];
	    pVertex[ 9] += pParticleX[X];
	    pVertex[10] += pParticleX[Y];
	    pVertex[11] += pParticleX[Z];
	}
    else
	for (int p = 0; p < pnumber; p++){
	    int i = (pArrayAndIndexZ + offset + p)->index;
	    int poffset = p*12;
	    float radius = radiusFactor * pRadius[i];
	    float *pVertex = pVertexBuffer + poffset;
	    for (int j = 0; j < 12; j++)
		pVertex[j] = pRotatedArray[j] * radius;
	    float *pParticleX = pX[i];
	    pVertex[ 0] += pParticleX[X];
	    pVertex[ 1] += pParticleX[Y];
	    pVertex[ 2] += pParticleX[Z];
	    pVertex[ 3] += pParticleX[X];
	    pVertex[ 4] += pParticleX[Y];
	    pVertex[ 5] += pParticleX[Z];
	    pVertex[ 6] += pParticleX[X];
	    pVertex[ 7] += pParticleX[Y];
	    pVertex[ 8] += pParticleX[Z];
	    pVertex[ 9] += pParticleX[X];
	    pVertex[10] += pParticleX[Y];
	    pVertex[11] += pParticleX[Z];
	}
    
    if (mpCGLAttribute->IsAtrasNeeded() == TRUE){
	int atrasGrid = mpCGLAttribute->GetPCObject()->GetTexGridSize();
	float atrasWidth = 1.0 / atrasGrid;
	float atrasHeight = 1.0 / atrasGrid;
	for (int p = 0; p < pnumber; p++){
	    int i = (pArrayAndIndexZ + offset + p)->index;
	    int subIndex = pCDataSnap->mpAtrasIndex[i];
	    int poffset = p*8;
	    float *pArray = pTexBufer + poffset;
	    int offsetX = subIndex % atrasGrid;
	    int offsetY = subIndex / atrasGrid;
	    pArray[ 0] = offsetX * atrasWidth;
	    pArray[ 1] = offsetY * atrasHeight;
	    pArray[ 2] = (offsetX + 1) * atrasWidth;
	    pArray[ 3] = offsetY * atrasHeight;
	    pArray[ 4] = (offsetX + 1) * atrasWidth;
	    pArray[ 5] = (offsetY + 1) * atrasHeight;
	    pArray[ 6] = offsetX * atrasWidth;
	    pArray[ 7] = (offsetY + 1) * atrasHeight;
	}
    }


    if (alphaType == _GL_ALPHA_INVERSE_SQUARE){
	float alphaFactor = steppingAlphaFactor;
	for (int p = 0; p < pnumber; p++){
	    int i = (pArrayAndIndexZ + offset + p)->index;
	    float radius  = radiusFactor * pRadius[i];
	    float *pColor = pColorBuffer + p*4;
	    memcpy(pColor, pCDataSnap->mpColor[i], sizeof(float)*4);
	    pColor[3] /= (radius * radius);
	    pColor[3] *= alphaFactor;
	    if (pColor[3] > alphaMax)
		pColor[3] = alphaMax;
	}
    }
    else{
	float alphaFactor = mpCGLAttribute->mCAlphaFactor.mBody 
	    * steppingAlphaFactor;
	for (int p = 0; p < pnumber; p++){
	    int i = (pArrayAndIndexZ + offset + p)->index;
	    float *pColor = pColorBuffer + p*4;
	    memcpy(pColor, pCDataSnap->mpColor[i], sizeof(float)*4);
	    pColor[3] *= alphaFactor;
	}
    }
}

wxThread::ExitCode ThreadCaleVertex_cls::
Entry(){
    CalcVertex();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

void GLDrawBillboard_cls::
GLDrawBillboardMultitheadDraw(float *pVertex, float *pTexCoord,
			      float *pColor, int number, bool bUseAtras){
    if (bUseAtras == TRUE){
	for (int i = 0; i < number; i++){
	    if (pColor[i*4+3] < 0.0035) continue;
	    glColor4fv(pColor + i*4); 
	    glBegin(GL_QUADS);
	    glTexCoord2fv(pTexCoord + i*8  );glVertex3fv(pVertex + i*12);
	    glTexCoord2fv(pTexCoord + i*8+2);glVertex3fv(pVertex + i*12 + 3);
	    glTexCoord2fv(pTexCoord + i*8+4);glVertex3fv(pVertex + i*12 + 6);
	    glTexCoord2fv(pTexCoord + i*8+6);glVertex3fv(pVertex + i*12 + 9);
	    glEnd();
	}
    }
    else{
	for (int i = 0; i < number; i++){
	    if (pColor[i*4+3] < 0.0035) continue;
	    glColor4fv(pColor + i*4); 
	    glBegin(GL_QUADS);
	    glTexCoord2f(0, 0);glVertex3fv(pVertex + i*12);
	    glTexCoord2f(1, 0);glVertex3fv(pVertex + i*12 + 3);
	    glTexCoord2f(1, 1);glVertex3fv(pVertex + i*12 + 6);
	    glTexCoord2f(0, 1);glVertex3fv(pVertex + i*12 + 9);
	    glEnd();
	}
    }
}

void GLDrawBillboard_cls::
AllocateBuffer(float **ppBuffer, int factor){
    if ((*ppBuffer) == NULL)
	(*ppBuffer) = new float [_GL_DRAW_BILLBOARD_BUFFER * factor];
}

void GLDrawBillboard_cls::
GLDrawBillboardMultithead(int downsized, 
			  float *pRotatedArray, float radiusFactor, 
			  bool bConstantRadius, bool bConstantColor,
			  float steppingAlphaFactor,
			  SnapData_cls *pCDataSnap,
			  GLAttribute_cls *pCGLAttribute, float *pValue1){

    int progressInterval = 200000;
    if (downsized > 2000000)
	progressInterval = downsized / 20;
    progressInterval /= _GL_DRAW_BILLBOARD_BUFFER;

    AllocateBuffer(&mpVertexArrayBufferA, 4*3);
    AllocateBuffer(&mpVertexArrayBufferB, 4*3);
    AllocateBuffer(&mpTexCoordArrayBufferA, 4*2);
    AllocateBuffer(&mpTexCoordArrayBufferB, 4*2);
    AllocateBuffer(&mpColorBufferA, 4);
    AllocateBuffer(&mpColorBufferB, 4);
    /*
    if (mpVertexArrayBufferA == NULL) 
	mpVertexArrayBufferA = new float [_GL_DRAW_BILLBOARD_BUFFER * 4 * 3];
    if (mpVertexArrayBufferB == NULL) 
	mpVertexArrayBufferB = new float [_GL_DRAW_BILLBOARD_BUFFER * 4 * 3];
    if (mpColorBufferA == NULL)
	mpColorBufferA = new float [_GL_DRAW_BILLBOARD_BUFFER * 4];
    if (mpColorBufferB == NULL)
	mpColorBufferB = new float [_GL_DRAW_BILLBOARD_BUFFER * 4];
    */

    float *pVertexArrayCalc;
    float *pTexCoordCalc;
    float *pColorCalc;
    float *pVertexArrayDraw = NULL;
    float *pTexCoordDraw = NULL;
    float *pColorDraw = NULL;

    bool bUseAtras = pCGLAttribute->IsAtrasNeeded();
    
    int calculating = 0;
    int drawing     = -1;
    int number2Draw = -1;
    for (int i_ = 0; i_ < downsized; i_ += _GL_DRAW_BILLBOARD_BUFFER){
	int number = _GL_DRAW_BILLBOARD_BUFFER; 
	if (i_ + number >= downsized) number = downsized - i_;

	if (i_ % (_GL_DRAW_BILLBOARD_BUFFER * progressInterval) == 0){
	    GDrawingPercentage = (long long)100 * i_ / downsized;
	    UpdateStatusBar();
	}

	if (calculating == 0){
	    pVertexArrayCalc = mpVertexArrayBufferA;
	    pTexCoordCalc    = mpTexCoordArrayBufferA;
	    pColorCalc       = mpColorBufferA;
	}
	else{
	    pVertexArrayCalc = mpVertexArrayBufferB;
	    pTexCoordCalc    = mpTexCoordArrayBufferB;
	    pColorCalc       = mpColorBufferB;
	}
	
	bool bFinished = FALSE;
	ThreadCaleVertex_cls *pCThread = 
	    new ThreadCaleVertex_cls(wxTHREAD_DETACHED);
	pCThread->SetParameters(&bFinished, i_, number, 
				pVertexArrayCalc, pTexCoordCalc, pColorCalc,
				pRotatedArray,
				pCGLAttribute, mspIndexAndZ,
				pCDataSnap, bConstantRadius,
				radiusFactor, steppingAlphaFactor);

	pCThread->Run();

	if (pVertexArrayDraw != NULL){
	    GLDrawBillboardMultitheadDraw
		(pVertexArrayDraw, pTexCoordDraw, pColorDraw,
		 number2Draw, bUseAtras);
	}
	
	while (bFinished == FALSE) wxMilliSleep(2);

	pVertexArrayDraw = pVertexArrayCalc;
	pTexCoordDraw    = pTexCoordCalc;
	pColorDraw       = pColorCalc;
	calculating      = 1 - calculating;
	number2Draw      = number;
    }
    
    //printf("Routine Draw Fin Begin\n");
    GLDrawBillboardMultitheadDraw(pVertexArrayDraw, pTexCoordDraw, pColorDraw, 
				  number2Draw, bUseAtras);
    //printf("Routine Draw Fin End\n");
}

void GLDrawBillboard_cls::
GLDrawBillboardSequential(float *pParticleX, float vertexArray[12],
			  int drawType, int prevDrawType,
			  float *pTexCoordArray){

    if (drawType != prevDrawType){
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glDisable(GL_NORMALIZE);
    }
    
    vertexArray[0] += pParticleX[X];
    vertexArray[1] += pParticleX[Y];
    vertexArray[2] += pParticleX[Z];
    vertexArray[3] += pParticleX[X];
    vertexArray[4] += pParticleX[Y];
    vertexArray[5] += pParticleX[Z];
    vertexArray[6] += pParticleX[X];
    vertexArray[7] += pParticleX[Y];
    vertexArray[8] += pParticleX[Z];
    vertexArray[9] += pParticleX[X];
    vertexArray[10] += pParticleX[Y];
    vertexArray[11] += pParticleX[Z];
    
    glBegin(GL_QUADS);
    glTexCoord2fv(pTexCoordArray+0);glVertex3fv(vertexArray + 0);
    glTexCoord2fv(pTexCoordArray+2);glVertex3fv(vertexArray + 3);
    glTexCoord2fv(pTexCoordArray+4);glVertex3fv(vertexArray + 6);
    glTexCoord2fv(pTexCoordArray+6);glVertex3fv(vertexArray + 9);
    glEnd();
}

unsigned int GLDrawBillboard_cls::mspIndices[PACKNUM * 4];
float        GLDrawBillboard_cls::msTexCoordArray[PACKNUM * 4 * 2];
float        GLDrawBillboard_cls::msTexCoordArrayAtras[PACKNUM * 4 * 2];
float        GLDrawBillboard_cls::msVertexArray[PACKNUM * 4 * 3];

ArrayAndIndex_cls* GLDrawBillboard_cls::mspIndexAndZ = NULL;
int                GLDrawBillboard_cls::msDownsizedAtFBO0 = 0;

FilterCtrl_cls* GLDrawBillboard_cls::mspCFilterCtrlObjectLink = NULL;
FilterCtrl_cls* GLDrawBillboard_cls::mspCFilterCtrlBooleanLink = NULL;
