//(setq c-basic-offset 4)

#include "../COMMON/MyVector.h"
#include "../FILTER/clsFilterCtrl.h"
#include "../FILTER/clsFilterObject.h"
#include "../GLATTRIB/clsColorCurve.h"
#include "../GLATTRIB/clsGLAttribute.h"
#include "../GLOBJECTS/clsSphere.h"
extern Sphere_cls *GpCSphere;
#include "../LUA/clsLuaBase.h"
#include "../MAINCANVAS/clsDrawOption.h"
#include "../MAINCANVAS/clsInformationGUI.h"
#include "../MAINCANVAS/clsInformation.h"
#include "../PANELS/clsPreference.h"

#include "clsArrayAndIndex.h"
#include "clsGLDrawData.h"
#include "clsParticleDataBlock.h"
#include "clsShadowVolume.h"
#include "clsSnapData.h"
#include "clsZSort.h"

#include "../../wxWidgetEnumMenu.h"

int              GDrawingPercentage = 100;
int              GDrawingStage = 0;
//extern bool      GbForceAbort;
extern wxFrame  *GpCMainFrameCLS;

#include <omp.h>
#include <algorithm>

#include <wx/stopwatch.h>

#include "../COMMON/CommonDefinitions.h"

ArrayAndIndex_cls::
ArrayAndIndex_cls(){
    alphaFactor = 1.0;
}

ThreadA_cls::
ThreadA_cls(wxThreadKind kind) : wxThread(kind){
}

void ThreadA_cls::
SetParameters(bool *pbFinishedFlag, float *pX, int offset, int pnumber, 
	      float *vertexArray, float *pRotatedArray){
    mpbFinishedFlagLink = pbFinishedFlag;
    mpX = pX;
    mOffset = offset;
    mPNumber = pnumber;
    mvertexArray = vertexArray;
    mpRotatedArray = pRotatedArray;
    
    if ( Create() != wxTHREAD_NO_ERROR) 
	;//MsgBoxExit(NULL, "Error!", "Can't create thread");
}

wxThread::ExitCode ThreadA_cls::
Entry(){

    for (int p = 0; p < mPNumber; p++){
	int poffset = p*12;
	for (int j = 0; j < 12; j++){
	    mvertexArray[poffset + j] = mpRotatedArray[j] * 0.1;
	}
	float *pParticleX = &mpX[(mOffset + p)*3];
	mvertexArray[poffset + 0] += pParticleX[X];
	mvertexArray[poffset + 1] += pParticleX[Y];
	mvertexArray[poffset + 2] += pParticleX[Z];
	mvertexArray[poffset + 3] += pParticleX[X];
	mvertexArray[poffset + 4] += pParticleX[Y];
	mvertexArray[poffset + 5] += pParticleX[Z];
	mvertexArray[poffset + 6] += pParticleX[X];
	mvertexArray[poffset + 7] += pParticleX[Y];
	mvertexArray[poffset + 8] += pParticleX[Z];
	mvertexArray[poffset + 9] += pParticleX[X];
	mvertexArray[poffset +10] += pParticleX[Y];
	mvertexArray[poffset +11] += pParticleX[Z];

    }

    *mpbFinishedFlagLink = TRUE;
    Delete();
    return 0;
}

GLDrawData_cls::
GLDrawData_cls(){
    if (msbInitialized == FALSE){
	for (int i = 0; i < 4 * PACKNUM; i++)
	    mspIndices[i] = i;
	for (int i = 0; i < PACKNUM; i++){
	    MakeAtrasForSequential(&msTexCoordArray[i*4*2], 1);
	    /*
	    msTexCoordArray[i * 4 * 2 + 0 * 2 + 0] = 0;
	    msTexCoordArray[i * 4 * 2 + 0 * 2 + 1] = 0;
	    msTexCoordArray[i * 4 * 2 + 1 * 2 + 0] = 1;
	    msTexCoordArray[i * 4 * 2 + 1 * 2 + 1] = 0;
	    msTexCoordArray[i * 4 * 2 + 2 * 2 + 0] = 1;
	    msTexCoordArray[i * 4 * 2 + 2 * 2 + 1] = 1;
	    msTexCoordArray[i * 4 * 2 + 3 * 2 + 0] = 0;
	    msTexCoordArray[i * 4 * 2 + 3 * 2 + 1] = 1;
	    */
	}
	msbInitialized = TRUE;
    }
    for (int i = 0; i < 3; i++) mpZero4[i] = 0;
    mpZero4[3] = 1;

    mpCShadowVolume = new ShadowVolume_cls(&mspIndexAndZ,
					   &mCParticlePos);
    mpCZSort = new ZSort_cls;
    mSortType = _SORT_ORTHO;

    mpIndices = NULL;
}

GLDrawData_cls::
~GLDrawData_cls(){
    if (mspCSnapShot != NULL){
	delete mspCSnapShot;
	mspCSnapShot = NULL;
    }
    if (mspIndexAndZ != NULL){
	delete [] mspIndexAndZ;
	mspIndexAndZ = NULL;
    }

    delete mpCShadowVolume;
    delete mpCZSort;
}

void GLDrawData_cls::
SetCameraPos(double *pCamPos){
    LOOP_XYZ mCameraPos[xyz] = pCamPos[xyz];
}

void GLDrawData_cls::
SetCameraDir(double *pCamDir){
    LOOP_XYZ mCameraDir[xyz] = pCamDir[xyz];
}

void GLDrawData_cls::
SetCopyTranrateInfo(float *pCopyTranslate){
    LOOP_XYZ mCopyTranslate[xyz] = pCopyTranslate[xyz];
}

void GLDrawData_cls::
SetLightSourceInfo(int lightNumber, 
		   float *pLightPos, //float *pLightClr, 
		   double lightPhi, double lightTheta){
    mLightNum = lightNumber;
    LOOP_XYZ mLightPos[xyz] = pLightPos[xyz];
    
    double lightTgt[3], lightTop[3];
    lightTop[0] = 0; lightTop[1] = 0; lightTop[2] = 1;
    lightTgt[0] = -mLightPos[0];
    lightTgt[1] = -mLightPos[1];
    lightTgt[2] = -mLightPos[2];
    MyVector::Normalize(lightTgt, 1.0f);
    MyVector::Orthogonalize(lightTgt, lightTop);
    MyVector::Normalize(lightTop, 1.0f);
    
    mCLightPos.Setv(lightTgt, lightTop);
    mCParticlePos = mCLightPos;

    mLightPhi      = lightPhi;
    mLightTheta    = lightTheta;
}

float GLDrawData_cls::
GetSteppingAlphaFactor(int step){
    float factor1 = 0.8 - 0.6*(float)step/100;
    if (factor1 < 0.2) factor1 = 0.2; 
    return 1.0f + ((float)step - 1.0f) * factor1;
}

float GLDrawData_cls::
GetSteppingRadiusFactor(int step){
    float factor1 = 0.2 + 0.6*(float)step/100;
    if (factor1 > 0.8) factor1 = 0.8;
    return sqrt(1.0f + ((float)step - 1.0f) * factor1);
}

void GLDrawData_cls::
GLDrawPrepareParticleSnap(PDataBlock_cls *pCDataBlock, GLAttribute_cls **ppCGLAttribute, double dT,
			  double doubleIndex, int frame){

    int step = ppCGLAttribute[0]->mspCDrawOptionLink->mCDownSize.mBody;
    
    if (mspCSnapShot == NULL){
	mspCSnapShot = new SnapData_cls(pCDataBlock, ppCGLAttribute, dT, step,
					doubleIndex, frame);
	//LuaBase_cls::SetSnapshotLink(mspCSnapShot);
    }
    
    bool bDiff = FALSE;
    if (mspCSnapShot->IsItSameSnap(pCDataBlock, dT, step, frame) != TRUE) bDiff = TRUE;

    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	bDiff |= ppCGLAttribute[i]->IsDifferenceExist();
	ppCGLAttribute[i]->Memorize();
    }

    if (bDiff == TRUE){
	delete mspCSnapShot;
	mspCSnapShot = new SnapData_cls(pCDataBlock, ppCGLAttribute, dT, step,
					doubleIndex, frame);	
	//LuaBase_cls::SetSnapshotLink(mspCSnapShot);

	if (mspCInfoPanel->mpCCtrlLink->mCType.mBody == 0)
	    mspCInfoPanel->UpdateInfo(mspCSnapShot);
	else
	    mspCInfoPanel->UpdateInfo(pCDataBlock);
    }
}

void GLDrawData_cls::
GLDraw(int fboIndex, bool bCopiedInstance, 
       PDataBlock_cls *pCDataBlock, GLAttribute_cls **ppCGLAttribute, 
       double dT){

    if (CheckAbortion() == TRUE) return;
    GDrawingPercentage = 0;

    //PrepareParticleSnap͊ɕKĂ΂Ă

    if (ppCGLAttribute[0]->mspCDrawOptionLink->mCClassify.mBody == 0){ 
	bool bDependOnColor = 
	    ppCGLAttribute[0]->mpCClrCurveCol->mCbColorCurve.mbBody;

	int downsized = ZSort(fboIndex, bCopiedInstance, mspCSnapShot, ppCGLAttribute[0]);
	
	// q̎ޕ
	switch(ppCGLAttribute[0]->mCDrawType.mBody){
	case _GL_DRAW_POINT:
	    if (bDependOnColor == TRUE)
		GLDrawPointAll(downsized, mspCSnapShot, ppCGLAttribute[0]);
	    else
		GLDrawPointArrayElement(downsized, mspCSnapShot, ppCGLAttribute[0]);
	    break;
	case _GL_DRAW_SPHERE:
	    GLDrawSphereAll(downsized, mspCSnapShot, ppCGLAttribute[0]);
	    break;
	case _GL_DRAW_BILLBOARD:
	case _GL_DRAW_PSEUDO_SPHERE:
	case _GL_DRAW_PSEUDO_OBJECT:
	    GLDrawBillboardAll(downsized, mspCSnapShot, ppCGLAttribute[0]);
	    break;
	case _GL_DRAW_OBJECT:
	    GLDrawObjectAll(downsized, mspCSnapShot, ppCGLAttribute[0]);
	    break;
	}
    }
    else
	GLDrawSequential(fboIndex, bCopiedInstance,
			 mspCSnapShot, ppCGLAttribute);
}

void GLDrawData_cls::
GLDrawEnd(){
    if (mpIndices != NULL){
	delete [] mpIndices;
	mpIndices = NULL;
    }
}

void GLDrawData_cls::
GLDrawPointAll(int downsized, SnapData_cls *pCDataSnap, 
	       GLAttribute_cls *pCGLAttrib){

    glDepthMask(GL_TRUE);
    glEnable(GL_DEPTH_TEST);

    glPointSize(pCGLAttrib->mCPointSize.mBody);

    glEnable(GL_BLEND);
    pCGLAttrib->SetBlendFunc();
    
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    
    int step = pCGLAttrib->mspCDrawOptionLink->mCDownSize.mBody;
    float steppingAlphaFactor = GetSteppingAlphaFactor(step);
    float totalAlphaFactor = steppingAlphaFactor * pCGLAttrib->mCAlphaFactor.mBody;
    
    int alphaType  = pCGLAttrib->mCAlphaType.mBody;
    float alphaMax = pCGLAttrib->mCAlphaMax.mBody; 
    float radiusFactor = pCGLAttrib->GetRadiusFactor();
    float (*pPos)[3] = pCDataSnap->mpX;
    float *pValue1;
    pCDataSnap->SetPValue1(&pValue1, pCGLAttrib, 0);
    
    //BeginProgress();
    int progressInterval = GetProgressInterval(downsized);

    float pColor[4];
    for (int i_ = 0; i_ < downsized; i_++){

	if (i_ % progressInterval == 0 && IsTimeLapsedEnough() == TRUE){
	    GDrawingPercentage = CalcPercentage(i_);
	    UpdateStatusBar();
	    if (CheckAbortion() == TRUE){
		i_ = downsized; continue;
	    }
	}

	int i = (mspIndexAndZ + i_)->index;
	
	pCGLAttrib->mpCClrCurveCol->GetColor4fScaled(pColor, pValue1[i]);
	if (alphaType == _GL_ALPHA_INVERSE_SQUARE){
	    pColor[3] /= (radiusFactor * radiusFactor);
	    if (pColor[3] > alphaMax) pColor[3] = alphaMax;
	}
	pColor[3] *= totalAlphaFactor;
	
	glColor4fv(pColor);

	glBegin(GL_POINTS);
	glVertex3fv(pPos[i]);
	glEnd();
    }
}

void GLDrawData_cls::
GLDrawPointArrayElement(int downsized, SnapData_cls *pCDataSnap, 
			GLAttribute_cls *pCGLAttrib){

    if (mpIndices == NULL){
	mpIndices = new unsigned int [downsized];
	for (int i = 0; i < downsized; i++) mpIndices[i] = mspIndexAndZ[i].index;
    }

    glDepthMask(GL_TRUE);
    glEnable(GL_DEPTH_TEST);
    glDisable(GL_LIGHTING);

    glPointSize(pCGLAttrib->mCPointSize.mBody);

    int step = pCGLAttrib->mspCDrawOptionLink->mCDownSize.mBody;
    float steppingAlphaFactor = GetSteppingAlphaFactor(step);
    
    float pColor[4];
    pCGLAttrib->mpCClrCurveCol->GetColor4fScaled(pColor, 0); //!!
    pColor[3] *= pCGLAttrib->mCAlphaFactor.mBody;
    pColor[3] *= steppingAlphaFactor;

    glColor4fv(pColor);
    
    if (pColor[3] < 1 || pCGLAttrib->mCBlendType.mBody == _GL_BLEND_ADD){
	glEnable(GL_BLEND);
	pCGLAttrib->SetBlendFunc();
    }

    glEnableClientState(GL_VERTEX_ARRAY);    
    int elementMax = 1000000;
    for (int i = 0; i < downsized; i+=elementMax){
	int fin = elementMax;
	if (i + fin > downsized) fin = downsized - i;
	glVertexPointer(3, GL_FLOAT, 0, pCDataSnap->mpX);
	glDrawElements(GL_POINTS, fin, GL_UNSIGNED_INT, mpIndices + i);
    }

    glDisableClientState(GL_VERTEX_ARRAY);

    glDisable(GL_BLEND);
}

void GLDrawData_cls::
GetVertexArrayLookAtCamera(float pParticlePos[3], float array[12]){
    static float *spCameraPos = mCameraPos;
    float x0[3] = {1, 0, 0};
    float x[3], y[3];
    float p[3] = {pParticlePos[0] - spCameraPos[0],
		  pParticlePos[1] - spCameraPos[1], 
		  pParticlePos[2] - spCameraPos[2]};
    if (p[1] == 0 && p[2] == 0){
	x0[0] = 0; x0[1] = 1; x0[2] = 0;
    }
    float normp2 = p[0]*p[0] + p[1]*p[1] + p[2]*p[2];
    float normp = sqrt(normp2);
    float iproduct_per_normp2 = (x0[0]*p[0] + x0[1]*p[1] + x0[2]*p[2]) / normp2;

    x[0] = x0[0] - iproduct_per_normp2 * p[0];
    x[1] = x0[1] - iproduct_per_normp2 * p[1];
    x[2] = x0[2] - iproduct_per_normp2 * p[2];
    
    float normx = sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2]);
    x[0] /= normx;
    x[1] /= normx;
    x[2] /= normx;
    
    y[0] = -(p[1]*x[2] - p[2]*x[1]) / normp;
    y[1] = -(p[2]*x[0] - p[0]*x[2]) / normp;
    y[2] = -(p[0]*x[1] - p[1]*x[0]) / normp;
    
    array[ 0] = - x[0] - y[0];
    array[ 1] = - x[1] - y[1];
    array[ 2] = - x[2] - y[2];
    array[ 3] = + x[0] - y[0];
    array[ 4] = + x[1] - y[1];
    array[ 5] = + x[2] - y[2];
    array[ 6] = + x[0] + y[0];
    array[ 7] = + x[1] + y[1];
    array[ 8] = + x[2] + y[2];
    array[ 9] = - x[0] + y[0];
    array[10] = - x[1] + y[1];
    array[11] = - x[2] + y[2];
}

void GLDrawData_cls::
MultiplyMatrixOnVertexArrayOrg(GLfloat matrix[16], float array[3]){
    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 3; j++)
	    array[i*3 + j] = 
		matrix[0+j] * msVertexArrayOrg[i*3 + 0] + 
		matrix[4+j] * msVertexArrayOrg[i*3 + 1] + 
		matrix[8+j] * msVertexArrayOrg[i*3 + 2];
}

void GLDrawData_cls::
GLDrawShadowVolumeAll(int downsized, float (*pPos)[3], float *pRadius,
		      float radiusFactor, GLAttribute_cls *pCGLAttrib, 
		      bool bConstantRadius, bool bUseOffset){
    if (pCGLAttrib->mCTexture.mCCastShadow.mBody == 0) return;
    
    glDepthMask(GL_FALSE);
	
    pCGLAttrib->BeginShader(mLightNum, TRUE);
    glDisable(GL_LIGHTING);
    glDisable(GL_BLEND);
    float lightDir[3] = {mLightPos[0], mLightPos[1], mLightPos[2]};
    
    float cameraPosCopied[3];
    LOOP_XYZ cameraPosCopied[xyz] = mCameraPos[xyz] - mCopyTranslate[xyz];
    mpCShadowVolume->CheckShadow
	(downsized, pPos, pRadius, radiusFactor, cameraPosCopied,
	 lightDir, mLightPhi, mLightTheta);
    
    glPushMatrix();
    if (bConstantRadius == TRUE)
	mpCShadowVolume->GLDrawCylinderConstantRadius
	    (downsized, pPos, pRadius, radiusFactor, bUseOffset);
    else
	mpCShadowVolume->GLDrawCylinderVariableRadius
	    (downsized, pPos, pRadius, radiusFactor, bUseOffset);
    glPopMatrix();
    pCGLAttrib->EndShader();
}

void GLDrawData_cls::
GLDrawBillboardAll(int downsized, SnapData_cls *pCDataSnap, 
		   GLAttribute_cls *pCGLAttrib){

    GLfloat mMatrix[16];
    glGetFloatv(GL_TRANSPOSE_MODELVIEW_MATRIX, mMatrix);
    float rotatedArray[12];
    MultiplyMatrixOnVertexArrayOrg(mMatrix, rotatedArray);

    glDisable(GL_LIGHTING);
    float pColor[4];
    pCGLAttrib->mpCClrCurveCol->GetColor4fScaled(pColor, 0); //!!
    pColor[3] *= pCGLAttrib->mCAlphaFactor.mBody;
    glColor4fv(pColor);
    
    int step = pCGLAttrib->mspCDrawOptionLink->mCDownSize.mBody;
    float steppingRadiusFactor = GetSteppingRadiusFactor(step);//sqrt(1.0f + ((float)step - 1.0f) * (0.2 + 0.6*(float)step/100));
    float steppingAlphaFactor = 1.0f + ((float)step - 1.0f) * (0.8 - 0.6*(float)step/100);

    glEnable(GL_TEXTURE_2D);
    pCGLAttrib->SetBlendFunc();
    pCGLAttrib->BeginGasShader(mLightNum);

    glEnable(GL_DEPTH_TEST);
    glDepthMask(pCGLAttrib->DoesUseDepthMask());
    
    glEnable(GL_BLEND);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    pCGLAttrib->BindTexture();
    
    glPushMatrix();
    
    int   alphaType    = pCGLAttrib->mCAlphaType.mBody;
    float radiusFactor = pCGLAttrib->GetRadiusFactor() *
	steppingRadiusFactor;
    bool  bConstantRadius = 
	(pCGLAttrib->mCRadiusType.mBody == _GL_RADIUS_CONSTANT);
    bool  bConstantColor = 
	(pCGLAttrib->mpCClrCurveCol->mCbColorCurve.mbBody == FALSE);
    float *pValue1;
    pCDataSnap->SetPValue1(&pValue1, pCGLAttrib, 0);

    bool bUseMultiThread = FALSE;
    if (downsized > 300000 && 
	(bConstantColor != TRUE || alphaType != _GL_ALPHA_NORMAL))
	bUseMultiThread = TRUE;

    int   quality = pCGLAttrib->mCDrawQuality.mBody;
    float (*pPos)[3] = pCDataSnap->mpX;
    float *pRadius   = pCDataSnap->mpRadius;

    if (mbShadowVolume == FALSE){
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	if (bUseMultiThread == TRUE)
	    GLDrawBillboardMultithead(downsized, rotatedArray,
				      radiusFactor, bConstantRadius, bConstantColor,
				      steppingAlphaFactor, pCDataSnap, 
				      pCGLAttrib, pValue1);
	if (bUseMultiThread == FALSE)
	    GLDrawBillboardBlock(downsized, rotatedArray,
				 radiusFactor, bConstantRadius, bConstantColor,
				 steppingAlphaFactor, pCDataSnap, 
				 pCGLAttrib, pValue1);
	
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
    }
    
    if (mbShadowVolume == TRUE)
	GLDrawShadowVolumeAll(downsized, pPos, pRadius, 
			      radiusFactor, pCGLAttrib, 
			      bConstantRadius, TRUE);
	
    glPopMatrix();
    
    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_DEPTH_TEST);
}

void GLDrawData_cls::
GLDrawSphereAll(int downsized, SnapData_cls *pCDataSnap, 
		GLAttribute_cls *pCGLAttrib){

    glEnable(GL_LIGHTING);

    int step = pCGLAttrib->mspCDrawOptionLink->mCDownSize.mBody;
    float steppingAlphaFactor = GetSteppingAlphaFactor(step);
    float steppingRadiusFactor = GetSteppingRadiusFactor(step);

    float pClrMat[3][4];
    for (int mat = 0; mat < 3; mat++){
	pCGLAttrib->
	    mpCClrCurveMat[mat]->GetColor4fScaled(pClrMat[mat], 0); //!!
	pClrMat[mat][3] *= pCGLAttrib->mCAlphaFactor.mBody;
	pClrMat[mat][3] *= steppingAlphaFactor;
    }

    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  pClrMat[0]);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  pClrMat[1]);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, pClrMat[2]);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 20);

    bool bUseTextue = FALSE;
    if (pCGLAttrib->mbTextureValid[1] == TRUE){
	bUseTextue = TRUE;
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, pCGLAttrib->mTexture[1]);
    }
    else{
	glDisable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 0);
    }

    pCGLAttrib->SetBlendFunc();
    
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_BLEND);
    glEnable(GL_NORMALIZE);

    float radiusFactor = pCGLAttrib->GetRadiusFactor() *
	steppingRadiusFactor;
    bool  bConstantRadius = 
	(pCGLAttrib->mCRadiusType.mBody == _GL_RADIUS_CONSTANT);
    bool  bConstantColor = 
	(pCGLAttrib->mpCClrCurveMat[0]->mCbColorCurve.mbBody == FALSE) &&
	(pCGLAttrib->mpCClrCurveMat[1]->mCbColorCurve.mbBody == FALSE) &&
	(pCGLAttrib->mpCClrCurveMat[2]->mCbColorCurve.mbBody == FALSE);

    if (mspCFilterCtrlBooleanLink->IsBooleanChangeConstantColor() == TRUE)
	bConstantColor = FALSE; 
    
    int   quality = pCGLAttrib->mCDrawQuality.mBody;
    
    float (*pPos)[3] = pCDataSnap->mpX;
    float *pRadius   = pCDataSnap->mpRadius;
    float *pValue1;
    // !! _FIX_ME !!AmbientŔ    
    pCDataSnap->SetPValue1(&pValue1, pCGLAttrib, 1);

    if (mbShadowVolume == FALSE){
	glDepthMask(GL_TRUE);
	pCGLAttrib->BeginShader(mLightNum);
	glBindTexture(GL_TEXTURE_2D, pCGLAttrib->mTexture[1]);
	glPushMatrix();
	if (bConstantRadius == TRUE){
	    glScalef(radiusFactor, radiusFactor, radiusFactor);
	    if (bConstantColor == TRUE)
		GLDrawSphereConstRadiusConstColor
		    (downsized, bUseTextue, pPos, radiusFactor, quality);
	    else
		GLDrawSphereConstRadiusVariableColor
		    (downsized, bUseTextue, pPos, pValue1, radiusFactor, 
		     quality, steppingAlphaFactor, pCGLAttrib);
	}
	else{
	    if (bConstantColor == TRUE)
		GLDrawSphereVariableRadiusConstColor
		    (downsized, bUseTextue, pPos, pRadius, radiusFactor, quality);
	    else
		GLDrawSphereVariableRadiusVariableColor
		    (downsized, bUseTextue, pPos, pRadius, pValue1, 
		     radiusFactor, quality, steppingAlphaFactor, pCGLAttrib);
	}
	glPopMatrix();
	pCGLAttrib->EndShader();
    }
    else
	GLDrawShadowVolumeAll(downsized, pPos, pRadius, radiusFactor,
			      pCGLAttrib, bConstantRadius, FALSE);
    
    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_DEPTH_TEST);
}

void GLDrawData_cls::
GLDrawObjectAll(int downsized, SnapData_cls *pCDataSnap, 
		GLAttribute_cls *pCGLAttrib){

    int objIndex = pCGLAttrib->mCObjectIndex.mBody;

    FilterObject_cls *pCObj = 
	mspCFilterCtrlObjectLink->GetPCObject(objIndex);
    if (pCObj == NULL) return;
    
   
    glEnable(GL_LIGHTING);

    int step = pCGLAttrib->mspCDrawOptionLink->mCDownSize.mBody;
    float steppingAlphaFactor = GetSteppingAlphaFactor(step);
    float steppingRadiusFactor = GetSteppingRadiusFactor(step);

    pCGLAttrib->SetBlendFunc();

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_BLEND);
    glEnable(GL_NORMALIZE);

    float radiusFactor = pCGLAttrib->GetRadiusFactor() *
	steppingRadiusFactor;
    bool  bConstantRadius = 
	(pCGLAttrib->mCRadiusType.mBody == _GL_RADIUS_CONSTANT);
    bool  bConstantColor = 
	(pCGLAttrib->mpCClrCurveMat[0]->mCbColorCurve.mbBody == FALSE) &&
	(pCGLAttrib->mpCClrCurveMat[1]->mCbColorCurve.mbBody == FALSE) &&
	(pCGLAttrib->mpCClrCurveMat[2]->mCbColorCurve.mbBody == FALSE);

    if (mspCFilterCtrlBooleanLink->IsBooleanChangeConstantColor() == TRUE)
	bConstantColor = FALSE; 
    
    int   quality = pCGLAttrib->mCDrawQuality.mBody;
    
    float (*pPos)[3] = pCDataSnap->mpX;
    float *pRadius   = pCDataSnap->mpRadius;
    float *pValue1;
    // !! _FIX_ME !!AmbientŔ    
    pCDataSnap->SetPValue1(&pValue1, pCGLAttrib, 1);

    if (mbShadowVolume == FALSE){
	glDepthMask(GL_TRUE);
	
	int progressInterval = GetProgressInterval(downsized);
	bool bClearList = FALSE;

	pCObj->GLDrawSetMaterial();
	pCObj->UpdateDisplayList(msCurrentIndex);

	int displayList[16];
	for (int i = 0; i < _MAX_MESH; i++)
	    displayList[i] = pCObj->mDisplayList[i];

	pCObj->GLDrawSetShaderBegin(mLightNum);
	
	if (bConstantRadius == TRUE){
	    glScalef(radiusFactor, radiusFactor, radiusFactor);
	    GLDrawSphereConstRadiusConstColor
		(downsized, TRUE, pPos, radiusFactor, 0, 
		 displayList, pCObj);
	}
	else{
	    GLDrawSphereVariableRadiusConstColor
		(downsized, TRUE, pPos, pRadius, radiusFactor, 0, 
		 displayList, pCObj);
	}
	
	pCObj->mbMatrixIdentity = FALSE;
	pCObj->GLDrawSetShaderEnd();
    }
    else
	GLDrawShadowVolumeAll(downsized, pPos, pRadius, radiusFactor,
			      pCGLAttrib, bConstantRadius, FALSE);
    
    glDisable(GL_BLEND);
    glDisable(GL_TEXTURE_2D);
    glEnable(GL_DEPTH_TEST);
}

// ----

void GLDrawData_cls::
ZSortExec(int PNum, int step, float (*pX)[3], unsigned char *pType,
	  int sortOrder){

    if (mspIndexAndZ != NULL) delete [] mspIndexAndZ;
    mspIndexAndZ =  new ArrayAndIndex_cls [PNum/step + 1];
    
    int downsized = 0;
    switch(sortOrder){
    case _SORT_NONE:
	for (int i = 0; i < PNum; i+=step){
	    mspIndexAndZ[downsized].index = i;
	    downsized++;
	}
	
	downsized = mspCFilterCtrlBooleanLink->
	    CheckBoolean(mspIndexAndZ, pX, downsized, msCurrentDoubleIndex);
	
	break;
    case _SORT_DESCENDANT: 
	if (mSortType == _SORT_ORTHO)
	    downsized = mpCZSort->
		ZSortPrepare(mspIndexAndZ, PNum, step, pX, 
			     mCameraPos, mCameraDir, 0);
	else
	    downsized = mpCZSort->
		ZSortPrepare(mspIndexAndZ, PNum, step, pX, 
			     mCameraPos, mCameraDir, 1);

	// !!_FIX_ME_!! ̏Ԃtɂł΁A
	// SortԂ񏭂ł
	downsized = mspCFilterCtrlBooleanLink->
	    CheckBoolean(mspIndexAndZ, pX, downsized, msCurrentDoubleIndex);
	
	if (downsized > 100000)
	    mpCZSort->ZSortParallel(mspIndexAndZ, downsized);
	else
	    std::sort(mspIndexAndZ, mspIndexAndZ + downsized);

	break;
    case _SORT_ASCENDANT:
	for (int i = 0; i < PNum; i+=step){ // ̒xłOpenMPőȂȂc
	    mspIndexAndZ[downsized].index = i;
	    mspIndexAndZ[downsized].z = //߂`
		(pX[i][X]-mCameraPos[X]) * (pX[i][X]-mCameraPos[X]) +
		(pX[i][Y]-mCameraPos[Y]) * (pX[i][Y]-mCameraPos[Y]) + 
		(pX[i][Z]-mCameraPos[Z]) * (pX[i][Z]-mCameraPos[Z]);
	    downsized++;
	}

	if (downsized > 100000)
	    mpCZSort->ZSortParallel(mspIndexAndZ, downsized);
	else
	    std::sort(mspIndexAndZ, mspIndexAndZ + downsized);
	break;
    case _SORT_TYPE:
	for (int i = 0; i < PNum; i+=step){ // ̒xłOpenMPőȂȂc
	    mspIndexAndZ[downsized].index = i;
	    mspIndexAndZ[downsized].z = (float)pType[i];
	    downsized++;
	}
	
	if (downsized > 100000)
	    mpCZSort->ZSortParallel(mspIndexAndZ, downsized);
	else
	    std::sort(mspIndexAndZ, mspIndexAndZ + downsized);
	break;
    }
    msDownsizedAtFBO0 = downsized;
}

int  GLDrawData_cls::
ZSort(int fboIndex,  bool bCopiedInstance,
      SnapData_cls* pCDataSnap, GLAttribute_cls *pCGLAttribute){
    int PNum = pCDataSnap->mPNumBlock;
    int step = pCGLAttribute->mspCDrawOptionLink->mCDownSize.mBody;
    int sortOrder = pCGLAttribute->mspCDrawOptionLink->mCSortOrder.mBody;
    bool bClassifyExpression = (pCGLAttribute->mspCDrawOptionLink->mCClassify.mBody == _CLASSIFY_EXPRESSION);
    
    if ((pCGLAttribute->mCDrawType.mBody == _GL_DRAW_BILLBOARD ||
	 pCGLAttribute->mCDrawType.mBody == _GL_DRAW_PSEUDO_SPHERE ||
	 pCGLAttribute->mCDrawType.mBody == _GL_DRAW_PSEUDO_OBJECT)  &&
	pCGLAttribute->mCDrawQuality.mBody >= 1)
	mSortType = _SORT_RADIAL;
    else
	mSortType = _SORT_ORTHO;

    bool bExecSort = FALSE;
    if (fboIndex == 0 && bCopiedInstance == FALSE && mbShadowVolume == FALSE) bExecSort = TRUE;
    if (mSortType == _SORT_RADIAL && bCopiedInstance == TRUE) bExecSort = TRUE;
    
    if (bExecSort == TRUE){
	if (bClassifyExpression == TRUE)
	    ZSortExec(PNum, step, pCDataSnap->mpX, pCDataSnap->mpTypeSnapLocal,
		      sortOrder);
	else
	    ZSortExec(PNum, step, pCDataSnap->mpX, pCDataSnap->mpTypeLink,
		      sortOrder);
    }
    return msDownsizedAtFBO0;
}

void GLDrawData_cls::
EnableLight(bool bEnable){
    if (mbSeqLightEnabled != bEnable){
	if (bEnable == TRUE)
	    glEnable(GL_LIGHTING);
	else
	    glDisable(GL_LIGHTING);
	mbSeqLightEnabled = bEnable;
    }
}

void GLDrawData_cls::
EnableTexture2D(bool bEnable){
    if (mbSeqTexEnabled != bEnable){
	if (bEnable == TRUE)
	    glEnable(GL_TEXTURE_2D);
	else
	    glDisable(GL_TEXTURE_2D);
	mbSeqLightEnabled = bEnable;
    }
}

void GLDrawData_cls::
EnableDepthMask(bool bEnable){
    if (mbSeqDepthEnabled != bEnable){
	if (bEnable == TRUE)
	    glDepthMask(GL_TRUE);
	else
	    glDepthMask(GL_FALSE);
	mbSeqDepthEnabled = bEnable;
    }
}

void GLDrawData_cls::
MakeAtrasForSequential(float *pTexCoord, int atrasGrid){
    if (atrasGrid == 1){
	pTexCoord[0*2 + 0] = 0;
	pTexCoord[0*2 + 1] = 0;
	pTexCoord[1*2 + 0] = 1;
	pTexCoord[1*2 + 1] = 0;
	pTexCoord[2*2 + 0] = 1;
	pTexCoord[2*2 + 1] = 1;
	pTexCoord[3*2 + 0] = 0;
	pTexCoord[3*2 + 1] = 1;
	return;
    }
    
    for (int atrasIndex = 0; atrasIndex < atrasGrid * atrasGrid; atrasIndex++){
	float widthPart = 1.0f / atrasGrid;
	float heightPart = 1.0f / atrasGrid;
	int indexOffsetX = atrasIndex % atrasGrid;
	int indexOffsetY = atrasIndex / atrasGrid;
	pTexCoord[atrasIndex*4*2 + 0*2 + 0] = widthPart * indexOffsetX;
	pTexCoord[atrasIndex*4*2 + 0*2 + 1] = heightPart * indexOffsetY;
	pTexCoord[atrasIndex*4*2 + 1*2 + 0] = widthPart * (indexOffsetX + 1);
	pTexCoord[atrasIndex*4*2 + 1*2 + 1] = heightPart * indexOffsetY;
	pTexCoord[atrasIndex*4*2 + 2*2 + 0] = widthPart * (indexOffsetX + 1);
	pTexCoord[atrasIndex*4*2 + 2*2 + 1] = heightPart * (indexOffsetY + 1);
	pTexCoord[atrasIndex*4*2 + 3*2 + 0] = widthPart * indexOffsetX;
	pTexCoord[atrasIndex*4*2 + 3*2 + 1] = heightPart * (indexOffsetY + 1);
    }
}

void GLDrawData_cls::
GLDrawSequential(int fboIndex, bool bCopiedInstance,
		 SnapData_cls *pCDataSnap, GLAttribute_cls **ppCGLAttrib){

    MakeAtrasForSequential(&msTexCoordArrayAtras[0*4*2], 1);
    MakeAtrasForSequential(&msTexCoordArrayAtras[1*4*2], 2);
    MakeAtrasForSequential(&msTexCoordArrayAtras[5*4*2], 4);
    
    for (int i = 0; i < 4; i++)
	for (int mat = 0; mat < 3; mat++){
	    ppCGLAttrib[i]->mpCClrCurveMat[mat]->PreCalcScaleFactor();
	}
    
    GLfloat mMatrix[16];
    glGetFloatv(GL_TRANSPOSE_MODELVIEW_MATRIX, mMatrix);
    float rotatedArray[12];
    float vertexArray[12];
    MultiplyMatrixOnVertexArrayOrg(mMatrix, rotatedArray);

    int PNum = pCDataSnap->mPNumBlock;

    int step = ppCGLAttrib[0]->mspCDrawOptionLink->mCDownSize.mBody;
    float steppingAlphaFactor = GetSteppingAlphaFactor(step);
    float steppingRadiusFactor = GetSteppingRadiusFactor(step);
    
    int downsized = ZSort(fboIndex, bCopiedInstance, pCDataSnap, ppCGLAttrib[0]);
    
    int prevParticleType = -1;
    int prevBlendMode = -1;
    int prevDrawType = -1;
    
    bool bConstantColorInBillboard[_MAX_PARTICLE_TYPE];
    bool bConstantColorInSphere[_MAX_PARTICLE_TYPE];
    int   alphaType[_MAX_PARTICLE_TYPE];
    float alphaDependBase[_MAX_PARTICLE_TYPE];
    float alphaMax[_MAX_PARTICLE_TYPE];
    float alphaFactor[_MAX_PARTICLE_TYPE];
    int   qualityList[_MAX_PARTICLE_TYPE];
    int   sphereDListList[_MAX_PARTICLE_TYPE];
    int   blendModeList[_MAX_PARTICLE_TYPE];
    int   drawTypeList[_MAX_PARTICLE_TYPE];
    int   radiusTypeList[_MAX_PARTICLE_TYPE];
    float radiusFactorList[_MAX_PARTICLE_TYPE];
    FilterObject_cls *pCObj[_MAX_PARTICLE_TYPE];
    GLTexture_cls *pCObjTexture[_MAX_PARTICLE_TYPE];
    bool  bUseTextureList[_MAX_PARTICLE_TYPE];
    bool  bCastShadowList[_MAX_PARTICLE_TYPE];
    bool  bShadowOffset[_MAX_PARTICLE_TYPE];
    int   displayListList[_MAX_PARTICLE_TYPE][_MAX_MESH];
    wxString strTextureList[_MAX_PARTICLE_TYPE];
    bool  bUseAtrasIndex[_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	bConstantColorInBillboard[i] = (ppCGLAttrib[i]->mpCClrCurveCol->mCbColorCurve.mbBody != TRUE);
	bConstantColorInSphere[i] = 
	    (ppCGLAttrib[i]->mpCClrCurveMat[0]->mCbColorCurve.mbBody!=TRUE)&
	    (ppCGLAttrib[i]->mpCClrCurveMat[1]->mCbColorCurve.mbBody!=TRUE)&
	    (ppCGLAttrib[i]->mpCClrCurveMat[2]->mCbColorCurve.mbBody!=TRUE);
	if (mspCFilterCtrlBooleanLink->
	    IsBooleanChangeConstantColor() == TRUE)
	    bConstantColorInSphere[i] = FALSE;
	alphaType[i]        = ppCGLAttrib[i]->mCAlphaType.mBody; 
	alphaDependBase[i]  = ppCGLAttrib[i]->mCAlphaDenendBase.mBody;
	alphaMax[i]         = ppCGLAttrib[i]->mCAlphaMax.mBody;
	alphaFactor[i]      = ppCGLAttrib[i]->mCAlphaFactor.mBody;
	qualityList[i]      = ppCGLAttrib[i]->mCDrawQuality.mBody;
	sphereDListList[i]  = GpCSphere->GetDList(1+qualityList[i]);
	blendModeList[i]    = ppCGLAttrib[i]->mCBlendType.mBody;
	drawTypeList[i]     = ppCGLAttrib[i]->mCDrawType.mBody;
	radiusTypeList[i]   = ppCGLAttrib[i]->mCRadiusType.mBody;
	radiusFactorList[i] = ppCGLAttrib[i]->GetRadiusFactor() * steppingRadiusFactor;
	bCastShadowList[i]  = (ppCGLAttrib[i]->mCTexture.mCCastShadow.mBody != 0);
	bShadowOffset[i] = FALSE;
	if (drawTypeList[i] == _GL_DRAW_BILLBOARD ||
	    drawTypeList[i] == _GL_DRAW_PSEUDO_SPHERE ||
	    drawTypeList[i] == _GL_DRAW_PSEUDO_OBJECT) bShadowOffset[i] = TRUE;
	    
	int objIndex = ppCGLAttrib[i]->mCObjectIndex.mBody;
	pCObj[i] = mspCFilterCtrlObjectLink->GetPCObject(objIndex);
	if (pCObj[i] != NULL){
	    pCObj[i]->UpdateDisplayList(msCurrentIndex);
	    for (int j = 0; j < _MAX_MESH; j++){
		displayListList[i][j] = pCObj[i]->mDisplayList[j];
	    }
	    pCObjTexture[i] = &pCObj[i]->mCGLTexture;
	    bUseTextureList[i] = TRUE; // TRUEȂꍇ
	}
	if (drawTypeList[i] == _GL_DRAW_BILLBOARD)
	    strTextureList[i] = ppCGLAttrib[i]->mCTexture.mCstrTexFormat[0].mstrBody;
	else if (drawTypeList[i] == _GL_DRAW_SPHERE)
	    strTextureList[i] = ppCGLAttrib[i]->mCTexture.mCstrTexFormat[1].mstrBody;
	else
	    strTextureList[i] << i; // KႤeNX`g

	bUseAtrasIndex[i] = FALSE;
	if (drawTypeList[i] == _GL_DRAW_PSEUDO_OBJECT &&
	    pCObj[i] != NULL && 
	    pCObj[i]->GetPCData()->mCbMultipleMesh.mbBody == TRUE)
	    bUseAtrasIndex[i] = TRUE;
    }

    bool bTextureMatrix[_MAX_PARTICLE_TYPE][_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	for (int j = 0; j < _MAX_PARTICLE_TYPE; j++){
	    if (strTextureList[i].IsSameAs(strTextureList[j]))
		bTextureMatrix[i][j] = TRUE;
	    else 
		bTextureMatrix[i][j] = FALSE;
	}
    }

    glEnable(GL_BLEND);
    glBindTexture(GL_TEXTURE_2D, ppCGLAttrib[0]->mTexture[0]);
    
    glEnable(GL_DEPTH_TEST);
    glDepthMask(GL_FALSE);
    glDisable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
    
    float pColor[4];
    float pClrMat[3][4];

    unsigned char *pType     = pCDataSnap->mpTypeLink;
    float (*pPos)[3] = pCDataSnap->mpX;
    float *pRadius = pCDataSnap->mpRadius;
    float *pValue1Mat[_MAX_PARTICLE_TYPE];    
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	pCDataSnap->SetPValue1(&pValue1Mat[i], ppCGLAttrib[i], 1);
    
    if (mbShadowVolume == TRUE){
	float lightDir[3] = {mLightPos[0], mLightPos[1], mLightPos[2]};
	float cameraPosCopied[3];
	LOOP_XYZ cameraPosCopied[xyz] = mCameraPos[xyz] - mCopyTranslate[xyz];
	mpCShadowVolume->CheckShadowSequential
	    (downsized, drawTypeList, radiusTypeList, 
	     radiusFactorList, pType, pPos, pRadius,
	     cameraPosCopied, lightDir, mLightPhi, mLightTheta);
	
	ppCGLAttrib[0]->BeginShader(0, TRUE);
	GLDrawShadowVolumeSequential
	    (pCDataSnap, downsized, drawTypeList, radiusTypeList, 
	     radiusFactorList, bCastShadowList, bShadowOffset,
	     pType, pPos, pRadius);
	ppCGLAttrib[0]->EndShader();
	return;
    }
    
    mbSeqTexEnabled = TRUE;
    mbSeqLightEnabled = FALSE;
    mbSeqDepthEnabled = FALSE;
    glEnable(GL_TEXTURE_2D);
    
    int progressInterval = GetProgressInterval(downsized);

    int typeCache = -1;
    float alphaFactorCache = -1;
    int iCache = -1;

    bool bCachedSort = (mpCZSort->mspCPreferenceLink->mCCachedSort.mBody == 1);

    for (int i_ = 0; i_ < downsized; i_++){
	
	if (i_ % progressInterval == 0 && IsTimeLapsedEnough() == TRUE){
	    GDrawingPercentage = CalcPercentage(i_);
	    UpdateStatusBar();
	    if (CheckAbortion() == TRUE){
		i_ = downsized; continue;
	    }
	}
	
	int i = (mspIndexAndZ+i_)->index;
	int particleType = pType[i];
	float filterAlphaFactor = ((mspIndexAndZ + i_))->alphaFactor;

	if (bCachedSort == TRUE){
	    if (typeCache != -1){
		if (particleType == typeCache){
		    i = iCache;
		    filterAlphaFactor = alphaFactorCache;
		    i_--;
		    typeCache = -1;
		}
	    }
	    else{
		if (particleType != prevParticleType){
		    typeCache = particleType;
		    alphaFactorCache = filterAlphaFactor;
		    iCache = i;
		    continue;
		}
	    }
	}
	
	GLAttribute_cls *pCGLAttrib =  ppCGLAttrib[particleType];
	int blendMode = blendModeList[particleType];
	int drawType  = drawTypeList[particleType];
	if (blendMode != prevBlendMode)
	    pCGLAttrib->SetBlendFunc();

	if (drawType == _GL_DRAW_POINT){
	    if (particleType != prevParticleType ||
		bConstantColorInBillboard[particleType] == FALSE){
		pCGLAttrib->EndShader();
		memcpy(pColor, pCDataSnap->mpColor[i], sizeof(float)*4);
		pColor[3] *= (steppingAlphaFactor*filterAlphaFactor);
		glColor4fv(pColor);
		EnableLight(FALSE);
		EnableTexture2D(FALSE);
		EnableDepthMask(TRUE);
		glPointSize(ppCGLAttrib[particleType]->mCPointSize.mBody);

	    }
	    glBegin(GL_POINTS);
	    glVertex3fv(pPos[i]);
	    glEnd();

	    prevParticleType = particleType;
	    prevBlendMode = blendMode;
	    prevDrawType = drawType;
	    continue;
	}

	float radiusFactor = radiusFactorList[particleType];
	if (radiusTypeList[particleType] != _GL_RADIUS_CONSTANT)
	    radiusFactor *= pRadius[i];
	
	switch(drawType){
	case _GL_DRAW_SPHERE: {

	    if (particleType != prevParticleType ||
		bConstantColorInSphere[particleType] == FALSE ||
		alphaType[particleType] != _GL_ALPHA_NORMAL){
		for (int mat = 0; mat < 3; mat++){
		    ppCGLAttrib[particleType]->
			mpCClrCurveMat[mat]->GetColor4fScaled
			(pClrMat[mat], pValue1Mat[particleType][i]); 
		    if (alphaType[particleType] == _GL_ALPHA_INVERSE_SQUARE){
			pClrMat[mat][3] *= alphaDependBase[particleType] * alphaDependBase[particleType] / (radiusFactor * radiusFactor);
			if (pClrMat[mat][3] > alphaMax[particleType])
			    pClrMat[mat][3] = alphaMax[particleType];
		    }
		    else
			pClrMat[mat][3] *= alphaFactor[particleType];
		    pClrMat[mat][3] *= (steppingAlphaFactor*filterAlphaFactor);
		    if (pClrMat[mat][3] > 1) pClrMat[mat][3] = 1;
		}
		
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  pClrMat[0]);
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  pClrMat[1]);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, pClrMat[2]);

		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 20);
		// ͋ʉ΃[v̊Oɏo͂
		
		if (particleType != prevParticleType){
		    if (prevParticleType == -1 ||
			bTextureMatrix[particleType][prevParticleType] == FALSE)
			pCGLAttrib->BindTexture();//
		    pCGLAttrib->BeginShader(mLightNum);
		}

		// drawTypeႤ͕̂KparticleTypeႤȂ̂
		// if̓
		if (drawType != prevDrawType){
		    EnableLight(TRUE);
		    EnableTexture2D(TRUE);
		    EnableDepthMask(TRUE);
		}
	    }

	    DrawDList(pPos[i], radiusFactor, sphereDListList[particleType]);
	    break; }
	case _GL_DRAW_BILLBOARD: 
	case _GL_DRAW_PSEUDO_SPHERE: 
	case _GL_DRAW_PSEUDO_OBJECT: {
	    
	    // F̔aɂωdownsizingˑ̂
	    // ŌvZ
	    if (particleType != prevParticleType ||
		bConstantColorInBillboard[particleType] == FALSE ||
		alphaType[particleType] != _GL_ALPHA_NORMAL){

		// prevParticleTypeɂ炸textureoffȂ̂
		if (drawType != prevDrawType){
		    EnableLight(FALSE);
		    EnableTexture2D(TRUE);
		    EnableDepthMask(pCGLAttrib->DoesUseDepthMask());
		}
		
		memcpy(pColor, pCDataSnap->mpColor[i], sizeof(float)*4);
		if (alphaType[particleType] == _GL_ALPHA_INVERSE_SQUARE){
		    pColor[3] /= (radiusFactor * radiusFactor);
		    if (pColor[3] > alphaMax[particleType])
			pColor[3] = alphaMax[particleType];
		}
		pColor[3] *= (steppingAlphaFactor*filterAlphaFactor);
		glColor4fv(pColor);
		if (particleType != prevParticleType){
		    if (prevParticleType == -1 ||
			bTextureMatrix[particleType][prevParticleType] == FALSE)
			pCGLAttrib->BindTexture();
		    pCGLAttrib->BeginGasShader(mLightNum);
		}
	    }
	    
	    if (pColor[3] < 0.0035){ 
		prevParticleType = particleType;
		prevBlendMode = blendMode;
		prevDrawType = drawType;
		continue;
	    }

	    if (qualityList[particleType] == 0){
		vertexArray[ 0] = rotatedArray[ 0] * radiusFactor;
		vertexArray[ 1] = rotatedArray[ 1] * radiusFactor;
		vertexArray[ 2] = rotatedArray[ 2] * radiusFactor;
		vertexArray[ 3] = rotatedArray[ 3] * radiusFactor;
		vertexArray[ 4] = rotatedArray[ 4] * radiusFactor;
		vertexArray[ 5] = rotatedArray[ 5] * radiusFactor;
		vertexArray[ 6] = rotatedArray[ 6] * radiusFactor;
		vertexArray[ 7] = rotatedArray[ 7] * radiusFactor;
		vertexArray[ 8] = rotatedArray[ 8] * radiusFactor;
		vertexArray[ 9] = rotatedArray[ 9] * radiusFactor;
		vertexArray[10] = rotatedArray[10] * radiusFactor;
		vertexArray[11] = rotatedArray[11] * radiusFactor;
	    }
	    else{
		GetVertexArrayLookAtCamera(pPos[i], vertexArray);
		for (int j = 0; j < 12; j++)
		    vertexArray[j] *= radiusFactor;
	    }

	    if (bUseAtrasIndex[particleType] == TRUE){
		int subIndex = pCDataSnap->mpAtrasIndex[i];
		GLDrawBillboardSequential(pPos[i], vertexArray,
					  drawType, prevDrawType,
					  &msTexCoordArrayAtras[(1+subIndex)*4*2]);
	    }
	    else
		GLDrawBillboardSequential(pPos[i], vertexArray,
					  drawType, prevDrawType,
					  msTexCoordArray);
	    break;}
	    
	    case _GL_DRAW_OBJECT: {
		if (pCObj[particleType] == NULL) break;
		if (particleType != prevParticleType ||
		    alphaType[particleType] != _GL_ALPHA_NORMAL){
		    
		    pCObj[particleType]->GLDrawSetMaterial();
		    pCObj[particleType]->GLDrawSetShaderBegin(mLightNum);
		    
		    pCObj[particleType]->GLDrawSetTexture(0); // Aj[VeNX`͑zOƂ

		    if (bUseTextureList[particleType] == TRUE)
			EnableTexture2D(TRUE); 
		    else
			EnableTexture2D(FALSE); 
		    pCObj[particleType]->GLDrawSetShaderBegin(mLightNum);

		    // drawTypeႤ͕̂KparticleTypeႤȂ̂
		    // if̓
		    if (drawType != prevDrawType){
			EnableLight(TRUE);
			EnableDepthMask(TRUE);
		    }
		}
		
		int subIndex = i % 3;
		DrawDList(pPos[i], radiusFactor, displayListList[particleType][subIndex]);
		break; }
	}

	prevParticleType = particleType;
	prevBlendMode = blendMode;
	prevDrawType = drawType;

    }

    glEnable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);

    ppCGLAttrib[0]->EndShader();
}

void GLDrawData_cls::
GLDrawShadowVolumeSequential
(SnapData_cls *pCDataSnap, int downsized, 
 int drawTypeList[_MAX_PARTICLE_TYPE],
 int radiusTypeList[_MAX_PARTICLE_TYPE],
 float radiusFactorList[_MAX_PARTICLE_TYPE],
 bool bCastShadowList[_MAX_PARTICLE_TYPE], 
 bool bShadowOffset[_MAX_PARTICLE_TYPE], 
 unsigned char *pType, float (*pPos)[3], float *pRadius){
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);
    
    int progressInterval = GetProgressInterval(downsized);

    for (int i_ = 0; i_ < downsized; i_++){
	
	if (i_ % progressInterval == 0 && IsTimeLapsedEnough() == TRUE){
	    GDrawingPercentage = CalcPercentage(i_);
	    UpdateStatusBar();
	    if (CheckAbortion() == TRUE){
		i_ = downsized; 
		continue;
	    }
	}
	
	int i = (mspIndexAndZ+i_)->index;
	int particleType = pType[i];
	int drawType = drawTypeList[particleType];
	if (bCastShadowList[particleType] == FALSE) continue;
	
	if (drawType == _GL_DRAW_POINT ||
	    drawType == _GL_HIDE ||
	    drawType == _GL_DRAW_BOTH) continue;
	
	float radiusFactor = radiusFactorList[particleType];
	if (radiusTypeList[particleType] != _GL_RADIUS_CONSTANT)
	    radiusFactor *= pRadius[i];

	mpCShadowVolume->DrawCylinder(pPos[i], radiusFactor, i, i_, 
				      bShadowOffset[particleType]);
    }

    glEnable(GL_DEPTH_TEST);
}

bool         GLDrawData_cls::msbInitialized = FALSE;
float        GLDrawData_cls::msVertexArrayOrg[12] = {
    -1, -1, 0,
    +1, -1, 0,
    +1, +1, 0,
    -1, +1, 0
};
 
SnapData_cls* GLDrawData_cls::mspCSnapShot = NULL;
InformationGUI_cls* GLDrawData_cls::mspCInfoPanel = NULL;
double GLDrawData_cls::msCurrentDoubleIndex = 0;
int    GLDrawData_cls::msCurrentIndex = 0;
