// -*- c -*- 
// "clsGLDrawSphere.h"

#include "clsGLDrawSphere.h"

#include "clsArrayAndIndex.h"
#include "../FILTER/clsFilterObject.h"
#include "../GLATTRIB/clsGLAttribute.h"
#include "../GLATTRIB/clsColorCurve.h"
#include "../GLOBJECTS/clsSphere.h"
extern Sphere_cls *GpCSphere;

#include "../COMMON/CommonDefinitions.h"

extern int       GDrawingPercentage;

ThreadCalcTranslate_cls::
ThreadCalcTranslate_cls() : wxThread(wxTHREAD_DETACHED){
}

void ThreadCalcTranslate_cls::
SetParameters(bool *pbFinishedFlag, int ini, int fin, int offset,
	      float (*pTranslateBuffer)[3], float *pRatioBuffer, 
	      ArrayAndIndex_cls *pIndexAndZ,
	      float (*pPos)[3], float *pRadius, float radiusFactor){
    mpbFinishedFlagLink = pbFinishedFlag;
    mIni = ini;
    mFin = fin;
    mOffset = offset;
    mpTranslateBuffer = pTranslateBuffer;
    mpRatioBuffer = pRatioBuffer;
    mpIndexAndZ = pIndexAndZ;
    mpPos = pPos;
    mpRadius = pRadius;
    mRadiusFactor = radiusFactor;

    if ( Create() != wxTHREAD_NO_ERROR) 
	//MsgBoxExit(NULL, "Error!", "Can't create thread");
	exit(0);
}

void ThreadCalcTranslate_cls::
CalcTranslate(){
    int ini = mIni;
    int fin = mFin;
    int offset = mOffset;
    float (*pTranslateBuffer)[3] = mpTranslateBuffer;
    float *pRatioBuffer = mpRatioBuffer;
    float (*pPos)[3] = mpPos;
    float *pRadius = mpRadius;
    float radiusFactor = mRadiusFactor;
    ArrayAndIndex_cls *pIndexAndZ = mpIndexAndZ;

    float scale = 1.0f;
    int prev = -1;
    float *pTrans = pTranslateBuffer[ini];
    for (int i_ = ini; i_ < fin; i_++){
	int i = (pIndexAndZ + offset + i_)->index;
	
	float ratio;
	if (i_ % PACKNUM == 0){
	    scale = 1.0f;
	    pTrans[X] = pPos[i][X];
	    pTrans[Y] = pPos[i][Y];
	    pTrans[Z] = pPos[i][Z];
	    ratio = pRadius[i] * radiusFactor;
	}
	else{
	    pTrans[X] = (pPos[i][X] - pPos[prev][X])/scale;
	    pTrans[Y] = (pPos[i][Y] - pPos[prev][Y])/scale;
	    pTrans[Z] = (pPos[i][Z] - pPos[prev][Z])/scale;
	    ratio = pRadius[i] / pRadius[prev];
	}
	pRatioBuffer[i_] = ratio;
	scale *= ratio;
	prev = i;
	pTrans +=3;
    }
}

wxThread::ExitCode ThreadCalcTranslate_cls::
Entry(){
    CalcTranslate();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ------

GLDrawSphere_cls::
GLDrawSphere_cls(){
    mpTranslateBufferA = NULL;
    mpTranslateBufferB = NULL;
    mpRatioBufferA = NULL;
    mpRatioBufferB = NULL;
}

GLDrawSphere_cls::
~GLDrawSphere_cls(){
    if (mpTranslateBufferA != NULL) delete [] mpTranslateBufferA;
    if (mpTranslateBufferB != NULL) delete [] mpTranslateBufferB;
    if (mpRatioBufferA != NULL) delete [] mpRatioBufferA;
    if (mpRatioBufferB != NULL) delete [] mpRatioBufferB;
}

void GLDrawSphere_cls::
MakeDList(int dList[_MAX_MESH], int *dListList, bool bUseTextue, int quality){
    for (int i = 0; i < _MAX_MESH; i++){
	if (dListList == NULL){
	    if (bUseTextue == TRUE) dList[i] = GpCSphere->GetDList(1+quality);
	    else                    dList[i] = GpCSphere->GetDListNoTex(1+quality);
	}
	else
	    dList[i] = dListList[i];
    }
}

void GLDrawSphere_cls::
GLDrawSphereConstRadiusConstColor(int downsized, bool bUseTextue, 
				  float (*pPos)[3], float radiusFactor,
				  int quality, int *dListList, 
				  FilterObject_cls *pCObj){

    int dList[_MAX_MESH];
    MakeDList(dList, dListList, bUseTextue, quality);

    for (int i = 0; i < _MAX_MESH; i++){
	if (dListList == NULL){
	    if (bUseTextue == TRUE) dList[i] = GpCSphere->GetDList(1+quality);
	    else                    dList[i] = GpCSphere->GetDListNoTex(1+quality);
	}
	else
	    dList[i] = dListList[i];
    }
    
    int dummy = GetProgressInterval(downsized);

    if (pCObj != NULL && pCObj->GetPCData()->mCbMultipleMesh.mbBody == FALSE)
	pCObj = NULL;

    for (int ibase = 0; ibase < downsized; ibase+=PACKNUM) {
	int number = PACKNUM;
	if (ibase + number >= downsized) number = downsized - ibase;
	
	if (ibase % (PACKNUM * 100) == 0 && IsTimeLapsedEnough() == TRUE){
	    GDrawingPercentage = CalcPercentage(ibase);
	    UpdateStatusBar();
	    if (CheckAbortion() == TRUE){
		ibase = downsized; continue;
	    }
	}
	
	glPushMatrix();
	int i = (mspIndexAndZ + ibase)->index;
	int subIndex = 0;
	if (pCObj != NULL) subIndex = pCObj->CallLuaSubIndex(i);
	glTranslatef(pPos[i][X]/radiusFactor, pPos[i][Y]/radiusFactor, pPos[i][Z]/radiusFactor);
	glCallList(dList[subIndex]);
	int prev = i;

	for (int i_ = ibase+1; i_ < ibase + number; i_ ++){
	    int i = (mspIndexAndZ + i_)->index;
	    int subIndex = 0;
	    if (pCObj != NULL) subIndex = pCObj->CallLuaSubIndex(i);
	    glTranslatef((pPos[i][X] - pPos[prev][X])/radiusFactor,
			 (pPos[i][Y] - pPos[prev][Y])/radiusFactor,
			 (pPos[i][Z] - pPos[prev][Z])/radiusFactor);
	    glCallList(dList[subIndex]);
	    prev = i;
	}
    	glPopMatrix();
    }
}

void GLDrawSphere_cls::
GLDrawSphereConstRadiusVariableColor
(int downsized, bool bUseTextue, float (*pPos)[3], float *pValue1,
 float radiusFactor, int quality, float steppingAlphaFactor,
 GLAttribute_cls *pCGLAttribute){
    
    int dList;
    if (bUseTextue == TRUE) dList = GpCSphere->GetDList(1+quality);
    else                    dList = GpCSphere->GetDListNoTex(1+quality);

    float pClrMat[3][4];
    ColorCurve_cls *pCClrCurveMat[3];
    for (int mat = 0; mat < 3; mat++){
	pCClrCurveMat[mat] = pCGLAttribute->mpCClrCurveMat[mat];
	pCClrCurveMat[mat]->PreCalcScaleFactor();
    }

    int dummy = GetProgressInterval(downsized);
    
    for (int ibase = 0; ibase < downsized; ibase+=PACKNUM) {
	int number = PACKNUM;
	if (ibase + number >= downsized) number = downsized - ibase;
	
	if (ibase % (PACKNUM * 100) == 0 && IsTimeLapsedEnough() == TRUE){
	    GDrawingPercentage = CalcPercentage(ibase);
	    UpdateStatusBar();
	    if (CheckAbortion() == TRUE){
		ibase = downsized; continue;
	    }
	}
	
	glPushMatrix();
	int i = (mspIndexAndZ + ibase)->index;
	float filterAlhplaFactor = ((mspIndexAndZ + ibase))->alphaFactor;
	glTranslatef(pPos[i][X]/radiusFactor, pPos[i][Y]/radiusFactor, pPos[i][Z]/radiusFactor);
	
	for (int mat = 0; mat < 3; mat++){
	    pCClrCurveMat[mat]->GetColor4fScaled(pClrMat[mat], pValue1[i]);
	    pClrMat[mat][3] *= pCGLAttribute->mCAlphaFactor.mBody;
	    pClrMat[mat][3] *= steppingAlphaFactor;
	    pClrMat[mat][3] *= filterAlhplaFactor;
	}
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  pClrMat[0]);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  pClrMat[1]);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, pClrMat[2]);
	
	glCallList(dList);
	
	int prev = i;
	for (int i_ = ibase+1; i_ < ibase + number; i_ ++){
	    int i = (mspIndexAndZ + i_)->index;
	    float filterAlhplaFactor = ((mspIndexAndZ + i_))->alphaFactor;
	    glTranslatef((pPos[i][X] - pPos[prev][X])/radiusFactor,
			 (pPos[i][Y] - pPos[prev][Y])/radiusFactor,
			 (pPos[i][Z] - pPos[prev][Z])/radiusFactor);

	    for (int mat = 0; mat < 3; mat++){
		pCClrCurveMat[mat]->GetColor4fScaled(pClrMat[mat], pValue1[i]);
		pClrMat[mat][3] *= pCGLAttribute->mCAlphaFactor.mBody;
		pClrMat[mat][3] *= steppingAlphaFactor;
		pClrMat[mat][3] *= filterAlhplaFactor;
	    }
	    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  pClrMat[0]);
	    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  pClrMat[1]);
	    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, pClrMat[2]);

	    glCallList(dList);
	    prev = i;
	}
	glPopMatrix();
    }
}

void GLDrawSphere_cls::
GLDrawSphereMTDraw(int number2Draw, bool bUseTextue, int ithread, int dList, 
		   float (*pTranslateBuffer)[3], float *pRatioBuffer){
   
    for (int ibase = 0; ibase < number2Draw; ibase+=PACKNUM) {
	int number = PACKNUM;
	if (ibase + number >= number2Draw) number = number2Draw - ibase;
	
	if (ibase % (PACKNUM * 100) == 0 && IsTimeLapsedEnough() == TRUE){
	    GDrawingPercentage = CalcPercentage(ibase+ithread);
	    UpdateStatusBar();
	    if (CheckAbortion() == TRUE){
		ibase = number2Draw; continue;
	    }
	}
	
	glPushMatrix();
	float *pTrans = pTranslateBuffer[ibase];
	for (int i_ = ibase; i_ < ibase + number; i_ ++){
	    glTranslatef(pTrans[X], pTrans[Y], pTrans[Z]);
	    float ratio = pRatioBuffer[i_];
	    if (ratio != 1)
		glScalef(ratio, ratio, ratio);
	    glCallList(dList);
	    pTrans += 3;
	}

	glPopMatrix();
    }
}

void GLDrawSphere_cls::
GLDrawSphereVariableRadiusConstColorMT
(int downsized, bool bUseTextue, float (*pPos)[3], float *pRadius,
 float radiusFactor, int dList){
    
    int progressInterval = GetProgressInterval(downsized);

    if (mpTranslateBufferA == NULL)
	mpTranslateBufferA = new float [_GL_DRAW_SPHERE_BUFFER][3];
    if (mpTranslateBufferB == NULL)
	mpTranslateBufferB = new float [_GL_DRAW_SPHERE_BUFFER][3];
    if (mpRatioBufferA == NULL)
	mpRatioBufferA = new float [_GL_DRAW_SPHERE_BUFFER];
    if (mpRatioBufferB == NULL)
	mpRatioBufferB = new float [_GL_DRAW_SPHERE_BUFFER];

    float (*pTranslateBufferCalc)[3];
    float *pRatioBufferCalc;
    float (*pTranslateBufferDraw)[3] = NULL;
    float *pRatioBufferDraw = NULL;
    
    int calculating = 0;
    int drawing     = -1;
    int number2Draw = -1;

    int ithread;
    for (ithread = 0; ithread < downsized; ithread += _GL_DRAW_SPHERE_BUFFER){
	int number = _GL_DRAW_SPHERE_BUFFER;
	if (ithread + number >= downsized) number = downsized - ithread;

	if (calculating == 0){
	    pTranslateBufferCalc = mpTranslateBufferA;
	    pRatioBufferCalc = mpRatioBufferA;
	}
	else{
	    pTranslateBufferCalc = mpTranslateBufferB;
	    pRatioBufferCalc = mpRatioBufferB;
	}
	
	int threadNum = 4;
	bool *pbFinished = new bool [threadNum];
	for (int thread = 0; thread < threadNum; thread++){
	    pbFinished[thread] = FALSE;
	    // !! ini  fin  PACKNUM PʂŕȂƂ߁I
	    
	    int init = thread * number / threadNum;
	    int fin = (thread + 1) * number / threadNum;
	    init -= init % PACKNUM;
	    if (thread != threadNum -1)	fin -= fin % PACKNUM;
	    ThreadCalcTranslate_cls *pCThread = new ThreadCalcTranslate_cls;
	    pCThread->SetParameters
		(&pbFinished[thread], init, fin, ithread,
		 pTranslateBufferCalc, pRatioBufferCalc,
		 mspIndexAndZ, pPos, pRadius, radiusFactor);
	    pCThread->Run();
	}

	if (pTranslateBufferDraw != NULL)
	    GLDrawSphereMTDraw(number2Draw, bUseTextue, ithread, dList,
			       pTranslateBufferDraw, pRatioBufferDraw);

	for (int thread = 0; thread < threadNum; thread++){
	    while (pbFinished[thread] == FALSE) wxMilliSleep(2);
	}

	pTranslateBufferDraw = pTranslateBufferCalc;
	pRatioBufferDraw = pRatioBufferCalc;
	calculating = 1 - calculating;
	number2Draw      = number;
	delete [] pbFinished;
    }
    
    GLDrawSphereMTDraw(number2Draw, bUseTextue, ithread, dList,
		       pTranslateBufferDraw, pRatioBufferDraw);
    
}

void GLDrawSphere_cls::
GLDrawSphereVariableRadiusConstColor
(int downsized, bool bUseTextue, float (*pPos)[3], float *pRadius,
 float radiusFactor, int quality, int *dListList,
 FilterObject_cls *pCObj){

    int dList[_MAX_MESH];
    MakeDList(dList, dListList, bUseTextue, quality);
    
    if (FALSE && downsized > PACKNUM * 20){
	GLDrawSphereVariableRadiusConstColorMT
	    (downsized, bUseTextue, pPos, pRadius,
	     radiusFactor, dList[0]);
	return;
    }

    if (pCObj != NULL && pCObj->GetPCData()->mCbMultipleMesh.mbBody == FALSE)
	pCObj = NULL;
    
    int progressInterval = GetProgressInterval(downsized);

    for (int ibase = 0; ibase < downsized; ibase+=PACKNUM) {
	int number = PACKNUM;
	if (ibase + number >= downsized) number = downsized - ibase;
	
	if (ibase % (PACKNUM * 100) == 0 && IsTimeLapsedEnough() == TRUE){
	    GDrawingPercentage = CalcPercentage(ibase);
	    UpdateStatusBar();
	    if (CheckAbortion() == TRUE){
		ibase = downsized; continue;
	    }
	}

	glPushMatrix();
	int i = (mspIndexAndZ + ibase)->index;
	int subIndex = 0;
	if (pCObj != NULL) subIndex = pCObj->CallLuaSubIndex(i);
	glTranslatef(pPos[i][X], pPos[i][Y], pPos[i][Z]);
	float scale = pRadius[i]*radiusFactor;
	glScalef(scale, scale, scale);
	glCallList(dList[subIndex]);
	int prev = i;
	
	float matrix[16] = {1, 0, 0, 0,
			    0, 1, 0, 0,
			    0, 0, 1, 0,
			    1, 1, 1, 1};

	for (int i_ = ibase+1; i_ < ibase + number; i_ ++){
	    int i = (mspIndexAndZ + i_)->index;
	    int subIndex = 0;
	    if (pCObj != NULL) subIndex = pCObj->CallLuaSubIndex(i);
	    
	    float ratio = pRadius[i] / pRadius[prev];
	    matrix[0] = ratio;
	    matrix[5] = ratio;
	    matrix[10] = ratio;
	    matrix[12] = (pPos[i][X] - pPos[prev][X])/scale;
	    matrix[13] = (pPos[i][Y] - pPos[prev][Y])/scale;
	    matrix[14] = (pPos[i][Z] - pPos[prev][Z])/scale;
	    glMultMatrixf(matrix);
	    scale *= ratio;
	    
	    glCallList(dList[subIndex]);
	    prev = i;
	}
	glPopMatrix();
    }
}

void GLDrawSphere_cls::
GLDrawSphereVariableRadiusVariableColor
(int downsized, bool bUseTextue, float (*pPos)[3], float *pRadius, 
 float *pValue1, float radiusFactor, float quality, 
 float steppingAlphaFactor, GLAttribute_cls *pCGLAttribute){

    int dList;
    if (bUseTextue == TRUE) dList = GpCSphere->GetDList(1+quality);
    else                    dList = GpCSphere->GetDListNoTex(1+quality);

    float pClrMat[3][4];
    ColorCurve_cls *pCClrCurveMat[3];
    for (int mat = 0; mat < 3; mat++){
	pCClrCurveMat[mat] = pCGLAttribute->mpCClrCurveMat[mat];
	pCClrCurveMat[mat]->PreCalcScaleFactor();
    }

    int progressInterval = GetProgressInterval(downsized);

    for (int i_ = 0; i_ < downsized; i_++){
	
	if (i_ % progressInterval == 0 && IsTimeLapsedEnough() == TRUE){
	    GDrawingPercentage = CalcPercentage(i_);
	    UpdateStatusBar();
	    if (CheckAbortion() == TRUE){
		i_ = downsized; continue;
	    }
	}
	
	int i = (mspIndexAndZ + i_)->index;
	float filterAlphaFactor = ((mspIndexAndZ + i_))->alphaFactor;
	float totalAlphaFactor = steppingAlphaFactor * filterAlphaFactor;
	
	for (int mat = 0; mat < 3; mat++){
	    pCClrCurveMat[mat]->GetColor4fScaled(pClrMat[mat], pValue1[i]);
	    pClrMat[mat][3] *= pCGLAttribute->mCAlphaFactor.mBody;
	    pClrMat[mat][3] *= totalAlphaFactor;
	}
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,  pClrMat[0]);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,  pClrMat[1]);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, pClrMat[2]);
	
	DrawDList(pPos[i], pRadius[i] * radiusFactor, dList);
    }
}

void GLDrawSphere_cls::
DrawDList(float pos[3], float scale, int dList){
    glPushMatrix();
    glTranslatef(pos[X], pos[Y], pos[Z]);
    glScalef(scale, scale, scale);
    glCallList(dList);
    glPopMatrix();
}
