
#include "clsSnapData.h"
#include "clsParticleDataBlock.h"
#include "clsParticleDataSnap.h"
#include "clsParticleDataSequence.h"

#include "INTERPOLATE_GET/clsThreadInterpolate.h"

#include "../FILTER/clsFilterObject.h"
#include "../GLATTRIB/clsColorCurve.h"
#include "../GLATTRIB/clsGLAttribute.h"
#include "../MAINCANVAS/clsDrawOption.h"
#include "../MYAPI/MsgBox.h"

bool GbSnapDataRefresh = FALSE;

#include <omp.h>

ThreadSnapInterpColor_cls::
ThreadSnapInterpColor_cls(wxThreadKind kind) :
    wxThread(kind) {
};

void ThreadSnapInterpColor_cls::
SetParameters(bool *pbFinishedFlag,
	      int PInit, int PFin, int downsizeStep, 
	      unsigned char *pTypeLinkMem, float (*pColorMem)[4],
	      float *pValue1ForColor[_MAX_PARTICLE_TYPE],
	      ColorCurve_cls* pColorCurveCol[_MAX_PARTICLE_TYPE],
	      int alphaType[_MAX_PARTICLE_TYPE], 
	      float alphaDependBase[_MAX_PARTICLE_TYPE], 
	      float alphaFactor[_MAX_PARTICLE_TYPE]){
    mpbFinishedFlagLink = pbFinishedFlag;
    mPIni = PInit; mPFin = PFin;
    mDownsizeStep = downsizeStep;
    mpTypeLinkMem = pTypeLinkMem;

    mpColorMem = pColorMem;
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	mpValue1ForColor[i] = pValue1ForColor[i];
	mpCClrCurveCol[i] = pColorCurveCol[i];
	mAlphaType[i] = alphaType[i];
	mAlphaDependBase[i] = alphaDependBase[i];
	mAlphaFactor[i] = alphaFactor[i];
    }

    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadSnapInterpColor_cls::
InterpolateColor(){

    unsigned char  *pTypeLinkMem = mpTypeLinkMem;
    float (*pColorMem)[4] = mpColorMem;
    float *pValue1ForColor[_MAX_PARTICLE_TYPE];
    ColorCurve_cls *pCClrCurveCol[_MAX_PARTICLE_TYPE];
    int alphaType[_MAX_PARTICLE_TYPE];
    float alphaDependBase[_MAX_PARTICLE_TYPE];
    float alphaFactor[_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	pValue1ForColor[i] = mpValue1ForColor[i];
	pCClrCurveCol[i] = mpCClrCurveCol[i];
	alphaType[i] = mAlphaType[i];
	alphaDependBase[i] = mAlphaDependBase[i];
	alphaFactor[i] = mAlphaFactor[i];
    }
    
    int downsizeStep = mDownsizeStep;
    int PFin = mPFin;
    
    if (downsizeStep == 1){
	for (int i = mPIni; i < PFin; i++){
	    int particleType = pTypeLinkMem[i];

	    pCClrCurveCol[particleType]->GetColor4fScaled(pColorMem[i], pValue1ForColor[particleType][i]);
	    if (alphaType[particleType] == _GL_ALPHA_INVERSE_SQUARE)
		pColorMem[i][3] *= alphaDependBase[particleType] * alphaDependBase[particleType];
	    else
		pColorMem[i][3] *= alphaFactor[particleType];
	}
	
	return;
    }
    else{
	int PInit = mPIni / downsizeStep;
	PInit *= downsizeStep;
	if (PInit != mPIni) PInit += downsizeStep;
	for (int i = PInit; i < PFin; i += downsizeStep){
	    int particleType = pTypeLinkMem[i];
	    
	    pCClrCurveCol[particleType]->GetColor4fScaled(pColorMem[i], pValue1ForColor[particleType][i]);
	    
	    if (alphaType[particleType] == _GL_ALPHA_INVERSE_SQUARE)
		pColorMem[i][3] *= alphaDependBase[particleType] * alphaDependBase[particleType];
	    else
		pColorMem[i][3] *= alphaFactor[particleType];
	}
	return;
    }
}

wxThread::ExitCode ThreadSnapInterpColor_cls::
Entry(){
    InterpolateColor();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ------------------


ThreadSnapInterpHermite_cls::
ThreadSnapInterpHermite_cls(wxThreadKind kind) :
    wxThread(kind) {
}

void ThreadSnapInterpHermite_cls::
SetParameters(bool *pbFinishedFlag,
	      int PInit, int PFin,
	      int downsizeStep, float DTime,
	      float (*pX)[3], float (*pV)[3],
	      float (*pA)[3], float (*pJ)[3], 
	      float *pRadius, float *pDRadius,
	      float *pValue1, float *pDValue1,
	      float (*pXMem)[3], 
	      float *pRadiusMem, 
	      float *pValue1Mem){
    
    mpbFinishedFlagLink = pbFinishedFlag;
    mPIni = PInit; mPFin = PFin;
    mDownsizeStep = downsizeStep;
    mDTime = DTime;
    mpX = pX; mpV = pV;
    mpA = pA; mpJ = pJ;
    mpRadius = pRadius;
    mpDRadius = pDRadius;
    mpValue1 = pValue1;
    mpDValue1 = pDValue1;

    mpXMem = pXMem;
    mpRadiusMem = pRadiusMem;
    mpValue1Mem = pValue1Mem;

    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadSnapInterpHermite_cls::
InterpolateDataHermite(){
    float dT = mDTime;
    
    float (*pX)[3]  = mpX;
    float (*pV)[3]  = mpV;
    float (*pA)[3]  = mpA;
    float (*pJ)[3]  = mpJ;
    float *pRadius  = mpRadius;
    float *pDRadius = mpDRadius;
    float *pValue1  = mpValue1;
    float *pDValue1 = mpDValue1;
    
    float (*pXMem)[3] = mpXMem;
    float *pRadiusMem = mpRadiusMem;
    float *pValue1Mem = mpValue1Mem;
        
    int downsizeStep = mDownsizeStep;
    int PFin = mPFin;

    if (downsizeStep == 1){
	memcpy(pXMem[mPIni], pX[mPIni], 4*3*(mPFin - mPIni)); 
	memcpy(&pRadiusMem[mPIni], &pRadius[mPIni], 4*(mPFin - mPIni));
	memcpy(&pValue1Mem[mPIni], &pValue1[mPIni], 4*(mPFin - mPIni)); 
	for (int i = mPIni; i < PFin; i++){
	    pXMem[i][0] += pV[i][0]*dT + pA[i][0]*dT*dT + pJ[i][0]*dT*dT*dT;
	    pXMem[i][1] += pV[i][1]*dT + pA[i][1]*dT*dT + pJ[i][1]*dT*dT*dT;
	    pXMem[i][2] += pV[i][2]*dT + pA[i][2]*dT*dT + pJ[i][2]*dT*dT*dT;
	    pRadiusMem[i] += pDRadius[i] * dT;
	    pValue1Mem[i] += pDValue1[i] * dT;
	}
    }
    else{
	int pInit = mPIni / downsizeStep;
	pInit *= downsizeStep;
	if (pInit != mPIni) pInit += downsizeStep;
	for (int i = pInit; i < PFin; i += downsizeStep){
	    pXMem[i][0] = pX[i][0] + pV[i][0] * dT;
	    pXMem[i][1] = pX[i][1] + pV[i][1] * dT;
	    pXMem[i][2] = pX[i][2] + pV[i][2] * dT;
	    pRadiusMem[i] = pRadius[i] + pDRadius[i] * dT;
	    pValue1Mem[i] = pValue1[i] + pDValue1[i] * dT;
	}
    }
}

wxThread::ExitCode ThreadSnapInterpHermite_cls::
Entry(){
    InterpolateDataHermite();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ---------

ThreadSnapInterpLinear_cls::
ThreadSnapInterpLinear_cls(wxThreadKind kind) :
    wxThread(kind) {
}

void ThreadSnapInterpLinear_cls::
SetParameters(bool *pbFinishedFlag,
	      int PInit, int PFin,
	      int downsizeStep, float DTime,
	      float (*pX)[3], float (*pV)[3],
	      float *pRadius, float *pDRadius,
	      float *pValue1, float *pDValue1,
	      float (*pXMem)[3], 
	      float *pRadiusMem, 
	      float *pValue1Mem){
    
    mpbFinishedFlagLink = pbFinishedFlag;
    mPIni = PInit;
    mPFin = PFin;
    mDownsizeStep = downsizeStep;
    mDTime = DTime;
    mpX = pX;
    mpV = pV;
    mpRadius = pRadius;
    mpDRadius = pDRadius;
    mpValue1 = pValue1;
    mpDValue1 = pDValue1;

    mpXMem = pXMem;
    mpRadiusMem = pRadiusMem;
    mpValue1Mem = pValue1Mem;

    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadSnapInterpLinear_cls::
InterpolateDataLinear(){
    float dT = mDTime;
    
    float (*pX)[3]  = mpX;
    float (*pV)[3]  = mpV;
    float *pRadius  = mpRadius;
    float *pDRadius = mpDRadius;
    float *pValue1  = mpValue1;
    float *pDValue1 = mpDValue1;
    
    float (*pXMem)[3] = mpXMem;
    float *pRadiusMem = mpRadiusMem;
    float *pValue1Mem = mpValue1Mem;

    int downsizeStep = mDownsizeStep;
    int PFin = mPFin;

    if (downsizeStep == 1){
	memcpy(pXMem[mPIni], pX[mPIni], 4*3*(mPFin - mPIni)); 
	memcpy(&pRadiusMem[mPIni], &pRadius[mPIni], 4*(mPFin - mPIni));
	memcpy(&pValue1Mem[mPIni], &pValue1[mPIni], 4*(mPFin - mPIni)); 
	for (int i = mPIni; i < PFin; i++){
	    pXMem[i][0] += pV[i][0] * dT;
	    pXMem[i][1] += pV[i][1] * dT;
	    pXMem[i][2] += pV[i][2] * dT;
	    pRadiusMem[i] += pDRadius[i] * dT;
	    pValue1Mem[i] += pDValue1[i] * dT;
	}
    }
    else{
	int pInit = mPIni / downsizeStep;
	pInit *= downsizeStep;
	if (pInit != mPIni) pInit += downsizeStep;
	for (int i = pInit; i < PFin; i += downsizeStep){
	    pXMem[i][0] = pX[i][0] + pV[i][0] * dT;
	    pXMem[i][1] = pX[i][1] + pV[i][1] * dT;
	    pXMem[i][2] = pX[i][2] + pV[i][2] * dT;
	    pRadiusMem[i] = pRadius[i] + pDRadius[i] * dT;
	    pValue1Mem[i] = pValue1[i] + pDValue1[i] * dT;
	}
    }
}

wxThread::ExitCode ThreadSnapInterpLinear_cls::
Entry(){
    InterpolateDataLinear();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// -----------------------------------

ThreadSetValue1DashType0_cls::
ThreadSetValue1DashType0_cls(wxThreadKind kind) :
    wxThread(kind) {
};

void ThreadSetValue1DashType0_cls::
SetParameters(bool *pbFinishedFlag, 
	      int PInit, int PFin, int downsizeStep,
	      int dependType, float *pValue1Dash,
	      float (*pX)[3], float (*pV)[3], int *pID){
    mpbFinishedFlagLink = pbFinishedFlag;
    mPIni = PInit; mPFin = PFin;
    mDownsizeStep = downsizeStep;
    
    mDependType = dependType;
    mpValue1Dash = pValue1Dash;
    mpX = pX; mpV = pV;
    mpID = pID;
    
    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadSetValue1DashType0_cls::
SetValue1Dash(){
    float (*pX)[3]  = mpX;
    float (*pV)[3]  = mpV;
    float *pValue1Dash = mpValue1Dash;
    int   *pID = mpID;
    
    int downsizeStep = mDownsizeStep;
    int PIni = mPIni;
    int PFin = mPFin;

    if (downsizeStep != 1){
	int PInit = mPIni / downsizeStep;
	PInit *= downsizeStep;
	if (PInit != mPIni) PInit += downsizeStep;
    }

    switch(mDependType){
    case _DEPEND_ON_VELOCITY:{
	for (int i = PIni; i < PFin; i += downsizeStep)
	    pValue1Dash[i] = sqrt(pV[i][0] * pV[i][0] + pV[i][1] * pV[i][1] + pV[i][2] * pV[i][2]);
	break; }
    case _DEPEND_ON_POSITION:{
	for (int i = PIni; i < PFin; i += downsizeStep)
	    pValue1Dash[i] = sqrt(pX[i][0] * pX[i][0] + pX[i][1] * pX[i][1] + pX[i][2] * pX[i][2]);
	break; }
    case _DEPEND_ON_POS_X: {
	for (int i = PIni; i < PFin; i += downsizeStep)
	    pValue1Dash[i] = pX[i][0];
	break; }	
    case _DEPEND_ON_POS_Y: {
	for (int i = PIni; i < PFin; i += downsizeStep)
	    pValue1Dash[i] = pX[i][1];
	break; }
    case _DEPEND_ON_POS_Z: {
	for (int i = PIni; i < PFin; i += downsizeStep)
	    pValue1Dash[i] = pX[i][2];
	break; }
    case _DEPEND_ON_V_X: {
	for (int i = PIni; i < PFin; i += downsizeStep)
	    pValue1Dash[i] = pV[i][2];
	break; }
    case _DEPEND_ON_V_Y: {
	for (int i = PIni; i < PFin; i += downsizeStep)
	    pValue1Dash[i] = pV[i][2];
	break; }
    case _DEPEND_ON_V_Z: {
	for (int i = PIni; i < PFin; i += downsizeStep)
	    pValue1Dash[i] = pV[i][2];
	break; }
    case _DEPEND_ON_ID: {
	for (int i = PIni; i < PFin; i+=mDownsizeStep)
	    pValue1Dash[i] = (float)pID[i];
	break; }
    }
}

wxThread::ExitCode ThreadSetValue1DashType0_cls::
Entry(){
    SetValue1Dash();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// -----------------------------------

ThreadSetValue1DashClassify_cls::
ThreadSetValue1DashClassify_cls(wxThreadKind kind) :
    wxThread(kind) {
};

void ThreadSetValue1DashClassify_cls::
SetParameters(bool *pbFinishedFlag, 
	      int PInit, int PFin, int downsizeStep,
	      int dependType, int particleType,
	      unsigned char *pType, float *pValue1Dash,
	      float (*pX)[3], float (*pV)[3], int *pID){
    mpbFinishedFlagLink = pbFinishedFlag;
    mPIni = PInit; mPFin = PFin;
    mDownsizeStep = downsizeStep;
    
    mDependType = dependType;
    mParticleType = particleType;
    mpType = pType;
    mpValue1Dash = pValue1Dash;
    mpX = pX; mpV = pV;
    mpID = pID;
    
    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadSetValue1DashClassify_cls::
SetValue1Dash(){
    float (*pX)[3]  = mpX;
    float (*pV)[3]  = mpV;
    unsigned char *pType = mpType;
    float *pValue1Dash = mpValue1Dash;
    int   *pID = mpID;
    
    int downsizeStep = mDownsizeStep;
    int particleType = mParticleType;
    int PIni = mPIni;
    int PFin = mPFin;

    if (downsizeStep != 1){
	int PInit = mPIni / downsizeStep;
	PInit *= downsizeStep;
	if (PInit != mPIni) PInit += downsizeStep;
    }

    switch(mDependType){
    case _DEPEND_ON_VELOCITY:{
	for (int i = PIni; i < PFin; i += downsizeStep){
	    if (pType[i] != particleType) continue;
	    pValue1Dash[i] = sqrt(pV[i][0] * pV[i][0] + pV[i][1] * pV[i][1] + pV[i][2] * pV[i][2]);
	}
	break; }
    case _DEPEND_ON_POSITION:{
	for (int i = PIni; i < PFin; i += downsizeStep){
	    if (pType[i] != particleType) continue;
	    pValue1Dash[i] = sqrt(pX[i][0] * pX[i][0] + pX[i][1] * pX[i][1] + pX[i][2] * pX[i][2]);
	}
	break; }
    case _DEPEND_ON_POS_X: {
	for (int i = PIni; i < PFin; i += downsizeStep){
	    if (pType[i] != particleType) continue;
	    pValue1Dash[i] = pX[i][0];
	}
	break; }	
    case _DEPEND_ON_POS_Y: {
	for (int i = PIni; i < PFin; i += downsizeStep){
	    if (pType[i] != particleType) continue;
	    pValue1Dash[i] = pX[i][1];
	}
	break; }
    case _DEPEND_ON_POS_Z: {
	for (int i = PIni; i < PFin; i += downsizeStep){
	    if (pType[i] != particleType) continue;
	    pValue1Dash[i] = pX[i][2];
	}
	break; }
    case _DEPEND_ON_V_X: {
	for (int i = PIni; i < PFin; i += downsizeStep){
	    if (pType[i] != particleType) continue;
	    pValue1Dash[i] = pV[i][2];
	}
	break; }
    case _DEPEND_ON_V_Y: {
	for (int i = PIni; i < PFin; i += downsizeStep){
	    if (pType[i] != particleType) continue;
	    pValue1Dash[i] = pV[i][2];
	}
	break; }
    case _DEPEND_ON_V_Z: {
	for (int i = PIni; i < PFin; i += downsizeStep){
	    if (pType[i] != particleType) continue;
	    pValue1Dash[i] = pV[i][2];
	}
	break; }
    case _DEPEND_ON_ID: {
	for (int i = PIni; i < PFin; i+=mDownsizeStep){
	    if (pType[i] != particleType) continue;
	    pValue1Dash[i] = (float)pID[i];
	}
	break; }
    }
}

wxThread::ExitCode ThreadSetValue1DashClassify_cls::
Entry(){
    SetValue1Dash();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// -----------------------------------

SnapData_cls::
SnapData_cls(PDataBlock_cls *pCDataBlockLink,
	     GLAttribute_cls **ppCGLAttribute,
	     double dT, int downsizeStep,
	     double doubleIndex, int frame){
    mpCDataBlockLink = pCDataBlockLink;
    mbAllocated  = FALSE;
    mbValue1dashAllocated = FALSE;
    mbTypeAllocated = FALSE;
    mbAtrasAllocated = FALSE;
    mTime = pCDataBlockLink->mpCParentSnap->mTime + dT;
    mDownsizeStep = downsizeStep;
    mCurrentDoubleIndex = doubleIndex;
    mCurrentFrame = frame;
    Allocate(pCDataBlockLink->mPNumBlock);
    mOrder = pCDataBlockLink->mOrder;
    Interpolate(ppCGLAttribute, dT);
    LuaBase_cls::SetSnapshotLink(this);
    Classify(ppCGLAttribute);
    InterpolateColor(ppCGLAttribute);
}

SnapData_cls::
~SnapData_cls(){
    ReleaseData();
}

void SnapData_cls::
Allocate(int pNum){
    mPNumBlock = pNum;
    if (pNum == 0) return;

    mpX = new float [mPNumBlock][3];
    mpRadius = new float [mPNumBlock];
    mpValue1 = new float [mPNumBlock];
    mpColor = new float [mPNumBlock][4];
    
    mbAllocated = TRUE;
}

void SnapData_cls::
AllocateValue1Dash(){
    mbValue1dashAllocated = TRUE;
    mpValue1dash = new float [mPNumBlock];
}

void SnapData_cls::
AllocateType(){
    mbTypeAllocated = TRUE;
    mpTypeSnapLocal = new unsigned char [mPNumBlock];
}

void SnapData_cls::
AllocateAtrasIndex(){
    mbAtrasAllocated = TRUE;
    mpAtrasIndex = new unsigned char [mPNumBlock];
}

void SnapData_cls::
ReleaseData(){
    if (mbAllocated == FALSE){
	printf("Release Snap failed\n");
	return;
    }
    
    delete[] mpX;
    delete[] mpRadius;
    delete[] mpValue1;
    delete[] mpColor;

    if (mbValue1dashAllocated == TRUE){
	delete [] mpValue1dash;
	mbValue1dashAllocated = FALSE;
	
    }
    if (mbTypeAllocated == TRUE){
	delete [] mpTypeSnapLocal;
	mbTypeAllocated = FALSE;
    }

    if (mbAtrasAllocated == TRUE){
	delete [] mpAtrasIndex;
	mbAtrasAllocated = FALSE;
    }

    mbAllocated = FALSE;
}

bool* SnapData_cls::
GetNewPCbFinishList(int &threadNumber, int pNum){
    threadNumber = 1;
    if (pNum > 10000) threadNumber = 2;
    if (pNum > 100000) threadNumber = 3;
    if (pNum > 1000000) threadNumber = 4;
    if (pNum > 10000000) threadNumber = 5;
    bool *bAFinished = new bool [threadNumber];
    for (int core = 0; core < threadNumber; core++)
	bAFinished[core] = FALSE;
    return bAFinished;
}

bool SnapData_cls::
IsAtrasNeeded(GLAttribute_cls **ppCGLAttribute){
    bool bRet = FALSE;
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	bRet |= ppCGLAttribute[i]->IsAtrasNeeded();
    }
    return bRet;
}

bool SnapData_cls::
IsItSameSnap(PDataBlock_cls *pCDataBlockLink, double dT, int step, int frame){
    if (GbSnapDataRefresh == TRUE){
	GbSnapDataRefresh = FALSE;
	printf("GbSnapDataRefresh!!\n");
	return FALSE;
    }
    
    if (mTime != pCDataBlockLink->mpCParentSnap->mTime + dT) return FALSE;
    if (mDownsizeStep != step && mDownsizeStep != 1) return FALSE;

    // !! FIX_ME !! tCX[frameႤɍč\zKvȂ̂́A
    // ClassifyexpressionntframeɈˑꍇ̂
    if (mCurrentFrame != frame) return FALSE;

    return TRUE;
}

void SnapData_cls::
SetPValue1(float **ppValue1, GLAttribute_cls *pCGLAttribute, int particleType){
    int dependType;
    
    if (pCGLAttribute->mCDrawType.mBody == _GL_DRAW_SPHERE)
	dependType = pCGLAttribute->mpCClrCurveMat[0]->mCDependType.mBody;
    else
	dependType = pCGLAttribute->mpCClrCurveCol->mCDependType.mBody;
	
    switch(dependType){
    case _DEPEND_ON_VALUE1:
	*ppValue1 = mpValue1; break;
    case _DEPEND_ON_RADIUS:
	*ppValue1 = mpRadius; break;
    case _DEPEND_ON_VELOCITY:
    case _DEPEND_ON_POSITION:
    case _DEPEND_ON_POS_X: case _DEPEND_ON_POS_Y: case _DEPEND_ON_POS_Z:
    case _DEPEND_ON_V_X:   case _DEPEND_ON_V_Y:   case _DEPEND_ON_V_Z:
    case _DEPEND_ON_ID:
	SetValue1Dash(dependType, particleType);
	*ppValue1 = mpValue1dash; break; 
    }
}

void SnapData_cls::
InterpolateNot(PDataBlock_cls *pCDataBlockLink, 
	       GLAttribute_cls **ppCGLAttribute){

    //int   alphaType[_MAX_PARTICLE_TYPE];
    //float alphaDependBase[_MAX_PARTICLE_TYPE];
    //float alphaMax[_MAX_PARTICLE_TYPE];
    //float alphaFactor[_MAX_PARTICLE_TYPE];
    //ColorCurve_cls *pCClrCurveCol[_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	//alphaType[i]        = ppCGLAttribute[i]->mCAlphaType.mBody; 
	//alphaDependBase[i]  = ppCGLAttribute[i]->mCAlphaDenendBase.mBody;
	//alphaMax[i]         = ppCGLAttribute[i]->mCAlphaMax.mBody;
	//alphaFactor[i]      = ppCGLAttribute[i]->mCAlphaFactor.mBody;
	ppCGLAttribute[i]->mpCClrCurveCol->PreCalcScaleFactor();
	//pCClrCurveCol[i] = ppCGLAttribute[i]->mpCClrCurveCol;
    }

    memcpy(mpX, pCDataBlockLink->mpX, sizeof(float)*3*mPNumBlock);
    memcpy(mpRadius, pCDataBlockLink->mpRadius, sizeof(float)*mPNumBlock);
    memcpy(mpValue1, pCDataBlockLink->mpValue1, sizeof(float)*mPNumBlock);

    /*
    float *pValue1ForColor[_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	SetPValue1(&pValue1ForColor[i], ppCGLAttribute[i], i);

    float (*pColorMem)[4] = mpColor;
    unsigned char *pTypeLinkMem = mpTypeLink;

    int threadNumber;
    bool *bAFinished = GetNewPCbFinishList(threadNumber, mPNumBlock);

    ThreadSnapInterpColor_cls **ppCThreadSnapInterp = 
	new ThreadSnapInterpColor_cls* [threadNumber];
    for (int core = 0; core < threadNumber; core++)
	bAFinished[core] = FALSE;
    
    for (int core = 0; core < threadNumber; core++){
	ppCThreadSnapInterp[core] = 
	    new ThreadSnapInterpColor_cls(wxTHREAD_DETACHED);

	int PInit = mPNumBlock / threadNumber * core;
	int PFin  = mPNumBlock / threadNumber * (core+1);
	if (core == threadNumber - 1) PFin = mPNumBlock;

	ppCThreadSnapInterp[core]->SetParameters
	    (&bAFinished[core], PInit, PFin, mDownsizeStep, 
	     pTypeLinkMem, pColorMem, pValue1ForColor, pCClrCurveCol,
	     alphaType, alphaDependBase, alphaFactor);

	ppCThreadSnapInterp[core]->Run();
    }

    bool bFinishedAll = FALSE;
    while (bFinishedAll == FALSE){
	wxMilliSleep(3);
	bFinishedAll = TRUE;
	for (int core = 0; core < threadNumber; core++)
	    bFinishedAll &= bAFinished[core];
    }
    
    delete [] ppCThreadSnapInterp;
    delete [] bAFinished;
    */

    /*
   
    for (int i = 0; i < mPNumBlock; i++){

	int particleType = pTypeLinkMem[i];

	if (particleType >= _MAX_PARTICLE_TYPE || particleType < 0){
	    particleType = 0;
	}
	
	// F̊{͌vZĂ
	pCClrCurveCol[particleType]->GetColor4fScaled(pColorMem[i], pValue1[particleType][i]);
	if (alphaType[particleType] == _GL_ALPHA_INVERSE_SQUARE){
	    pColorMem[i][3] *= alphaDependBase[particleType] * alphaDependBase[particleType];
	}
	else
	    pColorMem[i][3] *= alphaFactor[particleType];
    }
    */
}

bool SnapData_cls::
IsInterpolateRequired(GLAttribute_cls **ppCGLAttribute){
    bool bRequired = FALSE;
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	switch(ppCGLAttribute[i]->mCDrawType.mBody){
	case _GL_DRAW_POINT:
	case _GL_DRAW_BILLBOARD:
	case _GL_DRAW_BOTH:
	    if (ppCGLAttribute[i]->mpCClrCurveCol->mCDependType.mBody >= 
		_DEPEND_ON_POSITION)
		bRequired = TRUE;
	    break;
	case _GL_DRAW_SPHERE:
	    for (int j = 0; j < 3; j++)
		if (ppCGLAttribute[i]->mpCClrCurveMat[j]->mCDependType.mBody >= 
		    _DEPEND_ON_POSITION)
		    bRequired = TRUE;
	    break;
	}
    return bRequired;
}

void SnapData_cls::
Classify(GLAttribute_cls **ppCGLAttribute){
    int classifyType = GLAttribute_cls::mspCDrawOptionLink->mCClassify.mBody;

    switch(classifyType){
    case _CLASSIFY_ALL_PARTILE0: {
	if (mbTypeAllocated == FALSE) AllocateType();
	for (int i = 0; i < mPNumBlock; i++) mpTypeSnapLocal[i] = 0;
	mpTypeLink = mpTypeSnapLocal;
	break; }
    case _CLASSIFY_BY_DATA: 
	mpTypeLink = mpCDataBlockLink->mpType;
	break; 
    case _CLASSIFY_EXPRESSION: {
	if (mbTypeAllocated == FALSE) AllocateType();
	GLAttribute_cls::mspCDrawOptionLink->GetTypeClassified
	    (mPNumBlock, mpCDataBlockLink->mpID, mpCDataBlockLink->mpType,
	     mpTypeSnapLocal, mCurrentDoubleIndex, mCurrentFrame);
	mpTypeLink = mpTypeSnapLocal;
	break; }
    }

    if (IsAtrasNeeded(ppCGLAttribute) == TRUE){
	AllocateAtrasIndex();
	FilterObject_cls *pCObj[_MAX_PARTICLE_TYPE];
	for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	    pCObj[i] = ppCGLAttribute[i]->GetPCObject();

	// !! FIX_ME !! XbhƂ _MAX_PARTICLE_TYPE Luapӂ
	// sł͂B
	for (int i = 0; i < mPNumBlock; i++){
	    if (pCObj[mpTypeLink[i]] != NULL) 
		mpAtrasIndex[i] = pCObj[mpTypeLink[i]]->CallLuaSubIndex(i);
	}
    }
}

void SnapData_cls::
Interpolate(GLAttribute_cls **ppCGLAttribute, 
	    double dT){

    if (dT == 0 && IsInterpolateRequired(ppCGLAttribute) == FALSE){
	// !! FIX_ME !! xˑƂdT=0łԂKv
	InterpolateNot(mpCDataBlockLink, ppCGLAttribute);
	return;
    }

    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	ppCGLAttribute[i]->mpCClrCurveCol->PreCalcScaleFactor();
    }

    float (*pX)[3]  = mpCDataBlockLink->mpX;
    float (*pV)[3]  = mpCDataBlockLink->mpV;
    float (*pA)[3]  = mpCDataBlockLink->mpA;
    float (*pJ)[3]  = mpCDataBlockLink->mpJ;
    float *pRadius  = mpCDataBlockLink->mpRadius;
    float *pDRadius = mpCDataBlockLink->mpDRadius;
    float *pValue1  = mpCDataBlockLink->mpValue1;
    float *pDValue1 = mpCDataBlockLink->mpDValue1;
    
    float (*pXMem)[3] = mpX;
    float *pRadiusMem = mpRadius;
    float *pValue1Mem = mpValue1;

    

    /*
#ifdef _OPENMP
    // 1%xȂۂB
    omp_set_dynamic( 0 );
    omp_set_num_threads( 2 );
#pragma omp parallel for
#endif
    */

    PDataSnap_cls *pCSnap = mpCDataBlockLink->mpCParentSnap;
    //printf("Snap Interpolate Index=%d Order=%d InterpolateFinishd = %d\n", pCSnap->mSelfIndex, mOrder, pCSnap->mbInterpolateFinishd);
    
    int threadNumber;
    bool *bAFinished = GetNewPCbFinishList(threadNumber, mPNumBlock);
    
    if (mOrder == _DATABLOCK_ORDER_1){
	ThreadSnapInterpLinear_cls **ppCThreadSnapInterp = 
	    new ThreadSnapInterpLinear_cls* [threadNumber];
	
	for (int core = 0; core < threadNumber; core++){
	    ppCThreadSnapInterp[core] = 
		new ThreadSnapInterpLinear_cls(wxTHREAD_DETACHED);
	    
	    int PInit = mPNumBlock / threadNumber * core;
	    int PFin  = mPNumBlock / threadNumber * (core+1);
	    if (core == threadNumber - 1) PFin = mPNumBlock;
	    
	    ppCThreadSnapInterp[core]->SetParameters
		(&bAFinished[core], PInit, PFin, mDownsizeStep, dT,
		 pX, pV, pRadius, pDRadius, pValue1, pDValue1,
		 pXMem, pRadiusMem, pValue1Mem);
	    ppCThreadSnapInterp[core]->Run();
	}
	
	bool bFinishedAll = FALSE;
	while (bFinishedAll == FALSE){
	    wxMilliSleep(3);
	    bFinishedAll = TRUE;
	    for (int core = 0; core < threadNumber; core++)
		bFinishedAll &= bAFinished[core];
	}
	
	delete [] ppCThreadSnapInterp;
    }

    if (mOrder == _DATABLOCK_ORDER_3){
	ThreadSnapInterpHermite_cls **ppCThreadSnapInterp = 
	    new ThreadSnapInterpHermite_cls* [threadNumber];
	
	for (int core = 0; core < threadNumber; core++){
	    ppCThreadSnapInterp[core] = 
		new ThreadSnapInterpHermite_cls(wxTHREAD_DETACHED);
	    
	    int PInit = mPNumBlock / threadNumber * core;
	    int PFin  = mPNumBlock / threadNumber * (core+1);
	    if (core == threadNumber - 1) PFin = mPNumBlock;
	    
	    ppCThreadSnapInterp[core]->SetParameters
		(&bAFinished[core], PInit, PFin, mDownsizeStep, dT,
		 pX, pV, pA, pJ, pRadius, pDRadius, pValue1, pDValue1,
		 pXMem, pRadiusMem, pValue1Mem);
	    ppCThreadSnapInterp[core]->Run();
	}
	
	bool bFinishedAll = FALSE;
	while (bFinishedAll == FALSE){
	    wxMilliSleep(3);
	    bFinishedAll = TRUE;
	    for (int core = 0; core < threadNumber; core++)
		bFinishedAll &= bAFinished[core];
	}
	
	delete [] ppCThreadSnapInterp;
    }

    /*
    float (*pColorMem)[4] = mpColor;
    unsigned char *pTypeLinkMem = mpTypeLink;

    float *pValue1ForColor[_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	SetPValue1(&pValue1ForColor[i], ppCGLAttribute[i], i);

    ThreadSnapInterpColor_cls **ppCThreadSnapInterp = 
	new ThreadSnapInterpColor_cls* [threadNumber];
    for (int core = 0; core < threadNumber; core++)
	bAFinished[core] = FALSE;
    
    for (int core = 0; core < threadNumber; core++){
	ppCThreadSnapInterp[core] = 
	    new ThreadSnapInterpColor_cls(wxTHREAD_DETACHED);

	int PInit = mPNumBlock / threadNumber * core;
	int PFin  = mPNumBlock / threadNumber * (core+1);
	if (core == threadNumber - 1) PFin = mPNumBlock;

	ppCThreadSnapInterp[core]->SetParameters
	    (&bAFinished[core], PInit, PFin, mDownsizeStep, 
	     pTypeLinkMem, pColorMem, pValue1ForColor, pCClrCurveCol,
	     alphaType, alphaDependBase, alphaFactor);

	ppCThreadSnapInterp[core]->Run();
    }

    bool bFinishedAll = FALSE;
    while (bFinishedAll == FALSE){
	wxMilliSleep(3);
	bFinishedAll = TRUE;
	for (int core = 0; core < threadNumber; core++)
	    bFinishedAll &= bAFinished[core];
    }

    */
    
    //delete [] ppCThreadSnapInterp;
    delete [] bAFinished;
}

void SnapData_cls::
SetValue1Dash(int dependType, int particleType){
    
    if (mbValue1dashAllocated == FALSE) AllocateValue1Dash();

    int threadNumber;
    bool *bAFinished = GetNewPCbFinishList(threadNumber, mPNumBlock);
    
    if (GLAttribute_cls::mspCDrawOptionLink->mCClassify.mBody == 0){
	//ޕ̎
	if (particleType == 0){
	    ThreadSetValue1DashType0_cls **ppCThreadSetValue1Dash = 
	    new ThreadSetValue1DashType0_cls* [threadNumber];
	    
	    for (int core = 0; core < threadNumber; core++){
		ppCThreadSetValue1Dash[core] = 
		    new ThreadSetValue1DashType0_cls(wxTHREAD_DETACHED);
		
		int PInit = mPNumBlock / threadNumber * core;
		int PFin  = mPNumBlock / threadNumber * (core+1);
		if (core == threadNumber - 1) PFin = mPNumBlock;
		
		ppCThreadSetValue1Dash[core]->SetParameters
		    (&bAFinished[core], PInit, PFin, mDownsizeStep,
		     dependType, mpValue1dash, mpX, 
		     mpCDataBlockLink->mpV, mpCDataBlockLink->mpID);
		ppCThreadSetValue1Dash[core]->Run();
	    }
	    
	    bool bFinishedAll = FALSE;
	    while (bFinishedAll == FALSE){
		wxMilliSleep(3);
		bFinishedAll = TRUE;
		for (int core = 0; core < threadNumber; core++)
		    bFinishedAll &= bAFinished[core];
	    }

	    delete [] ppCThreadSetValue1Dash;
	}
	delete [] bAFinished;
	return;
    }
    
    // ޕꍇ
    ThreadSetValue1DashClassify_cls **ppCThreadSetValue1Dash = 
	new ThreadSetValue1DashClassify_cls* [threadNumber];
    
    for (int core = 0; core < threadNumber; core++){
	ppCThreadSetValue1Dash[core] = 
	    new ThreadSetValue1DashClassify_cls(wxTHREAD_DETACHED);

	int PInit = mPNumBlock / threadNumber * core;
	int PFin  = mPNumBlock / threadNumber * (core+1);
	if (core == threadNumber - 1) PFin = mPNumBlock;
	
	ppCThreadSetValue1Dash[core]->SetParameters
	    (&bAFinished[core], PInit, PFin, mDownsizeStep,
	     dependType, particleType,
	     mpTypeLink, mpValue1dash, mpX, 
	     mpCDataBlockLink->mpV, mpCDataBlockLink->mpID);
	ppCThreadSetValue1Dash[core]->Run();
    }

    bool bFinishedAll = FALSE;
    while (bFinishedAll == FALSE){
	wxMilliSleep(3);
	bFinishedAll = TRUE;
	for (int core = 0; core < threadNumber; core++)
	    bFinishedAll &= bAFinished[core];
    }
    
    delete [] ppCThreadSetValue1Dash;
    delete [] bAFinished;
}

void SnapData_cls::
InterpolateColor(GLAttribute_cls **ppCGLAttribute){

    int   alphaType[_MAX_PARTICLE_TYPE];
    float alphaDependBase[_MAX_PARTICLE_TYPE];
    //float alphaMax[_MAX_PARTICLE_TYPE];
    float alphaFactor[_MAX_PARTICLE_TYPE];
    ColorCurve_cls *pCClrCurveCol[_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	alphaType[i]        = ppCGLAttribute[i]->mCAlphaType.mBody; 
	alphaDependBase[i]  = ppCGLAttribute[i]->mCAlphaDenendBase.mBody;
	//alphaMax[i]         = ppCGLAttribute[i]->mCAlphaMax.mBody;
	alphaFactor[i]      = ppCGLAttribute[i]->mCAlphaFactor.mBody;
	pCClrCurveCol[i] = ppCGLAttribute[i]->mpCClrCurveCol;
    }
    
    float *pValue1ForColor[_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	SetPValue1(&pValue1ForColor[i], ppCGLAttribute[i], i);

    float (*pColorMem)[4] = mpColor;
    unsigned char *pTypeLinkMem = mpTypeLink;

    int threadNumber;
    bool *bAFinished = GetNewPCbFinishList(threadNumber, mPNumBlock);

    ThreadSnapInterpColor_cls **ppCThreadSnapInterp = 
	new ThreadSnapInterpColor_cls* [threadNumber];
    for (int core = 0; core < threadNumber; core++)
	bAFinished[core] = FALSE;
    
    for (int core = 0; core < threadNumber; core++){
	ppCThreadSnapInterp[core] = 
	    new ThreadSnapInterpColor_cls(wxTHREAD_DETACHED);

	int PInit = mPNumBlock / threadNumber * core;
	int PFin  = mPNumBlock / threadNumber * (core+1);
	if (core == threadNumber - 1) PFin = mPNumBlock;

	ppCThreadSnapInterp[core]->SetParameters
	    (&bAFinished[core], PInit, PFin, mDownsizeStep, 
	     pTypeLinkMem, pColorMem, pValue1ForColor, pCClrCurveCol,
	     alphaType, alphaDependBase, alphaFactor);

	ppCThreadSnapInterp[core]->Run();
    }

    bool bFinishedAll = FALSE;
    while (bFinishedAll == FALSE){
	wxMilliSleep(3);
	bFinishedAll = TRUE;
	for (int core = 0; core < threadNumber; core++)
	    bFinishedAll &= bAFinished[core];
    }

    delete [] ppCThreadSnapInterp;
    delete [] bAFinished;
}
