// -*- c -*- 
// "clsSnapData.cpp"

#ifndef _SNAP_DATA_H_
#define _SNAP_DATA_H_

class ColorCurve_cls;
class GLAttribute_cls;
class PDataBlock_cls;

#include <wx/thread.h>

#include "../COMMON/CommonDefinitionsForH.h"

class ThreadSnapInterpColor_cls : public wxThread {
 public:
    ThreadSnapInterpColor_cls(wxThreadKind kind);

    void SetParameters(bool *pbFinishedFlag,
		       int PInit, int PFin, int downsizeStep, 
		       unsigned char *pTypeLinkMem, float (*ColorMem)[4],
		       float *pValue1ForColor[_MAX_PARTICLE_TYPE],
		       ColorCurve_cls* pColorCurveCol[_MAX_PARTICLE_TYPE],
		       int alphaType[_MAX_PARTICLE_TYPE], 
		       float alphaDependBase[_MAX_PARTICLE_TYPE], 
		       float alphaFactor[_MAX_PARTICLE_TYPE]);
    
    bool *mpbFinishedFlagLink;

    unsigned char  *mpTypeLinkMem;
    float (*mpColorMem)[4];
    float *mpValue1ForColor[_MAX_PARTICLE_TYPE];
    ColorCurve_cls *mpCClrCurveCol[_MAX_PARTICLE_TYPE];

    int  mAlphaType[_MAX_PARTICLE_TYPE];
    float mAlphaDependBase[_MAX_PARTICLE_TYPE];
    float mAlphaFactor[_MAX_PARTICLE_TYPE];

    int   mPIni, mPFin;
    int   mDownsizeStep;

    void InterpolateColor();
    virtual ExitCode Entry();
};

class ThreadSnapInterpHermite_cls : public wxThread {
 public:
    ThreadSnapInterpHermite_cls(wxThreadKind kind);

    void SetParameters(bool *pbFinishedFlag,
		       int PInit, int PFin,
		       int downsizeStep,
		       float DTime,
		       float (*pX)[3], float (*pV)[3],
		       float (*pA)[3], float (*pJ)[3], 
		       float *pRadius, float *pDRadius,
		       float *pValue1, float *pDValue1,
		       float (*pXMem)[3],
		       float *pRadiusMem, 
		       float *pValue1Mem);

    bool *mpbFinishedFlagLink;
    float mDTime;
    float (*mpX)[3], (*mpV)[3], (*mpA)[3], (*mpJ)[3];
    float *mpRadius, *mpDRadius;
    float *mpValue1, *mpDValue1;
    
    float (*mpXMem)[3];
    float *mpRadiusMem;
    float *mpValue1Mem;

    int   mPIni, mPFin;
    int   mDownsizeStep;

    void InterpolateDataHermite();

    virtual ExitCode Entry();
};

class ThreadSnapInterpLinear_cls : public wxThread {
 public:
    ThreadSnapInterpLinear_cls(wxThreadKind kind);

    void SetParameters(bool *pbFinishedFlag,
		       int PInit, int PFin, int downsizeStep,
		       float DTime,
		       float (*pX)[3], float (*pV)[3],
		       float *pRadius, float *pDRadius,
		       float *pValue1, float *pDValue1,
		       float (*pXMem)[3],
		       float *pRadiusMem, 
		       float *pValue1Mem);

    bool *mpbFinishedFlagLink;
    float mDTime;
    float (*mpX)[3], (*mpV)[3], (*mpA)[3], (*mpJ)[3];
    float *mpRadius, *mpDRadius;
    float *mpValue1, *mpDValue1;
    
    float (*mpXMem)[3];
    float *mpRadiusMem;
    float *mpValue1Mem;

    int   mPIni, mPFin;
    int   mDownsizeStep;

    void InterpolateDataLinear();

    virtual ExitCode Entry();
};

class ThreadSetValue1DashType0_cls : public wxThread {
 public:
    ThreadSetValue1DashType0_cls(wxThreadKind kind);

    void SetParameters(bool *pbFinishedFlag,  
		       int PInit, int PFin, int downsizeStep,
		       int dependType, float *pValue1Dash,
		       float (*pX)[3], float (*pV)[3], int *pID);
    
    bool *mpbFinishedFlagLink;
    int mDependType;
    float *mpValue1Dash;
    float (*mpX)[3], (*mpV)[3];
    int   *mpID;

    int   mPIni, mPFin;
    int   mDownsizeStep;

    void SetValue1Dash();
    
    virtual ExitCode Entry();
};

class ThreadSetValue1DashClassify_cls : public wxThread {
 public:
    ThreadSetValue1DashClassify_cls(wxThreadKind kind);

    void SetParameters(bool *pbFinishedFlag,
		       int PInit, int PFin, int downsizeStep,
		       int dependType, int particleType,
		       unsigned char *pType, float *pValue1Dash,
		       float (*pX)[3], float (*pV)[3], int *pID);
    
    bool *mpbFinishedFlagLink;
    int mDependType, mParticleType; 
    unsigned char *mpType;
    float *mpValue1Dash;
    float (*mpX)[3], (*mpV)[3];
    int   *mpID;

    int   mPIni, mPFin;
    int   mDownsizeStep;

    void SetValue1Dash();
    
    virtual ExitCode Entry();
};

class SnapData_cls{
 public:
    SnapData_cls(PDataBlock_cls *pCDataBlockLink,
		 GLAttribute_cls **ppCGLAttribute,
		 double DT, int downsizeStep,
		 double doubleIndex, int frame);
    ~SnapData_cls();

    void Allocate(int pNum);
    void AllocateValue1Dash();
    void AllocateType();
    void AllocateAtrasIndex();
    void ReleaseData();
    bool *GetNewPCbFinishList(int &threadNumber, int pNum);
    
    bool IsAtrasNeeded(GLAttribute_cls **ppCGLAttribute);
    bool IsItSameSnap(PDataBlock_cls *pCDataBlockLink, double dT,
		      int downsizeStep, int frame);
    static bool IsInterpolateRequired(GLAttribute_cls **ppCGLAttribute);
    void SetPValue1(float **ppValue1, GLAttribute_cls *pCGLAttribut, 
		    int particleType);
    void InterpolateNot(PDataBlock_cls *pCDataBlockLink,  
			GLAttribute_cls **ppCGLAttribute);
    void Classify(GLAttribute_cls **ppCGLAttribute);
    void Interpolate(GLAttribute_cls **ppCGLAttribute,
		     double dT);
    void InterpolateColor(GLAttribute_cls **ppCGLAttribute);

    void SetValue1Dash(int dependType, int particleType);
    
    PDataBlock_cls *mpCDataBlockLink;

    bool mbAllocated;
    bool mbValue1dashAllocated;
    bool mbTypeAllocated;
    bool mbAtrasAllocated;
    int  mPNumBlock;
    double mTime;
    int    mDownsizeStep;
    double mCurrentDoubleIndex;
    int    mCurrentFrame;
    int   mOrder;
    
    float (*mpX)[3];
    float *mpRadius;
    float *mpValue1;
    float *mpValue1dash;
    float (*mpColor)[4];
    unsigned char *mpTypeSnapLocal;
    unsigned char *mpTypeLink;
    unsigned char *mpAtrasIndex;

};

#endif
