//(setq c-basic-offset 4)

#include "clsZSort.h"
#include "clsArrayAndIndex.h"

#include "../FILTER/clsFilterDataBox.h"
#include "../PANELS/clsPreference.h"
//#include "../SCENEOBJ/clsSceneObj.h"

#include <wx/wx.h>

#include <algorithm>

#include "../COMMON/CommonDefinitions.h"

extern bool GbSpecial;

ThreadSortPrepare_cls::
ThreadSortPrepare_cls(wxThreadKind kind) : wxThread(kind){
}

void ThreadSortPrepare_cls::
SetParameters(bool *pbFinishedFlag, 
	      ArrayAndIndex_cls *pCIndexAndZ, int PNum,
	      int step, float (*pX)[3], float cameraPos[3],
	      float cameraDir[3], int ini, int fin,
	      int downsizeIni, int type){
    mpbFinishedFlagLink = pbFinishedFlag;
    mpCIndexAndZ = pCIndexAndZ;
    mPNum = PNum;
    mStep = step;
    mpX = pX;
    mCameraPos = cameraPos;
    mCameraDir = cameraDir;
    mIni = ini; mFin = fin;
    mDownSizeIni = downsizeIni;
    mType = type;
    
    if ( Create() != wxTHREAD_NO_ERROR) 
	;//MsgBoxExit(NULL, "Error!", "Can't create thread");
}

wxThread::ExitCode ThreadSortPrepare_cls::
Entry(){
    const int PNum = mPNum;
    const int fin = mFin;
    const int ini = mIni;
    const int step = mStep;
    ArrayAndIndex_cls *pCIndexAndZ = mpCIndexAndZ;
    const float (*pX)[3] = mpX;
    const float *cameraPos = mCameraPos;
    const float *cameraDir = mCameraDir;
    
    int downsized = mDownSizeIni; //!!
    switch (mType){
    case 0:
	for (int i = ini; i < fin; i+=step){ 
	    pCIndexAndZ[downsized].index = i;
	    pCIndexAndZ[downsized].z = - 
		pX[i][X] * cameraDir[X] - 
		pX[i][Y] * cameraDir[Y] - 
		pX[i][Z] * cameraDir[Z];
	    downsized++;
	}
	break;
    case 1:
	for (int i = ini; i < fin; i+=step){ 
	    pCIndexAndZ[downsized].index = i;
	    pCIndexAndZ[downsized].z = - 
		(pX[i][X]-cameraPos[X]) * (pX[i][X]-cameraPos[X]) - 
		(pX[i][Y]-cameraPos[Y]) * (pX[i][Y]-cameraPos[Y]) - 
		(pX[i][Z]-cameraPos[Z]) * (pX[i][Z]-cameraPos[Z]);
	    downsized++;
	}
	break;
    }
    
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

//-----

ThreadSortParallel_cls::
ThreadSortParallel_cls(wxThreadKind kind) : wxThread(kind){
}

void ThreadSortParallel_cls::
SetParameters(bool *pbFinishedFlag,
	      ArrayAndIndex_cls *pCIndexAndZ, int ini, int fin){
    mpbFinishedFlagLink = pbFinishedFlag;
    mpCIndexAndZ = pCIndexAndZ;
    mIni = ini; mFin = fin;

    if ( Create() != wxTHREAD_NO_ERROR) 
	;//MsgBoxExit(NULL, "Error!", "Can't create thread");
}

wxThread::ExitCode ThreadSortParallel_cls::
Entry(){
    //const int fin = mFin;
    //const int ini = mIni;
    ArrayAndIndex_cls *pCIndexAndZ = mpCIndexAndZ;

    std::sort(pCIndexAndZ + mIni, pCIndexAndZ + mFin);
    
    *mpbFinishedFlagLink = TRUE;
    return 0;
}


//--------------

ThreadSortMergeOrd_cls::
ThreadSortMergeOrd_cls(wxThreadKind kind) : wxThread(kind){
}

void ThreadSortMergeOrd_cls::
SetParameters(bool *pbFinishedFlag,
	      ArrayAndIndex_cls *pCTgt, 
	      ArrayAndIndex_cls *pCSrc,
	      int cacheIni0, int cacheFin0,
	      int cacheIni1, int cacheFin1,
	      int block){
    
    mpbFinishedFlagLink = pbFinishedFlag;
    mpCTgt = pCTgt;
    mpCSrc = pCSrc;
    mCacheIni0 = cacheIni0;
    mCacheFin0 = cacheFin0;
    mCacheIni1 = cacheIni1;
    mCacheFin1 = cacheFin1;
    //mSize  = cacheFin1 - cacheIni0;
    mBlock = block;

    if ( Create() != wxTHREAD_NO_ERROR) 
	;//MsgBoxExit(NULL, "Error!", "Can't create thread");
}

wxThread::ExitCode ThreadSortMergeOrd_cls::
Entry(){
    ArrayAndIndex_cls *pCTgt = mpCTgt;
    ArrayAndIndex_cls *pCSrc = mpCSrc;

    int cacheIni0 = mCacheIni0; int cacheFin0 = mCacheFin0;
    int cacheIni1 = mCacheIni1; int cacheFin1 = mCacheFin1;
    
    bool bFinished0 = FALSE;
    bool bFinished1 = FALSE;

    int current0 = cacheIni0;
    int current1 = cacheIni1;

    int block = mBlock;
    
    int index = cacheIni0 + 0;
    int indexTarget = cacheFin0;
    const int copyBlock = sizeof(ArrayAndIndex_cls) * block;
    
TAG_REMERGE:

    if (block == 1){
	while (TRUE){
	    if (bFinished1 == TRUE || (pCSrc[current0].z < pCSrc[current1].z && bFinished0 == FALSE)){
		//memcpy(pCTgt + index, pCSrc + current0, sizeof(ArrayAndIndex_cls));
		pCTgt[index] = pCSrc[current0];
		current0++;
		if (current0 == cacheFin0) 
		    bFinished0 = TRUE;
	    }
	    else{
		//memcpy(pCTgt + index, pCSrc + current1, sizeof(ArrayAndIndex_cls));
		pCTgt[index] = pCSrc[current1];
		current1++;
		if (current1 == cacheFin1) 
		    bFinished1 = TRUE;
	    }

	    index++;
	    if (index == indexTarget)
		break;
	}
    }
    else{
	while (TRUE){
	    int remain = indexTarget - (index + block);
	    if (index + 2*block >= indexTarget) {
		block = 1; goto TAG_REMERGE;
	    }
	    
	    if (pCSrc[current0].z < pCSrc[current1].z && bFinished0 == FALSE){

		if (current0 + 2*block >= cacheFin0){
		    block = 1; goto TAG_REMERGE;
		}
		
		int maxcand = current1 + remain;
		if (maxcand >= cacheFin1) maxcand = cacheFin1 - 1;
		
		if (pCSrc[current0 + block - 1] > pCSrc[maxcand]){
		    block = 1; goto TAG_REMERGE;
		}
		
		memcpy(pCTgt + index, pCSrc + current0, sizeof(ArrayAndIndex_cls) * block);
		current0 += block;
		index    += block;
		
		if (current0 == cacheFin0) bFinished0 = TRUE;
	    }
	    else{
		if (current1 + block >= cacheFin1) {
		    block = 1; goto TAG_REMERGE;
		}
		
		int maxcand = current0 + remain;
		if (maxcand >= cacheFin0) maxcand = cacheFin0 - 1;
		
		if (pCSrc[current1 + block - 1] > pCSrc[maxcand]){
		    block = 1; goto TAG_REMERGE;
		}
		
		memcpy(pCTgt + index, pCSrc + current1, sizeof(ArrayAndIndex_cls) * block);
		current1 += block;
		index    += block;
		
		if (current1 == cacheFin1) bFinished1 = TRUE;
	    }
	    
	    if (index == indexTarget)
		break;

	    if (bFinished0 == TRUE && bFinished1 == TRUE) exit(0);
	}
    }

    *mpbFinishedFlagLink = TRUE;
    return 0;
}

//--------------

ThreadSortMergeInv_cls::
ThreadSortMergeInv_cls(wxThreadKind kind) : wxThread(kind){
}

void ThreadSortMergeInv_cls::
SetParameters(bool *pbFinishedFlag,
	      ArrayAndIndex_cls *pCTgt, 
	      ArrayAndIndex_cls *pCSrc,
	      int cacheIni0, int cacheFin0,
	      int cacheIni1, int cacheFin1,
	      int block){
    
    mpbFinishedFlagLink = pbFinishedFlag;
    mpCTgt = pCTgt;
    mpCSrc = pCSrc;
    mCacheIni0 = cacheIni0; mCacheFin0 = cacheFin0;
    mCacheIni1 = cacheIni1; mCacheFin1 = cacheFin1;
    //mSize  = cacheFin1 - cacheIni0;
    mBlock = block;
    
    if ( Create() != wxTHREAD_NO_ERROR) 
	;//MsgBoxExit(NULL, "Error!", "Can't create thread");
}

wxThread::ExitCode ThreadSortMergeInv_cls::
Entry(){
    ArrayAndIndex_cls *pCTgt = mpCTgt;
    ArrayAndIndex_cls *pCSrc = mpCSrc;
    
    int cacheIni0 = mCacheIni0; int cacheFin0 = mCacheFin0;
    int cacheIni1 = mCacheIni1; int cacheFin1 = mCacheFin1;
    
    bool bFinished0 = FALSE;
    bool bFinished1 = FALSE;
    int current0 = cacheFin0 - 1;
    int current1 = cacheFin1 - 1;

    int block = mBlock;
    
    block = 1;

    int candidate = 0;
    int index = cacheFin1 - 1;
    int indexTarget = cacheFin0;
    const int copyBlock = sizeof(ArrayAndIndex_cls) * block;

TAG_REMERGE:

    if (block == 1){
	while (TRUE){
	    if (bFinished1 == TRUE || (pCSrc[current0].z >= pCSrc[current1].z && bFinished0 == FALSE)){
		//memcpy(pCTgt + index, pCSrc + current0, sizeof(ArrayAndIndex_cls));
		pCTgt[index] = pCSrc[current0];
		current0--;
		if (current0 == cacheIni0 - 1)
		    bFinished0 = TRUE;
	    }
	    else{
		//memcpy(pCTgt + index, pCSrc + current1, sizeof(ArrayAndIndex_cls));
		pCTgt[index] = pCSrc[current1];
		current1--;
		if (current1 == cacheIni1 - 1)
		    bFinished1 = TRUE;
	    }
	    
	    index--;
	    if (index == indexTarget - 1)
		break;
	}
    }
    else{
	while (TRUE){
	    int remain = (index - block) - indexTarget + 1;
	    if (index - 2*block < indexTarget){
		block = 1; goto TAG_REMERGE;
	    }
	    
	    if (pCSrc[current0].z >= pCSrc[current1].z && bFinished0 == FALSE){
		
		if (current0 - 2*block < cacheIni0){
		    block = 1; goto TAG_REMERGE;
		}

		int mincand = current1 - remain;
		if (mincand < cacheIni1) mincand = cacheIni1;

		if (pCSrc[current0 - block + 1] > pCSrc[mincand]){
		    block = 1; goto TAG_REMERGE;
		}

		memcpy(pCTgt + (index + 1 - block), 
		       pCSrc + (current0 + 1 - block), sizeof(ArrayAndIndex_cls) * block);
		current0 -= block;	    
		index    -= block;

		if (current0 == cacheIni0 - 1) bFinished0 = TRUE;

	    }
	    else{
		
		if (current1 - 2*block < cacheIni0){
		    block = 1; goto TAG_REMERGE;
		}

		int mincand = current0 - remain;
		if (mincand < cacheIni0) mincand = cacheIni0;

		if (pCSrc[current1 - block + 1] > pCSrc[mincand]){
		    block = 1; goto TAG_REMERGE;
		}

		memcpy(pCTgt + (index + 1 - block), 
		       pCSrc + (current1 + 1 - block), sizeof(ArrayAndIndex_cls) * block);
		current1 -= block;	    
		index    -= block;
		
		if (current1 == cacheIni1 - 1) bFinished1 = TRUE;

	    }

	    if (index == indexTarget - 1) break;

	    if (bFinished0 == TRUE && bFinished1 == TRUE) exit(0);
	}
    }
    
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

//--------------

int ZSort_cls::
ZSortPrepare(ArrayAndIndex_cls *pCIndexAndZ, int PNum, 
	     int step, float (*pX)[3], float cameraPos[3],
	     float cameraDir[3], int type){

    float cameraDirCpy[3];
    LOOP_XYZ cameraDirCpy[xyz] = cameraDir[xyz];
    float cameraPosCpy[3];
    LOOP_XYZ cameraPosCpy[xyz] = cameraPos[xyz];

    bool bFinished[4] = {FALSE, FALSE, FALSE, FALSE};
    ThreadSortPrepare_cls *pCThread[4];
    for (int thread = 0; thread < 4; thread++){
	int ini = thread * PNum / 4;
	int fin = (thread + 1) * PNum / 4;
	ini = ini - ini % step;
	fin = fin - fin % step;
	int downsizeIni = ini / step;
	pCThread[thread] = new ThreadSortPrepare_cls(wxTHREAD_DETACHED);
	pCThread[thread]->SetParameters(&bFinished[thread], pCIndexAndZ, PNum,
					step, pX, cameraPosCpy, cameraDirCpy,
					ini, fin, downsizeIni, type);
    }
    
    pCThread[0]->Run();    pCThread[1]->Run();
    pCThread[2]->Run();    pCThread[3]->Run();

    for (int thread = 0; thread < 4; thread++)
	while (bFinished[thread] == FALSE) wxMilliSleep(5);

    pCIndexAndZ[PNum/step].index = 0;
    pCIndexAndZ[PNum/step].z = 9999;
    
    return PNum/step;
}

void ZSort_cls::
ZSortParallel(ArrayAndIndex_cls *pCIndexAndZ, int size){

    bool bFinished[4] = {FALSE, FALSE, FALSE, FALSE};

    ThreadSortParallel_cls *pCThread[4];

    int cacheIni[4];
    int cacheFin[4];

    for (int thread = 0; thread < 4; thread++){
	cacheIni[thread] = thread * size / 4;
	cacheFin[thread] = (thread + 1) * size / 4;
	pCThread[thread] = new ThreadSortParallel_cls(wxTHREAD_DETACHED);
	pCThread[thread]->SetParameters
	    (&bFinished[thread], pCIndexAndZ, 
	     cacheIni[thread], cacheFin[thread]);
    }

    for (int thread = 0; thread < 4; thread++) 
	pCThread[thread]->Run();

    for (int thread = 0; thread < 4; thread++)
	while (bFinished[thread] == FALSE) wxMilliSleep(5);

    if (GbSpecial == TRUE){
	FILE *fp = fopen("logA.txt", "wt");
	for (int i = 0; i < size; i++)
	    fprintf(fp, "%d %d %f\n", i, pCIndexAndZ[i].index, pCIndexAndZ[i].z);
	fclose(fp);
    }

    // }[W
    ArrayAndIndex_cls *pCCache = new ArrayAndIndex_cls [size];

    int block = 1;
    switch (mspCPreferenceLink->mCSortType.mBody){
    case 0:
	block = 1; break;
    case 1:
	block = 10; break;
    case 2:
	block = 100; break;
    case 3:
	block = 1000; break;
    }

    for (int i = 0; i < 4; i++){ bFinished[i] = FALSE; }

    ThreadSortMergeOrd_cls *pCThreadAOrd = new ThreadSortMergeOrd_cls(wxTHREAD_DETACHED);
    pCThreadAOrd->SetParameters(&bFinished[0], pCCache, pCIndexAndZ,
				cacheIni[0], cacheFin[0],
				cacheIni[1], cacheFin[1], block);
    
    ThreadSortMergeInv_cls *pCThreadAInv = new ThreadSortMergeInv_cls(wxTHREAD_DETACHED);
    pCThreadAInv->SetParameters(&bFinished[1],  pCCache, pCIndexAndZ,
				cacheIni[0], cacheFin[0],
				cacheIni[1], cacheFin[1], block);

    ThreadSortMergeOrd_cls *pCThreadBOrd = new ThreadSortMergeOrd_cls(wxTHREAD_DETACHED);
    pCThreadBOrd->SetParameters(&bFinished[2], pCCache, pCIndexAndZ, 
				cacheIni[2], cacheFin[2],
				cacheIni[3], cacheFin[3], block);
    
    ThreadSortMergeInv_cls *pCThreadBInv = new ThreadSortMergeInv_cls(wxTHREAD_DETACHED);
    pCThreadBInv->SetParameters(&bFinished[3], pCCache, pCIndexAndZ,
				cacheIni[2], cacheFin[2],
				cacheIni[3], cacheFin[3], block);
    pCThreadAOrd->Run();
    pCThreadAInv->Run();
    pCThreadBOrd->Run();
    pCThreadBInv->Run();

    for (int thread = 0; thread < 4; thread++)
	while (bFinished[thread] == FALSE) wxMilliSleep(5);

    //memcpy(pCCache + cacheIni[0], pCIndexAndZ + cacheIni[0], sizeof(ArrayAndIndex_cls)*(cacheFin[1] - cacheIni[0]));

    //memcpy(pCIndexAndZ, pCCache, sizeof(ArrayAndIndex_cls)*size);

    if (GbSpecial == TRUE){
	FILE *fp = fopen("logB.txt", "wt");
	for (int i = 0; i < size; i++)
	    fprintf(fp, "%d %d %f\n", i, pCIndexAndZ[i].index, pCIndexAndZ[i].z);
	fclose(fp);

	int overlap = 0;
	bool check[200000];
	for (int i = 0; i < size; i++)
	    check[i] = FALSE;
	for (int i = 0; i < size; i++){
	    int index = pCIndexAndZ[i].index;
	    if (check[index] == TRUE){
		overlap++;
	    }
	    check[index] = TRUE;
	}
	int num = 0;
	for (int i = 0; i < size; i++)
	    if (check[i] == TRUE) num++;
	
	num = num * 1;

	GbSpecial = FALSE;
    }
    
    for (int thread = 0; thread < 2; thread++) bFinished[thread] = FALSE;
    ThreadSortMergeOrd_cls *pCThreadCOrd = new ThreadSortMergeOrd_cls(wxTHREAD_DETACHED);
    pCThreadCOrd->SetParameters(&bFinished[0], pCIndexAndZ, pCCache, 
				cacheIni[0], cacheFin[1],
				cacheIni[2], cacheFin[3], block);

    ThreadSortMergeInv_cls *pCThreadCInv = new ThreadSortMergeInv_cls(wxTHREAD_DETACHED);
    pCThreadCInv->SetParameters(&bFinished[1], pCIndexAndZ, pCCache, 
				cacheIni[0], cacheFin[1],
				cacheIni[2], cacheFin[3], block);
    
    pCThreadCOrd->Run();
    pCThreadCInv->Run();

    for (int thread = 0; thread < 2; thread++)
	while (bFinished[thread] == FALSE) wxMilliSleep(5);

    /*
    bool check[5000];
    for (int i = 0; i < 5000; i++)
	check[i] = FALSE;
    for (int i = 0; i < size; i++){
	int index = pCIndexAndZ[i].index;
	check[index] = TRUE;
    }
    int num = 0;
    for (int i = 0; i < 5000; i++)
	if (check[i] == TRUE) num++;

    if (num != size){
	printf("!!\n");
    }
    */
    
    delete [] pCCache;
}

/*
void ZSort_cls::
SetSceneCoordinateInfo(FilterDataBoxData_cls *pCScene, double doubleIndex){
    mspCSceneLink = pCScene;
    msDoubleIndex = doubleIndex;
}
*/

Preference_cls *ZSort_cls::mspCPreferenceLink = NULL;
//FilterDataBoxData_cls *ZSort_cls::mspCSceneLink;
//double        ZSort_cls::msDoubleIndex;
