// "clsPovObjectMesh.h"
// (setq c-basic-offset 4)

#include "clsPovObjectMesh.h"
#include "clsPovThreadWriter.h"

#include "../clsPovRay.h"
#include "../clsPovSerialize.h"

#include "../../GLATTRIB/clsGLAttribute.h"
#include "../../GLATTRIB/clsShaderCtrl.h"
#include "../../MYAPI/MsgBox.h"
#include "../../../wxWidgetString.h"

#include "../../COMMON/MyVector.h"

#include "../../COMMON/CommonDefinitions.h"
#include "../../COMMON/CommonDefinitionsGL.h"

#include <wx/mstream.h>
#include <wx/txtstrm.h>
#include <wx/filename.h>

ThreadReadMesh2Sub::
ThreadReadMesh2Sub(char *pBuffer, int fileSize, int posStart,
		   int itemNum, int skipLine, std::vector < float > *pVData,
		   bool *pbFinishedFlagLink) :
    wxThread(wxTHREAD_DETACHED)
{
    mpbFinishedFlagLink = pbFinishedFlagLink;
    mpBuffer = pBuffer;
    mFileSize = fileSize;
    mPosStart = posStart;
    mItemNum = itemNum;
    mSkipLine = skipLine;
    mpVData = pVData;
    
    if ( Create() != wxTHREAD_NO_ERROR)
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

wxThread::ExitCode ThreadReadMesh2Sub::
Entry(){
    char *ptr = mpBuffer + mPosStart;
    
    int itemNum = mItemNum;
    float vectorX = 0, vectorY = 0, vectorZ = 0;
    
    std::vector <float> *pVData = mpVData;
    int item = 0;
    while (TRUE){
	while (*ptr != '<'){
	    ptr++;
	    if (ptr >= mpBuffer + mFileSize){
		ptr = NULL;
		break;
	    }
	}
	//ptr = strchr(ptr, '<');
	if (ptr == NULL) break;
	
	ptr++;
	vectorX = ReadFloat(&ptr, ',');
	ptr++;
	vectorY = ReadFloat(&ptr, ',');
	ptr++;
	vectorZ = ReadFloat(&ptr, '>');
	
	pVData->push_back(vectorX);
	pVData->push_back(vectorY);
	pVData->push_back(vectorZ);
	item++;

	if (item >= itemNum) break;
    }

    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ----

PovObjectMesh_cls::
PovObjectMesh_cls() : mCMesh(this){
    mCalledMaterialType = -1;
}

void PovObjectMesh_cls::
ClearMesh(){
    mCWFMaterial.Clear();
    mCMesh.ClearAll();
    
}

int PovObjectMesh_cls::
GetGLPolygon(int poly){
    if (poly == 4) return GL_QUADS;
    return GL_TRIANGLES;
}

void PovObjectMesh_cls::
FlushPolyList(std::vector < int > &VMP, int &polyMem, int &seqNum){
    if (seqNum >= 16){
	VMP.push_back(_MESH_POLYGONS_ARRAY);
	VMP.push_back(GetGLPolygon(polyMem));
	VMP.push_back(seqNum);
    }
    else
	for (int j = 0; j < seqNum; j++)
	    VMP.push_back(polyMem);
    seqNum = 0;
    polyMem = 0;
}

// VertexArrayg邩ǂ
void PovObjectMesh_cls::
CheckGeometry(){
    // vertexArrayg߂ɂ́A
    // FaceIndexVertex == FaceIndexNormal == FaceIndexUV2 łȂƂȂ
    // lKvB
    if (mCMesh.mVFaceIndexUV2.size() != 0) return;
    if (mCMesh.mVFaceIndexNormal.size() != 0) return;

    std::vector < int > &VMP = mCMesh.mVMaterialAndPolygon;
    std::vector < int > VOrg; VOrg.swap(VMP);
    
    int size = VOrg.size();
    int polyMem = 0;
    int seqNum = 0;
    std::vector < int > VCache;
    for (int i = 0; i < size; i++){
	int poly = VOrg[i];
	if (poly == _MESH_MATERIAL){
	    FlushPolyList(VMP, polyMem, seqNum);
	    VMP.push_back(poly);
	    i++; VMP.push_back(VOrg[i]);
	    continue;
	}
	if (poly == _MESH_POLYGONS_ARRAY){
	    FlushPolyList(VMP, polyMem, seqNum);
	    VMP.push_back(poly);
	    i++; VMP.push_back(VOrg[i]);
	    i++; VMP.push_back(VOrg[i]);
	    continue;
	}

	if (poly == polyMem){
	    seqNum++;
	}
	else{
	    FlushPolyList(VMP, polyMem, seqNum);
	}

	if (seqNum == 0){
	    polyMem = poly;
	    seqNum = 1;
	}
    }

    // ŌɎgĂȂLbV
    FlushPolyList(VMP, polyMem, seqNum);
    /*
    if (seqNum >= 16){
	VMP.push_back(_MESH_POLYGONS_ARRAY);
	VMP.push_back(GetGLPolygon(polyMem));
	VMP.push_back(seqNum);
    }
    else
	for (int j = 0; j < seqNum; j++)
	    VMP.push_back(polyMem);
    */

    /*
    // materialgȂ
    if (mbMaterialExist == TRUE) return;

    int polygonNum = mCMesh.mVMaterialAndPolygon.size();
    if (polygonNum == 0) return;
    for (int i = 1; i < polygonNum; i++){
	if (mCMesh.mVMaterialAndPolygon[i] != mCMesh.mVMaterialAndPolygon[i-1]) return;
    }
    */
}

int PovObjectMesh_cls::
GetFaceNum(){
    int face = 0;
    int count = (int)mCMesh.mVMaterialAndPolygon.size();
    for (int i = 0; i < count; i++){
	if (mCMesh.mVMaterialAndPolygon[i] >= 3) face++;
    }
    return face;
}

int PovObjectMesh_cls::
GetFaceNumTriangle(){
    int face = 0;
    int count = (int)mCMesh.mVMaterialAndPolygon.size();
    for (int i = 0; i < count; i++){
	if (mCMesh.mVMaterialAndPolygon[i] == _MESH_POLYGONS_ARRAY){
	    i++; int polyType = mCMesh.mVMaterialAndPolygon[i];
	    i++; int faceNum = mCMesh.mVMaterialAndPolygon[i];
	    int polyNum = 3;
	    if (polyType == GL_QUADS) polyNum = 4;
	    face += faceNum * (polyNum - 2);
	    continue;
	}
	if (mCMesh.mVMaterialAndPolygon[i] == _MESH_MATERIAL){
	    i++; continue;
	}
	if (mCMesh.mVMaterialAndPolygon[i] >= 3) 
	    face += mCMesh.mVMaterialAndPolygon[i] - 2;
    }
    return face;
}

void PovObjectMesh_cls::
ConstructNormalVectors(){
    mCMesh.mVNormal.resize(mCMesh.mVVertex.size());
    
    int normalSize = (int)mCMesh.mVNormal.size(); //_*3()
    for (int i = 0; i < normalSize; i++) mCMesh.mVNormal[i] = 0;
    int normalNum = normalSize / 3; // _
    int faceNum = GetFaceNum();
    
    // _͕̖ʂQƂ̂ŁA
    // flat shading ͒_Ɠ Normal  \Ȃ
    // smooth shading ɂȂB ̂悤Ȓ_ Normal ͕ωB
    std::vector < unsigned char > VOverlap(normalNum);
    
    int indexVertBase = 0;
    int size = (int)mCMesh.mVMaterialAndPolygon.size();
    for (int i = 0; i < size; i++){
	int ngon = mCMesh.mVMaterialAndPolygon[i];
	if (ngon == _MESH_MATERIAL){
	    i++; continue;
	}
	if (ngon == _MESH_POLYGONS_ARRAY){
	    // ̎_œZ߂ꂽ|S݂͑ĂȂ͂B
	    continue;
	}

	// ŏ3_normal߁An>3 gon̎ɂg
	int index0 = mCMesh.mVFaceIndexVertex[indexVertBase + 0];
	int index1 = mCMesh.mVFaceIndexVertex[indexVertBase + 1];
	int index2 = mCMesh.mVFaceIndexVertex[indexVertBase + 2];

	float *p0 = &mCMesh.mVVertex[index0*3];
	float *p1 = &mCMesh.mVVertex[index1*3];
	float *p2 = &mCMesh.mVVertex[index2*3];

	double normal[3];
	normal[X] = (p1[Y]-p0[Y])*(p2[Z]-p1[Z]) - (p1[Z]-p0[Z])*(p2[Y]-p1[Y]);
	normal[Y] = (p1[Z]-p0[Z])*(p2[X]-p1[X]) - (p1[X]-p0[X])*(p2[Z]-p1[Z]);
	normal[Z] = (p1[X]-p0[X])*(p2[Y]-p1[Y]) - (p1[Y]-p0[Y])*(p2[X]-p1[X]);
	
	if (MyVector::GetNorm(normal) == 0){
	    normal[X] = 0;
	    normal[Y] = 0;
	    normal[Z] = 1;
	}
	else
	    MyVector::Normalize(normal, 1);

	int overlap;
	for (int i = 0; i < ngon; i++){
	    int index = mCMesh.mVFaceIndexVertex[indexVertBase + i];
	    VOverlap[index]++;
	    overlap = VOverlap[index];
	    LOOP_XYZ mCMesh.mVNormal[index*3 + xyz] =
		(mCMesh.mVNormal[index*3 + xyz] * (overlap - 1) + normal[xyz]) / overlap;
	}
	indexVertBase += ngon;
    }

    for (int i = 0; i < normalNum; i++){
	if (MyVector::GetNorm(&mCMesh.mVNormal[i*3]) == 0){
	    mCMesh.mVNormal[i*3+X] = 0;
	    mCMesh.mVNormal[i*3+Y] = 0;
	    mCMesh.mVNormal[i*3+Z] = 1;
	}
	MyVector::Normalize(&mCMesh.mVNormal[i*3], 1);
    }
}


bool PovObjectMesh_cls::
ReadObjFile(wxString strFile){
     
    if (mstrLoaded == strFile) return FALSE;

    char *pBuffer = GetMemoryBufferFromFile(strFile);
    
    if (pBuffer == NULL){
	mstrLoaded = strFile;
	ClearMesh();
	return FALSE;
    }
    
    char *ptr = pBuffer;
    char *pEnd = pBuffer + mFileSize;
    
    bool bSmooth;
 
    ClearMesh();
    mCMesh.mVMaterialAndPolygon.reserve(mFileSize / 3 / 7);
    mCMesh.ReserveVertexNumber(mFileSize/10);
    mCMesh.ReserveFaceNumber(mFileSize/7);

    std::vector < float > &VVertex = mCMesh.mVVertex;
    std::vector < float > &VNormal = mCMesh.mVNormal;
    std::vector < float > &VUV2    = mCMesh.mVUV2;
    std::vector < int >   &VMaterialAndPolygon = mCMesh.mVMaterialAndPolygon;
    std::vector < unsigned int > &VFaceIndexVertex = mCMesh.mVFaceIndexVertex;
    std::vector < unsigned int > &VFaceIndexNormal = mCMesh.mVFaceIndexNormal;
    std::vector < unsigned int > &VFaceIndexUV2 = mCMesh.mVFaceIndexUV2;

    bool bMaterialExist = FALSE; 
	
    while (TRUE){
	if (ptr >= pEnd) break;	
	if (*ptr == '#'){
	    if (SearchChar(&ptr, '\n', pEnd) == FALSE) break;
	}
	if (*ptr == 'm'){ // mtllib
	    if (SearchChar(&ptr, ' ', pEnd) == FALSE) break;
	    ptr++;

	    char strMtlFile[256];
	    GetStrUptoLineEnd(&ptr, strMtlFile, pEnd);

	    wxFileName CSplit(strFile);
	    mCWFMaterial.ReadMtllib(CSplit.GetPath(), 
				    wxString::FromAscii(strMtlFile));
	    bMaterialExist = TRUE;
	}

	if (*ptr == 'u'){ //usemtl
	    if (SearchChar(&ptr, ' ', pEnd) == FALSE) break;
	    ptr++;

	    char strMtlTag[256];
	    GetStrUptoLineEnd(&ptr, strMtlTag, pEnd);
	    	    
	    int mtlIndex = mCWFMaterial.
		GetIndexFromTag(wxString::FromAscii(strMtlTag));
	    VMaterialAndPolygon.push_back(_MESH_MATERIAL);
	    VMaterialAndPolygon.push_back(mtlIndex);
	}
	
	if (*ptr == 'g'){
	    ptr++;
	    if (SearchChar(&ptr, '\n', pEnd) == FALSE) break;
	}
	if (*ptr == 'o'){
	    ptr++;
	    if (SearchChar(&ptr, '\n', pEnd) == FALSE) break;
	}
	if (*ptr == 'v'){
	    if (*(ptr+1) == 'n'){
		ptr++; ptr++; while ((*ptr) == ' ') ptr++;
		VNormal.push_back(ReadFloat(&ptr, ' ')); ptr++;
		VNormal.push_back(ReadFloat(&ptr, ' ')); ptr++;
		VNormal.push_back(ReadFloat(&ptr, '\n'));
	    }
	    if (*(ptr+1) == 't'){
		ptr++; ptr++; while ((*ptr) == ' ') ptr++;
		VUV2.push_back(ReadFloat(&ptr, ' ')); ptr++;
		VUV2.push_back(ReadFloat(&ptr, '\n'));
	    }
	    if (*(ptr+1) == ' '){
		ptr++; while ((*ptr) == ' ') ptr++;
		VVertex.push_back(ReadFloat(&ptr, ' ')); ptr++;
		VVertex.push_back(ReadFloat(&ptr, ' ')); ptr++;
		VVertex.push_back(ReadFloat(&ptr, '\n'));
	    }
	}
	if (*ptr == 's'){
	    ptr++; ptr++;
	    bSmooth = ReadBool(&ptr, '\n');
	}
	if (*ptr == 'f'){
	    ptr++; ptr++;
	    if (IsCharExist(ptr, ' ', '/') == TRUE){
		int ngon = 0;
		while(TRUE){
		    if (*ptr == ' ') ptr++;
		    if (*ptr == '\r') ptr++;
		    if (*ptr == '\n') break;
		    ngon++;
		    VFaceIndexVertex.push_back(ReadInt(&ptr, '/') - 1);
		    ptr++;
		    if (*ptr != '/'){
			VFaceIndexUV2.push_back(ReadInt(&ptr, '/') - 1);
			if (*ptr == '/') ptr++;
		    }
		    else
			ptr++;
		    if (*ptr != ' ' && *ptr != '\n')
			VFaceIndexNormal.push_back(ReadInt(&ptr, ' ') - 1);
		    if (*ptr == '\n') break;
		    if (ptr > pEnd) break;
		    ptr++;
		}
		VMaterialAndPolygon.push_back(ngon);
	    }
	    else{
		int ngon = 0;
		while (TRUE){
		    if (*ptr == ' ') ptr++;
		    if (*ptr == '\r') ptr++;
		    if (*ptr == '\n') break;
		    ngon++;
		    int index = ReadInt(&ptr, ' ') - 1;
		    VFaceIndexVertex.push_back(index);
		    //VFaceIndexUV2.push_back(index);
		    //VFaceIndexNormal.push_back(index);
		    if (*ptr == '\n') break;
		    if (ptr > pEnd) break;
		    ptr++;
		}
		mCMesh.mVMaterialAndPolygon.push_back(ngon);
	    }
	}
	ptr++;
    }

    delete pBuffer;
    mstrLoaded = strFile;

    // @xNg
    if (mCMesh.mVNormal.size() == 0) ConstructNormalVectors();

    mbMaterialExist = bMaterialExist;
    // }eA
    if (bMaterialExist == FALSE) mCWFMaterial.MakeMaterialIfEmpty();

    return TRUE;
}

bool PovObjectMesh_cls::
ReadMesh2File(wxString strFile){
    
    if (mstrLoaded == strFile) return FALSE;
    
    char *pBuffer = GetMemoryBufferFromFile(strFile);
    if (pBuffer == NULL){
	ClearMesh();
	mstrLoaded.Clear();
	return FALSE;
    }
    
    // ɈU߂قfgcB
    wxMemoryInputStream CMStream(pBuffer, mFileSize);
	
    wxString sep = wxT(" \t\n=,{}");
    wxTextInputStream CTStream(CMStream, sep);

    ClearMesh();

    mCMesh.mVMaterialAndPolygon.reserve(mFileSize / 3 / 7);
    mCMesh.ReserveVertexNumber(mFileSize/10);
    mCMesh.ReserveFaceNumber(mFileSize/7);

    int posNormal = -1, posIndex = -1;

    wxString strWord;
    strWord = CTStream.ReadWord();
    while (!strWord.Matches(L"#declare")){
	if (strWord.Matches(L"//normal_start_at"))
	    CTStream >> posNormal;
	if (strWord.Matches(L"//index_start_at"))
	    CTStream >> posIndex;
	strWord = CTStream.ReadWord();
    }
    mstrPovLabel = CTStream.ReadWord();
    while (!strWord.Matches(L"vertex_vectors")){
	strWord = CTStream.ReadWord();
    }
    int vertexNum;
    CTStream >> vertexNum;
    mCMesh.mVVertex.reserve(vertexNum * 3);

    // Verticex ǂݍ݂ʃXbhŊJn
    bool bFinishedVertex = FALSE;
    ThreadReadMesh2Sub *pCThreadVertices = new ThreadReadMesh2Sub
	(pBuffer, mFileSize, CMStream.TellI(), vertexNum,
	 1, &mCMesh.mVVertex, &bFinishedVertex);
    pCThreadVertices->Run();
    
    // m[}ʒum肵ĂȂΉsłǂĂ
    if (posNormal == -1){
	CTStream.ReadLine(); //dummy
	for (int i = 0; i < vertexNum; i++)
	    wxString strLine = CTStream.ReadLine();
    }
    else
	CMStream.SeekI(posNormal);
    
    int normalNum = -1;
    strWord = CTStream.ReadWord();
    printf ("%ls\n", strWord.c_str());
    if (strWord.Matches(L"normal_vectors")){
	CTStream >> normalNum;
	mCMesh.mVNormal.reserve(normalNum * 3);
    }

    bool bFinishedNormal = FALSE;
    ThreadReadMesh2Sub *pCThreadNormal;
    if (normalNum != -1){
	pCThreadNormal= new ThreadReadMesh2Sub
	    (pBuffer, mFileSize, CMStream.TellI(), normalNum,
	     1, &mCMesh.mVNormal, &bFinishedNormal);
	pCThreadNormal->Run();
    }
    
    // Indexʒum肵ĂȂΉsłǂĂ
    if (normalNum != -1){
	if (posIndex == -1){
	    CTStream.ReadLine(); //dummy
	    for (int i = 0; i < vertexNum; i++)
		wxString strLine = CTStream.ReadLine();
	}
	else
	    CMStream.SeekI(posIndex);
	
	// normal΁AstrWord͐Vɓǂ܂Ɉێ
	strWord = CTStream.ReadWord();
    }
    
    if (strWord.Matches(L"face_indices")){
	int faceNum;
	CTStream >> faceNum;
	mCMesh.mVFaceIndexVertex.reserve(faceNum * 3);

	int item = 0;
	char *ptr = pBuffer + CMStream.TellI();
	while(TRUE){
	    while (*ptr != '<'){
		ptr++;
		if (ptr >= pBuffer + mFileSize){
		    ptr = NULL;
		    break;
		}
	    }
	    if (ptr == NULL) break;
	    
	    ptr++; mCMesh.mVFaceIndexVertex.push_back(ReadInt(&ptr, ','));
	    ptr++; mCMesh.mVFaceIndexVertex.push_back(ReadInt(&ptr, ','));
	    ptr++; mCMesh.mVFaceIndexVertex.push_back(ReadInt(&ptr, '>'));
	    item++;
	    mCMesh.mVMaterialAndPolygon.push_back(3);
	    
	    if (item >= faceNum) break;
	}
    }

    while (TRUE){
	if (bFinishedVertex == TRUE) break;
	wxMilliSleep(3);
    }

    if (normalNum != -1) {
	while (TRUE){
	    if (bFinishedNormal == TRUE) break;
	    wxMilliSleep(3);
	}
    }

    delete [] pBuffer;
    mstrLoaded = strFile;

    // }eA
    mbMaterialExist = FALSE;
    mCWFMaterial.MakeMaterialIfEmpty();

    return TRUE;
}

void PovObjectMesh_cls::
ReadMeshFromFile(wxString strFile){
    wxFileName CSplit(strFile);
    wxString strExt = CSplit.GetExt();

    bool bNewData = FALSE;
    if (strExt.Matches(wxT("pov")) || strExt.Matches(wxT("POV")))
	bNewData |= ReadMesh2File(strFile);
    if (strExt.Matches(wxT("obj")) || strExt.Matches(wxT("OBJ")))
	bNewData |= ReadObjFile(strFile);

    if (bNewData == TRUE){
	CheckGeometry(); // vertex array \ȃf[^`FbN
	mCMesh.MinimizeMemoryUse();
    }
}

void PovObjectMesh_cls::
CallMaterial(int type){
    if (mCalledMaterialType != type){
	mCWFMaterial.GLSetMaterial(type, mbUseMaterialColor);
	mCalledMaterialType = type;
    }
}

void PovObjectMesh_cls::
GLDraw(bool bUseMaterialColor){
    if (mCMesh.mVVertex.size() == 0) return;

    mCalledMaterialType = -1;
    mbUseMaterialColor = bUseMaterialColor;

    bool bUseNornal = TRUE;
    if (mCMesh.mVNormal.size() == 0) bUseNornal = FALSE;
    bool bUseTexture = TRUE;
    if (mCMesh.mVUV2.size() == 0) bUseTexture = FALSE;

    if (bUseTexture == TRUE){
	glEnable(GL_TEXTURE_2D);
	GLAttribute_cls::mspCShaderCtrl->SetTextureState(_TEXTURE_ENABLED);
	glBindTexture(GL_TEXTURE_2D, 0);
    }
    else{
	glDisable(GL_TEXTURE_2D);
	GLAttribute_cls::mspCShaderCtrl->SetTextureState(_TEXTURE_DISABLED);
    }
    
    mCMesh.GLDraw(bUseNornal, bUseTexture);
}

void PovObjectMesh_cls::
GetStrPovMesh2(wxString& strPov, bool bUseMaterial, float clr[3][4],
	       wxString strFinishOption, wxString strNormal){

    if (bUseMaterial == TRUE)
	mCWFMaterial.GetStrPovTextureDeclare(strPov, mstrObjectLabel, NULL, strFinishOption, strNormal);
    else
	mCWFMaterial.GetStrPovTextureDeclare(strPov, mstrObjectLabel, &clr[0][0], strFinishOption, strNormal);

    strPov << wxT("#declare ") << mstrObjectLabel << wxT("=mesh2{") << ENDL;
    
    bool bFinishVertices = FALSE;
    char *pBufVertices = new char [mCMesh.mVVertex.size()*12];
    ThreadWriterVectors3_cls *pCWriterVertice = new ThreadWriterVectors3_cls;
    pCWriterVertice->SetParameters
	(&bFinishVertices, pBufVertices, (int)mCMesh.mVVertex.size(), 
	 mCMesh.mVVertex.data(), 6, 3);
    pCWriterVertice->Run();
    
    bool bFinishNormals = FALSE;
    char *pBufNormals = new char [mCMesh.mVNormal.size()*9];
    ThreadWriterVectors3_cls *pCWriterNormals = new ThreadWriterVectors3_cls;
    pCWriterNormals->SetParameters
	(&bFinishNormals, pBufNormals, (int)mCMesh.mVNormal.size(),
	 mCMesh.mVNormal.data(), 3, 4);
    pCWriterNormals->Run();
    
    bool bFinishUV2 = FALSE;
    char *pBufUV2 = new char [mCMesh.mVUV2.size()*11+1];
    ThreadWriterVectors2_cls *pCWriterUV2 = new ThreadWriterVectors2_cls;
    pCWriterUV2->SetParameters
	(&bFinishUV2, pBufUV2, (int)mCMesh.mVUV2.size(),
	 mCMesh.mVUV2.data(), 6, 5);
    pCWriterUV2->Run();
    
    int triangleNum = GetFaceNumTriangle();

    bool bFinishFaces = FALSE;
    char *pBufFaces = new char [triangleNum*32];
    ThreadWriterFaces_cls *pCWriterFace = new ThreadWriterFaces_cls;
    pCWriterFace->SetParameters
	(&bFinishFaces, pBufFaces, (int)mCMesh.mVMaterialAndPolygon.size(),
	 mCMesh.mVMaterialAndPolygon.data(), mCMesh.mVFaceIndexVertex.data(),
	 (int)mCWFMaterial.mVCItems.size(), 5);
    pCWriterFace->Run();
    
    int  faceNormalBuffer = triangleNum*32;
    if (mCMesh.mVFaceIndexNormal.size() == 0) faceNormalBuffer = 1;
    bool bFinishFaceNormal = FALSE;
    char *pBufFaceNormal = new char [faceNormalBuffer];
    if (mCMesh.mVFaceIndexNormal.size() != 0){
	ThreadWriterFaces_cls *pCWriterFaceNormal = new ThreadWriterFaces_cls;
	pCWriterFaceNormal->SetParameters
	    (&bFinishFaceNormal, pBufFaceNormal, (int)mCMesh.mVMaterialAndPolygon.size(),
	     mCMesh.mVMaterialAndPolygon.data(), mCMesh.mVFaceIndexNormal.data(),
	     1, 5);
	pCWriterFaceNormal->Run();
    }
    else
	bFinishFaceNormal = TRUE;

    int  faceUV2Buffer = triangleNum*32;
    if (mCMesh.mVFaceIndexUV2.size() == 0) faceUV2Buffer = 1;
    bool bFinishFaceUV2 = FALSE;
    char *pBufFaceUV2 = new char [faceUV2Buffer];
    if (mCMesh.mVFaceIndexUV2.size() != 0){
	ThreadWriterFaces_cls *pCWriterFaceUV2 = new ThreadWriterFaces_cls;
	pCWriterFaceUV2->SetParameters
	    (&bFinishFaceUV2, pBufFaceUV2, (int)mCMesh.mVMaterialAndPolygon.size(),
	     mCMesh.mVMaterialAndPolygon.data(), mCMesh.mVFaceIndexUV2.data(),
	     1, 5);
	pCWriterFaceUV2->Run();
    }
    else
	bFinishFaceUV2 = TRUE;

    while (bFinishVertices == FALSE)   wxMilliSleep(3);
    while (bFinishNormals == FALSE)    wxMilliSleep(3);
    while (bFinishUV2 == FALSE)        wxMilliSleep(3);
    while (bFinishFaces == FALSE)      wxMilliSleep(3);
    while (bFinishFaceNormal == FALSE) wxMilliSleep(3);
    while (bFinishFaceUV2 == FALSE)    wxMilliSleep(3);
    
    strPov << GetTab(1) << wxT("vertex_vectors{") << ENDL
	   << GetTab(2) << mCMesh.mVVertex.size() / 3 << wxT(",") << ENDL;
    strPov.Append(wxString::FromAscii(pBufVertices));
    strPov << wxT("\tnormal_vectors{") << ENDL 
	   << wxT("\t\t") << mCMesh.mVNormal.size() / 3 << wxT(",") << ENDL;
    strPov.Append(wxString::FromAscii(pBufNormals));

    if (mCMesh.mVUV2.size() != 0){
	strPov << GetTab(1) << wxT("uv_vectors{")  << ENDL
	       << GetTab(2) << mCMesh.mVUV2.size() / 2 << wxT(",") << ENDL;
	strPov.Append(wxString::FromAscii(pBufUV2));
    }

    mCWFMaterial.GetStrPovTextureList(strPov, mstrObjectLabel);

    strPov << GetTab(1) << wxT("face_indices{") << ENDL
	   << GetTab(2) << triangleNum << wxT(",") << ENDL;
    strPov.Append(wxString::FromAscii(pBufFaces));

    if (mCMesh.mVFaceIndexNormal.size() != 0){
	strPov << GetTab(1) << wxT("normal_indices{") << ENDL
	       << GetTab(2) << triangleNum << wxT(",") << ENDL;
	strPov.Append(wxString::FromAscii(pBufFaceNormal));
    }

    if (mCMesh.mVFaceIndexUV2.size() != 0){
	strPov << GetTab(1) << wxT("uv_indices{") << ENDL
	       << GetTab(2) << triangleNum << wxT(",") << ENDL;
	strPov.Append(wxString::FromAscii(pBufFaceUV2));
    }

    delete [] pBufVertices;
    delete [] pBufNormals;
    delete [] pBufUV2;
    delete [] pBufFaces;
    delete [] pBufFaceNormal;
    delete [] pBufFaceUV2;
    
    mCWFMaterial.GetStrPovTextureSingle(strPov, mstrObjectLabel);
    
    strPov << wxT("};") << ENDL;
}

void PovObjectMesh_cls::
GetStrPovInclude(wxString &strPov, wxString strLabel, bool bUseMaterial, float clr[3][4], wxString strFinishOption, wxString strNormal){
    mstrObjectLabel = wxT("MESH_");
    mstrObjectLabel << strLabel;
    
    bool bPov = FALSE;
    wxFileName CSplit(mstrLoaded);
    wxString strExt = CSplit.GetExt();
    if (strExt.Matches(wxT("pov")) || strExt.Matches(wxT("POV")))
	bPov = TRUE;

    if (bPov == TRUE){ // .povȂ
	if (mstrLoaded.IsEmpty() == FALSE){
	    mCWFMaterial.GetStrPovTextureDeclare(strPov, mstrObjectLabel, &clr[0][0], strFinishOption, strNormal);
	    strPov << wxT("#include \"") 
		   << mspCPovRayLink->GetStrRelativePath(mstrLoaded)
		   << wxT("\"") << ENDL;
	    strPov << wxT("#declare ") << mstrObjectLabel << wxT("=object{") << ENDL 
		   << GetTab(1) << mstrPovLabel << ENDL;
	    mCWFMaterial.GetStrPovTextureSingle(strPov, mstrObjectLabel); 
	    strPov << wxT("};") << ENDL;
	}
    }
    else if (mCMesh.mVVertex.size() == 0)
	strPov << wxT("#declare ") << mstrObjectLabel << wxT("=union{};") << ENDL;
    else
	GetStrPovMesh2(strPov, bUseMaterial, clr, strFinishOption, strNormal);
}

PovRayCtrl_cls* PovObjectMesh_cls::mspCPovRayLink = NULL;
