// "clsPovObjectStick.h"
// (setq c-basic-offset 4)

#include "clsPovObjectSticks.h"

#include "../clsPovRay.h"
#include "../../COMMON/MyVector.h"
#include "../../COMMON/clsFilename.h"
#include "../../FILTER/clsFilterStick.h"
#include "../../GLATTRIB/clsGLAttribute.h"
#include "../../GLATTRIB/clsColorCurve.h"
#include "../../GLATTRIB/clsShaderCtrl.h"
#include "../../PARTICLEDATA/clsPeriodicBoundary.h"
#include "../../PARTICLEDATA/clsSnapData.h"
#include "../../PARTICLEDATA/clsGLDrawBillboard.h"
#include "../../PARTICLEDATA/clsArrayAndIndex.h"

#include "../../MYAPI/MsgBox.h"
#include "../../../wxWidgetString.h"

#include "../../COMMON/CommonDefinitions.h"
#include "../../COMMON/CommonDefinitionsGL.h"

#include <string.h>

#include <wx/mstream.h>
#include <wx/txtstrm.h>

ThreadStickString_cls::
ThreadStickString_cls(bool *pbFinishedFlagLink,
		      char *pBuf, int init, int fin, int loop,
		      int *pVListA, int *pVListB, 
		      char *pVBoundaryType, float *pVFactor,
		      bool bConstancFactor,
		      float basedVector[8][3],  float (*pPos)[3]) :
    wxThread(wxTHREAD_DETACHED)
{
    mpbFinishedFlagLink = pbFinishedFlagLink;
    mpBuf = pBuf;
    mInit = init;
    mFin  = fin;
    mLoop = loop;
    mpVListA = pVListA;
    mpVListB = pVListB;
    mpVBoundaryType = pVBoundaryType;
    mpVFactor    = pVFactor;
    mbConstantFactor = bConstancFactor;
    mpBaseVector = basedVector;
    mpPos = pPos;
    if ( Create() != wxTHREAD_NO_ERROR)
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

bool ThreadStickString_cls::
GetCoordString(char strCoordA[64], char strCoordB[64],
	       float pPosA[3], float pPosB[3]){
    sprintf(strCoordA, "%.3f, %.3f, %.3f", pPosA[0], pPosA[1], pPosA[2]);
    sprintf(strCoordB, "%.3f, %.3f, %.3f", pPosB[0], pPosB[1], pPosB[2]);
    if (strcmp(strCoordA, strCoordB) == 0)
	return FALSE;
    return TRUE;
}

wxThread::ExitCode ThreadStickString_cls::
Entry(){
    char strCoordA[64], strCoordB[64];

    if (mbConstantFactor == TRUE)
	for (int i = mInit; i < mFin; i ++){
	    int boundaryType = mpVBoundaryType[i];
	    float posB[3];
	    if (mLoop == boundaryType){
		LOOP_XYZ posB[xyz] = mpPos[mpVListB[i]][xyz] + mpBaseVector[boundaryType][xyz];
		float *pPosA = mpPos[mpVListA[i]];

		if (GetCoordString(strCoordA, strCoordB, pPosA, posB) == TRUE){
		    int charnum = 0;
		    charnum = 
			sprintf(mpBuf, "CY(<%s>,<%s>)\n", strCoordA, strCoordB);
		    mpBuf += charnum;
		}
	    }
	}
    else
	for (int i = mInit; i < mFin; i ++){
	    int boundaryType = mpVBoundaryType[i];
	    float posB[3];
	    if (mLoop == boundaryType){
		LOOP_XYZ posB[xyz] = mpPos[mpVListB[i]][xyz] + mpBaseVector[boundaryType][xyz];
		float *pPosA = mpPos[mpVListA[i]];

		if (GetCoordString(strCoordA, strCoordB, pPosA, posB) == TRUE){
		    int charnum = 0;
		    charnum = 
			sprintf(mpBuf, "CY(<%s>,<%s>,%.3f)\n", strCoordA, strCoordB, mpVFactor[i]);
		    mpBuf += charnum;
		}
	    }
	}
    
    *mpbFinishedFlagLink = TRUE;
    return 0;
}
// ------

ThreadStickCalc_cls::
ThreadStickCalc_cls(bool *pbFinishedFlagLink,
		    std::vector < int > &VListA, 
		    std::vector < int > &VListB,
		    std::vector < char > &VBoundaryType,
		    std::vector < float > &VFactor) :
    mVListA(VListA), mVListB(VListB), mVBoundaryType(VBoundaryType), mVFactor(VFactor), wxThread(wxTHREAD_DETACHED){
    mpbFinishedFlagLink = pbFinishedFlagLink;
}

void ThreadStickCalc_cls::
SetParameter(float (*pX)[3], unsigned char *pValueType,
	     int init1, int fin1, int init2, int fin2, int num, 
	     int critType, int specType,
	     int specA, int specB,
	     float critA, float critB, float critA2, float critB2){
    mpX = pX;
    mpValueType = pValueType;
    mInit1 = init1; mInit2 = init2;
    mFin1 = fin1; mFin2 = fin2;
    mNum = num;
    mCritType = critType;
    mSpecType = specType;
    mSpecA = specA;
    mSpecB = specB;
    mCritA = critA;
    mCritB = critB;
    mCritA2 = critA2;
    mCritB2 = critB2;
    
    if ( Create() != wxTHREAD_NO_ERROR)
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadStickCalc_cls::
CheckStep(float (*pX)[3], int i, int j, float critA2){
    float distance2 = 
	(pX[i][X]-pX[j][X]) * (pX[i][X]-pX[j][X]) +
	(pX[i][Y]-pX[j][Y]) * (pX[i][Y]-pX[j][Y]) +
	(pX[i][Z]-pX[j][Z]) * (pX[i][Z]-pX[j][Z]);
    if (distance2 < critA2){
	mVListA.push_back(i);
	mVListB.push_back(j);
	mVBoundaryType.push_back(0);
	mVFactor.push_back(1);
    }
}

void ThreadStickCalc_cls::
CheckRamp(float (*pX)[3], int i, int j, float critA, float critB){
    if (critA == critB) return;

    float critA2 = critA * critA;
    float critB2 = critB * critB;
    float distance2 = 
	(pX[i][X]-pX[j][X]) * (pX[i][X]-pX[j][X]) +
	(pX[i][Y]-pX[j][Y]) * (pX[i][Y]-pX[j][Y]) +
	(pX[i][Z]-pX[j][Z]) * (pX[i][Z]-pX[j][Z]);
    float distance = sqrt(distance2);
    if (distance2 < critA2){
	mVListA.push_back(i);
	mVListB.push_back(j);
	mVBoundaryType.push_back(0);
	mVFactor.push_back(1);
    }
    else if (distance2 > critA2 && distance2 < critB2){
	mVListA.push_back(i);
	mVListB.push_back(j);
	mVBoundaryType.push_back(0);
	float factor = (critB - distance) / ( critB - critA);
	mVFactor.push_back(factor);
    }
}

void ThreadStickCalc_cls::
CheckStepBetween(float (*pX)[3], int i, int j, float critA2, float critB2){
    float distance2 = 
	(pX[i][X]-pX[j][X]) * (pX[i][X]-pX[j][X]) +
	(pX[i][Y]-pX[j][Y]) * (pX[i][Y]-pX[j][Y]) +
	(pX[i][Z]-pX[j][Z]) * (pX[i][Z]-pX[j][Z]);
    if (distance2 > critA2 && distance2 < critB2){
	mVListA.push_back(i);
	mVListB.push_back(j);
	mVBoundaryType.push_back(0);
	mVFactor.push_back(1);
    }
}

wxThread::ExitCode ThreadStickCalc_cls::
Entry(){
    float (*pX)[3] = mpX;
    unsigned char *pType = mpValueType;
    int init[2] = {mInit1, mInit2};
    int fin[2] = {mFin1, mFin2};
    int num = mNum;
    int specA = mSpecA;
    int specB = mSpecB;
    float critA = mCritA;
    float critB = mCritB;
    float critA2 = mCritA2;
    float critB2 = mCritB2;
    ArrayAndIndex_cls *pCArray = GLDrawBillboard_cls::mspIndexAndZ;

    for (int loop = 0; loop < 2; loop++){
	if (mCritType == 0){
	    switch(mSpecType){
	    case 0:
		for (int i_ = init[loop]; i_ < fin[loop]; i_++){
		    int i = pCArray[i_].index;
		    for (int j_ = i_ + 1; j_ < num; j_++){
			int j = pCArray[j_].index;
			CheckStep(pX, i, j, critA2);
		    }
		}
		break;
	    case 1:
		for (int i_ = init[loop]; i_ < fin[loop]; i_++){
		    int i = pCArray[i_].index;
		    for (int j_ = i_ + 1; j_ < num; j_++){
			int j = pCArray[j_].index;
			if (((pType[i] == specA && pType[j] == specB) ||
			     (pType[i] == specB && pType[j] == specA))
			    == FALSE) continue;
			CheckStep(pX, i, j, critA2);
		    }
		}
		break;
	    }
	}
	
	if (mCritType == 1){
	    if (mSpecType == 0)
		for (int i_ = init[loop]; i_ < fin[loop]; i_++){
		    int i = pCArray[i_].index;
		    for (int j_ = i_ + 1; j_ < num; j_++){
			int j = pCArray[j_].index;
			CheckRamp(pX, i, j, critA, critB);
		    }
		}
	    if (mSpecType == 1)
		for (int i_ = init[loop]; i_ < fin[loop]; i_++){
		    int i = pCArray[i_].index;
		    for (int j_ = i_ + 1; j_ < num; j_++){
			int j = pCArray[j_].index;
			if (((pType[i] == specA && pType[j] == specB) ||
			     (pType[i] == specB && pType[j] == specA))
			    == FALSE) continue;
			CheckRamp(pX, i, j, critA, critB);
		    }
		}
	}
	
	if (mCritType == 2){
	    if (mSpecType == 0)
		for (int i_ = init[loop]; i_ < fin[loop]; i_++){
		    int i = pCArray[i_].index;
		    for (int j_ = i_ + 1; j_ < num; j_++){
			int j = pCArray[j_].index;
			CheckStepBetween(pX, i, j, critA2, critB2);
		    }
		}
	    if (mSpecType == 1)
		for (int i_ = init[loop]; i_ < fin[loop]; i_++){
		    int i = pCArray[i_].index;
		    for (int j_ = i_ + 1; j_ < num; j_++){
			int j = pCArray[j_].index;
			if (((pType[i] == specA && pType[j] == specB) ||
			     (pType[i] == specB && pType[j] == specA))
			== FALSE) continue;
			CheckStepBetween(pX, i, j, critA2, critB2);
		    }
		}
	}
    }
    
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ------

PovObjectSticks_cls::
PovObjectSticks_cls(){
    mpCPeriodicLink = NULL;
    for (int j = 0; j <= 12; j++){
	float theta = - j * 30 * 3.14159 / 180;
	mCos[j] = cos(theta);
	mSin[j] = sin(theta);
    }
}

PovObjectSticks_cls::
~PovObjectSticks_cls(){
    if (mspIndexAndZ != NULL){
	delete [] mspIndexAndZ;
	mspIndexAndZ = NULL;
    }
}

void PovObjectSticks_cls::
ClearList(){
    // ̊mۗ̈悲Clear@(TCY0̈ꎞxNgƌ)
    std::vector<int>().swap(mVListA);
    std::vector<int>().swap(mVListB);
    std::vector<char>().swap(mVBoundaryType);
    std::vector<float>().swap(mVFactor);
}

bool PovObjectSticks_cls::
IsConstantFactor(){
    mbConstantFactor = TRUE;
    int size = mVFactor.size();
    for (unsigned int i = 0; i < mVFactor.size(); i++)
	if (mVFactor[i] != 1) mbConstantFactor = FALSE;
    return mbConstantFactor;
}

void PovObjectSticks_cls::
CheckValidity(SnapData_cls *pCSnapshot){
    if (pCSnapshot == NULL){
	mPNumAtChecked = 0;
	mbPNumValidity = FALSE;
	return;
    }

    int pnum = pCSnapshot->mPNumBlock;
    if (pnum == mPNumAtChecked) return;
    int num = mVListA.size();
    mbPNumValidity = TRUE;
    mPNumAtChecked = pnum;
    for (int i = 0; i < num; i++){
	if (mVListA[i] >= pnum || mVListB[i] >= pnum){
	    mbPNumValidity = FALSE; 
	    i = num; 
	}
    }
}

unsigned char* PovObjectSticks_cls::
GetMemoryBufferFromFile(wxString strFile){
    FILE *fp = fopen(strFile.ToAscii(), "rb"); //!!
    if (fp == NULL) {
	mstrLoaded = strFile; return NULL; }

    fseek( fp, 0, SEEK_END );
    mFileSize = ftell( fp );
    fseek( fp, 0, SEEK_SET);
    
    unsigned char *pBuffer = new unsigned char [mFileSize];
    fread( pBuffer, 1, mFileSize, fp);
    fclose(fp);
    return pBuffer;
}

void PovObjectSticks_cls::
ReadListFromBinary(wxString strFile){
    const unsigned char *pBuffer = GetMemoryBufferFromFile(strFile);
    
    if (pBuffer == NULL) return;

    if (strcmp((const char*)pBuffer, "bond") != 0) return;

    int type = *(int*)(pBuffer+5);
    int num  = *(int*)(pBuffer+9);

    if (mFileSize != num * 8 + 13) return;

    mVListA.resize(num);
    mVListB.resize(num);
    mVBoundaryType.resize(num);
    mVFactor.resize(num);

    for (int i = 0; i < num; i++){
	mVListA[i] = *(int*)(pBuffer+(13+i*8));
	mVListB[i] = *(int*)(pBuffer+(13+i*8+4));
	mVBoundaryType[i] = 0;
	mVFactor[i] = 1;
    }

    delete [] pBuffer;
    mstrLoaded = strFile;
}

void PovObjectSticks_cls::
ReadListFromFile(wxString strFormat, int currentIndex){
    
    wxString strFile = wxString::Format(strFormat, currentIndex);
    
    if (mstrLoaded == strFile) return;

    ClearList();

    SplitPath_cls CPath(strFile);
    if (CPath.GetPathStr(_FN_EXT).IsSameAs(wxT(".bond"), FALSE) == TRUE){
	ReadListFromBinary(strFile);
	return;
    }

    unsigned char *pBuffer = GetMemoryBufferFromFile(strFile);
    if (pBuffer == NULL) return;
    
    // ɈU߂قfgcB
    wxMemoryInputStream CMStream(pBuffer, mFileSize);

    wxString sep = wxT(" \t\n=,{}");
    wxTextInputStream CTStream(CMStream, sep);
    
    int indexA, indexB, type;
    float factor;
    while(TRUE){
	wxString strLine = CTStream.ReadLine();
	if (strLine.IsEmpty() == TRUE) break;
	wxWritableCharBuffer charBuffer = strLine.char_str();
	char *ptr = charBuffer;
	
	int scanNum = sscanf(ptr, "%d %d %d %f\n",
			     &indexA, &indexB, &type, &factor);
	if (scanNum == 2) type = 0;
	if (scanNum == 3) factor = 1;
	mVListA.push_back(indexA);
	mVListB.push_back(indexB);
	mVBoundaryType.push_back(type);
	mVFactor.push_back(factor);
    }
    
    delete [] pBuffer;
    mstrLoaded = strFile;
}

void PovObjectSticks_cls::
CalcFromData(SnapData_cls *pCSnapshot, int currentIndex,
	     int critType, float critA, float critB,
	     int specType, int specA, int specB){
    
    if (pCSnapshot == NULL) return;
    if (mCalcedTime == pCSnapshot->mTime &&
	mCalcedCritType == critType &&
	mCalcedCritA == critA && mCalcedCritB == critB &&
	mCalcedSpecType == specType &&
	mCalcedSpecA == specA && mCalcedSpecB == specB ) return;
    
    ClearList();
    
    float critA2 = critA * critA;
    float critB2 = critB * critB;
    float (*pX)[3] = pCSnapshot->mpX;
    unsigned char *pValueType = pCSnapshot->mpTypeLink;
    //int num = pCSnapshot->mPNumBlock;
    int num = GLDrawBillboard_cls::msDownsizedAtFBO0;

    std::vector < int > VListA[4];
    std::vector < int > VListB[4];
    std::vector < char > VBoundaryType[4];
    std::vector < float > VFactor[4];
    bool bFinished[4] = {FALSE, FALSE, FALSE, FALSE};
    ThreadStickCalc_cls *pCThread[4];
    for (int thread = 0; thread < 4; thread++){
	int ini1 = thread * num / 8;
	int fin1 = (thread + 1) * num / 8;
	int ini2 = (7 - thread) * num / 8;
	int fin2 = (8 - thread) * num / 8; 
	pCThread[thread] = new ThreadStickCalc_cls
	    (&bFinished[thread], VListA[thread], VListB[thread], 
	     VBoundaryType[thread], VFactor[thread]);
	pCThread[thread]->SetParameter
	    (pX, pValueType, ini1, fin1, ini2, fin2, num, critType, specType,
	     specA, specB, critA, critB, critA2, critB2);
    }

    pCThread[0]->Run(); pCThread[1]->Run();
    pCThread[2]->Run(); pCThread[3]->Run();
    
    for (int thread = 0; thread < 4; thread++)
	while (bFinished[thread] == FALSE)	wxMilliSleep(5);
    
    int sum = 0;
    for (int thread = 0; thread < 4; thread++)
	sum += VListA[thread].size();
    
    mVListA.resize(sum);
    mVListB.resize(sum);
    mVBoundaryType.resize(sum);
    mVFactor.resize(sum);

    sum = 0;
    for (int thread = 0; thread < 4; thread++){
	if (VListA[thread].size() != 0){
	    memcpy(&mVListA[sum], &VListA[thread][0], sizeof(int) * VListA[thread].size());
	    memcpy(&mVListB[sum], &VListB[thread][0], sizeof(int) * VListB[thread].size());
	    memcpy(&mVBoundaryType[sum], &VBoundaryType[thread][0], sizeof(char) * VBoundaryType[thread].size());
	    memcpy(&mVFactor[sum], &VFactor[thread][0], sizeof(float) * VFactor[thread].size());
	}
	sum += VListA[thread].size();
    }

    mCalcedIndex = currentIndex;
    mCalcedTime = pCSnapshot->mTime;
    mCalcedCritType = critType;
    mCalcedCritA = critA;
    mCalcedCritB = critB;
    mCalcedSpecType = specType;
    mCalcedSpecA = specA;
    mCalcedSpecB = specB;
}

void PovObjectSticks_cls::
SetPPeriodicBoundary(PeriodicBoundary_cls *pCPeriodic){
    mpCPeriodicLink = pCPeriodic;
    LOOP_XYZ mBaseVector[0][xyz] = 0;
    LOOP_XYZ mBaseVector[1][xyz] = mpCPeriodicLink->mCBaseVector[0].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[0];
    LOOP_XYZ mBaseVector[2][xyz] = mpCPeriodicLink->mCBaseVector[1].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[1];
    LOOP_XYZ mBaseVector[3][xyz] = 
	mpCPeriodicLink->mCBaseVector[0].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[0] +
	mpCPeriodicLink->mCBaseVector[1].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[1];
    LOOP_XYZ mBaseVector[4][xyz] = mpCPeriodicLink->mCBaseVector[2].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[2];
    LOOP_XYZ mBaseVector[5][xyz] = 
	mpCPeriodicLink->mCBaseVector[0].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[0] +
	mpCPeriodicLink->mCBaseVector[2].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[2];
    LOOP_XYZ mBaseVector[6][xyz] = 
	mpCPeriodicLink->mCBaseVector[1].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[1] +
	mpCPeriodicLink->mCBaseVector[2].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[2];
    LOOP_XYZ mBaseVector[7][xyz] = 
	mpCPeriodicLink->mCBaseVector[1].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[1] +
	mpCPeriodicLink->mCBaseVector[2].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[2] +
	mpCPeriodicLink->mCBaseVector[3].mpBody[xyz] * mpCPeriodicLink->mCSize.mpBody[3];
}

#include "../../PARTICLEDATA/clsZSort.h"
#include <algorithm>

void PovObjectSticks_cls::
GLDrawSort(SnapData_cls *pCSnapshot, int type, float alpha,
	   float radius, int width, int bitBoundary, 
	   PositionAndDirection_cls CCamPos,
	   std::vector < ColorCurve_cls* > &VpCColorCurve){

    int size = mVListA.size();

    ZSort_cls *pCZSort = new ZSort_cls;
    
    if (mspIndexAndZ != NULL){
	delete [] mspIndexAndZ;
    }
    mspIndexAndZ =  new ArrayAndIndex_cls [size + 1];
    
    float (*pX)[3] = pCSnapshot->mpX;
    float (*pBondX)[3] = new float [size][3];
    
    for (int i = 0; i < size; i++){
	int indexA = mVListA[i];
	int indexB = mVListB[i];
	pBondX[i][0] = (pX[indexA][0] + pX[indexA][0])/2;
	pBondX[i][1] = (pX[indexA][1] + pX[indexA][1])/2;
	pBondX[i][2] = (pX[indexA][2] + pX[indexA][2])/2;
    }
    
    float camPos[3], camDir[3];
    LOOP_XYZ camPos[xyz] = CCamPos.mPos[xyz];
    LOOP_XYZ camDir[xyz] = CCamPos.mTgt[xyz];
    pCZSort->ZSortPrepare(mspIndexAndZ, size, 1, pBondX, 
			  camPos, camDir, 0);
    
    if (size > 100000)
	pCZSort->ZSortParallel(mspIndexAndZ, size);
    else
	std::sort(mspIndexAndZ, mspIndexAndZ + size);
    
    delete pCZSort;
    delete [] pBondX;

    if (type == _STICK_LINE){
	glDisable(GL_LIGHTING);
	float (*pX)[3] = pCSnapshot->mpX;
	float (*pColor)[4] = pCSnapshot->mpColor;
	
	int widthOrg;
	glGetIntegerv(GL_LINE_WIDTH, &widthOrg);
	glLineWidth(width);
	glBegin(GL_LINES);
	for (int i_ = 0; i_ < size; i_++){
	    int i = mspIndexAndZ[i_].index;
	    glColor4f(0.7, 0.7, 0.7, alpha * mVFactor[i]);
	    int boundaryType = mVBoundaryType[i];
	    if (boundaryType == 0){
		glVertex3fv(pX[mVListA[i]]);
		glVertex3fv(pX[mVListB[i]]);
	    }
	    else{
		if ((bitBoundary & boundaryType) != boundaryType) continue;
		float posB[3];
		LOOP_XYZ posB[xyz] = pX[mVListB[i]][xyz] + 
		    mBaseVector[boundaryType][xyz];
		glVertex3fv(pX[mVListA[i]]);
		glVertex3fv(posB);
	    }
	}
	glEnd();
	glLineWidth(widthOrg);
    }

    if (type == _STICK_CYLINDER){
	float prevFactor = -1;
	float axis1[3];
	float axis3[3];
	glEnable(GL_LIGHTING);
	float (*pX)[3] = pCSnapshot->mpX;
	
	for (int i_ = 0; i_ < size; i_++){
	    int i = mspIndexAndZ[i_].index;
	    int boundaryType = mVBoundaryType[i];
	    float *posA = pX[mVListA[i]];
	    float posB[3];
	    if (boundaryType == 0){
		LOOP_XYZ
		    posB[xyz] = pX[mVListB[i]][xyz];
	    }
	    else{
		if ((bitBoundary & boundaryType) != boundaryType) continue;
		LOOP_XYZ 
		    posB[xyz] = pX[mVListB[i]][xyz] + 
		    mBaseVector[boundaryType][xyz];
	    }

	    float factor = mVFactor[i];
	    if (factor != prevFactor){

		float clrMaterial[3][4];
		VpCColorCurve[0]->GetColor4fNormal(clrMaterial[0], 0);
		VpCColorCurve[1]->GetColor4fNormal(clrMaterial[1], 0);
		VpCColorCurve[2]->GetColor4fNormal(clrMaterial[2], 0);

		clrMaterial[0][3] = alpha * factor;
		clrMaterial[1][3] = alpha * factor;
		clrMaterial[2][3] = alpha * factor;

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, clrMaterial[0]);
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, clrMaterial[1]);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, clrMaterial[2]);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 20);
		
		prevFactor = factor;
	    }

	    LOOP_XYZ axis1[xyz] = posB[xyz] - posA[xyz];
	    float axis2[3] = {0, 0, 1};
	    MyVector::Normalize(axis1, 1);
	    if (MyVector::Orthogonalize(axis1, axis2) == FALSE) continue;
	    MyVector::Normalize(axis2, 1);
	    MyVector::GetCrossedVector(axis1, axis2, axis3);

	    glBegin(GL_QUAD_STRIP);
	    for (int j = 0; j <= 12; j++){
		float cosf = mCos[j];
		float sinf = mSin[j];
		glNormal3f(cosf*axis2[X] + sinf*axis3[X],
			   cosf*axis2[Y] + sinf*axis3[Y],
			   cosf*axis2[Z] + sinf*axis3[Z]);
		glVertex3f(posA[X] + radius*(cosf*axis2[X] + sinf*axis3[X]),
			   posA[Y] + radius*(cosf*axis2[Y] + sinf*axis3[Y]),
			   posA[Z] + radius*(cosf*axis2[Z] + sinf*axis3[Z]));
		glNormal3f(cosf*axis2[X] + sinf*axis3[X],
			   cosf*axis2[Y] + sinf*axis3[Y],
			   cosf*axis2[Z] + sinf*axis3[Z]);
		glVertex3f(posB[X] + radius*(cosf*axis2[X] + sinf*axis3[X]),
			   posB[Y] + radius*(cosf*axis2[Y] + sinf*axis3[Y]),
			   posB[Z] + radius*(cosf*axis2[Z] + sinf*axis3[Z]));
	    }
	    glEnd();

	    glBegin(GL_QUAD_STRIP);
	    for (int j = 0; j <= 12; j++){
		float cosf = mCos[j];
		float sinf = mSin[j];
		glNormal3f(cosf*axis2[X] + sinf*axis3[X],
			   cosf*axis2[Y] + sinf*axis3[Y],
			   cosf*axis2[Z] + sinf*axis3[Z]);
		glVertex3f(posA[X] + radius*(cosf*axis2[X] + sinf*axis3[X]),
			   posA[Y] + radius*(cosf*axis2[Y] + sinf*axis3[Y]),
			   posA[Z] + radius*(cosf*axis2[Z] + sinf*axis3[Z]));
		glNormal3f(cosf*axis2[X] + sinf*axis3[X],
			   cosf*axis2[Y] + sinf*axis3[Y],
			   cosf*axis2[Z] + sinf*axis3[Z]);
		glVertex3f(posB[X] + radius*(cosf*axis2[X] + sinf*axis3[X]),
			   posB[Y] + radius*(cosf*axis2[Y] + sinf*axis3[Y]),
			   posB[Z] + radius*(cosf*axis2[Z] + sinf*axis3[Z]));
	    }
	    glEnd();

	}
    }
}

void PovObjectSticks_cls::
GLDraw(SnapData_cls *pCSnapshot, int type, float alpha,
       float radius, int width, int bitBoundary, 
       PositionAndDirection_cls CCamPos,
       std::vector < ColorCurve_cls* > &VpCColorCurve){

    if (mbPNumValidity == FALSE) return;

    /*
      GLAttribute_cls::mspCShaderCtrl->EndShader(0);
    */
    
    int size = mVListA.size();
    if (size == 0) return;

    glDisable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glEnable(GL_NORMALIZE);

    bool bSort = TRUE;
    if (bSort == TRUE){
	return GLDrawSort(pCSnapshot, type, alpha, radius, width, 
			  bitBoundary, CCamPos, VpCColorCurve);
    }
    
    if (type == _STICK_LINE){
	glDisable(GL_LIGHTING);
	float (*pX)[3] = pCSnapshot->mpX;
	float (*pColor)[4] = pCSnapshot->mpColor;
	
	int widthOrg;
	glGetIntegerv(GL_LINE_WIDTH, &widthOrg);
	glLineWidth(width);
	glBegin(GL_LINES);
	for (int i = 0; i < size; i++){
	    glColor4f(0.7, 0.7, 0.7, alpha * mVFactor[i]);
	    int boundaryType = mVBoundaryType[i];
	    if (boundaryType == 0){
		glVertex3fv(pX[mVListA[i]]);
		glVertex3fv(pX[mVListB[i]]);
	    }
	    else{
		if ((bitBoundary & boundaryType) != boundaryType) continue;
		float posB[3];
		LOOP_XYZ posB[xyz] = pX[mVListB[i]][xyz] + 
		    mBaseVector[boundaryType][xyz];
		glVertex3fv(pX[mVListA[i]]);
		glVertex3fv(posB);
	    }
	}
	glEnd();
	glLineWidth(widthOrg);
    }

    if (type == _STICK_CYLINDER){
	float prevFactor = -1;
	float axis1[3];
	float axis3[3];
	glEnable(GL_LIGHTING);
	float (*pX)[3] = pCSnapshot->mpX;
	
	for (int i = 0; i < size; i++){
	    
	    int boundaryType = mVBoundaryType[i];
	    float *posA = pX[mVListA[i]];
	    float posB[3];
	    if (boundaryType == 0){
		LOOP_XYZ
		    posB[xyz] = pX[mVListB[i]][xyz];
	    }
	    else{
		if ((bitBoundary & boundaryType) != boundaryType) continue;
		LOOP_XYZ 
		    posB[xyz] = pX[mVListB[i]][xyz] + 
		    mBaseVector[boundaryType][xyz];
	    }

	    float factor = mVFactor[i];
	    if (factor != prevFactor){
		float ambient[4] = {0.2, 0.16, 0.04, alpha * factor};
		float diffuse[4] = {0.1, 0.1, 0.1, alpha * factor};
		float specular[4] = {0.1, 0.1, 0.1, /*alpha **/ factor};
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
		prevFactor = factor;
	    }

	    LOOP_XYZ axis1[xyz] = posB[xyz] - posA[xyz];
	    float axis2[3] = {0, 0, 1};
	    MyVector::Normalize(axis1, 1);
	    if (MyVector::Orthogonalize(axis1, axis2) == FALSE) continue;
	    MyVector::Normalize(axis2, 1);
	    MyVector::GetCrossedVector(axis1, axis2, axis3);
	    glBegin(GL_QUAD_STRIP);
	    for (int j = 0; j <= 12; j++){
		float cosf = mCos[j];
		float sinf = mSin[j];
		glNormal3f(cosf*axis2[X] + sinf*axis3[X],
			   cosf*axis2[Y] + sinf*axis3[Y],
			   cosf*axis2[Z] + sinf*axis3[Z]);
		glVertex3f(posA[X] + radius*(cosf*axis2[X] + sinf*axis3[X]),
			   posA[Y] + radius*(cosf*axis2[Y] + sinf*axis3[Y]),
			   posA[Z] + radius*(cosf*axis2[Z] + sinf*axis3[Z]));
		glVertex3f(posB[X] + radius*(cosf*axis2[X] + sinf*axis3[X]),
			   posB[Y] + radius*(cosf*axis2[Y] + sinf*axis3[Y]),
			   posB[Z] + radius*(cosf*axis2[Z] + sinf*axis3[Z]));
	    }
	    glEnd();
	}
    }
}

void PovObjectSticks_cls::
GetStrPovSticksRef(wxString &strPData, SnapData_cls *pCSnapshot){
    
    CheckValidity(pCSnapshot);
    if (mbPNumValidity == FALSE){
	strPData << wxT("// PNumValidity Fails");
	return;
    }
    
    for (int loop = 0; loop < 8; loop++){
	strPData << wxT("#declare STICKS") << loop << wxT("=") << ENDL
		 << wxT("union{") << ENDL;

	int size = mVListA.size();

	char *pBuf[4];
	bool bFinished[4] = {FALSE, FALSE, FALSE, FALSE};
	for (int thread = 0; thread < 4; thread++){
	    pBuf[thread] = new char [(size + 4) * 160 / 4];
	    pBuf[thread][0] = '\0';
	}
	
	ThreadStickString_cls *pCThread[4];
	for (int thread = 0; thread < 4; thread++){
	    int ini = thread * size / 4;
	    int fin = (thread + 1) * size / 4;
	    pCThread[thread] = new ThreadStickString_cls
		(&bFinished[thread], pBuf[thread], ini, fin, loop, 
		 mVListA.data(), mVListB.data(),
		 mVBoundaryType.data(), mVFactor.data(),
		 mbConstantFactor, 
		 mBaseVector, pCSnapshot->mpX);
	}
	
	pCThread[0]->Run(); pCThread[1]->Run();
	pCThread[2]->Run(); pCThread[3]->Run();

	for (int thread = 0; thread < 4; thread++)
	    while (bFinished[thread] == FALSE)	wxMilliSleep(5);
	
	for (int thread = 0; thread < 4; thread++){
	    strPData.Append(wxString::FromAscii(pBuf[thread]));
	    delete [] pBuf[thread];
	}

	strPData << wxT("};") << ENDL;
    }
}

ArrayAndIndex_cls *PovObjectSticks_cls::mspIndexAndZ = NULL;
