// "clsPovThreadWriter.h"
// (c-basic-offset 4)

#include "clsPovThreadWriter.h"

#include "../../MESH/clsMeshBase.h"
#include "../../MYAPI/MsgBox.h"

#include "../../COMMON/CommonDefinitions.h"
#include "../../COMMON/CommonDefinitionsGL.h"

ThreadWriterBase_cls::
ThreadWriterBase_cls() : wxThread (wxTHREAD_DETACHED) {
    mItemIndex = 0;
    mItemNumber = 3;
}

void ThreadWriterBase_cls::
SerializeClose(char **ppBuf){
    if (mItemIndex % mItemNumber != 0){
	sprintf(*ppBuf, "\t"); (*ppBuf)++;
    }
    sprintf(*ppBuf, "\t}\n");
}

// ----------------


ThreadWriterVectors3_cls::
ThreadWriterVectors3_cls(){
};

void ThreadWriterVectors3_cls::
SetParameters(bool *pbFinishedFlag, char *pBuf,
	      int num, float *pVertices, int digit, int itemNumber){
    mpbFinishedFlagLink = pbFinishedFlag;
    mpBuf = pBuf;
    mNum = num;
    mpVertices = pVertices;
    mItemNumber = itemNumber;
    
    sprintf(mAFormat, "<%%.%df,%%.%df,%%.%df>,", digit, digit, digit);
    
    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadWriterVectors3_cls::
SerializeVertices(char **ppBuf, float *pVertices){
    if (mItemIndex % mItemNumber == 0){
	sprintf(*ppBuf, "\t"); (*ppBuf)++;
    }

    (*ppBuf) += sprintf(*ppBuf, mAFormat, 
			pVertices[0], pVertices[1], pVertices[2]); 

    if ((mItemIndex+1) % mItemNumber == 0){
	sprintf(*ppBuf, "\n"); (*ppBuf)++;
    }
    mItemIndex++;
}

void ThreadWriterVectors3_cls::
WriteToBuffer(){
    char *pBuf = mpBuf;
    int num = mNum;
    if (num == 0) return;
    
    float *pVertices = mpVertices;
    for (unsigned int i = 0; i < num; i+=3)
	SerializeVertices(&pBuf, pVertices+i);
    SerializeClose(&pBuf);
};

wxThread::ExitCode ThreadWriterVectors3_cls::
Entry(){
    WriteToBuffer();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ---------------------------

ThreadWriterVectors2_cls::
ThreadWriterVectors2_cls(){
};

void ThreadWriterVectors2_cls::
SetParameters(bool *pbFinishedFlag, char *pBuf,
	      int num, float *pVertices, int digit, int itemNumber){
    mpbFinishedFlagLink = pbFinishedFlag;
    mpBuf = pBuf;
    mNum = num;
    mpVertices = pVertices;
    mItemNumber = itemNumber;
    
    sprintf(mAFormat, "<%%.%df,%%.%df>,", digit, digit);

    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadWriterVectors2_cls::
SerializeVertices(char **ppBuf, float *pVertices){
    if (mItemIndex % mItemNumber == 0){
	sprintf(*ppBuf, "\t"); (*ppBuf)++;
    }

    (*ppBuf) += sprintf(*ppBuf, mAFormat, 
			pVertices[0], pVertices[1]); 
    if ((mItemIndex+1) % mItemNumber == 0){
	sprintf(*ppBuf, "\n"); (*ppBuf)++;
    }
    mItemIndex++;
}

void ThreadWriterVectors2_cls::
WriteToBuffer(){
    char *pBuf = mpBuf;
    int num = mNum;
    if (num == 0) return;
    
    float *pVertices = mpVertices;
    for (unsigned int i = 0; i < num; i+=2)
	SerializeVertices(&pBuf, pVertices+i);
    SerializeClose(&pBuf);
};

wxThread::ExitCode ThreadWriterVectors2_cls::
Entry(){
    WriteToBuffer();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// ------

ThreadWriterFaces_cls::
ThreadWriterFaces_cls(){
}

void ThreadWriterFaces_cls::
SetParameters(bool *pbFinishedFlag, char *pBuf, int num,
	      int *pMaterialAndPolygon, unsigned int *pVertexIndex, 
	      int materialNum, int itemNumber){
    mpbFinishedFlagLink = pbFinishedFlag;
    mpBuf = pBuf;
    mNum = num;

    mpMaterialAndPolygon = pMaterialAndPolygon;
    mpVertexIndex = pVertexIndex;
    mMaterialNum = materialNum;

    mItemNumber = itemNumber;
    
    if ( Create() != wxTHREAD_NO_ERROR) 
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

void ThreadWriterFaces_cls::
SerializeFaces(char **ppBuf, int vertex1, int vertex2, int vertex3,
	       int materialIndex){
    if (mItemIndex % mItemNumber == 0){
	sprintf(*ppBuf, "\t"); (*ppBuf)++;
    }

    (*ppBuf) += sprintf(*ppBuf, "<%d,%d,%d>,%d,",
			vertex1, vertex2, vertex3, materialIndex);
    if ((mItemIndex+1) % mItemNumber == 0){
	sprintf(*ppBuf, "\n"); (*ppBuf)++;
    }
    mItemIndex++;
}

void ThreadWriterFaces_cls::
SerializeFaces(char **ppBuf, int vertex1, int vertex2, int vertex3){
    if (mItemIndex % mItemNumber == 0){
	sprintf(*ppBuf, "\t"); (*ppBuf)++;
    }

    (*ppBuf) += sprintf(*ppBuf, "<%d,%d,%d>,",
			vertex1, vertex2, vertex3);
    if ((mItemIndex+1) % mItemNumber == 0){
	sprintf(*ppBuf, "\n"); (*ppBuf)++;
    }
    mItemIndex++;
}

void ThreadWriterFaces_cls::
WriteToBuffer(){
    char *pBuf = mpBuf;
    int num = mNum;
    if (num == 0) return;

    int *pMaterialAndPolygon = mpMaterialAndPolygon;
    unsigned int *pVertexIndex = mpVertexIndex;
    
    int materialIndex = 0;
    int index = 0;

    if (mMaterialNum > 1){
	for (unsigned int i = 0; i < num; i++){
	    int polySize = pMaterialAndPolygon[i];
	    if (polySize == _MESH_MATERIAL) {
		i++; materialIndex = pMaterialAndPolygon[i];
		continue;
	    }

	    int faceNum = 1;
	    if (polySize == _MESH_POLYGONS_ARRAY){
		i++; int polyType = pMaterialAndPolygon[i];
		if (polyType == GL_TRIANGLES) polySize = 3;
		if (polyType == GL_QUADS) polySize = 4;
		i++; faceNum = pMaterialAndPolygon[i];
	    }

	    for (int face = 0; face < faceNum; face++){
		for (int polyVertex = 1; polyVertex < polySize-1; polyVertex++){
		    SerializeFaces(&pBuf, 
				   pVertexIndex[index],
				   pVertexIndex[index+polyVertex],
				   pVertexIndex[index+polyVertex+1],
				   materialIndex);
		}
		index += polySize;
	    }
	}
	SerializeClose(&pBuf);
    }
    else{
	for (unsigned int i = 0; i < num; i++){
	    int polySize = pMaterialAndPolygon[i];
	    if (polySize == _MESH_MATERIAL) {
		i++; materialIndex = pMaterialAndPolygon[i];
		continue;
	    }

	    int faceNum = 1;
	    if (polySize == _MESH_POLYGONS_ARRAY){
		i++; int polyType = pMaterialAndPolygon[i];
		if (polyType == GL_TRIANGLES) polySize = 3;
		if (polyType == GL_QUADS) polySize = 4;
		i++; faceNum = pMaterialAndPolygon[i];
	    }

	    for (int face = 0; face < faceNum; face++){
		for (int polyVertex = 1; polyVertex < polySize-1; polyVertex++){
		    SerializeFaces(&pBuf, 
				   pVertexIndex[index],
				   pVertexIndex[index+polyVertex],
				   pVertexIndex[index+polyVertex+1]);
		}
		index += polySize;
	    }
	}
	SerializeClose(&pBuf);
    }
};

wxThread::ExitCode ThreadWriterFaces_cls::
Entry(){
    WriteToBuffer();
    *mpbFinishedFlagLink = TRUE;
    return 0;
}
