// "clsWavefrontMaterial.h"
// (c-basic-offset 4)

#include "clsWavefrontMaterial.h"

#include "../../COMMON/CommonDefinitionsGL.h"
#include "../../COMMON/CommonDefinitions.h"

#include "../../GLCANVAS/clsMainGLCanvas.h"

#include "../clsPovRay.h"
#include "../clsPovMaterialColor.h"

#include "../OPTIONAL/clsPovAdvanced.h"
#include "../OPTIONAL/clsPovFormat.h"

#include <string.h>

#ifdef WIN32
#define DELIMWX wxT("\\")
#else
#define DELIMWX wxT("/")
#endif

WavefrontMaterialItem_cls::
WavefrontMaterialItem_cls(wxString strName, wxString strDir, char **ptr, char *bufferEnd){
    mstrName = strName;

    LOOP_RGBA mDiffuse[rgba] = 0.5;
    LOOP_RGBA mAmbient[rgba] = 0.5;
    LOOP_RGBA mSpecular[rgba] = 0.5;

    if (strName.IsEmpty() == TRUE) return;
    
    mAlpha = 1;
    while (TRUE){
	if (*ptr >= bufferEnd) break;
	if (**ptr == 'n')
	    if (SearchTagStr(ptr, "newmtl", bufferEnd, FALSE) == TRUE){
		(*ptr)--; //
		break;
	    }
	if (**ptr == 'K'){
	    if (*((*ptr)+1) == 'd'){
		(*ptr)++;(*ptr)++; (*ptr)++; 
		mDiffuse[0] = ReadFloat(ptr, ' '); (*ptr)++;
		mDiffuse[1] = ReadFloat(ptr, ' '); (*ptr)++;
		mDiffuse[2] = ReadFloat(ptr, '\n');
	    }
	    if (*((*ptr)+1) == 'a'){
		(*ptr)++;(*ptr)++; (*ptr)++; 
		mAmbient[0] = ReadFloat(ptr, ' '); (*ptr)++;
		mAmbient[1] = ReadFloat(ptr, ' '); (*ptr)++;
		mAmbient[2] = ReadFloat(ptr, '\n');
	    }
	    if (*((*ptr)+1) == 's'){
		(*ptr)++;(*ptr)++; (*ptr)++; 
		mSpecular[0] = ReadFloat(ptr, ' '); (*ptr)++;
		mSpecular[1] = ReadFloat(ptr, ' '); (*ptr)++;
		mSpecular[2] = ReadFloat(ptr, '\n');
	    }
	}
	if (**ptr == 'd'){ 
	    (*ptr)++; (*ptr)++;
	    mAlpha = ReadFloat(ptr, '\n');
	}
	if (**ptr == 'T' && *((*ptr)+1) == 'r'){ 
	    (*ptr)++;(*ptr)++; (*ptr)++;
	    mAlpha = ReadFloat(ptr, '\n');
	}
	if (**ptr == 'T' && *((*ptr)+1) == 'f'){ 
	    (*ptr)++;(*ptr)++; (*ptr)++;
	    mTransmission[0] = ReadFloat(ptr, ' ');
	    mTransmission[1] = ReadFloat(ptr, ' ');
	    
	}
	if (**ptr == 'N' && *((*ptr)+1) == 'i'){ 
	    (*ptr)++;(*ptr)++;(*ptr)++;
	    mIOR = ReadFloat(ptr, '\n');
	}
	if (**ptr == 'm'){
	    if (SearchTagStr(ptr, "map_Kd", bufferEnd) ||
		SearchTagStr(ptr, "map_Ka", bufferEnd) ){
		(*ptr)++;
		char strMtlFile[512];
		GetStrUptoLineEnd(ptr, strMtlFile, bufferEnd);

		wxString strCandidA = 
		    strDir + wxT("textures") + DELIMWX + wxString::FromAscii(strMtlFile);
		wxString strCandidB = 
		    strDir + DELIMWX + wxString::FromAscii(strMtlFile);
		if (::wxFileExists(strCandidA) == TRUE)
		    mstrTexDiffuse = strCandidA;
		else if (::wxFileExists(strCandidB) == TRUE)
		    mstrTexDiffuse = strCandidB;
		else
		    mstrTexDiffuse = wxString::FromAscii(strMtlFile);
	    }
		
	}
	if (**ptr == 'i'){ // illum
	    mIllumMode = ReadInt(ptr, '\n');
	}
		
	(*ptr)++;
    }

    mAmbient[3] = mAlpha;
    mDiffuse[3] = mAlpha;
    mSpecular[3] = mAlpha;
}
void WavefrontMaterialItem_cls::
GLSetMaterial(bool bUseMaterialColor){
    if (mstrTexDiffuse.IsEmpty() == TRUE){
	//mCTexture.LoadImage(wxT("Resource/White.png"), TRUE);
	MainGLCanvas_cls::mspCWhite->BindTexture();
    }
    else{
	mCTexture.LoadImage(mstrTexDiffuse, TRUE);
	mCTexture.BindTexture();
    }

    if (bUseMaterialColor == FALSE) return;

    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mAmbient);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mDiffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mSpecular);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 20);
    
}

/*
void WavefrontMaterialItem_cls::
GetAlphaForMultiLayer2(float& alpha){
    // eNX`2d˂̏ꍇ
    float tr = 1.0-alpha;
    float trsq = sqrt(tr);
    alpha = 1.0 - trsq;
}
*/

void WavefrontMaterialItem_cls::
GetClrInfo(float clr0[3][4], float *clrOvwr){
    if (clrOvwr != NULL){
	LOOP_RGB clr0[0][rgb] = clrOvwr[rgb]  ; clr0[0][3] = clrOvwr[0+3];
	LOOP_RGB clr0[1][rgb] = clrOvwr[4+rgb]; clr0[1][3] = clrOvwr[4+3];
	LOOP_RGB clr0[2][rgb] = clrOvwr[8+rgb]; clr0[2][3] = clrOvwr[8+3];
    }
    else{
	LOOP_RGB clr0[0][rgb] = mAmbient[rgb] ; clr0[0][3] = mAmbient[3];
	LOOP_RGB clr0[1][rgb] = mDiffuse[rgb] ; clr0[1][3] = mDiffuse[3];
	LOOP_RGB clr0[2][rgb] = mSpecular[rgb]; clr0[2][3] = mSpecular[3];
    }
}

/*
void WavefrontMaterialItem_cls::
GetStrPovTextureDiffuse(wxString &strPov, wxString strPigment, float clr0[3][4], wxString strFinishOption, wxString strNormal){
    
    float clr[3][4];
    LOOP_RGBA clr[0][rgba] = 0;
    LOOP_RGB  clr[1][rgb ] = clr0[1][rgb] * 2; clr [1][3] = clr0[1][3];
    LOOP_RGBA clr[2][rgba] = 0;

    wxString strFinish;
    PovMaterialColor_cls::GetStrPovFinishIndividual(strFinish, GetTab(1), clr[0], 2, 0, 0, strFinishOption);
    strPov << PovMaterialColor_cls::GetStrPovTextureBase
	(wxES, strPigment, strFinish, strNormal);
    LOOP_RGB clr[1][rgb] /= 2;
    GetAlphaForMultiLayer2(clr[1][3]);
    PovMaterialColor_cls::GetStrPovTextureColorFilter(strPov, wxES, clr[1]);
}
*/

/*
void WavefrontMaterialItem_cls::
GetStrPovTextureAmbientSpecular(wxString &strPov, wxString strPigment, float clr0[3][4], wxString strFinishOption, wxString strNormal){
    float clr[3][4];
    LOOP_RGB clr[0][rgb] = clr0[0][rgb] * 2; clr[0][3] = clr0[0][3];
    LOOP_RGB clr[1][rgb] = clr0[2][rgb] * 2; clr[1][3] = clr0[2][3];
    LOOP_RGB clr[2][rgb] = clr0[2][rgb] * 2; clr[2][3] = clr0[2][3];
    LOOP_RGB if (clr[2][rgb] < 0.02){
	clr[2][rgb] = 0.02;
	clr[1][rgb] = 0.02;
    }
    
    wxString strFinish;
    GetAlphaForMultiLayer2(clr[2][3]);
    LOOP_RGB clr[0][rgb] /= (clr[2][rgb] / 2);
    LOOP_RGB clr[0][rgb] *= clr[2][3];
    PovMaterialColor_cls::GetStrPovFinishIndividual(strFinish, GetTab(1), clr[0], 0, 2 * clr[2][3], 0.156*0.156, strFinishOption);
    strPov << PovMaterialColor_cls::GetStrPovTextureBase(wxES, strPigment, strFinish, strNormal);
    LOOP_RGB clr[2][rgb] /= 2;
    PovMaterialColor_cls::GetStrPovTextureColorFilter(strPov, wxES, clr[2]);
}
*/

// ---------------------------

void WavefrontMaterialCtrl_cls::
Clear(){
    mVCItems.clear();
}

void WavefrontMaterialCtrl_cls::
ReadMtllib(wxString strDir, wxString strFile){
    
    mstrDirectory = strDir + DELIMWX;

    char *pBuffer = GetMemoryBufferFromFile(mstrDirectory + strFile);

    if (pBuffer == NULL) return;

    char *ptr = pBuffer;
    char *pEnd = pBuffer + mFileSize;

    while (TRUE){
	if (ptr >= pEnd) break;	
	
	if (*ptr == '#'){
	    if (SearchChar(&ptr, '\n', pEnd) == FALSE) return;
	}
	if (*ptr == 'n')
	    if (SearchTagStr(&ptr, "newmtl", pEnd) == TRUE){
		ptr++;
		char strItemName[512];
		GetStrUptoLineEnd(&ptr, strItemName, pEnd);
		
		mVCItems.push_back
		    (WavefrontMaterialItem_cls
		     (wxString::FromAscii(strItemName), mstrDirectory, 
		      &ptr, pEnd));
		/*
		char *pFileIni = ptr;
		if (SearchChar(&ptr, '\n', pEnd) == FALSE) return;
		char *pFileFin = ptr;
		memcpy(strMtlFile, pFileIni, pFileFin - pFileIni);
		strMtlFile[pFileFin - pFileIni] = '\0';
		*/
	    }
	ptr++;
    }
    
    delete pBuffer;
}

void WavefrontMaterialCtrl_cls::
MakeMaterialIfEmpty(){
    if (mVCItems.size() == 0)
	mVCItems.push_back(WavefrontMaterialItem_cls(wxES, wxES, NULL, NULL));
}

int WavefrontMaterialCtrl_cls::
GetIndexFromTag(wxString strTag){
    int size = (int)mVCItems.size();
    for (int i = 0; i < size; i++){
	int len0 = strTag.Len();
	int len1 = mVCItems[i].mstrName.Len();
	if (mVCItems[i].mstrName.IsSameAs(strTag) == TRUE)
	    return i;
    }
    return 0;
}

void WavefrontMaterialCtrl_cls::
GLSetMaterial(int index, bool bUseMaterialColor){
    if (index < -1 || index >= mVCItems.size()) return;
    if (index == -1){
	for (int i = 0; i < mVCItems.size(); i++)
	    mVCItems[i].GLSetMaterial(bUseMaterialColor);
	return;
    }
    mVCItems[index].GLSetMaterial(bUseMaterialColor);
}

void WavefrontMaterialCtrl_cls::
GetStrPovTextureSingle(wxString& strPov, wxString strLabel){
    if (mVCItems.size() != 1) return;

    strPov << GetTab(1) << wxT("texture { ") << strLabel << wxT("_Tex") << 0 << ENDLCLOSE;
}

void WavefrontMaterialCtrl_cls::
GetStrPovTextureList(wxString& strPov, wxString strLabel/*, float *clrOvwr*/){
    if (mVCItems.size() <= 1) return;
    
    strPov << GetTab(1) << wxT("texture_list {") << ENDL;
    strPov << GetTab(1) << mVCItems.size() << wxT(",") << ENDL;
    for (int i = 0; i < mVCItems.size(); i++)
	strPov << GetTab(1) << wxT("texture { ") << strLabel << wxT("_Tex") << i << ENDLCLOSE;
    //mVCItems[i].GetStrPovTexture(strPov, clrOvwr);
    strPov << GetTab(1) << ENDLCLOSE;
}

void WavefrontMaterialCtrl_cls::
GetStrPovTextureDeclare(wxString& strPov, wxString strLabel, float *clrOvwr,
			wxString strFinishOption, wxString strNormal){
    if (mVCItems.size() == 0) return;
       
    for (int i = 0; i < mVCItems.size(); i++){
		    
	float clr0[3][4]; // At@l͋ʂ
	mVCItems[i].GetClrInfo(clr0, clrOvwr);
    
	wxString strTex;
	strTex << strLabel << wxT("_Tex") << i;

	if (mVCItems[i].mstrTexDiffuse.IsEmpty() == TRUE){
	    GetStrPovTextureDeclareNoMap(strPov, strTex, clr0, strFinishOption, strNormal);
	    continue;
	}

	wxString strPigment;
	strPigment << strTex << wxT("_PIGMENT");

	float alpha = clr0[0][3]; 
	WavefrontMaterialItem_cls::GetAlphaForMultiLayer2(alpha);
	strPov << wxT("#declare ") << strPigment <<wxT("=pigment{");
	GetStrPovPigmentImageMap(strPov, mVCItems[i].mstrTexDiffuse, TRUE, 0, alpha, PovRayCtrl_cls::mspCThis->mpCAdvanced->mCTextureGamma.mBody);
	strPov << ENDLCLOSE;

	if (PovRayCtrl_cls::mspCThis->IsFastColor() == TRUE){
	    strPov << wxT("#declare ") << strTex << wxT("=") << ENDL;
	    GetStrPovTextureColor
		(strPov, wxES, TRUE, 
		 clr0, 0.156*0.156, strFinishOption, strNormal, strPigment, FALSE, FALSE);
	    GetAlphaForMultiLayer2(clr0[1][3]);
	    PovMaterialColor_cls::GetStrPovTextureColorFilter(strPov, wxES, clr0[1]);
	    strPov << wxT(";") << ENDL;
	    continue; 
	}
	
	strPov << wxT("#declare ") << strTex << wxT("_DIFFUSE=") << ENDL;
	GetStrPovTextureDiffuse(strPov, strPigment, clr0, strFinishOption, strNormal);
	strPov << wxT(";") << ENDL;
	strPov << wxT("#declare ") << strTex << wxT("_AMBIENT_SPECULAR=") << ENDL;
	GetStrPovTextureAmbientSpecular(strPov, strPigment, clr0, strFinishOption, strNormal);
	strPov << wxT(";") << ENDL;

	strPov << wxT("#declare ") << strTex << wxT("=") << ENDL;
	strPov << wxT("texture {") << ENDL
	       << GetTab(1) << wxT("average") << ENDL
	       << GetTab(1) << wxT("texture_map {") << ENDL
	       << GetTab(2) << wxT("[ ") << strTex << wxT("_DIFFUSE ]") << ENDL
	       << GetTab(2) << wxT("[ ") << strTex << wxT("_AMBIENT_SPECULAR ]") << ENDL;
	strPov << GetTab(1) << ENDLCLOSE;
	strPov << wxT("};") << ENDL;
    }
}

void WavefrontMaterialCtrl_cls::
GetStrPovTextureDeclareNoMap(wxString& strPov, wxString strTex, float clr0[3][4], wxString strFinishOption, wxString strNormal){
    strPov << wxT("#declare ") << strTex << wxT("=") << ENDL;
    GetStrPovTextureColor
	(strPov, wxES, PovRayCtrl_cls::mspCThis->IsFastColor(),
	 clr0, 0.156*0.156, strFinishOption, strNormal, wxES, FALSE, FALSE);
}
