// (setq c-basic-offset 4)
// "clsPovAdvanced.h"

#include "clsPovAdvanced.h"
#include "clsPovFormat.h"

#include "../../WIDGETCTRL/clsMultiInputPanel.h"

#include "../../COMMON/CommonDefinitions.h"

PovAdvanced_cls::
PovAdvanced_cls(PovFormat_cls *pCPovFormatLink){
    mpCPovFormatLink = pCPovFormatLink;
    SetToDefault();
}

void PovAdvanced_cls::
SetToDefault(){
    mCConfList.Clear();
    mCConfList.Add(&mCMaxTraceLevel, wxT("PovMaxTraceLevel="), wxT("12"));
    mCConfList.Add(&mCTextureGamma, wxT("PovTextureGamma="), wxT("1"));
}

void PovAdvanced_cls::
OpenDialog(wxWindow *pCParent){
    MultiInputDialog_cls *pCDLG = new
	MultiInputDialog_cls(pCParent, wxID_ANY, 
			     _("Pov-Ray Advanced Settings"), wxDP, wxDS);
    
    int maxTraceLevel = mCMaxTraceLevel.mBody;
    float AAThreshold = mpCPovFormatLink->mCAAThreshold.mBody;
    int   AADepth     = mpCPovFormatLink->mCAntiAliasDepth.mBody;
    unsigned int boundingMethod = mpCPovFormatLink->mCBoundingMethod.mBody;
    float textureGamma = mCTextureGamma.mBody;
    
    MultiInputPanel_cls *pCurr = pCDLG->mpCPanel;

    pCurr->mTextWidth = 100;
    pCurr->mItemWidth = -1;

    pCurr->CombinePointer(_ITEM_INTEGER_SPIN, &maxTraceLevel, wxT("max_trace_level"));

    pCurr->SetInputAndSlider(_ITEM_FLOAT, &textureGamma, _("Texture Gamma"));
    pCurr->SetRange(_ITEM_LASTITEM, 10, 300, 100);
    
    pCurr->SetInputAndSlider(_ITEM_FLOAT, &AAThreshold, _("AA Threshold"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 59, 100);
    
    pCurr->CombinePointer(_ITEM_INTEGER_SPIN, &AADepth, _("AA Depth"));
    pCurr->SetRange(_ITEM_LASTITEM, 2, 5);

    wxString strPovBoundingMethod[2] = {
	wxT("default"), wxT("Binary Partitioning")};
    pCurr->CombinePointerList
	(&boundingMethod, 2, strPovBoundingMethod, _("Bounding Method"));

    pCDLG->MakeGeometry();
    pCurr->UpdateValues();

    if (pCDLG->ShowModal() == FALSE) return;

    mCMaxTraceLevel.mBody = maxTraceLevel;
    mCTextureGamma.mBody = textureGamma;
    mpCPovFormatLink->mCAAThreshold.mBody    = AAThreshold;
    mpCPovFormatLink->mCAntiAliasDepth.mBody = AADepth;
    mpCPovFormatLink->mCBoundingMethod.mBody = boundingMethod;
}

wxString PovAdvanced_cls::
GetStrPovGlobal(){
    wxString strSetting;
    strSetting << wxT("\tmax_trace_level ") << mCMaxTraceLevel.mBody << ENDL;
    return strSetting;
}

void PovAdvanced_cls::
ReadStrLine(wxString strLine){
    mCConfList.ReadStrLineAll(strLine);
}

wxString PovAdvanced_cls::
Serialize(){
    return mCConfList.SerializeAll();
}
