// -*- c -*- 
// (setq c-basic-offset 4)
// "clsPovFormat.h"

#include "clsPovFormat.h"

#include "../clsPovRay.h"
#include "../clsPovRayGUI.h"
#include "../OPTIONAL/clsPovRender.h"
#include "../../COMMON/clsFilename.h"
#include "../../WIDGETCTRL/clsDialogEXKey.h"
#include "../../WIDGETCTRL/clsMultiInputPanel.h"

#include "../../../wxWidgetEnum.h"
#include "../../COMMON/CommonDefinitions.h"

PovFormat_cls::
PovFormat_cls(PovRayCtrl_cls *pCCtrlLink){
    mpCCtrlLink = pCCtrlLink;
    SetToDefault();

    mstrAFormat[_POV_PNG_8BIT]  = L"N";
    mstrAFormat[_POV_PNG_12BIT] = L"N";
    mstrAFormat[_POV_PNG_16BIT] = L"N";
    mstrAFormat[_POV_SYSTEM]    = L"S";
    mstrAFormat[_POV_PPM]       = L"P";
    mstrAFormat[_POV_JPG]       = L"J";
    mstrAFormat[_POV_EXR]       = L"E";
    mstrAFormat[_POV_HDR]       = L"H";
}

void PovFormat_cls::
SetToDefault(){
    mCConfList.Clear();
    mCConfList.Add(&mCOutputFormat, wxT("PovOutputFomat="), L"0");
    mCConfList.Add(&mCWidth,        wxT("PovCameraWidth="), L"640");
    mCConfList.Add(&mCHeight,       wxT("PovCameraHeight="), L"480");
    mCConfList.Add(&mCbAntiAlias,   wxT("PovAntialias="), L"0");
    mCConfList.Add(&mCbDisplay,     wxT("PovDisplay="), L"1");
    mCConfList.Add(&mCbPause,       wxT("PovPauseWhenDone="), L"0");
    mCConfList.Add(&mCbJitter,      wxT("PovJitter="), L"0");
    mCConfList.Add(&mCbOutputAlpha, wxT("PovOutputAlpha="), L"0");
    mCConfList.Add(&mCAAThreshold,  wxT("PovAntiAliasThreshold="), L"0.15");
    mCConfList.Add(&mCAntiAliasDepth, wxT("PovAntiAliasDepth="), L"3");
    mCConfList.Add(&mCJitterAmount, wxT("PovJitterAmount="), L"1");
    mCConfList.Add(&mCbFastColoring, wxT("PovFastColoring="), L"1");
    mCConfList.Add(&mCbAbsolutePath, wxT("PovFileAbsolute="), wxT("0"));
    mCConfList.Add(&mCBoundingMethod, wxT("PovBoundingMethod="), wxT("1"));
}

void PovFormat_cls::
SetMNumFormat(wxWindow* pCParentPanel){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMNum, pCParentPanel);
    
    //int boxWidth = 2;
    pCurr->mTextWidth     = -1;
    pCurr->mComboWidth = 108;
    pCurr->mItemWidth = 108;

    pCurr->CombinePointerSizer(2, _("Output Format"));

    wxString listTemplate[_IMG_SIZE_MAX];
    for (int i = 0; i < _IMG_SIZE_MAX; i++) listTemplate[i] = GetStrGeom(i);
    mIndexGeomTemplate = pCurr->CombinePointerList
	(&mGeometryTemplate, _IMG_SIZE_MAX, listTemplate);
    
    wxString listFormat[8] = {wxT("PNG(3x8bit)"), wxT("PNG(3x12bit)"), wxT("PNG(3x16bit)"), wxT("SYS(BMP/PICT)"), wxT("PPM"), wxT("JPEG"), wxT("EXR"), wxT("HDR") };
    pCurr->CombinePointerList(&mCOutputFormat.mBody, 8, listFormat, wxT(""));
    
    mIndexOutputGeometry = pCurr->CombinePointerSizer(2, _("Width,Height"));
    pCurr->CombinePointer(_ITEM_INTEGER_SPIN, &mCWidth .mBody);
    pCurr->SetRange(_ITEM_LASTITEM, 40, 8192);
    pCurr->CombinePointer(_ITEM_INTEGER_SPIN, &mCHeight.mBody);
    pCurr->SetRange(_ITEM_LASTITEM, 30, 8192);

    pCurr->mItemWidth = -1;
    pCurr->CombinePointerSizer(3);
    int indexAntalias = pCurr->CombinePointer(_ITEM_BOOL, &mCbAntiAlias.mbBody,
					      wxT("Antialias(+A)"));
    int indexJitter = pCurr->CombinePointer(_ITEM_BOOL, &mCbJitter.mbBody,
					    wxT("Jitter(+J)"));
    int indexDisplay = pCurr->CombinePointer(_ITEM_BOOL, &mCbDisplay.mbBody,
					     wxT("Display(+D)"));

    pCurr->CombinePointerSizer(3);
    int indexPause = pCurr->CombinePointer(_ITEM_BOOL, &mCbPause.mbBody,
					     wxT("Pause(+P)"));
    int indexAlpha = pCurr->CombinePointer(_ITEM_BOOL, &mCbOutputAlpha.mbBody,
					   wxT("Alpha(+UA)"));
    int indexAbs = pCurr->CombinePointer(_ITEM_BOOL, &mCbAbsolutePath.mbBody,
					 _("Absolute path"));
    
    pCurr->CombinePointerSizer(1);
    int indexFast = pCurr->CombinePointer(_ITEM_BOOL, &mCbFastColoring.mbBody,
					  _("Fast Coloring"));
    
#ifdef WIN32
    pCurr->mMinimumWidth = 320;
#else
    pCurr->mMinimumWidth = 340;
#endif
    //pCurr->mProportion = 1;
    pCurr->FinishSetting();

    pCurr->GetPCwxWindow(indexAntalias)->
	SetToolTip(_("Antialias to decrease jaggies"));
    pCurr->GetPCwxWindow(indexJitter)->
	SetToolTip(_("Add noise to decrease jaggies"));
    pCurr->GetPCwxWindow(indexDisplay)->
	SetToolTip(_("Show rendering image on the display"));
    pCurr->GetPCwxWindow(indexPause)->
	SetToolTip(_("Pause when the rendering is finished"));
    pCurr->GetPCwxWindow(indexAlpha)->
	SetToolTip(_("Sets alpha output on"));
    pCurr->GetPCwxWindow(indexAbs)->
	SetToolTip(_("Use absolute file path"));
    pCurr->GetPCwxWindow(indexFast)->
	SetToolTip(_("Use simple and fast coloring method. If disabled, GL coloring will be emulated (slower)")); 

    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_COMBOBOX);
    
    SetIndexFromGeometry(mCWidth.mBody, mCHeight.mBody);
    HideAndShow();
}

void PovFormat_cls::
HideAndShow(){
    HideAndShowBegin();
    
    int geometry = mpCMNum->mVCItemList[mIndexGeomTemplate].GetValueInt();
    mpCMNum->ShowItem(mIndexOutputGeometry, SetGeometry(geometry, &mCWidth.mBody, &mCHeight.mBody));

    HideAndShowEnd();
}

void PovFormat_cls::
HookFunc01(int id, void *pArg){
    HideAndShow();
    RecalcSize();
}

wxString PovFormat_cls::
GetStrPovFormat(wxString strFile){
    int jpegCompress = 95;
    wxString strFormat;

    strFormat << wxT("width=") << mCWidth.mBody << ENDL;
    strFormat << wxT("height=") << mCHeight.mBody << ENDL;

    strFormat << wxT("DISPLAY=") << ((mCbDisplay.mbBody == TRUE)? wxT("on") : wxT("off")) << ENDL;
    strFormat << wxT("Pause_When_Done=") << ((mCbPause.mbBody == TRUE)? wxT("on") : wxT("off")) << ENDL;

    strFormat << wxT("Version=3.7") << ENDL;
    strFormat << wxT("Display_Gamma=") << 1 << ENDL;
    strFormat << wxT("File_Gamma=") << 1 << ENDL;
    strFormat << wxT("Render_Block_Size=") << 48/*mCRenderBlockSize.mBody*/ << ENDL;
    strFormat << wxT("Render_Pattern=") << 3 << ENDL;
    if (mCBoundingMethod.mBody == 1)
	strFormat << wxT("Bounding_Method=2") << ENDL;
    
    /*
    if (mCPovRayVersion.mBody == _POVRAY_37){
	sInfo << wxT("Version=3.7") << ENDL;
	sInfo << wxT("Display_Gamma=") << mCDisplayGamma.mBody << ENDL
	      << wxT("File_Gamma=") << mCFileGamma.mBody << ENDL;
	if (mCWorkThread.mBody != 0 || mbRenderingTwoSnap == TRUE){
	    int thread = mCWorkThread.mBody;
	    if (mbRenderingTwoSnap == TRUE){
		if (thread == 0) thread = wxThread::GetCPUCount() / 2;
		else thread /= 2;
	    }
	    sInfo << wxT("Work_Threads=") << thread << ENDL;
	}
	sInfo << wxT("Render_Block_Size=") << mCRenderBlockSize.mBody << ENDL;
	if (mCBoundingMethod.mBody == 1){
	    sInfo << wxT("Bounding_Method=2") << ENDL;
	}
	int buffSize[4] = {32, 128, 512, 0}; 
	sInfo << wxT("Max_Image_Buffer_Memory=") << buffSize[mCBufferSize.mBody] << ENDL;
    }
    */

    strFormat << wxT("Antialias=") << mCbAntiAlias.mbBody << ENDL;
    if (mCbAntiAlias.mbBody == TRUE){
	strFormat << wxT("Antialias_Threshold=") << mCAAThreshold.mBody << ENDL;
	strFormat << wxT("Antialias_Depth=") << mCAntiAliasDepth.mBody << ENDL;
    }
    
    strFormat << wxT("Jitter=") << mCbJitter.mbBody << ENDL;
    if (mCbJitter.mbBody == TRUE)
	//strFormat << wxT("Jitter_Amount=") << GetShortString(mCJitterAmount.mBody) << ENDL;
	strFormat << wxT("Jitter_Amount=") << mCJitterAmount.mBody << ENDL;

    if (mCOutputFormat.mBody == _POV_PNG_12BIT)
	strFormat << wxT("Bits_Per_Color=12") << ENDL;
    if (mCOutputFormat.mBody == _POV_PNG_16BIT)
	strFormat << wxT("Bits_Per_Color=16") << ENDL;

    strFormat << wxT("Output_File_Type=") << mstrAFormat[mCOutputFormat.mBody] << ENDL;
    if (mCOutputFormat.mBody == _POV_JPG)
	strFormat << wxT("Compression=") << jpegCompress << ENDL;

    strFormat << wxT("Output_Alpha=") << mCbOutputAlpha.mbBody << ENDL;

    wxString strFilePov = strFile;
    if (mCbAbsolutePath.mbBody == FALSE){
	SplitPath_cls CPath(strFile);
	strFilePov = CPath.GetPathStr(_FN_FNAME_EXT);
    }
    strFilePov.Replace(wxT(".ini"), wxT(".pov"));
    strFormat << wxT("Input_File_Name=\"") << strFilePov << wxT("\"") << ENDL;

    wxString strFileOut = strFilePov;
    if (mpCCtrlLink->mpCRender->mCAnimationType.mBody == _RENDER_FLYTHROUGH){
	SplitPath_cls CPath(strFileOut);
	CPath.mstrFilename = wxT("Flythrough");
	strFileOut = CPath.GetPathStr();
    }
    strFileOut.Replace(wxT(".pov"), wxT(""));
    strFormat << wxT("Output_File_Name=\"") << strFileOut << wxT("\"") << ENDL;
    
    return strFormat;
}

void PovFormat_cls::
ReadStrLine(wxString strLine){
    mCConfList.ReadStrLineAll(strLine);

    SetIndexFromGeometry(mCWidth.mBody, mCHeight.mBody);
}
