// -*- c -*- 
// (setq c-basic-offset 4)
// "clsPovRadiosity.h"

#include "clsPovRadiosity.h"

#include "../clsPovRay.h"
#include "../clsPovRayGUI.h"
#include "../../../wxWidgetEnum.h"
#include "../../WIDGETCTRL/clsAtoSTextCtrl.h"
#include "../../WIDGETCTRL/clsDialogEXKey.h"
#include "../../WIDGETCTRL/clsMultiInputPanel.h"
#include "../../WIDGETCTRL/clsHookCtrl.h"
#include "../../WIDGETCTRL/EX/clsButtonEX.h"

PovRadiosity_cls::
PovRadiosity_cls(PovRayCtrl_cls *pCCtrlLink){
    mpCCtrlLink = pCCtrlLink;
    mstrListOnOff[0] = wxT("off");
    mstrListOnOff[1] = wxT("on");
    SetToDefault();
}

void PovRadiosity_cls::
SetToDefault(){
    mCConfList.Clear();
    mCConfList.Add(&mCType, wxT("PovRadiosityType="), L"0");
    mCConfList.Add(&mCReuse, wxT("PovRadiosityReuse="), L"0");
    mCConfList.Add(&mCPathStr, wxT("PovRadiosityPathDir="), L"");
    mCConfList.Add(&mCRayCount, wxT("PovRadiosityCount="), L"35");
    mCConfList.Add(&mCError, wxT("PovRadiosityError="), L"1.8");
    mCConfList.Add(&mCMinimumReuse, wxT("PovRadiosityMinimumReuse="), L"0.015");
    mCConfList.Add(&mCRecursionLimit, wxT("PovRadiosityRecursionLimit="), L"3");
    mCConfList.Add(&mCNearestCount, wxT("PovRadiosityNearestCount="), L"5");
    mCConfList.Add(&mCTraceStart, wxT("PovRadiosityTraceStrat="), L"0.08");
    mCConfList.Add(&mCTraceEnd, wxT("PovRadiosityTraceEnd="), L"0.04");
    mCConfList.Add(&mCbZeroAmbient, wxT("PovRadiosityZeroAmbient="), L"1");
    mCConfList.Add(&mCbMedia, wxT("PovRadiosityMedia="), L"0");
    mCConfList.Add(&mCbNormal, wxT("PovRadiosityNormal="), L"1");
    mCConfList.Add(&mCHighReproducibility, wxT("PovRadHighReproduce="), L"0");
}

void PovRadiosity_cls::
SetMNumRadiosity(wxWindow* pCParentPanel, wxString *psWorkingDir){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMNum, pCParentPanel);

    pCurr->mTextWidth  = -1;
    pCurr->mItemWidth = 180;
    //pCurr->mComboWidth = 108;

    wxString radiosityType[3] =
	{_("No Radiosity (fast)"), _("Pretrace Only"),
	 _("Fully Trace")};
    mIndexType = pCurr->CombinePointerList
	(&mCType.mBody,
	 3, radiosityType, _("radiosity"));
    
    pCurr->mTextWidth = 120;
    pCurr->mItemWidth = -1;
    //int boxWidth = 2;
    mIndexRecursionLimit = pCurr->SetInputAndSlider
	(_ITEM_INTEGER, &mCRecursionLimit.mBody, wxT("recursion_limit"));
    pCurr->SetRange(_ITEM_LASTITEMx2, 1, 20);

    mIndexCount = pCurr->SetInputAndSlider
	(_ITEM_INTEGER, &mCRayCount.mBody, _("ray count"));
    pCurr->SetRange(_ITEM_LASTITEMx2, 35, 3200);

    mIndexError = pCurr->SetInputAndSlider
	(_ITEM_FLOAT, &mCError.mBody, _("error bound"));
    pCurr->SetRange(_ITEM_LASTITEMx2, 1, 200, 100);
    
    mIndexMinReuse = pCurr->SetInputAndSlider
	(_ITEM_FLOAT, &mCMinimumReuse.mBody, wxT("minimum_reuse"));
    pCurr->SetRange(_ITEM_LASTITEMx2, 1, 30, 1000);

    mIndexNearestCount = pCurr->SetInputAndSlider
	(_ITEM_INTEGER, &mCNearestCount.mBody,  wxT("nearest_count"));
    pCurr->SetRange(_ITEM_LASTITEMx2, 3, 20);
    
    mIndexReproducibility = pCurr->CombinePointerList
	(&mCHighReproducibility.mBody,
	 2, mstrListOnOff, wxT("High Reproducibility"));
    
    mIndexTraceStartEnd = pCurr->CombinePointerSizer(2, _("Trace Start/End"));
    pCurr->CombinePointer(_ITEM_FLOAT, &mCTraceStart.mBody);
    pCurr->CombinePointer(_ITEM_FLOAT, &mCTraceEnd.mBody);
	
    mIndexBoolOptions = pCurr->CombinePointerSizer(3);
    pCurr->CombinePointer
	(_ITEM_BOOL, &mCbZeroAmbient.mbBody, wxT("ambient off"));
    pCurr->CombinePointer
	(_ITEM_BOOL, &mCbMedia.mbBody, wxT("media on"));
    pCurr->CombinePointer
	(_ITEM_BOOL, &mCbNormal.mbBody, wxT("normal on"));

    wxString radiosityReuse[3] =
	{_("Not reuse pretrace"), _("Save Pretrace"),
	 _("Load Pretrace")};
    mIndexReuse = pCurr->CombinePointerList
	(&mCReuse.mBody,
	 3, radiosityReuse, _("reuse pretrace"));
    /*
    mIndexPath = pCurr->CombinePointer
	(_ITEM_WXSTRING_PATH, &mCPathStr.msBody, _("Cache File"), TRUE);
    */
    
#ifdef WIN32
    pCurr->mMinimumWidth = 320;
#else
    pCurr->mMinimumWidth = 340;
#endif
    //pCurr->mProportion = 1;
    pCurr->FinishSetting();

    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_COMBOBOX);
    
    /*
    AtoWXSTextPathCtrl_cls* pSelector = (AtoWXSTextPathCtrl_cls*)pCurr->mVCPointers[mIndexPath].mpPointerCtrl;
    */
    
    /*
    pSelector->mpstrDefaultDirLink = psWorkingDir;
    pSelector->mstrFilter = wxT("radiosity cache file(*.txt)|*.txt|all|*");
    pSelector->mstrDefaultFileName = wxT("radiosity.txt");
    pSelector->mStyle = wxFD_SAVE;
    */
    HideAndShow();
}

void PovRadiosity_cls::
HideAndShow(){
    HideAndShowBegin();
    MultiInputPanel_cls *pCurr = mpCMNum;
    bool bShow = (mCType.mBody > _POV_RADIOSITY_NOUSE);

    pCurr->ShowItem(mIndexRecursionLimit, bShow);
    pCurr->ShowItem(mIndexCount, bShow);
    pCurr->ShowItem(mIndexError, bShow);
    // Ƃ肠܂ׂݒ͉BĂ
    bool bTweak = (mpCCtrlLink->mCShowTweaks.mBody == 1);
    pCurr->ShowItem(mIndexMinReuse, bTweak & bShow);
    pCurr->ShowItem(mIndexNearestCount, bTweak & bShow);
    pCurr->ShowItem(mIndexReproducibility, bTweak & bShow);
    
    pCurr->ShowItem(mIndexTraceStartEnd, bShow);
    pCurr->ShowItem(mIndexBoolOptions, bShow);
    pCurr->ShowItem(mIndexReuse,  bShow);

    /*
    bool bReuse = (mCReuse.mBody != 0);
    pCurr->ShowItem(mIndexPath, bShow & bReuse);
    */
    HideAndShowEnd();
}

void PovRadiosity_cls::
HookFunc01(int id, void *pArg){
    HideAndShow();
    RecalcSize();
}

bool PovRadiosity_cls::
IsRadiosityFormatOK(){
    if (mCType.mBody == 0) return TRUE;
    if (mCReuse.mBody == 0) return TRUE;
    
    if (mCPathStr.mstrBody == wxT("NONE")) return FALSE;

    switch (mCReuse.mBody){
    case 1: {
	SplitPath_cls CSplit(mCPathStr.mstrBody);
	wxFileName CTmpDir(CSplit.GetPathStr(_FN_DRIVE_DIR));
	printf("PathCheck= %ls\n", CSplit.GetPathStr(_FN_DRIVE_DIR).c_str());
	if (CTmpDir.IsDirWritable() == FALSE)
	    return FALSE; }
	break;
    case 2: {
	wxFileName CTmpFile(mCPathStr.mstrBody);
	if (CTmpFile.IsFileReadable() == FALSE)
	    return FALSE; }
	break;
    }
    return TRUE;
}

wxString PovRadiosity_cls::
GetStrPovGlobal(){
    if (mCType.mBody == 0) return wxT("");
    wxString strPov;

    strPov << wxT("\tradiosity {") << ENDL;
    if (mCType.mBody == 1)
	strPov << wxT("\t\talways_sample off") << ENDL;
    if (mCType.mBody == 2)
	strPov << wxT("\t\talways_sample on") << ENDL;
    
    strPov 
	<< wxT("\t\tpretrace_start ") << mCTraceStart.mBody << ENDL
	<< wxT("\t\tpretrace_end ") << mCTraceEnd.mBody << ENDL
	<< wxT("\t\tcount ") << mCRayCount.mBody << ENDL
	<< wxT("\t\tnearest_count ") << mCNearestCount.mBody << ENDL
	<< wxT("\t\terror_bound ") << mCError.mBody << ENDL
	<< wxT("\t\tminimum_reuse ") << mCMinimumReuse.mBody << ENDL
	<< wxT("\t\trecursion_limit ") << mCRecursionLimit.mBody << ENDL;
    
    if (mCbMedia.mbBody == TRUE)
	strPov << wxT("\t\tmedia on") << ENDL;
    if (mCbNormal.mbBody == TRUE)
	strPov << wxT("\t\tnormal on") << ENDL;

    /*
    if (mCReuse.mBody == 1)
	strPov << wxT("\t\tsave_file \"") << mCPathStr.msBody << wxT("\"") << ENDL;
    if (mCReuse.mBody == 2)
	strPov << wxT("\t\tload_file \"") << mCPathStr.msBody << wxT("\"") << ENDL;
    */
    
    strPov << wxT("\t}") << ENDL;
    if (mCbZeroAmbient.mbBody == TRUE)
	strPov << wxT("\tambient_light 0") << ENDL;

    return strPov;
}

wxString PovRadiosity_cls::
GetStrPovIni(){
    if (mCType.mBody == 0) return wxT("");
    wxString strIni;
    
    wxString strWorkingDir = mpCCtrlLink->GetWorkingDirWithSep(TRUE);
    wxString strPath = mCPathStr.mstrBody;
    if (mpCCtrlLink->mCbAbsolutePath.mbBody == FALSE)
	strPath.Replace(strWorkingDir, wxT("./"));

    if (mCReuse.mBody != 0 &&
	IsRadiosityFormatOK() == TRUE)
	strIni << wxT("Radiosity_File_Name=\"") << strPath << wxT("\"") << ENDL;
    if (mCReuse.mBody == 1)
	strIni << wxT("Radiosity_To_File=on") << ENDL;
    if (mCReuse.mBody == 2)
	strIni << wxT("Radiosity_From_File=on") << ENDL;

    if (mCType.mBody != 0)
	strIni << wxT("Radiosity_Vain_Pretrace=off") << ENDL
	       << wxT("High_Reproducibility=") << mstrListOnOff[mCHighReproducibility.mBody] << ENDL; 
    
    return strIni;
}
