// -*- c -*- 
// (setq c-basic-offset 4)
// "clsPovRay.h"

#include "clsPovRay.h"
#include "clsPovRayGUI.h"
#include "clsPovMaterialColor.h"
#include "clsPovThread.h"
#include "OPTIONAL/clsPovAdvanced.h"
#include "OPTIONAL/clsPovFocalBlur.h"
#include "OPTIONAL/clsPovFormat.h"
#include "OPTIONAL/clsPovOptions.h"
#include "OPTIONAL/clsPovPhotons.h"
#include "OPTIONAL/clsPovRadiosity.h"
#include "OPTIONAL/clsPovRender.h"

#include "../FILTER/clsFilterCtrl.h"
#include "../FILTER/clsFilterObject.h"
#include "../GLATTRIB/clsColorCurve.h"
#include "../GLATTRIB/clsGLAttribute.h"
#include "../GLCANVAS/clsStereoGram.h"
#include "../GLCANVAS/clsMainGLCanvas.h"
#include "../MAINCANVAS/clsDrawOption.h"
#include "../MYAPI/MsgBox.h"
#include "../PARTICLEDATA/clsSnapData.h"
#include "../PARTICLEDATA/clsGLDrawData.h"
#include "../PARTICLEDATA/clsArrayAndIndex.h"
#include "../TIMELINE/clsTimeCtrl.h"

#include "../../clsMain.h"

#include "../COMMON/CommonDefinitions.h"

#include <wx/filename.h>
#include <wx/file.h>

#ifdef WIN32
#define POVFILE wxT("pvengine*.exe files (pvengine.exe)|pvengine*.exe|all files|*")
#ifdef _WIN64
#define POVDEF wxT("c:\\Program Files\\POV-Ray\\v3.7\\bin\\pvengine64.exe")
#else
#define POVDEF wxT("c:\\Program Files\\POV-Ray\\v3.7\\bin\\pvengine32-sse2.exe")
#endif
#define SEPARATOR wxT("\\")
#else
#define POVFILE wxT("povray files (povray)|*povray*")
#define POVDEF wxT("/usr/bin/povray")
#define SEPARATOR wxT("/")
#endif

ThreadSPString_cls::
ThreadSPString_cls(bool *pbFinishedFlagLink, char *pBuf, 
		   int init, int fin, int step, unsigned char *pType, 
		   int drawTypeList[_MAX_PARTICLE_TYPE],
		   int radiusTypeList[_MAX_PARTICLE_TYPE],
		   float radiusFactorList[_MAX_PARTICLE_TYPE],
		   bool bConstantColorInSphere[_MAX_PARTICLE_TYPE],
		   bool bConstantAlphaInSphere[_MAX_PARTICLE_TYPE],
		   float (*pPos)[3], float *pRadius, 
		   float *pValue1Mat[_MAX_PARTICLE_TYPE]) :
    wxThread(wxTHREAD_DETACHED)
{
    mpbFinishedFlagLink = pbFinishedFlagLink;
    mpBuf = pBuf;
    mInit = init;
    mFin  = fin;
    mStep = step;
    mType = pType;
    mDrawTypeList = drawTypeList;
    mRadiusTypeList = radiusTypeList;
    mRadiusFactorList = radiusFactorList;
    mbConstantColorInSphere = bConstantColorInSphere;
    mbConstantAlphaInSphere = bConstantAlphaInSphere;
    mpPos = pPos;
    mpRadius = pRadius;
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	mpValue1Mat[i] = pValue1Mat[i];

    if ( Create() != wxTHREAD_NO_ERROR)
	MsgBoxExit(NULL, "Error!", "Can't create thread");
}

wxThread::ExitCode ThreadSPString_cls::
Entry(){
    ArrayAndIndex_cls *pIndexAndZ = GLDrawData_cls::mspIndexAndZ;

    for (int i_ = mInit; i_ < mFin; i_ += mStep){
	int i = (pIndexAndZ+i_)->index;
	
	int particleType = mType[i];
	
	if (mDrawTypeList[particleType] == _GL_DRAW_BOTH) continue;	
	if (mDrawTypeList[particleType] == _GL_HIDE) continue;

	float filterAlphaFactor = ((pIndexAndZ + i_))->alphaFactor;
	if (filterAlphaFactor == 0) continue;
	float radiusFactor = mRadiusFactorList[particleType];

	if (mRadiusTypeList[particleType] != _GL_RADIUS_CONSTANT)
	    radiusFactor *= mpRadius[i];

	if (mbConstantColorInSphere[particleType] == FALSE){
	    //F`
	    int charnum = 0;
	    if (mRadiusTypeList[particleType] == _GL_RADIUS_CONSTANT)
		charnum = 
		    sprintf(mpBuf, "SP%d(<%.3f,%.3f,%.3f>,%.3f,%.3f)\n", particleType, 
			    mpPos[i][X], mpPos[i][Y], mpPos[i][Z], mpValue1Mat[particleType][i], filterAlphaFactor);
	    else
		charnum = 
		    sprintf(mpBuf, "SP%d(<%.3f,%.3f,%.3f>,%.3f,%.3f,%.3f)\n", particleType, 
			    mpPos[i][X], mpPos[i][Y], mpPos[i][Z], mpValue1Mat[particleType][i], radiusFactor, filterAlphaFactor);
	    mpBuf += charnum;
	}
	else if (mbConstantAlphaInSphere[particleType] == FALSE){
	    //Alpha`
	    int charnum = 0;
	    if (mRadiusTypeList[particleType] == _GL_RADIUS_CONSTANT)
		charnum = 
		    sprintf(mpBuf, "SP%d(<%.3f,%.3f,%.3f>,%.3f)\n", particleType, 
			    mpPos[i][X], mpPos[i][Y], mpPos[i][Z], filterAlphaFactor);
	    else
		charnum = 
		    sprintf(mpBuf, "SP%d(<%.3f,%.3f,%.3f>,%.3f,%.3f)\n", particleType, 
			    mpPos[i][X], mpPos[i][Y], mpPos[i][Z], radiusFactor, filterAlphaFactor);
	    mpBuf += charnum;
	}
	else {
	    int charnum = 0;
	    if (mRadiusTypeList[particleType] == _GL_RADIUS_CONSTANT)
		charnum = 
		    sprintf(mpBuf, "SP%d(<%.3f,%.3f,%.3f>)\n", particleType, 
			    mpPos[i][X], mpPos[i][Y], mpPos[i][Z]);
	    else
		charnum = 
		    sprintf(mpBuf, "SP%d(<%.3f,%.3f,%.3f>,%.3f)\n", particleType, 
			    mpPos[i][X], mpPos[i][Y], mpPos[i][Z], radiusFactor);
	    mpBuf += charnum;
	}
    }

    *mpbFinishedFlagLink = TRUE;
    return 0;
}

// -----

PovRayCtrl_cls::
PovRayCtrl_cls(){
    mspCThis = this;    
    mpCGUI = NULL;

    mpCFormat    = new PovFormat_cls(this);
    mpCRadiosity = new PovRadiosity_cls(this);
    mpCRender    = new PovRender_cls(this);
    mpCPhotons   = new PovPhotons_cls;
    mpCAdvanced  = new PovAdvanced_cls(mpCFormat);
    mpCFocalBlur = new PovFocalBlur_cls;

    mbExportingAnim = FALSE;
    mbRenderingAnim = FALSE;
    mbPovRayThreadFinished = TRUE;

    SetToDefault();
}

PovRayCtrl_cls::
~PovRayCtrl_cls(){
    
    delete mpCAdvanced;
    delete mpCFocalBlur;
    delete mpCFormat;
    delete mpCRadiosity;
    delete mpCRender;
    delete mpCPhotons;

    // FIX_ME!! PovRay҂̎OosawȁIvZXɓƂ܂B
    // ǂďIVOi𑗂H
    if (mbPovRayThreadFinished == FALSE){
	mpCThreadPovRay->Delete();
    }
}

TimeCtrl_cls* PovRayCtrl_cls::
GetPCTimeCtrl(){
    return mpCParentFrameLink->GetPCTimeCtrl();
}

void PovRayCtrl_cls::
InitGUI(FrameMain_cls *pCParentFrameLink){
    mpCParentFrameLink = pCParentFrameLink;

    mpCGUI = new PovRayGUI_cls(this, pCParentFrameLink);
    SetPDialog(mpCGUI);

    PovOptions_cls::mspCDialogLink = mpCGUI;

    if (mpCGUI->IsWorkingDirOk() == FALSE)
	mCConfWorkingDir.mstrBody = GetDefaultWorkingDir();
}

wxString PovRayCtrl_cls::
GetDefaultWorkingDir(){
    bool bSuccess = FALSE;
    wxString strWork;
#ifdef WIN32
    wchar_t buf[512];
    if (GetCurrentDirectory(512, buf) != 0) bSuccess = TRUE;
    if (bSuccess == TRUE) strWork = std::wstring(buf) + SEPARATOR + L"POV";
#else
    char buf[512];
    if (getcwd (buf, 512) != NULL) bSuccess = TRUE;
    wchar_t buf2[512];
    wxConvFile.ToWChar(buf2, 512, buf, 512);
    if (bSuccess == TRUE) strWork = std::wstring(buf2) + SEPARATOR + L"POV";
#endif
    return strWork;
}

void PovRayCtrl_cls::
SetToDefault(){
    mSnapNumber = 0;
   
    wxString strWork = GetDefaultWorkingDir();
    
    mCConfListPovPath.Clear();
    mCConfListPovPath.Add(&mCConfPathPovray, wxT("PathPovRay="), POVDEF);
    mCConfListPovPath.Add(&mCConfWorkingDir, wxT("PathPovWork="), strWork);
    mCConfListPovPath.Add(&mCConfSnapFile, wxT("FilePovSnap="), wxT("povsnap%04d"));
    mCConfListPovPath.Add(&mCConfAnimFile, wxT("FilePovAnim="), wxT("povanim%04d"));
    //mCConfListPovPath.Add(&mCbAbsolutePath, wxT("FileAbsolute="), wxT("1"));
    
    mCConfListPovParam.Clear();
    mCConfListPovParam.Add(&mCShowTweaks, wxT("PovTweaks="), wxT("0"));
    mCConfListPovParam.Add(&mCbUseCommonSceneFile, wxT("UseCommonSceneFile="), wxT("1"));
}

void PovRayCtrl_cls::
UpdateItems(bool bForce){
    if (mpCGUI == NULL) return;
    
    mpCRadiosity->mpCMNum->UpdateValues();
    mpCRadiosity->HideAndShow();
    mpCPhotons->mpCMNum->UpdateValues();
    mpCPhotons->HideAndShow();
    mpCFormat->mpCMNum->UpdateValues();
    mpCFormat->HideAndShow();
    mpCFocalBlur->mpCMNum->UpdateValues();
    mpCFocalBlur->HideAndShow();
    
    mpCGUI->RecalcSize();
    mpCGUI->ResizeDialog();
    mpCGUI->Refresh(FALSE);

    mpCGUI->CheckFileExistencePov();
    mpCGUI->CheckFileExistenceWorkdir();
}

void PovRayCtrl_cls::
EnableMenuItem(bool  b0, bool b1, bool b2, bool b3){
    mpCParentFrameLink->EnableMenuItemPov(b0, b1, b2, b3);
}

bool PovRayCtrl_cls::
IsFastColor(){
    return mpCFormat->mCbFastColoring.mbBody;
}

int PovRayCtrl_cls::
GetRegionType(){
    return mpCRender->mCRegionType.mBody;
}

int PovRayCtrl_cls::
GetRegionInit(){
    return mpCRender->mCRegionCustomMin.mBody;
}

int PovRayCtrl_cls::
GetRegionFin(){
    return mpCRender->mCRegionCustomMax.mBody;
}

void PovRayCtrl_cls::
IncrementAnim(){
    mAnimNumber++;
}

wxString PovRayCtrl_cls::
GetFileStr(unsigned int type, unsigned int stereoId, int index){
    mpCGUI->UpdateFromText();

    if (type == _POV_COMMON){
	if (mpCFormat->mCbAbsolutePath.mbBody == TRUE)
	    return GetWorkingDirWithSep(TRUE) + wxT("CommonScene.pov");
	return wxT("CommonScene.pov");
    }
    
    if (index == -1){
	if (type == _POV_SNAP)
	    index = mSnapNumber;
	else
	    index = mAnimNumber;
    }
    
    wxString strFormat = (type == _POV_ANIM) ?
	mCConfAnimFile.mstrBody : mCConfSnapFile.mstrBody;
    
    if (stereoId == _STEREO_LFT)
	strFormat.Replace(wxT("%"), wxT("L%"));
    if (stereoId == _STEREO_RGT)
	strFormat.Replace(wxT("%"), wxT("R%"));
    
    return wxString::Format(GetWorkingDirWithSep(TRUE) + strFormat, index);
}


wxString PovRayCtrl_cls::
GetWorkingDirWithSep(bool bForceAbs){
    if (bForceAbs == TRUE/* || mCbConfAbsolutePath.mbBody == TRUE*/)
	return mCConfWorkingDir.mstrBody + SEPARATOR;
    return wxT("./");
}

/*
void PovRayCtrl_cls::
GetStrPovPigmentImageMap(wxString &strPov, wxString strFile, float alpha){
    if (strFile.IsEmpty() == TRUE) return;
    strPov << wxT("uv_mapping image_map { png \"") << strFile << wxT("\"");
    strPov << PovOutf(wxT(" gamma "), mspCThis->mpCAdvanced->mCTextureGamma.mBody, 2);
    strPov << wxT(" interpolate 2");
    if (alpha != 1)
	strPov << wxT(" transmit all ") << PovOutf(1.0-alpha, 3);
    strPov << wxT("}");
}
*/

/*
void PovRayCtrl_cls::
GetStrPovTextureColor(wxString &strPov, wxString strTab, 
		      float pClrMat[3][4], float roughness,
		      wxString strFinishOptions, wxString strNormal, 
		      bool bFilterAlpha, bool bSetDiffuseZero){
    strPov << strTab << wxT("texture {") << ENDL;
    if (mspCThis->mpCFormat->mCbFastColoring.mbBody == TRUE){
	strPov << PovMaterialColor_cls::GetStrPovTextureFastColor(strTab, pClrMat, roughness, strFinishOptions, bFilterAlpha, bSetDiffuseZero);
	if (strNormal.IsEmpty() == FALSE) strPov << strTab << strNormal;
    }
    else
	strPov << PovMaterialColor_cls::GetStrPovTextureGLColor(strTab, pClrMat, roughness, strFinishOptions, strNormal, bFilterAlpha, bSetDiffuseZero);

    strPov << strTab << ENDLCLOSE;
}
*/

void PovRayCtrl_cls::
GetStrPovTexColorMapPigment(wxString &strPov, wxString strTab, 
			    bool bFilterAlpha,
			    GLAttribute_cls *pCGLAttrib, int index){
    strPov << strTab << wxT("pigment{ ");
    pCGLAttrib->mpCClrCurveMat[index]->GetStrPovScaling(strPov, wxES);
    pCGLAttrib->mpCClrCurveMat[index]->GetStrPovColorMap(strPov, strTab + GetTab(2), bFilterAlpha, pCGLAttrib->mCAlphaFactor.mBody);
    strPov << strTab << ENDLCLOSE;
}

void PovRayCtrl_cls::
GetStrPovTexColorMapFast(wxString &strPov, wxString strTab, 
			 float pClrMat[3][4], float roughness,
			 wxString strFinishOptions, wxString strNormal,
			 GLAttribute_cls *pCGLAttrib, bool bFilterAlpha){
    int adopted = 2;
    if (pCGLAttrib->mpCClrCurveMat[1]->mCbColorCurve.mbBody == TRUE) adopted = 1;
    if (pCGLAttrib->mpCClrCurveMat[0]->mCbColorCurve.mbBody == TRUE) adopted = 0;

    GetStrPovTexColorMapPigment(strPov, strTab, bFilterAlpha, pCGLAttrib, adopted);
        
    wxString strEmission;
    if (adopted != 0)
	strEmission << pCGLAttrib->mpCClrCurveMat[0]->GetStrPov();
    else
	strEmission << 0.3;

    strPov << strTab << wxT("finish { emission ") << strEmission << wxT(" diffuse 1 specular 0.3") << strFinishOptions;
    strPov << strTab << ENDLCLOSE;
    if (strNormal.IsEmpty() == FALSE) strPov << strTab << strNormal;
}

void PovRayCtrl_cls::
GetStrPovTexColorMapGL(wxString &strPov, wxString strTab, 
		       float pClrMat[3][4], float roughness,
		       wxString strFinishOptions, wxString strNormal,
		       GLAttribute_cls *pCGLAttrib, bool bFilterAlpha){
    strPov << strTab << wxT("average") << ENDL
	   << strTab << wxT("texture_map {") << ENDL;

    strTab << GetTab(1);

    int itemNum = 3;
    if (pCGLAttrib->IsBlack(0) == TRUE) itemNum --;
    if (pCGLAttrib->IsBlack(2) == TRUE) itemNum --;

    if (itemNum == 3 &&
	pCGLAttrib->IsColorCurve(0) == FALSE &&
	pCGLAttrib->IsColorCurve(2) == FALSE){ // Ambient Specular 𓯎
	strPov << strTab << wxT("[") << ENDL;
	GetStrPovTexColorMapPigment(strPov, strTab+GetTab(1), bFilterAlpha, pCGLAttrib, 1);
	PovMaterialColor_cls::GetStrPovTextureFinish(strPov, strTab, 1, roughness, strFinishOptions, strNormal, 2); strPov << ENDL;
	strPov << strTab << wxT("]") << ENDL;
	
	strPov << strTab << wxT("[");
	PovMaterialColor_cls::GetStrPovMaterialColor(strPov, pClrMat[2], bFilterAlpha);
	strPov << PovMaterialColor_cls::GetStrPovTextureFinishAmbSp(roughness, pClrMat[0], pClrMat[2], strFinishOptions, strNormal, bFilterAlpha);
	strPov << strTab << wxT("]") << ENDL;
	strPov << strTab << ENDLCLOSE;
	return;
    }

    for (int mat = 0; mat < 3; mat++){
	if (pCGLAttrib->IsBlack(mat) == TRUE && mat != 1) continue;
	if (pCGLAttrib->IsColorCurve(mat) == TRUE){
	    strPov << strTab << wxT("[") << ENDL;
	    GetStrPovTexColorMapPigment(strPov, strTab+GetTab(1), bFilterAlpha, pCGLAttrib, mat);
	    PovMaterialColor_cls::GetStrPovTextureFinish(strPov, strTab, mat, roughness, strFinishOptions, strNormal, itemNum); strPov << ENDL;
	    strPov << strTab << wxT("]") << ENDL;
	}
	else{
	    strPov << strTab << wxT("[");
	    PovMaterialColor_cls::GetStrPovMaterialColor(strPov, pClrMat[mat], bFilterAlpha);
	    PovMaterialColor_cls::GetStrPovTextureFinish(strPov, wxES, mat, roughness, strFinishOptions, strNormal, itemNum);
	    strPov << wxT("]") << ENDL;
	}
    }
    
    strPov << strTab << ENDLCLOSE;
}

void PovRayCtrl_cls::
GetStrPovTexColorMap(wxString &strPov, wxString strTab, 
		     float pClrMat[3][4], float roughness,
		     wxString strFinishOptions, wxString strNormal,
		     GLAttribute_cls *pCGLAttrib, bool bFilterAlpha){
    strPov << strTab << wxT("texture {") << ENDL;

    if (mspCThis->mpCFormat->mCbFastColoring.mbBody == TRUE)
	GetStrPovTexColorMapFast(strPov, strTab+GetTab(1), pClrMat, roughness, strFinishOptions, strNormal, pCGLAttrib, bFilterAlpha);
    else
	GetStrPovTexColorMapGL(strPov, strTab+GetTab(1), pClrMat, roughness, strFinishOptions, strNormal, pCGLAttrib, bFilterAlpha);
    
    strPov << strTab << ENDLCLOSE;
}

void PovRayCtrl_cls::
GetStrPovPhotonSetting(wxString &strPov, double alpha, double reflectMin, double reflectMax, bool bTarget, bool bCollect){
    mspCThis->mpCPhotons->GetStrPovPhotonSetting
	(strPov, alpha, reflectMin, reflectMax, bTarget, bCollect);
}

// !!_FIX_ME_!! ܂ At@ ωꍇɂ̂ݑΉ

wxString PovRayCtrl_cls::
GetPovMacroSphereVariousColor(int index, float pClrMat[3][4], 
			      bool bConstantRadius,
			      float radius, float roughness,
			      GLAttribute_cls *pCGLAttrib){
    wxString strMacro;
    if (bConstantRadius == TRUE)
	strMacro << wxT("#macro SP") << index << wxT("(VPos,value1,FilterAlpha)") << ENDL
		 << wxT("sphere{ VPos,") << radius << ENDL;
    else
	strMacro << wxT("#macro SP") << index << wxT("(VPos,value1,Radi,FilterAlpha)") << ENDL
		 << wxT("sphere{ VPos, Radi") << ENDL;
    GetStrPovTexColorMap(strMacro, GetTab(1), pClrMat, roughness, 
			 pCGLAttrib->mCTexture.GetStrPovFinishOptions(),
			 pCGLAttrib->mCTexture.GetStrPovNormal(),
			 pCGLAttrib, TRUE);
    /*
    strMacro << ENDLCLOSE
	     << wxT("#end") << ENDL;
    */
    return strMacro;
}

wxString PovRayCtrl_cls::
GetPovMacroSphereVariousAlpha(int index, float pClrMat[3][4], 
			      bool bConstantRadius,
			      float radius, float roughness,
			      GLAttribute_cls *pCGLAttrib){
    wxString strMacro;
    if (bConstantRadius == TRUE)
	strMacro << wxT("#macro SP") << index << wxT("(VPos,FilterAlpha)") << ENDL
		 << wxT("sphere{ VPos,") << radius << ENDL;
    else
	strMacro << wxT("#macro SP") << index << wxT("(VPos,Radi,FilterAlpha)") << ENDL
		 << wxT("sphere{ VPos, Radi") << ENDL;
    GetStrPovTextureColor
	(strMacro, GetTab(1), 
	 mpCFormat->mCbFastColoring.mbBody, pClrMat, roughness, 
	 pCGLAttrib->mCTexture.GetStrPovFinishOptions(),
	 pCGLAttrib->mCTexture.GetStrPovNormal(), wxES, TRUE);
    /*
    strMacro << ENDLCLOSE
	     << wxT("#end") << ENDL;
    */
    return strMacro;
}

wxString PovRayCtrl_cls::
GetPovMacroSphereConstColor(int index, float pClrMat[3][4], 
			    bool bConstantRadius,
			    float radius, float roughness, 
			    GLAttribute_cls *pCGLAttrib){
    wxString strMacro;
    if (bConstantRadius == TRUE)
	strMacro << wxT("#macro SP") << index << wxT("(VPos)") << ENDL
		 << wxT("sphere{ VPos,") << radius << ENDL;
    else
	strMacro << wxT("#macro SP") << index << wxT("(VPos,Radi)") << ENDL
		 << wxT("sphere{ VPos, Radi") << ENDL;

    GetStrPovTextureColor
	(strMacro, GetTab(1), 
	 mpCFormat->mCbFastColoring.mbBody, pClrMat, roughness, 
	 pCGLAttrib->mCTexture.GetStrPovFinishOptions(),
	 pCGLAttrib->mCTexture.GetStrPovNormal());
    
    return strMacro;
}

void PovRayCtrl_cls::
GetStrPovParticleMacroObject(wxString &strPData,
			     int objIndex, double doubleIndex, 
			     wxString strTranslate){
    
    FilterObject_cls *pCObj = 
	GLDrawBillboard_cls::mspCFilterCtrlObjectLink->GetPCObject(objIndex);
    if (pCObj == NULL) return;
    
    pCObj->GetStrPovRealObject(strPData, doubleIndex, strTranslate, FALSE, GetTab(1), TRUE);

}

void PovRayCtrl_cls::
GetStrPovParticleMacro(wxString &strPData, double doubleIndex,
		       GLAttribute_cls **ppCGLAttribute){
    
    int step = ppCGLAttribute[0]->mspCDrawOptionLink->mCDownSize.mBody;

    float steppingRadiusFactor = sqrt(1.0f + ((float)step - 1.0f) * (0.2 + 0.6*(float)step/100));

    bool bConstantRadiusInSphere[_MAX_PARTICLE_TYPE];
    bool bConstantColorInSphere[_MAX_PARTICLE_TYPE];
    bool bConstantAlphaInSphere[_MAX_PARTICLE_TYPE];

    int   radiusTypeList[_MAX_PARTICLE_TYPE];
    float radiusFactorList[_MAX_PARTICLE_TYPE];
    float alphaFactor[_MAX_PARTICLE_TYPE];
    int   drawTypeList[_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	bConstantRadiusInSphere[i] = (radiusTypeList[i] == _GL_RADIUS_CONSTANT);
	bConstantColorInSphere[i] = 
	    (ppCGLAttribute[i]->mpCClrCurveMat[0]->mCbColorCurve.mbBody!=TRUE)&
	    (ppCGLAttribute[i]->mpCClrCurveMat[1]->mCbColorCurve.mbBody!=TRUE)&
	    (ppCGLAttribute[i]->mpCClrCurveMat[2]->mCbColorCurve.mbBody!=TRUE);

	bConstantAlphaInSphere[i] = TRUE;
	if (GLDrawData_cls::mspCFilterCtrlBooleanLink->
	    IsBooleanChangeConstantColor() == TRUE)
	    bConstantAlphaInSphere[i] = FALSE;
	
	alphaFactor[i]      = ppCGLAttribute[i]->mCAlphaFactor.mBody;
	drawTypeList[i]     = ppCGLAttribute[i]->mCDrawType.mBody;
	radiusTypeList[i]   = ppCGLAttribute[i]->mCRadiusType.mBody;
	radiusFactorList[i] = ppCGLAttribute[i]->GetRadiusFactor() * 
	    steppingRadiusFactor;
    }
    
    int macroNum = _MAX_PARTICLE_TYPE;
    if (GLAttribute_cls::mspCDrawOptionLink->mCClassify.mBody == 0) macroNum = 1;

    float pClrMat[3][4];
    for (int i = 0; i < macroNum; i++){
	if (drawTypeList[i] == _GL_HIDE) continue;

	GLAttribute_cls *pCGLAttrib = ppCGLAttribute[i];

	if (drawTypeList[i] == _GL_DRAW_OBJECT ||
	    drawTypeList[i] == _GL_DRAW_PSEUDO_OBJECT){
	    
	    strPData << wxT("#macro SP") << i << wxT("(Vpos, Radi)") << ENDL;
	    wxString strTranslate;
	    strTranslate << wxT("scale Radi") << ENDL << wxT("translate Vpos") << ENDL;

	    GetStrPovParticleMacroObject(strPData, 
					 pCGLAttrib->mCObjectIndex.mBody,
					 doubleIndex, strTranslate);
	    strPData << wxT("#end") << ENDL;
	    continue;
	}
	else{
	    for (int mat = 0; mat < 3; mat++){
		ppCGLAttribute[i]->
		    mpCClrCurveMat[mat]->GetColor4fScaled(pClrMat[mat], 0);
		pClrMat[mat][3] = alphaFactor[i];
	    }
	    
	    if (bConstantColorInSphere[i] == FALSE){
		// F`
		strPData << GetPovMacroSphereVariousColor
		    (i, pClrMat, bConstantRadiusInSphere[i], radiusFactorList[i],
		     0.156f*0.156f, pCGLAttrib);
	    }
	    else if (bConstantAlphaInSphere[i] == FALSE){
		// At@`
		strPData << GetPovMacroSphereVariousAlpha
		    (i, pClrMat, bConstantRadiusInSphere[i], radiusFactorList[i],
		     0.156f*0.156f, pCGLAttrib);
	    }
	    else {
		// FŒ
		strPData << GetPovMacroSphereConstColor
		    (i, pClrMat, bConstantRadiusInSphere[i], radiusFactorList[i], 
		     0.156f*0.156f, pCGLAttrib);
	    }
	}

	wxString strInteriorOptions = 
	    pCGLAttrib->mCTexture.GetStrPovInteriorOptions();
	if (strInteriorOptions.IsEmpty() == FALSE)
	    strPData << wxT("\tinterior {") << strInteriorOptions << wxT("}") << ENDL;
	
	double alpha = pCGLAttrib->mCTexture.GetAlpha();
	double reflectMin  = pCGLAttrib->mCTexture.mCReflectMin.mBody; // ˗
	double reflectMax  = pCGLAttrib->mCTexture.mCReflectMax.mBody;
	bool   bTarget = pCGLAttrib->mCTexture.mCbPhotonTarget.mbBody;
	bool   bCollect = pCGLAttrib->mCTexture.mCbPhotonCollect.mbBody;
	GetStrPovPhotonSetting(strPData, alpha, reflectMin, reflectMax, bTarget, bCollect);
	
	strPData << ENDLCLOSE
		 << wxT("#end") << ENDL;
    }
}
		       

void PovRayCtrl_cls::
GetStrPovParticle(wxString &strPData,
		  PDataBlock_cls *pCDataBlock, 
		  GLAttribute_cls **ppCGLAttribute, double dT){
    // ̎_ GLDrawData_cls::mspCSnapShot Lł邱ƂO
 
    SnapData_cls *pCDataSnap = GLDrawData_cls::mspCSnapShot;
    
    int step = ppCGLAttribute[0]->mspCDrawOptionLink->mCDownSize.mBody;
    int PNum = pCDataSnap->mPNumBlock;
    int downsized = GLDrawData_cls::msDownsizedAtFBO0;
    unsigned char *pType = pCDataSnap->mpTypeLink;
    float (*pPos)[3]     = pCDataSnap->mpX;
    float *pRadius       = pCDataSnap->mpRadius;
    float *pValue1Mat[_MAX_PARTICLE_TYPE];    
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++)
	pCDataSnap->SetPValue1(&pValue1Mat[i], ppCGLAttribute[i], 1);

    float steppingRadiusFactor = sqrt(1.0f + ((float)step - 1.0f) * (0.2 + 0.6*(float)step/100));

    //bool bConstantRadiusInSphere[_MAX_PARTICLE_TYPE];
    bool bConstantColorInSphere[_MAX_PARTICLE_TYPE];
    bool bConstantAlphaInSphere[_MAX_PARTICLE_TYPE];
    int   radiusTypeList[_MAX_PARTICLE_TYPE];
    float radiusFactorList[_MAX_PARTICLE_TYPE];
    //float alphaFactor[_MAX_PARTICLE_TYPE];
    int   drawTypeList[_MAX_PARTICLE_TYPE];
    for (int i = 0; i < _MAX_PARTICLE_TYPE; i++){
	bConstantColorInSphere[i] = 
	    (ppCGLAttribute[i]->mpCClrCurveMat[0]->mCbColorCurve.mbBody!=TRUE)&
	    (ppCGLAttribute[i]->mpCClrCurveMat[1]->mCbColorCurve.mbBody!=TRUE)&
	    (ppCGLAttribute[i]->mpCClrCurveMat[2]->mCbColorCurve.mbBody!=TRUE);

	bConstantAlphaInSphere[i] = TRUE;
	if (GLDrawBillboard_cls::mspCFilterCtrlBooleanLink->
	    IsBooleanChangeConstantColor() == TRUE)
	    bConstantAlphaInSphere[i] = FALSE;

	//alphaFactor[i]      = ppCGLAttribute[i]->mCAlphaFactor.mBody;
	drawTypeList[i]     = ppCGLAttribute[i]->mCDrawType.mBody;
	radiusTypeList[i]   = ppCGLAttribute[i]->mCRadiusType.mBody;
	radiusFactorList[i] = ppCGLAttribute[i]->GetRadiusFactor() * 
	    steppingRadiusFactor;
	//bConstantRadiusInSphere[i] = (radiusTypeList[i] == _GL_RADIUS_CONSTANT);
	if (drawTypeList[i] == _GL_DRAW_OBJECT ||
	    drawTypeList[i] == _GL_DRAW_PSEUDO_OBJECT){
	    bConstantColorInSphere[i] = TRUE;
	    bConstantAlphaInSphere[i] = TRUE;
	}
	if (drawTypeList[i] == _GL_DRAW_PSEUDO_SPHERE){
	    drawTypeList[i] = _GL_DRAW_SPHERE;
	}
    }
    
    strPData << wxT("#declare PARTICLES=") << ENDL
	     << wxT("union{") << ENDL;

    char *pBuf[4];
    bool bFinished[4] = {FALSE, FALSE, FALSE, FALSE};
    for (int thread = 0; thread < 4; thread++){
	pBuf[thread] = new char [PNum * 50 / 4];
	pBuf[thread][0] = '\0'; 
    }

    ThreadSPString_cls *pCThread[4];
    for (int thread = 0; thread < 4; thread++){
	int ini = thread * downsized / 4;
	int fin = (thread + 1) * downsized / 4;
	pCThread[thread] = new ThreadSPString_cls
	    (&bFinished[thread], pBuf[thread], ini, fin, 1, pType, 
	     drawTypeList, radiusTypeList, radiusFactorList, 
	     bConstantColorInSphere, bConstantAlphaInSphere,
	     pPos, pRadius, pValue1Mat);
    }

    // !! ini  fin  step 𔽉fĂȂB
    
    pCThread[0]->Run();    pCThread[1]->Run();
    pCThread[2]->Run();    pCThread[3]->Run();

    for (int thread = 0; thread < 4; thread++)
	while (bFinished[thread] == FALSE) wxMilliSleep(5);
    
    for (int thread = 0; thread < 4; thread++){
	strPData.Append(wxString::FromAscii(pBuf[thread]));
	delete [] pBuf[thread];
    }
    
    strPData << wxT("};") << ENDL;
    strPData << ENDL;
}

wxString PovRayCtrl_cls::
GetStrPovParticleObject(wxString strKey){
    wxString strPData;
    wxString strTab;
    for (int i = 0; i < mTabDepth; i++) strTab << wxT("\t");

    strPData << strTab << wxT("object {") << ENDL
	     << strTab << wxT("\t") << strKey << ENDL;
    if (mstrInstanceTranslate.IsEmpty() != TRUE)
	strPData << strTab << wxT("\t") << mstrInstanceTranslate << ENDL;
    strPData << strTab << ENDLCLOSE;
    return strPData;
}

wxString PovRayCtrl_cls::
GetStrPovGlobal(){
    wxString strGlobal;
    strGlobal << wxT("#include \"functions.inc\"") << ENDL;
    strGlobal << wxT("global_settings {") << ENDL;
    strGlobal << mpCAdvanced->GetStrPovGlobal();
    strGlobal << mpCRadiosity->GetStrPovGlobal();
    strGlobal << mpCPhotons->GetStrPovGlobal();
    strGlobal << GetTab(1) << wxT("charset utf8") << ENDL;
    strGlobal << ENDLCLOSE;

    strGlobal << wxT("background {") 
	      << MainGLCanvas_cls::mspCClrCurveBG->GetStrPov() << ENDLCLOSE;
    
    return strGlobal;
}

wxString PovRayCtrl_cls::
GetStrPovCameraAngle(float angle, bool bFishEye){
    wxString strAngle;
    strAngle << wxT("#declare CameraAngle=") << ENDL;
    if (bFishEye == TRUE) 
	strAngle << angle;
    else 
	strAngle << wxT("180 * atan2( tan (pi*") << angle << wxT("/180) * image_width / image_height, 1) / pi");
    strAngle << wxT(";") << ENDL;
    strAngle << wxT("#if (CameraAngle < 0)") << ENDL
	     << wxT("#declare CameraAngle = CameraAngle+180;") << ENDL
	     << wxT("#end") << ENDL
	     << ENDL;
    return strAngle;
}

void PovRayCtrl_cls::
GetStrPovCameraType(wxString& strCameraType, 
		    CameraStructure_cls CCamStruct, bool bBake){
    strCameraType = wxT("\tperspective");
    if (bBake == FALSE){
	if (CCamStruct.mbFishEye == TRUE) strCameraType = wxT("\tfisheye");
	if (CCamStruct.mbOrthographic == TRUE) strCameraType = wxT("\torthographic");
    }
    else
	strCameraType = wxT("\torthographic");
}

void PovRayCtrl_cls::
GetStrPovFocalBlurExpression(wxString& strFocalBlur, double apatureFactor){
    mpCFocalBlur->GetStrPovFocalBlurExpression(strFocalBlur, apatureFactor);
}

wxString PovRayCtrl_cls::
GetStrPovCamera(PositionAndDirection_cls CCameraPos, Position_cls CAnchorPos,
		float angle, bool bFishEye, bool bOrthographic,
		StereoGram_cls *pCStereogram){

    wxString strPov;
    strPov << GetStrPovCameraAngle(angle, bFishEye);

    if (mpCRender->mCAnimationType.mBody == _RENDER_FLYTHROUGH){
	mpCParentFrameLink->mpCParent->GetStrPovPathExpression(strPov);
	return strPov;
    }
    
    CameraStructure_cls CCamStruct(CCameraPos, CAnchorPos,
				   angle, bFishEye, bOrthographic);

    float ALocation[3];
    float ALookAt[3];
    float ASky[3];
    double distance = CCameraPos.GetDistanceToAnc();
    
    LOOP_XYZ{
	ALocation[xyz] = CCameraPos.mPos[xyz];
	ALookAt[xyz] = CCameraPos.mPos[xyz] + CCameraPos.mTgt[xyz] * distance * 2;
	ASky[xyz] = CCameraPos.mTop[xyz];
    }

    wxString strCameraType;
    GetStrPovCameraType(strCameraType, CCamStruct, FALSE);

    wxString strAngle;
    wxString strViewDimension;
    if (strCameraType.IsSameAs(wxT("\torthographic")) == TRUE){
	float vsize = (float)CCameraPos.GetDistanceToAnc() * 0.5;
	strViewDimension << wxT("\tright -x*image_width/image_height * " << vsize) << ENDL;
	strViewDimension << wxT("\tup y * ") << vsize;
    }
    else{
	strAngle << wxT("\tangle CameraAngle");
	strViewDimension << wxT("\tright -x*image_width/image_height");
    }

    float stereoFocus = pCStereogram->GetStereoFocus();

    if (strCameraType.IsEmpty() != TRUE) strCameraType << ENDL;
    if (strAngle.IsEmpty() != TRUE) strAngle << ENDL;
    double apatureFactor = 1;
    strPov << wxT("#declare Camera = camera {") << ENDL
	   << strCameraType 
	   << PovOut3f(wxT("\tlocation"), ALocation, 5) << ENDL
	   << PovOut3f(wxT("\tlook_at"), ALookAt, 5) << ENDL
	   << PovOut3f(wxT("\tsky"), ASky) << ENDL
	   << strAngle 
	   << strViewDimension << ENDL
	   << mpCFocalBlur->GetPovStrFocalBlur(CCameraPos, CAnchorPos, apatureFactor, stereoFocus) 
	   << ENDLCLOSE;
    return strPov;
}

wxString PovRayCtrl_cls::
GetPovRayCommand(wxString strFile){
    wxString strOption;// = POVOPTIONWX;
#ifdef WIN32
    if (mpCFormat->mCbPause.mbBody == FALSE) strOption += wxT(" /NR /EXIT ");
    wxString strCommand = 
	wxT("\"") + mCConfPathPovray.mstrBody + wxT("\" '") + strFile + wxT(".ini'") + strOption;
#else
    wxString strCommand = wxT("nice ");
    strCommand <<
	mCConfPathPovray.mstrBody + wxT(" ") + strFile + wxT(".ini") + strOption;
#endif
    return strCommand;
}

wxString PovRayCtrl_cls::
GetStrPovIni(wxString &strIni, wxString strFile, int jpegCompress){
    
    strIni << mpCFormat->GetStrPovFormat(strFile);
    
    wxString strRadiosity = mpCRadiosity->GetStrPovIni();
    if (strRadiosity.IsEmpty() == FALSE) strIni << strRadiosity;

    wxString strPhotons = mpCPhotons->GetStrPovIni();
    if (strPhotons.IsEmpty() == FALSE) strIni << strPhotons;
    
    /*
    if (mCRemoveFile.mBody == 1 && mbPovOutForRender == TRUE){
#ifdef WIN32
	mstrPostSceneCommand << L"del " << strInputFile << wxT(".pov\";");
	mstrPostSceneCommand << L"del " << strInputFile << wxT(".ini\";");
#else
	mstrPostSceneCommand << L"rm " << strInputFile << wxT(".pov\";");
	mstrPostSceneCommand << L"rm " << strInputFile << wxT(".ini\";");
#endif
    }
    */

    if (mpCRender->mCAnimationType.mBody == _RENDER_FLYTHROUGH){
	int frameMax = GetPCTimeCtrl()->mCMaxFrame.mBody;
	int ini, fin;
	mpCRender->GetRenderRegion(ini, fin);
	
	strIni << wxT("Initial_Frame=0") << ENDL
	       << wxT("Final_Frame=") << frameMax << ENDL
	       << wxT("Subset_Start_Frame=") << ini << ENDL
	       << wxT("Subset_End_Frame=") << fin << ENDL;
    }
    
    return strIni;
}

wxString PovRayCtrl_cls::
GetStrRelativePath(wxString strPath){
    if (mCbAbsolutePath.mbBody == TRUE) return strPath;

    wxFileName CPath(strPath);
    CPath.MakeRelativeTo(mCConfWorkingDir.mstrBody);
    wxString strRelative = CPath.GetFullPath();
    return strRelative;
}

bool PovRayCtrl_cls::
OutputPovRayIni(wxString strFile){
    wxFile CFile(strFile, wxFile::write);
    if (CFile.IsOpened() == FALSE) return FALSE;

    wxString strPovIni;
    GetStrPovIni(strPovIni, strFile, 95);
    
    CFile.Write(strPovIni, wxConvFile);
    return TRUE;
}

bool PovRayCtrl_cls::
RenderByPovSnap(unsigned int stereo){
    if (mbPovRayThreadFinished == FALSE) return FALSE;

    wxString strFile = GetFileStr(_POV_SNAP, stereo, -1);
    wxString strCommand = GetPovRayCommand(strFile);
    
    mpCThreadPovRay = new ThreadPovRay_cls;
    mpCThreadPovRay->SetParameters
	(this, strCommand, wxT(""), &mbPovRayThreadFinished);
    mbPovRayThreadFinished = FALSE;
    mpCThreadPovRay->Run();

    mpCGUI->UpdateButtonLabel();
    
    return TRUE;
}

bool PovRayCtrl_cls::
StartExportPovAnim(int type){
    mpCGUI->UpdateFromText();

    mAnimNumber = mpCRender->mCAnimInit.mBody;
    mCommonPDataStatus = 0;
    mCommonBGStatus = 0;

    if (mbRenderingAnim == TRUE){
	AbortRenderPovAnim();
	return FALSE;
    }
    
    if (mbExportingAnim == TRUE){
	AbortExportPovAnim();
	return FALSE; 
    }
    
    mbExportingAnim = TRUE;
    if (type == _POV_ANIM) mbRenderingAnim = TRUE;
    mpCGUI->UpdateButtonLabel();
    mpCParentFrameLink->Disable();
    return TRUE;
}

void PovRayCtrl_cls::
AbortExportPovAnim(){
    mbExportingAnim = FALSE;
    if (mbRenderingAnim == TRUE){
	StartRenderPovAnim();
	//mbRenderingAnim = FALSE;
    }
    mpCGUI->UpdateButtonLabel();
    mpCParentFrameLink->Enable();
}

void PovRayCtrl_cls::
PushBackCommands(unsigned int stereo, unsigned int frame){
    wxString strFile = GetFileStr(_POV_ANIM, stereo, frame);
    wxString strCommand = GetPovRayCommand(strFile);
    mVstrAnimCommand.push_back(strCommand);
    //mVstrAnimPostSceneCommand.push_back(mstrPostSceneCommand);
}

void PovRayCtrl_cls::
StartRenderPovAnim(){
    mVstrAnimCommand.clear();
    bool bPause = mpCFormat->mCbPause.mbBody;
    mpCFormat->mCbPause.mbBody = FALSE;
    // ̎_mAnimNumberExportꂽŌ̐̂͂
    for (int i = mpCRender->mCAnimInit.mBody; i < mAnimNumber; i++){
	PushBackCommands(0/*stereo*/, i);
    }
    mpCFormat->mCbPause.mbBody = bPause;
    mAnimNumber = 0;
}

void PovRayCtrl_cls::
ProcessRenderPovAnim(){
    if (mbRenderingAnim == FALSE) return;
    if (mbExportingAnim == TRUE) return;
    if (mbPovRayThreadFinished == FALSE) return;

    if (mVstrAnimCommand.size() == 0) AbortRenderPovAnim();

    mpCThreadPovRay = new ThreadPovRay_cls;
    mpCThreadPovRay->SetParameters
	(this, mVstrAnimCommand[mAnimNumber], wxT(""), &mbPovRayThreadFinished);
    mbPovRayThreadFinished = FALSE;
    mpCThreadPovRay->Run();
    mAnimNumber++;
    
    if (mAnimNumber >= (int)mVstrAnimCommand.size()) AbortRenderPovAnim();
}

void PovRayCtrl_cls::
AbortRenderPovAnim(){
    mbRenderingAnim = FALSE;
    mbExportingAnim = FALSE;
    mpCGUI->UpdateButtonLabel();
}

void PovRayCtrl_cls::
ReadStrLine(wxString& strLine){
    mCConfListPovPath.ReadStrLineAll(strLine);
    mCConfListPovParam.ReadStrLineAll(strLine);
    mpCRadiosity->ReadStrLine(strLine);
    mpCRender->ReadStrLine(strLine);
    mpCPhotons->ReadStrLine(strLine);
    mpCFocalBlur->ReadStrLine(strLine);
    mpCFormat->ReadStrLine(strLine);
    mpCAdvanced->ReadStrLine(strLine);
}

void PovRayCtrl_cls::
Serialize(wxString &strConf){
    strConf << mCConfListPovPath.SerializeAll();
    strConf << mCConfListPovParam.SerializeAll();
    strConf << mpCRadiosity->Serialize();
    strConf << mpCRender->Serialize();
    strConf << mpCPhotons->Serialize();
    strConf << mpCFocalBlur->Serialize();
    strConf << mpCFormat->Serialize();
    strConf << mpCAdvanced->Serialize();
}

wxString PovRayCtrl_cls::
SerializeForConfigure(){
    wxString strConf;
    strConf << mCConfPathPovray.Serialize();
    strConf << mCConfWorkingDir.Serialize();
    return strConf;
}

PovRayCtrl_cls *PovRayCtrl_cls::mspCThis = NULL;
