#ifndef _POV_RAY_H_
#define _POV_RAY_H_

#include "clsPovSerialize.h"

#include "../MYGL/clsPositionAndDirection.h"
#include "../MYGL/clsCameraStructure.h"
#include "../WIDGETCTRL/clsDialogController.h"

#include "../COMMON/CommonDefinitions.h"

#include "../../clsFrameMain.h"

#define _POV_SNAP 0
#define _POV_ANIM 1
#define _POV_COMMON 2

#define _POV_PERSPECTIVE  0
#define _POV_ORTHOGRAPHIC 1
#define _POV_FISHEYE      2
#define _POV_CYLINDER     3

#define _POV_RENDER_WORKAREA  0
#define _POV_RENDER_FULLSCENE 1
#define _POV_RENDER_CUSTOM    2

#define _POV_FOLLOW_CAMERA     0
#define _POV_SAME_AS_PREVIEW0  1
#define _POV_SAME_AS_PREVIEW1  2
#define _POV_SAME_AS_PREVIEW2  3
#define _POV_SAME_AS_PREVIEW3  4

#define _POVRAY_36 0
#define _POVRAY_37 1

#define _POVRAY_MAX_THREAD 16

#define _POV_EXPORT 0
#define _POV_RENDER 1

#include <wx/thread.h>

class ArrayAndIndex_cls;
class FilterBase_cls;
class GLAttribute_cls;
class PDataBlock_cls;
class PovAdvanced_cls;
class PovFocalBlur_cls;
class PovFormat_cls;
class PovIsosurfaceData_cls;
class PovRayGUI_cls;
class PovRadiosity_cls;
class PovRender_cls;
class PovPhotons_cls;
class StereoGram_cls;
class ThreadPovRay_cls;
class TimeCtrl_cls;

class wxNotebook;

class ThreadSPString_cls : public wxThread {
public:
    ThreadSPString_cls(bool *pbFinishedFlagLink, char *pBuf, 
		       int init, int fin, int step, unsigned char *pType, 
		       int   drawTypeList[_MAX_PARTICLE_TYPE],
		       int   radiusTypeList[_MAX_PARTICLE_TYPE],
		       float radiusFactorList[_MAX_PARTICLE_TYPE],
		       bool  bConstantColorInSphere[_MAX_PARTICLE_TYPE],
		       bool  bConstantAlphaInSphere[_MAX_PARTICLE_TYPE],
		       float (*pPos)[3], float *pRadius, float *pValue1[_MAX_PARTICLE_TYPE]);

    virtual ExitCode Entry();
    
    bool *mpbFinishedFlagLink;
    char *mpBuf;
    int mInit, mFin, mStep;
    unsigned char *mType;
    int *mDrawTypeList;
    int *mRadiusTypeList;
    float *mRadiusFactorList;
    bool  *mbConstantColorInSphere;
    bool  *mbConstantAlphaInSphere;
    float (*mpPos)[3];
    float *mpRadius;
    float *mpValue1Mat[_MAX_PARTICLE_TYPE];
};

class PovRayCtrl_cls : public DialogControl_cls, public PovSerialize_cls
{ 
 public:
    PovRayCtrl_cls();
    ~PovRayCtrl_cls();
    
    static PovRayCtrl_cls *mspCThis;

    PovRayGUI_cls*  mpCGUI;
    FrameMain_cls*  mpCParentFrameLink;
    TimeCtrl_cls*   GetPCTimeCtrl();

    bool              mbPovRayThreadFinished;
    ThreadPovRay_cls *mpCThreadPovRay;

    std::vector < wxString > mVstrAnimCommand;

    void InitGUI(FrameMain_cls *pC);
    wxString GetDefaultWorkingDir();
    void SetToDefault();
    
    void UpdateItems(bool bForce = FALSE);
    void EnableMenuItem(bool  b0, bool b1, bool b2, bool b3);
    
    bool IsFastColor();
    int GetRegionType();
    int GetRegionInit();
    int GetRegionFin();
    void IncrementAnim();
    wxString GetFileStr(unsigned int type, unsigned int stereoId,
			int index);
    wxString GetWorkingDirWithSep(bool bForceAbs = FALSE);

    
    static void GetStrPovTexColorMapPigment
	(wxString& strPov, wxString strTab, bool bFilterAlpha,
	 GLAttribute_cls *pCGLAttrib, int index);
    static void GetStrPovTexColorMapFast
	(wxString& strPov, wxString strTab, float pClrMat[3][4], 
	 float roughness, wxString strFinishOptions, wxString strNormal,
	 GLAttribute_cls *pCGLAttrib, bool bFilterAlpha);
    static void GetStrPovTexColorMapGL
	(wxString& strPov, wxString strTab, float pClrMat[3][4], 
	 float roughness, wxString strFinishOptions, wxString strNormal,
	 GLAttribute_cls *pCGLAttrib, bool bFilterAlpha);
    static void GetStrPovTexColorMap
	(wxString& strPov, wxString strTab, float pClrMat[3][4], 
	 float roughness, wxString strFinishOptions, wxString strNormal,
	 GLAttribute_cls *pCGLAttrib, bool bFilterAlpha);
    
    
    
    static void GetStrPovPhotonSetting 
	(wxString &strPov, double alpha, double reflectMin, double reflectMax, 
	 bool bTarget, bool bCollect);
    wxString GetPovMacroSphereVariousColor
	(int index, float pClrMat[3][4], bool bConstantRadius, 
	 float radius, float roughness, GLAttribute_cls *pCGLAttrib);
    wxString GetPovMacroSphereVariousAlpha
	(int index, float pClrMat[3][4], bool bConstantRadius, 
	 float radius, float roughness, GLAttribute_cls *pCGLAttrib);
    wxString GetPovMacroSphereConstColor
	(int index, float pClrMat[3][4], bool bConstantRadius, 
	 float radius, float roughness, GLAttribute_cls *pCGLAttrib);
    
    void GetStrPovParticleMacroObject(wxString &strPData,
				      int objIndex, double Index, 
				      wxString strTranslate);
    void GetStrPovParticleMacro(wxString &strPData, double doubleIndex, 
				GLAttribute_cls **ppCGLAttribute);
    void GetStrPovParticle(wxString &strPData,
			   PDataBlock_cls *pCDataBlock, 
			   GLAttribute_cls **ppCGLAttribute,
			   double dT);
    wxString GetStrPovParticleObject(wxString strTag);
    wxString GetStrPovGlobal();
    wxString GetStrPovCameraAngle(float angle, bool bFishEye);
    void GetStrPovCameraType
	(wxString& strCameraType, CameraStructure_cls CCamStruct, 
	 bool bBake);
    void GetStrPovFocalBlurExpression(wxString& strFocalBlur, 
				      double apatureFactor);
    wxString GetStrPovCamera(PositionAndDirection_cls CCameraPos, 
			     Position_cls CAnchorPos,
			     float angle, bool bFishEye, bool bOrthographic,
			     StereoGram_cls *pCStereogram);
    wxString GetPovRayCommand(wxString strFile);
    //wxString GetStrPovIni(wxString strFile, int jpegCompress);
    wxString GetStrRelativePath(wxString strPath);
    
    wxString GetStrPovIni(wxString& strIni, wxString strFile, int jpegCompress);
    bool  OutputPovRayIni(wxString strFile);
    bool  RenderByPovSnap(unsigned int stereo);

    bool  StartExportPovAnim(int type);
    void  AbortExportPovAnim();

    void  PushBackCommands(unsigned int stereo, unsigned int frame);
    void  StartRenderPovAnim();
    void  ProcessRenderPovAnim();
    void  AbortRenderPovAnim();
    
    int  mTabDepth;
    wxString mstrInstanceTranslate;
    
    bool mbExportingAnim, mbRenderingAnim;
    

    //--

    ConfItemList_cls mCConfListPovPath;
    ConfItemSTR_cls mCConfPathPovray;
    ConfItemSTR_cls mCConfWorkingDir;
    ConfItemSTR_cls mCConfSnapFile;
    ConfItemSTR_cls mCConfAnimFile;
    
    ConfItemBOOL_cls mCbAbsolutePath;

    ConfItemList_cls mCConfListPovParam;
    ConfItemUINT_cls   mCShowTweaks;
    ConfItemBOOL_cls   mCbUseCommonSceneFile;
    
    int mSnapNumber;
    int mAnimNumber;//, mAnimNumberInit;
    int mCommonPDataStatus;
    int mCommonBGStatus;

    PovAdvanced_cls  *mpCAdvanced;
    PovFocalBlur_cls *mpCFocalBlur;
    PovFormat_cls    *mpCFormat;
    PovRadiosity_cls *mpCRadiosity;
    PovRender_cls    *mpCRender;
    PovPhotons_cls   *mpCPhotons;
    
    void ReadStrLine(wxString& strLine);
    void Serialize(wxString &strConf);
    wxString SerializeForConfigure();
};

#endif
