// -*- c -*- 
// (setq c-basic-offset 4)
// "clsPovRay.h"

#include "clsPovRayGUI.h"
#include "clsPovRay.h"
#include "OPTIONAL/clsPovAdvanced.h"
#include "OPTIONAL/clsPovFocalBlur.h"
#include "OPTIONAL/clsPovFormat.h"
#include "OPTIONAL/clsPovRadiosity.h"
#include "OPTIONAL/clsPovRender.h"
#include "OPTIONAL/clsPovPhotons.h"

#include "../WIDGETCTRL/clsAtoiSpinCtrl.h"
#include "../WIDGETCTRL/clsAtoSTextCtrl.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/EX/clsButtonEX.h"
#include "../WIDGETCTRL/EX/clsFileDialogEX.h"
extern wxFileDialogEX *GpCFileDialog;
#include "../WIDGETCTRL/WX/clsFontProvider.h"
#include "../WIDGETCTRL/WX/myArtProv.h"

#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

#include "../COMMON/CommonDefinitions.h"

#include <wx/notebook.h>

#ifdef WIN32
#define POVFILE wxT("pvengine*.exe files (pvengine.exe)|pvengine*.exe|all files|*")
#define SEPARATOR wxT("\\")
#else
#define POVFILE wxT("povray files (povray)|*povray*")
#define SEPARATOR wxT("/")
#endif

BEGIN_EVENT_TABLE(PovRayGUI_cls, wxDialog)
EVT_TEXT(txtPovRefPovray, PovRayGUI_cls::OnTextRefPov)
EVT_TEXT(txtPovRefWorkdir, PovRayGUI_cls::OnTextRefWorkdir)
EVT_BUTTON(btnPovRefPovray,  PovRayGUI_cls::OnButtonPathPovray)
EVT_BUTTON(btnPovRefWorkdir, PovRayGUI_cls::OnButtonWorkingDir)
EVT_BUTTON(btnPovExportSnap, PovRayGUI_cls::OnButtonExportSnap)
EVT_BUTTON(btnPovExportAnim, PovRayGUI_cls::OnButtonExportAnim)
EVT_BUTTON(btnPovRenderSnap, PovRayGUI_cls::OnButtonRenderSnap)
EVT_BUTTON(btnPovRenderAnim, PovRayGUI_cls::OnButtonRenderAnim)
EVT_BUTTON(btnPovDefault,    PovRayGUI_cls::OnButtonDefault)
EVT_BUTTON(btnPovAdvanced,   PovRayGUI_cls::OnButtonAdvanced)
EVT_MENU(menuPovUpdateLabel, PovRayGUI_cls::OnMenuPovUpdateLabel)
END_EVENT_TABLE()

PovRayGUI_cls::
PovRayGUI_cls(PovRayCtrl_cls *pCCtrlLink,
	      wxWindow *pCParent) :
    wxDialogEXKey(pCParent, wxID_ANY, _("Pov-Ray Rendering Configure"), 
		  wxDPS, ARTWXFRAMEICON("wxART_POVRAY"))
{
    mpCCtrlLink = pCCtrlLink;
    SetFont(FontProvider_cls::GetFont(0));
    SetContents();
}

PovRayGUI_cls::
~PovRayGUI_cls(){
}

void PovRayGUI_cls::
SetContents(){
    // TCU[
    wxBoxSizer *pBoxSizerSrc    = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *pBoxSizerConf1  = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *pBoxSizerConf2 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *pBoxSizerConf3 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *pBoxSizerConf4 = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *pBoxSizerBTMA    = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *pBoxSizerBTMB    = new wxBoxSizer(wxHORIZONTAL);

    // pX1
    pBoxSizerConf1->Add( 5, -1);
    mpCCheckPovray = new wxPanel(this, wxID_ANY, wxDP, wxSize(10,10), wxBORDER_SUNKEN);
    pBoxSizerConf1->Add(mpCCheckPovray, 0, wxALIGN_CENTER_VERTICAL);
    pBoxSizerConf1->Add( 5, -1);
    
    wxBoxSizer *pBoxSizerPath = GetPSizerStaticTextVCenter(this, _("Exe Path"));
    wxString strPovRay = mpCCtrlLink->mCConfPathPovray.mstrBody;
    mpCPathPovray = new AtoSTextCtrl_cls(this, txtPovRefPovray, wxT(""));
    mpCPathPovray->SetValue(strPovRay);
    pBoxSizerConf1->Add( pBoxSizerPath, 1, wxEXPAND);
    pBoxSizerConf1->Add( mpCPathPovray, 4, wxEXPAND);
    mpCButtonPathPovRay = new wxButtonEX
	(this, btnPovRefPovray, wxT("..."), wxDP, wxSize(25, -1));
    pBoxSizerConf1->Add( mpCButtonPathPovRay, 0, wxALIGN_CENTER_VERTICAL);

    //pX2
    pBoxSizerConf2->Add( 5, -1);
    mpCCheckWorkDir = new wxPanel(this, wxID_ANY, wxDP, wxSize(10,10), wxBORDER_SUNKEN);
    pBoxSizerConf2->Add(mpCCheckWorkDir, 0, wxALIGN_CENTER_VERTICAL);
    pBoxSizerConf2->Add( 5, -1);

    wxBoxSizer *pBoxSizerWD = GetPSizerStaticTextVCenter(this, _("WorkPath"));
    mpCPathWorkDir = new AtoSTextCtrl_cls(this, txtPovRefWorkdir, wxT(""));
    mpCPathWorkDir->SetValue(mpCCtrlLink->mCConfWorkingDir.mstrBody);
    pBoxSizerConf2->Add( pBoxSizerWD, 1, wxEXPAND);
    pBoxSizerConf2->Add( mpCPathWorkDir, 4, wxEXPAND);
    
    mpCButtonPathWorkDir = new wxButtonEX
	(this, btnPovRefWorkdir, wxT("..."), wxDP, wxSize(25, -1));
    pBoxSizerConf2->Add( mpCButtonPathWorkDir, 0, wxALIGN_CENTER_VERTICAL);
    
    //SnapFile
    wxBoxSizer *pBoxSizerSnap = GetPSizerStaticTextVCenter(this, wxT("SnapFile"));
    mpCStrSnap = new AtoSTextCtrl_cls(this, wxID_ANY, mpCCtrlLink->mCConfSnapFile.mstrBody);
    wxBoxSizer *pBoxSizerCurr = GetPSizerStaticTextVCenter(this, wxT("current"));
    //wxStaticText *pCStaticSnapNum = new wxStaticText(this, wxID_ANY, wxT("current"));
    mpCSnapNum = new AtoiSpinCtrl_cls(this, wxID_ANY, mpCCtrlLink->mSnapNumber);
    
    pBoxSizerConf3->Add( pBoxSizerSnap, 0, wxEXPAND);
    pBoxSizerConf3->Add( mpCStrSnap, 4, wxEXPAND);
    pBoxSizerConf3->Add( pBoxSizerCurr, 0, wxEXPAND);
    pBoxSizerConf3->Add( mpCSnapNum, 1, wxEXPAND);

    //AnimFile
    wxBoxSizer *pBoxSizerAnim = GetPSizerStaticTextVCenter(this, wxT("AnimFile"));
    mpCStrAnim = new AtoSTextCtrl_cls(this, wxID_ANY, mpCCtrlLink->mCConfAnimFile.mstrBody);
        
    pBoxSizerConf4->Add( pBoxSizerAnim, 0, wxEXPAND);
    pBoxSizerConf4->Add( mpCStrAnim, 4, wxEXPAND);
    
    SetMNumBook2();
    SetMNumBook1();

    // BTM
    mpCButtonExportSnap = 
	new wxButtonEX(this, btnPovExportSnap, wxT("ExportSnap"));
    mpCButtonExportAnim = 
	new wxButtonEX(this, btnPovExportAnim, wxT("ExportAnim"));
    mpCButtonRenderSnap = 
	new wxButtonEX(this, btnPovRenderSnap,   wxT("RenderSnap"));
    mpCButtonRenderAnim = 
	new wxButtonEX(this, btnPovRenderAnim,   wxT("RenderAnim"));

    mpCButtonAdv   = new wxButtonEX (this, btnPovAdvanced, 
				     _("Advanced"));
    mpCButtonDEF   = new wxButtonEX (this, btnPovDefault, 
				     _("Set to Defalt"));

    pBoxSizerBTMA->Add( mpCButtonExportSnap, 0, wxEXPAND);
    pBoxSizerBTMA->Add( mpCButtonExportAnim, 0, wxEXPAND);
    pBoxSizerBTMA->Add( mpCButtonRenderSnap, 0, wxEXPAND);
    pBoxSizerBTMA->Add( mpCButtonRenderAnim, 0, wxEXPAND);

    pBoxSizerBTMB->Add( mpCButtonAdv,    0, wxEXPAND);
    pBoxSizerBTMB->Add( -1, -1, 1, wxEXPAND);
    pBoxSizerBTMB->Add( mpCButtonDEF,    0, wxEXPAND);
    
    // SRC
    pBoxSizerSrc->Add( pBoxSizerConf1, 0, wxEXPAND);
    pBoxSizerSrc->Add( pBoxSizerConf2, 0, wxEXPAND);
    pBoxSizerSrc->Add( pBoxSizerConf3, 0, wxEXPAND);
    pBoxSizerSrc->Add( pBoxSizerConf4, 0, wxEXPAND);
    pBoxSizerSrc->Add( mpCBook2, 0, wxEXPAND);
    pBoxSizerSrc->Add( mpCBook1, 0, wxEXPAND);
    pBoxSizerSrc->Add( pBoxSizerBTMA,  0, wxEXPAND);
    pBoxSizerSrc->Add( pBoxSizerBTMB,  0, wxEXPAND);

    SetSizerAndFit(pBoxSizerSrc);
    RecalcSize();
    ResizeDialog();
}

void PovRayGUI_cls::
SetMNumBook1(){
    mpCBook1 = new wxNotebook(this, ID_BOOK_POVRAY1);

    mpCCtrlLink->mpCFormat->SetMNumFormat(mpCBook1);
    mpCCtrlLink->mpCRender->SetMNumRender(mpCBook1);

    mpCBook1->AddPage(mpCCtrlLink->mpCFormat->mpCMNum, _("Format"));
    mpCBook1->AddPage(mpCCtrlLink->mpCRender->mpCMNum, _("RenderType"));
}

void PovRayGUI_cls::
SetMNumBook2(){
    mpCBook2 = new wxNotebook(this, ID_BOOK_POVRAY2);

    mpCCtrlLink->mpCRadiosity->SetMNumRadiosity
	(mpCBook2, &mpCCtrlLink->mCConfWorkingDir.mstrBody); 
    mpCCtrlLink->mpCPhotons->SetMNumPhotons
	(mpCBook2, &mpCCtrlLink->mCConfWorkingDir.mstrBody); 
    mpCCtrlLink->mpCFocalBlur->SetMNumFocalBlur(mpCBook2); 
    
    //mpCBook2->AddPage(mpCMNumVolume, _("Volume"));
    //mpCBook2->AddPage(mpCMNumIsosurf, _("Isosurface"));
    mpCBook2->AddPage(mpCCtrlLink->mpCRadiosity->mpCMNum, _("Radiosity"));
    mpCBook2->AddPage(mpCCtrlLink->mpCPhotons->mpCMNum, _("Photons"));
    mpCBook2->AddPage(mpCCtrlLink->mpCFocalBlur->mpCMNum, _("Focal Blur"));
    
}

void PovRayGUI_cls::
RecalcSize(){
    int minSize = 0;
    for (unsigned int i = 0; i < mpCBook1->GetPageCount(); i++){
	mpCBook1->GetPage(i)->Layout();
	wxSize size = mpCBook1->GetPage(i)->GetBestSize();
	if (minSize < size.GetHeight())
	    minSize = size.GetHeight();
    }
    mpCBook1->SetSizeHints(-1, minSize + 30);

    minSize = 0;
    for (unsigned int i = 0; i < mpCBook2->GetPageCount(); i++){
	mpCBook2->GetPage(i)->Layout();
	wxSize size = mpCBook2->GetPage(i)->GetBestSize();
	if (minSize < size.GetHeight())
	    minSize = size.GetHeight();
    }
    mpCBook2->SetSizeHints(-1, minSize + 30);
}

void PovRayGUI_cls::
EnableButtons(bool  b0, bool b1, bool b2, bool b3, bool bOthers){
    mpCButtonExportSnap->Enable(b0);
    mpCButtonExportAnim->Enable(b1);
    mpCButtonRenderSnap->Enable(b2);
    mpCButtonRenderAnim->Enable(b3);
    mpCButtonAdv->Enable(bOthers);
    mpCButtonDEF->Enable(bOthers);
    mpCButtonPathPovRay->Enable(bOthers);
    mpCButtonPathWorkDir->Enable(bOthers);
    
    mpCCtrlLink->EnableMenuItem(b0, b1, b2, b3);
}

void PovRayGUI_cls::
EnableAllOptions(bool bEnable){
    mpCCtrlLink->mpCFormat   ->EnableAll(bEnable);
    mpCCtrlLink->mpCRadiosity->EnableAll(bEnable);
    mpCCtrlLink->mpCPhotons  ->EnableAll(bEnable);
    mpCCtrlLink->mpCRender   ->EnableAll(bEnable);
}

void PovRayGUI_cls::
UpdateButtonLabel(){
    bool b0 = TRUE, b1 = TRUE, b2 = TRUE, b3 = TRUE, bOthers = TRUE;

    if (mpCCtrlLink->mpCRender->mCAnimationType.mBody == _RENDER_FLYTHROUGH){
	b1 = FALSE; b3 = FALSE;
    }

    if (mpCCtrlLink->mbExportingAnim == TRUE){
	if (mpCCtrlLink->mbRenderingAnim == TRUE){
	    b0 = FALSE; b1 = FALSE; b2 = FALSE; bOthers = FALSE;
	}
	else{
	    b0 = FALSE; b2 = FALSE; b3 = FALSE; bOthers = FALSE;
	}
	EnableAllOptions(FALSE);
    }
    else{
	EnableAllOptions(TRUE);
    }
    
    EnableButtons(b0, b1, b2, b3, bOthers);

    if (mpCCtrlLink->mbRenderingAnim == TRUE){
	mpCButtonRenderAnim->SetLabel(_("Abort"));
	mpCButtonRenderAnim->SetBackgroundColour(wxColour(wxT("RED")));
	mpCButtonRenderAnim->SetForegroundColour(wxColour(wxT("WHITE")));
	if (mpCCtrlLink->mbExportingAnim == TRUE){
	    mpCButtonExportAnim->SetLabel(wxT("Exporting..."));
	}
	else{
	    mpCButtonExportAnim->SetLabel(wxT("ExportAnim"));
	}
    }
    else {
	mpCButtonRenderAnim->SetLabel(wxT("RenderAnim"));
	mpCButtonRenderAnim->SetBackgroundColour(wxNullColour);
	mpCButtonRenderAnim->SetForegroundColour(wxNullColour);
	if (mpCCtrlLink->mbExportingAnim == TRUE){
	    mpCButtonExportAnim->SetLabel(_("Abort"));
	    mpCButtonExportAnim->SetBackgroundColour(wxColour(wxT("RED")));
	    mpCButtonExportAnim->SetForegroundColour(wxColour(wxT("WHITE")));
	}
	else{
	    mpCButtonExportAnim->SetLabel(wxT("ExportAnim"));
	    mpCButtonExportAnim->SetBackgroundColour(wxNullColour);
	    mpCButtonExportAnim->SetForegroundColour(wxNullColour);
	}
    }

    if (mpCCtrlLink->mbPovRayThreadFinished == FALSE){
	mpCButtonRenderSnap->SetLabel(wxT("Rendering.."));
	mpCButtonRenderSnap->SetBackgroundColour(wxColour(wxT("RED")));
	mpCButtonRenderSnap->SetForegroundColour(wxColour(wxT("WHITE")));
    }
    else{
	mpCButtonRenderSnap->SetLabel(wxT("RenderSnap"));
	mpCButtonRenderSnap->SetBackgroundColour(wxNullColour);
	mpCButtonRenderSnap->SetForegroundColour(wxNullColour);
    }

    
}

void PovRayGUI_cls::
UpdateFromText(){
    mpCCtrlLink->mCConfPathPovray.mstrBody = mpCPathPovray->GetLineText(0);
    mpCCtrlLink->mCConfWorkingDir.mstrBody = mpCPathWorkDir->GetLineText(0);
    mpCCtrlLink->mCConfSnapFile.mstrBody = mpCStrSnap->GetLineText(0);
    mpCCtrlLink->mCConfAnimFile.mstrBody = mpCStrAnim->GetLineText(0);
    mpCCtrlLink->mSnapNumber = mpCSnapNum->GetValue();

}

bool PovRayGUI_cls::
CheckConsistency(int type){
    wxString strWarning;

    if (CheckFileExistencePov() == FALSE)
	strWarning << _("Path discrepancy.") << ENDL;
    
    if (strWarning.Len() > 0){
	strWarning << _("Above warnings detected. Continue?") << ENDL;
	return (wxMessageBox(strWarning, wxT("Warning"), wxYES_NO, this) == wxYES);
    }

    return TRUE;
}

bool PovRayGUI_cls::
CheckFileExistencePov(){
    bool bCheck = TRUE;
    wxFileName CPovRay(mpCPathPovray->GetValue());
    if (CPovRay.IsFileExecutable() == TRUE){
	mpCCheckPovray->SetBackgroundColour(*wxGREEN);
	mpCCheckPovray->SetToolTip(_("File exists."));
    }
    else{
	mpCCheckPovray->SetBackgroundColour(*wxRED);
	mpCCheckPovray->SetToolTip(_("File dose not exist."));
	bCheck = FALSE;
    }

    mpCCheckPovray->Refresh(TRUE);
    return bCheck;
}

bool PovRayGUI_cls::
IsWorkingDirOk(){
    wxString strDir = mpCPathWorkDir->GetValue();
    strDir << SEPARATOR;
    wxFileName CWorkDir(strDir);
    return CWorkDir.DirExists();
}

bool PovRayGUI_cls::
CheckFileExistenceWorkdir(){
    bool bCheck = TRUE;
    
    if (IsWorkingDirOk() == TRUE){
	mpCCheckWorkDir->SetBackgroundColour(*wxGREEN);
	mpCCheckWorkDir->SetToolTip(_("File exists."));
    }
    else{
	mpCCheckWorkDir->SetBackgroundColour(*wxRED);
	mpCCheckWorkDir->SetToolTip(_("File dose not exist."));
	bCheck = FALSE;
    }
    
    mpCCheckWorkDir->Refresh(TRUE);
    return bCheck;
}

void PovRayGUI_cls::
OnTextRefPov(wxCommandEvent& evt_menu){
    CheckFileExistencePov();
}

void PovRayGUI_cls::
OnTextRefWorkdir(wxCommandEvent& evt_menu){
    CheckFileExistenceWorkdir();
}

void PovRayGUI_cls::
OnButtonPathPovray(wxCommandEvent &event){
    GpCFileDialog->SetMessage(_("Choose path of Pov-Ray"));
    //SplitPath_cls CPath(mpCCtrlLink->mCConfPathPovray.mstrBody);
    GpCFileDialog->SetInitPathFull(mpCCtrlLink->mCConfPathPovray.mstrBody);//(CPath.GetPathStr(_FN_DRIVE_DIR));
    GpCFileDialog->SetWildcard(POVFILE);
    GpCFileDialog->SetStyle(wxFD_FILE_MUST_EXIST);
    
    if (GpCFileDialog->ShowModal() == wxID_OK){
	mpCCtrlLink->mCConfPathPovray.mstrBody = GpCFileDialog->GetPath();
	mpCPathPovray->SetValue(GpCFileDialog->GetPath());
    }

    CheckFileExistencePov();
    mpCCtrlLink->UpdateItems();
}

void PovRayGUI_cls::
OnButtonWorkingDir(wxCommandEvent &event){
    wxDirDialog *pDialog = new wxDirDialog(this, wxT("Choose Working Directory"));
    pDialog->SetPath(mpCCtrlLink->mCConfWorkingDir.mstrBody);
    if (pDialog->ShowModal() == wxID_OK){
	mpCCtrlLink->mCConfWorkingDir.mstrBody = pDialog->GetPath();
	mpCPathWorkDir->SetValue(pDialog->GetPath());
    }
    
    CheckFileExistenceWorkdir();
    mpCCtrlLink->UpdateItems();
    delete pDialog;
}

void PovRayGUI_cls::
OnButtonExportSnap(wxCommandEvent& event){
    if (CheckConsistency(_POV_EXPORT) == FALSE) return;
    SendEventToTopParent(this, wxEVT_COMMAND_MENU_SELECTED, menuPovExportSnap);
}

void PovRayGUI_cls::
OnButtonExportAnim(wxCommandEvent& event){
    if (CheckConsistency(_POV_EXPORT) == FALSE) return;
    SendEventToTopParent(this, wxEVT_COMMAND_MENU_SELECTED, menuPovExportAnim);
}

void PovRayGUI_cls::
OnButtonRenderSnap(wxCommandEvent& event){
    if (CheckConsistency(_POV_RENDER) == FALSE) return;
    SendEventToTopParent(this, wxEVT_COMMAND_MENU_SELECTED, menuPovRenderSnap);
}

void PovRayGUI_cls::
OnButtonRenderAnim(wxCommandEvent& event){
    if (CheckConsistency(_POV_RENDER) == FALSE) return;
    SendEventToTopParent(this, wxEVT_COMMAND_MENU_SELECTED, menuPovRenderAnim);
}

void PovRayGUI_cls::
OnButtonDefault(wxCommandEvent& event){
    mpCCtrlLink->SetToDefault();
    mpCCtrlLink->UpdateItems();
}

void PovRayGUI_cls::
OnButtonAdvanced(wxCommandEvent& event){
    mpCCtrlLink->mpCAdvanced->OpenDialog(this);
}

void PovRayGUI_cls::
OnMenuPovUpdateLabel(wxCommandEvent& event){
    UpdateButtonLabel();
}

