// -*- c -*-
// "clsCameraPath.h"

#include "clsCameraPath.h"

#include "../GLCANVAS/clsMainGLCanvas.h"

#include "../GLOBJECTS/clsCamera.h"
#include "../UNDO/clsUndoManager.h"

#include "../COMMON/CommonDefinitions.h"

CameraPath_cls::
CameraPath_cls() : SceneObj_cls(wxT("Camera")) {

    mObjType  = _SCENEOBJ_CAMERA;
    mObjIndex = _SCENEOBJ_IDBASE_CAMERA;

    SetInitValueTrans(-3, 0, 3);
    SetInitValueRot  (0, 45, 0);
    SetInitValueScale(1, 1, 1);
    
    mCConfList.Clear();
    mCConfList.Add(&mCCamGeometry, wxT("CameraGeometry="), wxT("0"));
    mCConfList.Add(&mCVAngle, wxT("CameraVAngle="), wxT("35"));
    mCConfList.Add(&mCFishEyeOffset, wxT("FishEyeOffset="), wxT("0"));
    mCConfList.Add(&mCbLookAnchor, wxT("CareraLookAnchor="), wxT("0"));
    mCConfList.Add(&mCbVisible, wxT("CameraVisible="), wxT("1"));
}

void CameraPath_cls::
GLDraw(bool bHittest){
    glDisable(GL_LIGHTING);
    glDisable(GL_TEXTURE_2D);
    glColor4d(0.2, 1.0, 0.2, 1);

    glPushMatrix();
    GLSetMatrix();
    if (mCCamGeometry.mBody == _GEOM_FISHEYE)
	Camera_cls::Draw(_CAMERA_FISHEYE, 0.2, mCVAngle.mBody);
    else
	Camera_cls::Draw(_CAMERA_BASIC, 0.2, mCVAngle.mBody);

    glPopMatrix();

    if (bHittest == FALSE) GLDrawPath();
}

// ----

wxString CameraPath_cls::
Serialize(){
    wxString strOut;
    strOut << wxT("---CameraPath") << ENDL;
    strOut << mCConfList.SerializeAll();
    strOut << mpCTranslate->Serialize();
    strOut << mpCRotation->Serialize();
    return strOut;
}

void CameraPath_cls::
ReadStrLine(std::wistream &istream, wxString strLineTag){
    if (strLineTag == wxT("---CameraPath")){
	std::wstring wstrLine;
	wxString strLine;
	while ( std::getline (istream, wstrLine)){
	    strLine = wstrLine.c_str();
	    if (strLine.Find(wxT("ParamGroup=")) != wxNOT_FOUND) break;
	    mCConfList.ReadStrLineAll(strLine);
	}
	mpCTranslate->ReadStrLine(istream, strLine);
	std::getline (istream, wstrLine);
	strLine = wstrLine.c_str();
	mpCRotation->ReadStrLine(istream, strLine);
    }
}

// ----

AnchorPath_cls::
AnchorPath_cls() : SceneObj_cls(wxT("Anchor")) {
    SetInitValueTrans(0, 0, 0);
    SetInitValueRot  (0, 0, 0);
    SetInitValueScale(1, 1, 1);
    mObjIndex = _SCENEOBJ_IDBASE_CAMERA + 1;

    mCConfList.Add(&mCbVisible, wxT("AnchorVisible="), wxT("1"));
}

void AnchorPath_cls::
GLDraw(bool bHittest){
    glDisable(GL_LIGHTING);
    glDisable(GL_TEXTURE_2D);
    glColor4d(0.2, 1.0, 0.2, 1);

    glPushMatrix();
    GLSetMatrix();

    if (mbSelected == TRUE || mCCurrentObj.mPos[X] != 0 || mCCurrentObj.mPos[Y] != 0 || mCCurrentObj.mPos[Z] != 0)
	Camera_cls::Draw(_CAMERA_CROSSHAIR, 0.2);
    
    glPopMatrix();

    /*
    if (mbSelected == TRUE && mbDragging == FALSE){
	glPushMatrix();
	GLDrawGuide();
	glPopMatrix();
    }
    */
    
    
    if (bHittest == FALSE) GLDrawPath();
}

wxString AnchorPath_cls::
Serialize(){
    wxString strOut;
    strOut << wxT("---AnchorPath") << ENDL; 
    //strOut << mCConfList.SerializeAll();
    //strOut << wxT("--ParamGroup") << ENDL;
    strOut << mpCTranslate->Serialize();
    return strOut;
}

void AnchorPath_cls::
ReadStrLine(std::wistream &istream, wxString strLineTag){
    if (strLineTag == wxT("---AnchorPath")){
	std::wstring wstrLine;
	wxString strLine;
	while ( std::getline (istream, wstrLine)){
	    strLine = wstrLine.c_str();
	    if (strLine.Find(wxT("ParamGroup=")) != wxNOT_FOUND) break;
	    //mCConfList.ReadStrLineAll(strLine);
	}
	mpCTranslate->ReadStrLine(istream, strLine);
    }
}
