// "clsCameraPathGUI.h"
// (setq c-basic-offset 4)

#include "clsCameraPathGUI.h"
#include "clsCameraPath.h"

#include "../GLATTRIB/clsColorCurve.h"
#include "../GLCANVAS/clsMainGLCanvas.h"
extern MainGLCanvas_cls* GpCurrentGLPanel;
#include "../GLCANVAS/NAVIGATORS/clsTagObject.h"
#include "../PANELS/clsPreference.h"
#include "../UNDO/clsUndoManager.h"

#include "../../clsFrameMain.h"
#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/WX/myArtProv.h"
#include "../WIDGETCTRL/WX/clsFontProvider.h"

#include "../COMMON/CommonDefinitions.h"

BEGIN_EVENT_TABLE(CameraPathGUI_cls, wxDialog)
EVT_NOTEBOOK_PAGE_CHANGED(ID_BOOK_VIEW, CameraPathGUI_cls::OnNotebookChanged)
EVT_BUTTON    (btnViewDefault, CameraPathGUI_cls::OnButtonViewDefault)
END_EVENT_TABLE()

CameraPathGUI_cls::
CameraPathGUI_cls(CameraPathCtrl_cls* pCCtrl, wxWindow *pC) :
wxDialogEXKey(pC, wxID_ANY, _("View and Camera Config"), 
	      wxDPS, ARTWXFRAMEICON("wxART_CAMERA")) {
    mpCCtrl = pCCtrl;
    
    wxBoxSizer *pBoxSizerSrc     = new wxBoxSizer(wxVERTICAL);
    //wxBoxSizer *pBoxSizerAdv     = new wxBoxSizer(wxVERTICAL);
    //wxBoxSizer *pBoxSizerPath    = new wxBoxSizer(wxVERTICAL);

    wxFont CFont;
    FontProvider_cls::GetBasicFont(CFont, 10);
    mpCBook1 = new wxNotebook (this, ID_BOOK_VIEW);
    mpCBook1->SetFont(CFont);

    mpCBook2 = new wxNotebook (this, ID_BOOK_VIEW_ADV);
    mpCBook2->SetFont(CFont);
    
    for (int i = 0; i < 5; i++) SetMNumView(i, mpCBook1);

    SetMNumPath(mpCBook1);
    SetMNumBG  (mpCBook2);
    SetMNumRender(mpCBook2);

    //SetMNumPath(this);

    mpCBook1->AddPage(mpCMNumView[0], wxT("View0"));
    mpCBook1->AddPage(mpCMNumView[1], wxT("View1"));
    mpCBook1->AddPage(mpCMNumView[2], wxT("View2"));
    mpCBook1->AddPage(mpCMNumView[3], wxT("View3"));
    mpCBook1->AddPage(mpCMNumView[4], wxT("Extra"));
    mpCBook1->AddPage(mpCMNumPath, wxT("Camera"));

    mpCBook2->AddPage(mpCMNumBG, wxT("BG"));
    mpCBook2->AddPage(mpCMNumRender, wxT("Render"));

    pBoxSizerSrc->Add(mpCBook1, 0, wxEXPAND);
    pBoxSizerSrc->Add(mpCBook2, 0, wxEXPAND);
    //pBoxSizerSrc->Add(mpCMNumPath, 0, wxEXPAND);

    
    SetSizerAndFit(pBoxSizerSrc);
    
    int minimumWidth = GetClientSize().GetWidth() - 8;
    mpCMNumBG    ->SetPaddingWidth(minimumWidth);
    mpCMNumRender->SetPaddingWidth(minimumWidth);

    ResizeDialog();
}

void CameraPathGUI_cls::
SetMNumView(int i, wxWindow *pCWindow){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMNumView[i], pCWindow);

    MainGLCanvas_cls *pCGLCanvasLink = 
	mpCCtrl->mpCParentFrame->GetPCGLCanvas(i);

    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_CAMCONF | _COMMAND_UPDATE_DISPLAY_GL |
	_COMMAND_UPDATE_DISPLAY_IMGSEQ;
    
    int boxWidth = 2;
#ifdef WIN32
    pCurr->mTextWidth = 90;
#else
    pCurr->mTextWidth = 100;
#endif
    pCurr->mItemWidth = 80;
    pCurr->mComboWidth = 150;
    
    pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Geometry"));

    wxString strGeometry[4] = {
	_("Perspective"), _("Orthographic"), _("Fisheye"), _("Panoramic")};
    int typeSelect = 4;
    pCurr->CombinePointerList(&pCGLCanvasLink->mCGLGeometry.mBody,
			      typeSelect, strGeometry);
    
    pCurr->CombinePointerButton(btnViewDefault, wxT("Default"));
    pCurr->GetLastItem().mAlign = _ITEM_ALIGN_RIGHT;
	
    boxWidth = 3;
    int indexTranslate = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Translate"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mPos[X]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mPos[Y]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mPos[Z]);

    int indexRotation = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Rotate"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mYPR[X]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mYPR[Y]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &pCGLCanvasLink->mCCameraPos.mYPR[Z]);

    int indexAnchor = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Pivot"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &MainGLCanvas_cls::msCAnchorPos.mPos[X]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &MainGLCanvas_cls::msCAnchorPos.mPos[Y]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &MainGLCanvas_cls::msCAnchorPos.mPos[Z]);
    
    /*
    boxWidth = 2;
    mIndexPitchAngle = pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth, 
					     _("Pitch Angle"));
    pCurr->CombinePointer
	(_GP_DOUBLE_SLIDER, &mspCParent->GetActivePath()->mpCStereoGramLinked->mCPitchAngle.mBody);
    pCurr->SetRange(-180, 180, _MN_LASTITEM, 1);
    pCurr->CombinePointer(_GP_DOUBLE, &mspCParent->GetActivePath()->mpCStereoGramLinked->mCPitchAngle.mBody);
    */
    
    pCurr->mItemWidth = -1;
    boxWidth = 2;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, 
			  _("View Angle"));
    pCurr->CombinePointer
	(_ITEM_FLOAT_SLIDER, &pCGLCanvasLink->mCGLCameraProperty.mCGLPersAngle.mBody);
    pCurr->SetRange(_ITEM_LASTITEM, 1, 180);
    pCurr->CombinePointer(_ITEM_FLOAT, &pCGLCanvasLink->mCGLCameraProperty.mCGLPersAngle.mBody);
    
    pCurr->CombinePointerSizer(3, _("DepthMIN/MAX"));
    
    pCurr->mItemWidth = 80;
    pCurr->CombinePointer
	(_ITEM_FLOAT, &pCGLCanvasLink->mCGLCameraProperty.mCGLDepthMin.mBody);
    pCurr->CombinePointer
	(_ITEM_FLOAT, &pCGLCanvasLink->mCGLCameraProperty.mCGLDepthMax.mBody);
    mIndexDepthEditor = pCurr->CombinePointerButton(wxID_ANY, _("Editor"));

    mIndexDistortion = 
	pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth);
    pCurr->CombinePointer
	(_ITEM_BOOL, &pCGLCanvasLink->mCbDistortion.mbBody, _("Distortion"));
    pCurr->CombinePointer
	(_ITEM_BOOL, &pCGLCanvasLink->mCbFisheyeGrid.mbBody, _("Fisheye Grid"));

    pCurr->FinishSetting();
    
    pCurr->SetColorForXYZ(indexTranslate+1);
    pCurr->SetColorForXYZ(indexRotation+1);
    pCurr->SetColorForXYZ(indexAnchor+1);

    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX);
    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_SHOW);
    pCurr->SetHook(this, &HookItem_cls::HookFunc02, _HOOK_COMBOBOX);
    pCurr->SetHook(this, &HookItem_cls::HookFunc03, _HOOK_BUTTON, mIndexDepthEditor, pCGLCanvasLink);
    pCurr->SetHook(this, &HookItem_cls::HookFuncAssign, _HOOK_ASSIGN);
    
    HideAndShowView(i);
}

void CameraPathGUI_cls::
SetMNumBG(wxWindow *pC){
    MultiInputPanel_cls* pCurr =
	MultiInputPanel_cls::Initialize(&mpCMNumBG, pC);

    pCurr->mItemWidth = -1;
    //pCurr->CombinePointerButton(btnPanelHide, _("[+]View Advanced Setting"));
    
#ifdef WIN32
    pCurr->mTextWidth = 90;
#else
    pCurr->mTextWidth = 100;
#endif
    pCurr->mComboWidth = 120;
    
    int boxWidth = 3;
    int indexColor = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("BG Colors"));
    ColorCurveBtn_cls *pCBtnBG = new ColorCurveBtn_cls
	(pCurr, MainGLCanvas_cls::mspCClrCurveBG, TRUE, FALSE);
    ColorCurveBtn_cls *pCBtnOut = new ColorCurveBtn_cls
	(pCurr, MainGLCanvas_cls::mspCClrCurveOut, TRUE, FALSE);
    ColorCurveBtn_cls *pCBtnTag = new ColorCurveBtn_cls
	(pCurr, TagObjectCtrl_cls::mspCClrCurveTag, TRUE, FALSE);
    
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnBG);
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnOut);
    pCurr->CombinePointer(_ITEM_WXPANEL, pCBtnTag);
    
    pCurr->mItemWidth = 80;
    pCurr->mComboWidth = 80;
    boxWidth = 3;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Grid and Width"));
    wxString strListGrid[3] = {
	_("Grid"), _("Grid+Circle"), _("Polar")};
    mIndexGrid = pCurr->CombinePointerList
	(&GpCurrentGLPanel->mCGridType.mBody, 3, strListGrid);
    
    pCurr->mItemWidth = 80;
    pCurr->CombinePointer(_ITEM_FLOAT, &MainGLCanvas_cls::msCLineWidth.mBody);
    pCurr->CombinePointer(_ITEM_FLOAT_SLIDER, &MainGLCanvas_cls::msCLineWidth.mBody);
    pCurr->SetRange(_ITEM_LASTITEM, 1, 10);
    
    //pCurr->mTextWidth = 120;
    pCurr->mItemWidth = -1;
    wxString listTagFont[5] = {wxT("HELVETICA_10"), wxT("HELVETICA_12"), wxT("HELVETICA_18"), wxT("TIMES_ROMAN_10"), wxT("TIMES_ROMAN_24")};
    mIndexTagFont = pCurr->CombinePointerList
	(&TagObjectCtrl_cls::msCTagFontIndex.mBody,
	 5, listTagFont, _("Tag Font"));

#ifdef WIN32
    pCurr->mMinimumWidth = 280;
#else
    pCurr->mMinimumWidth = 320;
#endif
    pCurr->FinishSetting();

    pCurr->GetPCwxWindow(indexColor+1)->SetToolTip(_("Background Color"));
    pCurr->GetPCwxWindow(indexColor+2)->SetToolTip(_("Mask Color"));
    pCurr->GetPCwxWindow(indexColor+3)->SetToolTip(_("Tag Color"));

    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_SHOW);
    pCurr->SetHook(this, &HookItem_cls::HookFuncAssign, _HOOK_ASSIGN);
    /*
    pCurr->SetHook(this, &HookItem_cls::HookFuncHideButton,
		   _HOOK_BUTTON, 0, pCurr);
    */
}

void CameraPathGUI_cls::
SetMNumRender(wxWindow *pC){
    MultiInputPanel_cls* pCurr =
	MultiInputPanel_cls::Initialize(&mpCMNumRender, pC);

    pCurr->mItemWidth = -1;
    //pCurr->CombinePointerButton(btnPanelHide, _("[+]View Advanced Setting"));
    
#ifdef WIN32
    pCurr->mTextWidth = 120;
#else
    pCurr->mTextWidth = 120;
#endif
    pCurr->mItemWidth = 210;
    pCurr->mComboWidth = 104;
    
    wxString strListFBO[3] = { _("Direct Drawing"), _("Use 16bit FBO as buffers"), _("Use 8bit FBO as buffers")};
    pCurr->CombinePointerList
	(&mpCCtrl->mpCParentFrame->mpCPreference->mCUseFBO.mBody, 3, strListFBO, wxT("FBO"));

    mIndexFBOSize = pCurr->CombinePointerSizer(2, _("Buffer Size"));

    wxString listFBOSize[7] = {wxT("256x256"), wxT("512x512"), wxT("1024x1024"), wxT("2048x2048"), wxT("4096x4096"), wxT("8192x8192"), wxT("16384x16384")};
    pCurr->CombinePointerList
	(&MainGLCanvas_cls::msCFBOSizeIndex.mBody,
	 7, listFBOSize, _("Buffer Size"));

    wxString listFBOSizeFE[3] = {wxT("FishEye 2K"), wxT("FishEye 4K"), wxT("FishEye 8K")};
    pCurr->CombinePointerList
	(&MainGLCanvas_cls::msCFBOSizeFishEye.mBody,
	 3, listFBOSizeFE, wxES);

    wxString listType[3] = {wxT("RGB"), wxT("RGBA"), _("Depth Buffer")};
    pCurr->CombinePointerList
	(&MainGLCanvas_cls::msCRenderType.mBody,
	 3, listType, _("RenderType"));

    wxString listBlend[2] = {wxT("accurate"), wxT("classic")};
    mIndexBlend = pCurr->CombinePointerList
	(&mpCCtrl->mpCParentFrame->mpCPreference->mCBlendType.mBody,
	 2, listBlend, _("AlphaBlend"));

#ifdef WIN32
    pCurr->mMinimumWidth = 280;
#else
    pCurr->mMinimumWidth = 320;
#endif
    pCurr->FinishSetting();

    pCurr->SetHook(this, &HookItem_cls::HookFuncAssign, _HOOK_ASSIGN);
}

void CameraPathGUI_cls::
HookFuncHideButton(int id, void *pArg){
    MultiInputPanel_cls* pCurr = ((MultiInputPanel_cls*)pArg);
    pCurr->TogglePanelOpenStatus(1);
    ResizeDialog();
}

void CameraPathGUI_cls::
SetMNumPath(wxWindow *pC){
    MultiInputPanel_cls* pCurr =
	MultiInputPanel_cls::Initialize(&mpCMNumPath, pC);

    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_CAMCONF | _COMMAND_UPDATE_DISPLAY_GL |
	_COMMAND_UPDATE_DISPLAY_IMGSEQ;

    pCurr->mItemWidth = -1;
    
    //pCurr->mDefaultButton = _MN_NO_BUTTONS;
#ifdef WIN32
    pCurr->mTextWidth = 90;
#else
    pCurr->mTextWidth = 100;
    //pCurr->mComboWidth = 108;
#endif
    pCurr->mItemWidth = 80;
    //pCurr->mItemWidth = -1;
    pCurr->mComboWidth = 100;

    pCurr->CombinePointerSizer(2, _("Camera Geom."));
    
    wxString strGeometry[4] = {
	_("Perspective"), _("Orthographic"), _("Fisheye"), _("Panoramic")};
    int typeSelect = 4;
    pCurr->CombinePointerList
	(&mpCCtrl->mpCCameraPathLinked->mCCamGeometry.mBody,
	 typeSelect, strGeometry);
    
    pCurr->mItemWidth = -1;
    pCurr->CombinePointer(_ITEM_BOOL,
			  &mpCCtrl->mpCCameraPathLinked->mCbLookAnchor.mbBody, _("Look Anchor"));
    
    
    //pCurr->mItemWidth = 80;
    mIndexAnglePath = pCurr->CombinePointerSizer(2, _("View Angle"));
    pCurr->CombinePointer
	(_ITEM_DOUBLE_SLIDER, &mpCCtrl->mpCCameraPathLinked->mCVAngle.mBody);
    pCurr->SetRange(_ITEM_LASTITEM, 1, 180);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mCVAngle.mBody);
    
    
    //pCurr->mItemWidth = 80;
    mIndexFishEyeOffset = pCurr->CombinePointerSizer(2, _("Fisheye Offset"));
    pCurr->CombinePointer
	(_ITEM_FLOAT_SLIDER, &mpCCtrl->mpCCameraPathLinked->mCFishEyeOffset.mBody);
    pCurr->SetRange(_ITEM_LASTITEM, -90, 90);
    pCurr->CombinePointer(_ITEM_FLOAT, &mpCCtrl->mpCCameraPathLinked->mCFishEyeOffset.mBody);
    
    pCurr->mItemWidth = 80;
    int boxWidth = 3;
    int indexTranslate = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Translae"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCTranslate->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCTranslate->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCTranslate->mVCParam[Z].mInterpolatedValue);

    int indexRotation = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Rotation"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCRotation->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCRotation->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCCameraPathLinked->mpCRotation->mVCParam[Z].mInterpolatedValue);

    int indexAnchor = pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Anchor"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCAnchorPathLinked->mpCTranslate->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCAnchorPathLinked->mpCTranslate->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCAnchorPathLinked->mpCTranslate->mVCParam[Z].mInterpolatedValue);
    
    /*
    boxWidth = 1;
    pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth, wxT("Point Sky"));
    mIndexPointSky = pCurr->CombinePointer
	(_GP_BOOL, &mspCParent->GetActivePath()->mCbSky.mbBody);
    //pCurr->CombinePointer(_GP_STATIC_TEXT, NULL);
    //pCurr->CombinePointer(_GP_STATIC_TEXT, NULL);
    
    boxWidth = 3;
    pCurr->CombinePointer(_GP_BOXSIZERHORIZONTAL, &boxWidth, wxT("Sky Vector"));
    mIndexSky = pCurr->CombinePointer
	(_GP_DOUBLE, &mspCParent->GetActivePath()->mCSky.mpInterpolatedValue[0]);
    pCurr->CombinePointer(_GP_DOUBLE, &mspCParent->GetActivePath()->mCSky.mpInterpolatedValue[1]);
    pCurr->CombinePointer(_GP_DOUBLE, &mspCParent->GetActivePath()->mCSky.mpInterpolatedValue[2]);
    */
    
#ifdef WIN32
    pCurr->mMinimumWidth = 280;
#else
    pCurr->mMinimumWidth = 320;
#endif
    pCurr->FinishSetting(/*pBoxSizer*/);

    pCurr->SetColorForXYZ(indexTranslate+1);
    pCurr->SetColorForXYZ(indexRotation+1);
    pCurr->SetColorForXYZ(indexAnchor+1);

    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_SHOW);
    pCurr->SetHook(this, &HookItem_cls::HookFuncAssign, _HOOK_ASSIGN);
    /*
    pCurr->SetHook(this, &HookItem_cls::HookFuncHideButton,
		   _HOOK_BUTTON, 0, pCurr);
    */
}

void CameraPathGUI_cls::
ReconnectView(){
    mpCBook1->SetSelection(GpCurrentGLPanel->mSelfIndex);

    mpCCtrl->mActiveView = GpCurrentGLPanel->mSelfIndex; 
    
    // Adṽ̕N͕Kv FIX_ME !!

    /*
    pCurr->UpdateValues();
    */

    HideAndShowAll();
}

void CameraPathGUI_cls::
ReconnectViewBG(){
    MultiInputPanel_cls*  pCurr = mpCMNumBG;
    MainGLCanvas_cls *pCGLCanvasLink = GetPCActiveGLCanvas();

    pCurr->mVCItemList[mIndexGrid].
	CombinePointer(_ITEM_UINT_LIST, &pCGLCanvasLink->mCGridType.mBody);

    pCurr->UpdateValues();
}

MainGLCanvas_cls* CameraPathGUI_cls::
GetPCActiveGLCanvas(){
    int index = mpCBook1->GetSelection();
    if (index > 4) index = 0;
    return mpCCtrl->mpCParentFrame->GetPCGLCanvas(0);
}

bool CameraPathGUI_cls::
HideAndShowView(int i){
    static int sbFishEye[5] = {-1, -1, -1, -1, -1};
    MultiInputPanel_cls* pCurr = mpCMNumView[i];
    MainGLCanvas_cls *pCGLCanvasLink = 
	mpCCtrl->mpCParentFrame->GetPCGLCanvas(i);
    
    bool bChange = FALSE;
    bool bFishEye = 
	(pCGLCanvasLink->mCGLGeometry.mBody == _GEOM_FISHEYE);
    if (sbFishEye[i] != (int)bFishEye) bChange = TRUE;
    sbFishEye[i] = (int)bFishEye;

    if (bChange == FALSE) return FALSE;
    pCurr->ShowItem(mIndexDistortion, bFishEye);
    pCurr->Layout(); 
    return TRUE;
}

bool CameraPathGUI_cls::
HideAndShowRender(){
    static unsigned int sBuffer     = 99;
    static unsigned int sRenderType = 99;
    MultiInputPanel_cls* pCurr = mpCMNumRender;

    bool bChange = FALSE;
    int typeBuffer =  mpCCtrl->mpCParentFrame->mpCPreference->mCUseFBO.mBody;
    if (typeBuffer != sBuffer) bChange = TRUE;
    sBuffer = typeBuffer;
    int typeRender = MainGLCanvas_cls::msCRenderType.mBody;
    if (typeRender != sRenderType) bChange = TRUE;
    sRenderType = typeRender;
    
    if (bChange == FALSE) return FALSE;
    pCurr->ShowItem(mIndexFBOSize, (typeBuffer > 0));
    pCurr->ShowItem(mIndexBlend, (typeRender == 1));
    pCurr->Layout();
    return TRUE;
}

void CameraPathGUI_cls::
HideAndShowAll(){
    bool bChange = FALSE;
    for (int i = 0; i < _GLCANVAS_INDEX_MAX; i++) 
	bChange |= HideAndShowView(i);
    bChange |= HideAndShowRender();

    if (bChange == FALSE) return;
    
    mpCBook1->Layout();
    mpCBook2->Layout();
    
    int minSize1 = 0;
    for (unsigned int i = 0; i < mpCBook1->GetPageCount(); i++){
	wxSize size = /*mpCMNumView[i]*/mpCBook1->GetPage(i)->GetSizer()->GetMinSize();
	if (minSize1 < size.GetHeight())
	    minSize1 = size.GetHeight();
    }
    mpCBook1->SetSizeHints(-1, minSize1 + 32);

    int minSize2 = 0;
    for (unsigned int i = 0; i < mpCBook2->GetPageCount(); i++){
	wxSize size = mpCBook2->GetPage(i)->GetSizer()->GetMinSize();
	if (minSize2 < size.GetHeight())
	    minSize2 = size.GetHeight();
    }
    mpCBook2->SetSizeHints(-1, minSize2 + 32);

    GetSizer()->Layout();
    
    ResizeDialog();
}

void CameraPathGUI_cls::
OnNotebookChanged(wxNotebookEvent& event){
    //mspCParent->mActiveView = event.GetSelection();
    ReconnectViewBG();
}

void CameraPathGUI_cls::
OnButtonViewDefault(wxCommandEvent& event){
    int i = mpCCtrl->mActiveView;
    MainGLCanvas_cls *pCGLCanvasLink = 
	mpCCtrl->mpCParentFrame->GetPCGLCanvas(i);
    int type = pCGLCanvasLink->mCDisplayMode.mBody;
    switch(type){
    case 0:{
	pCGLCanvasLink->SetCameraToDefaultPos();
	pCGLCanvasLink->mCCameraPos.UpdateYPR();
	break; }
    }
    UpdateValuesAll();
    
    SendEventToTopParent
	(mpCMNumView[0], wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshGL);
}

void CameraPathGUI_cls::
HookFunc01(int id, void *pArg){
    ResizeDialog();
}

void CameraPathGUI_cls::
HookFunc02(int id, void *pArg){
    SendEventToTopParent
	(mpCMNumView[0], wxEVT_COMMAND_MENU_SELECTED, menuEditSyncGLCtrl);
}

void CameraPathGUI_cls::
HookFunc03(int id, void *pArg){
    ((MainGLCanvas_cls*)pArg)->OpenExpressionDialog();
}

void CameraPathGUI_cls::
HookFuncAssign(int id, void *pArg){
    mpCCtrl->mpCCameraPathLinked->mpCTranslate->SetKeyFrameIfChanged();
    mpCCtrl->mpCCameraPathLinked->mpCRotation->SetKeyFrameIfChanged();
    mpCCtrl->mpCAnchorPathLinked->mpCTranslate->SetKeyFrameIfChanged();
    
    SendEventToTopParent
	(mpCMNumView[0], wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshGL);
}

void CameraPathGUI_cls::
HookFuncHS(int id, void *pArg){
    HideAndShowAll();
}

void CameraPathGUI_cls::
UpdateValuesAll(){
    HideAndShowAll();
    for (int i = 0; i < _GLCANVAS_INDEX_MAX; i++){
	mpCMNumView[i]->UpdateValues();
    }
    mpCMNumBG    ->UpdateValues();
    mpCMNumRender->UpdateValues();
    mpCMNumPath  ->UpdateValues();
}

// --------

CameraPathCtrl_cls::
CameraPathCtrl_cls(CameraPath_cls* pCCameraPath,
		   AnchorPath_cls* pCAnchorPath){
    mpCGUI = NULL;
    mpCCameraPathLinked = pCCameraPath;
    mpCAnchorPathLinked = pCAnchorPath;
    mActiveView = 0;

    //InitGUI(pCParentFrame);
}

void CameraPathCtrl_cls::
InitGUI(FrameMain_cls *pCParentFrameLink){
    mpCParentFrame = pCParentFrameLink;
    mpCGUI = new CameraPathGUI_cls(this, pCParentFrameLink);
    SetPDialog(mpCGUI);
}

void CameraPathCtrl_cls::
UpdateItems(bool bForce){
    if (mpCGUI == NULL) return;

    mpCGUI->UpdateValuesAll();
}


