// "clsLightSourceGUI.h"

#include "clsLightSourceGUI.h"
#include "clsLightSource.h"

#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemConstParam.h"

#include "../GLATTRIB/clsGLAttribute.h"
#include "../GLATTRIB/clsShaderCtrl.h"
#include "../MAINCANVAS/clsParamGUICtrl.h"
#include "../PARAMETER/clsParamGroupGUI.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/clsIconSelector.h"
#include "../WIDGETCTRL/WX/myArtProv.h"

#include "../COMMON/CommonDefinitions.h"

#include "../../clsFrameMain.h"
#include "../../clsMain.h"
#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

BEGIN_EVENT_TABLE(LightSRCGUI_cls, wxDialog)
EVT_BUTTON(btnLightAdd,    LightSRCGUI_cls::OnButtonAdd)
EVT_BUTTON(btnLightRemove, LightSRCGUI_cls::OnButtonRemove)
END_EVENT_TABLE()

LightSRCGUI_cls::
LightSRCGUI_cls(LightSRCCtrl_cls* pCCtrl,
		wxWindow *pC, wxPoint pos, wxSize size) :
wxDialogEXKey(pC, wxID_ANY, _("Light Source Editor"), 
	      pos, size, ARTWXFRAMEICON("wxART_LIGHT")) {

    mpCCtrl = pCCtrl;

    wxIcon iconPointLight, iconSpotLight;
    iconPointLight.CopyFromBitmap(wxBitmap(wxT("ICONS/LightSource.png"), wxBITMAP_TYPE_PNG));
    iconSpotLight.CopyFromBitmap(wxBitmap(wxT("ICONS/SpotLight.png"), wxBITMAP_TYPE_PNG));
    mVLightTypeIcon.resize(2);
    mVLightTypeIcon[0] = iconPointLight;
    mVLightTypeIcon[1] = iconSpotLight;

    wxBoxSizer *pBoxSizerSrc   = new wxBoxSizer(wxVERTICAL);
    SetMNumMain();
    SetMNumParam();
    pBoxSizerSrc->Add(mpCMInputMain, 0, wxEXPAND);
    pBoxSizerSrc->Add(mpCMInputLight, 0, wxEXPAND);
    
    SetSizerAndFit(pBoxSizerSrc);
}

void LightSRCGUI_cls::
SetMNumMain(){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputMain, this);
    
    int boxWidth = 3;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth);

    pCurr->mItemWidth = -1;
    mIndexIcon = pCurr->CombinePointer
	(_ITEM_UINT_ICON, &mpCCtrl->mActiveItem);
   
    pCurr->mVCItemList[mIndexIcon].mpVIcon = &mVIcon;
    pCurr->mVCItemList[mIndexIcon].SetRange(0, mpCCtrl->mCLightNum.mBody, 1);

    pCurr->mItemWidth = 24;
    mIndexAdd    = pCurr->CombinePointerButton(btnLightAdd, wxT("+"));
    mIndexDelete = pCurr->CombinePointerButton(btnLightRemove, wxT("-"));

    pCurr->FinishSetting(NULL);

    Connect(mpCMInputMain->mVCItemList[mIndexIcon].mId, 
	    wxEVT_COMMAND_COMBOBOX_SELECTED,
	    wxCommandEventHandler(LightSRCGUI_cls::OnSelector));
}

void LightSRCGUI_cls::
SetMNumParam(){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputLight, this);
    
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_CAMCONF | _COMMAND_UPDATE_DISPLAY_GL |
	_COMMAND_UPDATE_DISPLAY_LIGHT; 

#ifdef WIN32
    pCurr->mTextWidth = 90;
    pCurr->mComboWidth = -1;
#else
    pCurr->mTextWidth = 100;
    pCurr->mComboWidth = 120;
#endif

    pCurr->CombinePointerSizer(2, _("Basic Option"));
    
    pCurr->mItemWidth = 100;
    mIndexEnable = pCurr->CombinePointer
	(_ITEM_BOOL, &mpCCtrl->mpCLightSRC[0]->mCbEnable.mbBody, _("Valid"));
    
    mIndexShadow = pCurr->CombinePointer
	(_ITEM_BOOL, &mpCCtrl->mpCLightSRC[0]->mCbCastShadow.mbBody, _("Cast Shadow"));
    
    pCurr->mItemWidth = -1;
    int boxWidth = 2;
    mIndexType = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Type/Fade"));
    
    wxString strListType[4] = {
	_("Point Light"), _("Spot Light"), _("Cylinder Light"), _("Parallel Light") };

    pCurr->CombinePointerList
	(&mpCCtrl->mpCLightSRC[0]->mCLightType.mBody, 4, strListType);
    wxString strListFade[3] = { 
	_("No Fade"), _("power 1"), _("power 2")};
    mIndexFadeType = pCurr->CombinePointerList
	(&mpCCtrl->mpCLightSRC[0]->mCFadeType.mBody, 3, strListFade);

    pCurr->mItemWidth = -1;
    wxString strListTarget[3] = {
	_("No Target"), _("Target Origin"), _("Target Anchor") };
    mIndexTarget = pCurr->CombinePointerList
	(&mpCCtrl->mpCLightSRC[0]->mCTarget.mBody,
	 3, strListTarget, _("Target"));
    
    pCurr->mItemWidth = 80;
    boxWidth = 3;
    mIndexTranslate = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Translate"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCTranslate->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCTranslate->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCTranslate->mVCParam[Z].mInterpolatedValue);

    mIndexRotate = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Rotate"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCRotation->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCRotation->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCRotation->mVCParam[Z].mInterpolatedValue);

    mIndexScale = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, wxT("Scale"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCScale->mVCParam[X].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCScale->mVCParam[Y].mInterpolatedValue);
    pCurr->CombinePointer(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCScale->mVCParam[Z].mInterpolatedValue);

    pCurr->mItemWidth = -1;
    boxWidth = 2;
    mIndexStrength =
	pCurr->SetInputAndSlider(_ITEM_DOUBLE, &mpCCtrl->mpCLightSRC[0]->mpCLightStrength->mVCParam[0].mInterpolatedValue, _("L. Strength"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 200, 100);

    mIndexFadeDistance = 
	pCurr->SetInputAndSlider(_ITEM_FLOAT, &mpCCtrl->mpCLightSRC[0]->mCFadeDistance.mBody, _("Fade Distance"));
    pCurr->SetRange(_ITEM_LASTITEM, 1, 1000, 100);
    
    mIndexSpotAngle = 
	pCurr->SetInputAndSlider(_ITEM_INTEGER, &mpCCtrl->mpCLightSRC[0]->mCSpotAngle.mBody, _("Spot Angle"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 180, 1);
    
    
    mIndexArea = pCurr->CombinePointer
	(_ITEM_BOOL, &mpCCtrl->mpCLightSRC[0]->mCbAreaLight.mbBody, _("Area Light"));
    mIndexAreaParam = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Size/Quality"));
    pCurr->CombinePointer
	(_ITEM_FLOAT, &mpCCtrl->mpCLightSRC[0]->mCAreaSize.mBody);
    pCurr->CombinePointer
	(_ITEM_INTEGER, &mpCCtrl->mpCLightSRC[0]->mCAreaQuality.mBody);
    
    mIndexDrawObject  = pCurr->CombinePointer
	(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Draw Object"));
    pCurr->CombinePointer
	(_ITEM_BOOL, &mpCCtrl->mpCLightSRC[0]->mCbDrawObject.mbBody/*, _("Draw Object")*/);
    wxString strListObjType[2] = { _("Sphere"), _("Billboard") };
    pCurr->CombinePointerList
	(&mpCCtrl->mpCLightSRC[0]->mCObjType.mBody,
	 2, strListObjType, _("Type"));

    pCurr->FinishSetting(NULL);

    pCurr->SetColorForXYZ(mIndexTranslate+1);
    pCurr->SetColorForXYZ(mIndexRotate+1);
    pCurr->SetColorForXYZ(mIndexScale+1);

    pCurr->SetHook(this, &HookItem_cls::HookFuncAssign, _HOOK_ASSIGN);
    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX, mIndexType + 1);
    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX, mIndexType + 2);
    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_CHECKBOX, mIndexArea);
    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_CHECKBOX, mIndexDrawObject+1);

    RemakeIconList();
}

void LightSRCGUI_cls::
ReconnectParam(){
    MultiInputPanel_cls* pCurr = mpCMInputLight;

    if (mpCCtrl->mCLightNum.mBody == 0) {
	// !! _FIX_ME_ !!
	pCurr->Disable();
	return;
    }

    if (pCurr->IsEnabled() == FALSE)
	pCurr->Enable();
    
    int i = mpCCtrl->mActiveItem;
    LightSRC_cls *pCLight = mpCCtrl->mpCLightSRC[i];

    if (pCurr->mVCItemList[mIndexEnable].mpValue == &pCLight->mCbEnable.mbBody)
	return;
    
    pCurr->mVCItemList[mIndexEnable].CombinePointer
	(_ITEM_BOOL, &pCLight->mCbEnable.mbBody);
    
    pCurr->mVCItemList[mIndexType+1].CombinePointer
	(_ITEM_UINT_LIST, &pCLight->mCLightType.mBody);
    pCurr->mVCItemList[mIndexType+2].CombinePointer
	(_ITEM_UINT_LIST, &pCLight->mCFadeType.mBody);
	
    pCurr->mVCItemList[mIndexTarget].CombinePointer
	(_ITEM_UINT_LIST, &pCLight->mCTarget.mBody);

    LOOP_XYZ{
	pCurr->mVCItemList[mIndexTranslate+1+xyz].CombinePointer
	    (_ITEM_DOUBLE, &pCLight->mpCTranslate->mVCParam[xyz].mInterpolatedValue);
    }

    LOOP_XYZ{
	pCurr->mVCItemList[mIndexRotate+1+xyz].CombinePointer
	    (_ITEM_DOUBLE, &pCLight->mpCRotation->mVCParam[xyz].mInterpolatedValue);
    }

    LOOP_XYZ{
	pCurr->mVCItemList[mIndexScale+1+xyz].CombinePointer
	    (_ITEM_DOUBLE, &pCLight->mpCScale->mVCParam[xyz].mInterpolatedValue);
    }
    
    pCurr->ReconnectInputAndSlider(mIndexStrength, _ITEM_DOUBLE, &pCLight->mpCLightStrength->mVCParam[0].mInterpolatedValue);
    pCurr->ReconnectInputAndSlider(mIndexFadeDistance, _ITEM_FLOAT, &pCLight->mCFadeDistance.mBody);
    pCurr->ReconnectInputAndSlider(mIndexSpotAngle, _ITEM_INTEGER, &pCLight->mCSpotAngle.mBody);
    
    pCurr->mVCItemList[mIndexShadow].CombinePointer
	(_ITEM_BOOL, &pCLight->mCbCastShadow.mbBody);
    pCurr->mVCItemList[mIndexArea].CombinePointer
	(_ITEM_BOOL, &pCLight->mCbAreaLight.mbBody);
    pCurr->mVCItemList[mIndexAreaParam+1].CombinePointer
	(_ITEM_FLOAT, &pCLight->mCAreaSize.mBody);
    pCurr->mVCItemList[mIndexAreaParam+2].CombinePointer
	(_ITEM_INTEGER, &pCLight->mCAreaQuality.mBody);

    pCurr->mVCItemList[mIndexDrawObject+1].CombinePointer
	(_ITEM_BOOL, &pCLight->mCbDrawObject.mbBody);
    pCurr->mVCItemList[mIndexDrawObject+2].CombinePointer
	(_ITEM_UINT_LIST, &pCLight->mCObjType.mBody);
    
    HideAndShowAll();
    pCurr->UpdateValues();
}

void LightSRCGUI_cls::
HideAndShowAll(){
    MultiInputPanel_cls *pCurr = mpCMInputLight;

    if (mpCCtrl->mCLightNum.mBody == 0) return;

    int i = mpCCtrl->mActiveItem;
    LightSRC_cls *pCLight = mpCCtrl->mpCLightSRC[i];

    bool bArea = (pCLight->mCbAreaLight.mbBody == TRUE);
    pCurr->ShowItem(mIndexAreaParam, bArea);

    bool bSpot = (pCLight->mCLightType.mBody == _LIGHT_SPOT);
    pCurr->ShowItem(mIndexSpotAngle, bSpot);

    bool bFade = (pCLight->mCFadeType.mBody != _LIGHT_FADE_OFF) &
	((pCLight->mCLightType.mBody == _LIGHT_POINT) ||
	 (pCLight->mCLightType.mBody == _LIGHT_SPOT));
    pCurr->ShowItem(mIndexFadeDistance, bFade);

    bool bObj = (pCLight->mCLightType.mBody == _LIGHT_POINT ||
		 pCLight->mCLightType.mBody == _LIGHT_SPOT);
    pCurr->ShowItem(mIndexDrawObject, bObj);
    pCurr->EnableItem(mIndexFadeType, bObj);

    bool bObjType = bObj & pCLight->mCbDrawObject.mbBody ;
    pCurr->ShowItem(mIndexDrawObject+2, bObjType);
    pCurr->ShowItem(mIndexScale, bObjType);
    
    pCurr->GetSizer()->Layout();
    ResizeDialog();
}

void LightSRCGUI_cls::
RemakeIconList(int selected){
    mVIcon.clear();
    for (int i = 0; i< mpCCtrl->mCLightNum.mBody; i++){
	if (mpCCtrl->mpCLightSRC[i]->mCLightType.mBody == _LIGHT_SPOT)
	    mVIcon.push_back(mVLightTypeIcon[1]);
	else
	    mVIcon.push_back(mVLightTypeIcon[0]);
    }

    if (selected > 0)
	((IconSelector_cls*)mpCMInputMain->mVCItemList[mIndexIcon].mpCCtrl)
	    ->SetValueInt(selected);
    if (selected == -2)
	((IconSelector_cls*)mpCMInputMain->mVCItemList[mIndexIcon].mpCCtrl)
	    ->SetValueInt(mpCCtrl->mActiveItem);
    ((IconSelector_cls*)mpCMInputMain->mVCItemList[mIndexIcon].mpCCtrl)
	->RemakeImageList(&mVIcon);
}

void LightSRCGUI_cls::
OnButtonAdd(wxCommandEvent &event){
    mpCCtrl->AddLight();
    RemakeIconList(mpCCtrl->mActiveItem);
}

void LightSRCGUI_cls::
OnButtonRemove(wxCommandEvent &event){
    if (mpCCtrl->mCLightNum.mBody == 0) return;
    mpCCtrl->RemoveLight();
    RemakeIconList();
}

void LightSRCGUI_cls::
OnSelector(wxCommandEvent& event){
    ReconnectParam();
    int i = mpCCtrl->mActiveItem;
    mpCCtrl->mpCParentFrameLink->mpCParent->
	SelectObject(mpCCtrl->mpCLightSRC[i]->mObjIndex);
}

void LightSRCGUI_cls::
HookFuncAssign(int id, void *pArg){
    int i = mpCCtrl->mActiveItem;
    mpCCtrl->mpCLightSRC[i]->mpCTranslate->    SetKeyFrameIfChanged();
    mpCCtrl->mpCLightSRC[i]->mpCRotation->     SetKeyFrameIfChanged();
    mpCCtrl->mpCLightSRC[i]->mpCScale->        SetKeyFrameIfChanged();
    mpCCtrl->mpCLightSRC[i]->mpCLightStrength->SetKeyFrameIfChanged();
    
    SendEventToTopParent(mpCMInputLight, wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshGL);
}

void LightSRCGUI_cls::
HookFuncHS(int id, void *pArg){
    HideAndShowAll();
    RemakeIconList();
}

// ------

LightSRCCtrl_cls::
LightSRCCtrl_cls(){

    mpCGUI = NULL;

    mCConfList.Clear();
    mCConfList.Add(&mCLightNum, wxT("LightNumber="), wxT("2"));
    mActiveItem = 0;
    mUpdateBit = _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_LIGHT;

    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	wxString strTag;
	strTag << wxT("LightSRC") << i;
	mpCLightSRC[i]   = new LightSRC_cls(i, strTag);
	mpCParamGroup[i] = new ParamGroup_cls(strTag, 0, NULL, 0);
	mpCParamGroup[i]->mVpCParamGroup.push_back(mpCLightSRC[i]->mpCTranslate);
	mpCParamGroup[i]->mVpCParamGroup.push_back(mpCLightSRC[i]->mpCRotation);
	mpCParamGroup[i]->mVpCParamGroup.push_back(mpCLightSRC[i]->mpCLightStrength);
	mpCParamGroup[i]->mpCSceneObjLink = mpCLightSRC[i];
	if (i >= mCLightNum.mBody)
	    mpCParamGroup[i]->mbShownOnTL = FALSE;
    }
    
    //InitGUI(pCParentFrame);
}

void LightSRCCtrl_cls::
InitGUI(FrameMain_cls *pCParentFrameLink){
    mpCParentFrameLink = pCParentFrameLink;
    mpCGUI = new LightSRCGUI_cls(this, pCParentFrameLink,
				 wxDefaultPosition, wxDefaultSize);
    SetPDialog(mpCGUI);
}

LightSRCCtrl_cls::
~LightSRCCtrl_cls(){
    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	delete mpCLightSRC[i];
	delete mpCParamGroup[i];
    }
}

void LightSRCCtrl_cls::
UpdateTLShown(){
    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	if (i < mCLightNum.mBody){
	    mpCParamGroup[i]->SetShownOnTLRecursive(TRUE);
	    //mpCParamGroup[i]->mbShownOnTL = TRUE;
	}
	else{
	    mpCParamGroup[i]->SetShownOnTLRecursive(FALSE);
	    //mpCParamGroup[i]->mbShownOnTL = FALSE;
	}
    }
    mpCParamGroup[0]->HideAndShowOnTL();
}

void LightSRCCtrl_cls::
SelectLight(int index){
    mActiveItem = index;
    mpCGUI->ReconnectParam();
}

void LightSRCCtrl_cls::
AddLight(){
    if (mCLightNum.mBody < _LIGHT_NUM_MAX){
	unsigned int oldValue = mCLightNum.mBody;
	unsigned int newValue = mCLightNum.mBody + 1;
	GCUndoList.OpenBlock();
	GCUndoList.PushNewCommand
	    (new CommandSetValue_cls(&mCLightNum.mBody, newValue, mUpdateBit),
	     new CommandSetValue_cls(&mCLightNum.mBody, oldValue, mUpdateBit));
	GCUndoList.ExecCurrent();
	GCUndoList.PushNewCommand
	    (new CommandSetValue_cls(&mActiveItem, newValue - 1, mUpdateBit),
	     new CommandSetValue_cls(&mActiveItem, mActiveItem , mUpdateBit));
	GCUndoList.ExecCurrent();
	GCUndoList.CloseBlock();
    }
    mpCGUI->ReconnectParam();
    UpdateTLShown();
}

void LightSRCCtrl_cls::
RemoveLight(){
    GCUndoList.OpenBlock();
    for (int i = mActiveItem; i < _LIGHT_NUM_MAX - 1; i++)
	mpCLightSRC[i]->CopyFromP(mpCLightSRC[i+1]);
    unsigned int oldValue = mCLightNum.mBody;
    unsigned int newValue = mCLightNum.mBody - 1;
    GCUndoList.PushNewCommand
	    (new CommandSetValue_cls(&mCLightNum.mBody, newValue, mUpdateBit),
	     new CommandSetValue_cls(&mCLightNum.mBody, oldValue, mUpdateBit));
    GCUndoList.ExecCurrent();
    if (mActiveItem == newValue){
	GCUndoList.PushNewCommand
	    (new CommandSetValue_cls(&mActiveItem, newValue-1, mUpdateBit),
	     new CommandSetValue_cls(&mActiveItem, newValue, mUpdateBit));
	GCUndoList.ExecCurrent();
    }
    GCUndoList.CloseBlock();
    mpCGUI->ReconnectParam();
    UpdateTLShown();
}

GLuint LightSRCCtrl_cls::
GetLightType(int lightIndex){
    GLuint GLLightType;
    switch(lightIndex){
    case 0: GLLightType = GL_LIGHT0; break;
    case 1: GLLightType = GL_LIGHT1; break;
    case 2: GLLightType = GL_LIGHT2; break;
    case 3: GLLightType = GL_LIGHT3; break;
    case 4: GLLightType = GL_LIGHT4; break;
    case 5: GLLightType = GL_LIGHT5; break;
    case 6: GLLightType = GL_LIGHT6; break;
    }
    return GLLightType;
}

int LightSRCCtrl_cls::
GetNumberLightEnabled(){
    int num = 0;
    for (int i = 0; i < mCLightNum.mBody; i++)
	if (mpCLightSRC[i]->mCbEnable.mbBody) num++;
    return num;
}

int LightSRCCtrl_cls::
GetNumberLightWithShadow(){
    int num = 0;
    for (int i = 0; i < mCLightNum.mBody; i++)
	if (mpCLightSRC[i]->mCbCastShadow.mbBody == TRUE &&
	    mpCLightSRC[i]->mCbEnable.mbBody) num++;
    return num;
}

int LightSRCCtrl_cls::
GetFirstShadowedLight(){
    int index = 0;
    for (int i = 0; i < mCLightNum.mBody; i++)
	if (mpCLightSRC[i]->mCbCastShadow.mbBody == TRUE &&
	    mpCLightSRC[i]->mCbEnable.mbBody) return i;
    return index;
}

int LightSRCCtrl_cls::
GetFirstValidLight(){
    for (int i = 0; i < mCLightNum.mBody; i++)
	if (mpCLightSRC[i]->mCbEnable.mbBody) return i;
    return -1;
}

void LightSRCCtrl_cls::
GetLightInfo(double doubleIndex, float pLightPos[4], float pLightThetaPhi[2],
	     int index, bool bSV, FilterDataBoxData_cls *pCScene){
    mpCLightSRC[index]->
	GetLightInfo(doubleIndex, pLightPos, pLightThetaPhi, bSV, pCScene);
}

void LightSRCCtrl_cls::
UpdateCurrentObj(double doubleIndex){
    for (int i = 0; i < mCLightNum.mBody; i++)
	mpCLightSRC[i]->UpdateCurrentObj(doubleIndex);
}

void LightSRCCtrl_cls::
GLDrawRealObj(){
    GLAttribute_cls::mspCShaderCtrl->EndShader(0);
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    glColor4f(1, 1, 1, 1);
    for (int i = 0; i < mCLightNum.mBody; i++)
	mpCLightSRC[i]->GLDrawRealObj();
    glPopAttrib();
}

void LightSRCCtrl_cls::
GLDraw(bool bHittest){
    for (int i = 0; i < mCLightNum.mBody; i++){
	glPushName(mpCLightSRC[i]->mObjIndex);
	bool bDirection = FALSE;
	if (mpCLightSRC[i]->mCLightType.mBody != _LIGHT_POINT)
	    bDirection = TRUE;
	mpCLightSRC[i]->GLDraw(bHittest);
	glPopName();
    }
}

void LightSRCCtrl_cls::
GLDisableAllLight(){
    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	glDisable(GetLightType(i));
	GLfloat pZero[4] = {0, 0, 0, 0};
	glLightfv(GL_LIGHT0, GL_AMBIENT , pZero);
	glLightfv(GL_LIGHT0, GL_DIFFUSE , pZero);
	glLightfv(GL_LIGHT0, GL_SPECULAR, pZero);
    }
}

void LightSRCCtrl_cls::
GLSetSingleDiffuse(int lightIndex, double doubleIndex, FilterDataBoxData_cls *pCScene){
    GLDisableAllLight();
    mpCLightSRC[lightIndex]->GLSetLightSourceComp(0, doubleIndex, FALSE, TRUE,
						  pCScene);
    ShaderCtrl_cls::msbSingleLightingMode = TRUE;
    ShaderCtrl_cls::msSingleLightingIndex = lightIndex;
}

void LightSRCCtrl_cls::
SetNonShadowLight(double doubleIndex, FilterDataBoxData_cls *pCScene){
    GLDisableAllLight();
    for (int i = 0; i < mCLightNum.mBody; i++)
	mpCLightSRC[i]->GLSetLightSourceComp(i, doubleIndex, (i == GetFirstValidLight()), TRUE, pCScene);
}

void LightSRCCtrl_cls::
SetBasicLight(double doubleIndex, int shadowMode, FilterDataBoxData_cls *pCScene){

    for (int i = 0; i < _LIGHT_NUM_MAX; i++) 
	glDisable(GetLightType(i));

    int first = GetFirstValidLight();

    if (first != -1){
	glEnable(GetLightType(first));
	
	if (mpCLightSRC[first]->mCbCastShadow.mbBody == TRUE)
	    mpCLightSRC[first]->GLSetLightSourceComp(first, doubleIndex, TRUE, FALSE, pCScene);
	else
	    mpCLightSRC[first]->GLSetLightSourceComp(first, doubleIndex, TRUE, TRUE, pCScene);
    }
    
    // e𗎂ƂȂ̌ʂ
    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	if (i == first) continue;
	mpCLightSRC[i]->GLSetLightSource
	    (doubleIndex, (i < mCLightNum.mBody) & (mpCLightSRC[i]->mCbCastShadow.mbBody == FALSE), shadowMode, pCScene);
    }
}

void LightSRCCtrl_cls::
UpdateItems(bool bForce){
    if (mpCGUI == NULL) return;
}

wxString LightSRCCtrl_cls::
GetStrPovLights(double doubleIndex, FilterDataBoxData_cls *pCScene){
    wxString strLight;
    for (int i = 0; i < mCLightNum.mBody; i++)
	strLight << mpCLightSRC[i]->GetStrPovLight(doubleIndex, pCScene);
    
    return strLight;
}

// ------

wxString LightSRCCtrl_cls::
Serialize(){
    wxString strOut;

    strOut << wxT("---AllLightSource") << ENDL;
    strOut << mCConfList.SerializeAll();

    for (int i = 0; i < mCLightNum.mBody; i++)
	strOut << mpCLightSRC[i]->Serialize();

    strOut << wxT("---AllLightSourceEnd") << ENDL;
    
    return strOut;
}

void LightSRCCtrl_cls::
ReadStrLine(std::wistream &istream, wxString strLineTag){
    if (strLineTag == wxT("---AllLightSource")){
	std::wstring wstrLine;
	wxString strLine;
	while ( std::getline (istream, wstrLine)){
	    strLine = wstrLine.c_str();
	    if (strLine.Find(wxT("---")) != wxNOT_FOUND) break;
	    mCConfList.ReadStrLineAll(strLine);
	}
	int lightIndex = 0;
	while (TRUE){
	    if (strLine == wxT("---AllLightSourceEnd")) break;
	    
	    if (strLine.Find(wxT("---LightSRC")) != wxNOT_FOUND){
		mpCLightSRC[lightIndex]->ReadStrLine(istream, strLine);
		lightIndex++;
	    }
	    std::getline(istream, wstrLine);
	    strLine = wstrLine.c_str();
	}
    }
}
