// -*- c -*-
// "clsSceneObj.h"

#include "clsSceneObj.h"

#include "../GLCANVAS/NAVIGATORS/clsNavigators.h"
#include "../GLCANVAS/clsMoveItem.h"
#include "../MYGL/clsGLGuides.h"
#include "../PARAMETER/clsParamGroupGUI.h"
#include "../POVRAY/clsPovRay.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemConstParam.h"

#include "../COMMON/CommonDefinitions.h"

SceneObj_cls::
SceneObj_cls(wxString strObjName, bool bDefaultParam, bool bAllParam){

    mbSelected = FALSE;
    mbDragging = FALSE;
    if (bAllParam == FALSE){
	mpCSceneObjParam = NULL;
	return;
    }

    mpCSceneObjParam = new ParamGroup_cls(strObjName, 0, NULL, 0);
    mpCSceneObjParam->mbGroupingGroup = TRUE;
    
    wxString listTag[3] = {wxT("x"), wxT("y"), wxT("z")};
    wxString strObjTrans    = strObjName;
    wxString strObjRotation = strObjName;
    wxString strObjScale    = strObjName;
    strObjTrans    << wxT(" Translate");
    strObjRotation << wxT(" Rotation");
    strObjScale    << wxT(" Scale");
    int updateBit = _COMMAND_UPDATE_DISPLAY_GL | _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM;
    mpCTranslate = new ParamGroup_cls(strObjTrans, 3, listTag, updateBit, FALSE);
    mpCRotation  = new ParamGroup_cls(strObjRotation, 3, listTag, updateBit, FALSE);
    mpCScale     = new ParamGroup_cls(strObjScale, 3, listTag, updateBit, FALSE);
    if (strObjName == wxT("Camera")){
	mpCRotation->mVCParam[0].mCPeriodic.mBody = 360;
	mpCRotation->mVCParam[2].mCPeriodic.mBody = 360;
    }

    if (bDefaultParam == FALSE){
	mpCSceneObjParam->mbUseTLGUI = FALSE;
	return;
    }

    mpCSceneObjParam->mVpCParamGroup.push_back(mpCTranslate);
    mpCSceneObjParam->mVpCParamGroup.push_back(mpCRotation);
    mpCSceneObjParam->mVpCParamGroup.push_back(mpCScale);
}

SceneObj_cls::
~SceneObj_cls(){
    if (mpCTranslate != NULL){
	delete mpCTranslate; mpCTranslate = NULL; }
    if (mpCRotation != NULL){
	delete mpCRotation; mpCRotation = NULL; }
    if (mpCScale != NULL){
	delete mpCScale; mpCScale = NULL;
    }
    delete mpCSceneObjParam;
}

void SceneObj_cls::
SetInitValueTrans(double x, double y, double z){
    double pTransInit[3] = {x, y, z};
    mpCTranslate->SetKeyFrame(pTransInit, TRUE);
}

void SceneObj_cls::
SetInitValueRot(double x, double y, double z){
    double pRotInit[3] = {x, y, z};
    mpCRotation->SetKeyFrame(pRotInit, TRUE);
}

void SceneObj_cls::
SetInitValueScale(double x, double y, double z){
    double pScaleInit[3] = {x, y, z};
    mpCScale->SetKeyFrame(pScaleInit, TRUE);
}

void SceneObj_cls::
CopyFromP_SceneObj(SceneObj_cls *pCFrom){
    int newValue = pCFrom->mObjType;
    int oldValue = mObjType;

    GCUndoList.PushNewCommand
	(new CommandSetValue_cls(&mObjType, newValue),
	 new CommandSetValue_cls(&mObjType, oldValue));
    GCUndoList.ExecCurrent();

    newValue = pCFrom->mCbVisible.mbBody;
    oldValue = mCbVisible.mbBody;
    GCUndoList.PushNewCommand
	(new CommandSetValue_cls(&mCbVisible.mbBody, newValue),
	 new CommandSetValue_cls(&mCbVisible.mbBody, oldValue));
    GCUndoList.ExecCurrent();

    newValue = pCFrom->mCTarget.mBody;
    oldValue = mCTarget.mBody;
    GCUndoList.PushNewCommand
	(new CommandSetValue_cls(&mCTarget.mBody, newValue),
	 new CommandSetValue_cls(&mCTarget.mBody, oldValue));
    GCUndoList.ExecCurrent();

    mpCTranslate->CopyFromP(pCFrom->mpCTranslate);
    mpCRotation ->CopyFromP(pCFrom->mpCRotation);
    mpCScale    ->CopyFromP(pCFrom->mpCScale);
}

ParamGroup_cls* SceneObj_cls::
GetPCParamGroupFromTag(wxString strTag){
    if (strTag.IsSameAs(mpCTranslate    ->mstrGroupLabel)) return mpCTranslate;
    if (strTag.IsSameAs(mpCRotation     ->mstrGroupLabel)) return mpCRotation;
    if (strTag.IsSameAs(mpCScale        ->mstrGroupLabel)) return mpCScale;
    if (strTag.IsSameAs(mpCSceneObjParam->mstrGroupLabel)) return mpCSceneObjParam;
    return NULL;
}

std::vector < double > SceneObj_cls::
GetVDirectionVector(double doubleIndex){
    std::vector < double > VDir;

    switch(mCTarget.mBody){
    case 1: {
	std::vector < double > VPos = 
	    mpCTranslate->GetExpressionValue(doubleIndex, 0);
	LOOP_XYZ VDir.push_back(VPos[xyz]);
	double distance2 = sqrt(VDir[0]*VDir[0] + VDir[1]*VDir[1] + VDir[2]*VDir[2]);
	if (distance2 == 0){ 
	    VDir[0] = 0; VDir[1] = 0; VDir[2] = -1;
	}
	else {
	    LOOP_XYZ VDir[xyz] = VDir[xyz]/distance2;
	}
	break; }
    default:
	std::vector < double > VRot = 
	    mpCRotation->GetExpressionValue(doubleIndex, 0);
	
	VDir.push_back(cos(3.14159*VRot[1]/180) * cos(3.14159*(VRot[2]/180+1)));
	VDir.push_back(cos(3.14159*VRot[1]/180) * sin(3.14159*(VRot[2]/180+1)));
	VDir.push_back(sin(3.14159*VRot[1]/180));
	
    }
    return VDir;
}

void SceneObj_cls::
RotAround(float vector[3], float rad, int axis){
    float vectorOrg[3];
    LOOP_XYZ vectorOrg[xyz] = vector[xyz];
    switch (axis){
    case X:
	vector[Y] = vectorOrg[Y] * cos(rad)  + vectorOrg[Z] * sin(rad);
	vector[Z] = -vectorOrg[Y] * sin(rad) + vectorOrg[Z] * cos(rad);
	break;
    case Y:
	vector[Z] = vectorOrg[Z] * cos(rad)  + vectorOrg[X] * sin(rad);
	vector[X] = -vectorOrg[Z] * sin(rad) + vectorOrg[X] * cos(rad);
	break;
    case Z:
	vector[X] = vectorOrg[X] * cos(rad)  + vectorOrg[Y] * sin(rad);
	vector[Y] = -vectorOrg[X] * sin(rad) + vectorOrg[Y] * cos(rad);
	break;
    }
}

void SceneObj_cls::
CalcRelativeVector(float vector[3], double doubleIndex, int rotateOrder){
    std::vector < double > VRotSceneRad =
	mpCRotation->GetExpressionValue(doubleIndex, 0);
    
    LOOP_XYZ VRotSceneRad[xyz] *= 3.14159 / 180;
    
    float vectorOrg[3];
    LOOP_XYZ vectorOrg[xyz] = vector[xyz];

    double rotZ = VRotSceneRad[Z];
    double rotY = VRotSceneRad[Y];
    double rotX = VRotSceneRad[X];
    
    switch(rotateOrder){
    case _ROTATE_EULER_XYZ:
	RotAround(vector, rotZ, Z); 
	RotAround(vector, rotY, Y); 
	RotAround(vector, rotX, X);
	break;
    case _ROTATE_EULER_XZY:
	RotAround(vector, rotY, Y); 
	RotAround(vector, rotZ, Z); 
	RotAround(vector, rotX, X);
	break;
    case _ROTATE_EULER_YZX:
	RotAround(vector, rotX, X);
	RotAround(vector, rotZ, Z);
	RotAround(vector, rotY, Y); 
	break;
    case _ROTATE_EULER_YXZ:
	RotAround(vector, rotZ, Z);
	RotAround(vector, rotX, X);
	RotAround(vector, rotY, Y); 
	break;
    case _ROTATE_EULER_ZXY:
	RotAround(vector, rotY, Y);
	RotAround(vector, rotX, X);
	RotAround(vector, rotZ, Z);

	break;
    case _ROTATE_EULER_ZYX:
	RotAround(vector, rotX, X);
	RotAround(vector, rotY, Y);
	RotAround(vector, rotZ, Z);
	break;
	
    }

    /*
    // z]
    vector[X] = vectorOrg[X] * cos(rotZ)  + vectorOrg[Y] * sin(rotZ);
    vector[Y] = -vectorOrg[X] * sin(rotZ) + vectorOrg[Y] * cos(rotZ);
    
    LOOP_XYZ vectorOrg[xyz] = vector[xyz]; 
    // y]
    vector[Z] = vectorOrg[Z] * cos(rotY)  + vectorOrg[X] * sin(rotY);
    vector[X] = -vectorOrg[Z] * sin(rotY) + vectorOrg[X] * cos(rotY);
    
    LOOP_XYZ vectorOrg[xyz] = vector[xyz];    
    // x]
    vector[Y] = vectorOrg[Y] * cos(rotX)  + vectorOrg[Z] * sin(rotX);
    vector[Z] = -vectorOrg[Y] * sin(rotX) + vectorOrg[Z] * cos(rotX);
    */
}

void SceneObj_cls::
CalcRelativePosition(float vector[3], double doubleIndex,
		     ParamGroup_cls *pCAnchor, int rotateOrder){
    std::vector < double > VTransScene =
	mpCTranslate->GetExpressionValue(doubleIndex, 0);
    std::vector < double > VScaleScene = 
	mpCScale->GetExpressionValue(doubleIndex, 0);

    if (pCAnchor != NULL){
	std::vector < double > VAnchorScene = 
	    pCAnchor->GetExpressionValue(doubleIndex, 0);
	LOOP_XYZ vector[xyz] += VAnchorScene[xyz];
	CalcRelativeVector(vector, doubleIndex);
	LOOP_XYZ vector[xyz] -= VAnchorScene[xyz];
    }
    else{
	CalcRelativeVector(vector, doubleIndex, rotateOrder);
    }

    LOOP_XYZ vector[xyz] /= VScaleScene[xyz];
    LOOP_XYZ vector[xyz] -= VTransScene[xyz];
}

void SceneObj_cls::
CalcGlobalVector(float vector[3], double doubleIndex, int rotateOrder){
    std::vector < double > VRotSceneRad =
	mpCRotation->GetExpressionValue(doubleIndex, 0);
    
    LOOP_XYZ VRotSceneRad[xyz] *= 3.14159 / 180;

    float vectorOrg[3];
    LOOP_XYZ vectorOrg[xyz] = vector[xyz];
    
    double rotX = VRotSceneRad[X];
    double rotY = VRotSceneRad[Y];
    double rotZ = VRotSceneRad[Z];

    // x]
    vector[Y] = vectorOrg[Y] * cos(rotX) - vectorOrg[Z] * sin(rotX);
    vector[Z] = vectorOrg[Y] * sin(rotX) + vectorOrg[Z] * cos(rotX);

    LOOP_XYZ vectorOrg[xyz] = vector[xyz]; 
    // y]
    vector[Z] = vectorOrg[Z] * cos(rotY) - vectorOrg[X] * sin(rotY);
    vector[X] = vectorOrg[Z] * sin(rotY) + vectorOrg[X] * cos(rotY);

    LOOP_XYZ vectorOrg[xyz] = vector[xyz]; 
    // z]
    vector[X] = vectorOrg[X] * cos(rotZ) - vectorOrg[Y] * sin(rotZ);
    vector[Y] = vectorOrg[X] * sin(rotZ) + vectorOrg[Y] * cos(rotZ);
}

void SceneObj_cls::
CalcGlobalPosition(float vector[3], double doubleIndex,
		   ParamGroup_cls *pCAnchor,
		   int rotOrder){
    std::vector < double > VTransScene =
	mpCTranslate->GetExpressionValue(doubleIndex, 0);
    std::vector < double > VScaleScene = 
	mpCScale->GetExpressionValue(doubleIndex, 0);

    LOOP_XYZ vector[xyz] += VTransScene[xyz];
    LOOP_XYZ vector[xyz] *= VScaleScene[xyz];
	
    if (pCAnchor != NULL){
	std::vector < double > VAnchorScene = 
	    pCAnchor->GetExpressionValue(doubleIndex, 0);
	LOOP_XYZ vector[xyz] -= VAnchorScene[xyz];
	CalcGlobalVector(vector, doubleIndex);
	LOOP_XYZ vector[xyz] += VAnchorScene[xyz];
    }
    else{
	CalcGlobalVector(vector, doubleIndex);
    }
}

void SceneObj_cls::
AssignSceneObjParam(){
    for (int i = 0; i < mpCSceneObjParam->mVpCParamGroup.size(); i++)
	mpCSceneObjParam->mVpCParamGroup[i]->SetKeyFrameIfChanged();
}

void SceneObj_cls::
Select(bool bSelect){
    mbSelected = bSelect;
    if (mpCSceneObjParam->mpCGUI != NULL) mpCSceneObjParam->mpCGUI->SetParamBackgroundColour();
}

void SceneObj_cls::
UpdateCurrentObj(double doubleIndex){
    std::vector < double > VPos = 
	mpCTranslate->GetExpressionValue(doubleIndex, 0);
    std::vector < double > VRot = 
	mpCRotation->GetExpressionValue(doubleIndex, 0);
    std::vector < double > VScale = 
	mpCScale->GetExpressionValue(doubleIndex, 0);

    if (mbSelected == TRUE && mbDragging == TRUE){
	LOOP_XYZ VPos[xyz] = MoveItem_cls::msSelectedItemPos[xyz];
	LOOP_XYZ VRot[xyz] += MoveItem_cls::msSelectedItemDRot[xyz];
    }
    
    Position_cls CPos(VPos[X], VPos[Y], VPos[Z]);
    mCCurrentObj.Set(CPos);
    double tgt[3] = {1, 0, 0};
    double top[3] = {0, 0, 1};
    mCCurrentObj.Setv(tgt, top);
    mCCurrentObj.NormalizeAndOrthogonalize();
    mCCurrentObj.RotateYPR(VRot[Z], 0, 0);
    mCCurrentObj.RotateYPR(0, VRot[Y], 0);
    mCCurrentObj.RotateYPR(0, 0, VRot[X]);
    mCCurrentObj.UpdateYPR();
    LOOP_XYZ mCCurrentObj.mScale[xyz] = VScale[xyz];
}

void SceneObj_cls::
Targetting(){
    switch(mCTarget.mBody){
    case 0:
	break;
    case 1:{
	double targetPos[3] = {0, 0, 0};
	mCCurrentObj.PointAt(targetPos);
	mCCurrentObj.UpdateYPR();
	LOOP_XYZ mpCRotation->mVCParam[xyz].mInterpolatedValue = 
	    mCCurrentObj.mYPR[xyz];
	mpCRotation->SetKeyFrameIfChanged();
	break; }
    }
}

void SceneObj_cls::
GLSetMatrix(){
    double matrix[16];
    mCCurrentObj.GetGLMatrix(matrix);
    glMultMatrixd(matrix);
    glScaled(mCCurrentObj.mScale[X], 
	     mCCurrentObj.mScale[Y], 
	     mCCurrentObj.mScale[Z]);
}

void SceneObj_cls::
GLSetMatrixInverse(){
    double matrix[16];
    mCCurrentObj.GetGLMatrixInvd(matrix);
    glScaled(1.0 / mCCurrentObj.mScale[X],
	     1.0 / mCCurrentObj.mScale[Y],
	     1.0 / mCCurrentObj.mScale[Z]);
    glMultMatrixd(matrix);
}


void SceneObj_cls::
GLDrawPath(){
    if (mCbVisible.mbBody == FALSE) return;
    
    std::vector < double > VPosOrg = mpCTranslate->GetInterpolatedValue(-1, 0);
    glColor4d(0.2, 1.0, 0.2, 1);
    glBegin(GL_POINTS);
    for (int i = 0; i < 100; i++){
	double doubleIndex = (double) i / 100;
	std::vector < double > VPos = 
	    mpCTranslate->GetInterpolatedValue(doubleIndex, 0);
	glVertex3dv(&VPos[0]);
    }
    glEnd();

    LOOP_XYZ
	mpCTranslate->mVCParam[xyz].mInterpolatedValue = VPosOrg[xyz];
	
}

void SceneObj_cls::
GLDrawWireframe(int type, float scale){
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glPushMatrix();
    
    GLSetMatrix();

    if (scale != 1)
	glScalef(scale, scale, scale);
    
    if (mbSelected == FALSE) glColor4f(0.3f, 0.8f, 0.3f, 1.0f);
    else                     glColor4f(0.8f, 0.7f, 0.3f, 1.0f);

    switch(type){
    case _WIREFRAME_BOX:
	glBegin(GL_LINES);
	glVertex3d(-0.5, -0.5, -0.5); glVertex3d(-0.5, -0.5, +0.5);
	glVertex3d(+0.5, -0.5, -0.5); glVertex3d(+0.5, -0.5, +0.5);
	glVertex3d(+0.5, +0.5, -0.5); glVertex3d(+0.5, +0.5, +0.5);
	glVertex3d(-0.5, +0.5, -0.5); glVertex3d(-0.5, +0.5, +0.5);
	glEnd();
	
	glBegin(GL_LINE_LOOP);
	glVertex3d(-0.5, -0.5, -0.5); glVertex3d(+0.5, -0.5, -0.5);
	glVertex3d(+0.5, +0.5, -0.5); glVertex3d(-0.5, +0.5, -0.5);
	glEnd();
	
	glBegin(GL_LINE_LOOP);
	glVertex3d(-0.5, -0.5, +0.5); glVertex3d(+0.5, -0.5, +0.5);
	glVertex3d(+0.5, +0.5, +0.5); glVertex3d(-0.5, +0.5, +0.5);
	glEnd();
	break;
    case _WIREFRAME_PLANE:
	glBegin(GL_LINE_LOOP);
	glVertex3d(-0.5, -0.5, 0); glVertex3d(+0.5, -0.5, 0);
	glVertex3d(+0.5, +0.5, 0); glVertex3d(-0.5, +0.5, 0);
	glEnd();
    }
	
    glPopMatrix();
    glPopAttrib();
}

void SceneObj_cls::
GetStrPovModelView(wxString &strModelView, double doubleIndex, std::vector < double > VAnchor, int rotateOrder){
    std::vector < double > VScale = 
	mpCScale->GetExpressionValue(doubleIndex, 0);
    std::vector < double > VTrans = 
	mpCTranslate->GetExpressionValue(doubleIndex, 0);
    std::vector < double > VRot = 
	mpCRotation->GetExpressionValue(doubleIndex, 0);
    float rotx[3] = {VRot[0], 0, 0};
    float roty[3] = {0, VRot[1], 0};
    float rotz[3] = {0, 0, VRot[2]};

    if (VScale[X] != 1.0 || VScale[Y] != 1.0 || VScale[Z] != 1.0)
	strModelView << PovOut3d(wxT("\tscale"), VScale.data(), 4) << ENDL;

    if (rotx[X] != 0 || roty[Y] != 0 || rotz[Z] != 0){
	if (VAnchor.size() != 0)
	    strModelView << PovOut3d(wxT("\ttranslate -"), VAnchor.data(), 4) << ENDL;
	wxString strRot = wxT("\trotate");
	switch(rotateOrder){
	case _ROTATE_EULER_XYZ:
	    if (rotx[X] != 0) strModelView << PovOut3f(strRot, rotx, 4) << ENDL;
	    if (roty[Y] != 0) strModelView << PovOut3f(strRot, roty, 4) << ENDL;
	    if (rotz[Z] != 0) strModelView << PovOut3f(strRot, rotz, 4) << ENDL;
	    break;
	case _ROTATE_EULER_XZY:
	    if (rotx[X] != 0) strModelView << PovOut3f(strRot, rotx, 4) << ENDL;
	    if (rotz[Z] != 0) strModelView << PovOut3f(strRot, rotz, 4) << ENDL;
	    if (roty[Y] != 0) strModelView << PovOut3f(strRot, roty, 4) << ENDL;
	    break;
	case _ROTATE_EULER_YZX:
	    if (roty[Y] != 0) strModelView << PovOut3f(strRot, roty, 4) << ENDL;
	    if (rotz[Z] != 0) strModelView << PovOut3f(strRot, rotz, 4) << ENDL;
	    if (rotx[X] != 0) strModelView << PovOut3f(strRot, rotx, 4) << ENDL;
	    break;
	case _ROTATE_EULER_YXZ:
	    if (roty[Y] != 0) strModelView << PovOut3f(strRot, roty, 4) << ENDL;
	    if (rotx[X] != 0) strModelView << PovOut3f(strRot, rotx, 4) << ENDL;
	    if (rotz[Z] != 0) strModelView << PovOut3f(strRot, rotz, 4) << ENDL;
	    break;
	case _ROTATE_EULER_ZYX:
	    if (rotz[Z] != 0) strModelView << PovOut3f(strRot, rotz, 4) << ENDL;
	    if (roty[Y] != 0) strModelView << PovOut3f(strRot, roty, 4) << ENDL;
	    if (rotx[X] != 0) strModelView << PovOut3f(strRot, rotx, 4) << ENDL;
	    break;
	case _ROTATE_EULER_ZXY:
	    if (rotz[Z] != 0) strModelView << PovOut3f(strRot, rotz, 4) << ENDL;
	    if (rotx[X] != 0) strModelView << PovOut3f(strRot, rotx, 4) << ENDL;
	    if (roty[Y] != 0) strModelView << PovOut3f(strRot, roty, 4) << ENDL;
	    break;
	}
	
	if (VAnchor.size() != 0)
		strModelView << PovOut3d(wxT("\ttranslate +"), VAnchor.data(), 4) << ENDL;
    }

    if (VTrans[X] != 0 || VTrans[Y] != 0 || VTrans[Z] != 0)
	strModelView << PovOut3d(wxT("\ttranslate"), VTrans.data(), 4) << ENDL;
}

void SceneObj_cls::
GetStrPovModelViewMatrix(wxString &strModelView){
    strModelView << GetTab(1) <<
	wxT("matrix <") << ENDL << GetTab(2) <<
	PovOutf(mCCurrentObj.mTgt[X], 4) << wxT(",") <<
	PovOutf(mCCurrentObj.mTgt[Y], 4) << wxT(",") <<
	PovOutf(mCCurrentObj.mTgt[Z], 4) << wxT(",") << ENDL << GetTab(2) <<
	PovOutf(mCCurrentObj.mRgt[X], 4) << wxT(",") <<
	PovOutf(mCCurrentObj.mRgt[Y], 4) << wxT(",") <<
	PovOutf(mCCurrentObj.mRgt[Z], 4) << wxT(",") << ENDL << GetTab(2) <<
	PovOutf(mCCurrentObj.mTop[X], 4) << wxT(",") <<
	PovOutf(mCCurrentObj.mTop[Y], 4) << wxT(",") <<
	PovOutf(mCCurrentObj.mTop[Z], 4) << wxT(",") << ENDL << GetTab(2) <<
	PovOutf(mCCurrentObj.mPos[X], 4) << wxT(",") <<
	PovOutf(mCCurrentObj.mPos[Y], 4) << wxT(",") <<
	PovOutf(mCCurrentObj.mPos[Z], 4) << wxT(">") << ENDL;
}

GLGuides_cls* SceneObj_cls::mspCGLGuideLink;
GLGuides_cls* SceneObj_cls::mspCGLGuideYPRLink;
GLGuides_cls* SceneObj_cls::mspCGLGuideScaleLink;
