// -*- c -*-
// "clsSceneObj.cpp"

#ifndef _SCENE_OBJ_H
#define _SCENE_OBJ_H

#include "../POVRAY/clsPovSerialize.h"

#include "../MYGL/clsPositionAndDirection.h"
#include "../PARAMETER/clsParamGroup.h"

#include "../COMMON/CommonDefinitionsGL.h"

#include <vector>

#define _SCENEOBJ_CAMERA 0
#define _SCENEOBJ_LIGHT  1

#define _SCENEOBJ_IDBASE_CAMERA 0
#define _SCENEOBJ_IDBASE_LIGHT  10

#define _SCENEOBJ_TARGET_NONE   0
#define _SCENEOBJ_TARGET_ORIGIN 1
#define _SCENEOBJ_TARGET_ANCHOR 2

#define _WIREFRAME_BOX 0
#define _WIREFRAME_PLANE 1

#define _ROTATE_EULER_XYZ 0
#define _ROTATE_EULER_XZY 1
#define _ROTATE_EULER_YZX 2
#define _ROTATE_EULER_YXZ 3
#define _ROTATE_EULER_ZXY 4
#define _ROTATE_EULER_ZYX 5

class GLGuides_cls;

class SceneObj_cls : public PovSerialize_cls{
 public:
    SceneObj_cls(wxString strObjName, bool bDefaultParam = TRUE, bool bAllParam = TRUE);
    ~SceneObj_cls();

    void SetInitValueTrans(double x, double y, double z);
    void SetInitValueRot(double x, double y, double z);
    void SetInitValueScale(double x, double y, double z);
    void CopyFromP_SceneObj(SceneObj_cls *pCFrom);

    bool mbSelected, mbDragging;
    int  mObjType, mObjIndex;

    ParamGroup_cls *mpCTranslate;
    ParamGroup_cls *mpCRotation;
    ParamGroup_cls *mpCScale;

    ParamGroup_cls *mpCSceneObjParam;
    
    ParamGroup_cls* GetPCParamGroupFromTag(wxString strTag);

    std::vector < double > GetVDirectionVector(double doubleIndex);
    
    void RotAround(float vector[3], float rad, int axis);
    void CalcRelativeVector(float vector[3], double doubleIndex,
			    int rotOrder = 0);
    void CalcRelativePosition(float vector[3], double doubleIndex,
			      ParamGroup_cls *pCAnchor = NULL,
			      int rotOrder = 0);
    void CalcGlobalVector(float vector[3], double doubleIndex,
			  int rotOrder = 0);
    void CalcGlobalPosition(float vector[3], double doubleIndex,
			    ParamGroup_cls *pCAnchor = NULL,
			    int rotOrder = 0);
    
    static GLGuides_cls* mspCGLGuideLink;
    static GLGuides_cls* mspCGLGuideYPRLink;
    static GLGuides_cls* mspCGLGuideScaleLink;

    virtual void AssignSceneObjParam();

    ConfItemList_cls  mCConfList;
    ConfItemUINT_cls  mCTarget;
    ConfItemBOOL_cls  mCbVisible;

    void Select(bool bSelect);
    virtual void UpdateCurrentObj(double dobuleIndex);
    PositionAndDirection_cls mCCurrentObj;
    PositionAndDirection_cls mCCurrentObjLookingAtCamera;

    void Targetting();

    void GLSetMatrix();
    void GLSetMatrixInverse();
    void GLDrawPath();
    void GLDrawWireframe(int type, float scale = 1.0f);
   
    void GetStrPovModelView(wxString &strModelView, double doubleIndex,
			    std::vector < double > VAnchor = std::vector < double >(), int rotateOrder = 0);
    void GetStrPovModelViewMatrix(wxString& strModelView);

    //wxString Serialize();
};

#endif
