// "clsLightSourceGUI.h"

#include "clsTexPropertyGUI.h"

#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemConstParam.h"

#include "../WIDGETCTRL/clsMultiInputPanel.h"
#include "../WIDGETCTRL/clsHookCtrl.h"
#include "../WIDGETCTRL/EX/clsStaticTextEX.h"
#include "../WIDGETCTRL/WX/myArtProv.h"

#include "../WIDGETCTRL/EX/clsFileDialogEX.h"
extern wxFileDialogEX *GpCFileDialog;

#include "../../clsFrameMain.h"

#include "../../wxWidgetEnum.h"

#include <wx/filename.h>

#ifdef WIN32
extern PFNGLPROGRAMPARAMETERIEXTPROC glProgramParameteri;
#endif

TexPropertyGUI_cls::
TexPropertyGUI_cls(TexPropertyCtrl_cls *pCCtrl, wxWindow *pC) :
    mstrTitle(_("Texture::")), 
    wxDialogEXKey(pC, wxID_ANY, _("Texture"), 
		  wxDPS, ARTWXFRAMEICON("wxART_TEXTURE")) {

    mpCFrameMainLink = pCCtrl->mpCFrameMainLink;
	    
    wxBoxSizer *pBoxSizerSrc   = new wxBoxSizer(wxVERTICAL);

    mDummyShader    = 2;
    mDummyShaderGas = 0;
    mDummyShininess = 10;
    mDummyTransmitType = 0;
    mDummyAlpha = 1;
    mDummyAlphaClipping = 0;
    mDummyScatterAmbientFactor = 0.5;
    mDummyCartoonBoldness = 0.2;
    mDummyOutlineBoldness = 0;
    mDummyReflectionType = 0;
    mDummyReflectionMin = 0.2; mDummyReflectionMax = 0.5;
    mbDummyReflectionFresnel = FALSE; mbDummyReflectionConserve = FALSE;
    mDummyIOR = 1;
    mpCTextureLinked = NULL;
    mDummyMappingType = 0;
    mDummyTextureSize = 0;
    mbDummySeqRepeat = 0;
    mDummySeqOffset = 0; mDummySeqLength = 0;
    
    for (int i = 0; i < _TEX_NORMAL_MAX; i++){
	mDummyNormalPattern[i] = 0;
	mDummyNormalDegree[i] = 0.3;
	mDummyNormalScale[i] = 1;
    }

    mDummyPhotonTarget = FALSE;
    mDummyPhotonCollect = FALSE;
    for (int i = 0; i < 7; i++) mbDummyLightRecieve[i] = TRUE;
    mbDummyFogRecieve = TRUE;
    mDummyCastShadow = 1;

    SetMNumParam();

    mpCBook1 = new wxNotebook(this, ID_BOOK_TEXTURE1);
    mpCBook2 = new wxNotebook(this, ID_BOOK_TEXTURE1);

    SetMNumTexFormat(mpCBook1);    
    SetMNumBump(mpCBook1);
    SetMNumMirror(mpCBook1);
    SetMNumInterior(mpCBook1);
    SetMNumPhotons(mpCBook2);
    SetMNumRecieve(mpCBook2);
    SetMNumCast(mpCBook2);
    
    mpCBook1->AddPage(mpCMInputTexFormat, _("Texture"));
    mpCBook1->AddPage(mpCMInputBump, _("Bump Map"));
    mpCBook1->AddPage(mpCMInputMirror, _("Mirror"));
    mpCBook1->AddPage(mpCMInputInterior, _("Interior"));
    mpCBook2->AddPage(mpCMInputPhotons, _("Photons"));
    mpCBook2->AddPage(mpCMInputRecieve, _("Recieve"));
    mpCBook2->AddPage(mpCMInputCast, _("Cast"));
    
    pBoxSizerSrc->Add (mpCMInputTexture, 0, wxEXPAND);
    pBoxSizerSrc->Add (mpCBook1, 0, wxEXPAND);
    pBoxSizerSrc->Add (mpCBook2, 0, wxEXPAND);
       
    SetSizerAndFit(pBoxSizerSrc);

    ReconnectTexture(NULL);
    Fit();
}

TexPropertyGUI_cls::
~TexPropertyGUI_cls(){
    if (mpCTextureLinked != NULL)
	mpCTextureLinked->mpCLinkedToMe = NULL;
}

void TexPropertyGUI_cls::
SetMNumParam(){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputTexture, this);

    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_GL |_COMMAND_UPDATE_DISPLAY_TEXTURE | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_ATTRIB; 

    pCurr->mTextWidth = 90;
    pCurr->mItemWidth = -1;
    pCurr->mComboWidth = -1;

    mIndexShader = pCurr->CombinePointerSizer(2, _("Shader"));
    
    wxString strListShader[4] = 
	{_("original"), _("Gouroud"), _("Phong"), _("Toon")};
    pCurr->CombinePointerList(&mDummyShader, 4, strListShader);

    wxString strListShaderGas[3] = 
	{_("original"), _("Scatter"), _("Alpha Clip")};
    pCurr->CombinePointerList(&mDummyShaderGas, 3, strListShaderGas);

    mIndexCartoonBoldness = 
	pCurr->SetInputAndSlider(_ITEM_FLOAT, &mDummyCartoonBoldness, 
				 _("Edge threshold"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 100, 100);

    mIndexOutlineBoldness = 
	pCurr->SetInputAndSlider(_ITEM_FLOAT, &mDummyOutlineBoldness, 
				 _("Outline Width"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 100, 100);

    mIndexAlphaClipping = 
	pCurr->SetInputAndSlider(_ITEM_FLOAT, &mDummyAlphaClipping, 
				 _("Alpha Clipping"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 100, 100);

    mIndexScatterAmbientFactor = 
	pCurr->SetInputAndSlider(_ITEM_FLOAT, &mDummyScatterAmbientFactor,
				 _("Ambient Factor"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 100, 100);

    mIndexShininess = 
	pCurr->SetInputAndSlider(_ITEM_INTEGER_SPIN, &mDummyShininess, 
				 _("Shininess"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 100, 1);
    
    mIndexTransmit = pCurr->CombinePointerSizer(1, _("Transmit Type"));
    
    wxString strListTransmit[3] = 
	{_("no transmit"), _("transmit"), _("filtered")};
    pCurr->CombinePointerList(&mDummyTransmitType, 3, strListTransmit);

    mIndexAlpha = 
	pCurr->SetInputAndSlider(_ITEM_FLOAT, &mDummyAlpha, _("Alpha"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 100, 100);
    

    pCurr->FinishSetting(NULL);

    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX, -1, pCurr);
}

void TexPropertyGUI_cls::
SetMNumTexFormat(wxWindow *pCWindow){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputTexFormat, pCWindow);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_GL |_COMMAND_UPDATE_DISPLAY_TEXTURE | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_ATTRIB; 
    
    pCurr->mUndoType = _MN_UNDO_MANUAL;

    pCurr->mItemWidth = -1;
    mIndexTexFormat[0] = pCurr->CombinePointerSizer(3, _("Billboard"));
    pCurr->CombinePointer(_ITEM_STRING, &mDummyTexFormat[0]);
    pCurr->mItemWidth = 30;
    pCurr->CombinePointerButton(wxID_ANY, wxT(".."));
    pCurr->mItemWidth = 40;
    int indexClear0 = pCurr->CombinePointerButton(wxID_ANY, wxT("Clear"));

    pCurr->mItemWidth = -1;
    mIndexTexFormat[1] = pCurr->CombinePointerSizer(3, _("Sphere"));
    pCurr->CombinePointer(_ITEM_STRING, &mDummyTexFormat[1]);
    pCurr->mItemWidth = 30;
    pCurr->CombinePointerButton(wxID_ANY, wxT(".."));
    pCurr->mItemWidth = 40;
    int indexClear1 = pCurr->CombinePointerButton(wxID_ANY, wxT("Clear"));

    pCurr->mUndoType = _MN_UNDO_AUTO_POINTER;

    pCurr->mTextWidth = 115;
    pCurr->mItemWidth = -1;
    mIndexTextureSize = pCurr->CombinePointerSizer(2, _("Baked texture size"));
    wxString strListSize[4] = {wxT("128x128"), wxT("256x256"), wxT("512x512"), wxT("1024x1024")};
    pCurr->CombinePointerList(&mDummyTextureSize, 4, strListSize);
    
    pCurr->mItemWidth = 120;
    mIndexTexDefault = pCurr->CombinePointerButton(wxID_ANY, wxT("Default"));
    
    pCurr->mTextWidth = 100;
    pCurr->mItemWidth = -1;
    wxString strListMapping[4] = {wxT("X"), wxT("Y"), wxT("Z"), wxT("UV")};
    mIndexMapping = pCurr->CombinePointerList
	(&mDummyMappingType, 4, strListMapping, _("Mapping Type"));

    mIndexImgSeq1 = pCurr->CombinePointerSizer(1, _("Sequence"));
    //pCurr->CombinePointer(_ITEM_BOOL, &mbDummySequence, wxT("Use"));
    pCurr->CombinePointer(_ITEM_BOOL, &mbDummySeqRepeat, wxT("Repeat"));
    mIndexImgSeq2 = pCurr->CombinePointerSizer(2, _("Offset/Length"));
    //pCurr->mItemWidth = 80;
    pCurr->CombinePointer(_ITEM_INTEGER, &mDummySeqOffset, wxT("Offset"));
    pCurr->CombinePointer(_ITEM_INTEGER, &mDummySeqLength, wxT("Length"));
    
    pCurr->FinishSetting(NULL);

    pCurr->mVCItemList[mIndexImgSeq2].
	mpCTextLabel->SetToolTip(_("Legnth < 0 means automatic."));
    
    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_BUTTON, mIndexTexFormat[0]+2, &HookCtrl_cls::ConstZero);
    pCurr->SetHook(this, &HookItem_cls::HookFunc01, _HOOK_BUTTON, mIndexTexFormat[1]+2, &HookCtrl_cls::ConstOne);
    pCurr->SetHook(this, &HookItem_cls::HookFunc02, _HOOK_BUTTON, mIndexTexDefault);
    pCurr->SetHook(this, &HookItem_cls::HookFunc03, _HOOK_BUTTON, indexClear0, &HookCtrl_cls::ConstZero);
    pCurr->SetHook(this, &HookItem_cls::HookFunc03, _HOOK_BUTTON, indexClear1, &HookCtrl_cls::ConstOne);

    ReconnectTexFormat(NULL);
    HideAndShowTexFormat();
}

void TexPropertyGUI_cls::
SetMNumMirror(wxWindow *pCWindow){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputMirror, pCWindow);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_GL |_COMMAND_UPDATE_DISPLAY_TEXTURE | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_ATTRIB;
    pCurr->mUndoType = _MN_UNDO_AUTO_POINTER;

    pCurr->mTextWidth = 90;
    pCurr->mComboWidth = -1;

    wxString reflexType[3] = 
	{_("no mirror"), _("mirror like"), _("metal like")};

    mIndexReflexType = pCurr->CombinePointerSizer(1, _("Reflex Type"));
    
    pCurr->CombinePointerList(&mDummyReflectionType, 3, reflexType);
    
    mIndexReflexMin = pCurr->SetInputAndSlider
	(_ITEM_FLOAT, &mDummyReflectionMin,_("Reflex min"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 100, 100);

    mIndexReflexMax = pCurr->SetInputAndSlider
	(_ITEM_FLOAT, &mDummyReflectionMax, _("Reflex max"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 100, 100);

    pCurr->mItemWidth = 120;
    mIndexReflexOptions = pCurr->CombinePointerSizer(2);
    pCurr->CombinePointer
	(_ITEM_BOOL, &mbDummyReflectionFresnel, _("Fresnel"));
    pCurr->CombinePointer
	(_ITEM_BOOL, &mbDummyReflectionConserve, _("Conserve En."));
       
    pCurr->FinishSetting(NULL);
    
    pCurr->mVCItemList[mIndexReflexType].
	mpCTextLabel->SetToolTip(_("Metal like reflection is affected by the color of material itself."));
    pCurr->mVCItemList[mIndexReflexMin].
	mpCTextLabel->SetToolTip(_("Reflection when viewed from a direction parallel to its normal."));
    pCurr->mVCItemList[mIndexReflexMax].
	mpCTextLabel->SetToolTip(_("Reflection when viewed from shallow angle. Many materials are more reflective in shallow angle."));

    ReconnectMirror(NULL);
    HideAndShowMirror();

    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX, -1, pCurr);
    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_CHECKBOX, mIndexReflexOptions+1, pCurr);
}

void TexPropertyGUI_cls::
SetMNumBump(wxWindow *pCWindow){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputBump, pCWindow);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_GL |_COMMAND_UPDATE_DISPLAY_TEXTURE | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_ATTRIB;
    pCurr->mUndoType = _MN_UNDO_AUTO_POINTER;

    pCurr->mTextWidth = -1;
    pCurr->mComboWidth = 80;

    wxString normalPatternList[12] = 
	{_("no normal pattern"), wxT("agate"), _("bozo"), wxT("bumps"), wxT("brick"), wxT("cells"), _("checker"), wxT("crackle"), /*wxT("cylindrical"), */wxT("dents"), wxT("facets"), /*wxT("mandel"), wxT("julia"), wxT("hexagon"),*/ wxT("marble"), wxT("ridged_mf")};
    
    int boxWidth = 2;
    for (int i = 0; i < _TEX_NORMAL_MAX; i++){
	pCurr->mItemWidth = -1;
	mIndexNormalPattern[i] = pCurr->CombinePointerList
	    (&mDummyNormalPattern[i], 
	     12, normalPatternList, _("Normal"));
	mIndexNormalDegree[i] = pCurr->SetInputAndSlider
	    (_ITEM_FLOAT, &mDummyNormalDegree[i], _("degree"));
	pCurr->SetRange(_ITEM_LASTITEM, 0, 100, 100);
	mIndexNormalScale[i] = pCurr->SetInputAndSlider
	    (_ITEM_FLOAT, &mDummyNormalScale[i], _("scale"));
	pCurr->SetRange(_ITEM_LASTITEM, 0, 1000, 100);
    }

    pCurr->FinishSetting(NULL);

    ReconnectBump(NULL);
    HideAndShowBump();

    pCurr->SetHook(this, &HookItem_cls::HookFuncHS, _HOOK_COMBOBOX, -1, pCurr);
}

void TexPropertyGUI_cls::
SetMNumInterior(wxWindow *pCWindow){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputInterior, pCWindow);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_GL |_COMMAND_UPDATE_DISPLAY_TEXTURE | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_ATTRIB;
    pCurr->mUndoType = _MN_UNDO_AUTO_POINTER;
	
    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;
    int boxWidth = 2;

    /*
    mIndexFadeDistance = 
	pCurr->SetInputAndSlider(_ITEM_INTEGER_SPIN, &mDummyFadeDistance, 
				 _("Fade Distance"));
    */
    mIndexIOR = 
	pCurr->SetInputAndSlider(_ITEM_FLOAT, &mDummyIOR, 
				 _("IOR"));
    pCurr->SetRange(_ITEM_LASTITEM, 0, 200, 100);

    pCurr->FinishSetting(NULL);
    
    ReconnectInterior(NULL);
    HideAndShowInterior();
}

void TexPropertyGUI_cls::
SetMNumPhotons(wxWindow *pCWindow){
    MultiInputPanel_cls* pCurr = 
	MultiInputPanel_cls::Initialize(&mpCMInputPhotons, pCWindow);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_GL |_COMMAND_UPDATE_DISPLAY_TEXTURE | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_ATTRIB;
    pCurr->mUndoType = _MN_UNDO_AUTO_POINTER;

    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;

    mIndexPhotonTarget = pCurr->CombinePointer
	(_ITEM_BOOL, &mDummyPhotonTarget, wxT("target"));
    mIndexPhotonCollect = pCurr->CombinePointer
	(_ITEM_BOOL, &mDummyPhotonCollect, wxT("collect"));
    
    pCurr->FinishSetting(NULL);

    ReconnectPhotons(NULL);
    HideAndShowPhotons();
}

void TexPropertyGUI_cls::
SetMNumRecieve(wxWindow *pCWindow){
    MultiInputPanel_cls* pCurr =
	MultiInputPanel_cls::Initialize(&mpCMInputRecieve, pCWindow);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_GL |_COMMAND_UPDATE_DISPLAY_TEXTURE | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_ATTRIB;
    //pCurr->mUndoType = _MN_UNDO_AUTO_POINTER;

    pCurr->mTextWidth = 80;
    pCurr->mItemWidth = -1;

    pCurr->CombinePointerSizer(7, _("Lighting"));
    for (int i = 0; i < 7; i++){
	wxString strNum; strNum << i;
	mIndexLightRecieve[i] = pCurr->CombinePointer
	    (_ITEM_BOOL, &mbDummyLightRecieve[i], strNum);
	pCurr->GetLastItem().mItemWidth = 32;
    }
    mIndexFogRecieve = pCurr->CombinePointer
	(_ITEM_BOOL, &mbDummyFogRecieve, _("Fog"));

    pCurr->FinishSetting(NULL);
    
    ReconnectRecieve(NULL);
    HideAndShowRecieve();
}

void TexPropertyGUI_cls::
SetMNumCast(wxWindow *pCWindow){
    MultiInputPanel_cls* pCurr =
	MultiInputPanel_cls::Initialize(&mpCMInputCast, pCWindow);
    pCurr->mUpdateBit = 
	_COMMAND_UPDATE_DISPLAY_GL |_COMMAND_UPDATE_DISPLAY_TEXTURE | _COMMAND_UPDATE_DISPLAY_FILTER_PARAM | _COMMAND_UPDATE_DISPLAY_ATTRIB;

    pCurr->mTextWidth = 120;
    pCurr->mItemWidth = -1;
    
    wxString shadowType[2] = 
	{_("No Shadow"), _("Spherical Shadow")};
    mIndexCastShadow = pCurr->CombinePointerList
	    (&mDummyCastShadow, 2, shadowType, _("Shadow Volume"));
    
    pCurr->FinishSetting(NULL);

    ReconnectCast(NULL);
    HideAndShowCast();
}

void TexPropertyGUI_cls::
RecalcSize(){
    int minSize1 = 0;
    for (int i = 0; i < mpCBook1->GetPageCount(); i++){
	mpCBook1->GetPage(i)->Layout();
	wxSize size = mpCBook1->GetPage(i)->GetBestSize();
	if (minSize1 < size.GetHeight())
	    minSize1 = size.GetHeight();
    }
    mpCBook1->SetSizeHints(-1, minSize1 + 30);

    int minSize2 = 0;
    for (int i = 0; i < mpCBook2->GetPageCount(); i++){
	mpCBook2->GetPage(i)->Layout();
	wxSize size = mpCBook2->GetPage(i)->GetBestSize();
	if (minSize2 < size.GetHeight())
	    minSize2 = size.GetHeight();
    }
    mpCBook2->SetSizeHints(-1, minSize2 + 30);
}

void TexPropertyGUI_cls::
UpdateValuesAll(){
    mpCMInputTexture->UpdateValues();
    mpCMInputTexFormat->UpdateValues();
    mpCMInputBump->UpdateValues();
    mpCMInputMirror->UpdateValues();
    mpCMInputInterior->UpdateValues();
    mpCMInputPhotons->UpdateValues();
    mpCMInputRecieve->UpdateValues();
    mpCMInputCast->UpdateValues();
}

void TexPropertyGUI_cls::
ReconnectBase(SceneObjTexture_cls *pCTex){
    ReconnectTexFormat(pCTex);
    ReconnectBump(pCTex);
    ReconnectMirror(pCTex);
    ReconnectInterior(pCTex);
    ReconnectPhotons(pCTex);
    ReconnectRecieve(pCTex);
    ReconnectCast(pCTex);
    HideAndShowTexture();
    HideAndShowTexFormat();
    HideAndShowBump();
    HideAndShowMirror();
    HideAndShowInterior();
    HideAndShowPhotons();
    HideAndShowRecieve();
    HideAndShowCast();
    RecalcSize();
}

void TexPropertyGUI_cls::
ReconnectTexture(SceneObjTexture_cls *pCTex, wxString strCustomName){
    MultiInputPanel_cls* pCurr = mpCMInputTexture;

    if (pCTex == NULL){
	SetTitle(mstrTitle + _("Not selected"));
	pCurr->Reconnect(mIndexShader+1, &mDummyShader);
	pCurr->Reconnect(mIndexShader+2, &mDummyShaderGas);
	pCurr->Reconnect(mIndexShininess+1, &mDummyShininess);
	pCurr->Reconnect(mIndexShininess+2, &mDummyShininess);
	pCurr->Reconnect(mIndexTransmit+1, &mDummyTransmitType);

	pCurr->ReconnectInputAndSlider
	    (mIndexCartoonBoldness, _ITEM_FLOAT, &mDummyCartoonBoldness);
	pCurr->ReconnectInputAndSlider
	    (mIndexOutlineBoldness, _ITEM_FLOAT, &mDummyOutlineBoldness);
	pCurr->ReconnectInputAndSlider
	    (mIndexAlpha, _ITEM_FLOAT, &mDummyAlpha);
	pCurr->ReconnectInputAndSlider
	    (mIndexAlphaClipping, _ITEM_FLOAT, &mDummyAlphaClipping);
	pCurr->ReconnectInputAndSlider
	    (mIndexScatterAmbientFactor, _ITEM_FLOAT, &mDummyScatterAmbientFactor);
	
	pCurr->UpdateValues();
	pCurr->EnableItem(mIndexShader+1, FALSE);
	pCurr->EnableItem(mIndexShader+2, FALSE);
	pCurr->EnableItem(mIndexShininess+1, FALSE);
	pCurr->EnableItem(mIndexShininess+2, FALSE);
	pCurr->EnableItem(mIndexTransmit+1, FALSE);
	pCurr->EnableItem(mIndexCartoonBoldness, FALSE);
	pCurr->EnableItem(mIndexOutlineBoldness, FALSE);
	pCurr->EnableItem(mIndexAlpha, FALSE);
	pCurr->EnableItem(mIndexAlphaClipping, FALSE);
	pCurr->EnableItem(mIndexScatterAmbientFactor, FALSE);
	
	ReconnectBase(pCTex);
	return;
    }

    SetTitle(mstrTitle + strCustomName);

    pCurr->Reconnect(mIndexShader+1, &(pCTex->mCShaderType.mBody));
    pCurr->Reconnect(mIndexShader+2, &(pCTex->mCGasShader.mBody));

    pCurr->Reconnect(mIndexShininess+1, &(pCTex->mCShininess.mBody));
    pCurr->Reconnect(mIndexShininess+2, &(pCTex->mCShininess.mBody));
    pCurr->Reconnect(mIndexTransmit+1, &(pCTex->mCTransmitType.mBody));

    pCurr->ReconnectInputAndSlider
	(mIndexCartoonBoldness, _ITEM_FLOAT, &(pCTex->mCCartoonBoldness.mBody));
    pCurr->ReconnectInputAndSlider
	(mIndexOutlineBoldness, _ITEM_FLOAT, &(pCTex->mCOutlineBoldness.mBody));
    pCurr->ReconnectInputAndSlider
	(mIndexAlpha, _ITEM_FLOAT, &(pCTex->mCAlpha.mBody));
    pCurr->ReconnectInputAndSlider
	(mIndexAlphaClipping, _ITEM_FLOAT, &(pCTex->mCAlphaClipping.mBody));
    pCurr->ReconnectInputAndSlider
	(mIndexScatterAmbientFactor, _ITEM_FLOAT, &(pCTex->mCScatterAmbientFacter.mBody));

    pCurr->EnableItem(mIndexShader+1, TRUE);
    pCurr->EnableItem(mIndexShader+2, TRUE);
    pCurr->EnableItem(mIndexShininess+1, TRUE);
    pCurr->EnableItem(mIndexShininess+2, TRUE);
    pCurr->EnableItem(mIndexTransmit+1, TRUE);

    pCurr->EnableItem(mIndexCartoonBoldness, TRUE);
#ifdef WIN32
    bool bOutlineEnable = 
	(glProgramParameteri != NULL) ? TRUE : FALSE;
#else
    bool bOutlineEnable = TRUE;
#endif
    pCurr->EnableItem(mIndexOutlineBoldness, bOutlineEnable);
    pCurr->EnableItem(mIndexAlpha, TRUE);
    pCurr->EnableItem(mIndexAlphaClipping, TRUE);
    pCurr->EnableItem(mIndexScatterAmbientFactor, TRUE);
    
    pCurr->UpdateValues();

    if (mpCTextureLinked != NULL) 
	mpCTextureLinked->mpCLinkedToMe = NULL;
    mpCTextureLinked = pCTex;
    pCTex->mpCLinkedToMe = this;
    
    ReconnectBase(pCTex);
    Layout();
    ResizeDialog();
}

void TexPropertyGUI_cls::
ReconnectTexFormat(SceneObjTexture_cls *pCTex){
    MultiInputPanel_cls* pCurr = mpCMInputTexFormat;
    
    if (pCTex == NULL){
	pCurr->Reconnect(mIndexTexFormat[0]+1, &mDummyTexFormat[0]);
	pCurr->Reconnect(mIndexTexFormat[1]+1, &mDummyTexFormat[1]);
	pCurr->Reconnect(mIndexTextureSize+1, &mDummyTextureSize);
	pCurr->Reconnect(mIndexMapping, &mDummyMappingType);
	pCurr->Reconnect(mIndexImgSeq1+1, &mbDummySeqRepeat);
	pCurr->Reconnect(mIndexImgSeq2+1, &mDummySeqOffset);
	pCurr->Reconnect(mIndexImgSeq2+2, &mDummySeqLength);
	return;
    }

    pCurr->Reconnect(mIndexTexFormat[0]+1, &(pCTex->mCstrTexFormat[0].mstrBody));
    pCurr->Reconnect(mIndexTexFormat[1]+1, &(pCTex->mCstrTexFormat[1].mstrBody));
    pCurr->Reconnect(mIndexTextureSize+1, &(pCTex->mCTextureSize.mBody));
    pCurr->Reconnect(mIndexMapping, &(pCTex->mCMappingType.mBody));
    pCurr->Reconnect(mIndexImgSeq1+1, &(pCTex->mCbImgSeqRepeat.mbBody));
    pCurr->Reconnect(mIndexImgSeq2+1, &(pCTex->mCImgSeqOffset.mBody));
    pCurr->Reconnect(mIndexImgSeq2+2, &(pCTex->mCImgSeqLength.mBody));
    pCurr->UpdateValues();
}

void TexPropertyGUI_cls::
ReconnectMirror(SceneObjTexture_cls *pCTex){
    MultiInputPanel_cls* pCurr = mpCMInputMirror;

    if (pCTex == NULL){
	pCurr->Reconnect(mIndexReflexType+1, &mDummyReflectionType);
	pCurr->ReconnectInputAndSlider
	    (mIndexReflexMin, _ITEM_FLOAT, &mDummyReflectionMin);
	pCurr->ReconnectInputAndSlider
	    (mIndexReflexMax, _ITEM_FLOAT, &mDummyReflectionMax);
	pCurr->Reconnect(mIndexReflexOptions+1, &mbDummyReflectionFresnel);
	pCurr->Reconnect(mIndexReflexOptions+2, &mbDummyReflectionConserve);
	
	pCurr->UpdateValues();
	pCurr->EnableItem(mIndexReflexType, FALSE);
	pCurr->EnableItem(mIndexReflexMin, FALSE);
	pCurr->EnableItem(mIndexReflexMax, FALSE);	
	pCurr->EnableItem(mIndexReflexOptions+1, FALSE);
	pCurr->EnableItem(mIndexReflexOptions+2, FALSE);
	return;
    }

    pCurr->Reconnect(mIndexReflexType+1, &(pCTex->mCReflectType.mBody));
    pCurr->ReconnectInputAndSlider
	(mIndexReflexMin, _ITEM_FLOAT, &(pCTex->mCReflectMin.mBody));
    pCurr->ReconnectInputAndSlider
	(mIndexReflexMax, _ITEM_FLOAT, &(pCTex->mCReflectMax.mBody));
    pCurr->Reconnect(mIndexReflexOptions+1, &(pCTex->mCbRefrectFresnel.mbBody));
    pCurr->Reconnect(mIndexReflexOptions+2, &(pCTex->mCbRefrectConserve.mbBody));
    
    pCurr->EnableItem(mIndexReflexType, TRUE);
    pCurr->EnableItem(mIndexReflexMin, TRUE);
    pCurr->EnableItem(mIndexReflexMax, TRUE);
    pCurr->EnableItem(mIndexReflexOptions+1, TRUE);
    pCurr->EnableItem(mIndexReflexOptions+2, TRUE);
    
    pCurr->UpdateValues();
}

void TexPropertyGUI_cls::
ReconnectBump(SceneObjTexture_cls *pCTex){
    MultiInputPanel_cls* pCurr = mpCMInputBump;

    if (pCTex == NULL){
	for (int i = 0; i < _TEX_NORMAL_MAX; i++){
	    pCurr->Reconnect(mIndexNormalPattern[i], &mDummyNormalPattern[i]);
	    pCurr->EnableItem(mIndexNormalPattern[i], FALSE);
	    
	    pCurr->ReconnectInputAndSlider
		(mIndexNormalDegree[i], _ITEM_FLOAT, &mDummyNormalDegree[i]);
	    pCurr->ReconnectInputAndSlider
		(mIndexNormalScale[i], _ITEM_FLOAT, &mDummyNormalScale[i]);
	    
	    pCurr->EnableItem(mIndexNormalDegree[i], FALSE);
	    pCurr->EnableItem(mIndexNormalScale[i], FALSE);
	}
	pCurr->UpdateValues();
	return;
    }

    for (int i = 0; i < _TEX_NORMAL_MAX; i++){
	pCurr->Reconnect(mIndexNormalPattern[i], &(pCTex->mCNormalPattern[i].mBody));
	pCurr->EnableItem(mIndexNormalPattern[i], TRUE);
	
	pCurr->ReconnectInputAndSlider
	    (mIndexNormalDegree[i], _ITEM_FLOAT, &(pCTex->mCNormalDegree[i].mBody));
	pCurr->ReconnectInputAndSlider
	    (mIndexNormalScale[i], _ITEM_FLOAT, &(pCTex->mCNormalScale[i].mBody));
	pCurr->EnableItem(mIndexNormalDegree[i], TRUE);
	pCurr->EnableItem(mIndexNormalScale[i], TRUE);
    }
    pCurr->UpdateValues();
}

void TexPropertyGUI_cls::
ReconnectInterior(SceneObjTexture_cls *pCTex){
    MultiInputPanel_cls* pCurr = mpCMInputInterior;

    if (pCTex == NULL){
	pCurr->ReconnectInputAndSlider
	    (mIndexIOR, _ITEM_FLOAT, &mDummyIOR);
	pCurr->UpdateValues();
	pCurr->EnableItem(mIndexIOR, FALSE);
	return;
    }
    
    pCurr->ReconnectInputAndSlider
	(mIndexIOR, _ITEM_FLOAT, &(pCTex->mCIOR.mBody));
    pCurr->EnableItem(mIndexIOR, TRUE);
    
    pCurr->UpdateValues();
}

void TexPropertyGUI_cls::
ReconnectPhotons(SceneObjTexture_cls *pCTex){
    MultiInputPanel_cls* pCurr = mpCMInputPhotons;

    if (pCTex == NULL){
	pCurr->Reconnect(mIndexPhotonTarget, &mDummyPhotonTarget);
	pCurr->Reconnect(mIndexPhotonCollect, &mDummyPhotonCollect);

	pCurr->EnableItem(mIndexPhotonTarget, FALSE);
	pCurr->EnableItem(mIndexPhotonCollect, FALSE);

	pCurr->UpdateValues();
	return;
    }

    pCurr->Reconnect(mIndexPhotonTarget, &(pCTex->mCbPhotonTarget.mbBody));
    pCurr->Reconnect(mIndexPhotonCollect, &(pCTex->mCbPhotonCollect.mbBody));

    pCurr->EnableItem(mIndexPhotonTarget, TRUE);
    pCurr->EnableItem(mIndexPhotonCollect, TRUE);

    pCurr->UpdateValues();
}

void TexPropertyGUI_cls::
ReconnectRecieve(SceneObjTexture_cls *pCTex){
    MultiInputPanel_cls* pCurr = mpCMInputRecieve;
    
    if (pCTex == NULL){
	for (int i = 0; i < 7; i++)
	    pCurr->Reconnect(mIndexLightRecieve[i], &mbDummyLightRecieve[i]);
	pCurr->Reconnect(mIndexFogRecieve, &mbDummyFogRecieve);

	for (int i = 0; i < 7; i++)
	    pCurr->EnableItem(mIndexLightRecieve[i], FALSE);
	pCurr->Reconnect(mIndexFogRecieve, FALSE);

	pCurr->UpdateValues();
	return;
    }

    for (int i = 0; i < 7; i++)
	pCurr->Reconnect(mIndexLightRecieve[i], &(pCTex->mCbLightRecieve.mpbBody[i]));
    pCurr->Reconnect(mIndexFogRecieve, &(pCTex->mCbFogRecieve.mbBody));

    for (int i = 0; i < 7; i++)
	pCurr->EnableItem(mIndexLightRecieve[i], TRUE);
    pCurr->EnableItem(mIndexFogRecieve, TRUE);
    
    pCurr->UpdateValues();
}

void TexPropertyGUI_cls::
ReconnectCast(SceneObjTexture_cls *pCTex){
    MultiInputPanel_cls* pCurr = mpCMInputCast;

    if (pCTex == NULL){
	pCurr->Reconnect(mIndexCastShadow, &mDummyCastShadow);
	pCurr->EnableItem(mIndexCastShadow, FALSE);
	pCurr->UpdateValues();
	return;
    }
    
    pCurr->Reconnect(mIndexCastShadow, &(pCTex->mCCastShadow.mBody));
    pCurr->EnableItem(mIndexCastShadow, TRUE);

    pCurr->UpdateValues();
}

void TexPropertyGUI_cls::
HideAndShowTexture(){
    MultiInputPanel_cls* pCurr = mpCMInputTexture;
    bool bValid = TRUE;
    if (mpCTextureLinked == NULL) bValid = FALSE;
    bool bTransmit = (pCurr->mVCItemList[mIndexTransmit+1].GetValueInt() != 0);
    pCurr->ShowItem(mIndexAlpha, bTransmit & bValid);
    bool bCartoon = (pCurr->mVCItemList[mIndexShader+1].GetValueInt() == 3);
    pCurr->ShowItem(mIndexCartoonBoldness, bCartoon & bValid);
    pCurr->ShowItem(mIndexOutlineBoldness, bCartoon & bValid);
    bool bParticle = TRUE;
    if (mpCTextureLinked == NULL ||
	mpCTextureLinked->mTexType != _TEX_FOR_PARTILCE) bParticle = FALSE;
    pCurr->EnableItem(mIndexShader+2, bParticle);
    bool bGasShader (pCurr->mVCItemList[mIndexShader+2].GetValueInt() != 0);
    pCurr->ShowItem(mIndexAlphaClipping, bGasShader & bValid);
    bool bScatterShader (pCurr->mVCItemList[mIndexShader+2].GetValueInt() == 1);
    pCurr->ShowItem(mIndexScatterAmbientFactor, bScatterShader & bValid);
}

void TexPropertyGUI_cls::
HideAndShowTexFormat(){

    MultiInputPanel_cls* pCurr = mpCMInputTexFormat;
    bool bSecondTex = FALSE;
    if (mpCTextureLinked != NULL &&
	mpCTextureLinked->mTexType == _TEX_FOR_PARTILCE) bSecondTex = TRUE;
    pCurr->ShowItem(mIndexTexFormat[0], TRUE);
    pCurr->ShowItem(mIndexTexFormat[1], bSecondTex);
    pCurr->ShowItem(mIndexTextureSize, bSecondTex);
    pCurr->ShowItem(mIndexMapping, !bSecondTex);
    pCurr->ShowItem(mIndexImgSeq1, !bSecondTex);
    pCurr->ShowItem(mIndexImgSeq2, !bSecondTex);

    if (bSecondTex == TRUE)
	pCurr->mVCItemList[mIndexTexFormat[0]].mpCTextLabel->SetLabel(_("Billboard"));
    else
	pCurr->mVCItemList[mIndexTexFormat[0]].mpCTextLabel->SetLabel(wxT("path"));
}

void TexPropertyGUI_cls::
HideAndShowMirror(){
    MultiInputPanel_cls* pCurr = mpCMInputMirror;
    bool bValid = TRUE;
    if (mpCTextureLinked == NULL) bValid = FALSE;
    bool bMirror = (pCurr->mVCItemList[mIndexReflexType+1].GetValueInt() != 0);
    pCurr->ShowItem(mIndexReflexMin, bMirror & bValid);
    pCurr->ShowItem(mIndexReflexMax, bMirror & bValid);
    pCurr->ShowItem(mIndexReflexOptions, bMirror & bValid);
}

void TexPropertyGUI_cls::
HideAndShowBump(){
    MultiInputPanel_cls* pCurr = mpCMInputBump;
    bool bValid = TRUE;
    if (mpCTextureLinked == NULL) bValid = FALSE;
    for (int i = 0; i < _TEX_NORMAL_MAX; i++){
	bool bNormal = (pCurr->mVCItemList[mIndexNormalPattern[i]].GetValueInt() != 0);
	pCurr->ShowItem(mIndexNormalDegree[i], bNormal & bValid);
	pCurr->ShowItem(mIndexNormalScale[i], bNormal & bValid);
    }
    
}

void TexPropertyGUI_cls::
HideAndShowInterior(){
    MultiInputPanel_cls* pCurr = mpCMInputInterior;
    MultiInputPanel_cls* pCurrTex = mpCMInputTexture;
    bool bTransmit = (pCurrTex->mVCItemList[mIndexTransmit+1].GetValueInt() != 0);
    bool bValid = TRUE;
    if (mpCTextureLinked == NULL) bValid = FALSE;
    else{
	if (mpCTextureLinked->mCReflectType.mBody != 0 &&
	    mpCTextureLinked->mCbRefrectFresnel.mbBody == TRUE)
	    bTransmit = TRUE;
    }
    
    pCurr->ShowItem(mIndexIOR, bTransmit & bValid);
}

void TexPropertyGUI_cls::
HideAndShowPhotons(){
}

void TexPropertyGUI_cls::
HideAndShowRecieve(){
}

void TexPropertyGUI_cls::
HideAndShowCast(){
}

void TexPropertyGUI_cls::
HookFuncHS(int id, void *pArg){
    MultiInputPanel_cls* pCPanel = (MultiInputPanel_cls*)pArg;

    if (pCPanel == mpCMInputMirror){
	HideAndShowMirror();
	HideAndShowInterior();
	RecalcSize();
	ResizeDialog();
    }
    
    if (pCPanel == mpCMInputBump){
	HideAndShowBump();
	RecalcSize();
	ResizeDialog();
    }

    if (pCPanel == mpCMInputTexture){
	HideAndShowTexture();
	HideAndShowInterior();
	RecalcSize();
	ResizeDialog();
    }
}

void TexPropertyGUI_cls::
HookFunc01(int id, void *pArg){
    if (mpCTextureLinked == NULL) return;

    int tex = *(int*)pArg;
    
    GpCFileDialog->SetMessage(wxT("Select Texture"));
    GpCFileDialog->SetWildcard(wxT("image (*.png;*.jpg)|*.png;*.jpg"));
    if (mpCTextureLinked->mCstrTexFormat[tex].mstrBody.IsSameAs(wxT("NONE"))){
	if (mpCFrameMainLink->mCLastOpenedFileTEX.mstrBody.IsEmpty() == FALSE){
	    wxFileName CName = mpCFrameMainLink->mCLastOpenedFileTEX.mstrBody;
	    GpCFileDialog->SetInitPath(CName.GetPath(), CName.GetFullName());
	}
    }
    else {
	wxFileName CName = mpCTextureLinked->mCstrTexFormat[tex].mstrBody;
	CName.MakeAbsolute();
	GpCFileDialog->SetInitPathFull(CName.GetFullPath());
    }
    if (GpCFileDialog->ShowModal() == wxID_CANCEL) return;
    mpCFrameMainLink->mCLastOpenedFileTEX.mstrBody = GpCFileDialog->GetPath();
    mpCTextureLinked->mCstrTexFormat[tex].mstrBody = GpCFileDialog->GetPath();
    mpCMInputTexFormat->UpdateValues();
}

void TexPropertyGUI_cls::
HookFunc02(int id, void *pArg){
    if (mpCTextureLinked == NULL) return;

    if (mpCTextureLinked->mTexType == _TEX_FOR_PARTILCE){
	mpCTextureLinked->mCstrTexFormat[0].mstrBody = wxT("Texture/ParticleA.png"); 
	mpCTextureLinked->mCstrTexFormat[1].mstrBody = wxT("NONE");
    }
    else
	mpCTextureLinked->mCstrTexFormat[0].mstrBody = wxT("NONE"); 
	

    mpCMInputTexFormat->UpdateValues();
}

void TexPropertyGUI_cls::
HookFunc03(int id, void *pArg){
    if (mpCTextureLinked == NULL) return;
    
    int tex = *(int*)pArg;
    mpCTextureLinked->mCstrTexFormat[tex].mstrBody = wxT("NONE"); 
    mpCMInputTexFormat->UpdateValues();
}

// ------

TexPropertyCtrl_cls::
TexPropertyCtrl_cls(){
    mpCGUI = NULL;
}

TexPropertyCtrl_cls::
~TexPropertyCtrl_cls(){
}

void TexPropertyCtrl_cls::
InitGUI(FrameMain_cls *pCParentFrameLink){
    mpCFrameMainLink = pCParentFrameLink;
    mpCGUI = new TexPropertyGUI_cls(this, pCParentFrameLink);
    SetPDialog(mpCGUI);
}

void TexPropertyCtrl_cls::
UpdateItems(bool bForce){
    if (mpCGUI == NULL) return;
}

void TexPropertyCtrl_cls::
ReconnectTexture(SceneObjTexture_cls *pCTexture, wxString strCustomName){
    mpCGUI->ReconnectTexture(pCTexture, strCustomName);
    mpCGUI->ResizeDialog();
}
