// -*- c++ -*-
// "clsTimeCtrlGUI.h"

#include "clsTimeCtrlGUI.h"
#include "clsTimeCtrl.h"

#include "../../clsMain.h"
#include "../COMMON/CommonDefinitions.h"

#include "../../wxWidgetEnumMenu.h"

BEGIN_EVENT_TABLE(TimeCtrlGUI_cls, wxPanel)
EVT_KEY_DOWN    (TLPanelBase_cls::OnKeyDown)
EVT_SIZE        (TimeCtrlGUI_cls::OnSize)
EVT_PAINT       (TimeCtrlGUI_cls::OnPaint)
EVT_MOUSE_EVENTS(TimeCtrlGUI_cls::OnMouse)
END_EVENT_TABLE()

TimeCtrlGUI_cls::
TimeCtrlGUI_cls(wxWindow *pCParent, TimeCtrl_cls *pCCtrl,
		wxWindowID id, wxPoint pos, wxSize size): 
TLPanelBase_cls(pCParent, pCCtrl, id, pos, size), mCwxPenPaleGreen(wxColour(192,255,192))
{
    mpCTimeSliderIcon = new wxIcon(wxT("ICONS/TimeSlider.png"), wxBITMAP_TYPE_PNG);
    mpCTimeSliderIcon->SetWidth(15);
    mpCTimeSliderIcon->SetHeight(18);
}

TimeCtrlGUI_cls::
~TimeCtrlGUI_cls(){
    delete mpCTimeSliderIcon;
}

void TimeCtrlGUI_cls::
UpdateDisplay(wxAutoBufferedPaintDC &dc){

    dc.SetBackground(wxBrush(wxColour(216,216,216), wxSOLID));
    dc.Clear();

    SetMemberSize(GetClientSize());
    UpdateTLFrame();
    
    wxBrush PlayModeBrush(wxColour(216,216,216), wxSOLID);
    
    dc.SetPen(*wxTRANSPARENT_PEN);
    
    DrawWorkingArea(dc, mMinFrameWA, mMaxFrameWA, mAAMin, mAAMax);
    
    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    // mە`
    dc.SetPen(*wxGREEN_PEN);
    for (int frame = mMinFrameWA; frame < mFrameMax; frame+=mAnimStep){
	//int index = mpCCtrlLink->GetRemappedIndex(frame);
	if (mpCCtrlLink->IsTheFrameReady(frame, TRUE) == TRUE){
	    CalcDeviceCoordinate(&mX0, &mX1, 
				 frame, frame+mAnimStep, mAAMin, mAAMax);
	    dc.DrawRectangle(mX0, 0, mX1-mX0, mHeight - 0);
	}
    }
    dc.SetPen(mCwxPenPaleGreen);
    for (int frame = mMinFrameWA; frame < mFrameMax; frame+=mAnimStep){
	//int index = mpCCtrlLink->GetRemappedIndex(frame);
    	if (mpCCtrlLink->IsTheFrameReady(frame, TRUE) == FALSE &&
	    mpCCtrlLink->IsTheFrameLoaded(frame) == TRUE){
	    CalcDeviceCoordinate(&mX0, &mX1, 
				 frame, frame+mAnimStep, mAAMin, mAAMax);
	    dc.DrawRectangle(mX0, 0, mX1-mX0, mHeight - 0);
	}
    }

    // SnippingTag`
    dc.SetPen(*wxBLACK_PEN);
    mpCCtrlLink->UpdateSnippingTag();
    for (int frame = mMinFrameWA; frame < mFrameMax; frame+=mAnimStep){
	if (mpCCtrlLink->mVbSnippingTag[frame] == TRUE)
	    CalcDeviceCoordinate(&mX0, &mX1, 
				 frame, frame+mAnimStep, mAAMin, mAAMax);
	    dc.DrawRectangle(mX0, mHeight-2, mX1-mX0, 2);
    }

    DrawWorkingAreaSlider(dc, mMinFrameWA, mMaxFrameWA, mAAMin, mAAMax);

    // ̎Ԃ`
    DrawCurrentTime(dc, mAAMin, mAAMax);
    
    wxString Mes;
    dc.SetTextForeground(*wxRED);

    // ԗp̃ACR`
    if (mSelected == _TL_SEL_SLIDER)
	dc.DrawIcon(*mpCTimeSliderIcon, mXDraggingFrame-7, 2);
    else
	dc.DrawIcon(*mpCTimeSliderIcon, mXCurrentFrame-7, 2);
    
    // bZ[W`
    int offset = 0;
    if (mMousePosCurrent > mWidth * 0.7) offset = -40;

    if (TRUE ||
	(mSelected == _TL_SEL_MIN || mSelected == _TL_SEL_MAX ||
	 mSelected == _TL_SEL_SLIDER)){
	wxString Mes;
	dc.SetFont(*wxSMALL_FONT);
	dc.SetTextForeground(*wxRED);
	
	Mes << wxT("current Frame=") << mCurrentFrame << wxT("+") << mCurrentStep << wxT("/") << mAnimInterp << wxT(": UsedMemoryMB=") << mpCCtrlLink->mpCMainLink->GetUsedTotalMemoryMB(); 
	//dc.DrawText(Mes, mMousePosCurrent + 15 + offset, 0);
	dc.DrawText(Mes, 10, 0);
    }
}

void TimeCtrlGUI_cls::
ForceUpdateDisplay(){
    wxPaintEvent com;
    OnPaint(com);
}

bool TimeCtrlGUI_cls::SetNewIndexDragging(){
    mpCCtrlLink->SetFrameNative(mFrame_Dragged, _COMMAND_UPDATE_DISPLAY_TL | _COMMAND_UPDATE_DISPLAY_SB);
    return TRUE;
}

bool TimeCtrlGUI_cls::SetNewMinDragging(){
    mpCCtrlLink->SetWAMinNative(mMin_Dragged); return TRUE;
}

bool TimeCtrlGUI_cls::SetNewMaxDragging(){
    mpCCtrlLink->SetWAMaxNative(mMax_Dragged); return TRUE;
}


void TimeCtrlGUI_cls::SetNewMinFix(int minAtClick){
    mpCCtrlLink->SetWAMinCommand(mMin_Dragged, minAtClick);
}

void TimeCtrlGUI_cls::SetNewMaxFix(int maxAtClick){
    mpCCtrlLink->SetWAMaxCommand(mMax_Dragged, maxAtClick);
}

void TimeCtrlGUI_cls::
OnSize(wxSizeEvent& event){
    Refresh(FALSE);
    event.Skip();
}

void TimeCtrlGUI_cls::
OnPaint(wxPaintEvent& event){
    wxAutoBufferedPaintDC dc(this);
    UpdateDisplay(dc);
    event.Skip();
}

void TimeCtrlGUI_cls::
OnMouse(wxMouseEvent& event){
    /*
    if (event.ButtonDown(wxMOUSE_BTN_RIGHT)){
	UpdateMenu();
#ifdef WIN32
	PopupMenu(mpPopupMenu);
#else
	PopupMenu(mpPopupMenu, event.GetX(), event.GetY());
#endif
    }
    */
    
    SetRegion(mAAMin, mAAMax);
    SetInitValue(event, mpCCtrlLink->mCWAMin.mBody, mpCCtrlLink->mCWAMax.mBody);

    ProcMouseEventBase(event);
    
    //SetFocus();

    event.Skip();
}
