// -*- c++ -*-
// (setq c-basic-offset 4)

#include "clsUndoItemFilterMake.h"

#include "../FILTER/clsFilterBase.h"
#include "../FILTER/clsFilterBG.h"
#include "../FILTER/clsFilterBoolean.h"
#include "../FILTER/clsFilterCtrl.h"
#include "../FILTER/clsFilterCopy.h"
#include "../FILTER/clsFilterDataBox.h"
#include "../FILTER/clsFilterFiltering.h"
#include "../FILTER/clsFilterFog.h"
#include "../FILTER/clsFilterGrid.h"
#include "../FILTER/clsFilterObject.h"
#include "../FILTER/clsFilterStick.h"
#include "../FILTER/clsFilterText.h"

#include "../PARAMETER/clsParamGroup.h"
#include "../../clsMain.h"

#include "../COMMON/CommonDefinitions.h"

CommandFilterMake_cls::
CommandFilterMake_cls(FilterCtrl_cls *pCTarget, int type, int pos,
		      FilterData_cls *pCData){
    mUpdateBit = _COMMAND_UPDATE_DISPLAY_FILTER | _COMMAND_UPDATE_DISPLAY_GL;
    mpCTarget = pCTarget;
    mType     = type;
    mPos      = pos;
    mpCFilterData = NULL;
    if (pCData != NULL){
	switch (mType){
	case _FILTER_BG:
	    mpCFilterData = new FilterBGData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	case _FILTER_BOOLEAN:
	    mpCFilterData = new FilterBooleanData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	case _FILTER_DATABOX:
	    mpCFilterData = new FilterDataBoxData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	case _FILTER_OBJECT:
	    mpCFilterData = new FilterObjectData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	case _FILTER_GRID:
	    mpCFilterData = new FilterGridData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	case _FILTER_TEXT:
	    mpCFilterData = new FilterTextData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	case _FILTER_FOG:
	    mpCFilterData = new FilterFogData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	case _FILTER_FILTERING:
	    mpCFilterData = new FilterFilteringData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	case _FILTER_COPY:
	    mpCFilterData = new FilterCopyData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	case _FILTER_STICK:
	    mpCFilterData = new FilterStickData_cls;
	    mpCFilterData->CopyFilterDataFromP(pCData); break;
	}
    }

    wxString strList[3] = {wxT("x"), wxT("y"), wxT("z")};
    mpCParamGroupTranslate = new ParamGroup_cls(wxT("Translate"), 3, strList,
						0, 0);
    mpCParamGroupRotation = new ParamGroup_cls(wxT("Rotate"), 3, strList,
						0, 0);
    mpCParamGroupScale = new ParamGroup_cls(wxT("Scale"), 3, strList,
					    0, 0);
    
    if (pCData != NULL){
	Parameters_cls::msbUndoCommandMake = FALSE;
	mpCParamGroupTranslate->CopyFromP(pCData->mpCTranslate);
	mpCParamGroupRotation->CopyFromP(pCData->mpCRotation);
	mpCParamGroupScale->CopyFromP(pCData->mpCScale);
	Parameters_cls::msbUndoCommandMake = TRUE;
    }

}

CommandFilterMake_cls::
~CommandFilterMake_cls(){

    if (mpCFilterData != NULL){
	switch(mType){
	case _FILTER_BG:
	    delete (FilterBGData_cls*)mpCFilterData; break;
	case _FILTER_TEXT:
	    delete (FilterTextData_cls*)mpCFilterData; break;
	case _FILTER_OBJECT:
	    delete (FilterObjectData_cls*)mpCFilterData; break;
	default:
	    delete mpCFilterData; break;
	}
    }

    delete mpCParamGroupScale;
    delete mpCParamGroupTranslate;
    delete mpCParamGroupRotation;
}

void CommandFilterMake_cls::
Execute(bool bFlag){
    Parameters_cls::msbUndoCommandMake = FALSE;
    if (mType == -1)
	mpCTarget->RemoveFilter(mPos);
    else
	mpCTarget->AddFilter(mType, mPos, mpCFilterData,
			     mpCParamGroupTranslate,
			     mpCParamGroupRotation,
			     mpCParamGroupScale);
    Parameters_cls::msbUndoCommandMake = TRUE;
    UpdateBit();
}

// --------------------------

CommandFilterMove_cls::
CommandFilterMove_cls(FilterCtrl_cls *pCTarget, int pos, int direction){
    mUpdateBit = _COMMAND_UPDATE_DISPLAY_FILTER | _COMMAND_UPDATE_DISPLAY_GL;
    mpCTarget  = pCTarget;
    mPos       = pos;
    mDirection = direction;
}

void CommandFilterMove_cls::
Execute(bool bFlag){
    mpCTarget->MoveFilterExe(mPos, mDirection);
    UpdateBit();
}
