// -*- c -*-
// "clsUndoManager.h"
// (setq c-basic-offset 4)

#ifndef _UNDO_MANAGER_H_
#define _UNDO_MANEGER_H_

#include "clsUndoManager.h"

#include "../PARTICLEDATA/clsSnapData.h"
extern bool GbSnapDataRefresh;

#include <wx/wx.h>
extern wxFrame *GpCMainFrameCLS;
#include "../../wxWidgetEnumMenu.h"

#include "../COMMON/CommonDefinitions.h"

UndoManager_cls GCUndoList;

CommandItem_cls::
CommandItem_cls(){
    mCommandType = _COMMAND_TYPE_NORMAL;
    mUpdateBit = 0;
}

CommandItem_cls::
~CommandItem_cls(){
}

void CommandItem_cls::
UpdateBit(){
    UpdateBit(mUpdateBit);
}

void CommandItem_cls::
UpdateBit(long long updateBit){
    
    GCUndoList.mUpdateAccumulateBit |= updateBit;
    
    // $B;~4VJQ2=$N$_$O$^$H$a$F=hM}$5$l$k$H:$$k(B
    if (GpCMainFrameCLS == 0) return; //$B=i4|@_Dj;~$K$^$@L5$$(B
    if ((updateBit & _COMMAND_UPDATE_DISPLAY_TL) != 0){ // static $B$J$N$GD>@\(B
	wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshTL);
	GpCMainFrameCLS->AddPendingEvent(com);
    }
    if ((updateBit & _COMMAND_UPDATE_DISPLAY_SB) != 0){ // static $B$J$N$GD>@\(B
	wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, menuEditRefreshSB);
	GpCMainFrameCLS->AddPendingEvent(com);
    }
}

// --------------------------

ComBlockOpen::
ComBlockOpen(wxString strUndoTag){
    mCommandType = _COMMAND_TYPE_BLOCK_OPEN;
    mstrUndoTag = strUndoTag;
}

void ComBlockOpen::
Execute(bool bFlag){
    GCUndoList.mBlockDepth++;
    //return FALSE;
}

ComBlockClose::
ComBlockClose(wxString strUndoTag){
    mCommandType = _COMMAND_TYPE_BLOCK_CLOSE;
    mstrUndoTag = strUndoTag;
}

void ComBlockClose::
Execute(bool bFlag){
    GCUndoList.mBlockDepth--;
    //return FALSE;
}

// --------------------------

UndoManager_cls::
UndoManager_cls(){
    mCurrentIndex = 0;
    mBlockDepth = 0;
    mUpdateAccumulateBit = 0;
}

UndoManager_cls::
~UndoManager_cls(){
    for (unsigned int i = 0; i < mVPCRedoList.size(); i++){
	delete mVPCRedoList[i];
	delete mVPCUndoList[i];
    }
}

wxString UndoManager_cls::
GetStrList(){
    wxString strList;
    for (unsigned int i = 0; i < mVPCRedoList.size(); i++){
	strList << i << mVPCRedoList[i]->mstrUndoTag;
	strList << wxT("[") << mVPCRedoList[i]->mstrID << wxT("]");
	if ((mVPCRedoList[i]->mUpdateBit & _COMMAND_UPDATE_DISPLAY_FILTER_PARAM) != 0)
	    strList << wxT("@");
	if (i == mCurrentIndex) strList << wxT("*");
	strList << wxT(":");
    }
    if (mCurrentIndex == mVPCRedoList.size()) strList << mCurrentIndex << wxT("*");
    return strList;
}

void UndoManager_cls::
Clear(){
    for (unsigned int i = 0; i < mVPCRedoList.size(); i++) RemoveItem(i);
    mVPCRedoList.clear();
    mVPCUndoList.clear();
    mCurrentIndex = 0;
}

void UndoManager_cls::
RemoveItem(unsigned int i){
    delete mVPCRedoList[i];
    delete mVPCUndoList[i];
}

void UndoManager_cls::
OpenBlock(wxString strUndoTag){
    PushNewCommand
	(new ComBlockOpen(strUndoTag), new ComBlockClose(strUndoTag));
    ExecCurrent();
}

void UndoManager_cls::
CloseBlock(wxString strUndoTag){
    int type = GetRedoCommandType();

    PushNewCommand
	(new ComBlockClose(strUndoTag), new ComBlockOpen(strUndoTag));
    ExecCurrent();

    // $B6u$C$]$N%V%m%C%/$J$i0UL#$,L5$$$N$G0l$DLa$k(B
    //UndoCurrent$B$GLa$k$K$O(BmBlockDepth$B$,(B0$B$G$J$$$H:$$k$N$GF~$l;RMQ$K2>$K(B0$B$K!#(B
    /*
    int blockDepthOrg = mBlockDepth;
    mBlockDepth = 0;
    if (type == _COMMAND_TYPE_BLOCK_OPEN)
	UndoCurrent(); 
    mBlockDepth = blockDepthOrg;
    */
}

void UndoManager_cls::
SendEventToFrame(int comID){
    wxCommandEvent com(wxEVT_COMMAND_MENU_SELECTED, comID);
    GpCMainFrameCLS->AddPendingEvent(com);
}

void UndoManager_cls::
ExecUpdate(){
    if (GpCMainFrameCLS == NULL) return; //$B=i4|@_Dj;~$K$^$@L5$$(B
    
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_TL) != 0)
	SendEventToFrame(menuEditRefreshTL);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_ATTRIB) != 0)
	SendEventToFrame(menuEditRefreshAttrib);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_GL) != 0){
        //GbSnapDataRefresh = TRUE;
	SendEventToFrame(menuEditRefreshGL);
    }
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_IMGSEQ) != 0)
	SendEventToFrame(menuEditRefreshIMGSEQ);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_CAMCONF) != 0)
	SendEventToFrame(menuEditRefreshCamConf);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_COLOR) != 0)
	SendEventToFrame(menuEditRefreshColor);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_EXPORTER) != 0)
	SendEventToFrame(menuEditRefreshExporter);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_DRAWOPTIONS) != 0)
	SendEventToFrame(menuEditRefreshDrawOptions);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_LIGHT) != 0)
	SendEventToFrame(menuEditRefreshLight);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_EXPOSURE) != 0)
	SendEventToFrame(menuEditRefreshExposure);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_STEREO) != 0)
	SendEventToFrame(menuEditRefreshStereo);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_FILTER) != 0)
	SendEventToFrame(menuFilterResize);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_FILTER_PARAM) != 0)
	SendEventToFrame(menuFilterRefresh);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_SNAPSHOT_DATA) != 0)
	GbSnapDataRefresh = TRUE;
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_TEXTURE) != 0)
	SendEventToFrame(menuEditRefreshTexture);
    if ((mUpdateAccumulateBit & _COMMAND_UPDATE_DISPLAY_SB) != 0)
	SendEventToFrame(menuEditRefreshSB);
    
    mUpdateAccumulateBit = 0;
}

void UndoManager_cls::
PushNewCommand(CommandItem_cls* pComRedo, CommandItem_cls* pComUndo){
    for (unsigned int i = mCurrentIndex; i < mVPCRedoList.size(); i++)
	RemoveItem(i);
    
    mVPCRedoList.resize(mCurrentIndex);
    mVPCUndoList.resize(mCurrentIndex);
    mVPCRedoList.push_back(pComRedo);
    mVPCUndoList.push_back(pComUndo);
}

#include "../MYAPI/MsgBox.h"

int UndoManager_cls::
GetRedoCommandType(int level){
    if (mCurrentIndex <= level) return _COMMAND_TYPE_NORMAL;
    if (mCurrentIndex - level > mVPCRedoList.size() - 1) return _COMMAND_TYPE_NORMAL;
    return mVPCRedoList[mCurrentIndex - level]->mCommandType;
}

int UndoManager_cls::
GetUndoCommandType(int level){
    if (mCurrentIndex <= level) return _COMMAND_TYPE_NORMAL;
    if (mCurrentIndex - level > mVPCUndoList.size() - 1) return _COMMAND_TYPE_NORMAL;
    return mVPCUndoList[mCurrentIndex - level]->mCommandType;
}

void UndoManager_cls::
SkipCurrent(){
    if (mCurrentIndex < mVPCRedoList.size()){
	mCurrentIndex++;
	if (mBlockDepth > 0) SkipCurrent();
    }
}

void UndoManager_cls::
ExecCurrent(){
    if (mCurrentIndex < mVPCRedoList.size() ){
	mVPCRedoList[mCurrentIndex]->Execute(_COMMAND_REDO);
	mCurrentIndex++;
	if (mBlockDepth > 0) ExecCurrent();
    }
    if (mBlockDepth == 0) ExecUpdate();
}

void UndoManager_cls::
UndoCurrent(){
    mCurrentIndex--;
    if (mCurrentIndex < 65535){ 
	mVPCUndoList[mCurrentIndex]->Execute(_COMMAND_UNDO);
	if (mBlockDepth > 0) UndoCurrent();
    }
    else 
	mCurrentIndex = 0;
    if (mBlockDepth == 0) ExecUpdate();
}

#endif
