// -*- c -*- 
// "clsDualSlider.h"

#include "clsDualSlider.h"
#include "../../wxWidgetEnum.h"
#include "../WIDGETCTRL/clsMultiInputPanel.h"

#include "../UNDO/clsUndoItemConstParam.h"
#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;

#include <wx/dcbuffer.h>

#ifndef TRUE
#define TRUE true
#define FALSE false
#endif

#define _BORDERWIDTH 5 

BEGIN_EVENT_TABLE(DualSlider_cls, wxPanel)
     EVT_PAINT       (DualSlider_cls::OnPaint)
     EVT_MOUSE_EVENTS(DualSlider_cls::OnMouse)
END_EVENT_TABLE()

DualSlider_cls::
DualSlider_cls(wxWindow *parent, InputItem_cls *pCParentInputItem,
	       wxWindowID id, 
	       int initvalue0, int initvalue1,
	       int min, int max,
	       wxPoint pos, wxSize size) :
wxPanel(parent, id, pos, size, wxNO_BORDER)
{
    SetBackgroundColour(wxColour(wxT("WHEAT")));
    SetBackgroundStyle(wxBG_STYLE_CUSTOM);

    mpCParentInputItem = pCParentInputItem;
    mValue[0] = initvalue0; mValue[1] = initvalue1;
    mMin = min; mMax = max;
    mDisplayMode = 0;
    mItemNum = 2;

    mSelected = -1;

    Connect(GetId(), wxEVT_KEY_DOWN,
	    wxKeyEventHandler(DualSlider_cls::OnKeyDown));
}

void DualSlider_cls::
SetMemberSize(wxSize size){
    mWidth = size.GetWidth();
    mHeight = size.GetHeight();
}

void DualSlider_cls::
SetValueDouble(double value0, double value1){
    mValue[0] = value0 * mScale;
    mValue[1] = value1 * mScale;
    Refresh(FALSE);
}

void DualSlider_cls::
UpdateValueFromSlider(){
    // do nothing
    // mValue͕\ƒĂ
}

void DualSlider_cls::
SetRangeSliderWrapperDouble(int min, int max, int scale){
    mMin = min; mMax = max; mScale = scale;
    //if (mScale > 10) SetPageSize(10);
    //SetRangeSliderWrapperInt(min, max);
    //SetRange(min, max);
}

double DualSlider_cls::
GetValue(int index){
    return (double)mValue[index] / mScale;
}

int DualSlider_cls::
GetValueFromPos(int posX){
    double normalValue = (double)(posX - _BORDERWIDTH) / (mWidth - _BORDERWIDTH*2);
    int value = normalValue * (mMax - mMin) + mMin;
    if (value < mMin) value = mMin;
    if (value > mMax) value = mMax;
    return value;
}

int DualSlider_cls::
GetSliderPosition(int index){
    return GetSliderPositionFromValue(mValue[index]);
}

int DualSlider_cls::
GetSliderPositionFromValue(float value){
    float normalValue = (float)(value - mMin) / (mMax - mMin);
    return _BORDERWIDTH + (mWidth - _BORDERWIDTH*2) * normalValue;
}

void DualSlider_cls::
UpdateDisplay(){
    wxAutoBufferedPaintDC dc(this);

    dc.SetBackground(wxBrush(wxColour(191, 216, 216), wxSOLID));
    dc.Clear();

    SetMemberSize(GetClientSize());

    dc.SetPen(*wxGREY_PEN);
    dc.SetBrush(*wxGREY_BRUSH);
    dc.DrawRectangle(_BORDERWIDTH, 7, mWidth - _BORDERWIDTH*2, 3);
    dc.SetPen(*wxLIGHT_GREY_PEN);
    dc.DrawRectangle(_BORDERWIDTH+1, 8, mWidth - _BORDERWIDTH*2 - 1, 2);

    dc.SetPen(*wxGREY_PEN);
    dc.SetBrush(*wxWHITE_BRUSH);
    for (int i = 0; i <= mMax/mScale; i++){
	dc.DrawRectangle
	    (GetSliderPositionFromValue((float)i*mScale)-1, 7, 3, 3);
    }

    wxPoint listPoint[3];
    listPoint[0] = wxPoint(0, 0);
    listPoint[1] = wxPoint(-5, 10);
    listPoint[2] = wxPoint(5, 10);

    if (IsEnabled() == TRUE)
	for (int i = 0; i < mItemNum; i++){
	    if (mSelected == i)
		dc.SetPen(*wxRED_PEN);
	    else
		dc.SetPen(*wxBLACK_PEN);
	    if (i == 0)
		dc.SetBrush(*wxBLACK_BRUSH);
	    else
		dc.SetBrush(*wxWHITE_BRUSH);
	    dc.DrawPolygon(3, listPoint, GetSliderPosition(i), 8);
	}
    else
	for (int i = 0; i < mItemNum; i++){
	    dc.SetPen  (*wxGREY_PEN);
	    dc.SetBrush(*wxGREY_BRUSH);
	    dc.DrawPolygon(3, listPoint, GetSliderPosition(i), 8);
	}

    for (int i = 0; i < mItemNum; i++){
	wxString Mes;
	if (mDisplayMode == 0)
	    Mes << mValue[i];
	else
	    Mes << wxString::Format(wxT("%0.2f"), ((float)mValue[i] / mScale));
	
	int pos = GetSliderPosition(i);
	if (pos > mWidth / 2) pos -= 32;
	else pos+=8;
	dc.SetFont(*wxSMALL_FONT);
	if (IsEnabled() == TRUE){
	    if (i == mSelected)
		dc.SetTextForeground(*wxRED);
	    else
		dc.SetTextForeground(*wxBLACK);
	}
	else
	    dc.SetTextForeground(wxColour(128, 128, 128));
	dc.DrawText(Mes, pos, 8);
    }
}

void DualSlider_cls::
OnPaint(wxPaintEvent& event){
    event.Skip();
    UpdateDisplay();
}

void DualSlider_cls::
CalcSelected(wxMouseEvent& event){
    mSelected = -1;
    int posX = event.GetX();
    int spos0 = GetSliderPosition(0);
    int spos1 = GetSliderPosition(1);
    if (posX > GetSliderPosition(0) - 4 && posX < GetSliderPosition(0) + 4)
	mSelected = 0;
    if (posX > GetSliderPosition(1) - 4 && posX < GetSliderPosition(1) + 4)
	mSelected = 1;
    return;
}

void DualSlider_cls::
CreateKeyDLG(int xpos, int ypos){

    wxString strLabel = mpCParentInputItem->mstrLabel;
    MultiInputDialog_cls *pCDLG = new
	MultiInputDialog_cls(this, wxID_ANY, strLabel,
			     GetScreenPosition() + wxPoint(xpos-40, ypos-20));
    MultiInputPanel_cls *pCurr = pCDLG->mpCPanel;
    
    double value[2];
    value[0] = (double)mValue[0]/mScale; 
    value[1] = (double)mValue[1]/mScale;
    double min = mMin/mScale, max = mMax/mScale;

    pCurr->mTextWidth = 60;
    
    int boxWidth = 2;
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("Min/Max"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &min);
    pCurr->CombinePointer(_ITEM_DOUBLE, &max);
    pCurr->CombinePointer(_ITEM_BOXSIZERHORIZONTAL, &boxWidth, _("value"));
    pCurr->CombinePointer(_ITEM_DOUBLE, &value[0]);
    pCurr->CombinePointer(_ITEM_DOUBLE, &value[1]);

    pCDLG->MakeGeometry();
    if (pCDLG->ShowModal() == wxID_OK){
	mMax = max * mScale;
	mMin = min * mScale;
	mValue[0] = value[0] * mScale;
	mValue[1] = value[1] * mScale;
	if (mpCParentInputItem != NULL)
	    mpCParentInputItem->Assign();
    }
}

void DualSlider_cls::
OnMouse(wxMouseEvent& event){
    if (event.LeftDClick())
	CreateKeyDLG(event.GetX(), event.GetY());

    if (event.ButtonDown(wxMOUSE_BTN_LEFT)){
	CalcSelected(event);
	if (mSelected != -1){
	    CaptureMouse();
	    mbCaptureMouse = TRUE;
	}
    }

    if (event.Dragging() == TRUE){
	mDraggingPos = event.GetX();
	if (mSelected != -1)
	    if (mValue[mSelected] != GetValueFromPos(mDraggingPos)){
		mValue[mSelected]  = GetValueFromPos(mDraggingPos);
		if (event.m_shiftDown == TRUE){
		    wxCommandEvent com(wxEVT_SCROLL_THUMBRELEASE);
		    com.SetId(GetId());
		    GetParent()->AddPendingEvent(com);
		}
	    }
    }

    if (event.ButtonUp(wxMOUSE_BTN_LEFT))
	if (mbCaptureMouse == TRUE){
	    ReleaseMouse();
	    mbCaptureMouse = FALSE;
	    mSelected = -1;
	    wxCommandEvent com(wxEVT_SCROLL_THUMBRELEASE);
	    com.SetId(GetId());
	    GetParent()->AddPendingEvent(com);
	}


    event.Skip();
    Refresh(FALSE);
}

wxWindow* DualSlider_cls::
GetPWindow(){
    return this;
}

void DualSlider_cls::
OnKeyDown(wxKeyEvent& event){
    //ProcessKeyEvent(event);
}
