// -*- c -*- 
// "clsGraphPanel.h"

#include "clsGraphPanel.h"
#include "../PARAMETER/clsParamGroup.h"

#include "../WIDGETCTRL/EX/clsColourDialogEX.h"
extern wxColourDialogEX* GpCClrDialog;

#include "../../wxWidgetEnum.h"
#include "../../wxWidgetEnumMenu.h"

extern wxFrame  *GpCMainFrameCLS;

BEGIN_EVENT_TABLE(GraphPanel_cls, wxPanel)
EVT_PAINT       (GraphPanel_cls::OnPaint)
EVT_KEY_DOWN    (GraphPanel_cls::OnKeyDown)
EVT_MOUSE_EVENTS(GraphPanel_cls::OnMouse)
EVT_MENU        (GraphMenuInsertKey, GraphPanel_cls::OnInsertKey)
EVT_MENU        (GraphMenuDeleteKey, GraphPanel_cls::OnDeleteKey)
EVT_MENU        (GraphMenuColor,     GraphPanel_cls::OnColor)
END_EVENT_TABLE()

#define X 0
#define Y 1

#define LOOP_RGB for (int rgb = 0; rgb < 3; rgb++)

GraphPanel_cls::
GraphPanel_cls(wxWindow *parent, wxWindowID id, 
	       wxPoint pos, wxSize size, bool bColor) :
wxPanel(parent, id, pos, size, wxFULL_REPAINT_ON_RESIZE)
{
    SetBackgroundStyle(wxBG_STYLE_CUSTOM);
    mbInitialized = FALSE;
    mbSelectGuide = FALSE;
    mbSelectRegion = FALSE;
    mViewMinY = 0; mViewMaxY = 1;
    mValueMinY = 0;
    mValueMaxY = 1;
    mViewMinX = 0; mViewMaxX = 1;
    mValueMinX = 0;
    mValueMaxX = 1;
    mGuideX = 0;

    mVTicsX.push_back(0); mVTicsX.push_back(1);
    mVTicsY.push_back(0); mVTicsY.push_back(1);

    mpCParamGroupLinked = NULL;
    mColorType = _GRAPH_RGBA;

    mpCPopupMenu = new wxMenu;
    mpCPopupMenu->Append(GraphMenuInsertKey, _("Insert Key"));
    mpCPopupMenu->Append(GraphMenuDeleteKey, _("Delete Key"));

    if (bColor == TRUE)
	mpCPopupMenu->Append(GraphMenuColor, _("Select Color"));
}

GraphPanel_cls::
~GraphPanel_cls(){
    delete mpCPopupMenu;
}

void GraphPanel_cls::
SetParamGroup(ParamGroup_cls *pCParamGroup){
    mpCParamGroupLinked = pCParamGroup;
}

void GraphPanel_cls::
ShowMenu(){
#ifdef WIN32
    PopupMenu(mpCPopupMenu, mCoordX, mCoordY);
#else
    PopupMenu(mpCPopupMenu);
#endif
    if (HasCapture() == TRUE) ReleaseMouse();
}

int GraphPanel_cls::
GetCoordX(double valueX){
    double doubleX = (valueX - mViewMinX) / (mViewMaxX - mViewMinX);
    return doubleX * (mPanelWidth - 2) + 1;
}

int GraphPanel_cls::
GetCoordY(double valueY){
    double doubleY = (valueY - mViewMinY) / (mViewMaxY - mViewMinY);
    return (mPanelHeight - 2) - doubleY * (mPanelHeight - 2) + 1;
}

double GraphPanel_cls::
GetDoubleX(int coordX){
    double XinPanel = ((double)coordX - 1) / (mPanelWidth - 2);
    double XValue = XinPanel * (mViewMaxX - mViewMinX) + mViewMinX;
    if (XValue < mValueMinX) XValue = mValueMinX;
    if (XValue > mValueMaxX) XValue = mValueMaxX;
    return XValue;
}

double GraphPanel_cls::
GetDoubleY(int coordY){
    double YinPanel =1.0 - ((double)coordY - 1) / (mPanelHeight - 2);
    double YValue = YinPanel * (mViewMaxY - mViewMinY) + mViewMinY;
    if (YValue < mValueMinY) YValue = mValueMinY;
    if (YValue > mValueMaxY) YValue = mValueMaxY;
    return YValue;
}

void GraphPanel_cls::
GetSnippedDelta(int &dx, int &dy){
    dx = (mCoordX - mCoordXAtClick);
    dy = (mCoordY - mCoordYAtClick);

    double thetaDeg =  atan2(dy, dx) * 180 / 3.14159;
    if (dx > 50 || dx < -50){
	if (dy < 25 && dy > -25) dy = 0;
    }
    else{
	if (thetaDeg < 15 && thetaDeg > -15)   dy = 0;
	if (thetaDeg > 165 || thetaDeg < -165) dy = 0;
    }
    if (dy > 50 || dy < -50){
	if (dx < 25 && dx > -25) dx = 0;
    }
    else{
	if (thetaDeg > 75 && thetaDeg < 105)   dx = 0;
	if (thetaDeg < -75 && thetaDeg > -105) dx = 0;
    }
}

void GraphPanel_cls::
GetCurrentColor(float clr[3]){
    if (mpCParamGroupLinked != NULL){
	std::vector < double > VValue = 
	    mpCParamGroupLinked->GetInterpolatedValue(GetDoubleX(mGuideX), 0);
	LOOP_RGB clr[rgb] = VValue[rgb];
    }
    else
	LOOP_RGB clr[rgb] = 0;
}

void GraphPanel_cls::
SetGuide(double doubleIndex){
    mGuideX = GetCoordX(doubleIndex);
    Refresh();
}

void GraphPanel_cls::
SetXViewRange(double minX, double maxX){
    mViewMinX = minX; mViewMaxX = maxX;
}

void GraphPanel_cls::
SetYViewRange(double minY, double maxY){
    mViewMinY = minY; mViewMaxY = maxY;
}

void GraphPanel_cls::
SetPenColor(wxAutoBufferedPaintDC &dc, int i){
    wxPen pen = wxPen(msColorRGBA[i]);
    if (i == mpCParamGroupLinked->mControlMode) pen.SetWidth(2);
    if (mColorType == _GRAPH_RGBA) dc.SetPen(pen);
}

void GraphPanel_cls::
SetBrushColor(wxAutoBufferedPaintDC &dc, int i){
    if (mColorType == _GRAPH_RGBA) dc.SetBrush(wxBrush(msColorRGBA[i]));
}

void GraphPanel_cls::
DrawGraphOfParamGroup(wxAutoBufferedPaintDC &dc){
    
    int dim = mpCParamGroupLinked->mCDimension.mBody;
    std::vector < std::vector < wxPoint > > VVPoint(dim);
    int xstep = 1;
    if (mPanelWidth > 320) xstep = 2;
    if (mPanelWidth > 640) xstep = 4;
    for (int x = 0; x < mPanelWidth; x += xstep){
	double valueX = GetDoubleX(x);;
	std::vector < double > VValue = 
	    (mpCParamGroupLinked->GetInterpolatedValue(valueX, 0));
	for (int i = 0; i < dim; i++)
	    VVPoint[i].push_back(wxPoint(GetCoordX(valueX), GetCoordY(VValue[i])));
    }
    
    for (int i = 0; i < dim; i++){
	SetPenColor(dc, i);
	for (int x = 0; x < mPanelWidth; x+=xstep)
	    dc.DrawLines(mPanelWidth / xstep, VVPoint[i].data());
    }
    
    for (int i = 0; i < dim; i++){
	SetPenColor(dc, i);
	Parameters_cls* pCParam = &(mpCParamGroupLinked->mVCParam[i]);
	for (int key = 0; key < pCParam->GetKeyNumber(); key++){
	    int coordX = GetCoordX(pCParam->mVCKeys[key].mDoubleIndex);
	    int coordY = GetCoordY(pCParam->mVCKeys[key].mValue);
	    if (pCParam->mVCKeys[key].mbSelected == TRUE){
		SetBrushColor(dc, i);
		if (HasCapture() == TRUE){
		    int dx, dy;
		    GetSnippedDelta(dx, dy);
		    dc.DrawRectangle(coordX-3 + dx/*(mCoordX - mCoordXAtClick)*/,
				     coordY-3 + dy/*(mCoordY - mCoordYAtClick)*/,
				     7, 7);
		}
	    }
	    else
		dc.SetBrush(*wxTRANSPARENT_BRUSH);
	    dc.DrawRectangle(coordX-3, coordY-3, 7, 7);
	}
    }
}


void GraphPanel_cls::
DrawTicsBox(wxAutoBufferedPaintDC& dc){
    
    wxDash dash1[2];
    dash1[0] = 2;  // Long dash
    dash1[1] = 3;  // Short gap

    wxPen CPen(wxColor(128, 128, 128), 1, wxUSER_DASH);
    CPen.SetDashes( 2, dash1 );

    dc.SetPen(CPen);
    
    for (unsigned int i = 0; i < mVTicsX.size(); i++){
	int coordX = GetCoordX(mVTicsX[i]);
	dc.DrawLine(coordX, 0, coordX, mPanelHeight);
    }
    for (unsigned int i = 0; i < mVTicsY.size(); i++){
	int coordY = GetCoordY(mVTicsY[i]);
	dc.DrawLine(0, coordY, mPanelWidth, coordY);
    }
}

void GraphPanel_cls::
DrawSelectionBox(wxAutoBufferedPaintDC& dc){
    if (mbSelectRegion == TRUE){
	wxRect rectSelect0(mCoordXAtClick, mCoordYAtClick,
			   mCoordX - mCoordXAtClick,
			   mCoordY - mCoordYAtClick);
	dc.DrawRectangle(rectSelect0);
    }
}

void GraphPanel_cls::
OnPaint(wxPaintEvent &event){
    wxAutoBufferedPaintDC dc(this);

    mPanelWidth = GetClientSize().GetWidth();
    mPanelHeight = GetClientSize().GetHeight();
    wxRect rectToDraw (0, 0, mPanelWidth, mPanelHeight);

    if (mbInitialized == FALSE){
	mGuideX = GetCoordX(0);
	mbInitialized = TRUE;
    }
        
    dc.SetPen(*wxBLACK_PEN);
    dc.SetBrush(wxBrush(*wxLIGHT_GREY));
    dc.DrawRectangle(rectToDraw);

    dc.SetPen(*wxGREY_PEN);
    DrawSelectionBox(dc);

    if (mpCParamGroupLinked != NULL) DrawGraphOfParamGroup(dc);

    DrawTicsBox(dc);

    dc.SetPen(*wxWHITE_PEN);
    dc.DrawLine(mGuideX, 0, mGuideX, mPanelHeight);
}

void GraphPanel_cls::
SelectionCheck(bool bAdditive){
    if (mpCParamGroupLinked->
	SelectSelectedKeyByXY(mValueX, 0.03, mValueY, 0.03) == FALSE &&
	bAdditive == FALSE)
	mpCParamGroupLinked->DeselectAll();

    if (mpCParamGroupLinked->
	SelectKeyByXY(mValueX, 0.03, mValueY, 0.03) == FALSE){

	if (mCoordX > mGuideX - 6 && mCoordX < mGuideX + 6)
	    mbSelectGuide = TRUE;
	else
	    mbSelectRegion = TRUE;
    }
}

void GraphPanel_cls::
OnMouse(wxMouseEvent &event){
    if (mpCParamGroupLinked == NULL) return;
    int x = event.GetX();
    int y = event.GetY();

    bool bRedraw = FALSE;
    bool bRedrawButton = mbSelectGuide;
    if (mbSelectGuide == TRUE)
	mGuideX = mCoordX;
    
    if (x != mCoordX || y != mCoordY){
	mCoordX = x; mCoordY = y;
	bRedraw = TRUE;
    }

    mValueX = GetDoubleX(x); 
    mValueY = GetDoubleY(y);

    if (event.ButtonDown(wxMOUSE_BTN_LEFT)){
	SelectionCheck(event.m_shiftDown);
	mCoordXAtClick = x; mCoordYAtClick = y;
	mValueXAtClick = mValueX; mValueYAtClick = mValueY;
	CaptureMouse();
	SetFocus();
    }
    
    if (event.ButtonDown(wxMOUSE_BTN_RIGHT)) ShowMenu();

    if (event.ButtonUp(wxMOUSE_BTN_LEFT)){
	if (HasCapture() == TRUE){
	    bRedraw = TRUE;
	    ReleaseMouse();
	    if (mbSelectGuide == TRUE) {
		mbSelectGuide = FALSE;
		if (mGuideX < GetCoordX(mValueMinX)) mGuideX = GetCoordX(mValueMinX);
		if (mGuideX > GetCoordX(mValueMaxX)) mGuideX = GetCoordX(mValueMaxX);
	    }
	    else if (mbSelectRegion == TRUE){
		mbSelectRegion = FALSE;
		mpCParamGroupLinked->SelectKeyFrameRegion
		    (mValueXAtClick, mValueX, mValueYAtClick, mValueY);
	    }
	    else{
		int dx, dy;
		GetSnippedDelta(dx, dy);
		double valueX = GetDoubleX(mCoordXAtClick + dx);
		double valueY = GetDoubleY(mCoordYAtClick + dy);
		mpCParamGroupLinked->
		    MoveSelectedKeyByXY(valueX - mValueXAtClick,
					valueY - mValueYAtClick,
					0.01, 0, 1, mValueMinY, mValueMaxY);
	    }
	}
    }

    if (bRedraw == TRUE) Refresh(FALSE);
    if (bRedrawButton == TRUE) 
	SendMenuEvent(GetParent(), menuClrRefreshButton);
}

void GraphPanel_cls::
InsertKey(){
    mpCParamGroupLinked->InsertKey(_PARAM_ALL, GetDoubleX(mGuideX), _PARAM_MAGICNUM, 1.0/255.0);
    mpCParamGroupLinked->DeselectAll();
    mpCParamGroupLinked->SelectKeyFrameRegion(GetDoubleX(mGuideX) - 0.001,
					      GetDoubleX(mGuideX) + 0.001,
					      0, 0);
}
void GraphPanel_cls::
DeleteKey(){
    mpCParamGroupLinked->DeleteSelectedKey();
}

void GraphPanel_cls::
SetKeyColor(){
    float clr[3];// = {VValue[0], VValue[1], VValue[2]};
    GetCurrentColor(clr);
    GpCClrDialog->SetColourFloat(clr);
    Freeze();
    if (GpCClrDialog->ShowModal() == wxID_OK){
	double newclr[4] = {0, 0, 0, 1};
	LOOP_RGB newclr[rgb] = GpCClrDialog->GetPColorFloat()[rgb];
	mpCParamGroupLinked->SetKeyFrame(newclr, TRUE, GetDoubleX(mGuideX));
    }
    Thaw();
}

void GraphPanel_cls::
OnInsertKey(wxCommandEvent& event){
    InsertKey();
}

void GraphPanel_cls::
OnDeleteKey(wxCommandEvent& event){
    DeleteKey();
}

void GraphPanel_cls::
OnColor(wxCommandEvent& event){
    SetKeyColor();
}

void GraphPanel_cls::
SetMode(int mode){
    if (mpCParamGroupLinked->mControlMode == mode)
	mpCParamGroupLinked->mControlMode = -1;
    else
	mpCParamGroupLinked->mControlMode = mode;
}

void GraphPanel_cls::
OnKeyDown(wxKeyEvent& event){
    if (event.GetModifiers() == wxMOD_NONE){ // _FIX_ME!! F̂Ƃ
	if (event.GetKeyCode() == 'R') SetMode(0);
	else if (event.GetKeyCode() == 'G') SetMode(1);
	else if (event.GetKeyCode() == 'B') SetMode(2);
	else if (event.GetKeyCode() == 'A') SetMode(3);
	else SetMode(-1);

	if (event.GetKeyCode() == WXK_DELETE) DeleteKey();

	SendMenuEvent(GetParent(), menuClrRefreshButton);
	Refresh(FALSE);
    }
    if (event.GetModifiers() == wxMOD_CONTROL)
	switch(event.GetKeyCode()){
	case 'Z': 
	case 'Y': 
	    SendKeyEventToTop(GpCMainFrameCLS, event);
	    break;
	}
    event.Skip();
}

wxColour GraphPanel_cls::msColorRGBA[4] = {
    wxColour(255, 0, 0), wxColour(0, 255, 0), wxColour(0, 0, 255), 
    wxColour(255,255,255) };
