// -*- c -*- 
// "clsIconSelector.h"

#include "clsIconSelector.h"

#include <wx/dcbuffer.h>
#include <wx/imaglist.h>

#include "../COMMON/CommonDefinitions.h"

BEGIN_EVENT_TABLE(IconSelector_cls, wxPanel)
     EVT_PAINT       (IconSelector_cls::OnPaint)
     EVT_MOUSE_EVENTS(IconSelector_cls::OnMouse)
END_EVENT_TABLE()

IconSelector_cls::
IconSelector_cls(wxWindow *parent, wxWindowID id, 
		 int initvalue, int max,
		 std::vector < wxIcon > *pVIcon,
		 wxPoint pos, wxSize size) :
     wxPanel(parent, id, pos, size/*, wxNO_BORDER*/)
{
    SetBackgroundColour(wxColour(216, 216, 216));
    SetBackgroundStyle(wxBG_STYLE_CUSTOM);

    mValue = initvalue;
    mMax = max;
#ifdef WIN32
    mFontHeight = 14;
#else
    mFontHeight = 18;
#endif
    mClientHeight = -1;
    mpImageListSmall = new wxImageList(16, 16, true);
    RemakeImageList(pVIcon);
}

IconSelector_cls::
~IconSelector_cls(){
    delete mpImageListSmall;
}

void IconSelector_cls::
RemakeImageList(std::vector < wxIcon > *pVIcon){
    mpImageListSmall->RemoveAll();
    for (unsigned int i = 0; i < pVIcon->size(); i++)
	mpImageListSmall->Add((*pVIcon)[i]);

    mMax = pVIcon->size();
    Refresh(FALSE);
}

void IconSelector_cls::
UpdateDisplay(wxAutoBufferedPaintDC &dc){
    if (mClientHeight == -1){
	mClientWidth = GetClientSize().GetWidth();
	mClientHeight = GetClientSize().GetHeight();
    }

    int offsetY = (mClientHeight - 20) / 2;
    
    dc.SetBackground(wxBrush(wxColour(216,216,216), wxSOLID));
    dc.Clear();

    dc.SetFont(*wxNORMAL_FONT);
    dc.SetPen(*wxTRANSPARENT_PEN);
    dc.SetBrush(wxColour(128,128,128));
    dc.DrawRectangle(0, offsetY, mMax*20, 20);
    
    for ( int i = 0; i < mMax; i++ ){
	dc.SetBrush(wxColour(255, 255, 0));
	if (i == mValue)
	    dc.DrawRectangle(i*20, offsetY, 20, 20);
	if (mpImageListSmall->GetImageCount() == 1){
	    wxIcon localIcon;
	    localIcon.CopyFromBitmap(mpImageListSmall->GetBitmap(0));
	    dc.DrawIcon(localIcon, i*20+2, 2 + offsetY);
	}
	else if (mpImageListSmall->GetImageCount() == mMax){
	    wxIcon localIcon;
	    localIcon.CopyFromBitmap(mpImageListSmall->GetBitmap(i));
	    dc.DrawIcon(localIcon, i*20+2, 2 + offsetY);
	}
    }

    if (mMax > 0){
	wxString Mes; Mes << mValue << wxT("/") << mMax;
	int pos = mClientWidth - 24; 
	if (mMax*20+4 > pos) pos = mMax*20+4;
	dc.DrawText(Mes, pos, (20-mFontHeight)/2);
    }
}

void IconSelector_cls::
OnPaint(wxPaintEvent& event){
    event.Skip();
    wxAutoBufferedPaintDC dc(this);
    UpdateDisplay(dc);
}

int IconSelector_cls::
GetSelected(wxMouseEvent& event){
    int posX = event.GetX();
    int mSelected = posX / 20;
    if (mSelected < mMax) return mSelected;
    return -1;
}

void IconSelector_cls::
OnMouse(wxMouseEvent& event){
    if (event.ButtonDown(wxMOUSE_BTN_LEFT)){
	int selected = GetSelected(event);
	if ( selected == -1) return;
	mValue = selected;

	wxCommandEvent com(wxEVT_COMMAND_COMBOBOX_SELECTED);
	com.SetId(GetId());
	GetParent()->AddPendingEvent(com);

	Refresh(FALSE);
    }
}

int IconSelector_cls::
GetValueInt(){
    return mValue;
}

void IconSelector_cls::
SetValueInt(int value){
    mValue = value;
    Refresh(FALSE);
}

wxWindow* IconSelector_cls::
GetPWindow(){
    return this;
}
