// -*- c -*- 
// "clsInputItem.h"

#include "../FILTER/clsFilterCtrl.h"

#include "../UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "../UNDO/clsUndoItemConstParam.h"
#include "../UNDO/clsUndoItemFilterParam.h"

#include "clsInputItem.h"
#include "EX/clsButtonEX.h"
#include "EX/clsCheckBoxEX.h"
#include "EX/clsStaticTextEX.h"
#include "clsAtoiComboCtrl.h"
#include "clsAtoiSpinCtrl.h"
#include "clsAtoiTextCtrl.h"
#include "clsAtofTextCtrl.h"
#include "clsAtoSTextCtrl.h"
#include "clsIconSelector.h"
#include "clsSliderInt.h"
#include "clsSliderDouble.h"
#include "clsSliderFloat.h"
#include "clsDualSlider.h"

#include "../COMMON/CommonDefinitions.h"

#include <wx/valgen.h>

InputItem_cls::
InputItem_cls() :
mpCCtrl(NULL), mpValue(NULL), mpVIcon(NULL), 
    mpCTextLabel(NULL), mpColorFloat(NULL)
{
    mUpdateBit = 0;
    mItemWidth = -1;
    //mExpand = _GP_EXPAND;
    mUndoType = 1; 
    mpCTgtFilterCtrlLink = NULL;
    mSizerIndex = -1;
    mMin = 0; mMax = 0;
    mStyle = 0;
}

// ----

wxSizerFlags InputItem_cls::
GetSizerFlagLabel(){
    if (mTextWidth == -1){
	wxSizerFlags sflag1(1);
	sflag1.Border(wxLEFT, 2).FixedMinSize().Align(wxALIGN_CENTER_VERTICAL);
	return sflag1;
    }
    
    wxSizerFlags sflag0(0);
    sflag0 .Border(wxLEFT, 2).FixedMinSize().Align(wxALIGN_CENTER_VERTICAL);
    return sflag0;
}

wxSizerFlags InputItem_cls::
GetSizerFlagH(int borderWidth){
    if (mItemWidth == -1){
	wxSizerFlags sflag(1);
	sflag.Expand().Border(wxLEFT, borderWidth).FixedMinSize();
	if (mAlign == _ITEM_ALIGN_RIGHT) 
	    sflag.Align(wxRIGHT);
	return sflag;
    }
    else {
	wxSizerFlags sflag(0);
	sflag.Border(wxLEFT, borderWidth).FixedMinSize().Align(wxALIGN_CENTER_VERTICAL);
	if (mAlign == _ITEM_ALIGN_RIGHT) 
	    sflag.Align(wxRIGHT);
	return sflag;
    }
    wxSizerFlags sflag(0);
    return sflag; // _~[
}

wxSizerFlags InputItem_cls::
GetSizerFlagV(){
    wxSizerFlags sflag;
    sflag .Border(wxLEFT, 2).FixedMinSize();

    if (mAlign == _ITEM_ALIGN_RIGHT) sflag.Right();
    else sflag.Expand();
    
    return sflag;
}


bool InputItem_cls::
IsButton(){
    if (/*mType == _ITEM_BUTTON_HIDE_PANEL ||*/
	mType == _ITEM_BUTTON)
	return TRUE;

    return FALSE;
}

void InputItem_cls::
Show(bool bShow){
    if (bShow == FALSE) {
	Hide(); return;
    }
    
    if (mType != _ITEM_BOXSIZERHORIZONTAL)
	GetPCwxWindow()->Show();

    if (mpCTextLabel != NULL) mpCTextLabel->Show();
}

void InputItem_cls::
Hide(){
    if (mType != _ITEM_BOXSIZERHORIZONTAL)
	GetPCwxWindow()->Hide();

    if (mpCTextLabel != NULL) mpCTextLabel->Hide();
}

void InputItem_cls::
Enable(bool bEnable){
    if (mType != _ITEM_BOXSIZERHORIZONTAL)
	GetPCwxWindow()->Enable(bEnable);

    if (mpCTextLabel != NULL) mpCTextLabel->Enable(bEnable);
}

void InputItem_cls::
SetParameter(int sizerCounter, 
	     int textWidth, int checkboxWidth, int comboWidth, int itemWidth){
    mTextWidth = textWidth;
    if (sizerCounter > 0 && mType == _ITEM_BOOL) mItemWidth = checkboxWidth;
    if (sizerCounter > 0 && mType == _ITEM_UINT_LIST) mItemWidth = comboWidth;
    else mItemWidth = itemWidth;
}

void InputItem_cls::
CombinePointerButton(int id, wxString strLabel, int type, bool bEnable){
    mType = type;
    mId = id;
    mpValue = NULL;
    mstrLabel = strLabel;
    mbEnable = bEnable;
    mbConnectSizer = FALSE;
}

void InputItem_cls::
CombinePointer(int type, void* pValue, wxString strLabel, bool bEnable){
    mType = type;
    mpValue = pValue;
    mstrLabel = strLabel;
    mbEnable = bEnable;
    mbConnectSizer = FALSE;
}

void InputItem_cls::
CombineList(unsigned int listMax, wxString *pChoiceList){
    mListMax = listMax;
    for (unsigned int i = 0; i < mListMax; i++)
	mVstrChoiceList.push_back(pChoiceList[i]);
}

void InputItem_cls::
SetRange(double min, double max, int scale){
    mMin = min; mMax = max; mScale = scale;
}

void InputItem_cls::
ApplyRange(){
    if (mMin == 0 && mMax == 0) return;
    switch (mType){
    case _ITEM_INTEGER:      case _ITEM_UINT:
	((AtoiTextCtrl_cls*)mpCCtrl)->SetRange((int)mMin, (int)mMax);
	break;
    case _ITEM_INTEGER_SPIN:
	((AtoiSpinCtrl_cls*)mpCCtrl)->SetRange((int)mMin, (int)mMax);
	break;
    case _ITEM_UINT_ICON:
	((IconSelector_cls*)mpCCtrl)->mMax = (int)mMax;
	break;
    case _ITEM_DUALSLIDER:
	((DualSlider_cls*)mpCCtrl)->
	    SetRangeSliderWrapperDouble((int)mMin, (int)mMax, mScale);
	break;
    }
}

bool InputItem_cls::
GetValueBool(){
    if (mpValue == NULL) return 0;
    if (mType == _ITEM_BOOL)
	return *(bool*)mpValue;
    else
	;//MsgBoxExit(NULL, "Called GetValueBool() for Not-Bool member", "Error");
    return 0;
}

float InputItem_cls::
GetValueFloat(){
    if (mpValue == NULL) return 0;
    if (mType == _ITEM_FLOAT || mType == _ITEM_FLOAT_SLIDER)
	return *(float*)mpValue;
    else 
	;//MsgBoxExit(NULL, "Called GetValueFloat() for Not-Float member", "Error");
    return 0;
}

double InputItem_cls::
GetValueDouble(int index){
    if (mpValue == NULL) return 0;
    if (mType == _ITEM_DOUBLE || mType == _ITEM_DOUBLE_SLIDER)
	return *(double*)mpValue;
    else if (mType == _ITEM_DUALSLIDER)
	return *((double*)mpValue + index);
    else 
	;//MsgBoxExit(NULL, "Error", "Called GetValueDouble() for Not-Double member");
    return 0;
}

int InputItem_cls::
GetValueInt(){
    if (mpValue == NULL) return 0;
    if (mType == _ITEM_INTEGER || mType == _ITEM_INTEGER_SLIDER || mType == _ITEM_INTEGER_SPIN)
	return *(int*)mpValue;
    else if (mType == _ITEM_UINT || mType == _ITEM_UINT_LIST ||
	     mType == _ITEM_UINT_SPIN || mType == _ITEM_UINT_ICON)
	return *(unsigned int*)mpValue;
    else 
	;//MsgBoxExit(NULL, "Error", "Called GetValueInt() for Not-Int member");
    return 0;
}

wxString InputItem_cls::
GetValueStr(){
    if (mpValue == NULL) return wxT("");
    if (mType == _ITEM_STRING || mType == _ITEM_STATIC_TEXT)
	return *(wxString*)mpValue;
    else 
	;//MsgBoxExit(NULL, "Error", "Called GetValueInt() for Not-Int member");
    return wxT("");
}

#define MACRO_ASSIGN()							\
    if (*porgValue != newValue){					\
    if (mUndoType == 1){						\
	GCUndoList.PushNewCommand					\
	    (new CommandSetValue_cls(porgValue, newValue, mUpdateBit),	\
	     new CommandSetValue_cls(porgValue, *porgValue, mUpdateBit)); \
	GCUndoList.ExecCurrent();					\
    }									\
    else if (mUndoType == 2){						\
	wxString strTag = mpCTgtFilterCtrlLink->			\
	    GetHeaderStrFromPointer(mpValue);				\
	GCUndoList.PushNewCommand					\
	    (new CommandFilterValue_cls(mpCTgtFilterCtrlLink, strTag, newValue, mUpdateBit), \
	     new CommandFilterValue_cls(mpCTgtFilterCtrlLink, strTag, *porgValue, mUpdateBit)); \
	GCUndoList.ExecCurrent();					\
    }									\
    else								\
	*porgValue = newValue;						\
    bChanged = TRUE;							\
 }									\
 
bool InputItem_cls::
Assign(){
    bool bChanged = FALSE;
    switch(mType){ 
    case _ITEM_BOOL:{
	bool *porgValue = (bool*)mpValue;
	bool newValue = ((wxCheckBox*)mpCCtrl)->GetValue();
	MACRO_ASSIGN();
	break; }
    case _ITEM_INTEGER:{
	int *porgValue = (int*)mpValue;
	int newValue = ((AtoiTextCtrl_cls*)mpCCtrl)->GetValue();
	MACRO_ASSIGN();
	break; }
    case _ITEM_INTEGER_SPIN:{
	int *porgValue = (int*)mpValue;
	int newValue   = ((AtoiSpinCtrl_cls*)mpCCtrl)->GetValue();
	MACRO_ASSIGN();
	break; }
    case _ITEM_INTEGER_SLIDER:{
	int *porgValue = (int*)mpValue;
	int newValue   = ((SliderInt_cls*)mpCCtrl)->GetValue();
	MACRO_ASSIGN();
	break; }
    case _ITEM_FLOAT:{
	float *porgValue = (float*)mpValue;
	float newValue = ((AtofTextCtrl_cls*)mpCCtrl)->GetValue();
	MACRO_ASSIGN();
	break; }
    case _ITEM_FLOAT_SLIDER:{
	float *porgValue = (float*)mpValue;
	float newValue   = ((SliderFloat_cls*)mpCCtrl)->GetValue();
	MACRO_ASSIGN();
	break; }
    case _ITEM_DOUBLE:{
	double *porgValue = (double*)mpValue;
	double newValue = ((AtofTextCtrl_cls*)mpCCtrl)->GetValue();
	MACRO_ASSIGN();
	break; }
    case _ITEM_DOUBLE_SLIDER:{
	double *porgValue = (double*)mpValue;
	double newValue   = ((SliderDouble_cls*)mpCCtrl)->GetValue();
	MACRO_ASSIGN();
	break; }
    case _ITEM_DUALSLIDER:{
	double *porgValue = (double*)mpValue;
	double newValue = ((DualSlider_cls*)mpCCtrl)->GetValue(0);
	MACRO_ASSIGN();
	porgValue = (double*)mpValue + 1;
	newValue = ((DualSlider_cls*)mpCCtrl)->GetValue(1);
	MACRO_ASSIGN();
	break;
    }
    case _ITEM_UINT_LIST: {
	unsigned int *porgValue = (unsigned int*)mpValue;
	unsigned int newValue   = ((AtoiComboCtrl_cls*)mpCCtrl)->GetValueInt();
	MACRO_ASSIGN();
	break; }
    case _ITEM_UINT_ICON: {
	unsigned int *porgValue = (unsigned int*)mpValue;
	unsigned int newValue = ((IconSelector_cls*)mpCCtrl)->GetValueInt();
	MACRO_ASSIGN();
	break; }
    case _ITEM_UINT_SPIN: {
	unsigned int *porgValue = (unsigned int*)mpValue;
	unsigned int newValue   = ((AtoiSpinCtrl_cls*)mpCCtrl)->GetValue();
	MACRO_ASSIGN(); 
	break; }
    case _ITEM_STRING:{
	wxString *porgValue = (wxString*)mpValue;
	wxString newValue   = ((AtoSTextCtrl_cls*)mpCCtrl)->GetValue();
	MACRO_ASSIGN(); 
	break; }
    }

    return bChanged;
}

void InputItem_cls::
UpdateDisplay(){
    switch(mType){
    case _ITEM_BOOL:
	if (((wxCheckBoxEX*)mpCCtrl)->GetValue() != GetValueBool())
	    ((wxCheckBoxEX*)mpCCtrl)->SetValue(GetValueBool());
	break;
    case _ITEM_INTEGER:
	if (((AtoiTextCtrl_cls*)mpCCtrl)->GetValue() != GetValueInt())
	    ((AtoiTextCtrl_cls*)mpCCtrl)->SetValueInt(GetValueInt());
	break;
    case _ITEM_INTEGER_SLIDER:{
	if (((SliderInt_cls*)mpCCtrl)->GetValue() != GetValueInt())
	    ((SliderInt_cls*)mpCCtrl)->SetValueInt(GetValueInt());
	break; }
    case _ITEM_FLOAT:
	if (((AtofTextCtrl_cls*)mpCCtrl)->GetValue() != GetValueFloat())
	    ((AtofTextCtrl_cls*)mpCCtrl)->SetValueFloat(GetValueFloat());
	break;
    case _ITEM_FLOAT_SLIDER:
	if (((SliderFloat_cls*)mpCCtrl)->GetValue() != GetValueFloat())
	    ((SliderFloat_cls*)mpCCtrl)->SetValueFloat(GetValueFloat());
	break;
    case _ITEM_DOUBLE:
	if (((AtofTextCtrl_cls*)mpCCtrl)->GetValue() != GetValueDouble())
	    ((AtofTextCtrl_cls*)mpCCtrl)->SetValueDouble(GetValueDouble());
	break;
    case _ITEM_DOUBLE_SLIDER:
	if (((SliderDouble_cls*)mpCCtrl)->GetValue() != GetValueDouble())
	    ((SliderDouble_cls*)mpCCtrl)->SetValueDouble(GetValueDouble());
	break;
    case _ITEM_DUALSLIDER:
	if (((DualSlider_cls*)mpCCtrl)->GetValue(0) != GetValueDouble(0) ||
	    ((DualSlider_cls*)mpCCtrl)->GetValue(1) != GetValueDouble(1))
	    ((DualSlider_cls*)mpCCtrl)->SetValueDouble
		(GetValueDouble(0), GetValueDouble(1));
	break;
    case _ITEM_UINT_LIST: 
	if (((AtoiComboCtrl_cls*)mpCCtrl)->GetValueInt() != GetValueInt())
	    ((AtoiComboCtrl_cls*)mpCCtrl)->SetValueInt(GetValueInt());
	break; 
    case _ITEM_UINT_ICON:
	if (((IconSelector_cls*)mpCCtrl)->GetValueInt() != GetValueInt())
	    ((IconSelector_cls*)mpCCtrl)->SetValueInt(GetValueInt());
	break;
    case _ITEM_INTEGER_SPIN: case _ITEM_UINT_SPIN:
	if (((AtoiSpinCtrl_cls*)mpCCtrl)->GetValue() != GetValueInt())
	    ((AtoiSpinCtrl_cls*)mpCCtrl)->SetValueInt(GetValueInt());
	break;
    case _ITEM_WXPANEL:
	((wxPanel*)mpCCtrl)->Refresh(FALSE);
	break;
    case _ITEM_STRING:
	((AtoSTextCtrl_cls*)mpCCtrl)->SetValue(GetValueStr());
	break;
    }
}

// ----

int InputItem_cls::
ConnectBoxSizer(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    
    wxSizerFlags sflagE(0);
    sflagE.Border(wxLEFT, 2).FixedMinSize().Expand().Align(wxALIGN_CENTER_VERTICAL);
    
    mpCCtrl = new wxBoxSizer(wxHORIZONTAL);
    pBoxSizer->Add((wxBoxSizer*)mpCCtrl, sflagE);
    
    int id = -1;
    if (mstrLabel != wxT("")){
	mpCTextLabel = 
	    new wxStaticTextEX(pWindow, wxID_ANY, mstrLabel,
			       wxPoint(0, -1), wxSize(mTextWidth, -1));
	id = mpCTextLabel->GetId();
	((wxBoxSizer*)mpCCtrl)->
	    Add(mpCTextLabel, GetSizerFlagLabel());
    }
    return id;
}

wxBoxSizer* InputItem_cls::
GetPNewSizer(wxWindow* pWindow){
    // ^CgtVKTCU[Ԃ
    wxBoxSizer* pBoxSizerIndividual = new wxBoxSizer(wxHORIZONTAL);
    if (IsButton() == FALSE) 
	if (mstrLabel != wxT("")){
	    mpCTextLabel = 
		new wxStaticTextEX(pWindow, wxID_ANY, mstrLabel, 
				   wxDP, wxSize(mTextWidth, -1));
	    pBoxSizerIndividual->Add(mpCTextLabel, GetSizerFlagLabel());
	}
    return pBoxSizerIndividual;
}

wxWindow* InputItem_cls::
GetPCwxWindow(){
    if (mpCCtrl == NULL) return NULL;
    switch(mType){
    case _ITEM_BOOL: 
	return (wxCheckBox*)mpCCtrl;
    case _ITEM_INTEGER:      case _ITEM_UINT:
	return (AtoiTextCtrl_cls*)mpCCtrl;
    case _ITEM_INTEGER_SPIN: case _ITEM_UINT_SPIN:
	return (AtoiSpinCtrl_cls*)mpCCtrl;
    case _ITEM_UINT_LIST:
	return (AtoiComboCtrl_cls*)mpCCtrl;
    case _ITEM_UINT_ICON:
	return (IconSelector_cls*)mpCCtrl;
    case _ITEM_INTEGER_SLIDER:
	return (SliderInt_cls*)mpCCtrl;
    case _ITEM_FLOAT_SLIDER:
	return (SliderFloat_cls*)mpCCtrl;
    case _ITEM_DOUBLE_SLIDER:
	return (SliderDouble_cls*)mpCCtrl;
    case _ITEM_DUALSLIDER:
	return (DualSlider_cls*)mpCCtrl;
    case _ITEM_FLOAT: case _ITEM_DOUBLE:
	return (AtofTextCtrl_cls*)mpCCtrl;
    case _ITEM_STRING:
	//case _ITEM_WXSTRING_PATH:
	return (AtoSTextCtrl_cls*)mpCCtrl;
    case _ITEM_WXPANEL:
	return (wxPanel*)mpCCtrl;
    case _ITEM_STATIC_TEXT:
	return (wxStaticTextEX*)mpCCtrl;
    case _ITEM_BUTTON:
	//case _ITEM_BUTTON_HIDE_PANEL:
	return (wxButton*)mpCCtrl;
	
    }
    return NULL;
}

// ------

void InputItem_cls::
ItemConnect(wxBoxSizer* pBoxSizer){
    if (pBoxSizer == NULL){
	printf("NULL!!\n");
	exit(0);
    }
    GetPCwxWindow()->Enable(mbEnable);
    pBoxSizer->Add(GetPCwxWindow(), GetSizerFlagH());
    mId = GetPCwxWindow()->GetId();
}

void InputItem_cls::
ItemConnect_Int(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    mpCCtrl = new AtoiTextCtrl_cls
	(pWindow, wxID_ANY, GetValueInt(), wxDP, wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_IntSpin(wxBoxSizer* pBoxSizer, wxWindow *pWindow){
    mpCCtrl = new AtoiSpinCtrl_cls
	(pWindow, wxID_ANY, GetValueInt(), wxDP, wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_IntSlider(wxBoxSizer* pBoxSizer, wxWindow *pWindow){
    mpCCtrl = new SliderInt_cls
	(pWindow, wxID_ANY, GetValueInt(), (int)mMin, (int)mMax,
	 wxDP, wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_Float(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    mpCCtrl = new AtofTextCtrl_cls
	(pWindow, wxID_ANY, GetValueFloat(), wxDP, wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_FloatSlider(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    mpCCtrl = new SliderFloat_cls
	(pWindow, wxID_ANY, (int)GetValueFloat() * mScale, 
	 (int)mMin, (int)mMax, mScale, wxDP, wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_Double(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    mpCCtrl = new AtofTextCtrl_cls
	(pWindow, wxID_ANY, GetValueDouble(), wxDP, wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_DoubleSlider(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    mpCCtrl = new SliderDouble_cls
	(pWindow, wxID_ANY, (int)GetValueDouble() * mScale, 
	 (int)mMin, (int)mMax, mScale, wxDP, wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_Bool(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    mpCCtrl = new wxCheckBoxEX
	(pWindow, wxID_ANY, (mbConnectSizer == TRUE) ? mstrLabel : (wxString)wxEmptyString,
	 wxDP, wxSize(mItemWidth, -1));
    bool value = *(bool*)mpValue;
    ((wxCheckBoxEX*)mpCCtrl)->SetValue(value);
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_List(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    mMax = mListMax;
    int list = GetValueInt();
    mpCCtrl = new AtoiComboCtrl_cls
	(pWindow, wxID_ANY, mVstrChoiceList[list], mListMax, &mVstrChoiceList[0], 0,
	 wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_Icon(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    //mMax = mListMax;
    mpCCtrl = new IconSelector_cls
	(pWindow, wxID_ANY, GetValueInt(), mMax, mpVIcon, wxDP, wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_String(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    mpCCtrl = new AtoSTextCtrl_cls
	(pWindow, wxID_ANY, wxEmptyString, wxDP, wxSize(mItemWidth, -1), mStyle);
    //wxSize CSize = ((AtoSTextCtrl_cls*)mpCCtrl)->GetBestSize();
    //((AtoSTextCtrl_cls*)mpCCtrl)->SetMaxSize(wxSize(-1, CSize.GetHeight()));
    ((AtoSTextCtrl_cls*)mpCCtrl)->SetValue(GetValueStr());
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_DualSlider(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    if (mItemWidth == -1) // vȂH
	mpCCtrl = new DualSlider_cls
	    (pWindow, this, wxID_ANY, 
	     (int)GetValueDouble(0) * mScale, 
	     (int)GetValueDouble(1) * mScale, 
	     (int)mMin, (int)mMax);
    else
	mpCCtrl = new DualSlider_cls
	    (pWindow, this, wxID_ANY, 
	     (int)GetValueDouble(0) * mScale, 
	     (int)GetValueDouble(1) * mScale, 
	     (int)mMin, (int)mMax, wxDP, wxSize(mItemWidth, -1));
    ((DualSlider_cls*)mpCCtrl)->mDisplayMode = 1;
    ItemConnect(pBoxSizer);
}

void InputItem_cls::// mpValue  wxPanel̔hNXĂ
ItemConnect_WXPanel(wxBoxSizer* pBoxSizer){
    mpCCtrl = mpValue;
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_StaticText(wxBoxSizer* pBoxSizer, wxWindow* pWindow){
    mpCCtrl = new wxStaticTextEX
	(pWindow, wxID_ANY, GetValueStr());
    ItemConnect(pBoxSizer);
}

void InputItem_cls::
ItemConnect_Button(wxBoxSizer* pBoxSizer, wxWindow* pWindow, bool bExpand){
    // {^̏ꍇmId͒`ς݂̂͂
    int itemWidth = mItemWidth;
    if (bExpand == TRUE) mItemWidth = -1;
    mpCCtrl = new wxButtonEX
	(pWindow, mId, mstrLabel, wxDP, wxSize(mItemWidth, -1));
    ItemConnect(pBoxSizer);
    itemWidth = mItemWidth;
}
