// -*- c -*- 
// "clsMultiInputsPanel.cpp"

#ifndef _MULTI_INPUT_PANEL_H_
#define _MULTI_INPUT_PANEL_H_

#include "clsDialogEXKey.h"
#include "clsInputItem.h"
#include "clsWidgetCommon.h"

#include <vector>

#include "../COMMON/CommonDefinitionsForH.h"

#define _MN_FORWARD 1
#define _MN_BACKWARD -1

#define _ITEM_LASTITEM  -1
#define _ITEM_LASTITEM2 -2
#define _ITEM_LASTITEMx2 -5
#define _MN_ALLITEM  -3
#define _MN_NOITEM   -4
#define _MN_ASSIGN_ALL -3

#define _MN_NO_BUTTONS 0
#define _MN_ASSIGN_UPDATE_HIDE 1
#define _MN_CANCEL_OK 2
#define _MN_OK_CANCEL 3
#define _MN_HELP      4
#define _MN_ADVANCED  8

#define _MN_PANEL  0
#define _MN_DIALOG 1

#define _MN_NOFORCE FALSE

#define _MN_DLG_MAKEGEOM TRUE
#define _MN_DLG_NOMAKEGEOM FALSE
#define _MN_DLG_MODAL TRUE

#define _MN_UNDO_MANUAL        0
#define _MN_UNDO_AUTO_POINTER  1
#define _MN_UNDO_AUTO_TAG      2


#ifdef WIN32
#define _MN_OK_CANCEL_DEFAULT _MN_OK_CANCEL
#else
#define _MN_OK_CANCEL_DEFAULT _MN_CANCEL_OK
#endif

class wxButtonEX;
class wxCheckBoxEX;

class FilterCtrl_cls;
class HookCtrl_cls;
class HookItem_cls;
class MultiInputPanel_cls;

class MultiInputBase_cls : public WidgetCommon_cls{
 public:
    MultiInputBase_cls(wxWindow *pCMInputPanel);
    ~MultiInputBase_cls();

    int         mType;
    wxWindow    *mpCParent;
    wxBoxSizer  *mpBoxSizerSrc;
    wxSizerItem *mpPadding;
    HookCtrl_cls *mpCHookCtrl;
    
    bool   mbPanelOpen;
    int    mTextWidth, mCheckBoxWidth, mComboWidth, mItemWidth;
    int    mMinimumWidth;
    int    mUpdateBit;
    int    mUndoType;
    FilterCtrl_cls *mpCTgtFilterCtrlLink;
    
    std::vector < InputItem_cls > mVCItemList;
    InputItem_cls& GetLastItem();
    int  GetItemId(int index);
    wxWindow* GetPCwxWindow(int index);
    std::vector < bool > GetVbShownItems();
    
    void SetColorForXYZ(unsigned int index);

    int  mSizerIndex, mSizerCounter;
    void ProcHorizontalBoxSizer(int currIndex, int type, void *pValue);

    int  CombinePointerButton(int id, wxString strLabel, 
			      int type = _ITEM_BUTTON, bool enabled = TRUE);
    int  CombinePointerSizer
	(int width, wxString strLabel = wxES, bool enable = TRUE);
    int  CombinePointer(int type, void *pValue, 
			wxString strLabel = wxES, bool enable = TRUE);
    int  CombinePointerList(void *pValue, 
			    unsigned int listMax, wxString *pChoiceList, 
			    wxString strLabel = wxES, bool enable = TRUE);
    void Reconnect(int index, void *pValue);

    int  SetInputAndSlider(int type, void *pPointerVal, wxString strLabel,
			   int itemWidth = -2, int itemWidth2 = -2);
    void ReconnectInputAndSlider(int index, int type, void *pPointerVal);

    void SetHook(HookItem_cls *pCInstance, 
		 void (HookItem_cls::*pFunc)(int, void*),
		 int type, int index = -1, void *pArg = NULL);
    
    void SetRange(int index, double min, double max, int scale = 1);
    
    void UpdateOtherItems(int index);
    void AssignAllAtClose();
    void AssignBase(int index = _MN_ASSIGN_ALL, 
		    bool bUseHookChange = TRUE,
		    bool bForceHookChange = FALSE);
    void AssignBaseById(int id);
    void ShowItem(unsigned int index, bool bShow = TRUE);
    void HideItem(unsigned int index);
    void UpdatePanelOpenStatus(unsigned int index);
    void TogglePanelOpenStatus(unsigned int index);
    void EnableItemAll(bool bEnable);
    void EnableItem(int index, bool bEnable);

    void OnScrollBase(wxScrollEvent& event);
    void OnScrollBaseChanged(wxScrollEvent& event);
    void OnCheckBoxBase(wxCommandEvent& event);
    void OnComboBoxBase(wxCommandEvent& event);

    virtual void ConnectButton  (wxWindow *pWindow, int id) = 0;
    virtual void ConnectClick   (wxWindow *pWindow, int id) = 0;
    virtual void ConnectBool    (wxWindow *pWindow, int id) = 0;
    virtual void ConnectComboBox(wxWindow *pWindow, int id) = 0;
    virtual void ConnectScroll  (wxWindow *pWindow, int id) = 0;
    //virtual void ConnnectSpin   (wxWindow *pWindow, int id) = 0;

    void MakeNewSizer(int index, wxBoxSizer *pBoxSizerSrc, wxWindow *pCPanel);
    void MakeGeometryBase(wxBoxSizer *pBoxSizerSrc, wxWindow *pCPanel);

    void UpdateValues(int index = _MN_ALLITEM);
};

class MultiInputPanel_cls : 
public wxPanel, public MultiInputBase_cls {
 public:
    MultiInputPanel_cls(wxWindow *pCParent);

    static MultiInputPanel_cls* 
	Initialize(MultiInputPanel_cls **ppMNum, wxWindow *pCParent);
  
    void SetPaddingWidth(int paddingWidth);
    int  GetPaddingWidth(int minimumWidth);

    void RefreshSlider();
    void MakeGeometry();

    void ConnectButton  (wxWindow *pWindow, int id);
    void ConnectClick   (wxWindow *pWindow, int id);
    void ConnectBool    (wxWindow *pWindow, int id);
    void ConnectComboBox(wxWindow *pWindow, int id);
    void ConnectScroll  (wxWindow *pWindow, int id);
    
    void OnButton(wxCommandEvent& event);
    void OnChar(wxKeyEvent& event);
    void OnKeyDown(wxKeyEvent& event);
    void OnCheckBox(wxCommandEvent &event);
    void OnComboBox(wxCommandEvent &event);
    void OnMouse(wxMouseEvent &event);
    void OnMouseOnItem(wxMouseEvent &event);
    void OnScroll(wxScrollEvent& event);
    void OnScrollChanged(wxScrollEvent& event);
   
    //--
    void AddThisAndHideButtonToSizer(wxBoxSizer *pBoxSizer);
    int  FinishSetting(wxBoxSizer *pBoxSizer = NULL);
    
    DECLARE_EVENT_TABLE()
};

class MultiInputDialog_cls :
public wxDialogEXKey{
 public:
    MultiInputDialog_cls(wxWindow *pCParent, wxWindowID id, wxString strLabel,
			 wxPoint pos  = wxDefaultPosition, 
			 wxSize size = wxDefaultSize,
			 wxFont font = wxNullFont, 
			 wxBitmap bitmap = wxNullBitmap,
			 long style = wxDEFAULT_DIALOG_STYLE);
    
    MultiInputPanel_cls   *mpCPanel;
    
    wxButtonEX *mpBTNOK;
    wxButtonEX *mpBTNCancel;

    void ShowItem(int index);
    void HideItem(int index);
	
    void OnOK(wxCommandEvent &event);
    void MakeGeometry();

    void UpdateValues(int index = _MN_ALLITEM);
   
};

#endif
