// -*- c++ -*- 
// "clsFrameMain.cpp"
// (setq c-basic-offset 4)

#ifndef _FRAME_MAIN_H_
#define _FRAME_MAIN_H_

#include "SRC/CONFITEM/clsConfItem.h"
#include "SRC/WIDGETCTRL/clsWidgetCommon.h"

#include <wx/wx.h>
#include <wx/splitter.h>
#include <wx/stopwatch.h>

class BtmHalfPanel_cls;
class CameraPathCtrl_cls;
class CameraPathGLGUI_cls;
class ControlPanel_cls;
class FilterCtrl_cls;
class FrameMain_cls; 
class GLCanvasCtrl_cls;
class GLInfo_cls;
class GLExtraFrame_cls;
class HelpLauncher_cls;
class Main_cls;
class MainGLCanvas_cls;
class PDataSeqCtrl_cls;
class PDataSequence_cls;
class PovRayCtrl_cls;
class Preference_cls;
class Scene_cls;
class Shortcut_cls;
class TopHalfPanel_cls;
class TimeCtrl_cls;

class wxButtonEX;
class wxCheckBoxEX;
class wxGauge;

#define _FM_DEFAULT_CONF false
#define _FM_PROJECT_CONF true

#define _FM_FRAME_CURRENT -1

class FrameMain_cls : public wxFrame, public WidgetCommon_cls {
public:
    FrameMain_cls(const wxString& title, Main_cls *pMain, int posx, int posy,
		  int width, int height);
    ~FrameMain_cls();

    void SetContents();

    // --- clsFrameMainInitMenu.cpp
    void AddBitmapMenu(wxMenu *pMenu, int ID, wxString strMain, 
		       wxString strSub, wxString strFile);
    void AddBitmapMenu(wxMenu *pMenu, int ID, wxString strMain,
		       wxString strSub, wxBitmap bitmap);
    void ClearMenu();
    void SetMenuItem();
    void UpdateMenu();
    void InitMenu();
    
    // --- MAIN ---
    
    static FrameMain_cls *mspCThis;
    Main_cls             *mpCParent;

    wxSplitterWindow  *mpCSplitterMain;
    TopHalfPanel_cls  *mpCTopHalfPanel;
    BtmHalfPanel_cls  *mpCBtmHalfPanel;
    HelpLauncher_cls  *mpCHelpLauncher;
    GLExtraFrame_cls  *mpCGLExtFrame;
    wxGauge           *mpCStatusGauge;
    

    void FitSplitterWindow(bool bForce = FALSE);

    wxMenuBar    *mpCMenuBar;
    wxMenu       *mpCMenuEdit;
    wxMenu       *mpCMenuFilterManipulate;
    wxMenu       *mpCMenuFilterBoolean, *mpCMenuFilterObject, *mpCMenuFilterWorld;

    wxMenuItem   *mpCMenuItemExportSnap, *mpCMenuItemExportAnim;
    wxMenuItem   *mpCMenuItemRenderSnap, *mpCMenuItemRenderAnim;

    wxTimer           mCTimer;

    Preference_cls    *mpCPreference;
    Shortcut_cls      *mpCShortcut;
    MainGLCanvas_cls  *mpCActiveGLCanvasLink;
    GLInfo_cls        *mpCGLInfo;

    int       mInitSash;
    
    // -----
    PDataSequence_cls *GetPCActiveFileSequence();
    GLCanvasCtrl_cls  *GetPCGLCanvasCtrl(int index);
    MainGLCanvas_cls  *GetPCGLCanvas(int index);
    
    void ReconnectCameraPathGUI();
    void PrepareTheFrame(int frame);
    void UpdateTitle();
    void EnableMenuItemPov(bool  b0, bool b1, bool b2, bool b3);
    bool IsTheFrameReady(int frame);
    bool IsTheFrameReading(int frame);
    bool IsTheFrameLoaded(int frame);
    
    // -----
    void OnClose(wxCloseEvent& event);
    void OnSize(wxSizeEvent& event);
    void OnQuit (wxCommandEvent& event);
    void OnAboutGL(wxCommandEvent& event);
    void OnAbout(wxCommandEvent& event);
    void OnHelp(wxCommandEvent& event);
    void OnAboutNetwork(wxCommandEvent& event);
    void OnKeyDown(wxKeyEvent& event);
    void OnSash(wxSplitterEvent& event);
    void PrepareFramesAhead(int range);
    bool IsReadThreadIdle();
    void OnIdle(wxIdleEvent& event);
    void OnTimer(wxTimerEvent& event);

    // -----
    void OnMenuOpenProject(wxCommandEvent& event);
    void OnMenuSaveProject(wxCommandEvent& event);
    void OnMenuSaveProjectAs(wxCommandEvent& event);
    void OnMenuLoadData(wxCommandEvent& event);
    void OnMenuExportData(wxCommandEvent& event);
    void OnMenuExportPathPov(wxCommandEvent& event);
    void OnMenuSaveImageSnap(wxCommandEvent& event);
    void OnMenuSaveImageSeq(wxCommandEvent& event);
    void OnMenuSaveImgSeqBegin(wxCommandEvent& event);
    void OnMenuExportDataBegin(wxCommandEvent& event);
    void OnMenuCloseData(wxCommandEvent& event);
    void OnMenuCameraPath(wxCommandEvent& event);
    void OnMenuLighgtSRC(wxCommandEvent& event);
    void OnMenuColorCurve(wxCommandEvent& event);
    void OnMenuPovRay(wxCommandEvent& event);
    void OnMenuTexProperty(wxCommandEvent& event);
    void OnMenuStereogram(wxCommandEvent& event);
    void OnMenuPeriodicBoundary(wxCommandEvent& event);
    void OnMenuUnlock(wxCommandEvent& event);
    void OnMenuSpecial(wxCommandEvent& event);
    void OnMenuViewPanelType(wxCommandEvent& event);
    void OnMenuViewExtra(wxCommandEvent& event);
    void OnMenuUndo(wxCommandEvent& event);
    void OnMenuRedo(wxCommandEvent& event);
    void OnMenuGo(wxCommandEvent& event);
    void OnMenuTimeRemap(wxCommandEvent& event);
    void OnMenuRemoveCacheSnap(wxCommandEvent& event);
    void OnMenuRemoveCacheAll(wxCommandEvent& event);
    void OnMenuScene(wxCommandEvent& event);
    void OnMenuPreference(wxCommandEvent& event);
    void OnMenuShortcut(wxCommandEvent& event);
    void OnMenuPovExportSnap(wxCommandEvent& event);
    void OnMenuPovRenderSnap(wxCommandEvent& event);
    void OnMenuPovExportAnim(wxCommandEvent& event);
    void OnMenuPovRenderAnim(wxCommandEvent& event);
    void OnMenuGLAttribSelect(wxCommandEvent& event);
    void OnMenuFilterPopup(wxCommandEvent& event);
    void OnMenuFilterRefresh(wxCommandEvent& event);
    void OnMenuFilterManipulate(wxCommandEvent& event);
    void OnMenuFilterResize(wxCommandEvent& event);
    void OnMenuFilterDelete(wxCommandEvent& event);
    void OnMenuFilterUp(wxCommandEvent& event);
    void OnMenuFilterDown(wxCommandEvent& event);
    void ProcAddFilter(FilterCtrl_cls *pCFilterCtrl, int filterType);
    int  GetFilterTypeFromEnum(int enumID);
    void OnMenuFilterAdd(wxCommandEvent& event);
    void OnGLMenu(wxCommandEvent& event);
        
    void OnMenuRefreshGL(wxCommandEvent& event);
    void OnMenuRefreshTL(wxCommandEvent& event);
    void OnMenuRefreshAttrib(wxCommandEvent& event);
    void OnMenuRefreshIMGSEQ(wxCommandEvent& event);
    void OnMenuRefreshCamConf(wxCommandEvent& event);
    void OnMenuRefreshColor(wxCommandEvent& event);
    void OnMenuRefreshExporter(wxCommandEvent& event);
    void OnMenuRefreshDrawOptions(wxCommandEvent& event);
    void OnMenuRefreshLight(wxCommandEvent& event);
    void OnMenuRefreshExposure(wxCommandEvent& event);
    void OnMenuRefreshStereo(wxCommandEvent& event);
    void OnMenuRefreshTexture(wxCommandEvent& event);
    
    void OnMenuDeselect(wxCommandEvent& event);

    void OnMenuSyncGLCtrl(wxCommandEvent& event);
    void OnMenuResizedTL(wxCommandEvent& event);
    void UpdateStatusText();
    void CheckAbortion();
    void OnMenuRefreshSB(wxCommandEvent& event);
    void OnMenuSetKeyPos   (wxCommandEvent& event);
    void OnMenuSetKeyPosRot(wxCommandEvent& event);
    void OnMenuSetKeyAnc   (wxCommandEvent& event);
    bool mbRepeatPlay;
    void OnMenuPlayStop(wxCommandEvent& event);

    void OnMenuFullScreen(wxCommandEvent& event);
    void OnMenuEscape(wxCommandEvent& event);
    
    PovRayCtrl_cls     *GetPCPovRayCtrl();
    TimeCtrl_cls       *GetPCTimeCtrl();
    PDataSeqCtrl_cls   *GetPCDataSeqCtrl();
    CameraPathCtrl_cls *GetPCPathCtrl();
    
    bool mbHasOpenedProject;
    ConfItemList_cls mCConfList;
    ConfItemSTR_cls  mCLastOpenedFileZJP;
    ConfItemSTR_cls  mCLastOpenedFileGRM;
    ConfItemSTR_cls  mCLastOpenedFileOUT;
    ConfItemSTR_cls  mCLastOpenedFileOBJ;
    ConfItemSTR_cls  mCLastOpenedFileTEX;

    wxString Serialize();
    wxString SerializeForConfigure();
    void ReadStrLine(std::wistream &istream, wxString strLine);

 private:
    // any class wishing to process wxWidgets events must use this macro
    DECLARE_EVENT_TABLE()
};

#endif
