// -*- c++ -*-
// (setq c-basic-offset 4)
// "clsMain.h"

#include "clsMain.h"
#include "clsFrameMain.h"

#include "SRC/COMMON/clsFilename.h"
#include "SRC/FILTER/clsFilterBase.h"
#include "SRC/FILTER/clsFilterCopy.h"
#include "SRC/FILTER/clsFilterCtrl.h"
#include "SRC/FILTER/clsFilterDataBox.h"
#include "SRC/FILTER/clsFilterFog.h"
#include "SRC/FILTER/clsFilterStick.h"
#include "SRC/GLATTRIB/clsColorCurveGUI.h"
#include "SRC/GLATTRIB/clsColorCurve.h"
#include "SRC/GLCANVAS/clsMainGLCanvas.h"
#include "SRC/GLCANVAS/clsGLCanvasCtrl.h"
#include "SRC/GLCANVAS/clsMoveItem.h"
#include "SRC/GLCANVAS/clsStereoGram.h"
#include "SRC/GRAPHIC/clsImgSequence.h"
#include "SRC/GRAPHIC/clsImageIO.h"
extern ImageIO_cls GCImgIO;
#include "SRC/MAINCANVAS/clsDrawOption.h"
#include "SRC/MAINCANVAS/clsExposure.h"
#include "SRC/MAINCANVAS/clsInformation.h"
#include "SRC/MAINCANVAS/clsOverlay.h"
#include "SRC/MAINCANVAS/clsParamGUICtrl.h"
#include "SRC/MAINCANVAS/clsTopHalfPanel.h"
#include "SRC/MAINCANVAS/clsBtmHalfPanel.h"
#include "SRC/MISC/clsInitializePararell.h"
#include "SRC/MISC/clsIpAddress.h"
#include "SRC/MISC/clsMacAddress.h"
#include "SRC/PANELS/clsDataLoader.h"
#include "SRC/PANELS/clsDataLoaderGUI.h"
#include "SRC/PANELS/clsDataExporter.h"
#include "SRC/PANELS/clsPreference.h"
#include "SRC/PANELS/clsScene.h"
#include "SRC/PANELS/clsShortcut.h"
#include "SRC/PARAMETER/clsParamTL.h"
#include "SRC/PARTICLEDATA/clsGLDrawData.h"
#include "SRC/PARTICLEDATA/clsParticleDataSeqCtrl.h"
#include "SRC/PARTICLEDATA/clsParticleDataSequence.h"
#include "SRC/PARTICLEDATA/clsParticleDataBlock.h"
#include "SRC/PARTICLEDATA/clsPeriodicBoundary.h"
#include "SRC/PARTICLEDATA/clsShadowVolume.h"
#include "SRC/PARTICLEDATA/clsSnapData.h"
#include "SRC/PARTICLEDATA/clsThreadFileReader.h"
extern ThreadFileReader_cls   *GApCThreadReader[_MAX_THREAD_READ];
extern bool                    GAbThreadReaderFinished[_MAX_THREAD_READ];
#include "SRC/PARTICLEDATA/INTERPOLATE_GET/clsThreadInterpolate.h"
extern ThreadInterpolator_cls *GpCThreadInterpolate;
extern bool                    GbThreadInterpolateFinished;
#include "SRC/PARTICLEDATA/clsZSort.h"
#include "SRC/POVRAY/clsPovRay.h"
#include "SRC/POVRAY/MESH/clsPovObjectMesh.h"
#include "SRC/POVRAY/OPTIONAL/clsPovFormat.h"

#include "SRC/SCENEOBJ/clsCameraPath.h"
#include "SRC/SCENEOBJ/clsCameraPathGUI.h"
#include "SRC/SCENEOBJ/clsLightSource.h"
#include "SRC/SCENEOBJ/clsLightSourceGUI.h"
#include "SRC/SCENEOBJ/clsTexPropertyGUI.h"
#include "SRC/TIMELINE/clsTimeCtrl.h"
#include "SRC/TIMELINE/clsPlayCtrlGUI.h"
#include "SRC/TIMELINE/clsTLPanelBase.h"

#include "SRC/UNDO/clsUndoManager.h"
extern UndoManager_cls GCUndoList;
#include "SRC/UNDO/clsUndoItemDataIO.h"

#include "SRC/WIDGETCTRL/WX/myArtProv.h"
#include "SRC/WIDGETCTRL/WX/clsFontProvider.h"
#include "SRC/WIDGETCTRL/WX/clsSizeAdjuster.h"

extern bool GbGLLockProjectLoading;

#include "wxWidgetEnum.h"
#include "wxWidgetEnumMenu.h"
#include "wxWidgetString.h"

#include "SRC/MYAPI/MsgBox.h"
#include <wx/clipbrd.h>
#include <wx/file.h>
#include <wx/gdicmn.h>
#include <wx/progdlg.h>
#include <wx/textfile.h>
#include <wx/tglbtn.h>
#include <wx/tooltip.h>

#include "SRC/COMMON/CommonDefinitions.h"

#ifdef WIN32
#define _WXDELIM_ wxT("\\")
#else
#define _WXDELIM_ wxT("/")
#endif

bool GbSpecial = FALSE;

Main_cls::
Main_cls() 
{
    /*
    AllocConsole(); // R\[tɂ
    freopen("CONOUT$", "w", stdout); //Wo͂R\[ɂ
    freopen("CONIN$", "r", stdin);   //W͂R\[ɂ
    */

    wxStopWatch CStopWatch;
    CStopWatch.Start();

    bool bFinished = FALSE;
    InitializeParallel_cls *pCInit = 
	new InitializeParallel_cls(wxTHREAD_DETACHED);
    pCInit->SetParameters(&bFinished, this);
    
    Parameters_cls::mspCMainLink = this;
    mpCPGCameraPath = NULL;
    mpCPGStereoGram = NULL;

    wxArtProvider::Push(new MyArtProvider);
    FontProvider_cls::InitializeStatic();

    mpCDrawOption   = new DrawOption_cls;
    for (int i = 0; i < _MAX_FILTER_CTRL; i++)
	mpCFilterCtrl[i] = new FilterCtrl_cls(this, i);
    mpCExposure     = new Exposure_cls;
    mpCInformation  = new Information_cls;
    mpCOverlay      = new Overlay_cls;

#ifdef WIN32
#else
    mpCMacAddress = new MacAddress_cls;
    mpCIpAddress = new IpAddress_cls;
#endif

    GbGLLockProjectLoading = TRUE; //O̕`h
    wxRect CRectFrame(0, 0, 1280, 960);
    CRectFrame = SizeAdjuster_cls::GetCRectWindowFitting(CRectFrame);
    mpCFrame = new FrameMain_cls(_FRAME_MAIN_APPNAME, this, 
				 CRectFrame.GetX(), CRectFrame.GetY(),
				 CRectFrame.GetWidth(), CRectFrame.GetHeight());

    pCInit->Run();

    mpCFrame->SetContents();

    mInitializeTimeA = CStopWatch.Time();

    while (bFinished == FALSE) wxMilliSleep(10);

    mInitializeTimeB = CStopWatch.Time();


    // p[^[plݒ
    mpCParamGUICtrlLink = mpCFrame->mpCBtmHalfPanel->mpCParamGUICtrl;

    mpCParamGUICtrlLink->AddParamGroup(mpCTime->mpCTimeRemap);
    mpCTime->mpCTimeRemap->HideItems();

    delete mpCCameraPath->mpCSceneObjParam;
    
    mpCPGCameraPath = new ParamGroup_cls(wxT("Camera Path"), 0, NULL, 0);
    mpCPGCameraPath->mpCSceneObjLink = mpCCameraPath;
    mpCPGCameraPath->mVpCParamGroup.push_back(mpCCameraPath->mpCTranslate);
    mpCPGCameraPath->mVpCParamGroup.push_back(mpCCameraPath->mpCRotation);
    mpCPGCameraPath->mVpCParamGroup.push_back(mpCAnchorPath->mpCTranslate);
    mpCPGCameraPath->mVpCParamGroup[2]->mpCSceneObjLink = mpCAnchorPath;

    mpCCameraPath->mpCSceneObjParam = mpCPGCameraPath;

    bool bFinishedB = FALSE;
    InitializeParallelB_cls *pCInitB = 
	new InitializeParallelB_cls(wxTHREAD_DETACHED);
    pCInitB->SetParameters(&bFinishedB, this);

    pCInitB->Run();

    mpCFrame->mpCPreference->InitGUI(mpCFrame);
    mpCCameraPathCtrl ->InitGUI(mpCFrame);
    mpCLightSRCCtrl   ->InitGUI(mpCFrame);
    mpCColorCurveCtrl ->InitGUI(mpCFrame);
    mpCTexPropertyCtrl->InitGUI(mpCFrame);
    mpCPeriodic       ->InitGUI(mpCFrame);
    mpCImgSeq         ->InitGUI(mpCFrame);
    mpCStereogram     ->InitGUI(mpCFrame);
    mpCPovRayCtrl     ->InitGUI(mpCFrame);
    
    mpCParamGUICtrlLink->AddParamGroup(mpCPGCameraPath);
    mpCPGCameraPath->HideItems();

    mpCParamGUICtrlLink->AddParamGroup(mpCStereogram->mpCStereoParam);
    mpCStereogram->mpCStereoParam->HideItems();

    for (int i = 0; i < _LIGHT_NUM_MAX; i++){
	mpCParamGUICtrlLink->AddParamGroup(mpCLightSRCCtrl->mpCParamGroup[i]);
	mpCLightSRCCtrl->mpCParamGroup[i]->HideItems();
    }

    mpCParamGUICtrlLink->HideAndShow();
    mpCParamGUICtrlLink->ResizeTL();

   
    mpCTime->mpCParamGUICtrlLink = mpCParamGUICtrlLink;

    // ÓIȃNX̃N
    Parameters_cls::mspCTimeCtrlLink      = mpCTime;
    Parameters_cls::mspCPDataSeqCtrlLink  = mpCPDataSeqCtrl;
    ParamTL_cls::mspCTimeCtrlLink         = mpCTime;
    PDataBlock_cls::mspCDataLoaderLink    = mpCDataLoader;
    ColorCurveBtn_cls::mspCColorCurveCtrlLinked = mpCColorCurveCtrl;
    MainGLCanvas_cls::mspCPeriodicLink    = mpCPeriodic;
    MainGLCanvas_cls::mspCStereogramLink  = mpCStereogram;
    MainGLCanvas_cls::mspCFilterCtrlBooleanLink = mpCFilterCtrl[0];
    MainGLCanvas_cls::mspCFilterCtrlObjectLink  = mpCFilterCtrl[1];
    MainGLCanvas_cls::mspCFilterCtrlWorldLink   = mpCFilterCtrl[2];
    GLDrawBillboard_cls::mspCFilterCtrlBooleanLink = mpCFilterCtrl[0];
    GLDrawBillboard_cls::mspCFilterCtrlObjectLink  = mpCFilterCtrl[1];
    GLDrawData_cls::mspCInfoPanel             = mpCInformation->mpCGUI;
    PovObjectMesh_cls::mspCPovRayLink     = mpCPovRayCtrl;
    ZSort_cls::mspCPreferenceLink            = mpCFrame->mpCPreference;
    ThreadFileReader_cls::mspCPreferenceLink = mpCFrame->mpCPreference;
    GLAttribute_cls::mspCDrawOptionLink = mpCDrawOption;
    GLAttribute_cls::mspCFilterCtrlObjectLink = mpCFilterCtrl[1];

    while (bFinishedB == FALSE) wxMilliSleep(10);
   
    UpdateGUIItems();

    SendMenuEvent(mpCFrame, menuEditRefreshTL);
    SendMenuEvent(mpCFrame, menuEditRefreshGL);
    
    mInitializeTimeC = CStopWatch.Time();
}

Main_cls::
~Main_cls(){
    delete mpCMacAddress;
    delete mpCIpAddress;
    delete mpCDrawOption;
    for (int i = 0; i < _MAX_FILTER_CTRL; i++)
	delete mpCFilterCtrl[i];
    delete mpCOverlay;
    delete mpCPDataSeqCtrl;
    delete mpCPeriodic;
    delete mpCDataLoader;
    delete mpCExposure;
    delete mpCInformation;
    delete mpCDataExporter;
    delete mpCCameraPath;
    delete mpCColorCurveCtrl;
    delete mpCTexPropertyCtrl;
    delete mpCPovRayCtrl;
    delete mpCAnchorPath;
    delete mpCTime;
    delete mpCLightSRCCtrl;
    delete mpCCameraPathCtrl;
    delete mpCImgSeq;
    delete mpCScene;
    delete mpCStereogram;
    //delete mpCPGCameraPath;
}

wxWindow* Main_cls::
GetPTopLevelWindow(){
    return mpCFrame;
}

Parameters_cls* Main_cls::
GetPCParamFromID(wxString strID){
    Parameters_cls *pCParam = NULL;

    for (int i = 0; i < _MAX_FILTER_CTRL; i++){
	if (mpCFilterCtrl[i] != NULL)
	    pCParam = mpCFilterCtrl[i]->GetPCParamFromID(strID);
	if (pCParam != NULL) return pCParam;
    }
    
    return pCParam;
}

GLAttribute_cls** Main_cls::
GetPPCGLAttribute(){
    return mpCPDataSeqCtrl->GetPCActiveFileSequence()->mpCGLAttribute;
}

int  Main_cls::
GetUsedTotalMemoryMB(){
    return mpCPDataSeqCtrl->mTotalUsedMemoryMB;
}

int  Main_cls::
GetAverageMemoryMB(){
    int totalMB = GetUsedTotalMemoryMB();
    int totalFrame = mpCPDataSeqCtrl->GetLoadedFrameNumber();
    if (totalFrame == 0) return -1;
    return totalMB / totalFrame;
}

int  Main_cls::
GetMemoryLimitMB(){
    return mpCPDataSeqCtrl->mMemoryLimitationMB;
}

void Main_cls::
SelectObject(int objectIndex, int guideIndex){
    MoveItem_cls::SetSelection
	(mpCParamGUICtrlLink->SelectObject(objectIndex, guideIndex));
}

void Main_cls::
ThisObjectIsSelected(int objectIndex){
    if (objectIndex >= _SCENEOBJ_IDBASE_LIGHT &&
	objectIndex < _SCENEOBJ_IDBASE_LIGHT + 7)
	mpCLightSRCCtrl->SelectLight(objectIndex - _SCENEOBJ_IDBASE_LIGHT);
}

void Main_cls::
SelectFilter(FilterBase_cls *pCFilterBase, bool bDeselectObject){
    bool bChanged = FALSE;
    for (int i = 0; i < _MAX_FILTER_CTRL; i++)
	bChanged |= mpCFilterCtrl[i]->SelectFilter(pCFilterBase);

    if (bChanged == TRUE){
	SendMenuEvent(mpCFrame, menuEditRefreshGL);
	
	if (bDeselectObject == TRUE){	
	    MoveItem_cls::SetSelection(mpCParamGUICtrlLink->SelectObject(NULL));
	}
    }

    if (pCFilterBase != NULL)
	MoveItem_cls::SetSelection(pCFilterBase->GetPCData());
    
}

void Main_cls::
SelectGLAttrib(GLAttribute_cls *pCGLAttrib){
    wxString strLabel = wxT("Attrib "); strLabel << pCGLAttrib->mSelfIndex;
    mpCTexPropertyCtrl->ReconnectTexture(&pCGLAttrib->mCTexture, strLabel);
    SelectFilter(NULL, TRUE);
}

void Main_cls::
SelectFilterByHittest(int itemName, bool bDeselectObject){
    if (itemName >= 20 && itemName < 40)
	SelectFilter(*(mpCFilterCtrl[0]->GetIterN(itemName - 20)), bDeselectObject);
    if (itemName >= 40)
	SelectFilter(*(mpCFilterCtrl[1]->GetIterN(itemName - 40)), bDeselectObject);
}

bool Main_cls::
IsTheIndexReady(int index, double indexmod){
    return mpCPDataSeqCtrl->IsTheIndexReady(index, indexmod, mpCTime->mCBlurCtrl.mbUseMotionBlur);
}

bool Main_cls::
IsTheIndexLoaded(int index){
    return mpCPDataSeqCtrl->IsTheIndexLoaded(index);
}

void Main_cls::
WaitAllThreadFinish(){
    // FIX_ME!! KXbhI܂őҋ@ɂ́H
    for (int i = 0; i < _MAX_THREAD_READ; i++)
	if (GApCThreadReader[i] != NULL){
	    while (GAbThreadReaderFinished[i] == FALSE)
		wxMilliSleep(10);
	    GApCThreadReader[i] = NULL;
	}
    
    if (GpCThreadInterpolate != NULL){
	while (GbThreadInterpolateFinished == FALSE)
	    wxMilliSleep(10);
	GpCThreadInterpolate = NULL;
    }
}

void Main_cls::
OpenColorCurveDialog(){
    mpCColorCurveCtrl->DialogControl_cls::OpenWindow(mpCFrame);
}

void Main_cls::
OpenPovRayDialog(){
    mpCPovRayCtrl->DialogControl_cls::OpenWindow(mpCFrame);
}

void Main_cls::
OpenDataFileCommand(wxString strFilePath){
    GCUndoList.OpenBlock();
    GCUndoList.PushNewCommand(new CommandLoadData_cls(this, wxT("")),
			      new CommandLoadData_cls(this, mpCPDataSeqCtrl->GetStrActiveFileSequencePath()));
    GCUndoList.ExecCurrent();
    GCUndoList.PushNewCommand(new CommandLoadData_cls(this, strFilePath),
			      new CommandLoadData_cls(this, wxT("")));
    GCUndoList.ExecCurrent();
    GCUndoList.CloseBlock();
}

void Main_cls::
OpenDataFile(wxString strFilePath){
    int snapNumInFile = mpCPDataSeqCtrl->
	OpenDataFile(strFilePath, mpCDataLoader);

    int maxFrame = mpCPDataSeqCtrl->GetActiveFileSequenceLength()-1;
    if (maxFrame < 0) maxFrame = 0;
    if (maxFrame >= _MAX_PDATA_INDEX){
	MsgBox(mpCFrame, _("Max file sequence number is 8000"), _("Warning"));
	mpCPDataSeqCtrl->GetPCActiveFileSequence()->
	    mCFileSequence.mSeqLength = 8000;
	maxFrame = 7999;
    }
    if (maxFrame == 0) maxFrame = 100;
    
    mpCTime->SetMaxFrameNative(maxFrame * snapNumInFile);
    SendMenuEvent(mpCFrame, menuEditRefreshGL);
}

void Main_cls::
CloseDataCommand(){
    GCUndoList.PushNewCommand(new CommandLoadData_cls(this, wxT("")),
			      new CommandLoadData_cls(this, mpCPDataSeqCtrl->GetStrActiveFileSequencePath()));
    GCUndoList.ExecCurrent();
}

void Main_cls::
MoveSelectedFilter(int direction){
    for (int i = 0; i < _MAX_FILTER_CTRL; i++)
	mpCFilterCtrl[i]->MoveSelectedFilter(direction);
}

void Main_cls::
DeleteSelectedFilter(){
    for (int i = 0; i < _MAX_FILTER_CTRL; i++)
	mpCFilterCtrl[i]->DeleteSelectedFilter();
}

void Main_cls::
ClearFilters(){
    for (int i = 0; i < _MAX_FILTER_CTRL; i++)
	mpCFilterCtrl[i]->ClearFilters();
}

void Main_cls::
ConnectTextureToDialog(SceneObjTexture_cls* pCTexture, wxString strCustomName){
    mpCTexPropertyCtrl->ReconnectTexture(pCTexture, strCustomName);
}

void Main_cls::
CloseData(){
    WaitAllThreadFinish();
    mpCPDataSeqCtrl->ClearData(_ACTIVE_SEQUENCE,_SEQUENCE_ALL_INDEX, TRUE);
    if (GLDrawData_cls::mspCSnapShot != NULL){
	delete GLDrawData_cls::mspCSnapShot;
	GLDrawData_cls::mspCSnapShot = NULL;
    }
}

void Main_cls::
Undo(){
    GCUndoList.UndoCurrent();
}

void Main_cls::
Redo(){
    GCUndoList.ExecCurrent();
}

void Main_cls::
ReadProjectFile(wxString strFile){

    std::wifstream ifstream(strFile.char_str(), std::ios::in);
    
#ifdef WIN32
    ifstream.imbue(std::locale("japanese"));
#endif

    wxString strMatch = wxT("*");
    if (ifstream.fail()){
	if (strFile.Matches(strMatch + _CONFIGURE_FILE) == FALSE)
	    wxMessageBox(wxT("Read Configure Fails") + strFile, wxT("Warning")); 
	return;
    }

    GbGLLockProjectLoading = TRUE;
    std::wstring wstrLine;
    while ( std::getline (ifstream, wstrLine)){
	wxString strLine = wstrLine.c_str();
	mpCDataLoader  ->ReadStrLine(strLine);
	mpCDataExporter->ReadStrLine(strLine);
	mpCFrame       ->ReadStrLine(ifstream, strLine);
	mpCTime        ->ReadStrLine(ifstream, strLine);
	mpCDrawOption  ->ReadStrLine(strLine);
	mpCOverlay     ->ReadStrLine(strLine);
	mpCExposure    ->ReadStrLine(strLine);
	mpCPDataSeqCtrl->ReadStrLine(ifstream, strLine, mpCDataLoader);
	mpCPeriodic    ->ReadStrLine(strLine);
	mpCCameraPath  ->ReadStrLine(ifstream, strLine);
	mpCAnchorPath  ->ReadStrLine(ifstream, strLine);
	mpCLightSRCCtrl->ReadStrLine(ifstream, strLine);
	mpCStereogram->ReadStrLine(ifstream, strLine);
	mpCPovRayCtrl->ReadStrLine(strLine);
	GCImgIO.ReadStrLine(strLine);
	mpCFrame->mpCPreference->ReadStrLine(strLine);
	mpCFrame->mpCShortcut->ReadStrLine(ifstream, strLine);
	MainGLCanvas_cls::ReadStrLineStaticMember(ifstream, strLine);
	ShadowVolume_cls::ReadStrLineStaticMember(strLine);
	//mpCFrame->mpCTopHalfPanel->ReadStrLine(ifstream, strLine);
	/*
	for (int i = 0; i < 4; i++)
	    mpCFrame->mpCTopHalfPanel->mpCGLCtrl[i]->ReadStrLine(strLine);
	*/
	
	for (int i = 0; i < _MAX_FILTER_CTRL; i++)
	    mpCFilterCtrl[i]->ReadStrLine(ifstream, strLine);
    }

    GbGLLockProjectLoading = FALSE;
}

void Main_cls::
UpdateGUIItems(){
    mpCFrame->Freeze();

    mpCDataLoader->SetLoadInfo();
    mpCLightSRCCtrl->UpdateTLShown();
    mpCStereogram->UpdateTLShown();
    mpCPovRayCtrl->UpdateItems(TRUE);
    mpCDataLoader->UpdateListFromGrammer();
    mpCFrame->mpCPreference->UpdateShortcutItems();
    mpCFrame->UpdateMenu();
    SendMenuEvent(mpCFrame, menuEditSyncGLCtrl);
    
    GCUndoList.Clear();
    GbGLLockProjectLoading = FALSE;

    mpCFrame->Thaw();
}

bool Main_cls::
SaveProjectFile(wxString strFileOut, bool flag){
    wxFile CFile(strFileOut, wxFile::write);
    if (CFile.IsOpened() == FALSE) return FALSE;
    
    wxString strMatch = wxT("*");
    if (! strFileOut.Matches(strMatch + _CONFIGURE_FILE))
	mpCFrame->mCLastOpenedFileZJP.mstrBody = strFileOut;
    
    wxString sOut;
    
    sOut << mpCDataLoader->Serialize();
    sOut << mpCDataExporter->Serialize();
    sOut << mpCFrame->SerializeForConfigure();
    sOut << mpCPovRayCtrl->SerializeForConfigure();
    sOut << GCImgIO.Serialize();
    sOut << mpCFrame->mpCPreference->SerializeForConfigure();
    
    if (flag != _FM_PROJECT_CONF){
	sOut << mpCFrame->mpCShortcut->SerializeForConfigure();
    }
    
    if (flag == _FM_PROJECT_CONF){
	sOut << mpCFrame->Serialize();
	sOut << mpCTime->Serialize();
	sOut << mpCDrawOption->Serialize();
	sOut << mpCOverlay->Serialize();
	sOut << mpCExposure->Serialize();
	sOut << mpCPDataSeqCtrl->Serialize();
	sOut << mpCPeriodic->Serialize();
	sOut << mpCCameraPath->Serialize();
	sOut << mpCAnchorPath->Serialize();
	sOut << mpCLightSRCCtrl->Serialize();
	sOut << mpCStereogram->Serialize();
	mpCPovRayCtrl->Serialize(sOut);
	sOut << mpCFrame->mpCPreference->Serialize();
	sOut << MainGLCanvas_cls::SerializeStaticMember();
	sOut << ShadowVolume_cls::SerializeStaticMember();
	//sOut << mpCFrame->mpCTopHalfPanel->Serialize();
	/*
	for (int i = 0; i < 4; i++)
	    sOut << mpCFrame->mpCTopHalfPanel->mpCGLCtrl[i]->Serialize();
	*/
	for (int i = 0; i < _MAX_FILTER_CTRL; i++)
	    sOut << mpCFilterCtrl[i]->Serialize();
    }

    CFile.Write(sOut, wxConvFile);

    return TRUE;
}

void Main_cls::
QuitProcess(){
    if (mpCDataLoader->mpCGUI != NULL)
	mpCDataLoader->UpdateGrammerFromList(mpCDataLoader->mpCGUI);
    if (mpCDataExporter->mpCGUI != NULL)
	mpCDataExporter->UpdateGrammerFromList(mpCDataExporter->mpCGUI);
    SaveProjectFile(mstrExecuteDir + _WXDELIM_ + _CONFIGURE_FILE, _FM_DEFAULT_CONF);
}

// ---

bool Main_cls::
IsPovrayCondistionSatisfiled(){
    if (mpCLightSRCCtrl->GetNumberLightEnabled() == 0)
	if (wxMessageBox(_("No Light"), _("Output .pov file"), wxYES_NO, mpCFrame) == wxNO) return FALSE;
    return TRUE;
}

bool Main_cls::
ExportPovSnap(int stereo, bool bForceAbsolute){
    if (IsPovrayCondistionSatisfiled() == FALSE) return FALSE;

    bool bAbsolute = mpCPovRayCtrl->mpCFormat->mCbAbsolutePath.mbBody;

    mpCPovRayCtrl->mpCFormat->mCbAbsolutePath.mbBody = TRUE;
    wxString strFileCommonAbsolute = 
	mpCPovRayCtrl->GetFileStr(_POV_COMMON, stereo, -1);
    mpCPovRayCtrl->mpCFormat->mCbAbsolutePath.mbBody = bAbsolute | bForceAbsolute;
    
    wxString strFileCommon = 
	mpCPovRayCtrl->GetFileStr(_POV_COMMON, stereo, -1);
    wxString strFilePov = 
	mpCPovRayCtrl->GetFileStr(_POV_SNAP, stereo, -1);
    
    wxString strFileIni = strFilePov;
    strFileIni << wxT(".ini");
    strFilePov << wxT(".pov");
    mpCPovRayCtrl->mCommonPDataStatus = 0;
    mpCPovRayCtrl->mCommonBGStatus = 0;

    bool bRet = FALSE;
    bRet |= mpCPovRayCtrl->OutputPovRayIni(strFileIni);
    bRet |= OutputPovRayPov(strFilePov, strFileCommon, strFileCommonAbsolute);
    mpCPovRayCtrl->mpCFormat->mCbAbsolutePath.mbBody = bAbsolute;
    return bRet;
}


bool Main_cls::
RenderPovSnap(int stereo){
    return mpCPovRayCtrl->RenderByPovSnap(stereo);
}

void Main_cls::
SetTimeForPovAnim(){
    switch(mpCPovRayCtrl->GetRegionType()){
    case 0:
	mpCTime->MoveFrameCommandLarge(-1); break;
    case 1:
	mpCTime->mbUseCustomWA = TRUE;
	mpCTime->mCustomWAMin = 0;
	mpCTime->mCustomWAMax = mpCTime->mCMaxFrame.mBody;
	mpCTime->SetFrameCommand(0); break;
    case 2:
	mpCTime->mbUseCustomWA = TRUE;
	mpCTime->mCustomWAMin = mpCPovRayCtrl->GetRegionInit();
	mpCTime->mCustomWAMax = mpCPovRayCtrl->GetRegionFin();;
	mpCTime->SetFrameCommand(mpCPovRayCtrl->GetRegionInit()); break;
    }

    if (mpCTime->mCbRepeat.mbBody == TRUE){
	mpCTime->mpCGUIPlay->mpCToggleRepeat->SetValue(FALSE);
	wxCommandEvent com;
	com.SetInt(0);
	mpCTime->mpCGUIPlay->OnToggleRepeat(com);
    }
}

bool Main_cls::
ExportPovAnim(int stereo){
    if (IsPovrayCondistionSatisfiled() == FALSE) return FALSE;

    bool bForceAbsolute = mpCPovRayCtrl->mbRenderingAnim;    
    bool bAbsolute = mpCPovRayCtrl->mpCFormat->mCbAbsolutePath.mbBody;

    mpCPovRayCtrl->mpCFormat->mCbAbsolutePath.mbBody = TRUE;
    wxString strFileCommonAbsolute = 
	mpCPovRayCtrl->GetFileStr(_POV_COMMON, stereo, -1);
    
    mpCPovRayCtrl->mpCFormat->mCbAbsolutePath.mbBody = 
	bAbsolute | bForceAbsolute;
    wxString strFileCommon = 
	mpCPovRayCtrl->GetFileStr(_POV_COMMON, stereo, -1);
    wxString strFileOutAnim = 
	mpCPovRayCtrl->GetFileStr(_POV_ANIM, stereo, -1);
    wxString strFileOutIni = strFileOutAnim;

    strFileOutIni << wxT(".ini");
    strFileOutAnim << wxT(".pov");
    mpCPovRayCtrl->IncrementAnim();
    
    bool bRet = FALSE;
    bRet |= mpCPovRayCtrl->OutputPovRayIni(strFileOutIni);
    bRet |= OutputPovRayPov(strFileOutAnim, strFileCommon, strFileCommonAbsolute);
    mpCPovRayCtrl->mpCFormat->mCbAbsolutePath.mbBody = bAbsolute;
    
    return bRet;
}

void Main_cls::
GetStrPovPathExpression(wxString& strExp){
    strExp << wxT("#declare t0 = clock;") << ENDL;
    
    CameraPath_cls *pCCameraPath = mpCCameraPath;
    AnchorPath_cls *pCAnchorPath = mpCAnchorPath;

    strExp << pCCameraPath->mpCTranslate->GetStrExpression(/*wxT("Camera_")*/);
    strExp << pCCameraPath->mpCRotation->GetStrExpression(/*wxT("Camera_")*/);

    if (pCCameraPath->mCbLookAnchor.mbBody == TRUE)
	strExp << pCAnchorPath->mpCTranslate->GetStrExpression(/*wxT("Anchor_")*/);

    wxString strCameraType;
    MainGLCanvas_cls *pCGLCanvas = mpCFrame->GetPCGLCanvas(0);
    CameraStructure_cls CCamStructure = pCGLCanvas->GetCCameraStructure(0);
    mpCPovRayCtrl->GetStrPovCameraType(strCameraType, CCamStructure, FALSE);

    wxString strFocalBlur;
    mpCPovRayCtrl->GetStrPovFocalBlurExpression(strFocalBlur, 1);

    wxString strTemplate;
    if (pCCameraPath->mCbLookAnchor.mbBody == TRUE)
	strTemplate = wxT("Resource/cameratemplate.pov");
    else
	strTemplate = wxT("Resource/cameratemplateNA.pov");
    
    wxTextFile CFile(strTemplate);
    CFile.Open();
    for (wxString str = CFile.GetFirstLine(); !CFile.Eof();
	 str = CFile.GetNextLine() ){
	str.Replace(wxT("CAMERATYPE"), strCameraType);
	str.Replace(wxT("FOCALBLUR"), strFocalBlur);
	strExp << str << ENDL;
    }
}

void Main_cls::
ExportPovPath(){
    wxString strExp;
    GetStrPovPathExpression(strExp);
    /*
    strExp << wxT("#declare t0 = clock;") << ENDL;
    strExp << mpCCameraPath->mpCTranslate->GetStrExpression();
    strExp << mpCCameraPath->mpCRotation->GetStrExpression();

    wxString strTemplate;
    if (mpCCameraPath->mCbLookAnchor.mbBody == TRUE){
	strExp << mpCAnchorPath->mpCTranslate->GetStrExpression();
	strTemplate = wxT("Resource/cameratemplate.pov");
    }
    else
	strTemplate = wxT("Resource/cameratemplateNA.pov");

    wxTextFile CFile(strTemplate);
    CFile.Open();
    for (wxString str = CFile.GetFirstLine(); !CFile.Eof();
	 str = CFile.GetNextLine() )
	strExp << str << ENDL;
    */
    
    wxTextDataObject *pCData = new wxTextDataObject(strExp);
    wxTheClipboard->Open();
    wxTheClipboard->SetData(pCData);
    wxTheClipboard->Close();
}

void Main_cls::
GetStrPovCopyLoopRef(wxString &strPData, int tabDepth, int sticknum){
    FilterCopy_cls *pCCopy = mpCFilterCtrl[2]->GetPCCopy();

    int    currentIndex = mpCTime->GetRemappedIndex();
    double doubleInterpStep = mpCTime->GetRemappedInterpStepDouble();
    double time = 0, dT = 0;
    mpCFrame->GetPCActiveFileSequence()->GetTimeInfo(time, dT, currentIndex, doubleInterpStep);
    
    mpCPovRayCtrl->mTabDepth = tabDepth;
    if (pCCopy != NULL){
	int copyNum = pCCopy->GetNumber();
	for (int i = 0; i < copyNum; i++){
	    if (pCCopy->GetBValidity(i, time+dT, FALSE, 1) == FALSE) continue;
	    mpCPovRayCtrl->mstrInstanceTranslate = pCCopy->GetStrPovTranslate(i, time+dT);
	    strPData << mpCPovRayCtrl->GetStrPovParticleObject(wxT("PARTICLES"));
	    if (sticknum > 0){
		int bitBoundary = pCCopy->GetBitBoundary(i);
		strPData << mpCPovRayCtrl->GetStrPovParticleObject(wxT("STICKS0"));
		for (int bound = 1; bound < 8; bound++)
		    if ((bitBoundary & bound) == bound){
			wxString wxTag = wxT("STICKS");
			wxTag << bound;
			strPData << mpCPovRayCtrl->GetStrPovParticleObject(wxTag);
		    }
	    }
	}
    }
    else{
	mpCPovRayCtrl->mstrInstanceTranslate = wxT("");
	strPData << mpCPovRayCtrl->GetStrPovParticleObject(wxT("PARTICLES"));
	if (sticknum > 0)
	    strPData << mpCPovRayCtrl->GetStrPovParticleObject(wxT("STICKS0"));
    }
}

void Main_cls::
GetStrPovCopyLoopRefObject(wxString &strPData, double doubleIndex, int tabDepth){

    MainGLCanvas_cls *pCGLCanvas = mpCFrame->GetPCGLCanvas(0);
    FilterCopy_cls *pCCopy = mpCFilterCtrl[2]->GetPCCopy();

    int    currentIndex = mpCTime->GetRemappedIndex();
    double doubleInterpStep = mpCTime->GetRemappedInterpStepDouble();
    double time = 0, dT = 0;
    mpCFrame->GetPCActiveFileSequence()->GetTimeInfo(time, dT, currentIndex, doubleInterpStep);
    
    if (pCCopy != NULL){
	int copyNum = pCCopy->GetNumber();
	for (int i = 0; i < copyNum; i++){
	    if (pCCopy->GetBValidity(i, time+dT, FALSE, 1) == FALSE) continue;
	    mpCFilterCtrl[1]->mstrInstanceTranslate = pCCopy->GetStrPovTranslate(i, time);
	    mpCFilterCtrl[1]->GetStrPovRealObject(strPData, doubleIndex, TRUE, tabDepth, pCGLCanvas->mCCameraPos);
	}
	mpCFilterCtrl[1]->mstrInstanceTranslate = wxT("");
	mpCFilterCtrl[1]->GetStrPovRealObject(strPData, doubleIndex, FALSE, tabDepth, pCGLCanvas->mCCameraPos);
    }
    else{
	mpCFilterCtrl[1]->mstrInstanceTranslate = wxT("");
	mpCFilterCtrl[1]->GetStrPovRealObject(strPData, doubleIndex, TRUE, tabDepth, pCGLCanvas->mCCameraPos);
	mpCFilterCtrl[1]->GetStrPovRealObject(strPData, doubleIndex, FALSE, tabDepth, pCGLCanvas->mCCameraPos);
    }
}

bool Main_cls::
OutputPovRayCommonScene(bool &bPDataExist, wxString strFileCommon,
			int currentIndex, double dT, 
			std::vector < FilterStick_cls* > VpCSticks ){
    wxFile CFileCommon(strFileCommon, wxFile::write);
    if (CFileCommon.IsOpened() == FALSE) return FALSE;

    wxString strCommon;
    bPDataExist = 
	mpCPDataSeqCtrl->GetPCActiveFileSequence()->GetPovDataRef
	(strCommon, mpCPovRayCtrl, currentIndex, dT);
    if (VpCSticks.size() > 0){
	for (unsigned int i = 0; i < VpCSticks.size(); i++){
	    strCommon << VpCSticks[i]->GetStrPovCylinderMacro();
	    VpCSticks[i]->GetStrPovSticksRef(strCommon, GLDrawData_cls::mspCSnapShot);
	}
    }

    mpCFilterCtrl[1]->GetStrPovInclude(strCommon, GetPPCGLAttribute());
        
    CFileCommon.Write(strCommon);
    CFileCommon.Close();
    return TRUE;
}

bool Main_cls::
IsCommonPDataSceneFileEffective(){
    if (mpCPovRayCtrl->mCbUseCommonSceneFile.mbBody == TRUE &&
	mpCPDataSeqCtrl->GetPCActiveFileSequence()->GetSeqLength() == 1)
	return TRUE;
    return FALSE;
}

bool Main_cls::
OutputPovRayPov(wxString strFile, wxString strFileCommon, wxString strFileCommonAbsolute){
    
    wxFile CFile(strFile, wxFile::write);
    if (CFile.IsOpened() == FALSE) return FALSE;

    MainGLCanvas_cls::PrepareDrawOrPovOutStatic();
    
    wxString strSettings, strPov, strCopyLoop;
    strSettings << mpCPovRayCtrl->GetStrPovGlobal();

    mpCFrame->GetPCGLCanvas(0)->PrepareDrawOrPovOut();
    mpCFrame->GetPCGLCanvas(0)->UpdateCameraPos();
    double doubleIndex = mpCTime->GetDoubleIndex();    

    MainGLCanvas_cls *pCGLCanvas = mpCFrame->GetPCGLCanvas(0);

    float angle   = pCGLCanvas->mCGLPerspDraw.mDeg;
    bool  bFishEye = pCGLCanvas->IsFishEye();
    bool  bOrthogaraphic = pCGLCanvas->IsOrthographic();

    //strSettings << mpCPovRayCtrl->GetStrPovCameraAngle(angle, bFishEye);
    strSettings << mpCPovRayCtrl->GetStrPovCamera
	(pCGLCanvas->mCCameraPos, pCGLCanvas->msCAnchorPos,
	 angle, bFishEye, bOrthogaraphic, mpCStereogram);

    strSettings << wxT("camera { Camera }") << ENDL;

    FilterDataBoxData_cls *pCScene = 
	(FilterDataBoxData_cls*)mpCFilterCtrl[2]->GetPCDataBox();

    strSettings << mpCLightSRCCtrl->GetStrPovLights(doubleIndex, pCScene);

    FilterFog_cls *pCFog = mpCFilterCtrl[2]->GetPCFog();
    if (pCFog != NULL)
	strPov << pCFog->GetStrPovFog(doubleIndex);

    mpCFilterCtrl[2]->GetStrPovBG(strPov, doubleIndex, mpCPovRayCtrl);
    
    int    currentIndex     = mpCTime->GetRemappedIndex();
    double doubleInterpStep = mpCTime->GetRemappedInterpStepDouble();
    double time, dT;
    mpCPDataSeqCtrl->GetPCActiveFileSequence()->GetTimeInfo(time, dT, currentIndex, doubleInterpStep);
    bool bPDataExist = TRUE;

    // SceneObjects
    if (IsCommonPDataSceneFileEffective() == FALSE)
	mpCFilterCtrl[1]->GetStrPovInclude(strCopyLoop, GetPPCGLAttribute());
    
    bool bRealObjectExist = mpCFilterCtrl[1]->IsObjectExist();
    if (bRealObjectExist == TRUE){
	if (pCScene == NULL){
	    GetStrPovCopyLoopRefObject(strCopyLoop, doubleIndex, 0);
	}
	else{ 
	    strCopyLoop << wxT("union {") << ENDL;
	    GetStrPovCopyLoopRefObject(strCopyLoop, doubleIndex, 1);
	    // !! FIX_ME !! DataBox Wɍڂ邩ǂ
	    pCScene->GetStrPovModelView(strCopyLoop, doubleIndex, 
					_OBJECT_NOT_PDATA);
	    strCopyLoop << wxT("}") << ENDL;
	}
    }
    
    //MainDataStick

    std::vector < FilterStick_cls* > VpCSticks = 
	mpCFilterCtrl[1]->GetVpCSticks();
    if (IsCommonPDataSceneFileEffective() == TRUE){
	if (mpCPovRayCtrl->mCommonPDataStatus == 0){
	    if (OutputPovRayCommonScene(bPDataExist, strFileCommonAbsolute,
					currentIndex, dT, VpCSticks) == FALSE){
		CFile.Close();
		return FALSE;
	    }
	    mpCPovRayCtrl->mCommonPDataStatus = 1;
	}
	strCopyLoop << wxT("#include \"") << strFileCommon << wxT("\"") << ENDL; 
    }
    else{
	mpCPovRayCtrl->GetStrPovParticleMacro
	    (strCopyLoop, doubleIndex, GetPPCGLAttribute());
	
	bPDataExist = mpCPDataSeqCtrl->GetPCActiveFileSequence()->
	    GetPovDataRef(strCopyLoop, mpCPovRayCtrl, currentIndex, dT);
	if (VpCSticks.size() > 0){
	    for (int unsigned i = 0; i < VpCSticks.size(); i++){
		strCopyLoop << VpCSticks[i]->GetStrPovCylinderMacro();
		VpCSticks[i]->GetStrPovSticksRef(strCopyLoop, GLDrawData_cls::mspCSnapShot);
	    }
	}
    }

    if (bPDataExist == TRUE){
	if (pCScene == NULL){
	    GetStrPovCopyLoopRef(strCopyLoop, 0, (int)VpCSticks.size());
	}
	else{
	    strCopyLoop << wxT("union {") << ENDL;
	    GetStrPovCopyLoopRef(strCopyLoop, 1, (int)VpCSticks.size());
	    pCScene->GetStrPovModelView(strCopyLoop, doubleIndex,
					_OBJECT_PDATA);
	    strCopyLoop << wxT("}") << ENDL;
	}
    }
    
    CFile.Write(strSettings, wxConvUTF8);
    CFile.Close();

    CFile.Open(strFile, wxFile::write_append);
    CFile.Write(strPov, wxConvUTF8);
    CFile.Write(strCopyLoop, wxConvUTF8);

    MainGLCanvas_cls::EndDrawOrPovOut();

    return TRUE;
}

void Main_cls::
Special(){
    GbSpecial = TRUE;
    /*
    int currentIndex = mpCTime->GetRemappedIndex();
    mpCPDataSeqCtrl->GetPCActiveFileSequence()->
	Special_Copy(currentIndex, currentIndex - 1);
    */
}
