// -*- c++ -*- 
// (setq c-basic-offset 4)
// "clsMain.cpp"

#ifndef _MAIN_H_
#define _MAIN_H_

#include <fstream>

#include "SRC/CONFITEM/clsConfItem.h"
#include "SRC/WIDGETCTRL/clsWidgetCommon.h"

class AnchorPath_cls;
class CameraPath_cls;
class CameraPathCtrl_cls;
class ColorCurveCtrl_cls;
class CustomColor_cls;
class DrawOption_cls;
class DrawingBlock_cls;
class Exposure_cls;
class FilterBase_cls;
class FilterCtrl_cls;
class FilterStick_cls;
class FrameMain_cls;
class GLAttribute_cls;
class ImgSequence_cls;
class ImportList_cls;
class Information_cls;
class IpAddress_cls;
class LightSRCCtrl_cls;
class MacAddress_cls;
class Main_cls;
class TexPropertyCtrl_cls;
class TimeCtrl_cls;
class TreeBoxRule_cls;
class Scene_cls;
class SceneObjTexture_cls;
class Overlay_cls;
class Parameters_cls;
class ParamGroup_cls;
class ParamGUICtrl_cls;
class PeriodicBoundary_cls;
class PDataSeqCtrl_cls;
class PovRayCtrl_cls;

class StereoGram_cls;
class DataLoader_cls;
class DataExporter_cls;

#define _MAX_FILTER_CTRL 3

class Main_cls : public WidgetCommon_cls {
 public:
    Main_cls();
    ~Main_cls();

    int mPID;
    int mInitializeTimeA, mInitializeTimeB, mInitializeTimeC;

    DrawOption_cls   *mpCDrawOption;
    FilterCtrl_cls   *mpCFilterCtrl[_MAX_FILTER_CTRL];
    Overlay_cls      *mpCOverlay;
    PDataSeqCtrl_cls *mpCPDataSeqCtrl;
    PeriodicBoundary_cls *mpCPeriodic;
    DataLoader_cls   *mpCDataLoader;
    DataExporter_cls *mpCDataExporter;
    Exposure_cls     *mpCExposure;
    Information_cls  *mpCInformation;
    CameraPath_cls   *mpCCameraPath;
    AnchorPath_cls   *mpCAnchorPath;
    CameraPathCtrl_cls *mpCCameraPathCtrl;
    LightSRCCtrl_cls   *mpCLightSRCCtrl;
    ColorCurveCtrl_cls *mpCColorCurveCtrl;
    TimeCtrl_cls    *mpCTime;
    Scene_cls       *mpCScene;
    StereoGram_cls  *mpCStereogram;
    FrameMain_cls   *mpCFrame;
    ImgSequence_cls *mpCImgSeq;
    MacAddress_cls  *mpCMacAddress;
    IpAddress_cls   *mpCIpAddress;

    ParamGUICtrl_cls *mpCParamGUICtrlLink;
    ParamGroup_cls   *mpCPGCameraPath;
    ParamGroup_cls   *mpCPGStereoGram;

    wxString         mstrExecuteDir;

    wxWindow* GetPTopLevelWindow();
    Parameters_cls* GetPCParamFromID(wxString strID);
    GLAttribute_cls** GetPPCGLAttribute(); 
    int  GetUsedTotalMemoryMB();
    int  GetAverageMemoryMB();
    int  GetMemoryLimitMB();
    void SelectGLAttrib(GLAttribute_cls* pCGLAttrib);
    void SelectObject(int objectIndex, int guideIndex = 0);
    void ThisObjectIsSelected(int objectIndex);
    void SelectFilter(FilterBase_cls *pCFilterBase, bool bDeselectObject = TRUE);
    void SelectFilterByHittest(int itemName, bool bDeselectObject = TRUE);
    
    bool IsTheIndexReady(int index, double indexmod);
    bool IsTheIndexLoaded(int index);
    void WaitAllThreadFinish();

    void OpenColorCurveDialog();
    void OpenPovRayDialog();

    void OpenDataFileCommand(wxString strFilePath);
    void OpenDataFile(wxString strFilePath);
    void CloseDataCommand();
    void CloseData();

    void MoveSelectedFilter(int direction);
    void DeleteSelectedFilter();
    void ClearFilters();
    void ConnectTextureToDialog(SceneObjTexture_cls* pCTexture,
				wxString strCustomName);
    
    //ConfItemBOOL_cls mCbToolTip;

    wxBitmap *mpBitmapColor, *mpBitmapLight;

    //----

    ImportList_cls*  mpCImportList;
    CustomColor_cls* mpCCustomColor;

    TexPropertyCtrl_cls *mpCTexPropertyCtrl;
    PovRayCtrl_cls      *mpCPovRayCtrl;

    //----

    bool bOneTimeWarningShowed_SceneLength;
    long mHungAppTimeoutMS;

    wxSize mDisplaySize;

    //----

    void RefreshDialogs();
    
    //----
    
    void ImportParamInitialize();
    void ImportParamMain(std::wistream &istream, std::wstring strLine);
    void ImportParam(unsigned int type);

    //----
    
    void Undo();
    void Redo();

    void ReadProjectFile(wxString strFile);
    void UpdateGUIItems();
    bool SaveProjectFile(wxString strFileOut, bool flag);
    void QuitProcess();

    //----
    
    bool IsPovrayCondistionSatisfiled();
    bool ExportPovSnap(int stereo, bool bForceAbsolute);
    bool RenderPovSnap(int stereo);
    void SetTimeForPovAnim();
    bool ExportPovAnim(int stereo);
    void GetStrPovPathExpression(wxString& strExp);
    void ExportPovPath();
    void GetStrPovCopyLoopRef(wxString &strPData, int tabDepth,  int sticknum);
    void GetStrPovCopyLoopRefObject(wxString &strPData, double doubleIndex, int tabDepth);
    bool OutputPovRayCommonScene(bool &bPDataExist, wxString strFileCommon,
				 int currentIndex, double dT,
				 std::vector < FilterStick_cls* > VpCSticks);
    bool IsCommonPDataSceneFileEffective();
    bool OutputPovRayPov(wxString strFile, wxString strFileCommon,
			 wxString strFileCommonAbsolute);

    void OutputPovRayMain(std::wfstream &ofstream, int frame);

    bool OutputPovRayIOAnim(unsigned int *pInit, unsigned int *pFin, 
			    bool bQuery = FALSE);
    
    void Special();

};

#endif
